Package: dcvar
Title: Dynamic Copula VAR Models for Time-Varying Dependence
Version: 0.9.3
Authors@R: 
    person("Benedikt", "Lugauer", , "benedikt.lugauer@uni-leipzig.de",
           role = c("aut", "cre"))
Description: Fits Bayesian copula vector autoregressive models for bivariate
    time series with dynamic, regime-switching, and constant dependence
    structures. The package includes simulation, data preparation,
    estimation with 'Stan' through 'rstan' or 'cmdstanr', posterior
    summaries, diagnostics, trajectory extraction, fitted and predictive
    summaries, and approximate leave-one-out cross-validation model
    comparison for supported fits. For Bayesian computation and model
    comparison, see Carpenter et al. (2017)
    <doi:10.18637/jss.v076.i01> and Vehtari, Gelman and Gabry (2017)
    <doi:10.1007/s11222-016-9696-4>.
License: GPL (>= 3)
URL: https://github.com/benlug/dcvar
BugReports: https://github.com/benlug/dcvar/issues
Encoding: UTF-8
Language: en-US
RoxygenNote: 7.3.3
Depends: R (>= 4.1.0)
Imports: rstan (>= 2.26.0), posterior (>= 1.5.0), loo (>= 2.7.0),
        ggplot2 (>= 3.4.0), patchwork (>= 1.1.0), bayesplot (>=
        1.10.0), rlang (>= 1.0.0), cli (>= 3.0.0), parallel, stats,
        tools, utils
Suggests: cmdstanr (>= 0.8.0), testthat (>= 3.0.0), knitr, rmarkdown,
        tibble, sn
Additional_repositories: https://stan-dev.r-universe.dev
VignetteBuilder: knitr
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-07-05 18:37:03 UTC; benlug
Author: Benedikt Lugauer [aut, cre]
Maintainer: Benedikt Lugauer <benedikt.lugauer@uni-leipzig.de>
Repository: CRAN
Date/Publication: 2026-07-05 20:00:08 UTC

# Generated by roxygen2: do not edit by hand

S3method(as.data.frame,dcvar_model_fit)
S3method(coef,dcvar_constant_fit)
S3method(coef,dcvar_covariate_fit)
S3method(coef,dcvar_fit)
S3method(coef,dcvar_hmm_fit)
S3method(coef,dcvar_multilevel_fit)
S3method(coef,dcvar_multilevel_tv_fit)
S3method(coef,dcvar_sem_fit)
S3method(coef,dcvar_sem_tv_fit)
S3method(coef,dcvar_tv_fit)
S3method(covariate_effects,dcvar_covariate_fit)
S3method(covariate_effects,default)
S3method(dcvar_diagnostics,dcvar_model_fit)
S3method(dcvar_diagnostics,default)
S3method(dependence_summary,dcvar_constant_fit)
S3method(dependence_summary,dcvar_covariate_fit)
S3method(dependence_summary,dcvar_fit)
S3method(dependence_summary,dcvar_hmm_fit)
S3method(dependence_summary,dcvar_multilevel_fit)
S3method(dependence_summary,dcvar_multilevel_tv_fit)
S3method(dependence_summary,dcvar_sem_fit)
S3method(dependence_summary,dcvar_sem_tv_fit)
S3method(dependence_summary,default)
S3method(draws,dcvar_model_fit)
S3method(draws,default)
S3method(fitted,dcvar_hmm_fit)
S3method(fitted,dcvar_model_fit)
S3method(fitted,dcvar_multilevel_fit)
S3method(fitted,dcvar_multilevel_tv_fit)
S3method(fitted,dcvar_sem_fit)
S3method(fitted,dcvar_tv_fit)
S3method(hmm_state_params,dcvar_hmm_fit)
S3method(hmm_state_params,default)
S3method(hmm_states,dcvar_hmm_fit)
S3method(hmm_states,default)
S3method(latent_states,dcvar_sem_fit)
S3method(latent_states,default)
S3method(loo,dcvar_constant_fit)
S3method(loo,dcvar_covariate_fit)
S3method(loo,dcvar_fit)
S3method(loo,dcvar_hmm_fit)
S3method(loo,dcvar_multilevel_fit)
S3method(loo,dcvar_sem_fit)
S3method(phi_trajectory,dcvar_multilevel_tv_fit)
S3method(phi_trajectory,dcvar_sem_tv_fit)
S3method(phi_trajectory,dcvar_tv_fit)
S3method(phi_trajectory,default)
S3method(pit_test,dcvar_model_fit)
S3method(pit_test,dcvar_multilevel_fit)
S3method(pit_test,dcvar_sem_fit)
S3method(pit_test,default)
S3method(pit_values,dcvar_hmm_fit)
S3method(pit_values,dcvar_model_fit)
S3method(pit_values,dcvar_multilevel_fit)
S3method(pit_values,dcvar_sem_fit)
S3method(pit_values,dcvar_tv_fit)
S3method(pit_values,default)
S3method(plot,dcvar_constant_fit)
S3method(plot,dcvar_covariate_fit)
S3method(plot,dcvar_fit)
S3method(plot,dcvar_hmm_fit)
S3method(plot,dcvar_multilevel_fit)
S3method(plot,dcvar_sem_fit)
S3method(plot,dcvar_tv_fit)
S3method(predict,dcvar_hmm_fit)
S3method(predict,dcvar_model_fit)
S3method(predict,dcvar_multilevel_fit)
S3method(predict,dcvar_multilevel_tv_fit)
S3method(predict,dcvar_sem_fit)
S3method(predict,dcvar_tv_fit)
S3method(print,dcvar_applicability)
S3method(print,dcvar_constant_fit)
S3method(print,dcvar_constant_summary)
S3method(print,dcvar_covariate_fit)
S3method(print,dcvar_covariate_summary)
S3method(print,dcvar_fit)
S3method(print,dcvar_hmm_fit)
S3method(print,dcvar_hmm_summary)
S3method(print,dcvar_multilevel_fit)
S3method(print,dcvar_multilevel_summary)
S3method(print,dcvar_multilevel_tv_fit)
S3method(print,dcvar_multilevel_tv_summary)
S3method(print,dcvar_sem_fit)
S3method(print,dcvar_sem_summary)
S3method(print,dcvar_sem_tv_fit)
S3method(print,dcvar_sem_tv_summary)
S3method(print,dcvar_summary)
S3method(print,dcvar_tv_fit)
S3method(print,dcvar_tv_summary)
S3method(random_effects,dcvar_multilevel_fit)
S3method(random_effects,default)
S3method(rho_trajectory,dcvar_constant_fit)
S3method(rho_trajectory,dcvar_covariate_fit)
S3method(rho_trajectory,dcvar_fit)
S3method(rho_trajectory,dcvar_hmm_fit)
S3method(rho_trajectory,dcvar_multilevel_fit)
S3method(rho_trajectory,dcvar_multilevel_tv_fit)
S3method(rho_trajectory,dcvar_sem_fit)
S3method(rho_trajectory,dcvar_sem_tv_fit)
S3method(rho_trajectory,default)
S3method(sigma_trajectory,dcvar_multilevel_tv_fit)
S3method(sigma_trajectory,dcvar_sem_tv_fit)
S3method(sigma_trajectory,dcvar_tv_fit)
S3method(sigma_trajectory,default)
S3method(summary,dcvar_constant_fit)
S3method(summary,dcvar_covariate_fit)
S3method(summary,dcvar_fit)
S3method(summary,dcvar_hmm_fit)
S3method(summary,dcvar_multilevel_fit)
S3method(summary,dcvar_multilevel_tv_fit)
S3method(summary,dcvar_sem_fit)
S3method(summary,dcvar_sem_tv_fit)
S3method(summary,dcvar_tv_fit)
S3method(var_params,dcvar_hmm_fit)
S3method(var_params,dcvar_model_fit)
S3method(var_params,dcvar_multilevel_fit)
S3method(var_params,dcvar_multilevel_tv_fit)
S3method(var_params,dcvar_sem_fit)
S3method(var_params,dcvar_sem_tv_fit)
S3method(var_params,dcvar_tv_fit)
S3method(var_params,default)
export(aggregate_metrics)
export(applicability_check)
export(compute_param_metrics)
export(compute_rho_metrics)
export(covariate_effects)
export(dcvar)
export(dcvar_compare)
export(dcvar_constant)
export(dcvar_covariate)
export(dcvar_diagnostics)
export(dcvar_hmm)
export(dcvar_multilevel)
export(dcvar_sem)
export(dcvar_stan_path)
export(dependence_summary)
export(draws)
export(hmm_state_params)
export(hmm_states)
export(interpret_rho_trajectory)
export(latent_states)
export(phi_trajectory)
export(pit_test)
export(pit_values)
export(plot_diagnostics)
export(plot_hmm_states)
export(plot_latent_states)
export(plot_phi)
export(plot_phi_trajectory)
export(plot_pit)
export(plot_ppc)
export(plot_random_effects)
export(plot_rho)
export(plot_sigma_trajectory)
export(plot_trajectories)
export(prepare_constant_data)
export(prepare_dcvar_covariate_data)
export(prepare_dcvar_data)
export(prepare_hmm_data)
export(prepare_multilevel_data)
export(prepare_sem_data)
export(random_effects)
export(rho_constant)
export(rho_decreasing)
export(rho_double_step)
export(rho_increasing)
export(rho_random_walk)
export(rho_scenario)
export(rho_step)
export(rho_trajectory)
export(sigma_trajectory)
export(simulate_breakpoint_data)
export(simulate_dcvar)
export(simulate_dcvar_multilevel)
export(simulate_dcvar_sem)
export(var_params)
importFrom(cli,cli_abort)
importFrom(cli,cli_alert_info)
importFrom(cli,cli_alert_success)
importFrom(cli,cli_inform)
importFrom(cli,cli_warn)
importFrom(loo,loo)
importFrom(rlang,"%||%")
importFrom(rlang,.data)
importFrom(rstan,sampling)
importFrom(rstan,stan_model)
importFrom(stats,coef)
importFrom(stats,complete.cases)
importFrom(stats,cor)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(utils,head)

# dcvar: Dynamic Copula VAR Models for Time-Varying Dependence

[<img src="man/figures/logo.png" align="right" width="15%" height="15%" alt="dcvar logo"/>](https://github.com/benlug/dcvar)

<!-- badges: start -->
[![R-CMD-check](https://github.com/benlug/dcvar/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/benlug/dcvar/actions/workflows/R-CMD-check.yaml)
[![Lifecycle: experimental](https://img.shields.io/badge/lifecycle-experimental-orange.svg)](https://lifecycle.r-lib.org/articles/stages.html#experimental)
[![License: GPL v3+](https://img.shields.io/badge/License-GPL%20v3%2B-blue.svg)](https://www.gnu.org/licenses/gpl-3.0.en.html)
<!-- badges: end -->

dcvar fits Bayesian copula vector autoregressions of order one for bivariate
time series. The modelling idea is to keep two questions separate: how each
series is distributed on its own, and how the two series move together. A
copula carries the dependence, so each margin may follow its own distribution
while a single association parameter describes how the series are coupled at a
given time.

The conditional mean follows a first-order vector autoregression, so each
series depends on the recent past of both. The innovations are joined by a
Gaussian copula, with a Clayton copula also available for the constant
baseline. The association can be held constant, allowed to drift smoothly as a
random walk, allowed to switch between latent regimes, or expressed as a
function of covariates. In the dynamic model the autoregressive coefficients
and the innovation scales may vary over time as well; in the HMM model, the
regime can also govern the intercepts, autoregressive coefficients, innovation
scales, and marginal families. Everything is estimated in a fully Bayesian way
through [Stan](https://mc-stan.org/), which returns posterior distributions for
all quantities.

Two further families, a multilevel version for panel data and a
structural-equation version for latent processes, are provided as experimental
extensions.

## Installation

dcvar uses [`rstan`](https://mc-stan.org/rstan/) as its default backend.

Install dcvar from CRAN:

```r
install.packages("dcvar")
```

For the development version:

```r
install.packages("remotes")
remotes::install_github("benlug/dcvar")
```

Optionally, you can use [`cmdstanr`](https://mc-stan.org/cmdstanr/) as an
alternative backend:

```r
install.packages(
  "cmdstanr",
  repos = c("https://stan-dev.r-universe.dev", getOption("repos"))
)
cmdstanr::install_cmdstan()
```

For skew-normal margins, install `sn`:

```r
install.packages("sn")
```

## Example

The example below simulates a bivariate series whose association declines over
time, fits the dynamic copula model, and compares it with the regime-switching
and constant-association alternatives by cross-validation.

```r
library(dcvar)

# simulate data with decreasing coupling
sim <- simulate_dcvar(
  n_time = 150,
  rho_trajectory = rho_decreasing(150, rho_start = 0.7, rho_end = 0.3)
)

# fit the DC-VAR model
fit <- dcvar(sim$Y_df, vars = c("y1", "y2"))

# inspect results
summary(fit)
plot_rho(fit, true_rho = sim$true_params$rho)

# compare models via leave-one-out cross-validation
fit_hmm <- dcvar_hmm(sim$Y_df, vars = c("y1", "y2"), K = 2)
fit_con <- dcvar_constant(sim$Y_df, vars = c("y1", "y2"))
dcvar_compare(dcvar = fit, hmm = fit_hmm, constant = fit_con)
```

## Models

The core models differ in how dependence, marginal distributions, and the
vector-autoregressive mean are allowed to change over time.

| Model | Function | Association over time | Status |
| --- | --- | --- | --- |
| DC-VAR | `dcvar()` | Drifts smoothly as a random walk on the Fisher z scale; the autoregressive coefficients and innovation scales may also drift | Core |
| HMM Copula | `dcvar_hmm()` | Switches between K latent regimes | Core |
| Constant Copula | `dcvar_constant()` | Held constant (Gaussian or Clayton) | Core |
| Covariate DC-VAR | `dcvar_covariate()` | A function of observed covariates on the Fisher z scale | Core |
| Multilevel | `dcvar_multilevel()` | Series-specific autoregressive coefficients across many units | Experimental |
| SEM | `dcvar_sem()` | Latent processes measured by observed indicators | Experimental |

Because the copula keeps the margins separate from the dependence, every model
accepts per-variable margins. Pass a length-two vector such as
`margins = c("normal", "exponential")` to let each series follow a different
marginal family. The single-level models `dcvar()`, `dcvar_hmm()`, and
`dcvar_constant()` with a Gaussian copula support all four families (normal,
exponential, skew-normal, and gamma), used on their own or in combination.
`dcvar_multilevel()` and `dcvar_sem()` support all four families as a single
family or in combination.
`dcvar_constant(copula = "clayton")` provides a Clayton-copula baseline with
normal or mixed margins.

In `dcvar()`, the association always evolves over time. Setting `tv_phi = TRUE`
additionally lets the autoregressive coefficients drift as random walks, and
`tv_sigma = TRUE` lets the innovation scales drift. `tv_phi` also accepts
`"ar"` or `"cross"` to let only the autoregressive or only the cross-lagged
coefficients vary. With both options off, the model reduces to the
constant-coefficient dynamic copula.

In `dcvar_hmm()`, `switch = "rho"` gives the classic regime-switching copula
model, where only the copula correlation changes by latent state. Passing
components such as `switch = c("rho", "mu", "phi", "sigma")` fits a fuller
Markov-switching VAR(1). The HMM can also use state-specific marginal families
by passing a length-`K` list to `margins`; `hmm_state_params()` extracts the
effective state-specific intercepts, VAR matrices, scale parameters, and
families.

## Estimation and checks

Fitted and one-step-ahead predicted values are available for every fitted
model. For multilevel fits these are specific to each unit; for
structural-equation fits they cover both the latent states (`type = "link"`)
and the observed indicators (`type = "response"`).

Leave-one-out cross-validation through `loo()` is available for the
single-level fits, the covariate fits, the multilevel fits, and the naive
structural-equation score fits, and `dcvar_compare()` places several fits on a
common predictive scale. Comparisons that mix model families whose pointwise
predictive densities are not on the same footing are flagged with a warning.

Residual checks based on the probability integral transform, `pit_values()` and
`pit_test()`, are provided for the single-level models. They use posterior
means and are best read as heuristic checks rather than exact calibration
tests. Posterior predictive checks through `plot_ppc()` are available for
normal and exponential margins; gamma and skew-normal fits do not yet store
replicated residuals on the observed margin scale.

The package also includes a constant Clayton-copula baseline, a multilevel
model with exponential margins, and naive structural-equation score models that
were used in the accompanying simulation studies.

## Documentation

- Getting started vignette: [vignettes/getting-started.Rmd](vignettes/getting-started.Rmd)
- Model comparison vignette: [vignettes/model-comparison.Rmd](vignettes/model-comparison.Rmd)
- Simulation tools vignette: [vignettes/simulation-tools.Rmd](vignettes/simulation-tools.Rmd)
- Full Quarto walkthrough: [vignettes/dcvar-walkthrough.qmd](https://github.com/benlug/dcvar/blob/main/vignettes/dcvar-walkthrough.qmd)
- Source code and issue tracker: <https://github.com/benlug/dcvar>

## Citation

If you use dcvar in your work, cite it with:

```r
citation("dcvar")
```

## Getting Help

- Report bugs or request features at <https://github.com/benlug/dcvar/issues>
- For usage questions, include a minimal reproducible example when possible

# dcvar: Dynamic Copula VAR Models for Time-Varying Dependence

[<img src="man/figures/logo.png" align="right" width="15%" height="15%" alt="dcvar logo"/>](https://github.com/benlug/dcvar)

<!-- badges: start -->
[![R-CMD-check](https://github.com/benlug/dcvar/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/benlug/dcvar/actions/workflows/R-CMD-check.yaml)
[![Lifecycle: experimental](https://img.shields.io/badge/lifecycle-experimental-orange.svg)](https://lifecycle.r-lib.org/articles/stages.html#experimental)
[![License: GPL v3+](https://img.shields.io/badge/License-GPL%20v3%2B-blue.svg)](https://www.gnu.org/licenses/gpl-3.0.en.html)
<!-- badges: end -->

`dcvar` is an R package for fitting Bayesian copula VAR(1) models to bivariate
time series. Most bundled models use Gaussian copulas; the constant-copula
baseline also supports a Clayton copula with normal margins. The core scope is
single-level dynamic, regime-switching, and constant-copula specifications, all
estimated through [Stan](https://mc-stan.org/). The package also ships
experimental multilevel and SEM extensions.

## Installation

`dcvar` uses [`rstan`](https://mc-stan.org/rstan/) as its default backend.

Install `dcvar` from CRAN:

```r
install.packages("dcvar")
```

For the development version:

```r
install.packages("remotes")
remotes::install_github("benlug/dcvar")
```

Optionally, you can use [`cmdstanr`](https://mc-stan.org/cmdstanr/) as an
alternative backend:

```r
install.packages(
  "cmdstanr",
  repos = c("https://stan-dev.r-universe.dev", getOption("repos"))
)
cmdstanr::install_cmdstan()
```

CI includes a dedicated Ubuntu release lane that runs the `backend = "cmdstanr"`
regression tests when both `cmdstanr` and CmdStan are available.

For skew-normal margins, install `sn`:

```r
install.packages("sn")
```

## Example

The example below simulates a bivariate time series with decreasing
dependence, fits the baseline DC-VAR model, and compares it to HMM and
constant-copula alternatives.

```r
library(dcvar)

# simulate data with decreasing coupling
sim <- simulate_dcvar(
  n_time = 150,
  rho_trajectory = rho_decreasing(150, rho_start = 0.7, rho_end = 0.3)
)

# fit the DC-VAR model
fit <- dcvar(sim$Y_df, vars = c("y1", "y2"))

# inspect results
summary(fit)
plot_rho(fit, true_rho = sim$true_params$rho)

# compare models via LOO-CV
fit_hmm <- dcvar_hmm(sim$Y_df, vars = c("y1", "y2"), K = 2)
fit_con <- dcvar_constant(sim$Y_df, vars = c("y1", "y2"))
dcvar_compare(dcvar = fit, hmm = fit_hmm, constant = fit_con)
```

## Supported Models

| Model | Function | Dependence Structure | Status |
| --- | --- | --- | --- |
| **DC-VAR** | `dcvar()` | Continuous random-walk on Fisher-z scale | Core |
| **HMM Copula** | `dcvar_hmm()` | Discrete regime-switching with K states | Core |
| **Constant Copula** | `dcvar_constant()` | Time-invariant Gaussian or Clayton baseline | Core |
| **Multilevel** | `dcvar_multilevel()` | Random VAR coefficients for panel data | Experimental |
| **SEM** | `dcvar_sem()` | Fixed measurement model for latent processes | Experimental |

Most models use Gaussian copulas. The core three time-series models
(`dcvar()`, `dcvar_hmm()`, and `dcvar_constant(copula = "gaussian")`) support
four marginal distributions: **normal**, **exponential**, **skew-normal**, and
**gamma**. `dcvar_constant(copula = "clayton")` supports a Clayton-copula
baseline with normal margins. The multilevel and SEM variants support normal
and exponential margins.

If you are reading the accompanying manuscript, note that `dcvar` now includes
the constant Clayton-copula baseline, the exponential-margin multilevel model,
and naive SEM score models used in the simulation studies.

`fitted()` and `predict()` are implemented for the public fit classes. The
multilevel methods return unit-specific trajectories and intervals; the SEM
methods support both latent-state (`type = "link"`) and indicator-scale
(`type = "response"`) summaries. `pit_values()` and `pit_test()` currently
support the single-level models only. `loo()` supports single-level fits,
covariate fits, exponential-margin multilevel fits, and naive SEM score fits.
`plot_ppc()` is available for normal and exponential margins; gamma and
skew-normal fits do not yet have replicated residuals on the observed margin
scale.

`pit_values()` and `pit_test()` are approximate residual diagnostics based on
posterior means. They are useful as heuristic checks, not exact posterior
predictive calibration tests.

## Documentation

- Getting started vignette: [vignettes/getting-started.Rmd](vignettes/getting-started.Rmd)
- Model comparison vignette: [vignettes/model-comparison.Rmd](vignettes/model-comparison.Rmd)
- Simulation tools vignette: [vignettes/simulation-tools.Rmd](vignettes/simulation-tools.Rmd)
- Full Quarto walkthrough: [vignettes/dcvar-walkthrough.qmd](https://github.com/benlug/dcvar/blob/main/vignettes/dcvar-walkthrough.qmd)
- Source code and issue tracker: <https://github.com/benlug/dcvar>

## Citation

If you use `dcvar` in your work, cite it with:

```r
citation("dcvar")
```

## Getting Help

- Report bugs or request features at <https://github.com/benlug/dcvar/issues>
- For usage questions, include a minimal reproducible example when possible

## ----setup, include = FALSE---------------------------------------------------
has_stan <- requireNamespace("rstan", quietly = TRUE)
eval_fits <- identical(Sys.getenv("DCVAR_EVAL_VIGNETTES"), "true") && has_stan
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = eval_fits
)

## ----trajectories, eval = TRUE------------------------------------------------
library(dcvar)

# Smooth trajectories
rho_constant(100, rho = 0.5)
rho_decreasing(100, rho_start = 0.7, rho_end = 0.3)
rho_increasing(100, rho_start = 0.3, rho_end = 0.7)
rho_random_walk(100, sigma_omega = 0.05, seed = 42)

# Step-function trajectories
rho_step(100, rho_before = 0.7, rho_after = 0.3, breakpoint = 0.5)
rho_double_step(100, rho_levels = c(0.7, 0.3, 0.7))

# Named scenarios
rho_scenario("double_relapse", n_time = 150)

## ----plot-trajectories, eval = TRUE-------------------------------------------
# Compare all built-in scenarios
plot_trajectories(100)

# Subset of scenarios
plot_trajectories(100, scenarios = c("decreasing", "single_middle", "double_relapse"))

## ----simulate, eval = TRUE----------------------------------------------------
sim <- simulate_dcvar(
  n_time = 150,
  rho_trajectory = rho_decreasing(150),
  mu = c(0, 0),
  Phi = matrix(c(0.3, 0.1, 0.1, 0.3), 2, 2),
  sigma_eps = c(1, 1),
  seed = 42
)

## ----breakpoint, eval = TRUE--------------------------------------------------
# Single breakpoint
sim_bp <- simulate_breakpoint_data(n_time = 100, type = "single", seed = 42)
head(sim_bp$Y_df)
sim_bp$true_params$breakpoint

# Double breakpoint
sim_bp2 <- simulate_breakpoint_data(n_time = 100, type = "double", seed = 42)

## ----prepare, eval = TRUE-----------------------------------------------------
df <- data.frame(time = 1:100, y1 = rnorm(100), y2 = rnorm(100))

# DC-VAR data (includes sigma_omega prior)
stan_data <- prepare_dcvar_data(df, vars = c("y1", "y2"))
str(stan_data[c("n_time", "D", "sigma_mu_prior", "sigma_omega_prior")])

# HMM data (includes K, kappa, alpha_off)
hmm_data <- prepare_hmm_data(df, vars = c("y1", "y2"), K = 3)
str(hmm_data[c("K", "kappa", "alpha_off")])

# Constant data (simplest)
const_data <- prepare_constant_data(df, vars = c("y1", "y2"))

## ----metrics-fit, include = FALSE---------------------------------------------
# fit <- dcvar(
#   sim$Y_df,
#   vars = c("y1", "y2"),
#   chains = 2,
#   iter_warmup = 250,
#   iter_sampling = 250,
#   seed = 99,
#   refresh = 0
# )

## ----metrics------------------------------------------------------------------
# # Extract estimated rho
# rho_df <- rho_trajectory(fit)
# 
# # Compute recovery metrics
# metrics <- compute_rho_metrics(
#   rho_true = sim$true_params$rho,
#   rho_est = rho_df$mean,
#   rho_lower = rho_df$q2.5,
#   rho_upper = rho_df$q97.5
# )
# 
# metrics$bias
# metrics$coverage
# metrics$correlation

## ----mini-study---------------------------------------------------------------
# n_reps <- 5
# results <- lapply(1:n_reps, function(i) {
#   sim <- simulate_dcvar(
#     n_time = 100,
#     rho_trajectory = rho_decreasing(100),
#     seed = i * 1000
#   )
# 
#   fit <- dcvar(sim$Y_df, vars = c("y1", "y2"),
#                chains = 2, iter_warmup = 500, iter_sampling = 1000,
#                seed = i, refresh = 0)
# 
#   rho_df <- rho_trajectory(fit)
# 
#   list(rho = compute_rho_metrics(
#     rho_true = sim$true_params$rho,
#     rho_est = rho_df$mean,
#     rho_lower = rho_df$q2.5,
#     rho_upper = rho_df$q97.5
#   ))
# })
# 
# # Aggregate across replications
# agg <- aggregate_metrics(results)
# print(agg$rho)

## ----setup, include = FALSE---------------------------------------------------
has_stan <- requireNamespace("rstan", quietly = TRUE)
eval_fits <- identical(Sys.getenv("DCVAR_EVAL_VIGNETTES"), "true") && has_stan
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = eval_fits
)

## ----install, eval = FALSE----------------------------------------------------
# # Install dcvar from CRAN
# install.packages("dcvar")

## ----install-dev, eval = FALSE------------------------------------------------
# install.packages("remotes")
# remotes::install_github("benlug/dcvar")

## ----install-cmdstanr, eval = FALSE-------------------------------------------
# install.packages("cmdstanr",
#   repos = c("https://stan-dev.r-universe.dev", getOption("repos"))
# )
# cmdstanr::install_cmdstan()

## ----simulate, eval = TRUE----------------------------------------------------
library(dcvar)

# Generate a decreasing rho trajectory (therapy effect)
rho_true <- rho_decreasing(n_time = 150, rho_start = 0.7, rho_end = 0.3)

# Simulate bivariate VAR(1) data with this trajectory
sim <- simulate_dcvar(
  n_time = 150,
  rho_trajectory = rho_true,
  seed = 42
)

head(sim$Y_df)

## ----fit----------------------------------------------------------------------
# fit <- dcvar(
#   data = sim$Y_df,
#   vars = c("y1", "y2"),
#   chains = 4,
#   iter_warmup = 1000,
#   iter_sampling = 2000,
#   seed = 123
# )

## ----inspect------------------------------------------------------------------
# # Quick overview
# print(fit)
# 
# # Detailed summary
# summary(fit)
# 
# # Posterior means for VAR parameters
# coef(fit)
# 
# # Time-varying rho as a data frame
# rho_df <- rho_trajectory(fit)
# head(rho_df)
# 
# # MCMC diagnostics
# dcvar_diagnostics(fit)

## ----fitted-predict-----------------------------------------------------------
# # VAR(1) fitted values: y_hat[t] = mu + Phi * (y[t-1] - mu)
# fit_df <- fitted(fit)
# head(fit_df)
# 
# # Predictions with 95% marginal intervals
# pred_df <- predict(fit)
# head(pred_df)
# 
# # Custom interval level
# pred_90 <- predict(fit, ci_level = 0.90)

## ----as-data-frame------------------------------------------------------------
# param_df <- as.data.frame(fit)
# head(param_df)

## ----interpret----------------------------------------------------------------
# interpret_rho_trajectory(fit)

## ----plot---------------------------------------------------------------------
# # Rho trajectory with credible intervals
# # Red line = true trajectory (simulation only)
# plot_rho(fit, true_rho = sim$true_params$rho)
# 
# # VAR coefficient heatmap
# plot_phi(fit)
# 
# # MCMC diagnostics (trace, Rhat, ESS)
# plot(fit, type = "diagnostics")
# 
# # Posterior predictive check
# # Currently available for normal and exponential margins
# plot_ppc(fit)

## ----compare------------------------------------------------------------------
# fit_constant <- dcvar_constant(sim$Y_df, vars = c("y1", "y2"))
# 
# dcvar_compare(dcvar = fit, constant = fit_constant)

## ----setup, include = FALSE---------------------------------------------------
has_stan <- requireNamespace("rstan", quietly = TRUE)
eval_fits <- identical(Sys.getenv("DCVAR_EVAL_VIGNETTES"), "true") && has_stan
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = eval_fits
)

## ----fit-all------------------------------------------------------------------
# library(dcvar)
# 
# # Simulate data with a step-function trajectory
# sim <- simulate_dcvar(
#   n_time = 200,
#   rho_trajectory = rho_step(200, rho_before = 0.7, rho_after = 0.3),
#   seed = 42
# )
# 
# # Fit all three models on the same data
# fit_dc  <- dcvar(sim$Y_df, vars = c("y1", "y2"), seed = 1)
# fit_hmm <- dcvar_hmm(sim$Y_df, vars = c("y1", "y2"), K = 2, seed = 2)
# fit_con <- dcvar_constant(sim$Y_df, vars = c("y1", "y2"), seed = 3)

## ----compare------------------------------------------------------------------
# dcvar_compare(dcvar = fit_dc, hmm = fit_hmm, constant = fit_con)

## ----tidy---------------------------------------------------------------------
# # Full parameter summary as a data frame
# param_df <- as.data.frame(fit_dc)
# head(param_df)
# 
# # Predictions with marginal intervals
# pred_df <- predict(fit_hmm)
# head(pred_df)

## ----hmm----------------------------------------------------------------------
# # State posteriors, Viterbi path, transition matrix
# states <- hmm_states(fit_hmm)
# 
# # State-specific rho values
# states$rho_state
# 
# # Transition matrix
# states$A
# 
# # Visualise state posteriors
# plot(fit_hmm, type = "states")
# 
# # Transition matrix heatmap
# plot(fit_hmm, type = "transition")

## ----interpret----------------------------------------------------------------
# interpret_rho_trajectory(fit_dc)
# interpret_rho_trajectory(fit_hmm)
# interpret_rho_trajectory(fit_con)

# ============================================================================
# Data Preparation Functions
# ============================================================================

#' Internal: coerce time-like vectors to a numeric ordering scale
#' @noRd
.time_to_numeric <- function(x) {
  if (inherits(x, "POSIXt") || inherits(x, "Date") || inherits(x, "difftime")) {
    return(as.numeric(x))
  }
  if (is.numeric(x)) {
    return(as.numeric(x))
  }

  xtfrm(x)
}

#' Internal: validate a positive finite scalar
#' @noRd
.prep_validate_positive_scalar <- function(x, arg_name) {
  if (!is.numeric(x) || length(x) != 1L || !is.finite(x) || x <= 0) {
    cli_abort("{.arg {arg_name}} must be a single positive finite numeric value.")
  }
}

#' Internal: validate a scalar logical flag
#' @noRd
.prep_validate_scalar_logical <- function(x, arg_name) {
  if (!is.logical(x) || length(x) != 1L || is.na(x)) {
    cli_abort("{.arg {arg_name}} must be a single logical value.")
  }
}

#' Internal: validate a finite numeric vector
#' @noRd
.prep_validate_numeric_vector <- function(x, arg_name) {
  if (!is.numeric(x) || length(x) == 0L || any(!is.finite(x))) {
    cli_abort("{.arg {arg_name}} must be a non-empty finite numeric vector.")
  }
}

#' Internal: validate that a variable vector has no duplicates
#' @noRd
.prep_validate_unique_vars <- function(vars, context = "vars") {
  if (anyDuplicated(vars)) {
    cli_abort("{.arg {context}} must contain distinct variable names.")
  }
}

#' Internal: validate ordered time values for VAR-style models
#' @noRd
.validate_time_values <- function(time_values, allow_gaps = FALSE, context = "data") {
  if (is.character(time_values) || (is.factor(time_values) && !is.ordered(time_values))) {
    cli_abort(c(
      "{.arg time_var} in {context} is a {.cls {class(time_values)[1]}} column.",
      "!" = "Character and unordered-factor time indexes sort lexicographically (e.g. \"T10\" before \"T2\"), which can silently scramble the time ordering.",
      "i" = "Use a numeric, Date, POSIXct, difftime, or ordered-factor time column."
    ))
  }
  if (anyNA(time_values)) {
    cli_abort("{.arg time_var} contains missing values in {context}.")
  }
  if (length(time_values) < 2) {
    return(invisible(time_values))
  }
  if (anyDuplicated(time_values)) {
    cli_abort(c(
      "Duplicate {.arg time_var} values found in {context}.",
      "i" = "VAR-style models require a unique observation time for each row."
    ))
  }

  steps <- diff(.time_to_numeric(time_values))
  if (any(!is.finite(steps)) || any(steps <= 0)) {
    cli_abort(c(
      "{.arg time_var} in {context} could not be ordered into a strictly increasing sequence.",
      "i" = "Use a numeric, Date, POSIXct, or otherwise orderable time index."
    ))
  }

  regular_steps <- isTRUE(all.equal(
    steps,
    rep(steps[[1]], length(steps)),
    tolerance = sqrt(.Machine$double.eps)
  ))
  if (!regular_steps) {
    if (!allow_gaps) {
      cli_abort(c(
        "{.arg time_var} in {context} is not evenly spaced.",
        "!" = "The VAR(1) model assumes consecutive, equally spaced observations.",
        "i" = "Reindex the series to consecutive time steps or set {.arg allow_gaps = TRUE} to proceed with a warning."
      ))
    }
    cli_warn(c(
      "{.arg time_var} in {context} is not evenly spaced.",
      "!" = "The VAR(1) model still treats adjacent rows as consecutive observations.",
      "i" = "Interpret lagged quantities with caution."
    ))
  }

  invisible(time_values)
}

#' Internal: derive the common multilevel time grid
#' @noRd
.validate_panel_time_grid <- function(data, ids, id_var, time_var) {
  time_grid <- NULL

  for (id in ids) {
    # Double-bracket extraction keeps this a plain vector for data.frame
    # subclasses (a tibble would return a 1-column tibble of length 1 and
    # silently skip every per-unit check below).
    unit_times <- data[[time_var]][data[[id_var]] == id]
    .validate_time_values(unit_times, allow_gaps = FALSE, context = paste0("unit ", id))

    if (is.null(time_grid)) {
      time_grid <- unit_times
    } else {
      same_grid <- isTRUE(all.equal(time_grid, unit_times, check.attributes = TRUE))
      if (!same_grid) {
        cli_abort(c(
          "All units must share the same {.arg time_var} grid.",
          "!" = "The multilevel model assumes aligned measurement occasions across units."
        ))
      }
    }
  }

  time_grid
}

#' Internal: shared data preparation logic for all copula VAR models
#'
#' @param data A data frame.
#' @param vars Character vector of exactly two variable names.
#' @param time_var Name of the time column.
#' @param standardize Logical scalar; whether to z-score the variables.
#' @param allow_gaps Logical scalar; if `FALSE` (default), interior missing values
#'   cause an error. If `TRUE`, they produce a warning and are removed.
#'
#' @return A list with elements `Y`, `T_obs`, `D`, and standardization metadata.
#' @noRd
.prepare_var_data <- function(data, vars, time_var = "time",
                              standardize = TRUE, allow_gaps = FALSE) {
  # Validate inputs
  if (!is.data.frame(data)) {
    cli_abort("{.arg data} must be a data frame.")
  }
  # Plain-data.frame semantics throughout: single-bracket column subsetting on
  # a tibble returns a tibble, not a vector, which breaks the time-axis
  # bookkeeping downstream.
  data <- as.data.frame(data)
  if (nrow(data) == 0) {
    cli_abort("{.arg data} must not be empty.")
  }
  if (length(vars) != 2) {
    cli_abort("Exactly 2 variables required (bivariate model). Got {.val {length(vars)}}.")
  }
  .prep_validate_unique_vars(vars)
  .prep_validate_scalar_logical(standardize, "standardize")
  .prep_validate_scalar_logical(allow_gaps, "allow_gaps")
  missing_vars <- setdiff(c(vars, time_var), names(data))
  if (length(missing_vars) > 0) {
    cli_abort("Column{?s} not found in data: {.val {missing_vars}}")
  }
  for (v in vars) {
    if (!is.numeric(data[[v]])) {
      cli_abort("Column {.val {v}} must be numeric, got {.cls {class(data[[v]])[1]}}.")
    }
  }

  # Sort by time
  data <- data[order(data[[time_var]]), , drop = FALSE]
  time_values <- data[[time_var]]
  .validate_time_values(time_values, allow_gaps = allow_gaps)

  # Extract variables
  Y <- as.matrix(data[, vars])

  # Check for non-finite values
  if (any(is.nan(Y) | is.infinite(Y))) {
    cli_abort("{.arg data} contains NaN or Inf values. Remove or impute them first.")
  }

  # Handle missing values
  cc <- complete.cases(Y)
  if (sum(!cc) > 0) {
    if (!any(cc)) {
      cli_abort("{.arg data} contains no complete rows for the selected variables.")
    }
    n_missing <- sum(!cc)
    missing_idx <- which(!cc)
    # Only NA rows strictly inside the first..last complete-case range break
    # adjacency; contiguous NA runs at either edge can be trimmed safely.
    keep_range <- range(which(cc))
    interior <- missing_idx[missing_idx > keep_range[1] & missing_idx < keep_range[2]]

    if (length(interior) > 0) {
      n_interior <- length(interior)
      if (!allow_gaps) {
        cli_abort(c(
          "Found {n_interior} interior missing value{?s} that would break time series adjacency.",
          "!" = "The VAR(1) model assumes consecutive observations.",
          "i" = "Consider imputation or explicit missing-data handling.",
          "i" = "Set {.arg allow_gaps = TRUE} to remove these rows with a warning instead."
        ))
      }
      cli_warn(c(
        "Removing {n_missing} row{?s} with missing values.",
        "!" = "This breaks time series adjacency: the VAR(1) model assumes consecutive observations.",
        "i" = "Consider imputation or explicit missing-data handling for real data."
      ))
    } else {
      cli_warn("Removing {n_missing} row{?s} with missing values (edges only).")
    }
    Y <- Y[cc, , drop = FALSE]
    data <- data[cc, , drop = FALSE]
    time_values <- time_values[cc]
  }

  # Require at least 3 observations for a VAR(1) model
  if (nrow(Y) < 3) {
    cli_abort("At least 3 observations required for a VAR(1) model, got {.val {nrow(Y)}}.")
  }

  # Standardize if requested
  Y_means <- NULL
  Y_sds <- NULL
  if (standardize) {
    Y_means <- colMeans(Y)
    Y_sds <- apply(Y, 2, sd)
    if (any(Y_sds < 1e-10)) {
      cli_abort("Cannot standardize: one or more variables have zero (or near-zero) variance.")
    }
    Y <- scale(Y)
  }

  list(
    Y = Y,
    T_obs = nrow(Y),
    D = ncol(Y),
    standardized = standardize,
    time_values = time_values,
    Y_means = Y_means,
    Y_sds = Y_sds,
    vars = vars,
    data = data
  )
}


#' Internal: add the per-dimension family array for multilevel/SEM mixed models
#'
#' Like [.add_margin_stan_data()] but without `sigma_eps_prior` (the multilevel
#' and SEM models parameterise the normal innovation scale through their own
#' `prior_sigma_sd`, not an exponential rate).
#' @noRd
.add_mixed_family_data <- function(stan_data, margins, D, skew_direction) {
  margins_vec <- if (length(margins) == 1L) rep(margins, D) else margins
  stan_data$family <- unname(as.integer(.family_codes[margins_vec]))
  if (any(margins_vec %in% c("exponential", "gamma")) && is.null(skew_direction)) {
    cli_abort(c(
      "Mixed margins {.val {margins_vec}} require {.arg skew_direction}.",
      "i" = "{.arg skew_direction} must be a length-{.val {D}} vector with values 1 or -1."
    ))
  }
  stan_data$skew_direction <- if (is.null(skew_direction)) rep(1, D) else as.numeric(skew_direction)
  stan_data
}

#' Internal: add margin-specific entries to a Stan data list
#'
#' Shared by the constant, DC-VAR, and HMM data-prep functions, which build the
#' same margin-dependent Stan data: for a mixed (per-variable) specification an
#' integer `family` array, the normal innovation prior, and a `skew_direction`
#' vector (defaulting to `+1` on dimensions that do not consult it); for a single
#' family the normal prior or the exponential/gamma `skew_direction`.
#'
#' @param stan_data The Stan data list to extend.
#' @param margins Normalised margin spec (length-1, or length-D when mixed).
#' @param D Number of variables.
#' @param skew_direction Length-D `+1/-1` vector, or `NULL`.
#' @param prior_sigma_eps_rate Prior mean for the normal innovation SDs.
#' @return The extended Stan data list.
#' @noRd
.add_margin_stan_data <- function(stan_data, margins, D, skew_direction,
                                  prior_sigma_eps_rate) {
  if (length(margins) > 1L && length(margins) != D) {
    cli_abort("{.arg margins} must have length 1 or {.val {D}}, got {.val {length(margins)}}.")
  }

  # sigma_eps (and its prior) only exists for normal dimensions; a non-default
  # prior_sigma_eps_rate with no normal dimension would be silently ignored.
  margins_all <- if (length(margins) == 1L) rep(margins, D) else margins
  if (!any(margins_all == "normal") && !isTRUE(all.equal(prior_sigma_eps_rate, 1))) {
    cli_warn(c(
      "{.arg prior_sigma_eps_rate} is ignored: no dimension uses a {.val normal} margin.",
      "i" = "The {.val {unique(margins_all)}} margin scale priors are fixed in the Stan models."
    ))
  }

  if (.is_mixed_margins(margins)) {
    margins_vec <- if (length(margins) == 1L) rep(margins, D) else margins
    stan_data$family <- unname(as.integer(.family_codes[margins_vec]))
    stan_data$sigma_eps_prior <- prior_sigma_eps_rate
    if (any(margins_vec %in% c("exponential", "gamma")) && is.null(skew_direction)) {
      cli_abort(c(
        "Mixed margins {.val {margins_vec}} require {.arg skew_direction}.",
        "i" = "{.arg skew_direction} must be a length-{.val {D}} vector with values 1 or -1."
      ))
    }
    stan_data$skew_direction <- if (is.null(skew_direction)) {
      rep(1, D)
    } else {
      as.numeric(skew_direction)
    }
  } else {
    m <- margins[[1L]]
    if (m == "normal") {
      stan_data$sigma_eps_prior <- prior_sigma_eps_rate
    }
    if (m %in% c("exponential", "gamma")) {
      if (is.null(skew_direction)) {
        cli_abort(c(
          "Margin {.val {m}} requires {.arg skew_direction}.",
          "i" = "{.arg skew_direction} must be a numeric vector of length {.val {D}} with values 1 or -1."
        ))
      }
      stan_data$skew_direction <- as.numeric(skew_direction)
    }
  }

  stan_data
}


#' Internal: augment a prepared Stan data list into the dcvar_dynamic.stan block
#'
#' The unified engine `dcvar_dynamic.stan` has a superset data block: the
#' time-varying margin codes (`family`, `skew_direction`, `tv_phi`,
#' `phi_tv_mask`, `tv_sigma`, the TV priors, `barrier_k`) plus the covariate
#' predictor (`P`, `X`, `sigma_beta_prior`), the residual-drift convention
#' (`zero_init_eta`), and the copula fast-path flag (`copula_normal_fastpath`).
#' Either prepare function produces a subset of these; this shim fills in every
#' field the engine declares, preferring values already present so the TV path
#' keeps its margin/TV settings and the covariate path keeps its `P`/`X`/drift
#' settings. It is run inside the fit engines just before sampling and never
#' inside the exported prepare functions, so their byte-identical outputs are
#' preserved (the flags-off regression tests stay green).
#'
#' @param stan_data A list produced by [prepare_dcvar_data()] (TV path) or
#'   [prepare_dcvar_covariate_data()] (covariate path).
#' @return The same list augmented with the full `dcvar_dynamic.stan` data block.
#' @noRd
.as_dynamic_stan_data <- function(stan_data) {
  D <- stan_data$D
  n_time <- stan_data$n_time

  # Time-varying margin block (present for the TV path, absent for covariate).
  stan_data$family <- stan_data$family %||% rep(1L, D)
  stan_data$skew_direction <- stan_data$skew_direction %||% rep(1, D)
  stan_data$tv_phi <- stan_data$tv_phi %||% 0L
  stan_data$phi_tv_mask <- stan_data$phi_tv_mask %||% rep(0L, 4L)
  stan_data$tv_sigma <- stan_data$tv_sigma %||% 0L
  stan_data$tau_phi_prior <- stan_data$tau_phi_prior %||% 0.025
  stan_data$tau_sigma_prior <- stan_data$tau_sigma_prior %||% 0.05
  stan_data$barrier_k <- stan_data$barrier_k %||% 8
  stan_data$sigma_eps_prior <- stan_data$sigma_eps_prior %||% 1

  # Covariate predictor + residual-drift block (present for covariate,
  # absent for the TV path, where P = 0 makes the predictor identically zero).
  stan_data$P <- stan_data$P %||% 0L
  if (is.null(stan_data$X)) {
    stan_data$X <- matrix(0, nrow = n_time, ncol = 0L)
  }
  stan_data$sigma_beta_prior <- stan_data$sigma_beta_prior %||% 1
  stan_data$zero_init_eta <- stan_data$zero_init_eta %||% 0L
  stan_data$copula_normal_fastpath <- stan_data$copula_normal_fastpath %||% 0L

  stan_data
}


#' Internal: build the K x D skew-direction matrix for the switching HMM
#'
#' Only exp/gamma dimensions consult skew; others default to +1 (inert). Accepts
#' a length-D vector (recycled across states) or a length-K list of length-D
#' vectors, and errors (naming the state) when a dimension that needs skew has
#' none supplied.
#' @noRd
.hmm_skew_matrix <- function(skew_direction, margins_char, K, D = 2L) {
  needs <- matrix(margins_char %in% c("exponential", "gamma"), nrow = K, ncol = D)
  skew <- matrix(1L, nrow = K, ncol = D)
  if (!any(needs)) {
    return(skew)
  }

  if (is.list(skew_direction)) {
    if (length(skew_direction) != K) {
      cli_abort("Per-state {.arg skew_direction} must be a length-{K} list aligned with {.arg margins}.")
    }
    skew_in <- do.call(rbind, lapply(seq_len(K), function(k) {
      s <- skew_direction[[k]]
      if (is.null(s) || length(s) != D || !all(s %in% c(-1, 1))) {
        cli_abort("Each per-state {.arg skew_direction} element must be a length-{D} vector of +1/-1.")
      }
      as.integer(s)
    }))
  } else if (is.null(skew_direction)) {
    skew_in <- NULL
  } else {
    if (length(skew_direction) != D || !all(skew_direction %in% c(-1, 1))) {
      cli_abort("{.arg skew_direction} must be a length-{D} vector of +1/-1 (or a length-{K} list for per-state margins).")
    }
    skew_in <- matrix(as.integer(skew_direction), nrow = K, ncol = D, byrow = TRUE)
  }

  for (k in seq_len(K)) {
    for (d in seq_len(D)) {
      if (needs[k, d]) {
        if (is.null(skew_in)) {
          cli_abort(c(
            "State {k} margin {.val {margins_char[k, d]}} requires {.arg skew_direction}.",
            "i" = "Supply a length-{D} +1/-1 vector (or a length-{K} list for per-state margins)."
          ))
        }
        skew[k, d] <- skew_in[k, d]
      }
    }
  }
  skew
}


#' Internal: resolve the HMM margin configuration (global or per-state)
#'
#' Normalises the `margins` argument of [dcvar_hmm()] into the K x D family-code
#' and skew matrices the switching engine needs, validating both the global form
#' (scalar / length-D) and the per-state form (length-K list). An all-identical
#' per-state list collapses back to the global form so the legacy default path is
#' never disturbed.
#'
#' @return A list with `per_state` (logical, post-collapse), `margins_global`
#'   (the global spec when not per-state, else `NULL`), `margins_char` (K x D
#'   character matrix), `family` (K x D integer codes), `skew` (K x D integer),
#'   and `per_state_differ` (logical).
#' @noRd
.hmm_margin_config <- function(margins, skew_direction, K, D = 2L) {
  input_is_list <- is.list(margins) && !is.data.frame(margins)
  per_state <- input_is_list

  if (input_is_list) {
    if (length(margins) != K) {
      cli_abort(c(
        "Per-state {.arg margins} must be a length-{K} list (one family spec per state).",
        "i" = "Got a list of length {length(margins)}."
      ))
    }
    rows <- lapply(seq_len(K), function(k) {
      m <- margins[[k]]
      .validate_margin_families(m)
      if (length(m) == 1L) rep(m, D) else m
    })
    margins_char <- do.call(rbind, rows)
    margins_global <- NULL
  } else {
    margins_global <- .normalize_margins_spec(margins)
    .validate_margin_families(margins_global)
    row <- if (length(margins_global) == 1L) rep(margins_global, D) else margins_global
    margins_char <- matrix(row, nrow = K, ncol = D, byrow = TRUE)
  }

  skew <- .hmm_skew_matrix(skew_direction, margins_char, K, D)

  margins_differ <- input_is_list && nrow(unique(margins_char)) > 1L
  skew_differ <- input_is_list &&
    any(margins_char %in% c("exponential", "gamma")) &&
    nrow(unique(skew)) > 1L

  if (input_is_list && !margins_differ && !skew_differ) {
    # Identical across states, including any consulted skew orientation: collapse
    # to the global form so the legacy path remains available.
    per_state <- FALSE
    margins_global <- .normalize_margins_spec(margins_char[1, ])
  }

  family <- matrix(as.integer(.family_codes[as.vector(margins_char)]), nrow = K, ncol = D)
  per_state_differ <- margins_differ || skew_differ
  skew_global <- if (!per_state && any(margins_char[1, ] %in% c("exponential", "gamma"))) {
    as.integer(skew[1, ])
  } else {
    NULL
  }

  list(
    per_state = per_state,
    margins_global = margins_global,
    margins_char = margins_char,
    family = family,
    skew = skew,
    skew_global = skew_global,
    per_state_differ = per_state_differ
  )
}


#' Internal: augment prepared HMM data into the hmm_switching.stan block
#'
#' Runs inside [dcvar_hmm()] just before sampling (never inside the exported
#' [prepare_hmm_data()], preserving its byte-identical default output). Overrides
#' the base family/skew with the K x D matrices and adds the switch flags and the
#' Phi-deviation prior SD.
#' @noRd
.as_hmm_switching_stan_data <- function(stan_data, switch_spec, family_mat, skew_mat,
                                        prior_phi_dev_sd) {
  stan_data$family <- family_mat
  stan_data$skew_direction <- skew_mat
  stan_data$switch_mu <- as.integer(switch_spec$mu)
  stan_data$switch_phi <- as.integer(any(switch_spec$phi_mask > 0L))
  stan_data$phi_switch_mask <- unname(as.integer(switch_spec$phi_mask))
  stan_data$switch_margins <- as.integer(switch_spec$margins)
  stan_data$sigma_eps_prior <- stan_data$sigma_eps_prior %||% 1
  stan_data$prior_phi_dev_sd <- prior_phi_dev_sd
  stan_data
}


#' Prepare data for the DC-VAR model
#'
#' Transforms a data frame into a list suitable for the DC-VAR Stan model.
#' Handles sorting, missing values, and optional standardization.
#'
#' @section Prior naming conventions:
#' Parameters ending in `_sd` specify normal prior standard deviations
#' (location parameters). Parameters ending in `_rate` specify exponential
#' prior means (scale parameters), where the exponential rate is
#' `1/prior_*_rate`. The constant and HMM models use `prior_z_rho_sd`
#' (normal prior on the Fisher-z scale), while the DC-VAR model uses
#' `prior_sigma_omega_rate` (exponential prior on the random-walk SD)
#' because the two quantities have fundamentally different roles.
#'
#' @param data A data frame with time series observations.
#' @param vars Character vector of two variable names to model.
#' @param time_var Name of the time column (default: `"time"`).
#' @param standardize Logical; whether to z-score variables (default: `TRUE`).
#' @param margins Marginal distribution specification. Either a single string
#'   applied to both variables, or a length-2 character vector giving a
#'   per-variable (mixed) margin (for example `c("normal", "exponential")`).
#'   Each entry is one of `"normal"` (default), `"exponential"`,
#'   `"skew_normal"`, or `"gamma"`. When the two entries differ the returned
#'   data carries an integer `family` array for the generic mixed-margins Stan
#'   model; identical entries are equivalent to the scalar form and reuse the
#'   specialised single-family model.
#' @param skew_direction Integer vector of length D indicating skew direction
#'   for asymmetric margins. Each element must be `1` (right-skewed) or `-1`
#'   (left-skewed). Required whenever any dimension uses an `"exponential"` or
#'   `"gamma"` margin.
#' @param prior_mu_sd Prior SD for intercepts: `mu ~ normal(0, prior_mu_sd)`.
#' @param prior_phi_sd Prior SD for VAR coefficients: `Phi ~ normal(0, prior_phi_sd)`.
#' @param prior_sigma_eps_rate Prior mean for innovation SDs:
#'   `sigma_eps ~ exponential(1/prior_sigma_eps_rate)`. Default `1` gives
#'   `exponential(1)` with prior mean 1.
#' @param prior_sigma_omega_rate Prior mean for rho process SD:
#'   `sigma_omega ~ exponential(1/prior_sigma_omega_rate)`. Default `0.1` gives
#'   `exponential(10)` with prior mean 0.1.
#' @param prior_rho_init_sd Prior SD for initial rho on Fisher-z scale.
#' @param tv_phi Selects which VAR(1) coefficients evolve as random walks
#'   around the constant baseline `Phi`. Either a logical (`TRUE` = all four,
#'   `FALSE` = none, the default) or a character selector: `"ar"` (the
#'   autoregressive effects phi11, phi22), `"cross"` (the cross-lagged effects
#'   phi12, phi21), or specific names from
#'   `c("phi11", "phi12", "phi21", "phi22")`.
#' @param tv_sigma Logical; if `TRUE`, the residual scales evolve as log-scale
#'   random walks around their constant baselines (default: `FALSE`). Applies
#'   to all margin families; exponential and gamma dimensions use a
#'   soft-barrier construction (see [dcvar()]).
#' @param prior_tau_phi_rate Prior mean for the four Phi random-walk
#'   innovation SDs: `tau_phi ~ exponential(1/prior_tau_phi_rate)`. Only used
#'   when `tv_phi = TRUE`. The default `0.025` allows a total drift SD of
#'   about 0.3 over 150 time points while shrinking toward constant
#'   coefficients.
#' @param prior_tau_sigma_rate Prior mean for the per-dimension log-scale
#'   random-walk innovation SDs:
#'   `tau_sigma ~ exponential(1/prior_tau_sigma_rate)`. Only used when
#'   `tv_sigma = TRUE`. The default `0.05` allows scale factors of roughly
#'   0.5--2 over 150 time points.
#' @param tv_sigma_k Soft-barrier sharpness for time-varying exponential/gamma
#'   scales (default `8`); see [dcvar()].
#' @param allow_gaps Logical; if `FALSE` (default), interior missing values
#'   cause an error. If `TRUE`, they produce a warning and are removed.
#'
#' @return A named list suitable as Stan data input.
#' @export
prepare_dcvar_data <- function(data, vars, time_var = "time",
                               standardize = TRUE,
                               margins = "normal",
                               skew_direction = NULL,
                               prior_mu_sd = 2,
                               prior_phi_sd = 0.5,
                               prior_sigma_eps_rate = 1,
                               prior_sigma_omega_rate = 0.1,
                               prior_rho_init_sd = 1,
                               tv_phi = FALSE,
                               tv_sigma = FALSE,
                               prior_tau_phi_rate = 0.025,
                               prior_tau_sigma_rate = 0.05,
                               tv_sigma_k = 8,
                               allow_gaps = FALSE) {
  margins <- .normalize_margins_spec(margins)
  .validate_margins(margins, skew_direction)
  .prep_validate_positive_scalar(prior_mu_sd, "prior_mu_sd")
  .prep_validate_positive_scalar(prior_phi_sd, "prior_phi_sd")
  .prep_validate_positive_scalar(prior_sigma_eps_rate, "prior_sigma_eps_rate")
  .prep_validate_positive_scalar(prior_sigma_omega_rate, "prior_sigma_omega_rate")
  .prep_validate_positive_scalar(prior_rho_init_sd, "prior_rho_init_sd")
  phi_mask <- .resolve_phi_tv_mask(tv_phi)
  any_phi <- sum(phi_mask) > 0L
  .prep_validate_scalar_logical(tv_sigma, "tv_sigma")
  .prep_validate_positive_scalar(prior_tau_phi_rate, "prior_tau_phi_rate")
  .prep_validate_positive_scalar(prior_tau_sigma_rate, "prior_tau_sigma_rate")
  .prep_validate_positive_scalar(tv_sigma_k, "tv_sigma_k")
  prep <- .prepare_var_data(data, vars, time_var, standardize, allow_gaps)

  stan_data <- list(
    n_time = prep$T_obs,
    D = prep$D,
    Y = prep$Y,
    sigma_mu_prior = prior_mu_sd,
    sigma_phi_prior = prior_phi_sd,
    sigma_omega_prior = prior_sigma_omega_rate,
    rho_init_prior_sd = prior_rho_init_sd
  )

  stan_data <- .add_margin_stan_data(stan_data, margins, prep$D, skew_direction,
                                     prior_sigma_eps_rate)

  if (any_phi || tv_sigma) {
    # The generic TV model dispatches on per-dimension family codes and always
    # expects the full margin data, including for homogeneous specifications.
    margins_vec <- rep(margins, length.out = prep$D)
    stan_data$family <- unname(as.integer(.family_codes[margins_vec]))
    stan_data$sigma_eps_prior <- stan_data$sigma_eps_prior %||% prior_sigma_eps_rate
    stan_data$skew_direction <- stan_data$skew_direction %||% rep(1, prep$D)
    stan_data$tv_phi <- as.integer(any_phi)
    stan_data$phi_tv_mask <- unname(phi_mask)
    stan_data$tv_sigma <- as.integer(tv_sigma)
    stan_data$tau_phi_prior <- prior_tau_phi_rate
    stan_data$tau_sigma_prior <- prior_tau_sigma_rate
    stan_data$barrier_k <- tv_sigma_k
    attr(stan_data, "tv_phi") <- any_phi
    attr(stan_data, "phi_tv_mask") <- phi_mask
    attr(stan_data, "tv_sigma") <- tv_sigma
  }

  attr(stan_data, "vars") <- prep$vars
  attr(stan_data, "standardized") <- prep$standardized
  attr(stan_data, "margins") <- margins
  attr(stan_data, "time_values") <- prep$time_values
  if (!is.null(skew_direction)) attr(stan_data, "skew_direction") <- skew_direction
  if (prep$standardized) {
    attr(stan_data, "Y_means") <- prep$Y_means
    attr(stan_data, "Y_sds") <- prep$Y_sds
  }

  stan_data
}


#' Prepare data for the HMM copula model
#'
#' Transforms a data frame into a list suitable for the HMM copula Stan model.
#' Includes HMM-specific prior hyperparameters.
#'
#' @inheritParams prepare_dcvar_data
#' @param K Number of hidden states (default: 2).
#' @param prior_kappa Sticky Dirichlet self-transition concentration (default: 10).
#' @param prior_alpha_off Sticky Dirichlet off-diagonal concentration (default: 1).
#' @param prior_z_rho_sd Prior SD for state-specific z_rho values (default: 1.0).
#'
#' @return A named list suitable as Stan data input.
#' @export
prepare_hmm_data <- function(data, vars, K = 2, time_var = "time",
                             standardize = TRUE,
                             margins = "normal",
                             skew_direction = NULL,
                             prior_mu_sd = 2,
                             prior_phi_sd = 0.5,
                             prior_sigma_eps_rate = 1,
                             prior_kappa = 10,
                             prior_alpha_off = 1,
                             prior_z_rho_sd = 1.0,
                             allow_gaps = FALSE) {
  margins <- .normalize_margins_spec(margins)
  .validate_margins(margins, skew_direction)
  .prep_validate_positive_scalar(prior_mu_sd, "prior_mu_sd")
  .prep_validate_positive_scalar(prior_phi_sd, "prior_phi_sd")
  .prep_validate_positive_scalar(prior_sigma_eps_rate, "prior_sigma_eps_rate")
  .prep_validate_positive_scalar(prior_kappa, "prior_kappa")
  .prep_validate_positive_scalar(prior_alpha_off, "prior_alpha_off")
  .prep_validate_positive_scalar(prior_z_rho_sd, "prior_z_rho_sd")
  if (!is.numeric(K) || length(K) != 1 || K < 2 || K != as.integer(K)) {
    cli_abort("{.arg K} must be an integer >= 2, got {.val {K}}.")
  }
  prep <- .prepare_var_data(data, vars, time_var, standardize, allow_gaps)

  stan_data <- list(
    n_time = prep$T_obs,
    D = prep$D,
    Y = prep$Y,
    K = as.integer(K),
    sigma_mu_prior = prior_mu_sd,
    sigma_phi_prior = prior_phi_sd,
    kappa = prior_kappa,
    alpha_off = prior_alpha_off,
    z_rho_prior_sd = prior_z_rho_sd
  )

  stan_data <- .add_margin_stan_data(stan_data, margins, prep$D, skew_direction,
                                     prior_sigma_eps_rate)

  attr(stan_data, "vars") <- prep$vars
  attr(stan_data, "standardized") <- prep$standardized
  attr(stan_data, "margins") <- margins
  attr(stan_data, "time_values") <- prep$time_values
  if (!is.null(skew_direction)) attr(stan_data, "skew_direction") <- skew_direction
  if (prep$standardized) {
    attr(stan_data, "Y_means") <- prep$Y_means
    attr(stan_data, "Y_sds") <- prep$Y_sds
  }

  stan_data
}


#' Prepare data for the multilevel copula VAR model
#'
#' @param data A data frame in long (panel) format.
#' @param vars Character vector of two variable names.
#' @param id_var Name of the unit/person ID column.
#' @param time_var Name of the time column.
#' @param center Logical scalar; person-mean center (default: `TRUE`).
#' @param prior_phi_bar_sd Prior SD for phi_bar.
#' @param prior_tau_phi_scale Prior scale for tau_phi.
#' @param prior_sigma_sd Prior SD for sigma.
#' @param prior_rho_sd Prior SD for rho.
#' @param tv_phi Selects which population VAR(1) coefficients carry a shared
#'   time-varying drift. See [dcvar()] for accepted selectors.
#' @param tv_sigma Logical; if `TRUE`, residual scales evolve over time.
#' @param prior_tau_phi_rate Prior mean for time-drift Phi random-walk SDs.
#' @param prior_tau_sigma_rate Prior mean for log-scale random-walk SDs.
#' @param tv_sigma_k Soft-barrier sharpness for time-varying exponential/gamma
#'   scales.
#' @param margins Marginal distribution specification. A single string applies
#'   the same family to both variables. Normal and exponential use specialised
#'   Stan data contracts; skew-normal, gamma, and length-2 per-variable specs use
#'   the generic `multilevel_mixed` Stan model with per-dimension family codes.
#' @param skew_direction Length-2 integer vector of +1/-1. Required whenever any
#'   dimension uses an `"exponential"` or `"gamma"` margin.
#'
#' @return A named list suitable as Stan data input.
#' @export
prepare_multilevel_data <- function(data, vars, id_var = "id",
                                    time_var = "time", center = TRUE,
                                    prior_phi_bar_sd = 0.5,
                                    prior_tau_phi_scale = 0.2,
                                    prior_sigma_sd = 1,
                                    prior_rho_sd = 0.5,
                                    tv_phi = FALSE,
                                    tv_sigma = FALSE,
                                    prior_tau_phi_rate = 0.025,
                                    prior_tau_sigma_rate = 0.05,
                                    tv_sigma_k = 8,
                                    margins = "normal",
                                    skew_direction = NULL) {
  if (!is.data.frame(data)) cli_abort("{.arg data} must be a data frame.")
  # Plain-data.frame semantics throughout: single-bracket subsetting on a
  # tibble returns a tibble, not a vector, which silently skips the per-unit
  # time-grid validation and corrupts the stored time axis.
  data <- as.data.frame(data)
  .prep_validate_scalar_logical(center, "center")
  margins <- .normalize_margins_spec(margins)
  .validate_margins(margins, skew_direction)
  if (length(vars) != 2) {
    cli_abort("Exactly 2 variables required for multilevel models. Got {.val {length(vars)}}.")
  }
  .prep_validate_unique_vars(vars)
  .prep_validate_positive_scalar(prior_phi_bar_sd, "prior_phi_bar_sd")
  .prep_validate_positive_scalar(prior_tau_phi_scale, "prior_tau_phi_scale")
  .prep_validate_positive_scalar(prior_sigma_sd, "prior_sigma_sd")
  .prep_validate_positive_scalar(prior_rho_sd, "prior_rho_sd")
  phi_mask <- .resolve_phi_tv_mask(tv_phi)
  any_phi <- sum(phi_mask) > 0L
  .prep_validate_scalar_logical(tv_sigma, "tv_sigma")
  .prep_validate_positive_scalar(prior_tau_phi_rate, "prior_tau_phi_rate")
  .prep_validate_positive_scalar(prior_tau_sigma_rate, "prior_tau_sigma_rate")
  .prep_validate_positive_scalar(tv_sigma_k, "tv_sigma_k")
  missing_cols <- setdiff(c(vars, id_var, time_var), names(data))
  if (length(missing_cols) > 0) {
    cli_abort("Column{?s} not found: {.val {missing_cols}}")
  }
  for (v in vars) {
    if (!is.numeric(data[[v]])) {
      cli_abort("Column {.val {v}} must be numeric, got {.cls {class(data[[v]])[1]}}.")
    }
  }

  id_values <- data[[id_var]]
  if (anyNA(id_values)) {
    cli_abort(c(
      "{.arg id_var} contains missing values.",
      "i" = "Remove or impute missing unit IDs before fitting the multilevel model."
    ))
  }
  if (is.factor(id_values)) {
    id_values <- droplevels(id_values)
    data[[id_var]] <- id_values
  }

  # Split by unit
  ids <- unique(id_values)
  N <- length(ids)

  # Sort within each unit
  data <- data[order(data[[id_var]], data[[time_var]]), , drop = FALSE]

  # Check balanced panel
  counts <- table(id_values)
  if (length(unique(counts)) != 1) {
    cli_abort("Unbalanced panels not yet supported. All units must have the same number of observations.")
  }
  T_obs <- as.integer(unique(counts))
  if (T_obs < 3) {
    cli_abort("At least 3 observations per unit required for a VAR(1) model, got {.val {T_obs}}.")
  }
  time_grid <- .validate_panel_time_grid(data, ids, id_var, time_var)

  # Build 3D array
  y_array <- array(NA_real_, dim = c(N, T_obs, 2))
  person_means <- matrix(NA_real_, N, 2)

  for (i in seq_len(N)) {
    unit_data <- data[data[[id_var]] == ids[i], ]
    y_i <- as.matrix(unit_data[, vars])

    # Check for non-finite values
    if (any(is.nan(y_i) | is.infinite(y_i))) {
      cli_abort(c(
        "Unit {.val {ids[i]}} contains NaN or Inf values.",
        "i" = "Multilevel models require complete cases. Remove or impute non-finite values first."
      ))
    }

    # Check for NAs
    if (any(is.na(y_i))) {
      cli_abort(c(
        "Unit {.val {ids[i]}} contains missing values (NA).",
        "i" = "Multilevel models require complete cases. Consider imputation before fitting."
      ))
    }

    if (center) {
      person_means[i, ] <- colMeans(y_i)
      y_i <- sweep(y_i, 2, person_means[i, ])
    }

    y_array[i, , ] <- y_i
  }

  # Convert to list of matrices (cmdstanr array format)
  y_list <- lapply(seq_len(N), function(i) y_array[i, , ])

  stan_data <- list(
    N = N,
    n_time = T_obs,
    y = y_list,
    prior_phi_bar_sd = prior_phi_bar_sd,
    prior_tau_phi_scale = prior_tau_phi_scale,
    prior_sigma_sd = prior_sigma_sd,
    prior_rho_sd = prior_rho_sd
  )
  if (.uses_sem_multilevel_mixed_engine("multilevel", margins)) {
    stan_data <- .add_mixed_family_data(
      stan_data,
      .sem_multilevel_engine_margins("multilevel", margins, 2L),
      2L,
      skew_direction
    )
  } else if (identical(margins, "exponential")) {
    stan_data$skew_direction <- as.numeric(skew_direction)
  }

  if (any_phi || tv_sigma) {
    margins_vec <- rep(margins, length.out = 2L)
    stan_data$family <- unname(as.integer(.family_codes[margins_vec]))
    stan_data$skew_direction <- stan_data$skew_direction %||% rep(1, 2L)
    stan_data$tv_phi <- as.integer(any_phi)
    stan_data$phi_tv_mask <- unname(phi_mask)
    stan_data$tv_sigma <- as.integer(tv_sigma)
    stan_data$tau_phi_prior <- prior_tau_phi_rate
    stan_data$tau_sigma_prior <- prior_tau_sigma_rate
    stan_data$barrier_k <- tv_sigma_k
    attr(stan_data, "tv_phi") <- any_phi
    attr(stan_data, "phi_tv_mask") <- phi_mask
    attr(stan_data, "tv_sigma") <- tv_sigma
  }

  attr(stan_data, "vars") <- vars
  attr(stan_data, "person_means") <- person_means
  attr(stan_data, "ids") <- ids
  attr(stan_data, "time_values") <- time_grid
  attr(stan_data, "margins") <- margins
  if (!is.null(skew_direction)) {
    attr(stan_data, "skew_direction") <- as.numeric(skew_direction)
  }

  stan_data
}


#' Prepare data for the SEM copula VAR model
#'
#' Transforms a data frame of indicator variables into a list suitable for
#' the SEM copula Stan model. The measurement model parameters (lambda,
#' sigma_e) are fixed and passed through to Stan.
#'
#' @param data A data frame with time series of indicator variables.
#' @param indicators A list of two character vectors, each naming J indicator
#'   columns per latent variable.
#' @param J Number of indicators per latent variable.
#' @param lambda Numeric vector of length J with fixed factor loadings.
#' @param sigma_e Fixed measurement error SD (scalar).
#' @param margins Latent innovation margin specification. A single string
#'   applies the same family to both latent variables. Normal and exponential
#'   use specialised Stan data contracts; skew-normal, gamma, and length-2
#'   per-variable specs use the generic `sem_mixed` / `sem_naive_mixed` Stan
#'   model with per-dimension family codes.
#' @param skew_direction Integer vector of length 2 of +1/-1. Required whenever
#'   any dimension uses an `"exponential"` or `"gamma"` margin.
#' @param time_var Name of the time column (default: `"time"`).
#' @param prior_mu_sd Prior SD for intercepts.
#' @param prior_phi_sd Prior SD for VAR coefficients.
#' @param prior_sigma_sd Prior SD for the lognormal prior on the latent
#'   innovation scale parameter.
#' @param prior_rho_sd Prior SD for rho_raw.
#' @param tv_phi Selects which latent VAR(1) coefficients vary over time.
#'   See [dcvar()] for accepted selectors.
#' @param tv_sigma Logical; if `TRUE`, latent innovation scales evolve over time.
#' @param prior_sigma_omega_rate Prior mean for the Fisher-z rho random-walk SD
#'   used by TV SEM fits.
#' @param prior_tau_phi_rate Prior mean for Phi random-walk innovation SDs.
#' @param prior_tau_sigma_rate Prior mean for log-scale random-walk SDs.
#' @param tv_sigma_k Soft-barrier sharpness for time-varying exponential/gamma
#'   scales.
#' @param method Character string: `"indicator"` for the fixed measurement
#'   model or `"naive"` for row-mean factor scores.
#'
#' @return A named list suitable as Stan data input.
#' @export
prepare_sem_data <- function(data, indicators, J = NULL, lambda = NULL, sigma_e = NULL,
                             margins = "normal",
                             skew_direction = NULL,
                             time_var = "time",
                             prior_mu_sd = 0.25,
                             prior_phi_sd = 0.5,
                             prior_sigma_sd = 0.5,
                             prior_rho_sd = 0.75,
                             tv_phi = FALSE,
                             tv_sigma = FALSE,
                             prior_sigma_omega_rate = 0.1,
                             prior_tau_phi_rate = 0.025,
                             prior_tau_sigma_rate = 0.05,
                             tv_sigma_k = 8,
                             method = c("indicator", "naive")) {
  method <- match.arg(method)
  if (!is.data.frame(data)) {
    cli_abort("{.arg data} must be a data frame.")
  }
  margins <- .normalize_margins_spec(margins)
  .validate_sem_margins(margins, skew_direction)
  if (!is.list(indicators) || length(indicators) != 2) {
    cli_abort("{.arg indicators} must be a list of two character vectors.")
  }
  if (is.null(J) && identical(method, "naive")) {
    J <- length(indicators[[1]])
  }
  if (!is.numeric(J) || length(J) != 1L || !is.finite(J) || J < 1 || J != as.integer(J)) {
    cli_abort("{.arg J} must be a positive integer.")
  }
  if (length(indicators[[1]]) != J || length(indicators[[2]]) != J) {
    cli_abort("Each element of {.arg indicators} must have {.val {J}} indicator names.")
  }
  if (identical(method, "indicator")) {
    .prep_validate_numeric_vector(lambda, "lambda")
    if (length(lambda) != J) {
      cli_abort("{.arg lambda} must have length {.val {J}}.")
    }
    .prep_validate_positive_scalar(sigma_e, "sigma_e")
  }
  .prep_validate_positive_scalar(prior_mu_sd, "prior_mu_sd")
  .prep_validate_positive_scalar(prior_phi_sd, "prior_phi_sd")
  .prep_validate_positive_scalar(prior_sigma_sd, "prior_sigma_sd")
  .prep_validate_positive_scalar(prior_rho_sd, "prior_rho_sd")
  phi_mask <- .resolve_phi_tv_mask(tv_phi)
  any_phi <- sum(phi_mask) > 0L
  .prep_validate_scalar_logical(tv_sigma, "tv_sigma")
  .prep_validate_positive_scalar(prior_sigma_omega_rate, "prior_sigma_omega_rate")
  .prep_validate_positive_scalar(prior_tau_phi_rate, "prior_tau_phi_rate")
  .prep_validate_positive_scalar(prior_tau_sigma_rate, "prior_tau_sigma_rate")
  .prep_validate_positive_scalar(tv_sigma_k, "tv_sigma_k")
  if (identical(method, "naive") && (any_phi || tv_sigma)) {
    cli_abort(c(
      "Time-varying SEM parameters are not implemented for {.code method = \"naive\"}.",
      "i" = "Use {.code method = \"indicator\"} with fixed measurement parameters, or fit a single-level {.fun dcvar} model to score data."
    ))
  }

  all_ind <- c(indicators[[1]], indicators[[2]])
  if (anyDuplicated(all_ind)) {
    cli_abort(c(
      "Indicator names must be unique across both latent variables.",
      "i" = "No indicator may appear in both measurement blocks."
    ))
  }
  missing_cols <- setdiff(c(all_ind, time_var), names(data))
  if (length(missing_cols) > 0) {
    cli_abort("Column{?s} not found in data: {.val {missing_cols}}")
  }
  for (col in all_ind) {
    if (!is.numeric(data[[col]])) {
      cli_abort("Indicator column {.val {col}} must be numeric, got {.cls {class(data[[col]])[1]}}.")
    }
  }

  # Sort by time
  data <- data[order(data[[time_var]]), , drop = FALSE]
  time_values <- data[[time_var]]
  .validate_time_values(time_values, allow_gaps = FALSE, context = "SEM data")

  # Build T x 2J matrix (latent 1 indicators first, then latent 2)
  y_ind <- as.matrix(data[, all_ind])
  T_obs <- nrow(y_ind)
  if (any(is.nan(y_ind) | is.infinite(y_ind))) {
    cli_abort("{.arg data} contains NaN or Inf values in the indicator columns.")
  }
  if (any(is.na(y_ind))) {
    cli_abort("{.arg data} contains missing values in the indicator columns.")
  }

  if (T_obs < 3) {
    cli_abort("At least 3 observations required, got {.val {T_obs}}.")
  }

  if (identical(method, "naive")) {
    y <- cbind(
      rowMeans(data[, indicators[[1]], drop = FALSE]),
      rowMeans(data[, indicators[[2]], drop = FALSE])
    )
    colnames(y) <- NULL
    rownames(y) <- NULL
    stan_data <- list(
      n_time = T_obs,
      y = y,
      prior_mu_sd = prior_mu_sd,
      prior_phi_sd = prior_phi_sd,
      prior_sigma_sd = prior_sigma_sd,
      prior_rho_sd = prior_rho_sd
    )
  } else {
    stan_data <- list(
      n_time = T_obs,
      J = as.integer(J),
      y = y_ind,
      lambda = lambda,
      sigma_e = sigma_e,
      prior_mu_sd = prior_mu_sd,
      prior_phi_sd = prior_phi_sd,
      prior_sigma_sd = prior_sigma_sd,
      prior_rho_sd = prior_rho_sd
    )
  }

  model_type <- if (identical(method, "naive")) "sem_naive" else "sem"
  if (.uses_sem_multilevel_mixed_engine(model_type, margins)) {
    stan_data <- .add_mixed_family_data(
      stan_data,
      .sem_multilevel_engine_margins(model_type, margins, 2L),
      2L,
      skew_direction
    )
  } else if (identical(margins, "exponential")) {
    stan_data$skew_direction <- as.numeric(skew_direction)
  }

  if (any_phi || tv_sigma) {
    margins_vec <- rep(margins, length.out = 2L)
    stan_data$family <- unname(as.integer(.family_codes[margins_vec]))
    stan_data$skew_direction <- stan_data$skew_direction %||% rep(1, 2L)
    stan_data$tv_phi <- as.integer(any_phi)
    stan_data$phi_tv_mask <- unname(phi_mask)
    stan_data$tv_sigma <- as.integer(tv_sigma)
    stan_data$sigma_omega_prior <- prior_sigma_omega_rate
    stan_data$tau_phi_prior <- prior_tau_phi_rate
    stan_data$tau_sigma_prior <- prior_tau_sigma_rate
    stan_data$barrier_k <- tv_sigma_k
    attr(stan_data, "tv_phi") <- any_phi
    attr(stan_data, "phi_tv_mask") <- phi_mask
    attr(stan_data, "tv_sigma") <- tv_sigma
  }

  # Use list names as latent variable names, or default
  latent_names <- names(indicators)
  if (is.null(latent_names) || any(latent_names == "")) {
    latent_names <- c("latent1", "latent2")
  }

  attr(stan_data, "vars") <- latent_names
  attr(stan_data, "indicators") <- indicators
  attr(stan_data, "time_values") <- time_values
  attr(stan_data, "margins") <- margins
  attr(stan_data, "method") <- method
  attr(stan_data, "J") <- as.integer(J)
  if (!is.null(skew_direction)) {
    attr(stan_data, "skew_direction") <- as.numeric(skew_direction)
  }

  stan_data
}


#' Prepare data for the constant copula model
#'
#' Transforms a data frame into a list suitable for the constant copula Stan model.
#'
#' @inheritParams prepare_dcvar_data
#' @param margins Marginal distribution specification. Either a single string
#'   applied to both variables, or a length-2 character vector giving a
#'   per-variable (mixed) margin (for example `c("normal", "exponential")`).
#'   Each entry is one of `"normal"` (default), `"exponential"`,
#'   `"skew_normal"`, or `"gamma"`. When the two entries differ the returned
#'   data carries an integer `family` array for the generic mixed-margins Stan
#'   model; identical entries are equivalent to the scalar form and reuse the
#'   specialised single-family model.
#' @param skew_direction Integer vector of length 2 of `1` (right-skewed) or
#'   `-1` (left-skewed). Required whenever any dimension uses an
#'   `"exponential"` or `"gamma"` margin; only those dimensions consult it.
#' @param prior_z_rho_sd Prior SD for rho on Fisher-z scale (default: 1.0).
#'
#' @return A named list suitable as Stan data input.
#' @export
prepare_constant_data <- function(data, vars, time_var = "time",
                                  standardize = TRUE,
                                  margins = "normal",
                                  skew_direction = NULL,
                                  prior_mu_sd = 2,
                                  prior_phi_sd = 0.5,
                                  prior_sigma_eps_rate = 1,
                                  prior_z_rho_sd = 1.0,
                                  allow_gaps = FALSE) {
  .prep_validate_unique_vars(vars)
  .prep_validate_positive_scalar(prior_mu_sd, "prior_mu_sd")
  .prep_validate_positive_scalar(prior_phi_sd, "prior_phi_sd")
  .prep_validate_positive_scalar(prior_sigma_eps_rate, "prior_sigma_eps_rate")
  .prep_validate_positive_scalar(prior_z_rho_sd, "prior_z_rho_sd")
  margins <- .normalize_margins_spec(margins)
  .validate_margins(margins, skew_direction)
  prep <- .prepare_var_data(data, vars, time_var, standardize, allow_gaps)

  stan_data <- list(
    n_time = prep$T_obs,
    D = prep$D,
    Y = prep$Y,
    sigma_mu_prior = prior_mu_sd,
    sigma_phi_prior = prior_phi_sd,
    z_rho_prior_sd = prior_z_rho_sd
  )

  stan_data <- .add_margin_stan_data(stan_data, margins, prep$D, skew_direction,
                                     prior_sigma_eps_rate)

  attr(stan_data, "vars") <- prep$vars
  attr(stan_data, "standardized") <- prep$standardized
  attr(stan_data, "margins") <- margins
  attr(stan_data, "time_values") <- prep$time_values
  if (!is.null(skew_direction)) attr(stan_data, "skew_direction") <- skew_direction
  if (prep$standardized) {
    attr(stan_data, "Y_means") <- prep$Y_means
    attr(stan_data, "Y_sds") <- prep$Y_sds
  }

  stan_data
}

test_that("prepare_dcvar_data works with valid input", {
  df <- data.frame(time = 1:50, y1 = rnorm(50), y2 = rnorm(50))
  result <- prepare_dcvar_data(df, vars = c("y1", "y2"))

  expect_equal(result$n_time, 50)
  expect_equal(result$D, 2)
  expect_equal(nrow(result$Y), 50)
  expect_equal(ncol(result$Y), 2)
  expect_true(attr(result, "standardized"))
  expect_equal(attr(result, "time_values"), 1:50)
})

test_that("prepare_dcvar_data rejects non-bivariate input", {
  df <- data.frame(time = 1:50, y1 = rnorm(50), y2 = rnorm(50), y3 = rnorm(50))
  expect_error(prepare_dcvar_data(df, vars = "y1"), "Exactly 2")
  expect_error(prepare_dcvar_data(df, vars = c("y1", "y2", "y3")), "Exactly 2")
})

test_that("prepare_dcvar_data rejects duplicate vars", {
  df <- data.frame(time = 1:50, y1 = rnorm(50), y2 = rnorm(50))
  expect_error(prepare_dcvar_data(df, vars = c("y1", "y1")), "distinct")
})

test_that("prepare_dcvar_data rejects missing columns", {
  df <- data.frame(time = 1:50, y1 = rnorm(50))
  expect_error(prepare_dcvar_data(df, vars = c("y1", "y2")), "not found")
})

test_that("prepare_dcvar_data errors on interior missing values by default", {
  df <- data.frame(time = 1:50, y1 = rnorm(50), y2 = rnorm(50))
  df$y1[25] <- NA
  expect_error(prepare_dcvar_data(df, vars = c("y1", "y2")), "interior missing")
})

test_that("prepare_dcvar_data reports the interior missing count", {
  df <- data.frame(time = 1:10, y1 = rnorm(10), y2 = rnorm(10))
  df$y1[c(1, 2, 6)] <- NA
  expect_error(
    prepare_dcvar_data(df, vars = c("y1", "y2")),
    "Found 1 interior missing value"
  )
})

test_that("prepare_dcvar_data can drop interior missing values when allow_gaps = TRUE", {
  df <- data.frame(time = 1:50, y1 = rnorm(50), y2 = rnorm(50))
  df$y1[25] <- NA

  expect_warning(
    out <- prepare_dcvar_data(df, vars = c("y1", "y2"), allow_gaps = TRUE),
    "Removing 1 row"
  )
  expect_equal(out$n_time, 49)
  expect_equal(attr(out, "time_values"), c(1:24, 26:50))
})

test_that("prepare_dcvar_data warns on edge missing values", {
  df <- data.frame(time = 1:50, y1 = rnorm(50), y2 = rnorm(50))
  df$y1[1] <- NA

  expect_warning(
    out <- prepare_dcvar_data(df, vars = c("y1", "y2")),
    "edges only"
  )
  expect_equal(out$n_time, 49)
  expect_equal(attr(out, "time_values"), 2:50)
})

test_that("prepare_dcvar_data preserves standardization metadata", {
  df <- data.frame(time = 1:50, y1 = rnorm(50, 5, 2), y2 = rnorm(50, -3, 0.5))
  result <- prepare_dcvar_data(df, vars = c("y1", "y2"), standardize = TRUE)

  expect_true(!is.null(attr(result, "Y_means")))
  expect_true(!is.null(attr(result, "Y_sds")))
  expect_equal(length(attr(result, "Y_means")), 2)
})

test_that("prepare_dcvar_data skips standardization when FALSE", {
  df <- data.frame(time = 1:50, y1 = rnorm(50, 5, 2), y2 = rnorm(50))
  result <- prepare_dcvar_data(df, vars = c("y1", "y2"), standardize = FALSE)

  expect_false(attr(result, "standardized"))
  expect_null(attr(result, "Y_means"))
})

test_that("prepare_dcvar_data sets correct prior hyperparameters", {
  df <- data.frame(time = 1:50, y1 = rnorm(50), y2 = rnorm(50))
  result <- prepare_dcvar_data(df, vars = c("y1", "y2"),
                               prior_mu_sd = 3, prior_sigma_omega_rate = 0.2)

  expect_equal(result$sigma_mu_prior, 3)
  expect_equal(result$sigma_omega_prior, 0.2)
})

test_that("prepare_dcvar_data rejects invalid prior scalars", {
  df <- data.frame(time = 1:50, y1 = rnorm(50), y2 = rnorm(50))
  expect_error(
    prepare_dcvar_data(df, vars = c("y1", "y2"), prior_mu_sd = 0),
    "prior_mu_sd"
  )
  expect_error(
    prepare_dcvar_data(df, vars = c("y1", "y2"), prior_sigma_eps_rate = -1),
    "prior_sigma_eps_rate"
  )
})

test_that("prepare_hmm_data adds HMM-specific fields", {
  df <- data.frame(time = 1:50, y1 = rnorm(50), y2 = rnorm(50))
  result <- prepare_hmm_data(df, vars = c("y1", "y2"), K = 3)

  expect_equal(result$K, 3L)
  expect_equal(result$kappa, 10)
  expect_equal(result$alpha_off, 1)
  expect_equal(result$z_rho_prior_sd, 1.0)
})

test_that("prepare_hmm_data validates K consistently with dcvar_hmm", {
  df <- data.frame(time = 1:50, y1 = rnorm(50), y2 = rnorm(50))
  expect_error(prepare_hmm_data(df, vars = c("y1", "y2"), K = 2.7), "integer >= 2")
})

test_that("prepare_hmm_data rejects invalid prior scalars", {
  df <- data.frame(time = 1:50, y1 = rnorm(50), y2 = rnorm(50))
  expect_error(
    prepare_hmm_data(df, vars = c("y1", "y2"), prior_kappa = 0),
    "prior_kappa"
  )
})

test_that("prepare_constant_data adds copula prior", {
 df <- data.frame(time = 1:50, y1 = rnorm(50), y2 = rnorm(50))
  result <- prepare_constant_data(df, vars = c("y1", "y2"))

  expect_equal(result$z_rho_prior_sd, 1.0)
  expect_null(result$sigma_omega_prior)
})

test_that("prepare_constant_data rejects duplicate vars", {
  df <- data.frame(time = 1:50, y1 = rnorm(50), y2 = rnorm(50))
  expect_error(prepare_constant_data(df, vars = c("y1", "y1")), "distinct")
})

test_that("prepare_constant_data rejects invalid prior scalars", {
  df <- data.frame(time = 1:50, y1 = rnorm(50), y2 = rnorm(50))
  expect_error(
    prepare_constant_data(df, vars = c("y1", "y2"), prior_mu_sd = 0),
    "prior_mu_sd"
  )
  expect_error(
    prepare_constant_data(df, vars = c("y1", "y2"), prior_sigma_eps_rate = -1),
    "prior_sigma_eps_rate"
  )
  expect_error(
    prepare_constant_data(df, vars = c("y1", "y2"), prior_z_rho_sd = Inf),
    "prior_z_rho_sd"
  )
})

test_that("data is sorted by time_var", {
  df <- data.frame(time = c(3, 1, 2), y1 = c(30, 10, 20), y2 = c(33, 11, 22))
  result <- prepare_dcvar_data(df, vars = c("y1", "y2"), standardize = FALSE)

  expect_equal(result$Y[1, 1], 10)
  expect_equal(result$Y[2, 1], 20)
  expect_equal(result$Y[3, 1], 30)
  expect_equal(attr(result, "time_values"), c(1, 2, 3))
})

test_that("prepare_dcvar_data rejects duplicate time values", {
  df_duplicate <- data.frame(time = c(1, 2, 2), y1 = rnorm(3), y2 = rnorm(3))
  expect_error(
    prepare_dcvar_data(df_duplicate, vars = c("y1", "y2")),
    "Duplicate"
  )
})

test_that("prepare_dcvar_data handles irregular time values consistently", {
  df_irregular <- data.frame(time = c(1, 3, 4), y1 = rnorm(3), y2 = rnorm(3))
  expect_error(
    prepare_dcvar_data(df_irregular, vars = c("y1", "y2")),
    "not evenly spaced"
  )

  expect_warning(
    out <- prepare_dcvar_data(
      df_irregular,
      vars = c("y1", "y2"),
      allow_gaps = TRUE
    ),
    "not evenly spaced"
  )
  expect_equal(attr(out, "time_values"), c(1, 3, 4))
})

test_that("prepare_dcvar_data rejects character and unordered-factor time columns", {
  df <- data.frame(time = paste0("T", 1:12), y1 = rnorm(12), y2 = rnorm(12))
  expect_error(prepare_dcvar_data(df, vars = c("y1", "y2")), "lexicographically")

  df$time <- factor(df$time)
  expect_error(prepare_dcvar_data(df, vars = c("y1", "y2")), "lexicographically")
})

test_that("prepare_dcvar_data accepts ordered-factor time columns", {
  lv <- paste0("T", 1:12)
  df <- data.frame(time = factor(lv, levels = lv, ordered = TRUE),
                   y1 = rnorm(12), y2 = rnorm(12))
  out <- prepare_dcvar_data(df, vars = c("y1", "y2"))
  expect_equal(out$n_time, 12)
})

test_that("prepare_dcvar_data treats edge runs of missing rows as edges, not gaps", {
  df <- data.frame(time = 1:50, y1 = rnorm(50), y2 = rnorm(50))
  df$y1[1:2] <- NA
  df$y2[49:50] <- NA

  expect_warning(
    out <- prepare_dcvar_data(df, vars = c("y1", "y2")),
    "edges only"
  )
  expect_equal(out$n_time, 46)
  expect_equal(attr(out, "time_values"), 3:48)
})

test_that("prepare_dcvar_data still errors on genuinely interior missing runs", {
  df <- data.frame(time = 1:50, y1 = rnorm(50), y2 = rnorm(50))
  df$y1[20:21] <- NA
  expect_error(prepare_dcvar_data(df, vars = c("y1", "y2")), "interior missing")
})

test_that("prepare_dcvar_data aborts cleanly when too few complete rows remain", {
  df <- data.frame(time = 1:5, y1 = c(NA, NA, 1, NA, NA), y2 = rnorm(5))
  expect_warning(
    expect_error(prepare_dcvar_data(df, vars = c("y1", "y2")), "At least 3 observations"),
    "edges only"
  )

  df_all_na <- data.frame(time = 1:5, y1 = NA_real_, y2 = rnorm(5))
  expect_error(prepare_dcvar_data(df_all_na, vars = c("y1", "y2")), "no complete rows")
})

test_that("prepare_dcvar_data accepts tibble input identically to data.frame", {
  skip_if_not_installed("tibble")
  df <- data.frame(time = 1:30, y1 = rnorm(30), y2 = rnorm(30))
  out_df <- prepare_dcvar_data(df, vars = c("y1", "y2"))
  out_tbl <- prepare_dcvar_data(tibble::as_tibble(df), vars = c("y1", "y2"))
  expect_equal(out_tbl$Y, out_df$Y)
  expect_equal(attr(out_tbl, "time_values"), attr(out_df, "time_values"))
})

make_multilevel_exponential_stub_fit <- function() {
  N <- 2
  n_time <- 4
  variables <- c(
    paste0("phi_bar[", 1:4, "]"),
    paste0("tau_phi[", 1:4, "]"),
    paste0(
      "z_phi[",
      rep(seq_len(N), each = 4),
      ",",
      rep(seq_len(4), times = N),
      "]"
    ),
    "eta[1]", "eta[2]",
    paste0(
      "phi_unit[",
      rep(seq_len(N), each = 4),
      ",",
      rep(seq_len(4), times = N),
      "]"
    ),
    "sigma_exp[1]", "sigma_exp[2]",
    "rho",
    paste0(
      "log_lik[",
      rep(seq_len(N), each = n_time - 1),
      ",",
      rep(seq_len(n_time - 1), times = N),
      "]"
    )
  )
  draws <- array(
    rnorm(40 * length(variables), 0, 0.1),
    dim = c(40, 1, length(variables)),
    dimnames = list(NULL, NULL, variables)
  )
  draws[, , paste0("tau_phi[", 1:4, "]")] <- runif(40 * 4, 0.05, 0.2)
  draws[, , c("sigma_exp[1]", "sigma_exp[2]")] <- runif(40 * 2, 0.5, 1.5)
  draws[, , "rho"] <- runif(40, -0.4, 0.4)
  log_lik_vars <- grep("^log_lik\\[", variables, value = TRUE)
  draws[, , log_lik_vars] <- -abs(draws[, , log_lik_vars])

  stan_data <- structure(
    list(
      N = N,
      n_time = n_time,
      y = list(matrix(rnorm(n_time * 2), n_time, 2), matrix(rnorm(n_time * 2), n_time, 2)),
      skew_direction = c(1, -1)
    ),
    ids = c("a", "b"),
    time_values = 11:14,
    margins = "exponential",
    skew_direction = c(1, -1),
    person_means = matrix(0, N, 2)
  )

  new_dcvar_multilevel_fit(
    fit = posterior::as_draws_array(draws),
    stan_data = stan_data,
    N = N,
    vars = c("y1", "y2"),
    centered = TRUE,
    person_means = attr(stan_data, "person_means"),
    margins = "exponential",
    skew_direction = c(1, -1),
    backend = "rstan",
    priors = list(),
    meta = list(chains = 1, iter_warmup = 10, iter_sampling = 40)
  )
}

test_that("prepare_multilevel_data handles exponential margins", {
  df <- data.frame(
    id = rep(1:2, each = 4),
    time = rep(1:4, times = 2),
    y1 = rnorm(8),
    y2 = rnorm(8)
  )

  expect_error(
    prepare_multilevel_data(df, vars = c("y1", "y2"), margins = "exponential"),
    "skew_direction"
  )

  out <- prepare_multilevel_data(
    df,
    vars = c("y1", "y2"),
    margins = "exponential",
    skew_direction = c(1, -1)
  )
  expect_equal(out$skew_direction, c(1, -1))
  expect_equal(attr(out, "margins"), "exponential")
  expect_equal(.margin_stan_file("multilevel", "exponential"), "multilevel_EG.stan")
})

test_that("prepare_multilevel_data handles homogeneous skew_normal and gamma via mixed engine", {
  df <- data.frame(
    id = rep(1:2, each = 4),
    time = rep(1:4, times = 2),
    y1 = rnorm(8),
    y2 = rnorm(8)
  )

  gamma_out <- prepare_multilevel_data(
    df,
    vars = c("y1", "y2"),
    margins = "gamma",
    skew_direction = c(1, -1)
  )
  skew_out <- prepare_multilevel_data(
    df,
    vars = c("y1", "y2"),
    margins = "skew_normal"
  )

  expect_equal(gamma_out$family, c(4L, 4L))
  expect_equal(gamma_out$skew_direction, c(1, -1))
  expect_equal(skew_out$family, c(3L, 3L))
  expect_equal(skew_out$skew_direction, c(1, 1))
  expect_equal(.margin_stan_file("multilevel", "gamma"), "multilevel_mixed.stan")
  expect_equal(.margin_stan_file("multilevel", "skew_normal"), "multilevel_mixed.stan")
})

test_that("multilevel exponential stub fit extractors and loo work", {
  fit <- make_multilevel_exponential_stub_fit()

  co <- coef(fit)
  expect_named(co, c("phi_bar", "tau_phi", "sigma_exp", "rho"))
  expect_true(length(co$sigma_exp) == 2)

  vp <- var_params(fit)
  expect_named(vp, c("phi_bar", "tau_phi", "sigma_exp", "rho"))

  s <- summary(fit)
  expect_s3_class(s, "dcvar_multilevel_summary")
  expect_true("random_effects" %in% names(s))

  out <- suppressWarnings(loo::loo(fit))
  expect_s3_class(out, "loo")
})

# ============================================================================
# Tests for edge-case validation behaviors
# ============================================================================

# --- NA handling in prepare_dcvar_data ---------------------------------------

test_that("interior NA errors by default", {
  df <- data.frame(time = 1:10, y1 = c(1, 2, NA, 4, 5, 6, 7, 8, 9, 10), y2 = rnorm(10))
  expect_error(prepare_dcvar_data(df, vars = c("y1", "y2")), "interior|adjacency|missing")
})

test_that("interior NA warns when allow_gaps = TRUE", {
  df <- data.frame(time = 1:10, y1 = c(1, 2, NA, 4, 5, 6, 7, 8, 9, 10), y2 = rnorm(10))
  expect_warning(prepare_dcvar_data(df, vars = c("y1", "y2"), allow_gaps = TRUE), "missing")
})

test_that("edge-only NA still just warns", {
  df <- data.frame(time = 1:10, y1 = c(NA, 2, 3, 4, 5, 6, 7, 8, 9, 10), y2 = rnorm(10))
  expect_warning(prepare_dcvar_data(df, vars = c("y1", "y2")), "missing")
})

test_that("all-NA data errors", {
  df <- data.frame(time = 1:5, y1 = rep(NA_real_, 5), y2 = rep(NA_real_, 5))
  expect_error(prepare_dcvar_data(df, vars = c("y1", "y2")), "no complete rows")
})

# --- Minimum observation requirements ---------------------------------------

test_that("T=2 errors (too few observations)", {
  df <- data.frame(time = 1:2, y1 = c(1, 2), y2 = c(3, 4))
  expect_error(prepare_dcvar_data(df, vars = c("y1", "y2")), "3 observations")
})

test_that("T=3 succeeds (minimum viable)", {
  df <- data.frame(time = 1:3, y1 = c(1, 2, 3), y2 = c(4, 5, 6))
  result <- prepare_dcvar_data(df, vars = c("y1", "y2"))
  expect_equal(result$n_time, 3)
})

# --- Data type validation ---------------------------------------------------

test_that("NaN or Inf values error", {
  df_nan <- data.frame(time = 1:5, y1 = c(1, NaN, 3, 4, 5), y2 = rnorm(5))
  expect_error(prepare_dcvar_data(df_nan, vars = c("y1", "y2")), "NaN|Inf")

  df_inf <- data.frame(time = 1:5, y1 = c(1, Inf, 3, 4, 5), y2 = rnorm(5))
  expect_error(prepare_dcvar_data(df_inf, vars = c("y1", "y2")), "NaN|Inf")
})

test_that("non-numeric column errors", {
  df <- data.frame(time = 1:5, y1 = letters[1:5], y2 = rnorm(5))
  expect_error(prepare_dcvar_data(df, vars = c("y1", "y2")), "numeric")
})

test_that("zero-variance column errors on standardize", {
  df <- data.frame(time = 1:10, y1 = rep(5, 10), y2 = rnorm(10))
  expect_error(prepare_dcvar_data(df, vars = c("y1", "y2")), "zero|variance")
})

test_that("zero-variance column after NA removal errors", {
  df <- data.frame(time = 1:10, y1 = c(5, 5, NA, 5, 5, 5, 5, 5, 5, 5), y2 = rnorm(10))
  expect_warning(
    expect_error(prepare_dcvar_data(df, vars = c("y1", "y2"),
                                    allow_gaps = TRUE), "zero|variance"),
    "Removing 1 row"
  )
})

# --- Margin validation ------------------------------------------------------

test_that("exponential margins require skew_direction", {
  df <- data.frame(time = 1:20, y1 = abs(rnorm(20)), y2 = abs(rnorm(20)))
  expect_error(prepare_dcvar_data(df, vars = c("y1", "y2"),
                                  margins = "exponential"), "skew_direction")
})

test_that("gamma margins require skew_direction", {
  df <- data.frame(time = 1:20, y1 = abs(rnorm(20)), y2 = abs(rnorm(20)))
  expect_error(prepare_dcvar_data(df, vars = c("y1", "y2"),
                                  margins = "gamma"), "skew_direction")
})

test_that("invalid margin type errors", {
  df <- data.frame(time = 1:20, y1 = rnorm(20), y2 = rnorm(20))
  expect_error(prepare_dcvar_data(df, vars = c("y1", "y2"),
                                  margins = "invalid_margin"), "margin")
})

test_that("normal margins work without skew_direction", {
  df <- data.frame(time = 1:20, y1 = rnorm(20), y2 = rnorm(20))
  result <- prepare_dcvar_data(df, vars = c("y1", "y2"), margins = "normal")
  expect_equal(result$n_time, 20)
})

test_that("prepare_dcvar_data validates scalar logical flags", {
  df <- data.frame(time = 1:20, y1 = rnorm(20), y2 = rnorm(20))
  expect_error(
    prepare_dcvar_data(df, vars = c("y1", "y2"), standardize = 1),
    "single logical"
  )
  expect_error(
    prepare_dcvar_data(df, vars = c("y1", "y2"), allow_gaps = c(TRUE, FALSE)),
    "single logical"
  )
  expect_error(
    prepare_dcvar_data(df, vars = c("y1", "y2"), standardize = NA),
    "single logical"
  )
})

# --- HMM K validation -------------------------------------------------------

test_that("K must be integer-like", {
  skip_if_no_rstan()
  sim <- simulate_dcvar(n_time = 30, rho_trajectory = rho_step(30), seed = 1)
  expect_error(dcvar_hmm(sim$Y_df, vars = c("y1", "y2"), K = 2.5), "integer")
})

test_that("K < 2 errors", {
  skip_if_no_rstan()
  sim <- simulate_dcvar(n_time = 30, rho_trajectory = rho_step(30), seed = 1)
  expect_error(dcvar_hmm(sim$Y_df, vars = c("y1", "y2"), K = 1), "integer.*>= 2")
})

test_that("non-numeric K errors", {
  skip_if_no_rstan()
  sim <- simulate_dcvar(n_time = 30, rho_trajectory = rho_step(30), seed = 1)
  expect_error(dcvar_hmm(sim$Y_df, vars = c("y1", "y2"), K = "two"), "integer")
})

# --- Multilevel validation ---------------------------------------------------

test_that("multilevel errors on missing columns", {
  df <- data.frame(time = 1:10, y1 = rnorm(10), y2 = rnorm(10), id = rep(1, 10))
  expect_error(prepare_multilevel_data(df, vars = c("y1", "missing_var"), id_var = "id"),
               "not found")
})

test_that("multilevel errors on unbalanced panels", {
  df <- data.frame(
    time = c(1:5, 1:3),
    y1 = rnorm(8), y2 = rnorm(8),
    id = c(rep(1, 5), rep(2, 3))
  )
  expect_error(prepare_multilevel_data(df, vars = c("y1", "y2"), id_var = "id"),
               "Unbalanced|balanced")
})

test_that("multilevel errors on non-dataframe input", {
  expect_error(prepare_multilevel_data(list(a = 1), vars = c("y1", "y2"), id_var = "id"),
               "data frame")
})

test_that("multilevel rejects missing unit ids", {
  df <- data.frame(
    time = c(1, 2, 1, 2),
    y1 = rnorm(4),
    y2 = rnorm(4),
    id = c("A", NA, "B", "B")
  )
  expect_error(
    prepare_multilevel_data(df, vars = c("y1", "y2"), id_var = "id"),
    "missing values"
  )
})

test_that("multilevel drops unused factor levels in ids", {
  df <- data.frame(
    time = c(1, 2, 3, 1, 2, 3),
    y1 = rnorm(6),
    y2 = rnorm(6),
    id = factor(c("A", "A", "A", "B", "B", "B"), levels = c("A", "B", "C"))
  )
  out <- prepare_multilevel_data(df, vars = c("y1", "y2"), id_var = "id")
  expect_equal(out$N, 2)
  expect_equal(as.character(attr(out, "ids")), c("A", "B"))
  expect_equal(levels(attr(out, "ids")), c("A", "B"))
})

test_that("multilevel errors on NAs in unit data", {
  df <- data.frame(
    time = rep(1:5, 2),
    y1 = c(1, 2, NA, 4, 5, 6, 7, 8, 9, 10),
    y2 = rnorm(10),
    id = rep(1:2, each = 5)
  )
  expect_error(prepare_multilevel_data(df, vars = c("y1", "y2"), id_var = "id"),
               "missing|NA")
})

test_that("multilevel errors on Inf values", {
  df <- data.frame(
    time = rep(1:5, 2),
    y1 = c(1, 2, Inf, 4, 5, 6, 7, 8, 9, 10),
    y2 = rnorm(10),
    id = rep(1:2, each = 5)
  )
  expect_error(prepare_multilevel_data(df, vars = c("y1", "y2"), id_var = "id"),
               "NaN|Inf")
})

# --- SEM validation ----------------------------------------------------------

test_that("SEM errors on wrong indicator count", {
  skip_if_no_rstan()
  expect_error(
    dcvar_sem(data.frame(), indicators = list(a = "x1", b = c("x2", "x3")),
              J = 2, lambda = c(1, 1), sigma_e = 1),
    "indicator"
  )
})

test_that("SEM errors on mismatched lambda length", {
  df <- data.frame(time = 1:10, y1_1 = rnorm(10), y1_2 = rnorm(10),
                   y2_1 = rnorm(10), y2_2 = rnorm(10))
  expect_error(
    prepare_sem_data(df, indicators = list(a = c("y1_1", "y1_2"), b = c("y2_1", "y2_2")),
                     J = 2, lambda = c(1, 1, 1), sigma_e = 1),
    "lambda.*length|length.*2"
  )
})

test_that("SEM exponential margins require skew_direction", {
  df <- data.frame(time = 1:10, y1_1 = rnorm(10), y1_2 = rnorm(10),
                   y2_1 = rnorm(10), y2_2 = rnorm(10))
  expect_error(
    prepare_sem_data(
      df,
      indicators = list(a = c("y1_1", "y1_2"), b = c("y2_1", "y2_2")),
      J = 2,
      lambda = c(1, 1),
      sigma_e = 1,
      margins = "exponential"
    ),
    "skew_direction"
  )
})

test_that("SEM accepts supported non-normal margins", {
  df <- data.frame(time = 1:10, y1_1 = rnorm(10), y1_2 = rnorm(10),
                   y2_1 = rnorm(10), y2_2 = rnorm(10))

  sem_gamma <- prepare_sem_data(
    df,
    indicators = list(a = c("y1_1", "y1_2"), b = c("y2_1", "y2_2")),
    J = 2,
    lambda = c(1, 1),
    sigma_e = 1,
    margins = "gamma",
    skew_direction = c(1, 1)
  )
  expect_identical(sem_gamma$family, c(4L, 4L))

  sem_skew <- prepare_sem_data(
      df,
      indicators = list(a = c("y1_1", "y1_2"), b = c("y2_1", "y2_2")),
      J = 2,
      lambda = c(1, 1),
      sigma_e = 1,
      margins = "skew_normal",
      skew_direction = c(1, 1)
  )
  expect_identical(sem_skew$family, c(3L, 3L))
})

test_that("J must be a positive integer", {
  skip_if_no_rstan()
  expect_error(dcvar_sem(data.frame(), indicators = list(), J = 0,
                         lambda = numeric(), sigma_e = 1), "positive integer|J")
  expect_error(dcvar_sem(data.frame(), indicators = list(), J = -1,
                         lambda = numeric(), sigma_e = 1), "positive integer|J")
})

# --- Probs validation -------------------------------------------------------

test_that("probs must be in [0, 1]", {
  skip_if_no_rstan()
  fit <- get_dcvar_fit()
  expect_error(rho_trajectory(fit, probs = c(-0.5, 0.5)), "probs")
  expect_error(rho_trajectory(fit, probs = c(0.5, 1.5)), "probs")
})

# Per-variable (mixed) margins for the multilevel and SEM models (Phase 3).

# --- multilevel routing / prep (fast) ---------------------------------------

test_that("multilevel mixed margins route and build a family array", {
  expect_equal(.margin_stan_file("multilevel", c("normal", "exponential")),
               "multilevel_mixed.stan")
  # Homogeneous normal/exponential keep the specialised models.
  expect_equal(.margin_stan_file("multilevel", c("normal", "normal")),
               "multilevel_copula_var.stan")
  expect_equal(.margin_stan_file("multilevel", c("exponential", "exponential")),
               "multilevel_EG.stan")
  expect_equal(.margin_stan_file("multilevel", c("gamma", "gamma")),
               "multilevel_mixed.stan")
  expect_equal(.margin_stan_file("multilevel", c("skew_normal", "skew_normal")),
               "multilevel_mixed.stan")

  dfm <- data.frame(id = rep(1:3, each = 12), time = rep(1:12, 3),
                    y1 = rnorm(36), y2 = rnorm(36))
  sd_ml <- prepare_multilevel_data(dfm, vars = c("y1", "y2"), id_var = "id",
                                   margins = c("normal", "gamma"),
                                   skew_direction = c(1, 1))
  expect_equal(sd_ml$family, c(1L, 4L))
  expect_equal(sd_ml$skew_direction, c(1, 1))
})

test_that("homogeneous multilevel gamma/skew_normal build family arrays", {
  dfm <- data.frame(id = rep(1:3, each = 12), time = rep(1:12, 3),
                    y1 = rnorm(36), y2 = rnorm(36))
  gamma_data <- prepare_multilevel_data(
    dfm, vars = c("y1", "y2"), id_var = "id",
    margins = "gamma", skew_direction = c(1, 1)
  )
  skew_data <- prepare_multilevel_data(
    dfm, vars = c("y1", "y2"), id_var = "id",
    margins = "skew_normal"
  )

  expect_equal(gamma_data$family, c(4L, 4L))
  expect_equal(gamma_data$skew_direction, c(1, 1))
  expect_equal(attr(gamma_data, "margins"), "gamma")
  expect_equal(skew_data$family, c(3L, 3L))
  expect_equal(skew_data$skew_direction, c(1, 1))
  expect_equal(attr(skew_data, "margins"), "skew_normal")
})

# --- multilevel mixed fit ---------------------------------------------------

test_that("dcvar_multilevel() fits mixed normal + exponential margins", {
  skip_if_no_rstan()

  fit <- get_multilevel_mixed_fit()
  expect_s3_class(fit, "dcvar_multilevel_fit")
  expect_equal(fit$margins, c("normal", "exponential"))
  expect_equal(fit$stan_data$family, c(1L, 2L))
})

test_that("multilevel mixed fit emits only known diagnostic warnings", {
  skip_if_no_rstan()

  expect_known_fit_warnings(get_multilevel_mixed_fit_warnings(), "multilevel mixed")
})

test_that("multilevel mixed coef()/var_params() report per-dimension families", {
  skip_if_no_rstan()

  fit <- get_multilevel_mixed_fit()
  co <- coef(fit)
  expect_equal(names(co$sigma_eps), "sigma_eps[1]")
  expect_equal(names(co$sigma_exp), "sigma_exp[2]")

  vp <- var_params(fit)
  expect_true(all(c("phi_bar", "tau_phi", "sigma_eps", "sigma_exp", "rho") %in% names(vp)))
})

test_that("multilevel mixed recovers the global copula correlation", {
  skip_if_no_rstan()

  rho <- coef(get_multilevel_mixed_fit())$rho
  expect_true(rho > 0.2 && rho < 0.8)
})


# --- SEM routing / prep (fast) ----------------------------------------------

test_that("SEM mixed margins route and build a family array", {
  expect_equal(.margin_stan_file("sem", c("normal", "exponential")),
               "sem_mixed.stan")
  expect_equal(.margin_stan_file("sem_naive", c("normal", "gamma")),
               "sem_naive_mixed.stan")
  # Homogeneous normal/exponential keep the specialised models.
  expect_equal(.margin_stan_file("sem", c("normal", "normal")),
               "sem_copula_var.stan")
  expect_equal(.margin_stan_file("sem", c("gamma", "gamma")),
               "sem_mixed.stan")
  expect_equal(.margin_stan_file("sem_naive", c("skew_normal", "skew_normal")),
               "sem_naive_mixed.stan")

  df <- data.frame(time = 1:30, y1_1 = rnorm(30), y1_2 = rnorm(30),
                   y2_1 = rnorm(30), y2_2 = rnorm(30))
  indicators <- list(latent1 = c("y1_1", "y1_2"), latent2 = c("y2_1", "y2_2"))
  sd_sem <- prepare_sem_data(df, indicators = indicators, J = 2,
                             lambda = c(0.8, 0.8), sigma_e = 0.5,
                             margins = c("normal", "exponential"),
                             skew_direction = c(1, 1))
  expect_equal(sd_sem$family, c(1L, 2L))
})

# --- SEM mixed fit ----------------------------------------------------------

test_that("dcvar_sem() fits mixed margins (indicator method)", {
  skip_if_no_rstan()

  fit <- get_sem_mixed_fit()
  expect_s3_class(fit, "dcvar_sem_fit")
  expect_equal(fit$margins, c("normal", "exponential"))
  expect_equal(fit$stan_data$family, c(1L, 2L))

  co <- coef(fit)
  expect_equal(names(co$sigma_eps), "sigma_eps[1]")
  expect_equal(names(co$sigma_exp), "sigma_exp[2]")

  vp <- var_params(fit)
  expect_true(all(c("mu", "Phi", "sigma_eps", "sigma_exp", "rho") %in% names(vp)))
})

test_that("SEM mixed fit emits only known diagnostic warnings", {
  skip_if_no_rstan()

  expect_known_fit_warnings(get_sem_mixed_fit_warnings(), "sem mixed")
})

test_that("SEM mixed (indicator) recovers a positive copula correlation", {
  skip_if_no_rstan()

  rho <- coef(get_sem_mixed_fit())$rho
  expect_true(rho > 0.1 && rho < 0.9)
})

test_that("dcvar_sem() fits mixed margins (naive method)", {
  skip_if_no_rstan()

  fit <- get_sem_naive_mixed_fit()
  expect_s3_class(fit, "dcvar_sem_fit")
  expect_equal(fit$method, "naive")
  expect_equal(fit$margins, c("normal", "exponential"))
  expect_equal(fit$stan_data$family, c(1L, 2L))
  expect_true(all(c("sigma_eps", "sigma_exp") %in% names(coef(fit))))
})

# ============================================================================
# Tests for prepare_multilevel_data() and prepare_sem_data()
# ============================================================================

# --- prepare_multilevel_data ------------------------------------------------

test_that("prepare_multilevel_data returns correct Stan data structure", {
  sim <- simulate_dcvar_multilevel(N = 5, n_time = 20, rho = 0.5, seed = 1)
  stan_data <- prepare_multilevel_data(sim$data, vars = c("y1", "y2"),
                                       id_var = "id")

  expect_type(stan_data, "list")
  expect_equal(stan_data$N, 5)
  expect_equal(stan_data$n_time, 20)
  expect_type(stan_data$y, "list")
  expect_equal(length(stan_data$y), 5)

  # Each element is a T x 2 matrix
  expect_equal(nrow(stan_data$y[[1]]), 20)
  expect_equal(ncol(stan_data$y[[1]]), 2)
})

test_that("prepare_multilevel_data includes prior hyperparameters", {
  sim <- simulate_dcvar_multilevel(N = 3, n_time = 15, rho = 0.5, seed = 2)
  stan_data <- prepare_multilevel_data(sim$data, vars = c("y1", "y2"),
                                       id_var = "id",
                                       prior_phi_bar_sd = 0.8,
                                       prior_tau_phi_scale = 0.3)

  expect_equal(stan_data$prior_phi_bar_sd, 0.8)
  expect_equal(stan_data$prior_tau_phi_scale, 0.3)
})

test_that("prepare_multilevel_data errors on unbalanced panels", {
  df <- data.frame(
    id = c(1, 1, 1, 2, 2),
    time = c(1, 2, 3, 1, 2),
    y1 = rnorm(5),
    y2 = rnorm(5)
  )

  expect_error(prepare_multilevel_data(df, vars = c("y1", "y2"), id_var = "id"),
               "Unbalanced")
})

test_that("prepare_multilevel_data errors on missing columns", {
  df <- data.frame(id = 1:10, time = 1:10, y1 = rnorm(10))
  expect_error(
    prepare_multilevel_data(df, vars = c("y1", "y2"), id_var = "id"),
    "not found"
  )
})

test_that("prepare_multilevel_data validates vars length", {
  sim <- simulate_dcvar_multilevel(N = 3, n_time = 10, rho = 0.5, seed = 5)
  expect_error(
    prepare_multilevel_data(sim$data, vars = "y1", id_var = "id"),
    "Exactly 2 variables"
  )
})

test_that("prepare_multilevel_data validates numeric columns", {
  sim <- simulate_dcvar_multilevel(N = 3, n_time = 10, rho = 0.5, seed = 6)
  sim$data$y1 <- as.character(sim$data$y1)
  expect_error(
    prepare_multilevel_data(sim$data, vars = c("y1", "y2"), id_var = "id"),
    "must be numeric"
  )
})

test_that("prepare_multilevel_data rejects duplicate vars and invalid priors", {
  sim <- simulate_dcvar_multilevel(N = 3, n_time = 10, rho = 0.5, seed = 7)
  expect_error(
    prepare_multilevel_data(sim$data, vars = c("y1", "y1"), id_var = "id"),
    "distinct"
  )
  expect_error(
    prepare_multilevel_data(sim$data, vars = c("y1", "y2"), id_var = "id",
                            prior_tau_phi_scale = 0),
    "prior_tau_phi_scale"
  )
  expect_error(
    prepare_multilevel_data(sim$data, vars = c("y1", "y2"), id_var = "id",
                            center = "no"),
    "center"
  )
})

test_that("prepare_multilevel_data handles centering correctly", {
  sim <- simulate_dcvar_multilevel(N = 3, n_time = 20, rho = 0.5,
                                   center = FALSE, seed = 3)
  stan_centered <- prepare_multilevel_data(sim$data, vars = c("y1", "y2"),
                                           id_var = "id", center = TRUE)
  stan_uncentered <- prepare_multilevel_data(sim$data, vars = c("y1", "y2"),
                                             id_var = "id", center = FALSE)

  # Centered data person means should be near zero
  for (i in seq_len(stan_centered$N)) {
    expect_true(abs(mean(stan_centered$y[[i]][, 1])) < 1e-10)
    expect_true(abs(mean(stan_centered$y[[i]][, 2])) < 1e-10)
  }

  # Uncentered data should differ from centered
  expect_false(identical(stan_centered$y[[1]], stan_uncentered$y[[1]]))
})

test_that("prepare_multilevel_data stores attributes", {
  sim <- simulate_dcvar_multilevel(N = 4, n_time = 15, rho = 0.5, seed = 4)
  stan_data <- prepare_multilevel_data(sim$data, vars = c("y1", "y2"),
                                       id_var = "id")

  expect_equal(attr(stan_data, "vars"), c("y1", "y2"))
  expect_equal(length(attr(stan_data, "ids")), 4)
  expect_equal(nrow(attr(stan_data, "person_means")), 4)
  expect_equal(attr(stan_data, "time_values"), 1:15)
})

test_that("prepare_multilevel_data requires a shared regular time grid", {
  df <- data.frame(
    id = rep(c("A", "B"), each = 3),
    time = c(1, 2, 3, 2, 3, 4),
    y1 = rnorm(6),
    y2 = rnorm(6)
  )

  expect_error(
    prepare_multilevel_data(df, vars = c("y1", "y2"), id_var = "id"),
    "same.*time.*grid"
  )
})

test_that("prepare_multilevel_data rejects duplicate unit time values", {
  df <- data.frame(
    id = rep(c("A", "B"), each = 3),
    time = c(1, 2, 2, 1, 2, 3),
    y1 = rnorm(6),
    y2 = rnorm(6)
  )

  expect_error(
    prepare_multilevel_data(df, vars = c("y1", "y2"), id_var = "id"),
    "Duplicate"
  )
})


# --- prepare_sem_data -------------------------------------------------------

test_that("prepare_sem_data returns correct Stan data structure", {
  J <- 3
  sim <- simulate_dcvar_sem(n_time = 50, J = J,
                            lambda = rep(0.8, J),
                            rho = 0.5, seed = 10)
  indicators <- list(
    latent1 = paste0("y1_", seq_len(J)),
    latent2 = paste0("y2_", seq_len(J))
  )
  stan_data <- prepare_sem_data(sim$data, indicators = indicators,
                                J = J, lambda = rep(0.8, J),
                                sigma_e = sqrt(0.2))

  expect_type(stan_data, "list")
  expect_equal(stan_data$n_time, 50)
  expect_equal(stan_data$J, J)
  expect_equal(nrow(stan_data$y), 50)
  expect_equal(ncol(stan_data$y), 2 * J)
  expect_equal(stan_data$lambda, rep(0.8, J))
  expect_equal(stan_data$sigma_e, sqrt(0.2))
})

test_that("prepare_sem_data validates indicator columns exist", {
  df <- data.frame(time = 1:20, y1_1 = rnorm(20), y1_2 = rnorm(20))
  indicators <- list(
    latent1 = c("y1_1", "y1_2"),
    latent2 = c("y2_1", "y2_2")
  )

  expect_error(
    prepare_sem_data(df, indicators = indicators, J = 2,
                     lambda = c(0.8, 0.8), sigma_e = 0.5),
    "not found"
  )
})

test_that("prepare_sem_data validates numeric indicator columns", {
  df <- data.frame(time = 1:10, y1_1 = letters[1:10], y1_2 = rnorm(10),
                   y2_1 = rnorm(10), y2_2 = rnorm(10))
  indicators <- list(
    latent1 = c("y1_1", "y1_2"),
    latent2 = c("y2_1", "y2_2")
  )

  expect_error(
    prepare_sem_data(df, indicators = indicators, J = 2,
                     lambda = c(0.8, 0.8), sigma_e = 0.5),
    "must be numeric"
  )
})

test_that("prepare_sem_data rejects overlapping indicators and invalid scales", {
  df <- data.frame(
    time = 1:10,
    y1_1 = rnorm(10),
    y1_2 = rnorm(10),
    y2_1 = rnorm(10),
    y2_2 = rnorm(10)
  )
  overlapping <- list(
    latent1 = c("y1_1", "y1_2"),
    latent2 = c("y2_1", "y1_2")
  )

  expect_error(
    prepare_sem_data(df, indicators = overlapping, J = 2,
                     lambda = c(0.8, 0.8), sigma_e = 0.5),
    "unique"
  )
  expect_error(
    prepare_sem_data(df, indicators = list(
      latent1 = c("y1_1", "y1_2"),
      latent2 = c("y2_1", "y2_2")
    ), J = 2, lambda = c(0.8, NA_real_), sigma_e = 0.5),
    "lambda"
  )
  expect_error(
    prepare_sem_data(df, indicators = list(
      latent1 = c("y1_1", "y1_2"),
      latent2 = c("y2_1", "y2_2")
    ), J = 2, lambda = c(0.8, 0.8), sigma_e = 0),
    "sigma_e"
  )
})

test_that("prepare_sem_data rejects missing and non-finite values", {
  df <- data.frame(time = 1:10, y1_1 = rnorm(10), y1_2 = rnorm(10),
                   y2_1 = rnorm(10), y2_2 = rnorm(10))
  indicators <- list(
    latent1 = c("y1_1", "y1_2"),
    latent2 = c("y2_1", "y2_2")
  )

  df_na <- df
  df_na$y1_1[3] <- NA
  expect_error(
    prepare_sem_data(df_na, indicators = indicators, J = 2,
                     lambda = c(0.8, 0.8), sigma_e = 0.5),
    "missing values"
  )

  df_inf <- df
  df_inf$y1_1[3] <- Inf
  expect_error(
    prepare_sem_data(df_inf, indicators = indicators, J = 2,
                     lambda = c(0.8, 0.8), sigma_e = 0.5),
    "NaN or Inf"
  )
})

test_that("prepare_sem_data errors when indicators is not a list of length 2", {
  df <- data.frame(time = 1:20, y1 = rnorm(20), y2 = rnorm(20))

  expect_error(
    prepare_sem_data(df, indicators = c("y1", "y2"), J = 1,
                     lambda = 0.8, sigma_e = 0.5),
    "list of two"
  )
  expect_error(
    prepare_sem_data(df, indicators = list("y1"), J = 1,
                     lambda = 0.8, sigma_e = 0.5),
    "list of two"
  )
})

test_that("prepare_sem_data errors when lambda length mismatches J", {
  J <- 3
  sim <- simulate_dcvar_sem(n_time = 30, J = J, lambda = rep(0.8, J),
                            rho = 0.5, seed = 11)
  indicators <- list(
    latent1 = paste0("y1_", seq_len(J)),
    latent2 = paste0("y2_", seq_len(J))
  )

  expect_error(
    prepare_sem_data(sim$data, indicators = indicators, J = J,
                     lambda = c(0.8, 0.8), sigma_e = 0.5),
    "lambda"
  )
})

test_that("prepare_sem_data correct dimensions for indicator matrix", {
  for (J in c(2, 4)) {
    sim <- simulate_dcvar_sem(n_time = 40, J = J, lambda = rep(0.8, J),
                              rho = 0.5, seed = 200 + J)
    indicators <- list(
      latent1 = paste0("y1_", seq_len(J)),
      latent2 = paste0("y2_", seq_len(J))
    )
    stan_data <- prepare_sem_data(sim$data, indicators = indicators,
                                  J = J, lambda = rep(0.8, J),
                                  sigma_e = 0.4)

    expect_equal(nrow(stan_data$y), 40, info = paste("J =", J))
    expect_equal(ncol(stan_data$y), 2 * J, info = paste("J =", J))
    expect_equal(stan_data$J, J, info = paste("J =", J))
  }
})

test_that("prepare_sem_data stores latent variable name attributes", {
  J <- 2
  sim <- simulate_dcvar_sem(n_time = 30, J = J, lambda = rep(0.8, J),
                            rho = 0.5, seed = 12)
  indicators <- list(
    anxiety = paste0("y1_", seq_len(J)),
    depression = paste0("y2_", seq_len(J))
  )
  stan_data <- prepare_sem_data(sim$data, indicators = indicators,
                                J = J, lambda = rep(0.8, J),
                                sigma_e = 0.4)

  expect_equal(attr(stan_data, "vars"), c("anxiety", "depression"))
  expect_equal(attr(stan_data, "indicators"), indicators)
  expect_equal(attr(stan_data, "time_values"), 1:30)
})

test_that("prepare_sem_data rejects irregular or duplicate time values", {
  df <- data.frame(
    time = c(1, 3, 4),
    y1_1 = rnorm(3),
    y1_2 = rnorm(3),
    y2_1 = rnorm(3),
    y2_2 = rnorm(3)
  )
  indicators <- list(
    latent1 = c("y1_1", "y1_2"),
    latent2 = c("y2_1", "y2_2")
  )

  expect_error(
    prepare_sem_data(df, indicators = indicators, J = 2,
                     lambda = c(0.8, 0.8), sigma_e = 0.5),
    "not evenly spaced"
  )

  df$time <- c(1, 2, 2)
  expect_error(
    prepare_sem_data(df, indicators = indicators, J = 2,
                     lambda = c(0.8, 0.8), sigma_e = 0.5),
    "Duplicate"
  )
})

test_that("prepare_multilevel_data accepts tibble input and validates its time grid", {
  skip_if_not_installed("tibble")
  sim <- simulate_dcvar_multilevel(N = 4, n_time = 12, rho = 0.4, seed = 5)

  out_df <- prepare_multilevel_data(sim$data, vars = c("y1", "y2"), id_var = "id")
  out_tbl <- prepare_multilevel_data(tibble::as_tibble(sim$data),
                                     vars = c("y1", "y2"), id_var = "id")
  expect_equal(out_tbl$y, out_df$y)
  expect_equal(attr(out_tbl, "time_values"), attr(out_df, "time_values"))
  expect_true(is.atomic(attr(out_tbl, "time_values")))

  # Per-unit time validation must fire for tibbles too
  bad <- tibble::as_tibble(sim$data)
  bad$time[2] <- bad$time[1] # duplicate time within unit 1
  expect_error(
    prepare_multilevel_data(bad, vars = c("y1", "y2"), id_var = "id"),
    "Duplicate"
  )
})

test_that("prepare_multilevel_data requires at least 3 occasions per unit", {
  sim <- simulate_dcvar_multilevel(N = 4, n_time = 6, rho = 0.4, seed = 6)
  short <- sim$data[sim$data$time <= 2, ]
  expect_error(
    prepare_multilevel_data(short, vars = c("y1", "y2"), id_var = "id"),
    "At least 3 observations per unit"
  )
})

test_that("simulate_dcvar with exponential margins (positive skew) works", {
  traj <- rho_constant(50, 0.5)
  sim <- simulate_dcvar(n_time = 50, rho_trajectory = traj,
                        margins = "exponential",
                        skew_direction = c(1, 1),
                        seed = 1)

  expect_type(sim, "list")
  expect_named(sim, c("Y", "Y_df", "true_params"))
  expect_equal(nrow(sim$Y), 50)
  expect_equal(ncol(sim$Y), 2)
  expect_equal(nrow(sim$Y_df), 50)
  expect_equal(sim$true_params$margins, "exponential")
  expect_equal(sim$true_params$skew_direction, c(1, 1))
})

test_that("simulate_dcvar with exponential margins (negative skew) works", {
  traj <- rho_constant(50, 0.5)
  sim <- simulate_dcvar(n_time = 50, rho_trajectory = traj,
                        margins = "exponential",
                        skew_direction = c(-1, -1),
                        seed = 2)

  expect_equal(nrow(sim$Y), 50)
  expect_equal(ncol(sim$Y), 2)
  expect_equal(sim$true_params$margins, "exponential")
  expect_equal(sim$true_params$skew_direction, c(-1, -1))
})

test_that("simulate_dcvar with gamma margins works", {
  traj <- rho_constant(50, 0.5)
  sim <- simulate_dcvar(n_time = 50, rho_trajectory = traj,
                        margins = "gamma",
                        skew_direction = c(1, 1),
                        skew_params = list(shape = 2),
                        seed = 3)

  expect_equal(nrow(sim$Y), 50)
  expect_equal(ncol(sim$Y), 2)
  expect_equal(sim$true_params$margins, "gamma")
  expect_equal(sim$true_params$skew_direction, c(1, 1))
  expect_equal(sim$true_params$skew_params$shape, 2)
})

test_that("simulate_dcvar with skew_normal margins works", {
  skip_if_not_installed("sn")

  traj <- rho_constant(50, 0.5)
  sim <- simulate_dcvar(n_time = 50, rho_trajectory = traj,
                        margins = "skew_normal",
                        skew_params = list(alpha = c(3, -3)),
                        seed = 4)

  expect_equal(nrow(sim$Y), 50)
  expect_equal(ncol(sim$Y), 2)
  expect_equal(sim$true_params$margins, "skew_normal")
  expect_equal(sim$true_params$skew_params$alpha, c(3, -3))
})

test_that("simulate_dcvar with non-normal margins has correct Y_df columns", {
  traj <- rho_constant(30, 0.3)
  sim <- simulate_dcvar(n_time = 30, rho_trajectory = traj,
                        margins = "exponential",
                        skew_direction = c(1, -1),
                        seed = 5)

  expect_true(all(c("time", "y1", "y2") %in% names(sim$Y_df)))
  expect_equal(sim$Y_df$time, 1:30)
})

test_that("simulate_dcvar with non-normal margins is reproducible with seed", {
  traj <- rho_constant(40, 0.5)
  s1 <- simulate_dcvar(n_time = 40, rho_trajectory = traj,
                       margins = "exponential",
                       skew_direction = c(1, 1), seed = 42)
  s2 <- simulate_dcvar(n_time = 40, rho_trajectory = traj,
                       margins = "exponential",
                       skew_direction = c(1, 1), seed = 42)
  expect_identical(s1$Y, s2$Y)
})

test_that("simulate_dcvar normal margins stores sigma_eps in true_params", {
  traj <- rho_constant(30, 0.5)
  sim <- simulate_dcvar(n_time = 30, rho_trajectory = traj,
                        margins = "normal", seed = 6)

  expect_equal(sim$true_params$margins, "normal")
  expect_equal(sim$true_params$sigma_eps, c(1, 1))
})

test_that("simulate_dcvar supports mixed (per-variable) margins", {
  traj <- rho_constant(40, 0.5)
  sim <- simulate_dcvar(n_time = 40, rho_trajectory = traj,
                        margins = c("normal", "exponential"),
                        skew_direction = c(1, 1), seed = 7)

  expect_equal(ncol(sim$Y), 2)
  expect_equal(sim$true_params$margins, c("normal", "exponential"))
  expect_equal(sim$true_params$sigma_eps, c(1, 1))
  expect_equal(sim$true_params$skew_direction, c(1, 1))
})

test_that("simulate_dcvar mixed margins are reproducible with a seed", {
  traj <- rho_constant(40, 0.4)
  s1 <- simulate_dcvar(n_time = 40, rho_trajectory = traj,
                       margins = c("normal", "exponential"),
                       skew_direction = c(1, -1), seed = 99)
  s2 <- simulate_dcvar(n_time = 40, rho_trajectory = traj,
                       margins = c("normal", "exponential"),
                       skew_direction = c(1, -1), seed = 99)
  expect_identical(s1$Y, s2$Y)
})

test_that("simulate_dcvar collapses identical margin vectors to scalar", {
  traj <- rho_constant(30, 0.5)
  sim <- simulate_dcvar(n_time = 30, rho_trajectory = traj,
                        margins = c("normal", "normal"), seed = 8)
  expect_identical(sim$true_params$margins, "normal")

  # An all-identical vector must reproduce the scalar simulation exactly.
  sim_scalar <- simulate_dcvar(n_time = 30, rho_trajectory = traj,
                               margins = "normal", seed = 8)
  expect_identical(sim$Y, sim_scalar$Y)
})

test_that("mixed-margin simulation feeds every prep function", {
  # Mixed margins are supported by all prep paths; each builds a per-dimension
  # family array.
  traj <- rho_constant(20, 0.5)
  sim <- simulate_dcvar(n_time = 20, rho_trajectory = traj,
                        margins = c("normal", "exponential"),
                        skew_direction = c(1, 1), seed = 9)
  for (prep in list(
    prepare_constant_data(sim$Y_df, vars = c("y1", "y2"),
                          margins = c("normal", "exponential"),
                          skew_direction = c(1, 1)),
    prepare_dcvar_data(sim$Y_df, vars = c("y1", "y2"),
                       margins = c("normal", "exponential"),
                       skew_direction = c(1, 1)),
    prepare_hmm_data(sim$Y_df, vars = c("y1", "y2"), K = 2,
                     margins = c("normal", "exponential"),
                     skew_direction = c(1, 1))
  )) {
    expect_equal(prep$family, c(1L, 2L))
  }

  # Multilevel now accepts mixed margins too.
  dfm <- data.frame(id = rep(1:2, each = 10), time = rep(1:10, 2),
                    y1 = rnorm(20), y2 = rnorm(20))
  sd_ml <- prepare_multilevel_data(dfm, vars = c("y1", "y2"), id_var = "id",
                                   margins = c("normal", "exponential"),
                                   skew_direction = c(1, 1))
  expect_equal(sd_ml$family, c(1L, 2L))
})

#' Fit the constant copula model
#'
#' Fits a copula VAR(1) model with a single time-invariant correlation
#' parameter. This serves as a baseline for comparison with the DC-VAR
#' and HMM copula models.
#'
#' @inheritParams dcvar
#' @param margins Marginal distribution specification. Either a single string
#'   applied to both variables, or a length-2 character vector giving a
#'   per-variable (mixed) margin (for example
#'   `c("normal", "exponential")`). Each entry is one of `"normal"` (default),
#'   `"exponential"`, `"skew_normal"`, or `"gamma"`. When the two entries
#'   differ the fit uses a generic mixed-margins Stan model; identical entries
#'   are equivalent to the scalar form and reuse the specialised single-family
#'   model. Mixed margins are supported with the Gaussian copula, and (for
#'   `dcvar_constant`) additionally with the Clayton copula.
#' @param copula Character string specifying the copula family. One of
#'   `"gaussian"` (default) or `"clayton"`. Clayton is currently available with
#'   normal margins or a per-variable (mixed) margin vector; a single
#'   non-normal family with Clayton is not yet supported.
#' @param skew_direction Integer vector of length 2 indicating skew direction
#'   for asymmetric margins. Each element must be `1` (right-skewed) or `-1`
#'   (left-skewed). Required whenever any dimension uses an `"exponential"` or
#'   `"gamma"` margin; only those dimensions consult it.
#' @param prior_z_rho_sd Prior SD for rho on Fisher-z scale (default: 1.0).
#' @param adapt_delta Target acceptance rate (default: 0.999). The constant
#'   model uses a higher default than DC-VAR (0.99) because its simpler
#'   posterior geometry benefits from tighter step-size adaptation without
#'   significant cost, reducing occasional divergences near the rho boundary.
#'
#' @details `adapt_delta` defaults to 0.999 because the constant-rho model has
#'   a simpler correlation structure that benefits from tighter step-size
#'   adaptation without significant computational cost, reducing occasional
#'   divergences near the rho boundary.
#'
#' @param backend Character: `"auto"` (default, uses rstan), `"rstan"`, or
#'   `"cmdstanr"`. Can also be set globally via
#'   `options(dcvar.backend = "cmdstanr")`.
#' @param ... Additional backend-specific sampling arguments.
#'
#' @return A `dcvar_constant_fit` object.
#'
#' @seealso [dcvar()] for the time-varying model, [dcvar_hmm()] for the
#'   regime-switching model, [dcvar_compare()] for LOO-CV model comparison.
#' @export
#'
#' @examples
#' \donttest{
#' sim <- simulate_dcvar(
#'   n_time = 12,
#'   rho_trajectory = rho_constant(12, rho = 0.5),
#'   seed = 1
#' )
#' fit <- dcvar_constant(
#'   sim$Y_df,
#'   vars = c("y1", "y2"),
#'   chains = 1,
#'   iter_warmup = 10,
#'   iter_sampling = 10,
#'   refresh = 0,
#'   seed = 1
#' )
#' print(fit)
#' }
dcvar_constant <- function(data, vars,
                           time_var = "time",
                           standardize = TRUE,
                           margins = "normal",
                           copula = "gaussian",
                           skew_direction = NULL,
                           allow_gaps = FALSE,
                           prior_mu_sd = 2,
                           prior_phi_sd = 0.5,
                           prior_sigma_eps_rate = 1,
                           prior_z_rho_sd = 1.0,
                           chains = 4,
                           iter_warmup = 2000,
                           iter_sampling = 4000,
                           adapt_delta = 0.999,
                           max_treedepth = 12,
                           seed = NULL,
                           cores = NULL,
                           refresh = 500,
                           init = NULL,
                           stan_file = NULL,
                           backend = getOption("dcvar.backend", "auto"),
                           ...) {
  backend <- .resolve_backend(backend)
  .validate_sampling_args(chains, iter_warmup, iter_sampling,
                          adapt_delta, max_treedepth)
  margins <- .normalize_margins_spec(margins)
  .validate_margins(margins, skew_direction)
  .validate_copula(copula)
  if (identical(copula, "clayton") &&
      !.is_mixed_margins(margins) && !all(margins == "normal")) {
    cli_abort(c(
      "Clayton copula support in {.fun dcvar_constant} requires {.arg margins = 'normal'} or a per-variable (mixed) margin vector.",
      "i" = "A single non-normal family with the Clayton copula is not yet available."
    ))
  }

  # Prepare data
  stan_data <- prepare_constant_data(
    data, vars, time_var, standardize, margins, skew_direction,
    prior_mu_sd, prior_phi_sd, prior_sigma_eps_rate,
    prior_z_rho_sd,
    allow_gaps = allow_gaps
  )
  if (identical(copula, "clayton")) {
    # Clayton models parameterize dependence via theta (fixed lognormal
    # prior), so the Gaussian-copula z_rho prior does not apply.
    if (!isTRUE(all.equal(prior_z_rho_sd, 1))) {
      cli_warn(c(
        "{.arg prior_z_rho_sd} is ignored for {.val clayton} copula fits.",
        "i" = "Clayton dependence uses {.code theta ~ lognormal(0, 1)}; there is no z-scale rho parameter."
      ))
    }
    stan_data$z_rho_prior_sd <- NULL
  }
  attr(stan_data, "copula") <- copula

  margins_label <- if (all(margins == "normal")) {
    ""
  } else {
    paste0(" [", paste(margins, collapse = ", "), "]")
  }
  copula_label <- if (identical(copula, "gaussian")) "" else paste0(" [", copula, " copula]")
  cli_inform("Fitting constant copula model{margins_label}{copula_label} (n_time = {stan_data$n_time}, D = {stan_data$D})...")

  # Compile model
  model <- .compile_model("constant", margins = margins, copula = copula,
                          stan_file = stan_file, backend = backend)

  # Default init
  if (is.null(init)) {
    D <- stan_data$D
    init <- function() .init_constant_params(D, margins, copula = copula)
  }

  cores <- .normalize_cores(cores, chains)

  # Fit
  fit <- .sample_model(
    compiled_model = model,
    stan_data = stan_data,
    backend = backend,
    chains = chains,
    iter_warmup = iter_warmup,
    iter_sampling = iter_sampling,
    adapt_delta = adapt_delta,
    max_treedepth = max_treedepth,
    seed = seed,
    cores = cores,
    init = init,
    refresh = refresh,
    ...
  )

  # Wrap in S3 class
  out <- new_dcvar_constant_fit(
    fit = fit,
    stan_data = stan_data,
    vars = vars,
    standardized = standardize,
    margins = margins,
    copula = copula,
    skew_direction = skew_direction,
    backend = backend,
    priors = c(
      list(
        mu_sd = prior_mu_sd,
        phi_sd = prior_phi_sd,
        sigma_eps_rate = prior_sigma_eps_rate
      ),
      # z_rho_sd does not apply to Clayton fits; recording it would suggest
      # the prior was used.
      if (identical(copula, "gaussian")) list(z_rho_sd = prior_z_rho_sd),
      list(copula = copula)
    ),
    meta = list(
      chains = chains,
      iter_warmup = iter_warmup,
      iter_sampling = iter_sampling,
      adapt_delta = adapt_delta,
      max_treedepth = max_treedepth,
      seed = seed
    )
  )

  .report_sampling_outcome(out$fit, "Constant copula",
                           chains = chains, backend = backend, object = out)
  out
}

test_that("dcvar_constant() returns a dcvar_constant_fit object", {
  skip_if_no_rstan()

  fit <- get_constant_fit()
  expect_s3_class(fit, "dcvar_constant_fit")
  expect_s3_class(fit, "dcvar_model_fit")
  expect_equal(fit$model, "constant")
})

test_that("print.dcvar_constant_fit runs without error", {
  skip_if_no_rstan()

  fit <- get_constant_fit()
  out <- capture.output(print(fit))
  expect_true(any(grepl("Constant", out)))
  expect_true(any(grepl("rho:", out)))
})

test_that("summary.dcvar_constant_fit returns expected class", {
  skip_if_no_rstan()

  fit <- get_constant_fit()
  s <- summary(fit)
  expect_s3_class(s, "dcvar_constant_summary")
  out <- capture.output(print(s))
  expect_true(any(grepl("Constant", out)))
})

test_that("summary.dcvar_constant_fit prints custom quantiles", {
  skip_if_no_rstan()

  fit <- get_constant_fit()
  s <- summary(fit, probs = c(0.1, 0.9))
  expect_no_error(capture.output(print(s)))
})

test_that("coef.dcvar_constant_fit returns named list", {
  skip_if_no_rstan()

  fit <- get_constant_fit()
  co <- coef(fit)
  expect_type(co, "list")
  expect_named(co, c("mu", "Phi", "sigma_eps", "rho"))
})

test_that("plot.dcvar_constant_fit dispatches without error", {
  skip_if_no_rstan()

  fit <- get_constant_fit()
  expect_s3_class(plot(fit, type = "phi"), "ggplot")
})

test_that("dcvar_constant() fits gamma margins", {
  skip_if_no_rstan()

  fit <- get_constant_gamma_fit()
  expect_s3_class(fit, "dcvar_constant_fit")
  expect_equal(fit$margins, "gamma")
})

test_that("dcvar_constant() fits skew-normal margins", {
  skip_if_no_rstan()
  skip_if_not_installed("sn")

  fit <- get_constant_skew_normal_fit()
  expect_s3_class(fit, "dcvar_constant_fit")
  expect_equal(fit$margins, "skew_normal")
})

test_that("constant fit cache emits only known diagnostic warnings", {
  skip_if_no_rstan()

  expect_known_fit_warnings(get_constant_fit_warnings(), "constant")
  expect_known_fit_warnings(get_constant_gamma_fit_warnings(), "constant gamma")

  skip_if_not_installed("sn")
  expect_known_fit_warnings(
    get_constant_skew_normal_fit_warnings(),
    "constant skew_normal"
  )
})

# ============================================================================
# SEM Simulation
# ============================================================================

#' Internal: validate a positive finite scalar for SEM simulation inputs
#' @noRd
.simulate_sem_validate_positive_scalar <- function(x, arg_name) {
  if (!is.numeric(x) || length(x) != 1L || !is.finite(x) || x <= 0) {
    cli_abort("{.arg {arg_name}} must be a single positive finite numeric value.")
  }
}

#' Internal: validate a finite numeric vector for SEM simulation inputs
#' @noRd
.simulate_sem_validate_numeric_vector <- function(x, arg_name) {
  if (!is.numeric(x) || length(x) == 0L || any(!is.finite(x))) {
    cli_abort("{.arg {arg_name}} must be a non-empty finite numeric vector.")
  }
}

#' Simulate data from a SEM copula VAR(1) model
#'
#' Generates indicator-level time series data from a latent VAR(1) process
#' with Gaussian copula dependence and a fixed measurement model.
#'
#' @param n_time Number of time points.
#' @param J Number of indicators per latent variable.
#' @param lambda Numeric vector of length J with factor loadings.
#' @param sigma_e Measurement error SD (scalar).
#' @param Phi 2x2 VAR coefficient matrix.
#' @param mu Length-2 intercept vector.
#' @param margins Latent-innovation marginal family. Either a single string
#'   applied to both latent variables, or a length-2 character vector for
#'   per-variable (mixed) margins, e.g. `c("normal", "gamma")`, where each entry
#'   is one of `"normal"`, `"exponential"`, `"skew_normal"`, or `"gamma"`.
#' @param sigma Length-2 latent innovation scale vector used by normal,
#'   skew-normal, gamma, and mixed-margin specifications. Values are on each
#'   family's natural residual scale: innovation SD for normal/skew-normal,
#'   and `sigma_gam` for gamma. Homogeneous exponential margins use
#'   `sigma_exp` instead.
#' @param sigma_exp Length-2 shifted-exponential scale vector for the
#'   single-family exponential path.
#' @param skew_direction Integer vector of length 2 of `1`/`-1`. Required
#'   whenever any dimension uses an `"exponential"` or `"gamma"` margin.
#' @param skew_params Named list of margin-specific parameters for mixed
#'   margins: `alpha` (length-2 skew-normal shape) and/or `shape` (scalar gamma
#'   shape).
#' @param rho Copula correlation.
#' @param rho_trajectory Optional numeric vector of length `n_time - 1` for a
#'   time-varying copula correlation. Mutually exclusive with `rho`.
#' @param phi_trajectory Optional time-varying latent VAR coefficient paths,
#'   accepted in the same forms as [simulate_dcvar()].
#' @param sigma_trajectory Optional time-varying latent innovation scale paths,
#'   accepted in the same forms as [simulate_dcvar()]. The supplied value is
#'   each family's natural scale: innovation SD (normal), residual SD
#'   (skew-normal), `sigma_exp` (exponential), `sigma_gam` (gamma).
#' @param tv_sigma_k Soft-barrier sharpness for time-varying exponential/gamma
#'   scales.
#' @param burnin Retained for backward compatibility but ignored. Default `0`
#'   keeps the default simulation path aligned with the fitted SEM model,
#'   which conditions on `x_0 = 0` and treats the first returned state as
#'   observed rather than drawn after a burn-in period.
#' @param seed Random seed for reproducibility.
#'
#' @return A named list with:
#'   - `data`: data frame with columns `time`, `y1_1`, ..., `y1_J`,
#'     `y2_1`, ..., `y2_J`
#'   - `true_params`: list of true parameter values
#'   - `latent_states`: `n_time x 2` matrix of true latent states
#'   - `innovations`: `n_time x 2` matrix of true innovations
#' @export
simulate_dcvar_sem <- function(n_time = 200, J = 3,
                                lambda = rep(sqrt(0.8), 3),
                                sigma_e = sqrt(0.2),
                                Phi = matrix(c(0.5, 0.15, 0.15, 0.3), 2, 2),
                                mu = c(0, 0),
                                margins = "normal",
                                sigma = c(1, 1),
                                sigma_exp = c(1, 1),
                                skew_direction = NULL,
                                skew_params = NULL,
                                rho = 0.3,
                                rho_trajectory = NULL,
                                phi_trajectory = NULL,
                                sigma_trajectory = NULL,
                                tv_sigma_k = NULL,
                                burnin = 0,
                                seed = NULL) {
  if (!is.null(seed)) set.seed(seed)
  margins <- .normalize_margins_spec(margins)
  .validate_sem_margins(margins, skew_direction)
  margins_vec <- if (length(margins) == 1L) rep(margins, 2L) else margins
  mixed <- .is_mixed_margins(margins)
  mixed_engine <- .uses_sem_multilevel_mixed_engine("sem", margins)
  engine_margins <- .sem_multilevel_engine_margins("sem", margins, 2L)
  skew_params <- if (is.list(skew_params)) skew_params else list()
  if (any(margins_vec == "skew_normal")) skew_params$alpha <- skew_params$alpha %||% rep(0, 2L)
  if (any(margins_vec == "gamma")) skew_params$shape <- skew_params$shape %||% 1
  if (!is.numeric(n_time) || length(n_time) != 1L || n_time != as.integer(n_time) || n_time < 1) {
    cli_abort("{.arg n_time} must be an integer >= 1, got {.val {n_time}}.")
  }
  if (!is.numeric(J) || length(J) != 1L || J != as.integer(J) || J < 1) {
    cli_abort("{.arg J} must be an integer >= 1, got {.val {J}}.")
  }

  if (length(lambda) != J) {
    cli_abort("{.arg lambda} must have length {.val {J}}, got {.val {length(lambda)}}.")
  }
  .simulate_sem_validate_numeric_vector(lambda, "lambda")
  .simulate_sem_validate_positive_scalar(sigma_e, "sigma_e")
  if (!is.matrix(Phi) || !all(dim(Phi) == c(2L, 2L)) || any(!is.finite(Phi))) {
    cli_abort("{.arg Phi} must be a finite 2x2 matrix.")
  }
  if (!is.null(phi_trajectory) && !missing(Phi)) {
    cli_abort("Supply either {.arg Phi} (constant) or {.arg phi_trajectory} (time-varying), not both.")
  }
  .simulate_sem_validate_numeric_vector(mu, "mu")
  if (length(mu) != 2L) {
    cli_abort("{.arg mu} must have length 2, got {.val {length(mu)}}.")
  }
  # sigma is the constant scale for every non-homogeneous-exponential path.
  if (!(!mixed && identical(margins, "exponential"))) {
    .simulate_sem_validate_numeric_vector(sigma, "sigma")
    if (length(sigma) != 2L) {
      cli_abort("{.arg sigma} must have length 2, got {.val {length(sigma)}}.")
    }
    if (any(sigma <= 0)) {
      cli_abort("{.arg sigma} values must be positive.")
    }
  }
  if (!mixed && identical(margins, "exponential")) {
    .simulate_sem_validate_numeric_vector(sigma_exp, "sigma_exp")
    if (length(sigma_exp) != 2L) {
      cli_abort("{.arg sigma_exp} must have length 2, got {.val {length(sigma_exp)}}.")
    }
    if (any(sigma_exp <= 0)) {
      cli_abort("{.arg sigma_exp} values must be positive.")
    }
  }
  if (!is.numeric(rho) || length(rho) != 1L || !is.finite(rho) || rho < -1 || rho > 1) {
    cli_abort("{.arg rho} must be a single finite numeric value in [-1, 1].")
  }
  if (!is.null(rho_trajectory) && !missing(rho)) {
    cli_abort("Supply either {.arg rho} (constant) or {.arg rho_trajectory} (time-varying), not both.")
  }
  if (!is.null(sigma_trajectory) && !missing(sigma)) {
    cli_abort("Supply either {.arg sigma} (constant) or {.arg sigma_trajectory} (time-varying), not both.")
  }
  if (!is.null(sigma_trajectory) && !missing(sigma_exp)) {
    cli_abort("Supply either {.arg sigma_exp} (constant) or {.arg sigma_trajectory} (time-varying), not both.")
  }
  if (!is.null(tv_sigma_k)) {
    .simulate_validate_positive_scalar(tv_sigma_k, "tv_sigma_k")
  }

  if (!identical(burnin, 0L) && !identical(burnin, 0)) {
    cli_warn(
      c(
        "{.arg burnin} is ignored for SEM simulation.",
        "i" = "The fitted SEM model treats the first returned state as the observed draw implied by `x_0 = 0`."
      )
    )
  }

  rho_path <- if (is.null(rho_trajectory)) {
    rep(rho, max(n_time - 1L, 1L))
  } else {
    if (length(rho_trajectory) != n_time - 1L) {
      cli_abort("{.arg rho_trajectory} must have length {.val {n_time - 1L}}.")
    }
    if (any(!is.finite(rho_trajectory)) || any(abs(rho_trajectory) > 1)) {
      cli_abort("{.arg rho_trajectory} values must be finite and in [-1, 1].")
    }
    rho_trajectory
  }
  phi_mat <- if (is.null(phi_trajectory)) {
    .tv_resolve_trajectory(Phi, n_time, 4L, "Phi")
  } else {
    .tv_resolve_trajectory(phi_trajectory, n_time, 4L, "phi_trajectory")
  }
  scale_mat <- if (is.null(sigma_trajectory)) {
    NULL
  } else {
    m <- .tv_resolve_trajectory(sigma_trajectory, n_time, 2L, "sigma_trajectory")
    if (any(m <= 0)) {
      cli_abort("{.arg sigma_trajectory} values must be positive.")
    }
    if (is.null(tv_sigma_k)) {
      for (d in seq_len(2L)) {
        if (margins_vec[d] %in% c("exponential", "gamma") && length(unique(m[, d])) > 1L) {
          cli_abort(c(
            "{.arg sigma_trajectory} is not constant on latent dimension {d} ({.val {margins_vec[d]}} margin).",
            "i" = "Set {.arg tv_sigma_k} to simulate from the soft-barrier model, or keep the scale constant."
          ))
        }
      }
    }
    m
  }

  base_scales <- if (!mixed && identical(margins, "exponential")) {
    sigma_exp
  } else {
    sigma
  }

  # Generate correlated innovations via Gaussian copula
  zeta <- matrix(NA_real_, n_time, 2)
  for (time_index in seq_len(n_time)) {
    tt <- if (time_index == 1L) 1L else time_index - 1L
    rho_t <- rho_path[tt]
    L <- matrix(c(1, rho_t, 0, sqrt(1 - rho_t^2)), 2, 2)
    z <- rnorm(2)
    w <- drop(L %*% z)
    scales_t <- if (is.null(scale_mat)) base_scales else scale_mat[tt, ]
    sim_margins <- if (mixed_engine) engine_margins else margins_vec
    zeta[time_index, ] <- .sim_marginal_quantile_scaled(
      w, sim_margins, scales_t, skew_direction, skew_params,
      barrier_k = tv_sigma_k
    )
  }

  # Latent VAR(1) recursion matching the SEM Stan model, which conditions on x_0 = 0.
  state <- matrix(0, n_time, 2)
  state[1, ] <- mu + zeta[1, ]
  for (time_index in seq_len(n_time - 1L) + 1L) {
    Phi_t <- matrix(phi_mat[time_index - 1L, ], 2, 2, byrow = TRUE)
    state[time_index, ] <- mu + as.vector(Phi_t %*% state[time_index - 1L, ]) + zeta[time_index, ]
  }

  # Measurement model: y_{ij,t} = lambda_j * state_{i,t} + e_{ij,t}
  y <- matrix(NA_real_, n_time, 2 * J)
  for (j in seq_len(J)) {
    y[, j]     <- lambda[j] * state[, 1] + rnorm(n_time, 0, sigma_e)
    y[, J + j] <- lambda[j] * state[, 2] + rnorm(n_time, 0, sigma_e)
  }
  colnames(y) <- c(paste0("y1_", seq_len(J)), paste0("y2_", seq_len(J)))

  data <- data.frame(time = seq_len(n_time), y, check.names = FALSE)

  true_params <- list(
    Phi = if (is.null(phi_trajectory)) {
      Phi
    } else {
      colnames(phi_mat) <- c("phi11", "phi12", "phi21", "phi22")
      phi_mat
    },
    mu = mu,
    margins = margins,
    rho = if (is.null(rho_trajectory)) rho else rho_path,
    lambda = lambda,
    sigma_e = sigma_e,
    J = J
  )
  if (mixed_engine) {
    true_params$sigma <- if (is.null(scale_mat)) sigma else scale_mat
    if (any(engine_margins %in% c("exponential", "gamma"))) {
      true_params$skew_direction <- skew_direction
    }
    if (any(engine_margins %in% c("skew_normal", "gamma"))) {
      true_params$skew_params <- skew_params
    }
  } else if (identical(margins, "normal")) {
    true_params$sigma <- if (is.null(scale_mat)) sigma else scale_mat
  } else {
    true_params$sigma_exp <- if (is.null(scale_mat)) sigma_exp else scale_mat
    true_params$skew_direction <- skew_direction
  }

  list(
    data = data,
    true_params = true_params,
    latent_states = state,
    innovations = zeta
  )
}

test_that("simulate_dcvar_sem returns correct structure", {
  sim <- simulate_dcvar_sem(n_time = 50, rho = 0.5, seed = 1)

  expect_type(sim, "list")
  expect_named(sim, c("data", "true_params", "latent_states", "innovations"))
  expect_s3_class(sim$data, "data.frame")
})

test_that("simulate_dcvar_sem data has correct dimensions", {
  J <- 3
  n_time_val <- 50
  sim <- simulate_dcvar_sem(n_time = n_time_val, J = J, rho = 0.5, seed = 2)

  # Data: T rows, 1 time column + 2*J indicator columns
  expect_equal(nrow(sim$data), n_time_val)
  expect_equal(ncol(sim$data), 1 + 2 * J)
  expect_true("time" %in% names(sim$data))
})

test_that("simulate_dcvar_sem indicator column names are correct", {
  J <- 3
  sim <- simulate_dcvar_sem(n_time = 40, J = J, rho = 0.3, seed = 3)

  expected_cols <- c("time",
                     paste0("y1_", seq_len(J)),
                     paste0("y2_", seq_len(J)))
  expect_equal(names(sim$data), expected_cols)
})

test_that("simulate_dcvar_sem latent_states has correct dimensions", {
  n_time_val <- 60
  sim <- simulate_dcvar_sem(n_time = n_time_val, rho = 0.5, seed = 4)

  expect_equal(nrow(sim$latent_states), n_time_val)
  expect_equal(ncol(sim$latent_states), 2)
})

test_that("simulate_dcvar_sem innovations has correct dimensions", {
  n_time_val <- 60
  sim <- simulate_dcvar_sem(n_time = n_time_val, rho = 0.5, seed = 5)

  expect_equal(nrow(sim$innovations), n_time_val)
  expect_equal(ncol(sim$innovations), 2)
})

test_that("simulate_dcvar_sem true_params contains expected elements", {
  J <- 3
  sim <- simulate_dcvar_sem(n_time = 50, J = J, rho = 0.4, seed = 6)

  expect_named(sim$true_params,
               c("Phi", "mu", "margins", "rho", "lambda", "sigma_e", "J", "sigma"))
  expect_equal(sim$true_params$rho, 0.4)
  expect_equal(sim$true_params$J, J)
  expect_equal(length(sim$true_params$lambda), J)
  expect_equal(dim(sim$true_params$Phi), c(2, 2))
  expect_equal(sim$true_params$margins, "normal")
})

test_that("simulate_dcvar_sem errors when lambda length mismatches J", {
  expect_error(
    simulate_dcvar_sem(n_time = 50, J = 3, lambda = c(0.8, 0.8), rho = 0.5),
    "lambda"
  )
  expect_error(
    simulate_dcvar_sem(n_time = 50, J = 2, lambda = c(0.8, 0.8, 0.8), rho = 0.5),
    "lambda"
  )
})

test_that("simulate_dcvar_sem validates lambda finiteness and sigma_e", {
  expect_error(
    simulate_dcvar_sem(n_time = 50, J = 2, lambda = c(0.8, NA_real_), rho = 0.5),
    "lambda"
  )
  expect_error(
    simulate_dcvar_sem(n_time = 50, J = 2, lambda = c(0.8, 0.8), sigma_e = 0, rho = 0.5),
    "sigma_e"
  )
})

test_that("simulate_dcvar_sem validates Phi and sigma shapes", {
  expect_error(
    simulate_dcvar_sem(n_time = 20, Phi = matrix(1:3, nrow = 1), rho = 0.5),
    "Phi"
  )
  expect_error(
    simulate_dcvar_sem(n_time = 20, sigma = c(1, 1, 1), rho = 0.5),
    "sigma"
  )
})

test_that("simulate_dcvar_sem supports exponential latent margins", {
  sim <- simulate_dcvar_sem(
    n_time = 40,
    J = 2,
    lambda = rep(sqrt(0.8), 2),
    margins = "exponential",
    sigma_exp = c(0.7, 1.1),
    skew_direction = c(1, -1),
    rho = 0.25,
    seed = 12
  )

  expect_equal(sim$true_params$margins, "exponential")
  expect_equal(sim$true_params$sigma_exp, c(0.7, 1.1))
  expect_equal(sim$true_params$skew_direction, c(1, -1))
  expect_true(all(is.finite(sim$innovations)))
})

test_that("simulate_dcvar_sem records the scale used for skew-normal and gamma innovations", {
  skip_if_not_installed("sn")

  sim_sn <- simulate_dcvar_sem(
    n_time = 8000,
    J = 1,
    lambda = 1,
    sigma_e = 1e-12,
    Phi = matrix(0, 2, 2),
    margins = "skew_normal",
    sigma = c(2, 2.5),
    skew_params = list(alpha = c(0, 0)),
    rho = 0,
    seed = 21
  )
  expect_equal(sim_sn$true_params$sigma, c(2, 2.5))
  expect_equal(unname(apply(sim_sn$innovations, 2, stats::sd)), sim_sn$true_params$sigma, tolerance = 0.06)

  sim_gam <- simulate_dcvar_sem(
    n_time = 8000,
    J = 1,
    lambda = 1,
    sigma_e = 1e-12,
    Phi = matrix(0, 2, 2),
    margins = "gamma",
    sigma = c(2, 2.5),
    skew_direction = c(1, 1),
    skew_params = list(shape = 2),
    rho = 0,
    seed = 22
  )
  expect_equal(sim_gam$true_params$sigma, c(2, 2.5))
  expect_equal(unname(apply(sim_gam$innovations, 2, stats::sd)), sim_gam$true_params$sigma, tolerance = 0.08)
})

test_that("simulate_dcvar_sem exponential margins require skew_direction", {
  expect_error(
    simulate_dcvar_sem(n_time = 20, margins = "exponential", sigma_exp = c(1, 1)),
    "skew_direction"
  )
})

test_that("simulate_dcvar_sem works with different J values", {
  for (J in c(2, 4, 5)) {
    sim <- simulate_dcvar_sem(n_time = 40, J = J,
                              lambda = rep(0.8, J),
                              rho = 0.5, seed = 100 + J)
    expect_equal(ncol(sim$data), 1 + 2 * J, info = paste("J =", J))
    expect_equal(nrow(sim$data), 40, info = paste("J =", J))
    expect_equal(length(sim$true_params$lambda), J, info = paste("J =", J))
  }
})

test_that("simulate_dcvar_sem is reproducible with seed", {
  s1 <- simulate_dcvar_sem(n_time = 30, rho = 0.5, seed = 42)
  s2 <- simulate_dcvar_sem(n_time = 30, rho = 0.5, seed = 42)
  expect_identical(s1$data, s2$data)
  expect_identical(s1$latent_states, s2$latent_states)
})

test_that("simulate_dcvar_sem warns that burnin is ignored", {
  expect_warning(
    simulate_dcvar_sem(n_time = 20, burnin = 10, seed = 11),
    "ignored"
  )
})

#' Get path to bundled Stan model file
#'
#' Returns the file path to a Stan model file included with the package.
#'
#' @param model Character string: `"dcvar"`, `"dcvar_tv"`, `"dcvar_dynamic"`
#'   (the unified time-varying / drift covariate engine), `"dcvar_covariate"`,
#'   `"dcvar_covariate_nodrift"`, `"hmm"`, `"constant"`, `"multilevel"`,
#'   `"multilevel_tv"`, `"sem"`, `"sem_tv"`, or `"sem_naive"`.
#' @param margins Character string: margin type (`"normal"`, `"exponential"`,
#'   `"skew_normal"`, `"gamma"`). Default: `"normal"`.
#' @param copula Character string: copula family (`"gaussian"` or
#'   `"clayton"`). Default: `"gaussian"`.
#'
#' @return File path to the Stan model file.
#' @export
#'
#' @examples
#' dcvar_stan_path("dcvar")
#' dcvar_stan_path("constant", margins = "exponential")
dcvar_stan_path <- function(model = c("dcvar", "dcvar_tv", "dcvar_dynamic",
                                      "dcvar_covariate",
                                      "dcvar_covariate_nodrift", "hmm",
                                      "hmm_switching",
                                      "constant", "multilevel", "multilevel_tv",
                                      "sem", "sem_tv", "sem_naive"),
                            margins = "normal",
                            copula = "gaussian") {
  model <- match.arg(model)
  .validate_copula(copula)

  if (model %in% c("dcvar_covariate", "dcvar_covariate_nodrift")) {
    if (!identical(margins, "normal")) {
      cli_abort("Covariate DC-VAR Stan models currently support only {.val normal} margins.")
    }
    if (!identical(copula, "gaussian")) {
      cli_abort("Covariate DC-VAR Stan models currently support only the {.val gaussian} copula.")
    }
    # The public path helper returns files matching the exported prepare_*()
    # data contracts. The fit wrapper routes drift fits to dcvar_dynamic.stan
    # internally when stan_file is NULL.
    file <- if (identical(model, "dcvar_covariate")) {
      "dcvar_covariate_ncp.stan"
    } else {
      "dcvar_covariate_nodrift.stan"
    }
  } else {
    file <- .margin_stan_file(model, margins, copula = copula)
  }

  system.file("stan", file, package = "dcvar", mustWork = TRUE)
}

#' Internal: compare a user-supplied Stan path against a bundled target
#'
#' Returns `FALSE` for `NULL`, `NA`, non-scalar, or non-existing paths so callers
#' can safely use it before the usual compile-time file validation.
#' @noRd
.same_stan_file <- function(stan_file, target_file) {
  if (is.null(stan_file) || length(stan_file) != 1L ||
      is.na(stan_file) || !nzchar(stan_file)) {
    return(FALSE)
  }
  if (is.null(target_file) || length(target_file) != 1L ||
      is.na(target_file) || !nzchar(target_file)) {
    return(FALSE)
  }

  tryCatch(
    identical(
      normalizePath(stan_file, winslash = "/", mustWork = TRUE),
      normalizePath(target_file, winslash = "/", mustWork = TRUE)
    ),
    error = function(e) FALSE
  )
}

#' Internal: should this call use the bundled dynamic engine?
#' @noRd
.uses_dynamic_stan_file <- function(stan_file, margins = "normal") {
  is.null(stan_file) ||
    .same_stan_file(stan_file, dcvar_stan_path("dcvar_dynamic", margins = margins))
}

#' Internal: was the bundled Markov-switching HMM engine explicitly requested?
#'
#' Unlike the dynamic helper, this does NOT treat `stan_file = NULL` as the
#' engine: the default HMM routes to the legacy files, and the engine is selected
#' by the feature flags. It only recognises a `stan_file` that IS the bundled
#' `hmm_switching.stan`, so handing that file forces the engine (with data
#' augmentation + switching init) even on a rho-only config.
#' @noRd
.uses_hmm_switching_stan_file <- function(stan_file) {
  .same_stan_file(stan_file, dcvar_stan_path("hmm_switching"))
}

#' Validate common MCMC sampling arguments
#'
#' @param chains,iter_warmup,iter_sampling,adapt_delta,max_treedepth
#'   Sampling arguments to validate.
#' @return Invisible `TRUE` if all checks pass.
#' @noRd
.validate_sampling_args <- function(chains, iter_warmup, iter_sampling,
                                    adapt_delta, max_treedepth) {
  is_positive_integer_scalar <- function(x) {
    is.numeric(x) && length(x) == 1L && is.finite(x) && x >= 1 && x == floor(x)
  }

  if (!is_positive_integer_scalar(chains)) {
    cli_abort("{.arg chains} must be a positive integer, got {.val {chains}}.")
  }
  if (!is_positive_integer_scalar(iter_warmup)) {
    cli_abort("{.arg iter_warmup} must be a positive integer, got {.val {iter_warmup}}.")
  }
  if (!is_positive_integer_scalar(iter_sampling)) {
    cli_abort("{.arg iter_sampling} must be a positive integer, got {.val {iter_sampling}}.")
  }
  if (!is.numeric(adapt_delta) || length(adapt_delta) != 1L ||
      !is.finite(adapt_delta) || adapt_delta <= 0 || adapt_delta >= 1) {
    cli_abort("{.arg adapt_delta} must be in (0, 1), got {.val {adapt_delta}}.")
  }
  if (!is_positive_integer_scalar(max_treedepth)) {
    cli_abort("{.arg max_treedepth} must be a positive integer, got {.val {max_treedepth}}.")
  }
  invisible(TRUE)
}


#' Check cmdstanr availability
#'
#' @return Invisible `TRUE` if cmdstanr and CmdStan are available.
#' @noRd
.check_cmdstanr <- function() {
  if (!requireNamespace("cmdstanr", quietly = TRUE)) {
    cli_abort(c(
      "Package {.pkg cmdstanr} is required but not installed.",
      "i" = "Install with: {.code install.packages('cmdstanr', repos = c('https://stan-dev.r-universe.dev', getOption('repos')))}",
      "i" = "Then run: {.code cmdstanr::install_cmdstan()}"
    ))
  }
  cmdstan_path <- tryCatch(
    cmdstanr::cmdstan_path(),
    error = function(e) NULL
  )
  if (is.null(cmdstan_path)) {
    cli_abort(c(
      "CmdStan is not installed.",
      "i" = "Run: {.code cmdstanr::install_cmdstan()}"
    ))
  }
  invisible(TRUE)
}

#' Build a platform-appropriate cached executable path
#'
#' @param cache_dir Directory used for compiled model caching.
#' @param cache_key Basename for the cached executable.
#' @return Absolute path to the cached executable.
#' @noRd
.compiled_exe_path <- function(cache_dir, cache_key) {
  exe_file <- file.path(cache_dir, cache_key)
  if (.Platform$OS.type == "windows" && !grepl("\\.exe$", exe_file, ignore.case = TRUE)) {
    exe_file <- paste0(exe_file, ".exe")
  }
  exe_file
}

#' Compile a Stan model with caching
#'
#' @param model_type Character string: `"dcvar"`, `"dcvar_covariate"`,
#'   `"dcvar_covariate_nodrift"`, `"hmm"`, `"constant"`, `"multilevel"`,
#'   `"multilevel_tv"`, `"sem"`, `"sem_tv"`, or `"sem_naive"`.
#' @param margins Character: margin type (default: `"normal"`).
#' @param copula Character: copula family (default: `"gaussian"`).
#' @param stan_file Optional path to a custom Stan file. If `NULL`, uses the
#'   bundled model.
#' @param backend Character: `"rstan"` or `"cmdstanr"`.
#' @param force_recompile Force recompilation even if cached.
#' @param quiet Suppress compilation messages.
#'
#' @return A compiled model object (backend-dependent class).
#' @noRd
.compile_model <- function(model_type = c("dcvar", "dcvar_tv", "dcvar_dynamic",
                                          "dcvar_covariate",
                                          "dcvar_covariate_nodrift", "hmm",
                                          "hmm_switching",
                                          "constant", "multilevel", "multilevel_tv",
                                          "sem", "sem_tv", "sem_naive"),
                           margins = "normal",
                           copula = "gaussian",
                           stan_file = NULL,
                           backend = "rstan",
                           force_recompile = FALSE,
                           quiet = FALSE) {
  model_type <- match.arg(model_type)

  if (is.null(stan_file)) {
    stan_file <- dcvar_stan_path(model_type, margins = margins, copula = copula)
  }

  .compile_model_backend(
    stan_file = stan_file,
    backend = backend,
    force_recompile = force_recompile,
    quiet = quiet
  )
}

# ============================================================================
# S3 Class: dcvar_hmm_fit
# ============================================================================

#' Construct a dcvar_hmm_fit object
#' @noRd
new_dcvar_hmm_fit <- function(fit, stan_data, K, vars, standardized,
                              margins = "normal", skew_direction = NULL,
                              backend = "rstan", priors, meta,
                              switch = NULL, switching = FALSE,
                              margins_matrix = NULL) {
  structure(
    list(
      fit = fit,
      stan_data = stan_data,
      model = "hmm",
      K = K,
      vars = vars,
      standardized = standardized,
      margins = margins,
      skew_direction = skew_direction,
      switch = switch,
      switching = switching,
      margins_matrix = margins_matrix,
      backend = backend,
      priors = priors,
      meta = meta
    ),
    class = c("dcvar_hmm_fit", "dcvar_model_fit")
  )
}


#' S3 methods for dcvar_hmm_fit objects
#'
#' @param x,object A `dcvar_hmm_fit` object.
#' @param ... Additional arguments (unused).
#'
#' @name dcvar_hmm_fit-methods
NULL

#' @describeIn dcvar_hmm_fit-methods Print a concise overview of the HMM fit.
#' @return Invisibly returns `x`.
#' @export
print.dcvar_hmm_fit <- function(x, ...) {
  .print_fit_header(x, "HMM Copula Model Fit")
  cat(sprintf("n_time = %d, D = %d, K = %d states\n",
              x$stan_data$n_time, x$stan_data$D, x$K))
  .print_fit_footer(x)

  states <- hmm_states(x)
  cat("\nState-specific rho:\n")
  for (k in seq_along(states$rho_state$mean)) {
    cat(sprintf("  State %d: %.3f [%.3f, %.3f]\n",
                k, states$rho_state$mean[k],
                states$rho_state$lower[k], states$rho_state$upper[k]))
  }

  cat("\nTransition matrix diagonal: ")
  cat(sprintf("%.3f", diag(states$A)), sep = ", ")
  cat("\n")

  invisible(x)
}


#' @describeIn dcvar_hmm_fit-methods Produce a detailed summary including
#'   state information, VAR parameters, and diagnostics.
#' @param probs Numeric vector of quantile probabilities for the rho trajectory
#'   (default: `c(0.025, 0.5, 0.975)`).
#' @return A `dcvar_hmm_summary` object (a list).
#' @export
summary.dcvar_hmm_fit <- function(object, probs = c(0.025, 0.5, 0.975), ...) {
  rho_df <- rho_trajectory(object, probs = probs)
  vp <- var_params(object)
  states <- hmm_states(object)
  diag <- dcvar_diagnostics(object)

  out <- list(
    model = "hmm",
    n_time = object$stan_data$n_time,
    D = object$stan_data$D,
    K = object$K,
    rho_trajectory = rho_df,
    states = states,
    state_params = if (isTRUE(object$switching)) hmm_state_params(object) else NULL,
    var_params = vp,
    diagnostics = diag
  )
  class(out) <- "dcvar_hmm_summary"
  out
}


#' Print a dcvar_hmm_summary object
#'
#' @param x A `dcvar_hmm_summary` object as returned by
#'   [summary.dcvar_hmm_fit()].
#' @param ... Additional arguments (unused).
#' @return Invisibly returns `x`.
#' @export
print.dcvar_hmm_summary <- function(x, ...) {
  cat("HMM Copula Model Summary\n")
  cat(strrep("=", 50), "\n")
  cat(sprintf("n_time = %d, D = %d, K = %d states\n\n", x$n_time, x$D, x$K))

  cat("State-Specific Rho:\n")
  for (k in seq_along(x$states$rho_state$mean)) {
    cat(sprintf("  State %d: %.3f [%.3f, %.3f]\n",
                k, x$states$rho_state$mean[k],
                x$states$rho_state$lower[k], x$states$rho_state$upper[k]))
  }

  cat("\nTransition Matrix:\n")
  print(round(x$states$A, 3))

  if (!is.null(x$state_params)) {
    cat("\nState-Specific VAR(1) Parameters (posterior means):\n")
    for (k in seq_len(x$K)) {
      cat(sprintf("  State %d mu: %s\n", k, paste(sprintf("%.3f", x$state_params$mu[k, ]), collapse = ", ")))
      cat(sprintf("  State %d Phi:\n", k))
      print(round(x$state_params$Phi[[k]], 3))
    }
    if (!is.null(x$state_params$scales) && nrow(x$state_params$scales) > 0L) {
      cat("\n  Margin scales / shapes:\n")
      scale_cols <- c("state", "variable", "family", "parameter", "mean")
      print(x$state_params$scales[, scale_cols], row.names = FALSE)
    }
  } else {
    cat("\nVAR(1) Parameters:\n")
    cat("  mu:\n")
    print(x$var_params$mu[, c("variable", "mean", "q2.5", "q97.5")], row.names = FALSE)
    cat("\n  Phi:\n")
    print(x$var_params$Phi[, c("variable", "mean", "q2.5", "q97.5")], row.names = FALSE)
    .print_margin_params(x$var_params)
  }

  cat("\nDiagnostics:\n")
  cat(sprintf("  Divergences: %d\n", x$diagnostics$n_divergent))
  cat(sprintf("  Max Rhat: %.3f\n", x$diagnostics$max_rhat))
  cat(sprintf("  Min ESS bulk: %.0f\n", x$diagnostics$min_ess_bulk))

  invisible(x)
}


#' @describeIn dcvar_hmm_fit-methods Extract posterior means of model
#'   coefficients including state-specific rho values.
#' @return A named list of posterior means: `mu`, `Phi`, margin-specific
#'   scale/shape parameters (e.g. `sigma_eps` for normal margins), `z_rho`, and
#'   `rho_state`.
#' @export
coef.dcvar_hmm_fit <- function(object, ...) {
  summ <- .fit_summary(object$fit, backend = object$backend)

  if (isTRUE(object$switching)) {
    # The switching engine samples state-indexed mu and a Phi baseline + per-state
    # deviations; margin scales are per config (state when margins switch).
    sw <- object$switch
    K <- object$K
    Mrg_K <- if (isTRUE(sw$margins == 1L)) K else 1L
    result <- list(
      mu = .extract_required_coef(summ, "^mu\\[", "mu", "coef.dcvar_hmm_fit()"),
      Phi = .extract_required_coef(summ, "^Phi_base\\[", "Phi_base", "coef.dcvar_hmm_fit()")
    )
    if (any(sw$phi_mask > 0L)) {
      result$Phi_dev <- .extract_required_coef(summ, "^Phi_dev\\[", "Phi_dev", "coef.dcvar_hmm_fit()")
    }
    report <- .hmm_switching_report_vars(object$margins_matrix, Mrg_K)
    for (nm in names(report)) {
      idx <- match(report[[nm]], summ$variable)
      idx <- idx[!is.na(idx)]
      if (length(idx) > 0L) {
        result[[nm]] <- stats::setNames(summ$mean[idx], summ$variable[idx])
      }
    }
    result$z_rho <- .extract_required_coef(summ, "^z_rho\\[", "z_rho", "coef.dcvar_hmm_fit()")
    result$rho_state <- .extract_required_coef(summ, "^rho_state\\[", "rho_state", "coef.dcvar_hmm_fit()")
    return(result)
  }

  result <- list(
    mu = .extract_required_coef(summ, "^mu\\[", "mu", "coef.dcvar_hmm_fit()"),
    Phi = .extract_required_coef(summ, "^Phi\\[", "Phi", "coef.dcvar_hmm_fit()")
  )
  # Margin-specific scale params before HMM-specific params
  margins <- object$margins %||% "normal"
  result <- c(result, .extract_margin_coefs(summ, margins))
  result$z_rho <- .extract_required_coef(summ, "^z_rho\\[", "z_rho", "coef.dcvar_hmm_fit()")
  result$rho_state <- .extract_required_coef(summ, "^rho_state\\[", "rho_state", "coef.dcvar_hmm_fit()")
  result
}


#' @describeIn dcvar_hmm_fit-methods Dispatch to a plot type: `"rho"`,
#'   `"states"`, `"transition"`, `"phi"`, `"diagnostics"`, `"ppc"`, or `"pit"`.
#' @param type Character; one of `"rho"`, `"states"`, `"transition"`, `"phi"`,
#'   `"diagnostics"`, `"ppc"`, or `"pit"`.
#' @return A ggplot object.
#' @export
plot.dcvar_hmm_fit <- function(x,
                               type = c("rho", "states", "transition", "phi", "diagnostics", "ppc", "pit"),
                               ...) {
  type <- match.arg(type)
  if (isTRUE(x$switching) && type %in% c("phi", "ppc", "pit")) {
    cli_abort(c(
      "{.code plot(type = \"{type}\")} is not available for state-specific HMM fits.",
      "i" = "Use {.fun hmm_state_params} for per-state VAR coefficients/scales, and {.fun hmm_states} for the regime path."
    ))
  }
  switch(type,
    rho = plot_rho(x, ...),
    states = plot_hmm_states(x, ...),
    transition = {
      states <- hmm_states(x)
      .plot_transition_matrix(states$A, x$K)
    },
    phi = plot_phi(x, ...),
    diagnostics = plot_diagnostics(x, ...),
    ppc = plot_ppc(x, ...),
    pit = plot_pit(x, ...)
  )
}

# ============================================================================
# predict() and fitted() methods
# ============================================================================

#' Fitted values from a copula VAR model
#'
#' Returns the one-step-ahead fitted values (posterior mean of y_hat) from the
#' VAR(1) component. For single-level fits (constant, dynamic, HMM, covariate)
#' this is the mean-centered form `y_hat[t] = mu + Phi * (y[t-1] - mu)`.
#'
#' If the model was fit with `standardize = TRUE` (the default), fitted values
#' are on the standardized (z-scored) scale by default. Use `type = "response"`
#' to back-transform to the original data scale.
#'
#' `fitted()` and `predict()` are implemented for the public fit classes. For
#' multilevel fits, the methods return unit-specific trajectories; because the
#' data are already person-mean-centered, the link-scale fitted values apply the
#' unit's VAR matrix to the previous observation with no `mu` term
#' (`y_hat[t] = Phi_unit %*% y[t-1]`), and the person mean is re-added only for
#' `type = "response"`. For SEM fits, `type = "link"`
#' returns latent-state summaries and `type = "response"` returns observed
#' indicator-scale summaries.
#'
#' @param object A fitted model object.
#' @param type Character; `"link"` (default) returns values on the model's
#'   internal scale (standardized if applicable), `"response"` back-transforms
#'   to the original data scale.
#' @param ... Additional arguments (unused).
#'
#' @return A data frame of posterior-mean fitted values. Single-level fits
#'   return columns `time` plus one column per modeled variable. Multilevel
#'   fits additionally include `unit`. SEM fits return either latent-state
#'   columns (`type = "link"`) or observed-indicator columns
#'   (`type = "response"`).
#' @export
fitted.dcvar_model_fit <- function(object, type = c("link", "response"), ...) {
  type <- match.arg(type)
  D <- object$stan_data$D
  n_time_obs <- object$stan_data$n_time
  Y <- object$stan_data$Y
  mu_draws <- posterior::as_draws_matrix(.fit_draws(
    object$fit, "mu", backend = object$backend,
    required = .stan_output_group_pattern("mu"),
    required_type = "pattern",
    context = "fitted.dcvar_model_fit()",
    output_type = "parameter group"
  ))
  Phi_draws <- posterior::as_draws_matrix(.fit_draws(
    object$fit, "Phi", backend = object$backend,
    required = .stan_output_group_pattern("Phi"),
    required_type = "pattern",
    context = "fitted.dcvar_model_fit()",
    output_type = "parameter group"
  ))

  mu_cols <- paste0("mu[", seq_len(D), "]")
  phi_cols <- unlist(lapply(seq_len(D), function(i) paste0("Phi[", i, ",", seq_len(D), "]")))
  mu_mat <- mu_draws[, mu_cols, drop = FALSE]
  phi_mat <- Phi_draws[, phi_cols, drop = FALSE]

  y_hat <- matrix(NA_real_, n_time_obs - 1L, D)
  for (time_index in 2:n_time_obs) {
    centered_prev <- sweep(mu_mat, 2, Y[time_index - 1L, ], FUN = function(mu_val, y_prev) y_prev - mu_val)
    y_hat_draws <- matrix(NA_real_, nrow(mu_mat), D)

    for (d in seq_len(D)) {
      eta <- mu_mat[, d]
      for (j in seq_len(D)) {
        eta <- eta + phi_mat[, (d - 1) * D + j] * centered_prev[, j]
      }
      y_hat_draws[, d] <- eta
    }

    y_hat[time_index - 1L, ] <- colMeans(y_hat_draws)
  }

  if (type == "response" && isTRUE(object$standardized)) {
    Y_means <- attr(object$stan_data, "Y_means")
    Y_sds <- attr(object$stan_data, "Y_sds")
    if (!is.null(Y_means) && !is.null(Y_sds)) {
      for (d in seq_len(D)) {
        y_hat[, d] <- y_hat[, d] * Y_sds[d] + Y_means[d]
      }
    }
  }

  out <- data.frame(time = .observed_time_values(object$stan_data, drop_first = TRUE), y_hat)
  names(out) <- c("time", object$vars)
  out
}


#' One-step-ahead fitted values for a Markov-switching HMM fit
#'
#' For a state-specific `dcvar_hmm()` fit the one-step prediction is the
#' smoothed-state mixture `sum_k gamma[t, k] * (mu_k + Phi_k (Y[t] - mu_k))`,
#' evaluated at posterior means. A non-switching HMM fit delegates to the shared
#' method.
#'
#' @param object A `dcvar_hmm_fit` object.
#' @param type `"link"` (default) or `"response"` (back-transform to the data scale).
#' @param ... Additional arguments (unused).
#' @return A data frame with a `time` column and one column per variable.
#' @export
fitted.dcvar_hmm_fit <- function(object, type = c("link", "response"), ...) {
  type <- match.arg(type)
  if (!isTRUE(object$switching)) {
    return(NextMethod())
  }

  D <- object$stan_data$D
  n_time_obs <- object$stan_data$n_time
  K <- object$K
  Y <- object$stan_data$Y
  sp <- hmm_state_params(object)
  gamma <- hmm_states(object)$gamma

  y_hat <- matrix(NA_real_, n_time_obs - 1L, D)
  for (t in seq_len(n_time_obs - 1L)) {
    y_prev <- Y[t, ]
    pred <- numeric(D)
    for (k in seq_len(K)) {
      mu_k <- sp$mu[k, ]
      pred <- pred + gamma[t, k] * (mu_k + as.numeric(sp$Phi[[k]] %*% (y_prev - mu_k)))
    }
    y_hat[t, ] <- pred
  }

  if (type == "response" && isTRUE(object$standardized)) {
    Y_means <- attr(object$stan_data, "Y_means")
    Y_sds <- attr(object$stan_data, "Y_sds")
    if (!is.null(Y_means) && !is.null(Y_sds)) {
      for (d in seq_len(D)) y_hat[, d] <- y_hat[, d] * Y_sds[d] + Y_means[d]
    }
  }

  out <- data.frame(time = .observed_time_values(object$stan_data, drop_first = TRUE), y_hat)
  names(out) <- c("time", object$vars)
  out
}


#' Prediction intervals for a Markov-switching HMM fit
#'
#' State-specific HMM fits do not yet support marginal prediction intervals (the
#' predictive is a regime mixture). Non-switching fits delegate to the shared
#' method.
#'
#' @param object A `dcvar_hmm_fit` object.
#' @param ... Passed to the shared method for non-switching fits.
#' @return A data frame of prediction intervals (non-switching fits only).
#' @export
predict.dcvar_hmm_fit <- function(object, ...) {
  if (isTRUE(object$switching)) {
    cli_abort(c(
      "{.fun predict} intervals are not yet available for state-specific HMM fits (the predictive is a regime mixture).",
      "i" = "Use {.fun fitted} for gamma-weighted point predictions, or {.fun hmm_states} / {.fun hmm_state_params}."
    ))
  }
  NextMethod()
}


#' Internal: extract posterior mean unit-specific Phi coefficients
#' @noRd
.multilevel_phi_means <- function(object) {
  re <- random_effects(object)
  unit_ids <- attr(object$stan_data, "ids")
  if (is.null(unit_ids)) {
    unit_ids <- seq_len(object$N)
  }
  unit_ids <- as.character(unit_ids)

  param_names <- c("phi11", "phi12", "phi21", "phi22")
  phi_means <- matrix(
    NA_real_,
    nrow = object$N,
    ncol = length(param_names),
    dimnames = list(unit_ids, param_names)
  )

  for (i in seq_len(object$N)) {
    for (p in param_names) {
      row <- which(as.character(re$unit) == unit_ids[i] & re$parameter == p)
      if (length(row) != 1L) {
        cli_abort(c(
          "{.fun fitted} and {.fun predict} require unit-specific VAR coefficients that are not present in the fitted model.",
          "i" = "Missing random effect: {.val {p}} for unit {.val {unit_ids[i]}}.",
          "i" = "Custom Stan files must preserve the expected parameter names."
        ))
      }
      phi_means[i, p] <- re$mean[row]
    }
  }

  phi_means
}


#' Internal: compute multilevel one-step-ahead fitted values
#' @noRd
.multilevel_fitted_values <- function(object, response = FALSE) {
  phi_means <- .multilevel_phi_means(object)
  y_list <- object$stan_data$y
  unit_ids <- attr(object$stan_data, "ids")
  if (is.null(unit_ids)) {
    unit_ids <- seq_len(object$N)
  }
  unit_ids <- as.character(unit_ids)

  person_means <- object$person_means %||% attr(object$stan_data, "person_means")
  response_shift <- isTRUE(object$centered) &&
    !is.null(person_means) &&
    all(is.finite(person_means))

  time_values <- .observed_time_values(object$stan_data, drop_first = TRUE)
  out <- vector("list", object$N)

  for (i in seq_len(object$N)) {
    phi <- phi_means[i, ]
    Phi <- matrix(
      c(phi[["phi11"]], phi[["phi12"]], phi[["phi21"]], phi[["phi22"]]),
      nrow = 2,
      byrow = FALSE
    )

    y_i <- y_list[[i]]
    T_eff <- nrow(y_i) - 1L
    fitted_mat <- matrix(NA_real_, nrow = T_eff, ncol = 2)
    for (t in 2:nrow(y_i)) {
      fitted_mat[t - 1L, ] <- as.numeric(y_i[t - 1L, , drop = FALSE] %*% Phi)
    }

    if (response && response_shift) {
      fitted_mat <- sweep(fitted_mat, 2, person_means[i, ], "+")
    }

    df <- data.frame(
      unit = rep(unit_ids[i], T_eff),
      time = time_values,
      fitted_mat,
      check.names = FALSE
    )
    names(df)[3:4] <- object$vars
    out[[i]] <- df
  }

  do.call(rbind, out)
}


#' @rdname fitted.dcvar_model_fit
#' @export
fitted.dcvar_multilevel_fit <- function(object, type = c("link", "response"), ...) {
  type <- match.arg(type)
  .multilevel_fitted_values(object, response = identical(type, "response"))
}


#' @rdname fitted.dcvar_model_fit
#' @export
fitted.dcvar_multilevel_tv_fit <- function(object, type = c("link", "response"), ...) {
  type <- match.arg(type)
  if (!isTRUE(object$tv_phi)) {
    return(NextMethod())
  }

  y_list <- object$stan_data$y
  unit_ids <- attr(object$stan_data, "ids") %||% seq_len(object$N)
  unit_ids <- as.character(unit_ids)
  n_time_eff <- object$stan_data$n_time - 1L
  time_values <- .observed_time_values(object$stan_data, drop_first = TRUE)

  phi_draws <- posterior::as_draws_matrix(.fit_draws(
    object$fit, "phi_unit_t", backend = object$backend,
    required = .stan_output_group_pattern("phi_unit_t"),
    required_type = "pattern",
    context = "fitted.dcvar_multilevel_tv_fit()",
    output_type = "generated quantity"
  ))

  person_means <- object$person_means %||% attr(object$stan_data, "person_means")
  response_shift <- identical(type, "response") &&
    isTRUE(object$centered) &&
    !is.null(person_means) &&
    all(is.finite(person_means))

  out <- vector("list", object$N)
  for (i in seq_len(object$N)) {
    y_i <- y_list[[i]]
    fitted_mat <- matrix(NA_real_, nrow = n_time_eff, ncol = 2)
    for (t in seq_len(n_time_eff)) {
      phi <- vapply(1:4, function(k) {
        mean(phi_draws[, paste0("phi_unit_t[", i, ",", t, ",", k, "]")])
      }, numeric(1))
      Phi_T <- matrix(c(phi[1], phi[3], phi[2], phi[4]), nrow = 2, byrow = TRUE)
      fitted_mat[t, ] <- as.numeric(y_i[t, , drop = FALSE] %*% Phi_T)
    }
    if (response_shift) {
      fitted_mat <- sweep(fitted_mat, 2, person_means[i, ], "+")
    }
    df <- data.frame(
      unit = rep(unit_ids[i], n_time_eff),
      time = time_values,
      fitted_mat,
      check.names = FALSE
    )
    names(df)[3:4] <- object$vars
    out[[i]] <- df
  }

  do.call(rbind, out)
}


#' Internal: build SEM latent state summaries as a list
#' @noRd
.sem_state_summaries <- function(object) {
  states <- latent_states(object)
  time_values <- .observed_time_values(object$stan_data)
  latent_names <- object$vars

  if (!all(latent_names %in% states$variable)) {
    cli_abort("Latent state summaries are missing expected variable names.")
  }

  out <- vector("list", length(latent_names))
  names(out) <- latent_names
  for (i in seq_along(latent_names)) {
    df <- states[states$variable == latent_names[i], , drop = FALSE]
    df <- df[match(time_values, df$time), , drop = FALSE]
    if (nrow(df) != length(time_values) || anyNA(df$time)) {
      cli_abort(c(
        "{.fun fitted} and {.fun predict} require latent state summaries that are not present in the fitted model.",
        "i" = "Missing latent-state times for {.val {latent_names[i]}}.",
        "i" = "Custom Stan files must preserve the expected parameter and generated-quantity names."
      ))
    }
    out[[i]] <- df
  }

  out
}


#' Internal: observed indicator means from SEM latent states
#' @noRd
.sem_indicator_means <- function(object, state_summaries) {
  indicator_groups <- object$indicators
  latent_names <- names(indicator_groups)
  if (is.null(latent_names) || any(latent_names == "")) {
    latent_names <- object$vars
  }

  time_values <- .observed_time_values(object$stan_data)
  out <- data.frame(time = time_values)
  for (i in seq_along(latent_names)) {
    latent_df <- state_summaries[[latent_names[i]]]
    latent_mean <- latent_df$mean
    for (j in seq_len(object$J)) {
      out[[indicator_groups[[i]][j]]] <- object$lambda[j] * latent_mean
    }
  }

  out
}


#' Internal: SEM indicator prediction intervals
#' @noRd
.sem_indicator_predictions <- function(object, state_summaries, ci_level) {
  indicator_groups <- object$indicators
  latent_names <- names(indicator_groups)
  if (is.null(latent_names) || any(latent_names == "")) {
    latent_names <- object$vars
  }

  z_crit <- stats::qnorm(1 - (1 - ci_level) / 2)
  rows <- vector("list", length(latent_names) * object$J)
  idx <- 1L

  for (i in seq_along(latent_names)) {
    latent_df <- state_summaries[[latent_names[i]]]
    for (j in seq_len(object$J)) {
      mean <- object$lambda[j] * latent_df$mean
      sd <- sqrt((object$lambda[j] * latent_df$sd)^2 + object$sigma_e^2)
      rows[[idx]] <- data.frame(
        time = latent_df$time,
        variable = indicator_groups[[i]][j],
        mean = mean,
        lower = mean - z_crit * sd,
        upper = mean + z_crit * sd
      )
      idx <- idx + 1L
    }
  }

  do.call(rbind, rows)
}


#' Internal: SEM latent-state predictions
#' @noRd
.sem_latent_predictions <- function(object, state_summaries, ci_level) {
  z_crit <- stats::qnorm(1 - (1 - ci_level) / 2)
  rows <- vector("list", length(state_summaries))

  for (i in seq_along(state_summaries)) {
    latent_df <- state_summaries[[i]]
    rows[[i]] <- data.frame(
      time = latent_df$time,
      variable = latent_df$variable,
      mean = latent_df$mean,
      lower = latent_df$mean - z_crit * latent_df$sd,
      upper = latent_df$mean + z_crit * latent_df$sd
    )
  }

  do.call(rbind, rows)
}

#' Internal: fitted values for naive SEM score models.
#' @noRd
.sem_naive_fitted_values <- function(object) {
  y <- object$stan_data$y
  n_time_obs <- object$stan_data$n_time

  summ <- .fit_summary(
    object$fit,
    variables = NULL,
    backend = object$backend,
    required = c("^mu\\[", "^Phi\\["),
    required_type = "pattern",
    context = ".sem_naive_fitted_values()",
    output_type = "parameter group",
    mean
  )
  get_mean <- function(name) {
    row <- which(summ$variable == name)
    if (length(row) != 1L) {
      cli_abort("Naive SEM fitted values require Stan output {.val {name}}.")
    }
    summ$mean[row]
  }

  mu <- c(get_mean("mu[1]"), get_mean("mu[2]"))
  Phi <- matrix(
    c(
      get_mean("Phi[1,1]"), get_mean("Phi[2,1]"),
      get_mean("Phi[1,2]"), get_mean("Phi[2,2]")
    ),
    nrow = 2
  )
  fitted_mat <- matrix(NA_real_, nrow = n_time_obs - 1L, ncol = 2)
  for (t in 2:n_time_obs) {
    fitted_mat[t - 1L, ] <- as.numeric(mu + y[t - 1L, , drop = FALSE] %*% t(Phi))
  }

  out <- data.frame(
    time = .observed_time_values(object$stan_data, drop_first = TRUE),
    fitted_mat
  )
  names(out) <- c("time", object$vars)
  out
}


#' @rdname fitted.dcvar_model_fit
#' @export
fitted.dcvar_sem_fit <- function(object, type = c("link", "response"), ...) {
  type <- match.arg(type)
  if (identical(object$method %||% "indicator", "naive")) {
    return(.sem_naive_fitted_values(object))
  }
  state_summaries <- .sem_state_summaries(object)

  if (identical(type, "link")) {
    out <- data.frame(time = .observed_time_values(object$stan_data))
    for (nm in names(state_summaries)) {
      out[[nm]] <- state_summaries[[nm]]$mean
    }
    return(out)
  }

  .sem_indicator_means(object, state_summaries)
}


#' One-step-ahead predictions from a copula VAR model
#'
#' Returns point predictions and **marginal** prediction intervals by combining
#' the VAR(1) fitted values with the estimated innovation SDs. Intervals are
#' computed per-variable using a normal approximation and do not account for
#' the copula dependence structure between variables. They are plug-in
#' intervals built from posterior-mean parameters: posterior parameter
#' uncertainty is not propagated, so coverage is understated on short series
#' where that uncertainty is non-negligible.
#'
#' `predict()` is implemented for the public fit classes. For multilevel fits,
#' the methods return unit-specific trajectories. For SEM fits, `type = "link"`
#' returns latent states and `type = "response"` returns observed indicator
#' predictions.
#'
#' @param object A fitted model object.
#' @param type Character; `"link"` (default) returns values on the model's
#'   internal scale (standardized if applicable), `"response"` back-transforms
#'   to the original data scale.
#' @param ci_level Prediction interval level (default: 0.95).
#' @param ... Additional arguments (unused).
#'
#' @return A data frame of marginal prediction intervals at the specified
#'   level. Single-level and SEM fits return columns `time`, `variable`,
#'   `mean`, `lower`, `upper`. Multilevel fits additionally include `unit`.
#' @export
predict.dcvar_model_fit <- function(object, type = c("link", "response"),
                                    ci_level = 0.95, ...) {
  type <- match.arg(type)
  .validate_interval_level(ci_level, arg_name = "ci_level")
  margins <- object$margins %||% "normal"
  if (!all(margins == "normal")) {
    cli_abort("Prediction intervals are currently only supported for normal margins, not {.val {margins}}.")
  }

  fit_df <- stats::fitted(object, type = "link")
  sigma_draws <- posterior::as_draws_matrix(.fit_draws(
    object$fit, "sigma_eps", backend = object$backend,
    required = .stan_output_group_pattern("sigma_eps"),
    required_type = "pattern",
    context = "predict.dcvar_model_fit()",
    output_type = "parameter group"
  ))
  sigma_cols <- paste0("sigma_eps[", seq_len(object$stan_data$D), "]")
  sigma_eps <- colMeans(sigma_draws[, sigma_cols, drop = FALSE])

  z_crit <- stats::qnorm(1 - (1 - ci_level) / 2)

  D <- object$stan_data$D
  vars <- object$vars

  rows <- list()
  time_values <- .observed_time_values(object$stan_data, drop_first = TRUE)
  for (d in seq_len(D)) {
    rows[[d]] <- data.frame(
      time = time_values,
      variable = vars[d],
      mean = fit_df[[vars[d]]],
      lower = fit_df[[vars[d]]] - z_crit * sigma_eps[d],
      upper = fit_df[[vars[d]]] + z_crit * sigma_eps[d]
    )
  }

  out <- do.call(rbind, rows)

  if (type == "response" && isTRUE(object$standardized)) {
    Y_means <- attr(object$stan_data, "Y_means")
    Y_sds <- attr(object$stan_data, "Y_sds")
    if (!is.null(Y_means) && !is.null(Y_sds)) {
      for (d in seq_len(D)) {
        idx <- out$variable == vars[d]
        out$mean[idx] <- out$mean[idx] * Y_sds[d] + Y_means[d]
        out$lower[idx] <- out$lower[idx] * Y_sds[d] + Y_means[d]
        out$upper[idx] <- out$upper[idx] * Y_sds[d] + Y_means[d]
      }
    }
  }

  out
}

#' @rdname predict.dcvar_model_fit
#' @export
predict.dcvar_multilevel_fit <- function(object, type = c("link", "response"),
                                         ci_level = 0.95, ...) {
  type <- match.arg(type)
  .validate_interval_level(ci_level, arg_name = "ci_level")
  margins <- object$margins %||% "normal"
  if (!all(margins == "normal")) {
    cli_abort("Prediction intervals are currently only supported for normal multilevel margins, not {.val {margins}}.")
  }

  fit_df <- .multilevel_fitted_values(object, response = identical(type, "response"))
  sigma <- coef(object)$sigma
  sigma <- as.numeric(sigma)
  z_crit <- stats::qnorm(1 - (1 - ci_level) / 2)

  rows <- vector("list", length(object$vars))
  for (d in seq_along(object$vars)) {
    mean <- fit_df[[object$vars[d]]]
    rows[[d]] <- data.frame(
      unit = fit_df$unit,
      time = fit_df$time,
      variable = object$vars[d],
      mean = mean,
      lower = mean - z_crit * sigma[d],
      upper = mean + z_crit * sigma[d],
      check.names = FALSE
    )
  }

  do.call(rbind, rows)
}

#' @rdname predict.dcvar_model_fit
#' @export
predict.dcvar_multilevel_tv_fit <- function(object, type = c("link", "response"),
                                            ci_level = 0.95, ...) {
  type <- match.arg(type)
  .validate_interval_level(ci_level, arg_name = "ci_level")
  margins <- object$margins %||% "normal"
  if (!all(margins == "normal")) {
    cli_abort("Prediction intervals are currently only supported for normal TV multilevel margins, not {.val {margins}}.")
  }

  fit_df <- stats::fitted(object, type = type)
  D <- length(object$vars)
  n_time_eff <- object$stan_data$n_time - 1L
  sigma_mat <- if (isTRUE(object$tv_sigma)) {
    sigma_draws <- posterior::as_draws_matrix(.fit_draws(
      object$fit, "sigma_t", backend = object$backend,
      required = .stan_output_group_pattern("sigma_t"),
      required_type = "pattern",
      context = "predict.dcvar_multilevel_tv_fit()",
      output_type = "generated quantity"
    ))
    out <- matrix(NA_real_, n_time_eff, D)
    for (d in seq_len(D)) {
      cols <- paste0("sigma_t[", seq_len(n_time_eff), ",", d, "]")
      out[, d] <- colMeans(sigma_draws[, cols, drop = FALSE])
    }
    out
  } else {
    sigma_draws <- posterior::as_draws_matrix(.fit_draws(
      object$fit, "sigma_eps", backend = object$backend,
      required = .stan_output_group_pattern("sigma_eps"),
      required_type = "pattern",
      context = "predict.dcvar_multilevel_tv_fit()",
      output_type = "parameter group"
    ))
    sigma_eps <- colMeans(sigma_draws[, paste0("sigma_eps[", seq_len(D), "]"), drop = FALSE])
    matrix(rep(sigma_eps, each = n_time_eff), n_time_eff, D)
  }

  z_crit <- stats::qnorm(1 - (1 - ci_level) / 2)
  rows <- vector("list", length(object$vars))
  for (d in seq_along(object$vars)) {
    mean <- fit_df[[object$vars[d]]]
    sigma <- rep(sigma_mat[, d], times = object$N)
    rows[[d]] <- data.frame(
      unit = fit_df$unit,
      time = fit_df$time,
      variable = object$vars[d],
      mean = mean,
      lower = mean - z_crit * sigma,
      upper = mean + z_crit * sigma,
      check.names = FALSE
    )
  }

  do.call(rbind, rows)
}

#' @rdname predict.dcvar_model_fit
#' @export
predict.dcvar_sem_fit <- function(object, type = c("link", "response"),
                                  ci_level = 0.95, ...) {
  type <- match.arg(type)
  .validate_interval_level(ci_level, arg_name = "ci_level")
  if (identical(object$method %||% "indicator", "naive")) {
    margins <- object$margins %||% "normal"
    if (!all(margins == "normal")) {
      cli_abort("Prediction intervals are currently only supported for normal naive SEM margins, not {.val {margins}}.")
    }
    fit_df <- .sem_naive_fitted_values(object)
    sigma <- as.numeric(coef(object)$sigma)
    z_crit <- stats::qnorm(1 - (1 - ci_level) / 2)
    rows <- vector("list", length(object$vars))
    for (d in seq_along(object$vars)) {
      mean <- fit_df[[object$vars[d]]]
      rows[[d]] <- data.frame(
        time = fit_df$time,
        variable = object$vars[d],
        mean = mean,
        lower = mean - z_crit * sigma[d],
        upper = mean + z_crit * sigma[d]
      )
    }
    return(do.call(rbind, rows))
  }

  state_summaries <- .sem_state_summaries(object)
  if (identical(type, "link")) {
    .sem_latent_predictions(object, state_summaries, ci_level)
  } else {
    .sem_indicator_predictions(object, state_summaries, ci_level)
  }
}


# ============================================================================
# Time-varying fits (dcvar_tv_fit)
# ============================================================================

#' @rdname fitted.dcvar_model_fit
#' @export
fitted.dcvar_tv_fit <- function(object, type = c("link", "response"), ...) {
  type <- match.arg(type)
  if (!isTRUE(object$tv_phi)) {
    # Constant Phi: the inherited one-step-ahead computation is correct.
    return(NextMethod())
  }

  D <- object$stan_data$D
  n_time_obs <- object$stan_data$n_time
  Y <- object$stan_data$Y
  mu_draws <- posterior::as_draws_matrix(.fit_draws(
    object$fit, "mu", backend = object$backend,
    required = .stan_output_group_pattern("mu"),
    required_type = "pattern",
    context = "fitted.dcvar_tv_fit()",
    output_type = "parameter group"
  ))
  phi_draws <- posterior::as_draws_matrix(.fit_draws(
    object$fit, "phi_t", backend = object$backend,
    required = .stan_output_group_pattern("phi_t"),
    required_type = "pattern",
    context = "fitted.dcvar_tv_fit()",
    output_type = "generated quantity"
  ))

  mu_mat <- mu_draws[, paste0("mu[", seq_len(D), "]"), drop = FALSE]

  y_hat <- matrix(NA_real_, n_time_obs - 1L, D)
  for (time_index in 2:n_time_obs) {
    t_eff <- time_index - 1L
    centered_prev <- sweep(mu_mat, 2, Y[time_index - 1L, ], FUN = function(mu_val, y_prev) y_prev - mu_val)
    y_hat_draws <- matrix(NA_real_, nrow(mu_mat), D)

    for (d in seq_len(D)) {
      eta <- mu_mat[, d]
      for (j in seq_len(D)) {
        k <- (d - 1L) * D + j
        eta <- eta + phi_draws[, paste0("phi_t[", t_eff, ",", k, "]")] * centered_prev[, j]
      }
      y_hat_draws[, d] <- eta
    }

    y_hat[t_eff, ] <- colMeans(y_hat_draws)
  }

  if (type == "response" && isTRUE(object$standardized)) {
    Y_means <- attr(object$stan_data, "Y_means")
    Y_sds <- attr(object$stan_data, "Y_sds")
    if (!is.null(Y_means) && !is.null(Y_sds)) {
      for (d in seq_len(D)) {
        y_hat[, d] <- y_hat[, d] * Y_sds[d] + Y_means[d]
      }
    }
  }

  out <- data.frame(time = .observed_time_values(object$stan_data, drop_first = TRUE), y_hat)
  names(out) <- c("time", object$vars)
  out
}


#' @rdname predict.dcvar_model_fit
#' @export
predict.dcvar_tv_fit <- function(object, type = c("link", "response"),
                                 ci_level = 0.95, ...) {
  type <- match.arg(type)
  .validate_interval_level(ci_level, arg_name = "ci_level")
  margins <- object$margins %||% "normal"
  if (!all(margins == "normal")) {
    cli_abort("Prediction intervals are currently only supported for normal margins, not {.val {margins}}.")
  }

  fit_df <- stats::fitted(object, type = "link")
  D <- object$stan_data$D
  n_time_eff <- object$stan_data$n_time - 1L
  vars <- object$vars

  # Per-time plug-in scales: posterior-mean sigma_t when the scales vary,
  # otherwise the constant posterior-mean sigma_eps tiled over time.
  sigma_mat <- if (isTRUE(object$tv_sigma)) {
    summ <- .fit_summary(object$fit, backend = object$backend)
    out <- matrix(NA_real_, n_time_eff, D)
    for (d in seq_len(D)) {
      rows <- match(paste0("sigma_t[", seq_len(n_time_eff), ",", d, "]"), summ$variable)
      if (anyNA(rows)) {
        cli_abort("Predictions require the Stan output {.val sigma_t}, which is missing from the fit.")
      }
      out[, d] <- summ$mean[rows]
    }
    out
  } else {
    sigma_draws <- posterior::as_draws_matrix(.fit_draws(
      object$fit, "sigma_eps", backend = object$backend,
      required = .stan_output_group_pattern("sigma_eps"),
      required_type = "pattern",
      context = "predict.dcvar_tv_fit()",
      output_type = "parameter group"
    ))
    sigma_eps <- colMeans(sigma_draws[, paste0("sigma_eps[", seq_len(D), "]"), drop = FALSE])
    matrix(rep(sigma_eps, each = n_time_eff), n_time_eff, D)
  }

  z_crit <- stats::qnorm(1 - (1 - ci_level) / 2)
  time_values <- .observed_time_values(object$stan_data, drop_first = TRUE)

  rows <- list()
  for (d in seq_len(D)) {
    rows[[d]] <- data.frame(
      time = time_values,
      variable = vars[d],
      mean = fit_df[[vars[d]]],
      lower = fit_df[[vars[d]]] - z_crit * sigma_mat[, d],
      upper = fit_df[[vars[d]]] + z_crit * sigma_mat[, d]
    )
  }
  out <- do.call(rbind, rows)

  if (type == "response" && isTRUE(object$standardized)) {
    Y_means <- attr(object$stan_data, "Y_means")
    Y_sds <- attr(object$stan_data, "Y_sds")
    if (!is.null(Y_means) && !is.null(Y_sds)) {
      for (d in seq_len(D)) {
        idx <- out$variable == vars[d]
        out$mean[idx] <- out$mean[idx] * Y_sds[d] + Y_means[d]
        out$lower[idx] <- out$lower[idx] * Y_sds[d] + Y_means[d]
        out$upper[idx] <- out$upper[idx] * Y_sds[d] + Y_means[d]
      }
    }
  }

  out
}

# ============================================================================
# Parameter Recovery Metrics
# ============================================================================

#' Compute rho trajectory recovery metrics
#'
#' Evaluates how well the estimated rho trajectory recovers the true values.
#'
#' @param rho_true Numeric vector of true rho values (length T-1).
#' @param rho_est Numeric vector of estimated rho (posterior mean).
#' @param rho_lower Numeric vector of lower interval bounds (2.5% quantiles).
#' @param rho_upper Numeric vector of upper interval bounds (97.5% quantiles).
#'
#' @return A named list with:
#'   - `bias`: mean bias
#'   - `relative_bias`: relative bias (%), the mean error normalized by the
#'     mean true value (by the mean absolute true value for mixed-sign
#'     trajectories; `NA` with a warning when all true values are near zero)
#'   - `coverage`: proportion of intervals containing true value
#'   - `interval_width`: mean interval width
#'   - `correlation`: Pearson correlation between true and estimated
#'   - `bias_start`, `bias_end`: bias at first/last time point
#'   - `coverage_start`, `coverage_end`: coverage at endpoints
#' @export
compute_rho_metrics <- function(rho_true, rho_est, rho_lower, rho_upper) {
  .validate_numeric_vector(rho_true, "rho_true")
  .validate_numeric_vector(rho_est, "rho_est")
  .validate_numeric_vector(rho_lower, "rho_lower")
  .validate_numeric_vector(rho_upper, "rho_upper")

  n <- length(rho_true)
  if (length(rho_est) != n || length(rho_lower) != n || length(rho_upper) != n) {
    cli_abort(c(
      "All rho metric vectors must have the same length.",
      "i" = "Expected {.val {n}} values for each of {.arg rho_est}, {.arg rho_lower}, and {.arg rho_upper}."
    ))
  }
  if (any(rho_lower > rho_upper)) {
    cli_abort(c(
      "{.arg rho_lower} must not exceed {.arg rho_upper}.",
      "i" = "Check that every credible interval is ordered lower <= upper."
    ))
  }

  list(
    bias = mean(rho_est - rho_true),
    relative_bias = .relative_bias(rho_est, rho_true),
    coverage = mean(rho_true >= rho_lower & rho_true <= rho_upper),
    interval_width = mean(rho_upper - rho_lower),
    correlation = .safe_cor(rho_est, rho_true),
    bias_start = rho_est[1] - rho_true[1],
    bias_end = rho_est[n] - rho_true[n],
    coverage_start = as.numeric(rho_true[1] >= rho_lower[1] & rho_true[1] <= rho_upper[1]),
    coverage_end = as.numeric(rho_true[n] >= rho_lower[n] & rho_true[n] <= rho_upper[n])
  )
}


#' Compute scalar parameter recovery metrics
#'
#' @param true_value True parameter value.
#' @param est_mean Estimated posterior mean.
#' @param est_lower Lower interval bound (2.5% quantile).
#' @param est_upper Upper interval bound (97.5% quantile).
#'
#' @return A named list with `bias`, `relative_bias`, `covered`,
#'   `interval_width`.
#' @export
compute_param_metrics <- function(true_value, est_mean, est_lower, est_upper) {
  .validate_scalar_numeric(true_value, "true_value")
  .validate_scalar_numeric(est_mean, "est_mean")
  .validate_scalar_numeric(est_lower, "est_lower")
  .validate_scalar_numeric(est_upper, "est_upper")

  if (est_lower > est_upper) {
    cli_abort(c(
      "{.arg est_lower} must not exceed {.arg est_upper}.",
      "i" = "Check that the reported credible interval is ordered lower <= upper."
    ))
  }

  list(
    bias = est_mean - true_value,
    relative_bias = .relative_bias(est_mean, true_value),
    covered = as.numeric(true_value >= est_lower & true_value <= est_upper),
    interval_width = est_upper - est_lower
  )
}


#' Aggregate metrics across simulation replications
#'
#' @param metrics_list A list of metric lists (one per replication), each
#'   containing a `$rho` element as returned by [compute_rho_metrics()].
#'
#' @return A named list with:
#'   - `rho`: data frame of aggregated rho metrics (mean, SD, quantiles) for
#'     every field in the per-replication `rho` metric list
#'   - `n_reps`: number of replications
#' @export
aggregate_metrics <- function(metrics_list) {
  n_reps <- length(metrics_list)
  if (n_reps == 0) {
    cli_abort("{.arg metrics_list} must contain at least one replication.")
  }
  if (!is.list(metrics_list) || !all(vapply(metrics_list, function(x) is.list(x) && !is.null(x$rho), logical(1)))) {
    cli_abort("Each element of {.arg metrics_list} must be a list with a {.val rho} element.")
  }

  expected_metric_names <- c(
    "bias", "relative_bias", "coverage", "interval_width",
    "correlation", "bias_start", "bias_end",
    "coverage_start", "coverage_end"
  )
  metric_names <- names(metrics_list[[1]]$rho)
  if (is.null(metric_names) || any(metric_names == "")) {
    cli_abort("The {.val rho} element must be a named list of scalar metrics.")
  }
  if (!identical(metric_names, expected_metric_names)) {
    cli_abort("The {.val rho} element must contain the full set of compute_rho_metrics() outputs in the documented order.")
  }
  if (!all(vapply(metrics_list, function(x) identical(names(x$rho), expected_metric_names), logical(1)))) {
    cli_abort("All {.val rho} elements must contain the same metric names in the same order.")
  }

  rho_rows <- vapply(metrics_list, function(m) {
    vals <- m$rho[expected_metric_names]
    # NA is a documented value for relative_bias (null conditions) and for
    # correlation (degenerate series); the summaries below use na.rm = TRUE.
    ok <- vapply(
      vals,
      function(x) is.numeric(x) && length(x) == 1L && !is.nan(x) && !is.infinite(x),
      logical(1)
    )
    if (!all(ok)) {
      cli_abort("Each metric in {.val rho} must be a single finite or NA numeric value.")
    }
    unlist(vals, use.names = FALSE)
  }, numeric(length(expected_metric_names)))

  rho_df <- as.data.frame(t(rho_rows))
  names(rho_df) <- expected_metric_names

  summarise_col <- function(x) {
    c(mean = mean(x, na.rm = TRUE),
      sd = sd(x, na.rm = TRUE),
      q2.5 = unname(quantile(x, 0.025, na.rm = TRUE)),
      q97.5 = unname(quantile(x, 0.975, na.rm = TRUE)))
  }

  rho_summ <- as.data.frame(t(sapply(rho_df, summarise_col)))
  rho_summ$metric <- rownames(rho_summ)
  rownames(rho_summ) <- NULL
  rho_summ <- rho_summ[, c("metric", "mean", "sd", "q2.5", "q97.5")]

  list(rho = rho_summ, n_reps = n_reps)
}


#' Internal: compute relative bias (%)
#'
#' Normalized form: mean error relative to the mean true value. Averaging
#' pointwise ratios instead would explode (or flip sign) whenever a single
#' true value is at or near zero -- e.g. a null-dependence rho trajectory --
#' silently dominating any aggregated summaries.
#'
#' When all true values share a sign, the denominator is `mean(true)`, which
#' preserves the conventional sign of relative bias for negative parameters
#' (attenuation toward zero is negative). For mixed-sign trajectories the
#' pointwise convention is ill-defined, so the mean error is normalized by the
#' mean absolute true value instead.
#' @noRd
.relative_bias <- function(estimated, true, epsilon = 1e-10) {
  denom <- mean(abs(true))
  if (denom < epsilon) {
    cli_warn(c(
      "Relative bias is undefined when all true values are (near) zero; returning {.val NA}.",
      "i" = "Use the absolute {.val bias} metric for null conditions."
    ))
    return(NA_real_)
  }
  if (all(true >= 0) || all(true <= 0)) {
    return(mean(estimated - true) / mean(true) * 100)
  }
  mean(estimated - true) / denom * 100
}

#' Internal: validate a numeric vector for metric calculations
#' @noRd
.validate_numeric_vector <- function(x, arg_name) {
  if (!is.numeric(x)) {
    cli_abort("{.arg {arg_name}} must be a numeric vector.")
  }
  if (length(x) == 0) {
    cli_abort("{.arg {arg_name}} must not be empty.")
  }
  if (any(!is.finite(x))) {
    cli_abort("{.arg {arg_name}} must contain only finite values.")
  }
}

#' Internal: validate a single finite numeric value for metric calculations
#' @noRd
.validate_scalar_numeric <- function(x, arg_name) {
  if (!is.numeric(x) || length(x) != 1L || !is.finite(x)) {
    cli_abort("{.arg {arg_name}} must be a single finite numeric value.")
  }
}

test_that("compute_rho_metrics returns all expected fields", {
  n <- 50
  rho_true <- seq(0.7, 0.3, length.out = n)
  rho_est <- rho_true + rnorm(n, 0, 0.01)
  rho_lower <- rho_est - 0.1
  rho_upper <- rho_est + 0.1

  result <- compute_rho_metrics(rho_true, rho_est, rho_lower, rho_upper)

  expect_true(is.list(result))
  expected_names <- c("bias", "relative_bias", "coverage", "interval_width",
                      "correlation", "bias_start", "bias_end",
                      "coverage_start", "coverage_end")
  expect_true(all(expected_names %in% names(result)))
})

test_that("compute_rho_metrics rejects malformed inputs", {
  expect_error(
    compute_rho_metrics(1:3, 1:2, 1:3, 1:3),
    "same length"
  )
  expect_error(
    compute_rho_metrics(c(1, NA_real_), c(1, 2), c(0, 0), c(2, 2)),
    "finite"
  )
  expect_error(
    compute_rho_metrics(c(1, 2), c(1, 2), c(0.8, 0.9), c(0.7, 1.0)),
    "must not exceed"
  )
})

test_that("compute_rho_metrics has 100% coverage when CI is wide", {
  n <- 20
  rho_true <- rep(0.5, n)
  rho_est <- rep(0.5, n)
  rho_lower <- rep(-1, n)
  rho_upper <- rep(1, n)

  result <- compute_rho_metrics(rho_true, rho_est, rho_lower, rho_upper)
  expect_equal(result$coverage, 1)
})

test_that("compute_rho_metrics has 0% coverage when CI misses", {
  n <- 20
  rho_true <- rep(0.5, n)
  rho_est <- rep(-0.5, n)
  rho_lower <- rep(-0.9, n)
  rho_upper <- rep(-0.1, n)

  result <- compute_rho_metrics(rho_true, rho_est, rho_lower, rho_upper)
  expect_equal(result$coverage, 0)
})

test_that("compute_param_metrics works for scalars", {
  result <- compute_param_metrics(
    true_value = 0.3,
    est_mean = 0.32,
    est_lower = 0.25,
    est_upper = 0.39
  )

  expect_equal(result$bias, 0.02)
  expect_equal(result$covered, 1)
  expect_equal(result$interval_width, 0.14)
})

test_that("compute_param_metrics detects non-coverage", {
  result <- compute_param_metrics(
    true_value = 0.5,
    est_mean = 0.1,
    est_lower = 0.05,
    est_upper = 0.15
  )
  expect_equal(result$covered, 0)
})

test_that("compute_param_metrics rejects malformed inputs", {
  expect_error(
    compute_param_metrics(true_value = 1:2, est_mean = 0.1, est_lower = 0, est_upper = 1),
    "single finite numeric"
  )
  expect_error(
    compute_param_metrics(true_value = 1, est_mean = 0.1, est_lower = 0.5, est_upper = 0.1),
    "must not exceed"
  )
})

test_that("aggregate_metrics returns correct structure", {
  rho_true <- seq(0.7, 0.3, length.out = 10)
  metrics_list <- lapply(1:5, function(i) {
    rho_est <- rho_true + rnorm(length(rho_true), 0, 0.01)
    rho_lower <- rho_est - 0.1
    rho_upper <- rho_est + 0.1
    list(rho = compute_rho_metrics(rho_true, rho_est, rho_lower, rho_upper))
  })

  result <- aggregate_metrics(metrics_list)

  expect_equal(result$n_reps, 5)
  expect_true(is.data.frame(result$rho))
  expect_true("metric" %in% names(result$rho))
  expect_true("mean" %in% names(result$rho))
  expect_equal(nrow(result$rho), 9)
  expect_equal(
    result$rho$metric,
    c(
      "bias", "relative_bias", "coverage", "interval_width",
      "correlation", "bias_start", "bias_end",
      "coverage_start", "coverage_end"
    )
  )
})

test_that("aggregate_metrics rejects empty input", {
  expect_error(aggregate_metrics(list()), "at least one replication")
})

test_that("aggregate_metrics rejects malformed rho elements", {
  bad_metrics <- list(
    list(rho = list(
      bias = 0,
      relative_bias = 0,
      coverage = 1,
      interval_width = 0.1,
      correlation = 1,
      bias_start = 0,
      bias_end = 0,
      coverage_start = 1
    ))
  )

  expect_error(
    aggregate_metrics(bad_metrics),
    "full set"
  )
})

test_that(".relative_bias preserves the sign convention and handles edge cases", {
  # all-positive truth: ratio-of-means
  expect_equal(.relative_bias(rep(0.6, 5), rep(0.5, 5)), 20)
  # all-negative truth: attenuation toward zero stays negative (conventional)
  expect_equal(.relative_bias(-0.4, -0.5), -20)
  # mixed-sign truth: magnitude-normalized, no explosion at zero crossings
  rb <- .relative_bias(c(-0.4, 0, 0.6), c(-0.5, 0, 0.5))
  expect_true(is.finite(rb))
  # all-zero truth: NA with a warning
  expect_warning(rb0 <- .relative_bias(c(0.1, -0.1), c(0, 0)), "undefined")
  expect_true(is.na(rb0))
})

test_that("aggregate_metrics tolerates NA relative_bias from null conditions", {
  m <- suppressWarnings(compute_rho_metrics(
    rho_true = rep(0, 10),
    rho_est = rnorm(10, 0, 0.01),
    rho_lower = rep(-0.2, 10),
    rho_upper = rep(0.2, 10)
  ))
  expect_true(is.na(m$relative_bias))
  agg <- aggregate_metrics(list(list(rho = m), list(rho = m)))
  expect_s3_class(agg$rho, "data.frame")
  expect_true(is.nan(agg$rho$mean[agg$rho$metric == "relative_bias"]) ||
                is.na(agg$rho$mean[agg$rho$metric == "relative_bias"]))
})

# ============================================================================
# Margin Type Infrastructure
# ============================================================================

#' Valid margin types
#' @noRd
.valid_margins <- c("normal", "exponential", "skew_normal", "gamma")

#' Skew-normal residual SD as a fraction of omega
#' @noRd
.skew_normal_residual_sd_factor <- function(delta) {
  sqrt(1 - 2 * delta^2 / pi)
}

#' Per-dimension margin family codes passed to the generic mixed Stan model
#'
#' These integer codes label each dimension's marginal family for the generic
#' mixed Stan models (`constant_mixed`, `constant_mixed_clayton`,
#' `dcvar_mixed_ncp`, `hmm_mixed`, `multilevel_mixed`, `multilevel_tv_mixed`,
#' `sem_mixed`, `sem_tv_mixed`, and `sem_naive_mixed`). The order must match the
#' `family[i] == k` dispatch in the Stan code.
#' @noRd
.family_codes <- c(normal = 1L, exponential = 2L, skew_normal = 3L, gamma = 4L)

#' Collapse an all-identical margins vector to a single string
#'
#' Per-dimension margins are accepted as a length-1 (recycled) or length-D
#' character vector. When every entry is identical the specification is
#' equivalent to the scalar form, so it is collapsed back to a single string.
#' This keeps the homogeneous code paths (specialised Stan models, scalar
#' `margins` attribute, all existing extractors) completely unchanged. The
#' result is therefore always either length-1 (homogeneous) or a length-D vector
#' with at least two distinct families (genuinely mixed).
#' @noRd
.normalize_margins_spec <- function(margins) {
  if (length(margins) > 1L && length(unique(margins)) == 1L) {
    margins[[1L]]
  } else {
    margins
  }
}

#' Is this a genuinely mixed (per-variable) margin specification?
#'
#' Returns `TRUE` only when the margins vector contains more than one distinct
#' family, i.e. the fit must route to the generic mixed Stan model.
#' @noRd
.is_mixed_margins <- function(margins) {
  length(margins) > 1L && length(unique(margins)) > 1L
}

#' Does this SEM/multilevel margin spec use the generic mixed engine?
#'
#' Multilevel and SEM have specialised Stan files for homogeneous normal and
#' exponential margins. Homogeneous skew-normal and gamma are served by the
#' generic mixed-margin engines, using the same family code on both dimensions.
#' @noRd
.uses_sem_multilevel_mixed_engine <- function(model_type, margins) {
  if (!model_type %in% c("multilevel", "sem", "sem_naive")) {
    return(FALSE)
  }
  if (.is_mixed_margins(margins)) {
    return(TRUE)
  }

  length(margins) >= 1L &&
    length(unique(margins)) == 1L &&
    margins[[1L]] %in% c("skew_normal", "gamma")
}

#' Margin vector passed to a SEM/multilevel mixed engine
#' @noRd
.sem_multilevel_engine_margins <- function(model_type, margins, D = 2L) {
  if (!.uses_sem_multilevel_mixed_engine(model_type, margins)) {
    return(margins)
  }
  if (.is_mixed_margins(margins)) {
    return(margins)
  }

  rep(margins[[1L]], D)
}

#' Sampled per-dimension margin parameters to monitor for a mixed fit
#'
#' Returns the indexed names of the *sampled* parameter each dimension uses for
#' its family (the union's unused parameters merely sample from their priors and
#' need not be monitored for convergence). Used by the diagnostics machinery.
#' @noRd
.mixed_diagnostic_margin_vars <- function(margins) {
  unlist(lapply(seq_along(margins), function(i) {
    switch(margins[[i]],
      normal = paste0("sigma_eps[", i, "]"),
      exponential = paste0("eta[", i, "]"),
      skew_normal = c(paste0("omega[", i, "]"), paste0("delta[", i, "]")),
      gamma = c(paste0("eta[", i, "]"), paste0("shape_gam[", i, "]"))
    )
  }))
}

#' Reportable per-state, per-dimension scale/shape variables for the switching HMM
#'
#' Like [.mixed_margin_report_vars()] but lifted to the engine's 2D-indexed
#' `[m, d]` layout: maps each margin config `m` (state when margins switch, else
#' the shared config) and dimension to the interpretable scale/shape variable for
#' its family (e.g. `sigma_exp[m, d]` for an exponential dimension).
#' @noRd
.hmm_switching_report_vars <- function(margins_char, Mrg_K) {
  D <- ncol(margins_char)
  groups <- list()
  add <- function(nm, m, d) {
    groups[[nm]] <<- c(groups[[nm]], paste0(nm, "[", m, ",", d, "]"))
  }
  for (m in seq_len(Mrg_K)) {
    for (d in seq_len(D)) {
      switch(margins_char[m, d],
        normal = add("sigma_eps", m, d),
        exponential = add("sigma_exp", m, d),
        skew_normal = {
          add("omega", m, d)
          add("delta", m, d)
        },
        gamma = {
          add("sigma_gam", m, d)
          add("shape_gam", m, d)
        }
      )
    }
  }
  groups
}

#' Sampled per-state, per-dimension margin parameters for the switching HMM
#'
#' The switching engine declares the margin union as `array[Mrg_K] vector[D]`, so
#' the sampled names are 2D-indexed `[m, d]`. For each margin config `m` (state
#' when margins switch, otherwise the single shared config) and dimension `d`,
#' returns the indexed name of the parameter that config/dimension actually uses
#' for its family. `margins_char` is the K x D character family matrix; only the
#' first `Mrg_K` rows are consulted.
#' @noRd
.hmm_switching_diagnostic_margin_vars <- function(margins_char, Mrg_K) {
  D <- ncol(margins_char)
  out <- character()
  for (m in seq_len(Mrg_K)) {
    for (d in seq_len(D)) {
      v <- switch(margins_char[m, d],
        normal = paste0("sigma_eps[", m, ",", d, "]"),
        exponential = paste0("eta[", m, ",", d, "]"),
        skew_normal = c(paste0("omega[", m, ",", d, "]"), paste0("delta[", m, ",", d, "]")),
        gamma = c(paste0("eta[", m, ",", d, "]"), paste0("shape_gam[", m, ",", d, "]"))
      )
      out <- c(out, v)
    }
  }
  out
}

#' Per-dimension scale/shape variables to display in trace plots for a mixed fit
#'
#' Returns the indexed names of the interpretable scale/shape variable each
#' dimension uses (e.g. `sigma_exp[d]` for an exponential dimension). Used by the
#' diagnostic plots.
#' @noRd
.mixed_plot_margin_vars <- function(margins) {
  unlist(lapply(seq_along(margins), function(i) {
    switch(margins[[i]],
      normal = paste0("sigma_eps[", i, "]"),
      exponential = paste0("sigma_exp[", i, "]"),
      skew_normal = c(paste0("omega[", i, "]"), paste0("delta[", i, "]")),
      gamma = c(paste0("sigma_gam[", i, "]"), paste0("shape_gam[", i, "]"))
    )
  }))
}

#' Per-dimension scale/shape variables to report for a mixed fit
#'
#' Maps a mixed margins vector to the indexed Stan variable names that should be
#' reported for each dimension, restricted to the dimensions of each family.
#' For example `c("normal", "exponential")` yields
#' `list(sigma_eps = "sigma_eps[1]", sigma_exp = "sigma_exp[2]")`.
#' @noRd
.mixed_margin_report_vars <- function(margins) {
  out <- list()
  add <- function(name, dims) {
    if (length(dims) > 0L) out[[name]] <<- paste0(name, "[", dims, "]")
  }
  add("sigma_eps", which(margins == "normal"))
  add("sigma_exp", which(margins == "exponential"))
  add("omega", which(margins == "skew_normal"))
  add("delta", which(margins == "skew_normal"))
  add("sigma_gam", which(margins == "gamma"))
  add("shape_gam", which(margins == "gamma"))
  out
}

#' Valid copula families
#' @noRd
.valid_copulas <- c("gaussian", "clayton")

#' Validate copula specification
#'
#' @param copula Character string: one of `"gaussian"` or `"clayton"`.
#' @return Invisible TRUE if valid.
#' @noRd
.validate_copula <- function(copula) {
  if (!is.character(copula) || length(copula) != 1) {
    cli_abort("{.arg copula} must be a single character string.")
  }
  if (!copula %in% .valid_copulas) {
    cli_abort(
      "{.arg copula} must be one of {.val {(.valid_copulas)}}, got {.val {copula}}."
    )
  }

  invisible(TRUE)
}

#' Validate margin family names (length and membership only)
#'
#' Checks that `margins` is a length-1 or length-2 character vector of known
#' families, without requiring `skew_direction`. Used by routing/path helpers
#' (such as [dcvar_stan_path()]) where the marginal scale parameters are
#' irrelevant to file lookup.
#' @noRd
.validate_margin_families <- function(margins) {
  if (!is.character(margins) || length(margins) < 1L || length(margins) > 2L) {
    cli_abort("{.arg margins} must be a character vector of length 1 or 2.")
  }
  invalid <- setdiff(margins, .valid_margins)
  if (length(invalid) > 0L) {
    cli_abort(
      "{.arg margins} must be one of {.val {(.valid_margins)}}, got {.val {invalid}}."
    )
  }
  invisible(TRUE)
}

#' Validate margin specification
#'
#' Accepts either a single margin string (applied to both variables) or a
#' length-2 vector specifying a per-variable (mixed) margin for the bivariate
#' model. `skew_direction` is required whenever *any* dimension uses an
#' exponential or gamma margin.
#'
#' @param margins Character vector of length 1 or 2; each entry one of
#'   "normal", "exponential", "skew_normal", "gamma".
#' @param skew_direction Length-2 integer vector of +1/-1. Required when any
#'   dimension uses an exponential or gamma margin.
#' @return Invisible TRUE if valid.
#' @noRd
.validate_margins <- function(margins, skew_direction = NULL) {
  .validate_margin_families(margins)

  needs_skew <- intersect(margins, c("exponential", "gamma"))
  if (length(needs_skew) > 0L) {
    if (is.null(skew_direction)) {
      cli_abort(
        "{.arg skew_direction} is required for {.val {needs_skew}} margins."
      )
    }
    if (length(skew_direction) != 2 || !all(skew_direction %in% c(-1, 1))) {
      cli_abort(
        "{.arg skew_direction} must be a length-2 vector of +1 or -1."
      )
    }
  }

  invisible(TRUE)
}


#' Validate SEM margin specification
#'
#' SEM supports all four latent innovation margin families. Homogeneous normal
#' and exponential fits use specialised Stan files; homogeneous skew-normal,
#' homogeneous gamma, and per-variable specs use the generic mixed engines.
#'
#' @param margins Character vector of margin families.
#' @param skew_direction Length-2 integer vector of +1/-1 for exponential or
#'   gamma margins.
#' @return Invisible TRUE if valid.
#' @noRd
.validate_sem_margins <- function(margins, skew_direction = NULL) {
  margins <- .normalize_margins_spec(margins)
  .validate_margins(margins, skew_direction)

  invisible(TRUE)
}


#' Get Stan model suffix for a given margin type
#'
#' Maps margin names to Stan file suffixes used in model file naming.
#'
#' @param margins Character: margin type.
#' @return Character: suffix for Stan file (e.g., "_EG", "_SNG", "_GG"),
#'   or empty string for normal margins.
#' @noRd
.margin_stan_suffix <- function(margins) {
  switch(margins,
    normal = "",
    exponential = "_EG",
    skew_normal = "_SNG",
    gamma = "_GG",
    cli_abort("Unknown margin type: {.val {margins}}")
  )
}


#' Get Stan file name for a given model type and margin
#'
#' @param model_type Character: model family key.
#' @param margins Character: margin type.
#' @param copula Character: copula family.
#' @return Character: Stan file name (without path).
#' @noRd
.margin_stan_file <- function(model_type, margins, copula = "gaussian") {
  .validate_copula(copula)
  .validate_margin_families(margins)

  # The unified dynamic engine is a single generic file: per-dimension family
  # codes handle homogeneous and mixed margins alike.
  if (identical(model_type, "dcvar_dynamic")) {
    if (!identical(copula, "gaussian")) {
      cli_abort(c(
        "Time-varying VAR components are currently implemented only for the Gaussian copula.",
        "i" = "Use {.code copula = \"gaussian\"} (the default) with {.arg tv_phi} / {.arg tv_sigma}."
      ))
    }
    return("dcvar_dynamic.stan")
  }

  # The Markov-switching HMM engine is a single generic file: per-state, per-
  # dimension family codes handle homogeneous, mixed, and per-state margins alike.
  if (identical(model_type, "hmm_switching")) {
    if (!identical(copula, "gaussian")) {
      cli_abort(c(
        "State-specific HMM switching is currently implemented only for the Gaussian copula.",
        "i" = "Use {.code copula = \"gaussian\"} (the default)."
      ))
    }
    return("hmm_switching.stan")
  }

  # The public TV path remains the legacy-compatible Stan file, matching the
  # exported prepare_dcvar_data(tv_*) output. The dcvar() wrapper routes bundled
  # TV fits to dcvar_dynamic.stan internally.
  if (identical(model_type, "dcvar_tv")) {
    if (!identical(copula, "gaussian")) {
      cli_abort(c(
        "Time-varying VAR components are currently implemented only for the Gaussian copula.",
        "i" = "Use {.code copula = \"gaussian\"} (the default) with {.arg tv_phi} / {.arg tv_sigma}."
      ))
    }
    return("dcvar_tv_mixed.stan")
  }

  if (model_type %in% c("sem_tv", "multilevel_tv")) {
    if (!identical(copula, "gaussian")) {
      cli_abort(c(
        "Time-varying SEM and multilevel components are currently implemented only for the Gaussian copula.",
        "i" = "Use {.code copula = \"gaussian\"} (the default) with {.arg tv_phi} / {.arg tv_sigma}."
      ))
    }
    return(switch(model_type,
      sem_tv = "sem_tv_mixed.stan",
      multilevel_tv = "multilevel_tv_mixed.stan"
    ))
  }

  if (.is_mixed_margins(margins) ||
      .uses_sem_multilevel_mixed_engine(model_type, margins)) {
    if (identical(copula, "clayton")) {
      if (!identical(model_type, "constant")) {
        cli_abort(c(
          "Mixed margins with the Clayton copula are currently implemented only for the constant model.",
          "i" = "Use {.fun dcvar_constant} with {.code copula = \"clayton\"}."
        ))
      }
      return("constant_mixed_clayton.stan")
    }
    mixed_file <- switch(model_type,
      constant = "constant_mixed.stan",
      dcvar = "dcvar_mixed_ncp.stan",
      hmm = "hmm_mixed.stan",
      multilevel = "multilevel_mixed.stan",
      sem = "sem_mixed.stan",
      sem_naive = "sem_naive_mixed.stan",
      cli_abort(c(
        "Mixed per-variable margins are not implemented for the {.val {model_type}} model.",
        "i" = "Supported: {.val constant}, {.val dcvar}, {.val hmm}, {.val multilevel}, {.val sem}, {.val sem_naive}."
      ))
    )
    return(mixed_file)
  }
  margins <- margins[[1L]]

  if (identical(copula, "clayton")) {
    if (identical(model_type, "constant") && identical(margins, "normal")) {
      return("constant_NCl.stan")
    }
    cli_abort(c(
      "The Clayton copula is currently implemented only for the constant model with normal margins.",
      "i" = "Use {.code model_type = 'constant'}, {.code margins = 'normal'}, and {.code copula = 'clayton'}."
    ))
  }

  base_files <- c(
    constant = "constant_copula_var",
    dcvar = "dcvar_model_ncp",
    hmm = "hmm_copula_model",
    sem = "sem_copula_var"
  )
  if (identical(model_type, "multilevel")) {
    if (margins == "normal") {
      return("multilevel_copula_var.stan")
    }
    if (margins == "exponential") {
      return("multilevel_EG.stan")
    }
    if (margins %in% c("skew_normal", "gamma")) {
      return("multilevel_mixed.stan")
    }
    cli_abort(
      "Unknown multilevel margin family {.val {margins}}."
    )
  }
  if (identical(model_type, "sem")) {
    if (margins == "normal") {
      return("sem_copula_var.stan")
    }
    if (margins == "exponential") {
      return("sem_EG.stan")
    }
    if (margins %in% c("skew_normal", "gamma")) {
      return("sem_mixed.stan")
    }
    cli_abort(
      "Unknown SEM margin family {.val {margins}}."
    )
  }
  if (identical(model_type, "sem_naive")) {
    if (margins == "normal") {
      return("sem_naive_NG.stan")
    }
    if (margins == "exponential") {
      return("sem_naive_EG.stan")
    }
    if (margins %in% c("skew_normal", "gamma")) {
      return("sem_naive_mixed.stan")
    }
    cli_abort(
      "Unknown naive SEM margin family {.val {margins}}."
    )
  }
  if (margins == "normal") {
    paste0(base_files[model_type], ".stan")
  } else {
    suffix <- .margin_stan_suffix(margins)
    base_short <- c(
      constant = "constant",
      dcvar = "dcvar",
      hmm = "hmm"
    )
    # dcvar non-normal Stan files have _ncp suffix (e.g., dcvar_EG_ncp.stan)
    ncp <- if (model_type == "dcvar") "_ncp" else ""
    paste0(base_short[model_type], suffix, ncp, ".stan")
  }
}

test_that(".validate_margins accepts valid margin types", {
  expect_invisible(.validate_margins("normal"))
  expect_invisible(.validate_margins("exponential", c(1, -1)))
  expect_invisible(.validate_margins("skew_normal"))
  expect_invisible(.validate_margins("gamma", c(1, 1)))
})

test_that(".validate_copula accepts and rejects copula families", {
  expect_invisible(.validate_copula("gaussian"))
  expect_invisible(.validate_copula("clayton"))
  expect_error(.validate_copula("frank"), "must be one of")
})

test_that(".validate_margins rejects invalid margin types", {
  expect_error(.validate_margins("invalid"), "must be one of")
  expect_error(.validate_margins("student_t"), "must be one of")
})

test_that(".validate_margins requires skew_direction for exponential", {
  expect_error(
    .validate_margins("exponential"),
    "skew_direction.*required"
  )
})

test_that(".validate_margins requires skew_direction for gamma", {
  expect_error(
    .validate_margins("gamma"),
    "skew_direction.*required"
  )
})

test_that(".validate_margins validates skew_direction length and values", {
  expect_error(
    .validate_margins("exponential", c(1)),
    "length-2"
  )
  expect_error(
    .validate_margins("exponential", c(1, -1, 1)),
    "length-2"
  )
  expect_error(
    .validate_margins("exponential", c(1, 2)),
    "\\+1 or -1"
  )
  expect_error(
    .validate_margins("gamma", c(0, 1)),
    "\\+1 or -1"
  )
})

test_that(".margin_stan_suffix returns correct suffix for each margin type", {
  expect_equal(.margin_stan_suffix("normal"), "")
  expect_equal(.margin_stan_suffix("exponential"), "_EG")
  expect_equal(.margin_stan_suffix("skew_normal"), "_SNG")
  expect_equal(.margin_stan_suffix("gamma"), "_GG")
})

test_that(".margin_stan_file returns correct filename for normal margins", {
  expect_equal(.margin_stan_file("constant", "normal"),
               "constant_copula_var.stan")
  expect_equal(.margin_stan_file("dcvar", "normal"),
               "dcvar_model_ncp.stan")
  expect_equal(.margin_stan_file("hmm", "normal"),
               "hmm_copula_model.stan")
  expect_equal(.margin_stan_file("multilevel", "normal"),
               "multilevel_copula_var.stan")
  expect_equal(.margin_stan_file("sem_naive", "normal"),
               "sem_naive_NG.stan")
})

test_that(".margin_stan_file returns correct filename for non-normal margins", {
  expect_equal(.margin_stan_file("constant", "exponential"),
               "constant_EG.stan")
  expect_equal(.margin_stan_file("dcvar", "exponential"),
               "dcvar_EG_ncp.stan")
  expect_equal(.margin_stan_file("hmm", "exponential"),
               "hmm_EG.stan")
  expect_equal(.margin_stan_file("multilevel", "exponential"),
               "multilevel_EG.stan")
  expect_equal(.margin_stan_file("sem_naive", "exponential"),
               "sem_naive_EG.stan")

  expect_equal(.margin_stan_file("dcvar", "skew_normal"),
               "dcvar_SNG_ncp.stan")
  expect_equal(.margin_stan_file("hmm", "gamma"),
               "hmm_GG.stan")
  expect_equal(.margin_stan_file("constant", "normal", copula = "clayton"),
               "constant_NCl.stan")
})

# ---------------------------------------------------------------------------
# Per-variable (mixed) margins
# ---------------------------------------------------------------------------

test_that(".family_codes maps families to the Stan dispatch integers", {
  expect_equal(unname(.family_codes[c("normal", "exponential", "skew_normal", "gamma")]),
               c(1L, 2L, 3L, 4L))
})

test_that(".normalize_margins_spec collapses identical vectors only", {
  expect_identical(.normalize_margins_spec(c("normal", "normal")), "normal")
  expect_identical(.normalize_margins_spec("normal"), "normal")
  expect_identical(.normalize_margins_spec(c("normal", "exponential")),
                   c("normal", "exponential"))
})

test_that(".is_mixed_margins detects genuinely mixed specs", {
  expect_false(.is_mixed_margins("normal"))
  expect_false(.is_mixed_margins(c("gamma", "gamma")))
  expect_true(.is_mixed_margins(c("normal", "exponential")))
})

test_that(".validate_margins accepts length-2 vectors", {
  expect_invisible(.validate_margins(c("normal", "exponential"), c(1, 1)))
  expect_invisible(.validate_margins(c("normal", "skew_normal")))
  expect_invisible(.validate_margins(c("normal", "normal")))
})

test_that(".validate_margins requires skew_direction when any dim is exp/gamma", {
  expect_error(
    .validate_margins(c("normal", "exponential")),
    "skew_direction.*required"
  )
  expect_error(
    .validate_margins(c("gamma", "skew_normal")),
    "skew_direction.*required"
  )
})

test_that(".validate_margins rejects invalid entries and wrong length", {
  expect_error(.validate_margins(c("normal", "student_t")), "must be one of")
  expect_error(.validate_margins(c("normal", "exponential", "gamma")),
               "length 1 or 2")
})

test_that(".margin_stan_file routes mixed margins to the generic model", {
  expect_equal(.margin_stan_file("constant", c("normal", "exponential")),
               "constant_mixed.stan")
  expect_equal(.margin_stan_file("constant", c("skew_normal", "gamma")),
               "constant_mixed.stan")
  # Identical-family vectors keep routing to the specialised models.
  expect_equal(.margin_stan_file("constant", c("normal", "normal")),
               "constant_copula_var.stan")
  expect_equal(.margin_stan_file("constant", c("gamma", "gamma")),
               "constant_GG.stan")
})

test_that(".margin_stan_file routes mixed margins for every supported model", {
  expect_equal(.margin_stan_file("multilevel", c("normal", "exponential")),
               "multilevel_mixed.stan")
  expect_equal(.margin_stan_file("multilevel", "skew_normal"),
               "multilevel_mixed.stan")
  expect_equal(.margin_stan_file("multilevel", "gamma"),
               "multilevel_mixed.stan")
  expect_equal(.margin_stan_file("sem", c("normal", "exponential")),
               "sem_mixed.stan")
  expect_equal(.margin_stan_file("sem", "skew_normal"),
               "sem_mixed.stan")
  expect_equal(.margin_stan_file("sem", "gamma"),
               "sem_mixed.stan")
  expect_equal(.margin_stan_file("sem_naive", c("normal", "exponential")),
               "sem_naive_mixed.stan")
  expect_equal(.margin_stan_file("sem_naive", "skew_normal"),
               "sem_naive_mixed.stan")
  expect_equal(.margin_stan_file("sem_naive", "gamma"),
               "sem_naive_mixed.stan")
  # Clayton mixed is constant-only; the Gaussian copula covers the rest.
  expect_equal(
    .margin_stan_file("constant", c("normal", "gamma"), copula = "clayton"),
    "constant_mixed_clayton.stan"
  )
  expect_error(
    .margin_stan_file("dcvar", c("normal", "exponential"), copula = "clayton"),
    "Clayton.*constant"
  )
})

test_that(".mixed_margin_report_vars restricts each group to its own dims", {
  specs <- .mixed_margin_report_vars(c("normal", "exponential"))
  expect_equal(specs$sigma_eps, "sigma_eps[1]")
  expect_equal(specs$sigma_exp, "sigma_exp[2]")
  expect_null(specs$omega)
})

test_that("routing helpers validate family names before dispatching", {
  # Two distinct but invalid families must not silently route to the mixed model.
  expect_error(.margin_stan_file("constant", c("not_a_margin", "also_bad")),
               "must be one of")
  expect_error(dcvar_stan_path("constant", margins = c("not_a_margin", "x")),
               "must be one of")
  # The exported path helper still resolves a valid mixed request.
  expect_match(dcvar_stan_path("constant", margins = c("normal", "exponential")),
               "constant_mixed\\.stan$")
})

test_that("homogeneous SEM gamma/skew_normal use mixed-engine Stan data", {
  df <- data.frame(time = 1:30, y1_1 = rnorm(30), y1_2 = rnorm(30),
                   y2_1 = rnorm(30), y2_2 = rnorm(30))
  indicators <- list(latent1 = c("y1_1", "y1_2"), latent2 = c("y2_1", "y2_2"))
  gamma_data <- prepare_sem_data(
    df, indicators = indicators, J = 2,
    lambda = c(0.8, 0.8), sigma_e = 0.5,
    margins = "gamma", skew_direction = c(1, 1)
  )
  skew_data <- prepare_sem_data(
    df, indicators = indicators, J = 2,
    lambda = c(0.8, 0.8), sigma_e = 0.5,
    margins = "skew_normal"
  )

  expect_equal(gamma_data$family, c(4L, 4L))
  expect_equal(gamma_data$skew_direction, c(1, 1))
  expect_equal(attr(gamma_data, "margins"), "gamma")
  expect_equal(skew_data$family, c(3L, 3L))
  expect_equal(skew_data$skew_direction, c(1, 1))
  expect_equal(attr(skew_data, "margins"), "skew_normal")
})

# ============================================================================
# Backend Adapter Layer
#
# Provides a unified interface for rstan and cmdstanr backends.
# rstan is the default (in Imports); cmdstanr is an optional fallback
# (in Suggests). All backend-specific code is isolated here.
# ============================================================================

#' @importFrom rstan sampling stan_model
NULL

.rstan_model_cache <- new.env(parent = emptyenv())


# -- Backend resolution -------------------------------------------------------

#' Resolve which backend to use
#' @param backend Character: `"auto"`, `"rstan"`, or `"cmdstanr"`.
#' @return Resolved backend string.
#' @noRd
.resolve_backend <- function(backend = c("auto", "rstan", "cmdstanr")) {
  backend <- match.arg(backend)
  if (backend == "auto") {
    return("rstan")
  }

  if (backend == "cmdstanr" && !.backend_available("cmdstanr")) {
    cli_abort(c(
      "Backend {.val cmdstanr} requested but not available.",
      "i" = "Install with: {.code install.packages('cmdstanr', repos = c('https://stan-dev.r-universe.dev', getOption('repos')))}",
      "i" = "Then run: {.code cmdstanr::install_cmdstan()}"
    ))
  }

  backend
}

#' Check whether a backend is available
#' @param backend Character: `"rstan"` or `"cmdstanr"`.
#' @return Logical.
#' @noRd
.backend_available <- function(backend) {
  switch(backend,
    rstan = requireNamespace("rstan", quietly = TRUE),
    cmdstanr = {
      requireNamespace("cmdstanr", quietly = TRUE) &&
        !is.null(tryCatch(cmdstanr::cmdstan_path(), error = function(e) NULL))
    },
    FALSE
  )
}


# -- Core resolution ----------------------------------------------------------

#' Resolve a safe default core count
#' @param chains Number of chains requested.
#' @return Positive integer core count.
#' @noRd
.default_cores <- function(chains) {
  cores_opt <- getOption("mc.cores")
  if (is.numeric(cores_opt) && length(cores_opt) == 1L &&
      is.finite(cores_opt) && cores_opt >= 1) {
    cores <- as.integer(cores_opt)
  } else {
    detected <- suppressWarnings(parallel::detectCores(logical = FALSE))
    if (!is.numeric(detected) || length(detected) != 1L ||
        !is.finite(detected) || detected < 1) {
      cores <- 1L
    } else {
      cores <- as.integer(detected)
    }
  }

  limit_cores <- tolower(Sys.getenv("_R_CHECK_LIMIT_CORES_", ""))
  if (limit_cores %in% c("true", "t", "1")) {
    cores <- min(cores, 2L)
  }

  min(cores, as.integer(chains))
}


#' Validate and normalize the requested core count
#' @param cores Requested core count or `NULL`.
#' @param chains Number of chains requested.
#' @return Positive integer core count.
#' @noRd
.normalize_cores <- function(cores, chains) {
  if (is.null(cores)) {
    return(.default_cores(chains))
  }

  if (!is.numeric(cores) || length(cores) != 1L ||
      !is.finite(cores) || cores < 1) {
    cli_abort("{.arg cores} must be a positive integer or {.code NULL}, got {.val {cores}}.")
  }

  min(as.integer(cores), as.integer(chains))
}


# -- Model compilation --------------------------------------------------------

#' Compile a Stan model via the resolved backend
#'
#' @param stan_file Path to the `.stan` file.
#' @param backend Character: `"rstan"` or `"cmdstanr"`.
#' @param cache_dir Writable directory for caching compiled models.
#' @param force_recompile Logical; bypass cache.
#' @param quiet Logical; suppress compiler output.
#' @return A compiled model object (class depends on backend).
#' @noRd
.compile_model_backend <- function(stan_file,
                                   backend,
                                   cache_dir = file.path(tempdir(), "dcvar-cache"),
                                   force_recompile = FALSE,
                                   quiet = FALSE) {
  if (!file.exists(stan_file)) {
    cli_abort("Stan source file {.file {stan_file}} does not exist.")
  }

  if (!dir.exists(cache_dir)) {
    dir.create(cache_dir, recursive = TRUE, showWarnings = FALSE)
  }

  include_path <- .stan_include_paths(
    stan_file,
    system.file("stan", package = "dcvar", mustWork = TRUE)
  )
  stan_hash <- .stan_cache_fingerprint(stan_file, include_path)

  switch(backend,
    rstan = .compile_rstan(stan_file, include_path, cache_dir, stan_hash,
                           force_recompile, quiet),
    cmdstanr = .compile_cmdstanr(stan_file, include_path, cache_dir, stan_hash,
                                 force_recompile, quiet)
  )
}

#' Internal: resolve Stan include paths for bundled and custom models
#' @noRd
.stan_include_paths <- function(stan_file, package_include_path) {
  unique(vapply(
    c(dirname(stan_file), package_include_path),
    normalizePath,
    character(1),
    winslash = "/",
    mustWork = TRUE
  ))
}


#' Internal: collect direct `#include` targets from a Stan file
#' @noRd
.stan_direct_includes <- function(stan_file) {
  lines <- readLines(stan_file, warn = FALSE)
  include_lines <- grepl("^\\s*#include\\s+", lines)
  if (!any(include_lines)) {
    return(character())
  }

  includes <- sub("^\\s*#include\\s+", "", lines[include_lines])
  includes <- sub("\\s*(//.*)?$", "", includes)
  includes <- trimws(includes)
  includes <- gsub('^[\"<]|[\">]$', "", includes)
  includes[nzchar(includes)]
}


#' Internal: resolve a Stan include against the active include search path
#' @noRd
.stan_resolve_include <- function(include_name, current_dir, include_path) {
  if (grepl("^(/|[A-Za-z]:[\\\\/])", include_name)) {
    if (file.exists(include_name)) {
      return(normalizePath(include_name, winslash = "/", mustWork = TRUE))
    }
    return(NA_character_)
  }

  candidates <- c(
    file.path(current_dir, include_name),
    file.path(include_path, include_name)
  )

  for (candidate in candidates) {
    if (file.exists(candidate)) {
      return(normalizePath(candidate, winslash = "/", mustWork = TRUE))
    }
  }

  NA_character_
}


#' Internal: fingerprint a Stan file and all recursively included dependencies
#' @noRd
.stan_cache_fingerprint <- function(stan_file, include_path) {
  include_path <- unique(normalizePath(include_path, winslash = "/", mustWork = TRUE))

  visited <- character()
  fingerprint <- character()

  add_file <- function(file_path) {
    file_path <- normalizePath(file_path, winslash = "/", mustWork = TRUE)
    if (file_path %in% visited) {
      return(invisible(NULL))
    }

    visited <<- c(visited, file_path)
    fingerprint <<- c(fingerprint, unname(tools::md5sum(file_path)))

    for (include_name in .stan_direct_includes(file_path)) {
      resolved <- .stan_resolve_include(include_name, dirname(file_path), include_path)
      if (is.na(resolved)) {
        cli_abort(c(
          "Stan include {.val {include_name}} could not be resolved while computing the compile cache key.",
          "i" = "Checked {.file {dirname(file_path)}} and the configured include paths."
        ))
      }
      add_file(resolved)
    }

    invisible(NULL)
  }

  add_file(stan_file)

  signature_file <- tempfile("dcvar-stan-cache-signature-")
  on.exit(unlink(signature_file), add = TRUE)
  writeLines(
    c(
      paste0("include_paths:", paste(include_path, collapse = "|")),
      fingerprint
    ),
    signature_file,
    useBytes = TRUE
  )

  unname(tools::md5sum(signature_file))
}


#' @noRd
.compile_rstan <- function(stan_file, include_path, cache_dir, stan_hash,
                           force_recompile, quiet) {
  cache_key <- paste0("rstan_", stan_hash)

  if (!force_recompile && exists(cache_key, envir = .rstan_model_cache, inherits = FALSE)) {
    return(get(cache_key, envir = .rstan_model_cache, inherits = FALSE))
  }

  model <- rstan::stan_model(
    file = stan_file,
    isystem = include_path,
    auto_write = FALSE,
    verbose = !quiet
  )

  assign(cache_key, model, envir = .rstan_model_cache)

  model
}


#' @noRd
.compile_cmdstanr <- function(stan_file, include_path, cache_dir, stan_hash,
                              force_recompile, quiet) {
  .check_cmdstanr()

  cache_key <- paste0(stan_hash, "_cmdstanr")
  exe_file <- .compiled_exe_path(cache_dir, cache_key)

  cmdstanr::cmdstan_model(
    stan_file = stan_file,
    exe_file = exe_file,
    include_paths = include_path,
    force_recompile = force_recompile,
    quiet = quiet
  )
}


# -- Sampling -----------------------------------------------------------------

#' Run MCMC sampling via the resolved backend
#'
#' @param compiled_model A compiled model object.
#' @param stan_data Named list of data for Stan.
#' @param backend Character: `"rstan"` or `"cmdstanr"`.
#' @param chains,iter_warmup,iter_sampling,adapt_delta,max_treedepth
#'   Standard sampler arguments.
#' @param seed Random seed (or `NULL`).
#' @param cores Number of parallel chains.
#' @param init Init specification (function or list).
#' @param refresh Print frequency.
#' @param ... Additional backend-specific arguments.
#' @return A fitted model object (class depends on backend).
#' @noRd
.sample_model <- function(compiled_model, stan_data, backend,
                          chains, iter_warmup, iter_sampling,
                          adapt_delta, max_treedepth,
                          seed, cores, init, refresh, ...) {
  init <- .seeded_chain_inits(init, chains, seed)
  switch(backend,
    rstan = .sample_rstan(compiled_model, stan_data,
                          chains, iter_warmup, iter_sampling,
                          adapt_delta, max_treedepth,
                          seed, cores, init, refresh, ...),
    cmdstanr = .sample_cmdstanr(compiled_model, stan_data,
                                chains, iter_warmup, iter_sampling,
                                adapt_delta, max_treedepth,
                                seed, cores, init, refresh, ...)
  )
}


#' Materialize per-chain initial values under a deterministic RNG
#'
#' The default init closures draw starting values with `rnorm()`/`runif()`.
#' Without this step those draws come from the global, unseeded R RNG, so two
#' fits with the same `seed` argument start their chains from different points
#' and `seed` does not actually reproduce the MCMC output. When `seed` is
#' supplied, evaluate the init function once per chain under `set.seed(seed +
#' chain)` on a temporarily swapped RNG state (the caller's RNG stream is left
#' untouched) and hand both backends an explicit list of per-chain inits.
#' @noRd
.seeded_chain_inits <- function(init, chains, seed) {
  if (!is.function(init) || is.null(seed)) {
    return(init)
  }

  has_chain_arg <- "chain_id" %in% names(formals(init))
  had_seed <- exists(".Random.seed", envir = globalenv(), inherits = FALSE)
  old_seed <- if (had_seed) get(".Random.seed", envir = globalenv()) else NULL
  on.exit({
    if (had_seed) {
      assign(".Random.seed", old_seed, envir = globalenv())
    } else if (exists(".Random.seed", envir = globalenv(), inherits = FALSE)) {
      rm(".Random.seed", envir = globalenv())
    }
  }, add = TRUE)

  lapply(seq_len(chains), function(chain) {
    # Derive the per-chain seed in double arithmetic and wrap: the backends
    # accept seeds up to .Machine$integer.max, where `seed + chain` would
    # overflow to NA in integer arithmetic and abort set.seed().
    set.seed((as.numeric(seed) + chain) %% .Machine$integer.max)
    if (has_chain_arg) init(chain_id = chain) else init()
  })
}


#' @noRd
.sample_rstan <- function(compiled_model, stan_data,
                          chains, iter_warmup, iter_sampling,
                          adapt_delta, max_treedepth,
                          seed, cores, init, refresh, ...) {
  if (is.null(seed)) seed <- sample.int(.Machine$integer.max, 1L)

  rstan::sampling(
    object = compiled_model,
    data = stan_data,
    chains = chains,
    warmup = iter_warmup,
    iter = iter_warmup + iter_sampling,
    control = list(adapt_delta = adapt_delta, max_treedepth = max_treedepth),
    seed = seed,
    cores = cores,
    init = init,
    refresh = refresh,
    ...
  )
}


#' @noRd
.sample_cmdstanr <- function(compiled_model, stan_data,
                             chains, iter_warmup, iter_sampling,
                             adapt_delta, max_treedepth,
                             seed, cores, init, refresh, ...) {
  fit <- compiled_model$sample(
    data = stan_data,
    chains = chains,
    iter_warmup = iter_warmup,
    iter_sampling = iter_sampling,
    adapt_delta = adapt_delta,
    max_treedepth = max_treedepth,
    seed = seed,
    parallel_chains = cores,
    init = init,
    refresh = refresh,
    ...
  )

  # Eagerly read the posterior into the R6 object (what $save_object() does):
  # CmdStan writes its CSVs to the session tempdir, so a lazily-loaded fit
  # silently loses all draws after saveRDS() and an R restart.
  invisible(fit$draws())
  try(invisible(fit$sampler_diagnostics()), silent = TRUE)
  try(invisible(fit$init()), silent = TRUE)

  fit
}


# -- Post-fit extraction ------------------------------------------------------

#' Extract posterior draws in a `posterior`-compatible format
#'
#' @param fit Raw backend fit object.
#' @param variables Character vector of parameter names, or `NULL` for all.
#' @param format One of `"draws_array"`, `"draws_matrix"`, `"draws_df"`.
#' @param backend Character: `"rstan"` or `"cmdstanr"`.
#' @param required Character vector of variable names (or patterns) that must be
#'   present in the draws, passed to `.validate_required_stan_outputs()`; `NULL`
#'   skips the check.
#' @param required_type Either `"exact"` (match `required` as literal variable
#'   names) or `"pattern"` (match as regular expressions).
#' @param context,output_type Message-customisation strings used to build a
#'   clear error when a `required` output is missing (the calling method name
#'   and a human label for the expected output, respectively).
#' @return A `posterior::draws_*` object.
#' @noRd
.fit_draws <- function(fit, variables = NULL,
                       format = "draws_array", backend,
                       required = NULL,
                       required_type = c("exact", "pattern"),
                       context = "This method",
                       output_type = "Stan output") {
  required_type <- match.arg(required_type)
  d <- if (backend == "cmdstanr") {
    fit$draws(format = "draws_array")
  } else {
    posterior::as_draws_array(fit)
  }

  .validate_required_stan_outputs(
    d,
    required = required,
    required_type = required_type,
    context = context,
    output_type = output_type
  )

  if (!is.null(variables)) {
    d <- tryCatch(
      posterior::subset_draws(d, variable = variables),
      error = function(e) {
        cli_abort(c(
          "Stan output required by this method is missing from the fitted model.",
          "i" = "Requested variable{?s}: {.val {variables}}.",
          "i" = "Custom Stan files must preserve the expected parameter and generated-quantity names.",
          "x" = conditionMessage(e)
        ))
      }
    )
  }

  switch(format,
    draws_array = d,
    draws_matrix = posterior::as_draws_matrix(d),
    draws_df = posterior::as_draws_df(d),
    d
  )
}


#' Internal: assign stable names to common summary functions
#' @noRd
.normalize_summary_dots <- function(dots) {
  if (length(dots) == 0L) {
    return(dots)
  }

  dot_names <- names(dots)
  if (is.null(dot_names)) {
    dot_names <- rep("", length(dots))
  }

  for (i in seq_along(dots)) {
    if (nzchar(dot_names[[i]])) {
      next
    }

    dot_i <- dots[[i]]
    if (!is.function(dot_i)) {
      next
    }

    if (identical(dot_i, base::mean)) {
      dot_names[[i]] <- "mean"
    } else if (identical(dot_i, stats::sd)) {
      dot_names[[i]] <- "sd"
    } else if (identical(dot_i, posterior::rhat)) {
      dot_names[[i]] <- "rhat"
    } else if (identical(dot_i, posterior::ess_bulk)) {
      dot_names[[i]] <- "ess_bulk"
    } else if (identical(dot_i, posterior::ess_tail)) {
      dot_names[[i]] <- "ess_tail"
    }
  }

  names(dots) <- dot_names
  dots
}


#' Summarise posterior draws
#'
#' When called without extra summary functions in `...`, returns the standard
#' summary (mean, sd, q2.5, q97.5, rhat, ess_bulk, ess_tail).  When `...`
#' contains summary functions (e.g. `mean, sd, ~quantile2(...)`) they are
#' forwarded to [posterior::summarise_draws()].
#'
#' @param fit Raw backend fit object.
#' @param variables Character vector or `NULL`.
#' @param backend Character.
#' @param ... Optional summary functions forwarded to
#'   [posterior::summarise_draws()]. The names `required`, `required_type`,
#'   `context`, and `output_type` are reserved: they are consumed here for
#'   output validation (routed to `.fit_draws()` /
#'   `.validate_required_stan_outputs()`) and are stripped before forwarding, so
#'   they are never passed to [posterior::summarise_draws()].
#' @return A tibble.
#' @noRd
.fit_summary <- function(fit, variables = NULL, backend, ...) {
  dots <- list(...)
  required <- dots$required %||% NULL
  required_type <- dots$required_type %||% c("exact", "pattern")
  context <- dots$context %||% "This method"
  output_type <- dots$output_type %||% "Stan output"

  d <- .fit_draws(
    fit,
    variables = variables,
    format = "draws_array",
    backend = backend,
    required = required,
    required_type = required_type,
    context = context,
    output_type = output_type
  )

  dots$required <- NULL
  dots$required_type <- NULL
  dots$context <- NULL
  dots$output_type <- NULL
  if (length(dots) > 0) {
    dots <- .normalize_summary_dots(dots)
    suppressWarnings(do.call(posterior::summarise_draws, c(list(d), dots)))
  } else {
    suppressWarnings(
      posterior::summarise_draws(
        d,
        mean = mean,
        sd = stats::sd,
        ~posterior::quantile2(.x, probs = c(0.025, 0.975)),
        rhat = posterior::rhat,
        ess_bulk = posterior::ess_bulk,
        ess_tail = posterior::ess_tail
      )
    )
  }
}


#' Extract sampler diagnostics as a 3-D array
#'
#' Returns an array with dimensions `[iteration, chain, parameter]`.
#'
#' @param fit Raw backend fit object.
#' @param backend Character.
#' @return A 3-D numeric array.
#' @noRd
.fit_sampler_diagnostics <- function(fit, backend) {
  if (backend == "cmdstanr") {
    return(fit$sampler_diagnostics())
  }

  if (inherits(fit, "draws")) {
    return(array(
      numeric(0),
      dim = c(0L, 0L, 0L),
      dimnames = list(NULL, NULL, character())
    ))
  }

  # rstan: get_sampler_params returns a list of data frames (one per chain)
  sampler_params <- rstan::get_sampler_params(fit, inc_warmup = FALSE)
  n_chains <- length(sampler_params)
  if (n_chains == 0L) return(array(numeric(0), dim = c(0L, 0L, 0L)))

  n_iter <- nrow(sampler_params[[1L]])
  param_names <- colnames(sampler_params[[1L]])

  result <- array(NA_real_,
                  dim = c(n_iter, n_chains, length(param_names)),
                  dimnames = list(NULL, NULL, param_names))
  for (ch in seq_len(n_chains)) {
    result[, ch, ] <- as.matrix(sampler_params[[ch]])
  }
  result
}


#' Summarise divergences and treedepth issues
#'
#' @param fit Raw backend fit object.
#' @param backend Character.
#' @return A named list with `num_divergent` and `num_max_treedepth`.
#' @noRd
.fit_diagnostic_summary <- function(fit, backend) {
  if (backend == "cmdstanr") {
    return(fit$diagnostic_summary(quiet = TRUE))
  }

  # A bare draws object (e.g. a fit rebuilt without sampler state) carries no
  # sampler diagnostics; report unknown rather than claiming zero issues.
  if (inherits(fit, "draws")) {
    return(list(num_divergent = NA_integer_, num_max_treedepth = NA_integer_))
  }

  # rstan path
  sampler_diags <- .fit_sampler_diagnostics(fit, "rstan")
  diag_names <- dimnames(sampler_diags)[[3L]]

  num_divergent <- 0L
  if ("divergent__" %in% diag_names) {
    num_divergent <- sum(sampler_diags[, , "divergent__"])
  }

  num_max_treedepth <- 0L
  if ("treedepth__" %in% diag_names) {
    # rstan stores max_treedepth in the stan_args slot. as.numeric(NULL) is
    # numeric(0), which slips past both the tryCatch and is.na(), so guard on
    # length too (e.g. stanfits rebuilt from CSVs store no control list).
    max_td <- tryCatch(
      as.numeric(fit@stan_args[[1L]]$control$max_treedepth),
      error = function(e) 10
    )
    if (length(max_td) != 1L || is.na(max_td)) max_td <- 10
    num_max_treedepth <- sum(sampler_diags[, , "treedepth__"] >= max_td)
  }

  list(
    num_divergent = as.integer(num_divergent),
    num_max_treedepth = as.integer(num_max_treedepth)
  )
}

test_that("dcvar_stan_path returns valid file paths", {
  for (m in c("dcvar", "dcvar_dynamic", "hmm", "constant", "multilevel", "sem")) {
    path <- dcvar_stan_path(m)
    expect_true(file.exists(path), info = paste("model:", m))
    expect_true(grepl("\\.stan$", path), info = paste("model:", m))
  }
  expect_match(basename(dcvar_stan_path("dcvar_dynamic")), "^dcvar_dynamic\\.stan$")

  sem_exp <- dcvar_stan_path("sem", margins = "exponential")
  expect_true(file.exists(sem_exp))
  expect_match(basename(sem_exp), "^sem_EG\\.stan$")

  clayton_constant <- dcvar_stan_path("constant", copula = "clayton")
  expect_true(file.exists(clayton_constant))
  expect_match(basename(clayton_constant), "^constant_NCl\\.stan$")

  ml_exp <- dcvar_stan_path("multilevel", margins = "exponential")
  expect_true(file.exists(ml_exp))
  expect_match(basename(ml_exp), "^multilevel_EG\\.stan$")

  sem_naive <- dcvar_stan_path("sem_naive")
  expect_true(file.exists(sem_naive))
  expect_match(basename(sem_naive), "^sem_naive_NG\\.stan$")
})

test_that("dcvar_stan_path rejects invalid model names", {
  expect_error(dcvar_stan_path("invalid"))
})

test_that(".resolve_backend() resolves auto to rstan", {
  expect_equal(.resolve_backend("auto"), "rstan")
})

test_that(".validate_sampling_args requires scalar finite integer counts", {
  expect_invisible(.validate_sampling_args(1, 1, 1, 0.9, 10))

  expect_error(.validate_sampling_args(c(1, 2), 1, 1, 0.9, 10), "chains")
  expect_error(.validate_sampling_args(1.5, 1, 1, 0.9, 10), "chains")
  expect_error(.validate_sampling_args(1, NA_real_, 1, 0.9, 10), "iter_warmup")
  expect_error(.validate_sampling_args(1, 1, Inf, 0.9, 10), "iter_sampling")
  expect_error(.validate_sampling_args(1, 1, 1, c(0.8, 0.9), 10), "adapt_delta")
  expect_error(.validate_sampling_args(1, 1, 1, NaN, 10), "adapt_delta")
  expect_error(.validate_sampling_args(1, 1, 1, 0.9, 10.5), "max_treedepth")
})

test_that(".compiled_exe_path uses the correct platform suffix", {
  path <- .compiled_exe_path(tempdir(), "dcvar-model")

  if (.Platform$OS.type == "windows") {
    expect_match(path, "\\.exe$")
  } else {
    expect_false(grepl("\\.exe$", path, ignore.case = TRUE))
  }
})

test_that(".compile_model_backend() rejects missing Stan source files", {
  missing_file <- file.path(tempdir(), "does-not-exist.stan")
  expect_error(
    .compile_model_backend(missing_file, backend = "rstan"),
    "does not exist"
  )
})

test_that(".stan_include_paths() keeps custom and bundled include roots", {
  custom_dir <- tempfile("custom-stan-dir")
  dir.create(custom_dir)
  custom_file <- file.path(custom_dir, "model.stan")
  package_dir <- system.file("stan", package = "dcvar")
  custom_dir_norm <- normalizePath(custom_dir, winslash = "/", mustWork = TRUE)
  package_dir_norm <- normalizePath(package_dir, winslash = "/", mustWork = TRUE)

  include_paths <- .stan_include_paths(custom_file, package_dir)

  expect_equal(include_paths[[1]], custom_dir_norm)
  expect_true(package_dir_norm %in% include_paths)
  expect_equal(length(include_paths), length(unique(include_paths)))
})

test_that(".stan_cache_fingerprint changes when recursively included files change", {
  root <- tempfile("stan-cache-root")
  dir.create(file.path(root, "functions"), recursive = TRUE)

  model_file <- file.path(root, "model.stan")
  shared_file <- file.path(root, "functions", "shared.stan")
  nested_file <- file.path(root, "functions", "nested.stan")

  writeLines("#include functions/shared.stan", model_file)
  writeLines("#include functions/nested.stan", shared_file)
  writeLines("functions { real nested() { return 1; } }", nested_file)

  on.exit(unlink(root, recursive = TRUE), add = TRUE)

  fingerprint_1 <- .stan_cache_fingerprint(model_file, c(root, system.file("stan", package = "dcvar")))
  writeLines("functions { real nested() { return 2; } }", nested_file)
  fingerprint_2 <- .stan_cache_fingerprint(model_file, c(root, system.file("stan", package = "dcvar")))

  expect_false(identical(fingerprint_1, fingerprint_2))
})

test_that(".stan_cache_fingerprint respects include search path order", {
  model_root <- tempfile("stan-cache-model")
  extra_root <- tempfile("stan-cache-extra")
  package_root <- tempfile("stan-cache-package")

  dir.create(model_root)
  dir.create(file.path(extra_root, "functions"), recursive = TRUE)
  dir.create(file.path(package_root, "functions"), recursive = TRUE)

  model_file <- file.path(model_root, "model.stan")
  writeLines("#include functions/shared.stan", model_file)
  writeLines("functions { real shared() { return 1; } }", file.path(extra_root, "functions", "shared.stan"))
  writeLines("functions { real shared() { return 2; } }", file.path(package_root, "functions", "shared.stan"))

  on.exit(unlink(c(model_root, extra_root, package_root), recursive = TRUE), add = TRUE)

  fingerprint_1 <- .stan_cache_fingerprint(model_file, c(extra_root, package_root))
  fingerprint_2 <- .stan_cache_fingerprint(model_file, c(package_root, extra_root))

  expect_false(identical(fingerprint_1, fingerprint_2))
})

test_that(".fit_summary() assigns stable names to unnamed summary functions", {
  draws <- posterior::as_draws_array(
    array(
      rnorm(40),
      dim = c(10, 1, 4),
      dimnames = list(NULL, NULL, c("phi_unit[1,1]", "phi_unit[1,2]", "phi_unit[1,3]", "phi_unit[1,4]"))
    )
  )

  summ <- .fit_summary(
    draws,
    variables = NULL,
    backend = "rstan",
    required = paste0("phi_unit[1,", 1:4, "]"),
    required_type = "exact",
    context = "test",
    output_type = "parameter",
    mean, sd,
    ~posterior::quantile2(.x, probs = c(0.025, 0.975))
  )

  expect_true(all(c("variable", "mean", "sd", "q2.5", "q97.5") %in% names(summ)))
})

test_that("all bundled top-level Stan files translate successfully", {
  skip_if_no_cmdstanr_toolchain()

  stanc <- file.path(cmdstanr::cmdstan_path(), "bin", "stanc")
  stan_dir <- system.file("stan", package = "dcvar")
  stan_files <- list.files(stan_dir, pattern = "\\.stan$", full.names = TRUE, recursive = TRUE)
  stan_files <- stan_files[!grepl("/functions/", stan_files, fixed = TRUE)]

  for (stan_file in stan_files) {
    output_file <- tempfile(
      pattern = paste0(tools::file_path_sans_ext(basename(stan_file)), "_"),
      fileext = ".hpp"
    )
    on.exit(unlink(output_file), add = TRUE)

    result <- system2(
      stanc,
      c("--include-paths", stan_dir, "--output", output_file, normalizePath(stan_file)),
      stdout = TRUE,
      stderr = TRUE
    )
    status <- attr(result, "status")
    if (is.null(status)) status <- 0L
    expect_equal(status, 0L, info = paste(stan_file, paste(result, collapse = "\n")))
    expect_true(file.exists(output_file), info = paste("missing output for", stan_file))
  }
})

# Regression test: the seed argument must reproduce the full MCMC output,
# including the default chain initial values (which previously came from the
# unseeded global R RNG).

test_that("identical seeds reproduce identical draws", {
  skip_if_no_rstan()

  sim <- simulate_dcvar(n_time = 25, rho_trajectory = rho_constant(25, 0.4), seed = 9)
  fit_args <- list(
    sim$Y_df,
    vars = c("y1", "y2"),
    chains = 2,
    iter_warmup = 50,
    iter_sampling = 50,
    refresh = 0,
    seed = 123
  )
  fit1 <- suppressWarnings(do.call(dcvar_constant, fit_args))
  fit2 <- suppressWarnings(do.call(dcvar_constant, fit_args))

  d1 <- posterior::as_draws_matrix(draws(fit1))
  d2 <- posterior::as_draws_matrix(draws(fit2))
  expect_equal(unclass(d1), unclass(d2))
})

test_that("seeded default inits do not disturb the caller's RNG stream", {
  init <- function() list(x = rnorm(1))
  set.seed(77)
  expected <- rnorm(3)

  set.seed(77)
  inits <- .seeded_chain_inits(init, chains = 2, seed = 5)
  expect_length(inits, 2)
  expect_false(identical(inits[[1]]$x, inits[[2]]$x))
  expect_equal(rnorm(3), expected)

  # Same seed -> same inits
  inits2 <- .seeded_chain_inits(init, chains = 2, seed = 5)
  expect_identical(inits, inits2)
})

#' Fit an experimental multilevel copula VAR(1) model
#'
#' Fits a hierarchical copula VAR(1) model with unit-specific VAR coefficients
#' (random effects) and a global copula correlation. Uses non-centered
#' parameterization for the random Phi coefficients.
#'
#' @param data A data frame in long (panel) format with columns for unit ID,
#'   time, and two outcome variables.
#' @param vars Character vector of two variable names to model.
#' @param id_var Name of the unit/person ID column (default: `"id"`).
#' @param time_var Name of the time column (default: `"time"`).
#' @param center Logical; whether to person-mean center the data
#'   (default: `TRUE`). The bundled multilevel Stan model requires
#'   `center = TRUE`; set `center = FALSE` only with a custom
#'   `stan_file` that includes intercept terms.
#' @param margins Marginal distribution specification. A single string applies
#'   the same family to both variables. A length-2 character vector gives a
#'   per-variable (mixed) margin (for example `c("normal", "gamma")`). Normal,
#'   exponential, skew-normal, and gamma margins are supported; homogeneous
#'   skew-normal/gamma and per-variable specs route through the generic
#'   mixed-margin Stan model under the Gaussian copula.
#' @param skew_direction Integer vector of length 2 of `1`/`-1`. Required
#'   whenever any dimension uses an `"exponential"` or `"gamma"` margin; only
#'   those dimensions consult it.
#' @param prior_phi_bar_sd Prior SD for population-mean VAR coefficients.
#' @param prior_tau_phi_scale Prior scale for half-t(3) on tau_phi.
#' @param prior_sigma_sd Prior SD for the innovation scales. For normal
#'   margins this is the SD of a half-normal prior on `sigma`; for
#'   exponential/gamma margins it is the SD of the lognormal prior on the
#'   marginal scale.
#' @param prior_rho_sd Prior SD for normal on rho.
#' @param tv_phi Selects which population VAR(1) coefficients carry a shared
#'   time-varying drift around the unit-specific baselines. Either a logical
#'   scalar or a character selector as in [dcvar()].
#' @param tv_sigma Logical; if `TRUE`, residual scales evolve as shared
#'   log-scale random walks across time.
#' @param prior_tau_phi_rate Prior mean for the time-drift Phi random-walk
#'   innovation SDs.
#' @param prior_tau_sigma_rate Prior mean for log-scale random-walk innovation
#'   SDs.
#' @param tv_sigma_k Soft-barrier sharpness for time-varying exponential/gamma
#'   scales.
#' @param chains Number of MCMC chains.
#' @param iter_warmup Warmup iterations per chain.
#' @param iter_sampling Sampling iterations per chain.
#' @param adapt_delta Target acceptance rate.
#' @param max_treedepth Maximum tree depth.
#' @param seed Random seed. When supplied, it seeds both the Stan sampler and
#'   the default per-chain initial values, so repeated fits are reproducible.
#' @param cores Number of parallel chains.
#' @param refresh How often to print progress.
#' @param init Custom init function or `NULL`.
#' @param stan_file Custom Stan file path or `NULL`.
#' @param backend Character: `"auto"` (default, uses rstan), `"rstan"`, or
#'   `"cmdstanr"`. Can also be set globally via
#'   `options(dcvar.backend = "cmdstanr")`.
#' @param ... Additional backend-specific sampling arguments.
#'
#' @return A `dcvar_multilevel_fit` object.
#'
#' @details **Experimental extension.** This multilevel variant supports
#'   `fitted()` and `predict()`. PSIS-LOO is available for all multilevel
#'   fits; the stored `log_lik` values are conditional one-step-ahead
#'   densities given the unit-level random effects. PIT diagnostics are not yet implemented.
#'
#'   `adapt_delta` defaults to 0.90 and `max_treedepth` to 14 because the
#'   hierarchical structure with random effects benefits from deeper trees but
#'   does not require aggressive step-size adaptation.
#'
#' @note The bundled multilevel Stan program is defined for person-mean
#'   centered data and omits intercept terms. With the bundled model,
#'   `center = FALSE` is therefore not supported.
#'
#' @seealso [random_effects()] for extracting unit-specific coefficients,
#'   [simulate_dcvar_multilevel()] for data generation.
#' @export
dcvar_multilevel <- function(data, vars,
                             id_var = "id",
                             time_var = "time",
                             center = TRUE,
                             margins = "normal",
                             skew_direction = NULL,
                             prior_phi_bar_sd = 0.5,
                             prior_tau_phi_scale = 0.2,
                             prior_sigma_sd = 1,
                             prior_rho_sd = 0.5,
                             tv_phi = FALSE,
                             tv_sigma = FALSE,
                             prior_tau_phi_rate = 0.025,
                             prior_tau_sigma_rate = 0.05,
                             tv_sigma_k = 8,
                             chains = 4,
                             iter_warmup = 2000,
                             iter_sampling = 4000,
                             adapt_delta = 0.90,
                             max_treedepth = 14,
                             seed = NULL,
                             cores = NULL,
                             refresh = 500,
                             init = NULL,
                             stan_file = NULL,
                             backend = getOption("dcvar.backend", "auto"),
                             ...) {
  margins <- .normalize_margins_spec(margins)
  .validate_margins(margins, skew_direction)
  phi_mask <- .resolve_phi_tv_mask(tv_phi)
  any_phi <- sum(phi_mask) > 0L
  .prep_validate_scalar_logical(tv_sigma, "tv_sigma")
  is_tv <- any_phi || tv_sigma

  if (!any_phi && !isTRUE(all.equal(prior_tau_phi_rate, 0.025))) {
    cli_warn("{.arg prior_tau_phi_rate} is ignored when no VAR coefficient is time-varying.")
  }
  if (!tv_sigma && !isTRUE(all.equal(prior_tau_sigma_rate, 0.05))) {
    cli_warn("{.arg prior_tau_sigma_rate} is ignored when {.code tv_sigma = FALSE}.")
  }

  bundled_stan <- dcvar_stan_path(if (is_tv) "multilevel_tv" else "multilevel", margins = margins)
  uses_bundled_stan <- is.null(stan_file) || identical(
    normalizePath(stan_file, winslash = "/", mustWork = TRUE),
    normalizePath(bundled_stan, winslash = "/", mustWork = TRUE)
  )

  if (!isTRUE(center) && uses_bundled_stan) {
    cli_abort(c(
      "{.arg center = FALSE} is not supported by the bundled multilevel model.",
      "i" = "The bundled Stan program assumes person-mean centered data and omits intercept terms.",
      "i" = "Use {.code center = TRUE} or supply a custom {.arg stan_file} that implements intercepts."
    ))
  }

  backend <- .resolve_backend(backend)
  .validate_sampling_args(chains, iter_warmup, iter_sampling,
                          adapt_delta, max_treedepth)

  # Prepare panel data
  stan_data <- prepare_multilevel_data(
    data = data,
    vars = vars,
    id_var = id_var,
    time_var = time_var,
    center = center,
    prior_phi_bar_sd = prior_phi_bar_sd,
    prior_tau_phi_scale = prior_tau_phi_scale,
    prior_sigma_sd = prior_sigma_sd,
    prior_rho_sd = prior_rho_sd,
    tv_phi = phi_mask,
    tv_sigma = tv_sigma,
    prior_tau_phi_rate = prior_tau_phi_rate,
    prior_tau_sigma_rate = prior_tau_sigma_rate,
    tv_sigma_k = tv_sigma_k,
    margins = margins,
    skew_direction = skew_direction
  )

  N <- stan_data$N
  n_time_obs <- stan_data$n_time

  margins_label <- if (all(margins == "normal")) "" else paste0(" [", paste(margins, collapse = ", "), "]")
  model_label <- if (is_tv) "TV multilevel" else "multilevel"
  cli_inform("Fitting {model_label} copula VAR model{margins_label} (N = {N}, n_time = {n_time_obs})...")

  # Compile model
  model <- .compile_model(if (is_tv) "multilevel_tv" else "multilevel",
                          margins = margins, stan_file = stan_file, backend = backend)

  # Default init
  if (is.null(init)) {
    init <- if (is_tv) {
      function() .init_multilevel_tv_params(2, N, n_time_obs, margins = margins,
                                            tv_phi = phi_mask,
                                            tv_sigma = tv_sigma)
    } else {
      function() .init_multilevel_params(2, N, margins = margins)
    }
  }

  cores <- .normalize_cores(cores, chains)

  fit <- .sample_model(
    compiled_model = model,
    stan_data = stan_data,
    backend = backend,
    chains = chains,
    iter_warmup = iter_warmup,
    iter_sampling = iter_sampling,
    adapt_delta = adapt_delta,
    max_treedepth = max_treedepth,
    seed = seed,
    cores = cores,
    init = init,
    refresh = refresh,
    ...
  )

  priors <- c(
    list(
      phi_bar_sd = prior_phi_bar_sd,
      tau_phi_scale = prior_tau_phi_scale,
      sigma_sd = prior_sigma_sd,
      rho_sd = prior_rho_sd
    ),
    if (any_phi) list(tau_phi_rate = prior_tau_phi_rate),
    if (tv_sigma) list(tau_sigma_rate = prior_tau_sigma_rate)
  )
  meta <- list(
    chains = chains,
    iter_warmup = iter_warmup,
    iter_sampling = iter_sampling,
    adapt_delta = adapt_delta,
    max_treedepth = max_treedepth,
    seed = seed
  )

  common_args <- list(
    fit = fit,
    stan_data = stan_data,
    N = N,
    vars = vars,
    centered = center,
    person_means = attr(stan_data, "person_means"),
    margins = margins,
    skew_direction = attr(stan_data, "skew_direction"),
    backend = backend,
    priors = priors,
    meta = meta
  )
  out <- if (is_tv) {
    do.call(new_dcvar_multilevel_tv_fit, c(common_args, list(
      tv_phi = any_phi,
      phi_tv_mask = phi_mask,
      tv_sigma = tv_sigma
    )))
  } else {
    do.call(new_dcvar_multilevel_fit, common_args)
  }

  .report_sampling_outcome(out$fit, if (is_tv) "TV multilevel copula VAR" else "Multilevel copula VAR",
                           chains = chains, backend = backend, object = out)
  out
}

# ============================================================================
# MCMC Diagnostics
# ============================================================================

#' Extract MCMC diagnostics
#'
#' Returns a summary of sampling diagnostics including divergences,
#' tree depth warnings, Rhat, and effective sample size.
#' The convergence headline is computed from sampled parameters only and
#' excludes generated quantities and deterministic transformed outputs.
#'
#' @param object A fitted model object.
#' @param ... Additional arguments (unused).
#'
#' @return A named list with:
#'   - `n_divergent`: total number of divergent transitions
#'   - `n_max_treedepth`: transitions hitting max tree depth
#'   - `max_rhat`: worst (highest) Rhat across sampled parameters
#'   - `min_ess_bulk`: smallest bulk ESS among sampled parameters
#'   - `min_ess_tail`: smallest tail ESS among sampled parameters
#'   - `mean_accept_prob`: mean acceptance probability
#' @export
dcvar_diagnostics <- function(object, ...) {
  UseMethod("dcvar_diagnostics")
}

#' @rdname dcvar_diagnostics
#' @export
dcvar_diagnostics.default <- function(object, ...) {
  cli_abort("{.fun dcvar_diagnostics} is not defined for objects of class {.cls {class(object)[[1]]}}.")
}

#' Internal: validate model dimensions used by diagnostics
#' @noRd
.diagnostic_positive_int <- function(value, name, location) {
  if (!is.numeric(value) || length(value) != 1L || !is.finite(value) ||
      value < 1 || value != as.integer(value)) {
    cli_abort(
      "{.fun dcvar_diagnostics} requires {.field {name}} to be a positive integer in {.field {location}}."
    )
  }

  as.integer(value)
}

#' Internal: build the exact sampled-parameter set for diagnostics
#' @noRd
.diagnostic_parameter_variables <- function(object) {
  margins <- object$margins %||% "normal"
  model <- object$model

  if (identical(model, "multilevel_tv")) {
    N <- .diagnostic_positive_int(object$N, "N", "object")
    n_time_obs <- .diagnostic_positive_int(object$stan_data$n_time, "n_time", "stan_data")
    n_eff <- n_time_obs - 1L
    margin_vars <- .mixed_diagnostic_margin_vars(rep(margins, length.out = 2L))
    vars <- c(
      paste0("phi_bar[", seq_len(4), "]"),
      paste0("tau_phi[", seq_len(4), "]"),
      paste0(
        "z_phi[",
        rep(seq_len(N), each = 4),
        ",",
        rep(seq_len(4), times = N),
        "]"
      ),
      margin_vars,
      "rho"
    )
    if (isTRUE(object$tv_phi)) {
      n_phi_tv <- sum(object$phi_tv_mask %||% .resolve_phi_tv_mask(TRUE))
      vars <- c(
        vars,
        paste0("tau_phi_tv[", seq_len(n_phi_tv), "]"),
        paste0(
          "phi_raw[",
          rep(seq_len(n_eff), each = n_phi_tv), ",", rep(seq_len(n_phi_tv), times = n_eff),
          "]"
        )
      )
    }
    if (isTRUE(object$tv_sigma)) {
      vars <- c(
        vars,
        paste0("tau_sigma[", seq_len(2), "]"),
        paste0(
          "sigma_raw[",
          rep(seq_len(n_eff), each = 2), ",", rep(seq_len(2), times = n_eff),
          "]"
        )
      )
    }
    return(vars)
  }

  if (identical(model, "multilevel")) {
    N <- .diagnostic_positive_int(object$N, "N", "object")
    margin_vars <- if (.uses_sem_multilevel_mixed_engine("multilevel", margins)) {
      .mixed_diagnostic_margin_vars(
        .sem_multilevel_engine_margins("multilevel", margins, 2L)
      )
    } else if (identical(margins, "exponential")) {
      paste0("eta[", seq_len(2), "]")
    } else {
      paste0("sigma[", seq_len(2), "]")
    }
    return(c(
      paste0("phi_bar[", seq_len(4), "]"),
      paste0("tau_phi[", seq_len(4), "]"),
      paste0(
        "z_phi[",
        rep(seq_len(N), each = 4),
        ",",
        rep(seq_len(4), times = N),
        "]"
      ),
      margin_vars,
      "rho"
    ))
  }

  if (identical(model, "sem_tv")) {
    n_time_obs <- .diagnostic_positive_int(object$stan_data$n_time, "n_time", "stan_data")
    n_eff <- n_time_obs - 1L
    margin_vars <- .mixed_diagnostic_margin_vars(rep(margins, length.out = 2L))
    vars <- c(
      "mu[1]", "mu[2]",
      "Phi[1,1]", "Phi[1,2]", "Phi[2,1]", "Phi[2,2]",
      margin_vars,
      "z_rho_init",
      "sigma_omega",
      paste0("omega_raw[", seq_len(n_eff), "]"),
      paste0(
        "zeta[",
        rep(seq_len(n_time_obs), each = 2),
        ",",
        rep(seq_len(2), times = n_time_obs),
        "]"
      )
    )
    if (isTRUE(object$tv_phi)) {
      n_phi_tv <- sum(object$phi_tv_mask %||% .resolve_phi_tv_mask(TRUE))
      vars <- c(
        vars,
        paste0("tau_phi[", seq_len(n_phi_tv), "]"),
        paste0(
          "phi_raw[",
          rep(seq_len(n_eff), each = n_phi_tv), ",", rep(seq_len(n_phi_tv), times = n_eff),
          "]"
        )
      )
    }
    if (isTRUE(object$tv_sigma)) {
      vars <- c(
        vars,
        paste0("tau_sigma[", seq_len(2), "]"),
        paste0(
          "sigma_raw[",
          rep(seq_len(n_eff), each = 2), ",", rep(seq_len(2), times = n_eff),
          "]"
        )
      )
    }
    return(vars)
  }

  if (identical(model, "sem")) {
    n_time_obs <- .diagnostic_positive_int(object$stan_data$n_time, "n_time", "stan_data")
    method <- object$method %||% "indicator"
    margin_vars <- if (.uses_sem_multilevel_mixed_engine("sem", margins)) {
      .mixed_diagnostic_margin_vars(
        .sem_multilevel_engine_margins("sem", margins, 2L)
      )
    } else if (identical(margins, "exponential")) {
      paste0("eta[", seq_len(2), "]")
    } else {
      paste0("sigma[", seq_len(2), "]")
    }
    vars <- c(
      "mu[1]", "mu[2]",
      "phi11", "phi12", "phi21", "phi22",
      margin_vars,
      "rho_raw"
    )
    if (identical(method, "naive")) {
      return(vars)
    }
    return(c(
      vars,
      paste0(
        "zeta[",
        rep(seq_len(n_time_obs), each = 2),
        ",",
        rep(seq_len(2), times = n_time_obs),
        "]"
      )
    ))
  }

  D <- .diagnostic_positive_int(object$stan_data$D, "D", "stan_data")
  mu_vars <- paste0("mu[", seq_len(D), "]")
  phi_vars <- paste0(
    "Phi[",
    rep(seq_len(D), each = D),
    ",",
    rep(seq_len(D), times = D),
    "]"
  )
  margin_vars <- if (.is_mixed_margins(margins)) {
    # Generic mixed model: check the sampled parameter each dimension actually
    # uses for its own family (the union's unused parameters merely sample from
    # their priors and need not be monitored).
    .mixed_diagnostic_margin_vars(margins)
  } else {
    switch(margins[[1L]],
      normal = paste0("sigma_eps[", seq_len(D), "]"),
      exponential = paste0("eta[", seq_len(D), "]"),
      skew_normal = c(
        paste0("omega[", seq_len(D), "]"),
        paste0("delta[", seq_len(D), "]")
      ),
      gamma = c(
        paste0("eta[", seq_len(D), "]"),
        "shape_gam"
      ),
      paste0("sigma_eps[", seq_len(D), "]")
    )
  }

  if (identical(model, "constant")) {
    copula <- object$copula %||% "gaussian"
    dependence_var <- if (identical(copula, "clayton")) "theta" else "z_rho"
    return(c(mu_vars, phi_vars, margin_vars, dependence_var))
  }

  if (identical(model, "dcvar_tv")) {
    n_time_obs <- .diagnostic_positive_int(object$stan_data$n_time, "n_time", "stan_data")
    n_eff <- n_time_obs - 1L
    # The generic TV model always uses the mixed-union parameter block, so the
    # per-dimension sampled margin parameters follow the mixed convention even
    # for homogeneous margin requests.
    tv_margin_vars <- .mixed_diagnostic_margin_vars(rep(margins, length.out = D))
    vars <- c(
      mu_vars,
      phi_vars,
      tv_margin_vars,
      "z_rho_init",
      "sigma_omega",
      paste0("omega_raw[", seq_len(n_eff), "]")
    )
    if (isTRUE(object$tv_phi)) {
      # tau_phi / phi_raw are sized to the number of active coefficients.
      n_phi_tv <- sum(object$phi_tv_mask %||% .resolve_phi_tv_mask(TRUE))
      vars <- c(
        vars,
        paste0("tau_phi[", seq_len(n_phi_tv), "]"),
        paste0(
          "phi_raw[",
          rep(seq_len(n_eff), each = n_phi_tv), ",", rep(seq_len(n_phi_tv), times = n_eff),
          "]"
        )
      )
    }
    if (isTRUE(object$tv_sigma)) {
      vars <- c(
        vars,
        paste0("tau_sigma[", seq_len(D), "]"),
        paste0(
          "sigma_raw[",
          rep(seq_len(n_eff), each = D), ",", rep(seq_len(D), times = n_eff),
          "]"
        )
      )
    }
    return(vars)
  }

  if (identical(model, "dcvar")) {
    n_time_obs <- .diagnostic_positive_int(object$stan_data$n_time, "n_time", "stan_data")
    return(c(
      mu_vars,
      phi_vars,
      margin_vars,
      "z_rho_init",
      "sigma_omega",
      paste0("omega_raw[", seq_len(n_time_obs - 1L), "]")
    ))
  }

  if (identical(model, "dcvar_covariate") || identical(model, "dcvar_covariate_nodrift")) {
    P <- .diagnostic_positive_int(object$stan_data$P, "P", "stan_data")
    intercept_var <- if (identical(model, "dcvar_covariate") &&
                         isTRUE(object$dynamic_engine)) {
      "z_rho_init"
    } else {
      "beta_0"
    }
    vars <- c(
      mu_vars,
      phi_vars,
      margin_vars,
      intercept_var,
      paste0("beta[", seq_len(P), "]")
    )
    if (identical(model, "dcvar_covariate")) {
      n_time_obs <- .diagnostic_positive_int(object$stan_data$n_time, "n_time", "stan_data")
      n_omega <- (n_time_obs - 1L) - as.integer(isTRUE(object$zero_init_eta))
      if (n_omega > 0L) {
        vars <- c(vars, "sigma_omega", paste0("omega_raw[", seq_len(n_omega), "]"))
      } else {
        vars <- c(vars, "sigma_omega")
      }
    }
    return(vars)
  }

  if (identical(model, "hmm")) {
    K <- .diagnostic_positive_int(object$K, "K", "object")
    hmm_tail <- c(
      paste0("z_rho[", seq_len(K), "]"),
      paste0("pi0[", seq_len(K), "]"),
      paste0(
        "A[",
        rep(seq_len(K), each = K),
        ",",
        rep(seq_len(K), times = K),
        "]"
      )
    )

    if (isTRUE(object$switching)) {
      # The Markov-switching engine samples state-indexed mu / Phi_base+Phi_dev /
      # margin-union, so the monitored names follow the array layout.
      sw <- object$switch
      Mu_K <- if (isTRUE(sw$mu == 1L)) K else 1L
      Mrg_K <- if (isTRUE(sw$margins == 1L)) K else 1L
      mu_sw <- paste0(
        "mu[",
        rep(seq_len(Mu_K), each = D), ",", rep(seq_len(D), times = Mu_K),
        "]"
      )
      phi_sw <- paste0(
        "Phi_base[",
        rep(seq_len(D), each = D), ",", rep(seq_len(D), times = D),
        "]"
      )
      if (any(sw$phi_mask > 0L)) {
        ij <- list(c(1L, 1L), c(1L, 2L), c(2L, 1L), c(2L, 2L))
        active <- which(sw$phi_mask > 0L)
        dev <- unlist(lapply(seq_len(K), function(k) {
          vapply(active, function(a) {
            sprintf("Phi_dev[%d,%d,%d]", k, ij[[a]][1], ij[[a]][2])
          }, character(1))
        }))
        phi_sw <- c(phi_sw, dev)
      }
      margin_sw <- .hmm_switching_diagnostic_margin_vars(object$margins_matrix, Mrg_K)
      return(c(mu_sw, phi_sw, margin_sw, hmm_tail))
    }

    return(c(
      mu_vars,
      phi_vars,
      margin_vars,
      hmm_tail
    ))
  }

  c(mu_vars, phi_vars, margin_vars)
}

#' Internal: extract common diagnostics from a CmdStanMCMC fit
#' @noRd
.sampling_diagnostics_from_fit <- function(fit, backend = "rstan", object = NULL) {
  vars <- if (is.null(object)) NULL else .diagnostic_parameter_variables(object)
  diag_summ <- .fit_diagnostic_summary(fit, backend)
  summ <- suppressWarnings(.fit_summary(fit, variables = vars, backend = backend))
  sampler_diags <- .fit_sampler_diagnostics(fit, backend)
  diag_names <- dimnames(sampler_diags)[[3L]]
  accept_stat <- if ("accept_stat__" %in% diag_names) {
    as.numeric(sampler_diags[, , "accept_stat__", drop = TRUE])
  } else {
    numeric()
  }
  accept_stat <- accept_stat[is.finite(accept_stat)]

  rhat <- summ$rhat[is.finite(summ$rhat)]
  ess_bulk <- summ$ess_bulk[is.finite(summ$ess_bulk)]
  ess_tail <- summ$ess_tail[is.finite(summ$ess_tail)]

  list(
    n_divergent = sum(diag_summ$num_divergent),
    n_max_treedepth = sum(diag_summ$num_max_treedepth),
    max_rhat = if (length(rhat) > 0) max(rhat) else NA_real_,
    min_ess_bulk = if (length(ess_bulk) > 0) min(ess_bulk) else NA_real_,
    min_ess_tail = if (length(ess_tail) > 0) min(ess_tail) else NA_real_,
    mean_accept_prob = if (length(accept_stat) > 0) mean(accept_stat) else NA_real_
  )
}

#' Internal: report post-sampling diagnostics to the user
#' @noRd
.report_sampling_outcome <- function(fit, model_label, chains = NA_integer_,
                                     rhat_threshold = 1.10, backend = "rstan",
                                     object = NULL) {
  diag <- .sampling_diagnostics_from_fit(fit, backend = backend, object = object)
  issues <- character()

  if (diag$n_divergent > 0) {
    issues <- c(
      issues,
      sprintf("%d divergent transition%s.", diag$n_divergent,
              if (diag$n_divergent == 1) "" else "s")
    )
  }
  if (diag$n_max_treedepth > 0) {
    issues <- c(
      issues,
      sprintf("%d transition%s hit the maximum treedepth.", diag$n_max_treedepth,
              if (diag$n_max_treedepth == 1) "" else "s")
    )
  }
  if (isTRUE(chains > 1) && is.finite(diag$max_rhat) && diag$max_rhat > rhat_threshold) {
    issues <- c(
      issues,
      sprintf("Max R-hat is %.3f.", diag$max_rhat)
    )
  }

  if (length(issues) == 0) {
    cli_alert_success("{model_label} sampling complete.")
  } else {
    cli_warn(c(
      "{model_label} sampling finished with diagnostic issues.",
      setNames(issues, rep("!", length(issues))),
      "i" = "Inspect {.fun dcvar_diagnostics} before using this fit for inference."
    ))
  }

  invisible(diag)
}

#' @rdname dcvar_diagnostics
#' @export
dcvar_diagnostics.dcvar_model_fit <- function(object, ...) {
  .sampling_diagnostics_from_fit(object$fit, backend = object$backend %||% "rstan", object = object)
}

test_that("dcvar_diagnostics scopes headline convergence to sampled parameters", {
  object <- structure(
    list(
      fit = structure(list(), class = "mock_fit"),
      stan_data = list(n_time = 5, D = 2),
      model = "dcvar",
      vars = c("y1", "y2"),
      standardized = TRUE,
      margins = "normal",
      backend = "rstan",
      priors = list(),
      meta = list()
    ),
    class = c("dcvar_fit", "dcvar_model_fit")
  )

  requested_variables <- NULL

  testthat::local_mocked_bindings(
    .fit_diagnostic_summary = function(fit, backend) {
      list(num_divergent = 1L, num_max_treedepth = 2L)
    },
    .fit_sampler_diagnostics = function(fit, backend) {
      array(
        c(0.9, 0.8),
        dim = c(1L, 2L, 1L),
        dimnames = list(NULL, NULL, "accept_stat__")
      )
    },
    .fit_summary = function(fit, variables = NULL, backend, ...) {
      requested_variables <<- variables
      data.frame(
        variable = variables,
        rhat = seq(1.01, by = 0.01, length.out = length(variables)),
        ess_bulk = seq(200, by = -1, length.out = length(variables)),
        ess_tail = seq(150, by = -1, length.out = length(variables))
      )
    },
    .package = "dcvar"
  )

  diag <- dcvar_diagnostics(object)

  expect_identical(
    requested_variables,
    c(
      "mu[1]", "mu[2]",
      "Phi[1,1]", "Phi[1,2]", "Phi[2,1]", "Phi[2,2]",
      "sigma_eps[1]", "sigma_eps[2]",
      "z_rho_init", "sigma_omega",
      "omega_raw[1]", "omega_raw[2]", "omega_raw[3]", "omega_raw[4]"
    )
  )
  expect_equal(diag$n_divergent, 1L)
  expect_equal(diag$n_max_treedepth, 2L)
  expect_equal(diag$max_rhat, max(seq(1.01, by = 0.01, length.out = length(requested_variables))))
  expect_equal(diag$min_ess_bulk, min(seq(200, by = -1, length.out = length(requested_variables))))
  expect_equal(diag$min_ess_tail, min(seq(150, by = -1, length.out = length(requested_variables))))
  expect_equal(diag$mean_accept_prob, 0.85)
})

test_that("report_sampling_outcome uses the fit object's diagnostic parameter scope", {
  object <- structure(
    list(
      fit = structure(list(), class = "mock_fit"),
      stan_data = list(n_time = 5, D = 2),
      model = "dcvar",
      vars = c("y1", "y2"),
      standardized = TRUE,
      margins = "normal",
      backend = "rstan",
      priors = list(),
      meta = list()
    ),
    class = c("dcvar_fit", "dcvar_model_fit")
  )
  expected_variables <- .diagnostic_parameter_variables(object)
  requested_variables <- NULL

  testthat::local_mocked_bindings(
    .fit_diagnostic_summary = function(fit, backend) {
      list(num_divergent = 0L, num_max_treedepth = 0L)
    },
    .fit_sampler_diagnostics = function(fit, backend) {
      array(
        0.9,
        dim = c(1L, 1L, 1L),
        dimnames = list(NULL, NULL, "accept_stat__")
      )
    },
    .fit_summary = function(fit, variables = NULL, backend, ...) {
      requested_variables <<- variables
      data.frame(
        variable = variables,
        rhat = rep(1.01, length(variables)),
        ess_bulk = rep(200, length(variables)),
        ess_tail = rep(150, length(variables))
      )
    },
    cli_alert_success = function(...) NULL,
    .package = "dcvar"
  )

  diag <- .report_sampling_outcome(
    object$fit, "Mock DC-VAR", chains = 2, backend = "rstan", object = object
  )

  expect_identical(requested_variables, expected_variables)
  expect_equal(diag$max_rhat, 1.01)
})

test_that("dcvar_diagnostics handles SEM fits without stan_data$D", {
  object <- structure(
    list(
      fit = structure(list(), class = "mock_fit"),
      stan_data = list(n_time = 3),
      model = "sem",
      backend = "rstan",
      priors = list(),
      meta = list()
    ),
    class = c("dcvar_sem_fit", "dcvar_model_fit")
  )

  requested_variables <- NULL

  testthat::local_mocked_bindings(
    .fit_diagnostic_summary = function(fit, backend) {
      list(num_divergent = 0L, num_max_treedepth = 0L)
    },
    .fit_sampler_diagnostics = function(fit, backend) {
      array(
        0.9,
        dim = c(1L, 1L, 1L),
        dimnames = list(NULL, NULL, "accept_stat__")
      )
    },
    .fit_summary = function(fit, variables = NULL, backend, ...) {
      requested_variables <<- variables
      data.frame(
        variable = variables,
        rhat = rep(1.01, length(variables)),
        ess_bulk = rep(200, length(variables)),
        ess_tail = rep(150, length(variables))
      )
    },
    .package = "dcvar"
  )

  diag <- dcvar_diagnostics(object)

  expect_identical(
    requested_variables,
    c(
      "mu[1]", "mu[2]",
      "phi11", "phi12", "phi21", "phi22",
      "sigma[1]", "sigma[2]",
      "rho_raw",
      "zeta[1,1]", "zeta[1,2]",
      "zeta[2,1]", "zeta[2,2]",
      "zeta[3,1]", "zeta[3,2]"
    )
  )
  expect_equal(diag$max_rhat, 1.01)
})

test_that("dcvar_diagnostics handles multilevel fits without stan_data$D", {
  object <- structure(
    list(
      fit = structure(list(), class = "mock_fit"),
      stan_data = list(n_time = 4),
      model = "multilevel",
      N = 2,
      backend = "rstan",
      priors = list(),
      meta = list()
    ),
    class = c("dcvar_multilevel_fit", "dcvar_model_fit")
  )

  requested_variables <- NULL

  testthat::local_mocked_bindings(
    .fit_diagnostic_summary = function(fit, backend) {
      list(num_divergent = 0L, num_max_treedepth = 0L)
    },
    .fit_sampler_diagnostics = function(fit, backend) {
      array(
        0.95,
        dim = c(1L, 1L, 1L),
        dimnames = list(NULL, NULL, "accept_stat__")
      )
    },
    .fit_summary = function(fit, variables = NULL, backend, ...) {
      requested_variables <<- variables
      data.frame(
        variable = variables,
        rhat = rep(1.02, length(variables)),
        ess_bulk = rep(220, length(variables)),
        ess_tail = rep(170, length(variables))
      )
    },
    .package = "dcvar"
  )

  diag <- dcvar_diagnostics(object)

  expect_identical(
    requested_variables,
    c(
      "phi_bar[1]", "phi_bar[2]", "phi_bar[3]", "phi_bar[4]",
      "tau_phi[1]", "tau_phi[2]", "tau_phi[3]", "tau_phi[4]",
      "z_phi[1,1]", "z_phi[1,2]", "z_phi[1,3]", "z_phi[1,4]",
      "z_phi[2,1]", "z_phi[2,2]", "z_phi[2,3]", "z_phi[2,4]",
      "sigma[1]", "sigma[2]",
      "rho"
    )
  )
  expect_equal(diag$max_rhat, 1.02)
})

test_that("dcvar_diagnostics errors clearly when D is missing for single-level models", {
  object <- structure(
    list(
      fit = structure(list(), class = "mock_fit"),
      stan_data = list(n_time = 5),
      model = "dcvar",
      margins = "normal",
      backend = "rstan",
      priors = list(),
      meta = list()
    ),
    class = c("dcvar_fit", "dcvar_model_fit")
  )

  expect_error(
    dcvar_diagnostics(object),
    "requires .*D.*positive integer.*stan_data"
  )
})

# ============================================================================
# Internal Utility Functions
# ============================================================================

#' Safe correlation that returns NA for constant or too-short vectors
#'
#' @param x,y Numeric vectors.
#' @return Correlation coefficient, or `NA_real_` if either vector has zero
#'   variance or fewer than two observations (where `sd()` returns `NA` and
#'   `cor()` errors).
#' @noRd
.safe_cor <- function(x, y) {
  if (length(x) < 2L || length(y) < 2L) {
    return(NA_real_)
  }
  if (sd(x) < 1e-10 || sd(y) < 1e-10) {
    return(NA_real_)
  }

  cor(x, y)
}

#' Validate a credible/prediction interval level
#'
#' @param level Numeric scalar interval level.
#' @param arg_name Name of the argument for error messages.
#' @return Invisibly returns `level`.
#' @noRd
.validate_interval_level <- function(level, arg_name = "ci_level") {
  if (!is.numeric(level) || length(level) != 1L || !is.finite(level) ||
      level <= 0 || level >= 1) {
    cli_abort("{.arg {arg_name}} must be a single numeric value strictly between 0 and 1.")
  }

  invisible(level)
}

#' Validate quantile probability arguments
#'
#' @param probs Numeric vector of probabilities.
#' @param arg_name Name of the argument for error messages.
#' @return Invisibly returns `probs`.
#' @noRd
.validate_probs <- function(probs, arg_name = "probs") {
  if (!is.numeric(probs) || length(probs) == 0L ||
      any(!is.finite(probs)) || any(probs < 0 | probs > 1)) {
    cli_abort("{.arg {arg_name}} must be finite numeric values in [0, 1].")
  }

  invisible(probs)
}

#' Build a regex that matches a Stan output group by base name
#'
#' @param name Stan variable name or indexed element.
#' @return Regex matching the base output name with or without indices.
#' @noRd
.stan_output_group_pattern <- function(name) {
  base_name <- sub("\\[.*$", "", name)
  escaped <- gsub("([][{}()+*^$.|?\\\\])", "\\\\\\1", base_name)
  paste0("^", escaped, "(\\[|$)")
}

#' Internal: extract required coefficient summaries
#' @noRd
.extract_required_coef <- function(summ, pattern, label = NULL, context = NULL) {
  rows <- grep(pattern, summ$variable)
  if (length(rows) == 0) {
    if (is.null(label)) label <- pattern
    if (is.null(context)) context <- "Coefficient extraction"
    cli_abort(c(
      "{context} requires Stan output that is not present in the fitted model.",
      "i" = "Missing coefficient group: {.val {label}}.",
      "i" = "Custom Stan files must preserve the expected parameter names."
    ))
  }

  setNames(summ$mean[rows], summ$variable[rows])
}

#' Internal: validate that a fitted model exposes required Stan outputs
#' @noRd
.validate_required_stan_outputs <- function(draws, required = NULL,
                                            required_type = c("exact", "pattern"),
                                            context = "This method",
                                            output_type = "Stan output") {
  if (is.null(required) || length(required) == 0L) {
    return(invisible(TRUE))
  }

  required_type <- match.arg(required_type)
  vars <- posterior::variables(draws)

  missing <- switch(required_type,
    exact = setdiff(required, vars),
    pattern = required[!vapply(required, function(pattern) {
      any(grepl(pattern, vars))
    }, logical(1))]
  )

  if (length(missing) > 0) {
    output_label <- switch(output_type,
      "generated quantity" = "generated quantities",
      "generated quantities" = "generated quantities",
      "parameter group" = "parameter groups",
      "parameter" = "parameters",
      "parameters" = "parameters",
      "Stan output" = "Stan output",
      output_type
    )

    cli_abort(c(
      "{context} requires Stan output that is not present in the fitted model.",
      "i" = "Missing {output_label}: {.val {missing}}.",
      "i" = "Custom Stan files must preserve the expected parameter and generated-quantity names."
    ))
  }

  invisible(TRUE)
}


# ============================================================================
# Shared Initialization Helpers
# ============================================================================

# Default VAR initialization values shared across all model types.
# These are moderate starting values chosen for standardised data:
#   - Phi diagonal 0.25: mild autoregression, well within stationary region
#   - Phi off-diagonal jitter SD 0.05: small cross-lag starting values
#   - sigma_eps in [0.8, 1.2]: near unit variance (appropriate for z-scored data)
#   - mu near zero: centred around zero (appropriate for z-scored data)

#' Generate default VAR initialization values
#'
#' @param D Number of variables.
#' @param margins Margin type.
#' @return A named list with `mu`, `Phi`, and margin-specific scale params.
#' @noRd
.init_var_params <- function(D, margins = "normal") {
  base <- list(
    mu = rnorm(D, 0, 0.1),
    Phi = diag(0.25, D) + matrix(rnorm(D^2, 0, 0.05), D, D)
  )

  # Mixed (per-variable) margins use the generic Stan model, whose parameter
  # block is the union of all per-family parameters. Initialise the whole union
  # so every sampled parameter (including those that only sample from their
  # prior on a given dimension) starts from a sensible value. shape_gam is a
  # per-dimension vector in the generic model (unlike the shared scalar in the
  # specialised gamma model).
  if (.is_mixed_margins(margins)) {
    return(c(base, list(
      sigma_eps = runif(D, 0.8, 1.2),
      eta = rnorm(D, 0, 0.3),
      omega = runif(D, 0.5, 1.5),
      delta = runif(D, -0.3, 0.3),
      shape_gam = runif(D, 0.5, 2.0)
    )))
  }

  switch(margins[[1L]],
    normal = c(base, list(sigma_eps = runif(D, 0.8, 1.2))),
    exponential = c(base, list(eta = rnorm(D, 0, 0.3))),
    skew_normal = c(base, list(
      omega = runif(D, 0.5, 1.5),
      delta = runif(D, -0.3, 0.3)
    )),
    gamma = c(base, list(
      eta = rnorm(D, 0, 0.3),
      shape_gam = runif(1, 0.5, 2.0)
    )),
    c(base, list(sigma_eps = runif(D, 0.8, 1.2)))
  )
}


#' Generate default DC-VAR initialization values
#'
#' @param D Number of variables.
#' @param T_obs Number of time points.
#' @param margins Margin type.
#' @return A named list with VAR params plus `sigma_omega`, `z_rho_init`,
#'   `omega_raw`.
#' @noRd
.init_dcvar_params <- function(D, T_obs, margins = "normal") {
  c(
    .init_var_params(D, margins),
    list(
      sigma_omega = runif(1, 0.05, 0.15),
      z_rho_init = rnorm(1, 0, 0.1),
      omega_raw = rnorm(T_obs - 1, 0, 0.1)
    )
  )
}


#' Row-major VAR(1) coefficient labels (phi11, phi12, phi21, phi22)
#' @noRd
.phi_coef_names <- c("phi11", "phi12", "phi21", "phi22")

#' Resolve a `tv_phi` specification to a length-4 coefficient mask
#'
#' Accepts a logical scalar (`TRUE` = all four coefficients vary, `FALSE` =
#' none) or a character selector: `"ar"` (the autoregressive coefficients
#' phi11, phi22), `"cross"` / `"crosslag"` (the cross-lagged coefficients
#' phi12, phi21), or specific coefficient names from
#' `c("phi11", "phi12", "phi21", "phi22")`. Returns a named length-4 integer
#' vector in row-major order.
#' @noRd
.resolve_phi_tv_mask <- function(tv_phi) {
  mask <- stats::setNames(integer(4), .phi_coef_names)

  # Idempotent: an already-resolved length-4 0/1 mask is returned as-is
  # (so callers can pass either the user spec or the resolved mask).
  if (is.numeric(tv_phi) && !is.logical(tv_phi) &&
      length(tv_phi) == 4L && all(tv_phi %in% c(0, 1))) {
    mask[] <- as.integer(tv_phi)
    return(mask)
  }

  if (is.logical(tv_phi)) {
    if (length(tv_phi) != 1L || is.na(tv_phi)) {
      cli_abort("{.arg tv_phi} must be a single non-missing logical, or a character selector.")
    }
    mask[] <- as.integer(tv_phi)
    return(mask)
  }

  if (is.character(tv_phi)) {
    if (length(tv_phi) == 0L) {
      cli_abort("{.arg tv_phi} must not be an empty character vector; use {.code FALSE} for no time variation.")
    }
    selected <- unlist(lapply(tv_phi, function(token) {
      switch(token,
        ar = c("phi11", "phi22"),
        cross = ,
        crosslag = ,
        cl = c("phi12", "phi21"),
        token
      )
    }), use.names = FALSE)
    invalid <- setdiff(selected, .phi_coef_names)
    if (length(invalid) > 0L) {
      cli_abort(c(
        "Invalid {.arg tv_phi} selector{?s}: {.val {invalid}}.",
        "i" = "Use {.val TRUE}/{.val FALSE}, {.val ar}, {.val cross}, or names from {.val {(.phi_coef_names)}}."
      ))
    }
    mask[.phi_coef_names %in% selected] <- 1L
    return(mask)
  }

  cli_abort("{.arg tv_phi} must be a logical scalar or a character selector ({.val ar}, {.val cross}, or coefficient names).")
}

#' Resolve a `switch` specification for the Markov-switching HMM
#'
#' Maps the public `switch` argument of [dcvar_hmm()] to canonical flags. Accepts
#' `TRUE` (every component switches) or a character vector drawn from
#' `c("rho", "mu", "phi", "ar", "cross", "sigma")` plus explicit Phi coefficient
#' names. `rho` is mandatory (it is the `ordered[K]` label-switching anchor): a
#' selector that switches other components without `rho` is rejected, and a
#' selector that switches nothing is rejected (use [dcvar_constant()]). Phi
#' granularity reuses [.resolve_phi_tv_mask()].
#'
#' @param switch The user's `switch` argument.
#' @return A named list with `mu` (0/1), `phi_mask` (named length-4 integer),
#'   `rho` (always 1L), and `margins` (0/1, requesting state-specific scales).
#' @noRd
.resolve_switch_spec <- function(switch) {
  if (isTRUE(switch)) {
    return(list(mu = 1L, phi_mask = .resolve_phi_tv_mask(TRUE), rho = 1L, margins = 1L))
  }
  if (isFALSE(switch) ||
      (is.character(switch) && (length(switch) == 0L || identical(switch, "none")))) {
    cli_abort(c(
      "{.arg switch} must select at least one switching component.",
      "i" = "A Markov-switching model with no state-specific parameter is not identified; use {.fun dcvar_constant} or {.fun dcvar_hmm} with {.code switch = \"rho\"}."
    ))
  }
  if (!is.character(switch)) {
    cli_abort("{.arg switch} must be {.val TRUE} or a character selector (e.g. {.val rho}, {.val mu}, {.val phi}).")
  }

  valid <- c("rho", "mu", "sigma", "margins", "phi",
             "ar", "cross", "crosslag", "cl", .phi_coef_names)
  invalid <- setdiff(switch, valid)
  if (length(invalid) > 0L) {
    cli_abort(c(
      "Invalid {.arg switch} selector{?s}: {.val {invalid}}.",
      "i" = "Use {.val rho}, {.val mu}, {.val phi} (or {.val ar}/{.val cross}/coefficient names), {.val sigma}, or {.val TRUE}."
    ))
  }

  mu <- as.integer("mu" %in% switch)
  margins <- as.integer(any(c("sigma", "margins") %in% switch))
  phi_tokens <- intersect(switch, c("phi", "ar", "cross", "crosslag", "cl", .phi_coef_names))
  phi_mask <- if (length(phi_tokens) == 0L) {
    stats::setNames(integer(4), .phi_coef_names)
  } else if ("phi" %in% phi_tokens) {
    .resolve_phi_tv_mask(TRUE)
  } else {
    .resolve_phi_tv_mask(phi_tokens)
  }
  any_phi <- sum(phi_mask) > 0L

  if (!("rho" %in% switch)) {
    cli_abort(c(
      "{.arg switch} must include {.val rho}: the copula correlation is the label-switching anchor.",
      "i" = "State-specific {.val mu}/{.val phi}/{.val sigma} without an ordered {.code rho} are not identified."
    ))
  }

  list(mu = mu, phi_mask = phi_mask, rho = 1L, margins = margins)
}

#' Generate default initialization values for the unified dynamic engine
#'
#' Inits for `dcvar_dynamic.stan`, which always carries residual drift, the full
#' mixed-margin parameter union, the covariate effects, and the conditionally
#' sized time-varying random-walk containers. Only the ACTIVE components are
#' supplied: inactive ones (e.g. `beta` when `P = 0`, the TV deviation
#' containers when their flag is off) are omitted so a zero-extent matrix is
#' never serialised (cmdstanr aborts on those). Length-1 vectors are wrapped
#' with `array(..., dim = n)` so they keep their declared shape. The sampled
#' intercept is `z_rho_init` (the engine exposes `beta_0` as a
#' transformed-parameter alias, so it is not an init).
#'
#' @param D Number of variables.
#' @param T_obs Number of time points.
#' @param margins Margin spec (any; the union is always initialised).
#' @param tv_phi Logical scalar or character selector (see
#'   [.resolve_phi_tv_mask()]).
#' @param tv_sigma Logical; time-varying residual scales.
#' @param P Number of covariates (0 omits `beta`).
#' @param zero_init_eta Logical; whether `eta[1]` is fixed at zero (sets the
#'   `omega_raw` length to `(T_obs - 1) - zero_init_eta`).
#' @return A named list of initial values matching the engine's conditionally
#'   sized parameters.
#' @noRd
.init_dcvar_dynamic_params <- function(D, T_obs, margins = "normal",
                                       tv_phi = FALSE, tv_sigma = FALSE,
                                       P = 0L, zero_init_eta = FALSE) {
  n_eff <- T_obs - 1L
  phi_mask <- .resolve_phi_tv_mask(tv_phi)
  n_phi_tv <- sum(phi_mask)
  any_phi <- n_phi_tv > 0L
  n_omega <- n_eff - as.integer(isTRUE(zero_init_eta))

  base <- list(
    mu = rnorm(D, 0, 0.1),
    Phi = diag(0.25, D) + matrix(rnorm(D^2, 0, 0.05), D, D),
    # Full mixed-margin union (the engine declares every family's parameters).
    sigma_eps = runif(D, 0.8, 1.2),
    eta = rnorm(D, 0, 0.3),
    omega = runif(D, 0.5, 1.5),
    delta = runif(D, -0.3, 0.3),
    shape_gam = runif(D, 0.5, 2.0),
    sigma_omega = runif(1, 0.05, 0.15),
    z_rho_init = rnorm(1, 0, 0.1),
    omega_raw = array(rnorm(n_omega, 0, 0.1), dim = n_omega)
  )

  extra <- list()
  if (P > 0L) {
    extra$beta <- array(rnorm(P, 0, 0.05), dim = P)
  }
  if (any_phi) {
    extra$tau_phi <- runif(n_phi_tv, 0.02, 0.08)
    extra$phi_raw <- matrix(rnorm(n_eff * n_phi_tv, 0, 0.1), n_eff, n_phi_tv)
  }
  if (tv_sigma) {
    extra$tau_sigma <- runif(D, 0.02, 0.08)
    extra$sigma_raw <- matrix(rnorm(n_eff * D, 0, 0.1), n_eff, D)
  }

  c(base, extra)
}


#' Generate default covariate DC-VAR initialization values
#'
#' @param D Number of variables.
#' @param T_obs Number of time points.
#' @param P Number of covariates.
#' @param drift Logical; include residual random-walk drift.
#' @param zero_init_eta Logical; whether `eta[1]` is fixed at zero.
#' @return A named list with VAR params, covariate effects, and optionally
#'   residual drift parameters.
#' @noRd
.init_dcvar_covariate_params <- function(D, T_obs, P, drift = TRUE,
                                         zero_init_eta = TRUE) {
  base <- c(
    .init_var_params(D, "normal"),
    list(
      beta_0 = rnorm(1, 0, 0.1),
      beta = array(rnorm(P, 0, 0.05), dim = P)
    )
  )

  if (!isTRUE(drift)) {
    return(base)
  }

  n_omega <- (T_obs - 1L) - as.integer(isTRUE(zero_init_eta))
  c(
    base,
    list(
      sigma_omega = runif(1, 0.05, 0.15),
      omega_raw = array(rnorm(n_omega, 0, 0.1), dim = n_omega)
    )
  )
}


#' Generate default HMM initialization values
#'
#' @param D Number of variables.
#' @param K Number of hidden states.
#' @param margins Margin type.
#' @return A named list with VAR params plus `z_rho`, `pi0`, `A`.
#' @noRd
.init_hmm_params <- function(D, K, margins = "normal") {
  # Ordered z_rho: evenly spaced with jitter, sorted to satisfy ordering constraint
  z_rho_init <- sort(seq(0.2, 0.8, length.out = K) + rnorm(K, 0, 0.1))

  # Near-identity transition matrix: ~95% self-transition with small jitter
  A_init <- matrix(0.05 / (K - 1), K, K)
  diag(A_init) <- 0.95
  A_init <- A_init + matrix(runif(K * K, -0.01, 0.01), K, K)
  A_init <- pmax(A_init, 0.01)
  A_init <- A_init / rowSums(A_init)

  c(
    .init_var_params(D, margins),
    list(
      z_rho = z_rho_init,
      pi0 = rep(1 / K, K),
      A = A_init
    )
  )
}


#' Generate default initialization values for the Markov-switching HMM engine
#'
#' Inits for `hmm_switching.stan`. State-indexed groups are sized to the engine's
#' conditional extent (`K` when switching, `1` when shared); the full mixed-margin
#' union is initialised per margin config; `Phi_dev` (the per-state coefficient
#' deviations) is supplied only when any Phi coefficient switches, so a zero-extent
#' array is never serialised (cmdstanr hazard). `z_rho` is the ordered anchor; the
#' transition matrix uses a near-identity sticky init.
#'
#' @param D Number of variables.
#' @param K Number of hidden states.
#' @param switch_spec Resolved spec from [.resolve_switch_spec()].
#' @return A named list of initial values matching the engine's parameters.
#' @noRd
.init_hmm_switching_params <- function(D, K, switch_spec) {
  Mu_K <- if (isTRUE(switch_spec$mu == 1L)) K else 1L
  Mrg_K <- if (isTRUE(switch_spec$margins == 1L)) K else 1L
  any_phi <- sum(switch_spec$phi_mask) > 0L

  z_rho_init <- sort(seq(0.2, 0.8, length.out = K) + rnorm(K, 0, 0.1))

  A_init <- matrix(0.05 / (K - 1), K, K)
  diag(A_init) <- 0.95
  A_init <- A_init + matrix(runif(K * K, -0.01, 0.01), K, K)
  A_init <- pmax(A_init, 0.01)
  A_init <- A_init / rowSums(A_init)

  init <- list(
    # array[Mu_K] vector[D]  -> (Mu_K x D)
    mu = array(rnorm(Mu_K * D, 0, 0.1), dim = c(Mu_K, D)),
    Phi_base = diag(0.25, D) + matrix(rnorm(D^2, 0, 0.05), D, D),
    z_rho = z_rho_init,
    pi0 = rep(1 / K, K),
    A = A_init,
    # Full mixed-margin union per config (array[Mrg_K] vector[D] -> Mrg_K x D).
    sigma_eps = array(runif(Mrg_K * D, 0.8, 1.2), dim = c(Mrg_K, D)),
    eta = array(rnorm(Mrg_K * D, 0, 0.3), dim = c(Mrg_K, D)),
    omega = array(runif(Mrg_K * D, 0.5, 1.5), dim = c(Mrg_K, D)),
    delta = array(runif(Mrg_K * D, -0.3, 0.3), dim = c(Mrg_K, D)),
    shape_gam = array(runif(Mrg_K * D, 0.5, 2.0), dim = c(Mrg_K, D))
  )
  # Per-state Phi deviations only when a coefficient switches (else zero-extent).
  if (any_phi) {
    init$Phi_dev <- array(rnorm(K * D * D, 0, 0.05), dim = c(K, D, D))
  }
  init
}


#' Generate default constant copula initialization values
#'
#' @param D Number of variables.
#' @param margins Margin type.
#' @param copula Copula family.
#' @return A named list with VAR params plus `z_rho` or `theta`.
#' @noRd
.init_constant_params <- function(D, margins = "normal", copula = "gaussian") {
  dep_init <- if (identical(copula, "clayton")) {
    list(theta = runif(1, 0.2, 2.5))
  } else {
    list(z_rho = rnorm(1, 0, 0.3))
  }

  c(.init_var_params(D, margins), dep_init)
}


# ============================================================================
# Shared Coefficient Extraction Helper
# ============================================================================

#' Generate default multilevel initialization values
#'
#' @param D Number of variables.
#' @param N Number of units.
#' @param margins Margin type.
#' @return A named list with multilevel params.
#' @noRd
.init_multilevel_params <- function(D, N, margins = "normal") {
  base <- list(
    phi_bar = rnorm(4, 0, 0.1),
    tau_phi = runif(4, 0.05, 0.15),
    z_phi = matrix(rnorm(N * 4, 0, 0.5), N, 4),
    rho = runif(1, -0.3, 0.3)
  )
  if (.uses_sem_multilevel_mixed_engine("multilevel", margins)) {
    # Generic mixed multilevel model: union of per-dimension margin parameters.
    return(c(base, list(
      sigma_eps = runif(D, 0.8, 1.2),
      eta = rnorm(D, 0, 0.3),
      omega = runif(D, 0.5, 1.5),
      delta = runif(D, -0.3, 0.3),
      shape_gam = runif(D, 0.5, 2.0)
    )))
  }
  if (identical(margins[[1L]], "exponential")) {
    c(base, list(eta = rep(log(0.2), D)))
  } else {
    c(base, list(sigma = runif(D, 0.8, 1.2)))
  }
}


#' Generate default TV multilevel initialization values
#' @noRd
.init_multilevel_tv_params <- function(D, N, T_obs, margins = "normal",
                                       tv_phi = FALSE, tv_sigma = FALSE) {
  n_eff <- T_obs - 1L
  phi_mask <- .resolve_phi_tv_mask(tv_phi)
  n_phi_tv <- sum(phi_mask)

  init <- list(
    phi_bar = rnorm(4, 0, 0.1),
    tau_phi = runif(4, 0.05, 0.15),
    z_phi = matrix(rnorm(N * 4, 0, 0.5), N, 4),
    rho = runif(1, -0.3, 0.3),
    # Generic TV engine union of margin parameters.
    sigma_eps = runif(D, 0.8, 1.2),
    eta = rnorm(D, 0, 0.3),
    omega = runif(D, 0.5, 1.5),
    delta = runif(D, -0.3, 0.3),
    shape_gam = runif(D, 0.5, 2.0)
  )
  if (n_phi_tv > 0L) {
    init$tau_phi_tv <- runif(n_phi_tv, 0.02, 0.08)
    init$phi_raw <- matrix(rnorm(n_eff * n_phi_tv, 0, 0.1), n_eff, n_phi_tv)
  }
  if (isTRUE(tv_sigma)) {
    init$tau_sigma <- runif(D, 0.02, 0.08)
    init$sigma_raw <- matrix(rnorm(n_eff * D, 0, 0.1), n_eff, D)
  }
  init
}


#' Generate default SEM initialization values
#'
#' @param T_obs Number of time points.
#' @param margins Margin type. Homogeneous skew-normal/gamma and per-variable
#'   specs use the mixed engine, in which case the union of per-family parameters
#'   is initialised.
#' @return A named list with SEM params.
#' @noRd
.init_sem_params <- function(T_obs, margins = "normal") {
  init <- list(
    mu = rnorm(2, 0, 0.1),
    phi11 = runif(1, 0.1, 0.5),
    phi12 = runif(1, -0.2, 0.2),
    phi21 = runif(1, -0.2, 0.2),
    phi22 = runif(1, 0.1, 0.5),
    rho_raw = rnorm(1, 0, 0.3),
    zeta = matrix(rnorm(T_obs * 2, 0, 0.5), T_obs, 2)
  )

  if (.uses_sem_multilevel_mixed_engine("sem", margins)) {
    init$sigma_eps <- runif(2, 0.8, 1.2)
    init$eta <- rnorm(2, 0, 0.3)
    init$omega <- runif(2, 0.5, 1.5)
    init$delta <- runif(2, -0.3, 0.3)
    init$shape_gam <- runif(2, 0.5, 2.0)
  } else if (identical(margins[[1L]], "exponential")) {
    init$eta <- rnorm(2, 0, 0.2)
  } else {
    init$sigma <- runif(2, 0.5, 1.5)
  }

  init
}


#' Generate default TV SEM initialization values
#' @noRd
.init_sem_tv_params <- function(T_obs, margins = "normal",
                                tv_phi = FALSE, tv_sigma = FALSE) {
  n_eff <- T_obs - 1L
  phi_mask <- .resolve_phi_tv_mask(tv_phi)
  n_phi_tv <- sum(phi_mask)

  init <- list(
    mu = rnorm(2, 0, 0.1),
    Phi = diag(c(runif(1, 0.1, 0.5), runif(1, 0.1, 0.5))) +
      matrix(c(0, runif(1, -0.2, 0.2), runif(1, -0.2, 0.2), 0), 2, 2),
    z_rho_init = rnorm(1, 0, 0.3),
    sigma_omega = runif(1, 0.05, 0.15),
    omega_raw = array(rnorm(n_eff, 0, 0.1), dim = n_eff),
    zeta = matrix(rnorm(T_obs * 2, 0, 0.5), T_obs, 2),
    # Generic TV engine union of margin parameters.
    sigma_eps = runif(2, 0.8, 1.2),
    eta = rnorm(2, 0, 0.3),
    omega = runif(2, 0.5, 1.5),
    delta = runif(2, -0.3, 0.3),
    shape_gam = runif(2, 0.5, 2.0)
  )
  if (n_phi_tv > 0L) {
    init$tau_phi <- runif(n_phi_tv, 0.02, 0.08)
    init$phi_raw <- matrix(rnorm(n_eff * n_phi_tv, 0, 0.1), n_eff, n_phi_tv)
  }
  if (isTRUE(tv_sigma)) {
    init$tau_sigma <- runif(2, 0.02, 0.08)
    init$sigma_raw <- matrix(rnorm(n_eff * 2, 0, 0.1), n_eff, 2)
  }
  init
}

#' Generate default naive SEM initialization values
#'
#' @param y T x 2 matrix of row-mean factor scores.
#' @param margins Margin type. Homogeneous skew-normal/gamma and per-variable
#'   specs use the mixed engine, in which case the union of per-family parameters
#'   is initialised.
#' @return A named list with observed-score VAR params.
#' @noRd
.init_sem_naive_params <- function(y, margins = "normal") {
  y <- as.matrix(y)
  ylag <- rbind(c(0, 0), y[-nrow(y), , drop = FALSE])

  fit1 <- try(stats::lm(y[, 1] ~ ylag[, 1] + ylag[, 2]), silent = TRUE)
  fit2 <- try(stats::lm(y[, 2] ~ ylag[, 1] + ylag[, 2]), silent = TRUE)

  coef_or <- function(fit, idx, fallback) {
    if (inherits(fit, "try-error")) {
      return(fallback)
    }
    val <- stats::coef(fit)[idx]
    if (!is.finite(val)) fallback else val
  }
  clip_phi <- function(x) max(min(x, 0.9), -0.9)

  init <- list(
    mu = c(0, 0),
    phi11 = clip_phi(coef_or(fit1, 2, 0.55)),
    phi12 = clip_phi(coef_or(fit1, 3, 0.10)),
    phi21 = clip_phi(coef_or(fit2, 2, 0.10)),
    phi22 = clip_phi(coef_or(fit2, 3, 0.25)),
    rho_raw = rnorm(1, 0, 0.5)
  )

  if (.uses_sem_multilevel_mixed_engine("sem_naive", margins)) {
    init$sigma_eps <- runif(2, 0.8, 1.2)
    init$eta <- rnorm(2, 0, 0.3)
    init$omega <- runif(2, 0.5, 1.5)
    init$delta <- runif(2, -0.3, 0.3)
    init$shape_gam <- runif(2, 0.5, 2.0)
  } else if (identical(margins[[1L]], "exponential")) {
    init$eta <- rep(log(0.2), 2)
  } else {
    init$sigma <- runif(2, 0.8, 1.2)
  }

  init
}


#' Extract margin-specific coefficients from a summary
#'
#' @param summ A summary data frame from `object$fit$summary()`.
#' @param margins Character: margin type.
#' @return A named list of margin-specific coefficient vectors.
#' @noRd
.extract_margin_coefs <- function(summ, margins) {
  if (length(margins) > 1L) {
    # Report each dimension's own family scale/shape (restricted to that
    # family's dimensions), e.g. sigma_eps[1] for a normal dim and
    # sigma_exp[2] for an exponential dim. This also covers homogeneous
    # SEM/multilevel gamma/skew-normal fits routed through mixed engines.
    specs <- .mixed_margin_report_vars(margins)
    return(lapply(specs, function(vars) {
      rows <- match(vars, summ$variable)
      rows <- rows[!is.na(rows)]
      if (length(rows) == 0L) return(NULL)
      setNames(summ$mean[rows], summ$variable[rows])
    }))
  }
  switch(margins[[1L]],
    normal = list(sigma_eps = .extract_coef(summ, "^sigma_eps\\[")),
    exponential = list(sigma_exp = .extract_coef(summ, "^sigma_exp\\[")),
    skew_normal = list(
      omega = .extract_coef(summ, "^omega\\["),
      delta = .extract_coef(summ, "^delta\\[")
    ),
    gamma = list(
      sigma_gam = .extract_coef(summ, "^sigma_gam\\["),
      shape_gam = .extract_coef(summ, "^shape_gam$")
    ),
    list(sigma_eps = .extract_coef(summ, "^sigma_eps\\["))
  )
}


#' Extract posterior means from a fit summary by regex pattern
#'
#' Shared helper used by all `coef.*` methods to avoid duplicated grep logic.
#'
#' @param summ A summary data frame from `object$fit$summary()`.
#' @param pattern Regex pattern to match against `summ$variable`.
#' @return A named numeric vector of posterior means, or `NULL` if no match.
#' @noRd
.extract_coef <- function(summ, pattern) {
  rows <- grep(pattern, summ$variable)
  if (length(rows) == 0) return(NULL)
  setNames(summ$mean[rows], summ$variable[rows])
}


# ============================================================================
# Shared Print Helpers
# ============================================================================

#' Print margin-specific scale parameters from var_params
#'
#' Encapsulates the if/else logic for printing the appropriate scale
#' parameters depending on which margin was used (normal, exponential,
#' skew_normal, gamma).
#'
#' @param var_params A named list (the `var_params` element of a summary object).
#' @return Called for its side effect (printing); returns `NULL` invisibly.
#' @noRd
.print_margin_params <- function(var_params) {
  cols <- c("variable", "mean", "q2.5", "q97.5")
  # Independent checks (not else-if) so mixed fits print every family's scale
  # parameters; single-family fits still print exactly one group.
  for (nm in c("sigma_eps", "sigma_exp", "omega", "delta", "sigma_gam", "shape_gam")) {
    if (!is.null(var_params[[nm]])) {
      cat(sprintf("\n  %s:\n", nm))
      print(var_params[[nm]][, cols], row.names = FALSE)
    }
  }
  invisible(NULL)
}


#' Print shared header for all fit objects
#' @noRd
.print_fit_header <- function(x, model_label) {
  cat(model_label, "\n")
  cat(strrep("-", 40), "\n")
  cat(sprintf("Variables: %s\n", paste(x$vars, collapse = ", ")))
}


#' Print shared MCMC info and diagnostics footer
#' @noRd
.print_fit_footer <- function(x) {
  diag <- dcvar_diagnostics(x)
  cat(sprintf("Chains: %d | Warmup: %d | Sampling: %d\n",
              x$meta$chains, x$meta$iter_warmup, x$meta$iter_sampling))
  cat(sprintf("Divergences: %d | Max Rhat: %.3f\n",
              diag$n_divergent, diag$max_rhat))
}

# ============================================================================
# S3 Class: dcvar_constant_fit
# ============================================================================

#' Construct a dcvar_constant_fit object
#' @noRd
new_dcvar_constant_fit <- function(fit, stan_data, vars, standardized,
                                   margins = "normal", copula = "gaussian",
                                   skew_direction = NULL,
                                   backend = "rstan", priors, meta) {
  structure(
    list(
      fit = fit,
      stan_data = stan_data,
      model = "constant",
      vars = vars,
      standardized = standardized,
      margins = margins,
      copula = copula,
      skew_direction = skew_direction,
      backend = backend,
      priors = priors,
      meta = meta
    ),
    class = c("dcvar_constant_fit", "dcvar_model_fit")
  )
}


#' S3 methods for dcvar_constant_fit objects
#'
#' @param x,object A `dcvar_constant_fit` object.
#' @param ... Additional arguments (unused).
#'
#' @name dcvar_constant_fit-methods
NULL

#' @describeIn dcvar_constant_fit-methods Print a concise overview of the
#'   constant copula fit.
#' @return Invisibly returns `x`.
#' @export
print.dcvar_constant_fit <- function(x, ...) {
  .print_fit_header(x, "Constant Copula Model Fit")
  cat(sprintf("n_time = %d, D = %d\n", x$stan_data$n_time, x$stan_data$D))
  .print_fit_footer(x)

  if (identical(x$copula %||% "gaussian", "clayton")) {
    dep_df <- dependence_summary(x)
    cat(sprintf("Kendall's tau: %.3f [%.3f, %.3f]\n",
                dep_df$mean[1], dep_df$q2.5[1], dep_df$q97.5[1]))
  } else {
    rho_df <- rho_trajectory(x)
    cat(sprintf("rho: %.3f [%.3f, %.3f]\n",
                rho_df$mean[1], rho_df$q2.5[1], rho_df$q97.5[1]))
  }
  invisible(x)
}


#' @describeIn dcvar_constant_fit-methods Produce a detailed summary including
#'   constant rho, VAR parameters, and diagnostics.
#' @param probs Numeric vector of quantile probabilities for the rho estimate
#'   (default: `c(0.025, 0.5, 0.975)`).
#' @return A `dcvar_constant_summary` object (a list).
#' @export
summary.dcvar_constant_fit <- function(object, probs = c(0.025, 0.5, 0.975), ...) {
  copula <- object$copula %||% "gaussian"
  rho_df <- if (identical(copula, "gaussian")) rho_trajectory(object, probs = probs) else NULL
  dep_df <- dependence_summary(object, probs = probs)
  vp <- var_params(object)
  diag <- dcvar_diagnostics(object)

  out <- list(
    model = "constant",
    copula = copula,
    n_time = object$stan_data$n_time,
    D = object$stan_data$D,
    rho = rho_df,
    dependence = dep_df,
    var_params = vp,
    diagnostics = diag
  )
  class(out) <- "dcvar_constant_summary"
  out
}


#' Print a dcvar_constant_summary object
#'
#' @param x A `dcvar_constant_summary` object as returned by
#'   [summary.dcvar_constant_fit()].
#' @param ... Additional arguments (unused).
#' @return Invisibly returns `x`.
#' @export
print.dcvar_constant_summary <- function(x, ...) {
  dep <- if (identical(x$copula %||% "gaussian", "gaussian")) x$rho else x$dependence
  quantile_cols <- grep("^q", names(dep), value = TRUE)
  # Column order follows the probs argument, which need not be ascending;
  # sort by the numeric suffix so lower/upper are not swapped.
  quantile_cols <- quantile_cols[order(as.numeric(sub("^q", "", quantile_cols)))]
  lower_col <- quantile_cols[1]
  upper_col <- quantile_cols[length(quantile_cols)]

  cat("Constant Copula Model Summary\n")
  cat(strrep("=", 50), "\n")
  cat(sprintf("n_time = %d, D = %d\n\n", x$n_time, x$D))

  dep_label <- if (identical(x$copula %||% "gaussian", "clayton")) {
    "Kendall's tau (constant)"
  } else {
    "Rho (constant)"
  }
  cat(sprintf("%s: %.3f", dep_label, dep$mean[1]))
  if (length(quantile_cols) >= 2) {
    cat(sprintf(" [%.3f, %.3f]", dep[[lower_col]][1], dep[[upper_col]][1]))
  }
  cat("\n\n")

  cat("VAR(1) Parameters:\n")
  cat("  mu:\n")
  print(x$var_params$mu[, c("variable", "mean", "q2.5", "q97.5")], row.names = FALSE)
  cat("\n  Phi:\n")
  print(x$var_params$Phi[, c("variable", "mean", "q2.5", "q97.5")], row.names = FALSE)
  .print_margin_params(x$var_params)

  cat("\nDiagnostics:\n")
  cat(sprintf("  Divergences: %d\n", x$diagnostics$n_divergent))
  cat(sprintf("  Max Rhat: %.3f\n", x$diagnostics$max_rhat))
  cat(sprintf("  Min ESS bulk: %.0f\n", x$diagnostics$min_ess_bulk))

  invisible(x)
}


#' @describeIn dcvar_constant_fit-methods Extract posterior means of model
#'   coefficients.
#' @return A named list of posterior means: `mu`, `Phi`, margin-specific
#'   scale/shape parameters (e.g. `sigma_eps` for normal margins), and either
#'   `rho` (Gaussian copula) or `theta` (Clayton copula).
#' @export
coef.dcvar_constant_fit <- function(object, ...) {
  summ <- .fit_summary(object$fit, backend = object$backend)
  result <- list(
    mu = .extract_required_coef(summ, "^mu\\[", "mu", "coef.dcvar_constant_fit()"),
    Phi = .extract_required_coef(summ, "^Phi\\[", "Phi", "coef.dcvar_constant_fit()")
  )
  # Margin-specific scale params before rho
  margins <- object$margins %||% "normal"
  result <- c(result, .extract_margin_coefs(summ, margins))
  if (identical(object$copula %||% "gaussian", "clayton")) {
    result$theta <- .extract_required_coef(summ, "^theta$", "theta", "coef.dcvar_constant_fit()")
  } else {
    result$rho <- .extract_required_coef(summ, "^rho$", "rho", "coef.dcvar_constant_fit()")
  }
  result
}


#' @describeIn dcvar_constant_fit-methods Dispatch to a plot type: `"rho"`,
#'   `"phi"`, `"diagnostics"`, `"ppc"`, or `"pit"`.
#' @param type Character; one of `"rho"`, `"phi"`, `"diagnostics"`, `"ppc"`,
#'   `"pit"`.
#' @return A ggplot object.
#' @export
plot.dcvar_constant_fit <- function(x, type = c("rho", "phi", "diagnostics", "ppc", "pit"), ...) {
  type <- match.arg(type)
  if (identical(type, "rho") && identical(x$copula %||% "gaussian", "clayton")) {
    cli_abort("Clayton fits do not have a Gaussian copula rho. Use {.fun dependence_summary} for Kendall's tau.")
  }
  switch(type,
    rho = plot_rho(x, ...),
    phi = plot_phi(x, ...),
    diagnostics = plot_diagnostics(x, ...),
    ppc = plot_ppc(x, ...),
    pit = plot_pit(x, ...)
  )
}

# ============================================================================
# PIT (Probability Integral Transform) Diagnostics
# ============================================================================

utils::globalVariables("count")

#' Extract PIT values from a fitted model
#'
#' Computes approximate Probability Integral Transform values using posterior
#' mean residuals and posterior mean margin parameters. Large departures from
#' uniformity can indicate model misfit, but these are not exact posterior
#' predictive PIT values.
#'
#' @param object A fitted model object.
#' @param ... Additional arguments (unused).
#'
#' @return A data frame with columns `time`, `variable`, `pit`.
#'
#' @details
#' PIT values are computed from posterior mean residuals and posterior mean
#' margin parameters. Treat them as a fast plug-in diagnostic rather than an
#' exact posterior predictive transform that integrates over full posterior
#' uncertainty. PIT diagnostics are currently implemented for the three core
#' single-level fit classes only.
#' @export
pit_values <- function(object, ...) {
  UseMethod("pit_values")
}

#' @rdname pit_values
#' @export
pit_values.default <- function(object, ...) {
  cli_abort("{.fun pit_values} is not defined for objects of class {.cls {class(object)[[1]]}}.")
}

#' @rdname pit_values
#' @export
pit_values.dcvar_hmm_fit <- function(object, ...) {
  if (isTRUE(object$switching)) {
    cli_abort(c(
      "{.fun pit_values} is not yet available for state-specific HMM fits (residuals are per-state).",
      "i" = "Use {.fun hmm_states} / {.fun hmm_state_params} for state-specific summaries."
    ))
  }
  NextMethod()
}

#' @rdname pit_values
#' @export
pit_values.dcvar_model_fit <- function(object, ...) {
  margins <- object$margins %||% "normal"

  # Extract posterior mean residuals and parameters
  eps_draws <- posterior::as_draws_matrix(.fit_draws(object$fit, "eps", backend = object$backend))
  n_time_eff <- object$stan_data$n_time - 1L
  D <- object$stan_data$D

  # Posterior mean residuals
  eps_mean <- matrix(NA_real_, n_time_eff, D)
  for (d in seq_len(D)) {
    cols <- paste0("eps[", seq_len(n_time_eff), ",", d, "]")
    eps_mean[, d] <- colMeans(eps_draws[, cols, drop = FALSE])
  }

  # Compute PIT values based on margin type
  pit_mat <- .pit_compute(object, eps_mean, margins)

  # Convert to long format
  var_names <- object$vars
  time_values <- .observed_time_values(object$stan_data, drop_first = TRUE)
  data.frame(
    time = rep(time_values, D),
    variable = rep(var_names, each = n_time_eff),
    pit = as.vector(pit_mat)
  )
}


#' Internal: posterior means for a margin parameter group, validated
#'
#' Mirrors the per-name validation in `.pit_mixed()`: a margin parameter that
#' is absent from the Stan output must abort with a clear message instead of
#' silently yielding all-NA PIT values.
#' @noRd
.pit_margin_means <- function(summ, pattern, expected, label) {
  rows <- grep(pattern, summ$variable)
  if (length(rows) != expected) {
    cli_abort(c(
      "PIT values require the Stan output {.val {label}}, which is missing from the fit.",
      "i" = "Custom Stan files must preserve the expected parameter names."
    ))
  }
  summ$mean[rows]
}


#' Internal: compute PIT values for given residuals and margin type
#' @noRd
.pit_compute <- function(object, eps_mean, margins) {
  T_eff <- nrow(eps_mean)
  D <- ncol(eps_mean)

  if (.is_mixed_margins(margins)) {
    return(.pit_mixed(object, eps_mean, margins))
  }

  switch(margins[[1L]],
    normal = {
      summ <- .fit_summary(object$fit, backend = object$backend)
      sigma_mean <- .pit_margin_means(summ, "^sigma_eps\\[", D, "sigma_eps")
      pit_mat <- matrix(NA_real_, T_eff, D)
      for (d in seq_len(D)) {
        pit_mat[, d] <- stats::pnorm(eps_mean[, d] / sigma_mean[d])
      }
      pit_mat
    },
    exponential = {
      .pit_exponential(object, eps_mean)
    },
    skew_normal = {
      .pit_skew_normal(object, eps_mean)
    },
    gamma = {
      .pit_gamma(object, eps_mean)
    },
    cli_abort("Unknown margin type: {.val {margins[[1L]]}}")
  )
}


#' Internal: PIT for exponential margins
#' @noRd
.pit_exponential <- function(object, eps_mean) {
  T_eff <- nrow(eps_mean)
  D <- ncol(eps_mean)
  summ <- .fit_summary(object$fit, backend = object$backend)
  skew_dir <- object$skew_direction

  sigma_exp_mean <- .pit_margin_means(summ, "^sigma_exp\\[", D, "sigma_exp")
  rate_exp <- 1.0 / sigma_exp_mean

  pit_mat <- matrix(NA_real_, T_eff, D)
  for (d in seq_len(D)) {
    x_shifted <- sigma_exp_mean[d] + skew_dir[d] * eps_mean[, d]
    u <- stats::pexp(x_shifted, rate = rate_exp[d])
    if (skew_dir[d] < 0) u <- 1.0 - u
    pit_mat[, d] <- u
  }
  pit_mat
}


#' Internal: PIT for skew-normal margins
#' @noRd
.pit_skew_normal <- function(object, eps_mean) {
  if (!requireNamespace("sn", quietly = TRUE)) {
    cli_abort("Package {.pkg sn} is required for skew-normal PIT values.")
  }
  T_eff <- nrow(eps_mean)
  D <- ncol(eps_mean)
  summ <- .fit_summary(object$fit, backend = object$backend)

  omega_mean <- .pit_margin_means(summ, "^omega\\[", D, "omega")
  delta_mean <- .pit_margin_means(summ, "^delta\\[", D, "delta")

  alpha_mean <- delta_mean / sqrt(1 - delta_mean^2)
  xi_mean <- -omega_mean * delta_mean * sqrt(2 / pi)

  pit_mat <- matrix(NA_real_, T_eff, D)
  for (d in seq_len(D)) {
    pit_mat[, d] <- sn::psn(eps_mean[, d],
      xi = xi_mean[d], omega = omega_mean[d], alpha = alpha_mean[d]
    )
  }
  pit_mat
}


#' Internal: PIT for gamma margins
#' @noRd
.pit_gamma <- function(object, eps_mean) {
  T_eff <- nrow(eps_mean)
  D <- ncol(eps_mean)
  summ <- .fit_summary(object$fit, backend = object$backend)
  skew_dir <- object$skew_direction

  sigma_gam_mean <- .pit_margin_means(summ, "^sigma_gam\\[", D, "sigma_gam")
  shape_gam_mean <- .pit_margin_means(summ, "^shape_gam$", 1L, "shape_gam")

  sqrt_shape <- sqrt(shape_gam_mean)
  rate_gam <- sqrt_shape / sigma_gam_mean

  pit_mat <- matrix(NA_real_, T_eff, D)
  for (d in seq_len(D)) {
    mean_x <- sqrt_shape * sigma_gam_mean[d]
    x_shifted <- mean_x + skew_dir[d] * eps_mean[, d]
    u <- stats::pgamma(x_shifted, shape = shape_gam_mean, rate = rate_gam[d])
    if (skew_dir[d] < 0) u <- 1.0 - u
    pit_mat[, d] <- u
  }
  pit_mat
}


#' Internal: PIT for mixed (per-variable) margins
#'
#' Dispatches each dimension to its own family's plug-in PIT using posterior
#' mean residuals and posterior mean margin parameters.
#' @noRd
.pit_mixed <- function(object, eps_mean, margins) {
  T_eff <- nrow(eps_mean)
  D <- ncol(eps_mean)
  summ <- .fit_summary(object$fit, backend = object$backend)
  skew_dir <- object$skew_direction
  gm <- function(name) {
    row <- match(name, summ$variable)
    if (is.na(row)) {
      cli_abort("PIT for mixed margins requires Stan output {.val {name}}.")
    }
    summ$mean[row]
  }

  needs_sn <- any(margins == "skew_normal")
  if (needs_sn && !requireNamespace("sn", quietly = TRUE)) {
    cli_abort("Package {.pkg sn} is required for skew-normal PIT values.")
  }

  pit_mat <- matrix(NA_real_, T_eff, D)
  for (d in seq_len(D)) {
    fam <- margins[[d]]
    if (identical(fam, "normal")) {
      sigma_d <- gm(paste0("sigma_eps[", d, "]"))
      pit_mat[, d] <- stats::pnorm(eps_mean[, d] / sigma_d)
    } else if (identical(fam, "exponential")) {
      sigma_exp_d <- gm(paste0("sigma_exp[", d, "]"))
      x_shifted <- sigma_exp_d + skew_dir[d] * eps_mean[, d]
      u <- stats::pexp(x_shifted, rate = 1.0 / sigma_exp_d)
      if (skew_dir[d] < 0) u <- 1.0 - u
      pit_mat[, d] <- u
    } else if (identical(fam, "skew_normal")) {
      omega_d <- gm(paste0("omega[", d, "]"))
      delta_d <- gm(paste0("delta[", d, "]"))
      alpha_d <- delta_d / sqrt(1 - delta_d^2)
      xi_d <- -omega_d * delta_d * sqrt(2 / pi)
      pit_mat[, d] <- sn::psn(eps_mean[, d], xi = xi_d, omega = omega_d, alpha = alpha_d)
    } else {
      sigma_gam_d <- gm(paste0("sigma_gam[", d, "]"))
      shape_gam_d <- gm(paste0("shape_gam[", d, "]"))
      sqrt_shape <- sqrt(shape_gam_d)
      mean_x <- sqrt_shape * sigma_gam_d
      x_shifted <- mean_x + skew_dir[d] * eps_mean[, d]
      u <- stats::pgamma(x_shifted, shape = shape_gam_d, rate = sqrt_shape / sigma_gam_d)
      if (skew_dir[d] < 0) u <- 1.0 - u
      pit_mat[, d] <- u
    }
  }
  pit_mat
}


#' KS test for PIT uniformity
#'
#' Runs a Kolmogorov-Smirnov test per variable to assess whether PIT values
#' are approximately uniform. This is a heuristic check on the plug-in PIT
#' values returned by [pit_values()], not an exact posterior predictive test.
#'
#' @param object A fitted model object.
#' @param ... Additional arguments (unused).
#'
#' @return A data frame with columns `variable`, `ks_statistic`, `p_value`, `n`.
#'
#' @details
#' This applies a Kolmogorov-Smirnov test to the approximate PIT values
#' returned by [pit_values()]. The result is a heuristic check and does not
#' account for serial dependence or full posterior uncertainty. PIT tests are
#' currently implemented for single-level fits only.
#' @export
pit_test <- function(object, ...) {
  UseMethod("pit_test")
}

#' @rdname pit_test
#' @export
pit_test.default <- function(object, ...) {
  cli_abort("{.fun pit_test} is not defined for objects of class {.cls {class(object)[[1]]}}.")
}

#' @rdname pit_test
#' @export
pit_test.dcvar_model_fit <- function(object, ...) {
  pit_df <- pit_values(object)

  results <- lapply(unique(pit_df$variable), function(v) {
    pit_v <- pit_df$pit[pit_df$variable == v]
    ks <- stats::ks.test(pit_v, "punif")
    data.frame(
      variable = v,
      ks_statistic = unname(ks$statistic),
      p_value = ks$p.value,
      n = length(pit_v)
    )
  })

  do.call(rbind, results)
}


#' @export
pit_values.dcvar_multilevel_fit <- function(object, ...) {
  cli_abort("PIT values are not yet supported for multilevel models.")
}

#' @export
pit_values.dcvar_sem_fit <- function(object, ...) {
  cli_abort("PIT values are not yet supported for SEM models.")
}

#' @export
pit_test.dcvar_multilevel_fit <- function(object, ...) {
  cli_abort("PIT tests are not yet supported for multilevel models.")
}

#' @export
pit_test.dcvar_sem_fit <- function(object, ...) {
  cli_abort("PIT tests are not yet supported for SEM models.")
}


#' Plot PIT histograms
#'
#' Creates faceted histograms of the approximate PIT values returned by
#' [pit_values()]. Under good model fit, these histograms should be roughly
#' uniform, but they remain plug-in diagnostics rather than exact posterior
#' predictive checks.
#'
#' @param object A fitted model object.
#' @param bins Number of histogram bins (default: 20).
#' @param ... Additional arguments (unused).
#'
#' @return A ggplot object.
#'
#' @details PIT histograms visualize the approximate plug-in PIT values
#'   returned by [pit_values()]. They are currently implemented for the three
#'   core single-level fit classes only.
#' @export
plot_pit <- function(object, bins = 20, ...) {
  pit_df <- pit_values(object)

  ggplot2::ggplot(pit_df, ggplot2::aes(x = .data$pit)) +
    ggplot2::geom_histogram(
      bins = bins,
      binwidth = 1 / bins,
      boundary = 0,
      ggplot2::aes(y = ggplot2::after_stat(count / sum(count) * bins)),
      fill = "steelblue", alpha = 0.7, color = "white"
    ) +
    ggplot2::geom_hline(yintercept = 1, linetype = "dashed", color = "red", linewidth = 0.8) +
    ggplot2::facet_wrap(~ variable) +
    ggplot2::labs(
      x = "PIT Value",
      y = "Density",
      title = "Approximate PIT Diagnostics",
      subtitle = "Red dashed line: uniform plug-in reference"
    ) +
    ggplot2::theme_minimal() +
    ggplot2::theme(
      plot.title = ggplot2::element_text(face = "bold"),
      strip.text = ggplot2::element_text(face = "bold")
    )
}


#' @rdname pit_values
#' @export
pit_values.dcvar_tv_fit <- function(object, ...) {
  D <- object$stan_data$D
  n_time_eff <- object$stan_data$n_time - 1L
  margins_vec <- rep(object$margins %||% "normal", length.out = D)
  skew_dir <- object$skew_direction %||% rep(1, D)

  needs_sn <- any(margins_vec == "skew_normal")
  if (needs_sn && !requireNamespace("sn", quietly = TRUE)) {
    cli_abort("Package {.pkg sn} is required for skew-normal PIT values.")
  }

  eps_draws <- posterior::as_draws_matrix(.fit_draws(object$fit, "eps", backend = object$backend))
  eps_mean <- matrix(NA_real_, n_time_eff, D)
  for (d in seq_len(D)) {
    cols <- paste0("eps[", seq_len(n_time_eff), ",", d, "]")
    eps_mean[, d] <- colMeans(eps_draws[, cols, drop = FALSE])
  }

  summ <- .fit_summary(object$fit, backend = object$backend)
  gm <- function(name) {
    row <- match(name, summ$variable)
    if (is.na(row)) {
      cli_abort("PIT for time-varying fits requires Stan output {.val {name}}.")
    }
    summ$mean[row]
  }

  # Per-time plug-in scale for each dimension (posterior means): the sigma_t
  # path when scales vary, otherwise the constant baseline tiled over time.
  scale_mean <- matrix(NA_real_, n_time_eff, D)
  for (d in seq_len(D)) {
    if (isTRUE(object$tv_sigma)) {
      rows <- match(paste0("sigma_t[", seq_len(n_time_eff), ",", d, "]"), summ$variable)
      if (anyNA(rows)) {
        cli_abort("PIT for time-varying fits requires Stan output {.val sigma_t}.")
      }
      scale_mean[, d] <- summ$mean[rows]
    } else {
      base <- switch(margins_vec[d],
        normal = gm(paste0("sigma_eps[", d, "]")),
        exponential = gm(paste0("sigma_exp[", d, "]")),
        skew_normal = gm(paste0("omega[", d, "]")),
        gamma = gm(paste0("sigma_gam[", d, "]"))
      )
      scale_mean[, d] <- base
    }
  }

  pit_mat <- matrix(NA_real_, n_time_eff, D)
  for (d in seq_len(D)) {
    fam <- margins_vec[d]
    s <- scale_mean[, d]
    if (identical(fam, "normal")) {
      pit_mat[, d] <- stats::pnorm(eps_mean[, d] / s)
    } else if (identical(fam, "exponential")) {
      u <- stats::pexp(s + skew_dir[d] * eps_mean[, d], rate = 1 / s)
      if (skew_dir[d] < 0) u <- 1 - u
      pit_mat[, d] <- u
    } else if (identical(fam, "skew_normal")) {
      delta_d <- gm(paste0("delta[", d, "]"))
      alpha_d <- delta_d / sqrt(1 - delta_d^2)
      omega_t <- if (isTRUE(object$tv_sigma)) {
        s / .skew_normal_residual_sd_factor(delta_d)
      } else {
        s
      }
      xi_t <- -omega_t * delta_d * sqrt(2 / pi)
      pit_mat[, d] <- sn::psn(eps_mean[, d], xi = xi_t, omega = omega_t, alpha = alpha_d)
    } else {
      shape_d <- gm(paste0("shape_gam[", d, "]"))
      mean_x <- sqrt(shape_d) * s
      u <- stats::pgamma(mean_x + skew_dir[d] * eps_mean[, d],
                         shape = shape_d, rate = sqrt(shape_d) / s)
      if (skew_dir[d] < 0) u <- 1 - u
      pit_mat[, d] <- u
    }
  }

  time_values <- .observed_time_values(object$stan_data, drop_first = TRUE)
  data.frame(
    time = rep(time_values, D),
    variable = rep(object$vars, each = n_time_eff),
    pit = as.vector(pit_mat)
  )
}

# ============================================================================
# Tests for PIT diagnostics
# ============================================================================

test_that("pit_values returns correct structure for constant fit", {
  skip_if_no_rstan()
  fit <- get_constant_fit()
  pit_df <- pit_values(fit)

  expect_s3_class(pit_df, "data.frame")
  expect_named(pit_df, c("time", "variable", "pit"))
  expect_equal(nrow(pit_df), (fit$stan_data$n_time - 1) * fit$stan_data$D)
  expect_true(all(pit_df$pit >= 0 & pit_df$pit <= 1))
  expect_equal(unique(pit_df$time), attr(fit$stan_data, "time_values")[-1])
})

test_that("pit_values returns correct structure for dcvar fit", {
  skip_if_no_rstan()
  fit <- get_dcvar_fit()
  pit_df <- pit_values(fit)

  expect_s3_class(pit_df, "data.frame")
  expect_named(pit_df, c("time", "variable", "pit"))
  expect_true(all(pit_df$pit >= 0 & pit_df$pit <= 1))
  expect_equal(unique(pit_df$time), attr(fit$stan_data, "time_values")[-1])
})

test_that("pit_values supports gamma margins", {
  skip_if_no_rstan()
  fit <- get_constant_gamma_fit()
  pit_df <- pit_values(fit)

  expect_s3_class(pit_df, "data.frame")
  expect_named(pit_df, c("time", "variable", "pit"))
  expect_true(all(pit_df$pit >= 0 & pit_df$pit <= 1))
})

test_that("pit_values supports skew-normal margins", {
  skip_if_no_rstan()
  skip_if_not_installed("sn")
  fit <- get_constant_skew_normal_fit()
  pit_df <- pit_values(fit)

  expect_s3_class(pit_df, "data.frame")
  expect_named(pit_df, c("time", "variable", "pit"))
  expect_true(all(pit_df$pit >= 0 & pit_df$pit <= 1))
})

test_that("pit_test returns KS test results", {
  skip_if_no_rstan()
  fit <- get_constant_fit()
  ks_df <- pit_test(fit)

  expect_s3_class(ks_df, "data.frame")
  expect_named(ks_df, c("variable", "ks_statistic", "p_value", "n"))
  expect_equal(nrow(ks_df), 2)
  expect_true(all(ks_df$ks_statistic >= 0))
  expect_true(all(ks_df$p_value >= 0 & ks_df$p_value <= 1))
})

test_that("plot_pit returns a ggplot object", {
  skip_if_no_rstan()
  fit <- get_constant_fit()
  p <- plot_pit(fit)

  expect_s3_class(p, "ggplot")
})

test_that("pit plot dispatch works via plot(fit, type = 'pit')", {
  skip_if_no_rstan()
  fit <- get_constant_fit()
  p <- plot(fit, type = "pit")

  expect_s3_class(p, "ggplot")
})

test_that("pit_values.default errors for unsupported objects", {
  expect_error(pit_values(list()), class = "rlang_error")
})

test_that("pit_test.default errors for unsupported objects", {
  expect_error(pit_test(list()), class = "rlang_error")
})

test_that("pit_values honor preserved time values", {
  skip_if_no_rstan()

  fit <- get_constant_fit()
  attr(fit$stan_data, "time_values") <- seq.Date(
    as.Date("2022-01-01"),
    by = "week",
    length.out = fit$stan_data$n_time
  )
  pit_df <- pit_values(fit)

  expect_equal(unique(pit_df$time), attr(fit$stan_data, "time_values")[-1])
})

# ============================================================================
# Data Simulation
# ============================================================================

#' Internal: validate a positive finite scalar for simulation inputs
#' @noRd
.simulate_validate_positive_scalar <- function(x, arg_name) {
  if (!is.numeric(x) || length(x) != 1L || !is.finite(x) || x <= 0) {
    cli_abort("{.arg {arg_name}} must be a single positive finite numeric value.")
  }
}

#' Internal: validate a finite numeric vector for simulation inputs
#' @noRd
.simulate_validate_numeric_vector <- function(x, arg_name) {
  if (!is.numeric(x) || length(x) == 0L || any(!is.finite(x))) {
    cli_abort("{.arg {arg_name}} must be a non-empty finite numeric vector.")
  }
}

#' Internal: normalise a (possibly time-varying) trajectory specification
#'
#' Accepts an `(n_time - 1) x k` matrix, a list of `k` length-`(n_time - 1)`
#' vectors (so the `rho_*` generators can be reused per column), or a constant
#' specification (`k`-vector, or `2 x 2` matrix when `k = 4`, expanded over
#' time). Returns an `(n_time - 1) x k` matrix. For `k = 4` the column order
#' is row-major: phi11, phi12, phi21, phi22.
#' @noRd
.tv_resolve_trajectory <- function(x, n_time, k, arg_name) {
  n_eff <- n_time - 1L

  if (is.list(x)) {
    if (length(x) != k) {
      cli_abort("{.arg {arg_name}} as a list must have {.val {k}} elements, got {.val {length(x)}}.")
    }
    lens <- lengths(x)
    if (!all(lens == n_eff)) {
      cli_abort("Each element of {.arg {arg_name}} must have length {.val {n_eff}} (n_time - 1).")
    }
    x <- do.call(cbind, x)
  } else if (is.matrix(x)) {
    if (k == 4L && all(dim(x) == c(2L, 2L))) {
      # Constant 2x2 Phi, expanded row-major over time
      x <- matrix(rep(c(t(x)), each = n_eff), n_eff, k)
    } else if (!all(dim(x) == c(n_eff, k))) {
      cli_abort(c(
        "{.arg {arg_name}} must be a {.val {n_eff}} x {.val {k}} matrix.",
        "i" = "Rows follow the transitions 2..n_time; for Phi the columns are phi11, phi12, phi21, phi22."
      ))
    }
  } else if (is.numeric(x) && length(x) == k) {
    x <- matrix(rep(x, each = n_eff), n_eff, k)
  } else {
    cli_abort(c(
      "{.arg {arg_name}} must be a {.val {n_eff}} x {.val {k}} matrix, a list of {.val {k}} length-{.val {n_eff}} vectors, or a constant length-{.val {k}} specification.",
      "i" = "For Phi a constant {.val 2 x 2} matrix is also accepted."
    ))
  }

  x <- unname(as.matrix(x))
  if (!is.numeric(x) || any(!is.finite(x))) {
    cli_abort("{.arg {arg_name}} must contain only finite numeric values.")
  }
  x
}

#' Internal: per-t spectral radius of a row-major (phi11, phi12, phi21, phi22) path
#' @noRd
.tv_spectral_radius <- function(phi_mat) {
  apply(phi_mat, 1, function(p) {
    tr <- p[1] + p[4]
    det_phi <- p[1] * p[4] - p[2] * p[3]
    disc <- tr^2 - 4 * det_phi
    if (disc >= 0) {
      max(abs(0.5 * (tr + sqrt(disc))), abs(0.5 * (tr - sqrt(disc))))
    } else {
      sqrt(abs(det_phi))
    }
  })
}

#' Simulate data from a copula VAR(1) model
#'
#' Generates bivariate time series data with correlated innovations
#' driven by a specified rho trajectory.
#'
#' @param n_time Number of time points.
#' @param rho_trajectory Numeric vector of length `n_time - 1` specifying the
#'   correlation at each time step. Use [rho_constant()], [rho_decreasing()],
#'   etc.
#' @param mu Intercept vector of length 2 (default: `c(0, 0)`).
#' @param Phi VAR(1) coefficient matrix, 2x2 (default:
#'   `matrix(c(0.3, 0.1, 0.1, 0.3), 2, 2)`).
#' @param sigma_eps Innovation standard deviations, length 2 (default:
#'   `c(1, 1)`). Used for normal margins.
#' @param margins Marginal family. Either a single string applied to both
#'   variables, or a length-2 character vector for per-variable (mixed) margins,
#'   e.g. `c("normal", "exponential")`. Each entry is one of `"normal"`
#'   (default), `"exponential"`, `"skew_normal"`, or `"gamma"`.
#' @param skew_direction Length-2 integer vector of +1/-1. Required whenever any
#'   dimension uses an `"exponential"` or `"gamma"` margin; only those
#'   dimensions consult it.
#' @param skew_params Named list of margin-specific parameters. `alpha`
#'   (length-2 vector of skew-normal shape params) is used by skew-normal
#'   dimensions; `shape` (scalar gamma shape parameter) is used by gamma
#'   dimensions. Both may be supplied together for mixed margins.
#' @param phi_trajectory Optional time-varying VAR coefficient paths for data
#'   matching `dcvar(tv_phi = TRUE)`: an `(n_time - 1) x 4` matrix with
#'   columns in row-major order (phi11, phi12, phi21, phi22), a list of 4
#'   length-`(n_time - 1)` vectors (the `rho_*` trajectory generators can be
#'   reused per coefficient), or a constant `2 x 2` matrix. Mutually exclusive
#'   with `Phi`. `NULL` (default) keeps the constant `Phi`.
#' @param sigma_trajectory Optional time-varying residual scale paths for data
#'   matching `dcvar(tv_sigma = TRUE)`: an `(n_time - 1) x 2` positive matrix,
#'   a list of 2 length-`(n_time - 1)` vectors, or a constant length-2 vector.
#'   The scale is each family's natural scale (innovation SD for normal,
#'   residual SD for skew-normal, `sigma_exp` / `sigma_gam` for
#'   exponential / gamma). A non-constant path on an exponential or gamma
#'   dimension requires `tv_sigma_k` (soft-barrier generative model). Mutually
#'   exclusive with `sigma_eps`. `NULL` (default) keeps constant scales.
#' @param tv_sigma_k Soft-barrier sharpness for time-varying exponential/gamma
#'   scales. Supply the same value as the intended `dcvar(tv_sigma = TRUE,
#'   tv_sigma_k = ...)` fit so the simulated data matches that likelihood.
#'   `NULL` (default) uses the exact affine shifted margin and requires
#'   constant exp/gamma scales.
#' @param seed Random seed for reproducibility.
#'
#' @return A named list with:
#'   - `Y`: `n_time x 2` observation matrix
#'   - `Y_df`: data frame with columns `time`, `y1`, `y2` (ready for
#'     [dcvar()])
#'   - `true_params`: list of true parameter values. These are on the raw
#'     data scale; the fitting functions standardize by default, so round-trip
#'     comparisons of `mu`, `Phi`, and `sigma_eps` require fitting with
#'     `standardize = FALSE` (the `rho` trajectory is scale-invariant).
#'     Without `sigma_trajectory`, exponential and gamma dimensions are
#'     simulated with unit-SD standardized innovations, so their implied true
#'     scale (`sigma_exp`/`sigma_gam`) is 1. With trajectories supplied,
#'     `Phi` is the `(n_time - 1) x 4` coefficient path and `sigma` the
#'     `(n_time - 1) x 2` scale path.
#' @export
#'
#' @examples
#' sim <- simulate_dcvar(n_time = 100, rho_trajectory = rho_decreasing(100))
#' head(sim$Y_df)
#' plot(sim$true_params$rho, type = "l")
simulate_dcvar <- function(n_time,
                           rho_trajectory,
                           mu = c(0, 0),
                           Phi = matrix(c(0.3, 0.1, 0.1, 0.3), 2, 2),
                           sigma_eps = c(1, 1),
                           margins = "normal",
                           skew_direction = NULL,
                           skew_params = NULL,
                           phi_trajectory = NULL,
                           sigma_trajectory = NULL,
                           tv_sigma_k = NULL,
                           seed = NULL) {
  if (!is.null(seed)) set.seed(seed)
  if (!is.numeric(n_time) || length(n_time) != 1 || n_time != as.integer(n_time) || n_time < 2) {
    cli_abort("{.arg n_time} must be an integer >= 2, got {.val {n_time}}.")
  }

  if (length(rho_trajectory) != n_time - 1L) {
    cli_abort(
      "{.arg rho_trajectory} must have length n_time - 1 = {n_time - 1}, got {length(rho_trajectory)}."
    )
  }
  if (any(abs(rho_trajectory) > 1)) {
    cli_abort("{.arg rho_trajectory} values must be in [-1, 1].")
  }
  if (!is.null(tv_sigma_k)) {
    .simulate_validate_positive_scalar(tv_sigma_k, "tv_sigma_k")
  }

  D <- length(mu)
  if (D != 2) {
    cli_abort(
      "{.fun simulate_dcvar} currently supports bivariate (D = 2) models only, got D = {D}."
    )
  }
  if (!is.null(phi_trajectory) && !missing(Phi)) {
    cli_abort("Supply either {.arg Phi} (constant) or {.arg phi_trajectory} (time-varying), not both.")
  }
  if (!is.null(sigma_trajectory) && !missing(sigma_eps)) {
    cli_abort("Supply either {.arg sigma_eps} (constant) or {.arg sigma_trajectory} (time-varying), not both.")
  }
  if (!is.matrix(Phi) || !all(dim(Phi) == c(D, D))) {
    cli_abort("{.arg Phi} must be a {D}x{D} matrix.")
  }

  margins <- .normalize_margins_spec(margins)
  .validate_margins(margins, skew_direction)
  margins_vec <- if (length(margins) == 1L) rep(margins, D) else margins

  # Resolve the (possibly constant) per-transition parameter paths. Row t
  # governs the transition Y[t] -> Y[t+1], matching the Stan models' eps[t].
  phi_mat <- if (is.null(phi_trajectory)) {
    .tv_resolve_trajectory(Phi, n_time, 4L, "Phi")
  } else {
    .tv_resolve_trajectory(phi_trajectory, n_time, 4L, "phi_trajectory")
  }
  sr <- .tv_spectral_radius(phi_mat)
  if (any(sr >= 1)) {
    cli_warn(c(
      "{sum(sr >= 1)} time point{?s} of the VAR coefficient path {?is/are} nonstationary (spectral radius >= 1).",
      "i" = "Brief excursions are fine; persistent ones make the series explode."
    ))
  }

  scale_mat <- if (is.null(sigma_trajectory)) {
    NULL
  } else {
    m <- .tv_resolve_trajectory(sigma_trajectory, n_time, D, "sigma_trajectory")
    if (any(m <= 0)) {
      cli_abort("{.arg sigma_trajectory} values must be positive.")
    }
    # A non-constant scale path on a shifted exp/gamma dimension requires the
    # soft-barrier generative model, which the user opts into with tv_sigma_k
    # (matching dcvar(tv_sigma = TRUE, tv_sigma_k = ...)).
    if (is.null(tv_sigma_k)) {
      for (d in seq_len(D)) {
        if (margins_vec[d] %in% c("exponential", "gamma") && length(unique(m[, d])) > 1L) {
          cli_abort(c(
            "{.arg sigma_trajectory} is not constant on dimension {d} ({.val {margins_vec[d]}} margin).",
            "i" = "Set {.arg tv_sigma_k} (e.g. the value passed to {.code dcvar(tv_sigma = TRUE)}) to simulate from the soft-barrier model, or keep the scale constant."
          ))
        }
      }
    }
    m
  }

  if (!is.null(skew_params) && !is.list(skew_params)) {
    cli_abort("{.arg skew_params} must be a list.")
  }
  skew_params <- if (is.list(skew_params)) skew_params else list()

  # Normal dimensions use sigma_eps
  if (any(margins_vec == "normal")) {
    .simulate_validate_numeric_vector(sigma_eps, "sigma_eps")
    if (length(sigma_eps) != D) {
      cli_abort("{.arg sigma_eps} must have length {.val {D}}, got {.val {length(sigma_eps)}}.")
    }
    if (any(sigma_eps <= 0)) {
      cli_abort("{.arg sigma_eps} values must be positive.")
    }
  }
  # Skew-normal dimensions use a per-dimension alpha
  if (any(margins_vec == "skew_normal")) {
    alpha <- skew_params$alpha %||% rep(0, D)
    .simulate_validate_numeric_vector(alpha, "skew_params$alpha")
    if (length(alpha) != D) {
      cli_abort("{.arg skew_params$alpha} must have length {.val {D}}, got {.val {length(alpha)}}.")
    }
    skew_params$alpha <- alpha
  }
  # Gamma dimensions use a shared shape
  if (any(margins_vec == "gamma")) {
    gamma_shape <- skew_params$shape %||% 1
    .simulate_validate_positive_scalar(gamma_shape, "skew_params$shape")
    skew_params$shape <- gamma_shape
  }

  # Constant per-dimension scales used when no sigma_trajectory is supplied:
  # the innovation SD for normal dims, unit scale otherwise.
  base_scales <- rep(1, D)
  is_normal <- margins_vec == "normal"
  if (any(is_normal)) base_scales[is_normal] <- sigma_eps[is_normal]

  Y <- matrix(0, n_time, D)
  Y[1, ] <- mu

  for (time_index in 2:n_time) {
    t_eff <- time_index - 1L
    rho_t <- rho_trajectory[t_eff]

    # Generate copula uniforms via Gaussian copula
    L <- matrix(c(1, rho_t, 0, sqrt(1 - rho_t^2)), 2, 2)
    z <- rnorm(D)
    w <- L %*% z  # correlated standard normals

    # Transform through marginal quantiles with the per-transition scales.
    # A supplied tv_sigma_k routes exp/gamma dims through the soft-barrier
    # generative model (matching dcvar(tv_sigma = TRUE)).
    scales_t <- if (is.null(scale_mat)) base_scales else scale_mat[t_eff, ]
    eps <- .sim_marginal_quantile_scaled(w, margins_vec, scales_t, skew_direction,
                                         skew_params, barrier_k = tv_sigma_k)

    # VAR(1) update with the per-transition coefficients (byrow: the path
    # columns are row-major phi11, phi12, phi21, phi22)
    Phi_t <- matrix(phi_mat[t_eff, ], 2, 2, byrow = TRUE)
    Y[time_index, ] <- mu + Phi_t %*% (Y[time_index - 1L, ] - mu) + eps
  }

  Y_df <- data.frame(
    time = seq_len(n_time),
    y1 = Y[, 1],
    y2 = Y[, 2]
  )

  true_params <- list(
    rho = rho_trajectory,
    Phi = if (is.null(phi_trajectory)) {
      Phi
    } else {
      colnames(phi_mat) <- c("phi11", "phi12", "phi21", "phi22")
      phi_mat
    },
    mu = mu,
    margins = margins
  )

  # Add margin-specific true params for every family present (independent
  # checks so mixed margins record each family's parameters).
  if (!is.null(scale_mat)) {
    colnames(scale_mat) <- c("y1", "y2")
    true_params$sigma <- scale_mat
  } else if (any(margins_vec == "normal")) {
    true_params$sigma_eps <- sigma_eps
  }
  if (any(margins_vec %in% c("exponential", "gamma"))) {
    true_params$skew_direction <- skew_direction
  }
  if (any(margins_vec %in% c("skew_normal", "gamma"))) {
    true_params$skew_params <- skew_params
  }

  list(
    Y = Y,
    Y_df = Y_df,
    true_params = true_params
  )
}


#' Internal: scale-aware marginal quantile transform
#'
#' Core shared by the constant-parameter and time-varying simulators. `scales`
#' is the per-dimension scale on each family's natural reporting scale: the
#' innovation SD for normal dims, `sigma_exp` for exponential dims (mean and
#' SD of the shifted exponential), the SD (`sigma_gam`) for gamma dims, and
#' the residual SD for skew-normal dims. All residuals have mean zero by
#' construction.
#' @noRd
.sim_marginal_quantile_scaled <- function(w, margins_vec, scales, skew_direction, skew_params,
                                          barrier_k = NULL) {
  D <- length(w)
  eps <- numeric(D)
  # Inverse softplus, matching inst/stan/functions/softplus.stan
  inv_softplus_k <- function(y, k) log(expm1(k * y)) / k

  for (i in seq_len(D)) {
    fam <- margins_vec[[i]]
    s <- scales[i]
    if (identical(fam, "normal")) {
      # w[i] is already standard normal, just scale
      eps[i] <- w[i] * s
    } else if (identical(fam, "exponential")) {
      # Convert to uniform, then to a shifted exponential with mean/sd s.
      # The Stan likelihoods use u = 1 - F(x_shifted) for left-skewed dimensions,
      # so the uniform must be flipped before the quantile to keep the simulated
      # eps comonotone with the latent copula normal w.
      u <- stats::pnorm(w[i])
      if (skew_direction[i] < 0) u <- 1 - u
      x_shifted <- stats::qexp(u, rate = 1 / s)        # Exp(1/s) variate, mean s
      if (is.null(barrier_k)) {
        eps[i] <- skew_direction[i] * (x_shifted - s)  # exact affine shift
      } else {
        # Soft-barrier generative model (matches the tv_sigma Stan likelihood):
        # x_shifted = softplus_k(s + skew*eps)  =>  eps = skew*(invsoftplus(x) - s).
        # Floor away from 0 (matches the Stan eps_rep guard) so inv_softplus_k
        # stays in its y > 0 domain for an extreme lower-tail draw.
        eps[i] <- skew_direction[i] * (inv_softplus_k(max(x_shifted, 1e-12), barrier_k) - s)
      }
    } else if (identical(fam, "skew_normal")) {
      if (!requireNamespace("sn", quietly = TRUE)) {
        cli_abort("Package {.pkg sn} is required for skew-normal simulation.")
      }
      alpha_i <- (skew_params$alpha %||% rep(0, D))[i]
      delta <- alpha_i / sqrt(1 + alpha_i^2)
      omega_i <- s * sqrt(1 / (1 - 2 * delta^2 / pi))
      xi_i <- -omega_i * delta * sqrt(2 / pi)
      eps[i] <- sn::qsn(stats::pnorm(w[i]), xi = xi_i, omega = omega_i, alpha = alpha_i)
    } else if (identical(fam, "gamma")) {
      shape <- skew_params$shape %||% 1
      # Same uniform flip as the exponential branch (see comment there).
      # The gamma mean m = sqrt(shape) * s gives SD s; the mean shift keeps
      # E[eps] = 0 in the exact affine case.
      m <- sqrt(shape) * s
      u <- stats::pnorm(w[i])
      if (skew_direction[i] < 0) u <- 1 - u
      x_shifted <- stats::qgamma(u, shape = shape, rate = shape / m)  # Gamma, mean m
      if (is.null(barrier_k)) {
        eps[i] <- skew_direction[i] * (x_shifted - m)
      } else {
        eps[i] <- skew_direction[i] * (inv_softplus_k(max(x_shifted, 1e-12), barrier_k) - m)
      }
    } else {
      cli_abort("Unknown margin type: {.val {fam}}")
    }
  }

  eps
}

# Regression tests for the copula orientation of simulated non-normal margins.
#
# The Stan likelihoods use u = 1 - F(x_shifted) on left-skewed dimensions
# (skew_direction = -1), so simulated data must imply the recorded rho -- not
# its negative -- when residuals are pushed through that convention. Before
# the fix, any configuration with exactly one negative skew_direction silently
# recovered -rho.

# Map simulated residuals to copula-scale z-scores using the likelihood's
# PIT convention (standardized margins: sigma_exp = 1, sd-1 gamma).
likelihood_copula_z <- function(eps, margin, skew, shape = 1) {
  u <- switch(margin,
    exponential = stats::pexp(1 + skew * eps, rate = 1),
    gamma = stats::pgamma(sqrt(shape) + skew * eps, shape = shape, rate = sqrt(shape))
  )
  if (skew < 0) u <- 1 - u
  stats::qnorm(u)
}

test_that(".sim_marginal_quantile_scaled preserves copula orientation for all skew_direction combinations", {
  set.seed(42)
  n <- 20000
  rho <- 0.6
  w1 <- rnorm(n)
  w2 <- rho * w1 + sqrt(1 - rho^2) * rnorm(n)

  for (margin in c("exponential", "gamma")) {
    shape <- if (margin == "gamma") 2 else 1
    skew_params <- if (margin == "gamma") list(shape = shape) else list()
    for (skew in list(c(1, 1), c(1, -1), c(-1, 1), c(-1, -1))) {
      eps <- vapply(
        seq_len(n),
        function(i) {
          .sim_marginal_quantile_scaled(c(w1[i], w2[i]), rep(margin, 2), c(1, 1), skew, skew_params)
        },
        numeric(2)
      )
      z1 <- likelihood_copula_z(eps[1, ], margin, skew[1], shape)
      z2 <- likelihood_copula_z(eps[2, ], margin, skew[2], shape)
      expect_equal(
        cor(z1, z2), rho,
        tolerance = 0.05,
        info = sprintf("margin=%s skew=(%d,%d)", margin, skew[1], skew[2])
      )
    }
  }
})

test_that("simulate_dcvar data implies the recorded rho under the likelihood convention", {
  n_time <- 4000
  sim <- simulate_dcvar(
    n_time = n_time,
    rho_trajectory = rho_constant(n_time, 0.6),
    margins = "exponential",
    skew_direction = c(1, -1),
    seed = 7
  )
  tp <- sim$true_params
  Y <- sim$Y
  n_eff <- nrow(Y) - 1L
  eps <- matrix(NA_real_, n_eff, 2L)
  for (t in seq_len(n_eff)) {
    eps[t, ] <- Y[t + 1L, ] - (tp$mu + tp$Phi %*% (Y[t, ] - tp$mu))
  }

  z1 <- likelihood_copula_z(eps[, 1], "exponential", 1)
  z2 <- likelihood_copula_z(eps[, 2], "exponential", -1)
  expect_equal(cor(z1, z2), 0.6, tolerance = 0.05)
})

test_that("simulate_dcvar_sem exponential innovations imply the recorded rho", {
  sigma_exp <- c(1.5, 2)
  sim <- simulate_dcvar_sem(
    n_time = 4000,
    margins = "exponential",
    sigma_exp = sigma_exp,
    skew_direction = c(1, -1),
    rho = 0.6,
    seed = 11
  )
  zeta <- sim$innovations

  pit_sem <- function(zeta_i, sigma_exp_i, skew) {
    u <- stats::pexp(sigma_exp_i + skew * zeta_i, rate = 1 / sigma_exp_i)
    if (skew < 0) u <- 1 - u
    u
  }
  z1 <- stats::qnorm(pit_sem(zeta[, 1], sigma_exp[1], 1))
  z2 <- stats::qnorm(pit_sem(zeta[, 2], sigma_exp[2], -1))
  expect_equal(cor(z1, z2), 0.6, tolerance = 0.05)
})

# ============================================================================
# Time-varying simulator: trajectory resolution, Phi reshaping, and the
# copula orientation under per-t scales (the bug class fixed in 0.4.0).
# ============================================================================

test_that(".tv_resolve_trajectory accepts equivalent input forms", {
  n <- 21
  m <- cbind(rep(0.3, 20), rep(0.1, 20), rep(-0.2, 20), rep(0.4, 20))

  from_matrix <- .tv_resolve_trajectory(m, n, 4L, "phi_trajectory")
  from_list <- .tv_resolve_trajectory(
    list(rep(0.3, 20), rep(0.1, 20), rep(-0.2, 20), rep(0.4, 20)), n, 4L, "phi_trajectory"
  )
  from_const <- .tv_resolve_trajectory(
    matrix(c(0.3, -0.2, 0.1, 0.4), 2, 2), n, 4L, "phi_trajectory"
  )
  expect_equal(from_matrix, from_list)
  # Constant 2x2 expands row-major: (phi11, phi12, phi21, phi22)
  expect_equal(from_matrix, from_const)

  expect_error(.tv_resolve_trajectory(m[, 1:3], n, 4L, "phi_trajectory"), "matrix")
  expect_error(.tv_resolve_trajectory(list(rep(0.3, 19)), n, 4L, "phi_trajectory"), "4")
  expect_error(.tv_resolve_trajectory(cbind(rep(0.3, 20), rep(NA, 20)), n, 2L, "sigma_trajectory"), "finite")
})

test_that("simulate_dcvar with a constant phi_trajectory reproduces the constant-Phi path", {
  Phi <- matrix(c(0.3, 0.1, 0.1, 0.3), 2, 2)
  s_const <- simulate_dcvar(30, rho_constant(30, 0.4), seed = 11)
  s_traj <- simulate_dcvar(30, rho_constant(30, 0.4), seed = 11, phi_trajectory = Phi)
  expect_equal(s_const$Y, s_traj$Y)
})

test_that("asymmetric Phi paths are applied in row-major order", {
  # phi12 = 0.4, phi21 = -0.2: a transposed reshape would swap them.
  n <- 4000
  phi_const <- matrix(c(0.2, 0.4, -0.2, 0.3), 2, 2, byrow = TRUE)
  sim <- simulate_dcvar(n, rho_constant(n, 0), seed = 21, phi_trajectory = phi_const)
  Y <- sim$Y

  # OLS VAR(1) recovery of the coefficient matrix
  X <- Y[-n, ]
  resp <- Y[-1, ]
  bhat <- solve(crossprod(X), crossprod(X, resp))  # 2x2: column d = coefficients of y_d
  expect_equal(unname(bhat[2, 1]), 0.4, tolerance = 0.1)   # phi12: y2's effect on y1
  expect_equal(unname(bhat[1, 2]), -0.2, tolerance = 0.1)  # phi21: y1's effect on y2
})

test_that("exp/gamma dimensions reject non-constant scale paths", {
  expect_error(
    simulate_dcvar(30, rho_constant(30, 0.4),
                   margins = c("normal", "exponential"),
                   skew_direction = c(1, 1),
                   sigma_trajectory = cbind(rep(1, 29), seq(0.5, 2, length.out = 29))),
    "constant on dimension 2"
  )
  # Constant non-unit scales on exp dims are allowed
  sim <- simulate_dcvar(30, rho_constant(30, 0.4),
                        margins = c("normal", "exponential"),
                        skew_direction = c(1, 1),
                        sigma_trajectory = cbind(seq(0.5, 2, length.out = 29), rep(1.5, 29)))
  expect_equal(unname(sim$true_params$sigma[1, ]), c(0.5, 1.5))
})

# Map simulated residuals to copula-scale z-scores using the likelihood's
# per-t PIT convention (u = F(scale + skew * eps), flipped on left-skew dims).
.tv_likelihood_copula_z <- function(eps, margin, skew, scale, shape = 1) {
  u <- switch(margin,
    normal = stats::pnorm(eps / scale),
    exponential = stats::pexp(scale + skew * eps, rate = 1 / scale),
    gamma = stats::pgamma(sqrt(shape) * scale + skew * eps,
                          shape = shape, rate = sqrt(shape) / scale)
  )
  if (margin != "normal" && skew < 0) u <- 1 - u
  stats::qnorm(u)
}

test_that("TV simulation preserves the copula orientation for every skew combination", {
  n <- 20000
  rho <- 0.6
  scale_path <- seq(0.5, 2, length.out = n - 1)  # non-constant scale on the normal dim

  for (margin2 in c("exponential", "gamma")) {
    shape <- if (margin2 == "gamma") 2 else 1
    skew_params <- if (margin2 == "gamma") list(shape = shape) else NULL
    for (skew2 in c(1, -1)) {
      sim <- simulate_dcvar(
        n, rho_constant(n, rho),
        mu = c(0, 0),
        margins = c("normal", margin2),
        skew_direction = c(1, skew2),
        skew_params = skew_params,
        phi_trajectory = matrix(c(0.2, 0.4, -0.2, 0.3), 2, 2, byrow = TRUE),
        sigma_trajectory = cbind(scale_path, rep(1.5, n - 1)),
        seed = 100 + skew2 + (margin2 == "gamma") * 10
      )

      # Recompute the residuals with the TRUE per-t coefficients
      tp <- sim$true_params
      Y <- sim$Y
      eps <- matrix(NA_real_, n - 1, 2)
      for (t in seq_len(n - 1)) {
        Phi_t <- matrix(tp$Phi[t, ], 2, 2, byrow = TRUE)
        eps[t, ] <- Y[t + 1, ] - (tp$mu + Phi_t %*% (Y[t, ] - tp$mu))
      }

      # Push through the likelihood PIT with the TRUE per-t scales
      z1 <- .tv_likelihood_copula_z(eps[, 1], "normal", 1, tp$sigma[, 1])
      z2 <- .tv_likelihood_copula_z(eps[, 2], margin2, skew2, tp$sigma[, 2], shape)
      expect_equal(
        cor(z1, z2), rho,
        tolerance = 0.05,
        info = sprintf("margin2=%s skew2=%d", margin2, skew2)
      )
    }
  }
})

test_that("TV simulation with time-varying rho recovers the rho path shape", {
  n <- 30000
  rho_path <- rho_decreasing(n, 0.7, 0.1)
  sim <- simulate_dcvar(
    n, rho_path,
    margins = "normal",
    phi_trajectory = list(rho_constant(n, 0.3), rho_constant(n, 0.1),
                          rho_constant(n, 0.1), rho_constant(n, 0.3)),
    sigma_trajectory = cbind(seq(1, 2, length.out = n - 1), seq(2, 1, length.out = n - 1)),
    seed = 77
  )
  tp <- sim$true_params
  Y <- sim$Y
  eps <- matrix(NA_real_, n - 1, 2)
  for (t in seq_len(n - 1)) {
    Phi_t <- matrix(tp$Phi[t, ], 2, 2, byrow = TRUE)
    eps[t, ] <- Y[t + 1, ] - (tp$mu + Phi_t %*% (Y[t, ] - tp$mu))
  }
  z <- eps / tp$sigma

  # Windowed empirical correlations should track the declining rho path
  early <- cor(z[1:5000, 1], z[1:5000, 2])
  late <- cor(z[(n - 5001):(n - 1), 1], z[(n - 5001):(n - 1), 2])
  expect_equal(early, mean(rho_path[1:5000]), tolerance = 0.07)
  expect_equal(late, mean(rho_path[(n - 5001):(n - 1)]), tolerance = 0.07)
  expect_gt(early, late)
})

# Soft-barrier generative model for time-varying exponential/gamma scales.

test_that("non-constant exp/gamma scale needs tv_sigma_k, then simulates", {
  n <- 60
  # Without tv_sigma_k a non-constant exp scale is rejected
  expect_error(
    simulate_dcvar(n, rho_constant(n, 0.4),
                   margins = c("normal", "exponential"),
                   skew_direction = c(1, 1),
                   sigma_trajectory = cbind(rep(1, n - 1), seq(0.6, 1.8, length.out = n - 1))),
    "tv_sigma_k"
  )
  # With tv_sigma_k it is allowed
  sim <- simulate_dcvar(n, rho_constant(n, 0.4),
                        margins = c("normal", "exponential"),
                        skew_direction = c(1, 1),
                        sigma_trajectory = cbind(rep(1, n - 1), seq(0.6, 1.8, length.out = n - 1)),
                        tv_sigma_k = 8, seed = 5)
  expect_equal(nrow(sim$Y), n)
  expect_equal(unname(sim$true_params$sigma[, 2]), seq(0.6, 1.8, length.out = n - 1))
})

test_that("soft-barrier simulation preserves copula orientation under a TV exp scale", {
  n <- 20000
  rho <- 0.6
  k <- 8
  softplus <- function(a) log1p(exp(k * a)) / k
  scale_path <- seq(0.6, 1.8, length.out = n - 1)

  for (skew2 in c(1, -1)) {
    sim <- simulate_dcvar(
      n, rho_constant(n, rho),
      margins = c("normal", "exponential"),
      skew_direction = c(1, skew2),
      phi_trajectory = matrix(c(0.2, 0.4, -0.2, 0.3), 2, 2, byrow = TRUE),
      sigma_trajectory = cbind(rep(1, n - 1), scale_path),
      tv_sigma_k = k,
      seed = 200 + skew2
    )
    tp <- sim$true_params
    Y <- sim$Y
    eps <- matrix(NA_real_, n - 1, 2)
    for (t in seq_len(n - 1)) {
      P <- matrix(tp$Phi[t, ], 2, 2, byrow = TRUE)
      eps[t, ] <- Y[t + 1, ] - (tp$mu + P %*% (Y[t, ] - tp$mu))
    }
    z1 <- stats::qnorm(stats::pnorm(eps[, 1]))            # normal dim, scale 1
    m <- scale_path                                       # exp scale path
    u2 <- stats::pexp(softplus(m + skew2 * eps[, 2]), rate = 1 / m)
    if (skew2 < 0) u2 <- 1 - u2
    z2 <- stats::qnorm(u2)
    expect_equal(cor(z1, z2), rho, tolerance = 0.05,
                 info = sprintf("skew2=%d", skew2))
  }
})

test_that("soft-barrier exp/gamma change-of-variables density is normalized (Jacobian present and correct)", {
  # The eps-density of the soft-barrier margin is
  #   exp/gamma_lpdf(softplus_k(m + skew*eps) | rate) + log_inv_logit(k*(m + skew*eps)).
  # A missing, doubled, or sign-flipped Jacobian would make this integrate to
  # something other than 1. This is the falsifiable guard the orientation
  # round-trips (which only check the copula rank correlation) cannot provide.
  k <- 8
  softplus <- function(a) log1p(exp(k * a)) / k
  log_jac <- function(a) stats::plogis(k * a, log.p = TRUE)   # log_inv_logit(k*a)

  exp_eps_density <- function(eps, m, skew) {
    arg <- m + skew * eps
    exp(stats::dexp(softplus(arg), rate = 1 / m, log = TRUE) + log_jac(arg))
  }
  gam_eps_density <- function(eps, m, skew, shape) {
    arg <- m + skew * eps
    exp(stats::dgamma(softplus(arg), shape = shape, rate = shape / m, log = TRUE) + log_jac(arg))
  }

  for (skew in c(1, -1)) {
    for (m in c(0.7, 1.3)) {
      area_exp <- stats::integrate(exp_eps_density, -Inf, Inf, m = m, skew = skew,
                                   rel.tol = 1e-8)$value
      expect_equal(area_exp, 1, tolerance = 1e-4,
                   info = sprintf("exp m=%.1f skew=%d", m, skew))
      area_gam <- stats::integrate(gam_eps_density, -Inf, Inf, m = m, skew = skew,
                                   shape = 2, rel.tol = 1e-8)$value
      expect_equal(area_gam, 1, tolerance = 1e-4,
                   info = sprintf("gamma m=%.1f skew=%d", m, skew))
    }
  }
})

test_that("the soft-barrier log-Jacobian is the exact transform Jacobian", {
  # log_inv_logit(k*a) must equal log|d softplus_k(a)/da|; a sign flip or a
  # constant offset (the easy regressions) would break this finite-difference
  # identity that the Stan model relies on.
  k <- 8
  softplus <- function(a) log1p(exp(k * a)) / k
  a <- c(-2, -0.5, 0, 0.5, 2, 5)
  num <- (softplus(a + 1e-6) - softplus(a - 1e-6)) / 2e-6
  ana <- exp(stats::plogis(k * a, log.p = TRUE))
  expect_equal(ana, num, tolerance = 1e-5)
})

# ============================================================================
# Extraction Functions
# ============================================================================

#' Extract the rho trajectory with credible intervals
#'
#' Returns a data frame with the posterior mean, SD, and quantiles of the
#' time-varying correlation at each time point.
#'
#' @param object A fitted model object (`dcvar_fit`, `dcvar_covariate_fit`,
#'   `dcvar_hmm_fit`, `dcvar_constant_fit`, `dcvar_multilevel_fit`, or
#'   `dcvar_sem_fit`).
#' @param probs Numeric vector of quantile probabilities (default: `c(0.025, 0.1, 0.5, 0.9, 0.975)`).
#' @param ... Additional arguments (unused).
#'
#' @return A data frame with columns `time`, `mean`, `sd`, and one column per
#'   quantile (e.g., `q2.5`, `q10`, `q50`, `q90`, `q97.5`). For
#'   `dcvar_constant_fit` objects, the constant rho is expanded to all `n_time - 1`
#'   time points for consistency with the time-varying models.
#'
#' @seealso [plot_rho()] to visualise the trajectory,
#'   [interpret_rho_trajectory()] for a text-based summary,
#'   [var_params()] for VAR parameter extraction.
#' @export
rho_trajectory <- function(object, ...) {
  UseMethod("rho_trajectory")
}

#' @rdname rho_trajectory
#' @export
rho_trajectory.default <- function(object, ...) {
  cli_abort("{.fun rho_trajectory} is not defined for objects of class {.cls {class(object)[[1]]}}.")
}

#' Extract a unified dependence summary
#'
#' Returns posterior summaries for Kendall's tau, using the fitted copula
#' family to transform the model-specific dependence parameter. For Gaussian
#' copulas, `tau = 2 / pi * asin(rho)`. For Clayton copulas,
#' `tau = theta / (theta + 2)`.
#'
#' @param object A fitted model object.
#' @param probs Numeric vector of quantile probabilities.
#' @param ... Additional arguments (unused).
#'
#' @return A data frame with columns `time`, `mean`, `sd`, and one column per
#'   requested quantile.
#' @export
dependence_summary <- function(object, ...) {
  UseMethod("dependence_summary")
}

#' @rdname dependence_summary
#' @export
dependence_summary.default <- function(object, ...) {
  cli_abort("{.fun dependence_summary} is not defined for objects of class {.cls {class(object)[[1]]}}.")
}

#' Internal helper: recover observed time values from Stan data
#' @noRd
.observed_time_values <- function(stan_data, drop_first = FALSE) {
  time_values <- attr(stan_data, "time_values")
  if (is.null(time_values)) {
    time_values <- seq_len(stan_data$n_time)
  }

  if (drop_first) {
    time_values[-1]
  } else {
    time_values
  }
}

#' Internal helper: summarise time-varying rho draws into a data frame
#' @noRd
.summarise_rho_draws <- function(rho_draws, probs, time_values = NULL) {
  .validate_probs(probs)

  n_time_eff <- ncol(rho_draws)
  if (is.null(time_values)) {
    time_values <- 2:(n_time_eff + 1)
  }
  if (length(time_values) != n_time_eff) {
    cli_abort("Time axis length does not match rho draw length.")
  }

  rho_summary <- data.frame(
    time = time_values,
    mean = colMeans(rho_draws),
    sd = apply(rho_draws, 2, sd)
  )

  quantiles <- apply(rho_draws, 2, quantile, probs = probs)
  if (is.null(dim(quantiles))) {
    quantiles <- matrix(quantiles, nrow = 1)
  }
  for (i in seq_along(probs)) {
    rho_summary[[paste0("q", probs[i] * 100)]] <- quantiles[i, ]
  }

  rho_summary
}

#' @rdname rho_trajectory
#' @export
rho_trajectory.dcvar_fit <- function(object, probs = c(0.025, 0.1, 0.5, 0.9, 0.975), ...) {
  rho_draws <- posterior::as_draws_matrix(.fit_draws(
    object$fit, "rho", backend = object$backend,
    required = .stan_output_group_pattern("rho"),
    required_type = "pattern",
    context = "rho_trajectory.dcvar_fit()",
    output_type = "transformed parameter group"
  ))
  .summarise_rho_draws(rho_draws, probs, .observed_time_values(object$stan_data, drop_first = TRUE))
}

#' @rdname dependence_summary
#' @export
dependence_summary.dcvar_fit <- function(object, probs = c(0.025, 0.1, 0.5, 0.9, 0.975), ...) {
  rho_draws <- posterior::as_draws_matrix(.fit_draws(
    object$fit, "rho", backend = object$backend,
    required = .stan_output_group_pattern("rho"),
    required_type = "pattern",
    context = "dependence_summary.dcvar_fit()",
    output_type = "transformed parameter group"
  ))
  tau_draws <- 2 / pi * asin(rho_draws)
  .summarise_rho_draws(tau_draws, probs, .observed_time_values(object$stan_data, drop_first = TRUE))
}

#' @rdname rho_trajectory
#' @export
rho_trajectory.dcvar_covariate_fit <- function(object, probs = c(0.025, 0.1, 0.5, 0.9, 0.975), ...) {
  rho_draws <- posterior::as_draws_matrix(.fit_draws(
    object$fit, "rho", backend = object$backend,
    required = .stan_output_group_pattern("rho"),
    required_type = "pattern",
    context = "rho_trajectory.dcvar_covariate_fit()",
    output_type = "transformed parameter group"
  ))
  .summarise_rho_draws(rho_draws, probs, .observed_time_values(object$stan_data, drop_first = TRUE))
}

#' @rdname dependence_summary
#' @export
dependence_summary.dcvar_covariate_fit <- function(object, probs = c(0.025, 0.1, 0.5, 0.9, 0.975), ...) {
  rho_draws <- posterior::as_draws_matrix(.fit_draws(
    object$fit, "rho", backend = object$backend,
    required = .stan_output_group_pattern("rho"),
    required_type = "pattern",
    context = "dependence_summary.dcvar_covariate_fit()",
    output_type = "transformed parameter group"
  ))
  tau_draws <- 2 / pi * asin(rho_draws)
  .summarise_rho_draws(tau_draws, probs, .observed_time_values(object$stan_data, drop_first = TRUE))
}

#' @rdname rho_trajectory
#' @export
rho_trajectory.dcvar_hmm_fit <- function(object, probs = c(0.025, 0.1, 0.5, 0.9, 0.975), ...) {
  rho_draws <- posterior::as_draws_matrix(.fit_draws(
    object$fit, "rho_hmm", backend = object$backend,
    required = .stan_output_group_pattern("rho_hmm"),
    required_type = "pattern",
    context = "rho_trajectory.dcvar_hmm_fit()",
    output_type = "generated quantity"
  ))
  .summarise_rho_draws(rho_draws, probs, .observed_time_values(object$stan_data, drop_first = TRUE))
}

#' @rdname dependence_summary
#' @export
dependence_summary.dcvar_hmm_fit <- function(object, probs = c(0.025, 0.1, 0.5, 0.9, 0.975), ...) {
  # The HMM dependence at time t is a state mixture, so Kendall's tau must be
  # averaged on the tau scale: sum_k gamma[t, k] * (2/pi) asin(rho_state[k]).
  # Applying asin to the gamma-weighted average rho (rho_hmm) understates tau
  # whenever the smoothed state probabilities are mixed.
  gamma_draws <- posterior::as_draws_matrix(.fit_draws(
    object$fit, "gamma", backend = object$backend,
    required = .stan_output_group_pattern("gamma"),
    required_type = "pattern",
    context = "dependence_summary.dcvar_hmm_fit()",
    output_type = "generated quantity"
  ))
  rho_state_draws <- posterior::as_draws_matrix(.fit_draws(
    object$fit, "rho_state", backend = object$backend,
    required = .stan_output_group_pattern("rho_state"),
    required_type = "pattern",
    context = "dependence_summary.dcvar_hmm_fit()",
    output_type = "transformed parameter group"
  ))

  n_time_eff <- object$stan_data$n_time - 1L
  K <- ncol(rho_state_draws)
  tau_state <- 2 / pi * asin(rho_state_draws)

  tau_draws <- matrix(0, nrow(gamma_draws), n_time_eff)
  for (k in seq_len(K)) {
    gamma_cols <- paste0("gamma[", seq_len(n_time_eff), ",", k, "]")
    tau_draws <- tau_draws + as.matrix(gamma_draws[, gamma_cols, drop = FALSE]) * as.numeric(tau_state[, k])
  }
  .summarise_rho_draws(tau_draws, probs, .observed_time_values(object$stan_data, drop_first = TRUE))
}

#' Internal helper: constant rho trajectory (shared by constant and multilevel)
#' @noRd
.rho_trajectory_constant_impl <- function(object, probs) {
  .validate_probs(probs)

  rho_draws <- posterior::as_draws_matrix(.fit_draws(
    object$fit, "rho", backend = object$backend,
    required = "rho",
    context = ".rho_trajectory_constant_impl()",
    output_type = "transformed parameter"
  ))

  n_time_obs <- object$stan_data$n_time
  n_time_eff <- n_time_obs - 1L

  rho_mean <- mean(rho_draws[, 1])
  rho_sd <- sd(rho_draws[, 1])
  quants <- quantile(rho_draws[, 1], probs = probs)
  time_values <- .observed_time_values(object$stan_data, drop_first = TRUE)

  rho_summary <- data.frame(
    time = time_values,
    mean = rep(rho_mean, n_time_eff),
    sd = rep(rho_sd, n_time_eff)
  )

  for (i in seq_along(probs)) {
    rho_summary[[paste0("q", probs[i] * 100)]] <- rep(quants[i], n_time_eff)
  }

  rho_summary
}

#' Internal helper: constant Kendall's tau summary.
#' @noRd
.dependence_summary_constant_impl <- function(object, probs) {
  .validate_probs(probs)

  copula <- object$copula %||% "gaussian"
  if (identical(copula, "clayton")) {
    theta_draws <- posterior::as_draws_matrix(.fit_draws(
      object$fit, "theta", backend = object$backend,
      required = "theta",
      context = ".dependence_summary_constant_impl()",
      output_type = "parameter"
    ))
    dep_draws <- theta_draws[, 1] / (theta_draws[, 1] + 2)
  } else {
    rho_draws <- posterior::as_draws_matrix(.fit_draws(
      object$fit, "rho", backend = object$backend,
      required = "rho",
      context = ".dependence_summary_constant_impl()",
      output_type = "transformed parameter"
    ))
    dep_draws <- 2 / pi * asin(rho_draws[, 1])
  }

  n_time_obs <- object$stan_data$n_time
  n_time_eff <- n_time_obs - 1L
  quants <- quantile(dep_draws, probs = probs)
  time_values <- .observed_time_values(object$stan_data, drop_first = TRUE)

  dep_summary <- data.frame(
    time = time_values,
    mean = rep(mean(dep_draws), n_time_eff),
    sd = rep(sd(dep_draws), n_time_eff)
  )

  for (i in seq_along(probs)) {
    dep_summary[[paste0("q", probs[i] * 100)]] <- rep(quants[i], n_time_eff)
  }

  dep_summary
}

#' @rdname rho_trajectory
#' @export
rho_trajectory.dcvar_constant_fit <- function(object, probs = c(0.025, 0.1, 0.5, 0.9, 0.975), ...) {
  if (identical(object$copula %||% "gaussian", "clayton")) {
    cli_abort("Clayton fits do not have a Gaussian copula rho. Use {.fun dependence_summary} for Kendall's tau.")
  }
  .rho_trajectory_constant_impl(object, probs)
}

#' @rdname dependence_summary
#' @export
dependence_summary.dcvar_constant_fit <- function(object, probs = c(0.025, 0.1, 0.5, 0.9, 0.975), ...) {
  .dependence_summary_constant_impl(object, probs)
}


#' Extract VAR(1) parameter summaries
#'
#' Returns posterior summaries for the VAR parameters: intercepts (mu),
#' coefficients (Phi), innovation SDs (sigma_eps), and sigma_omega (DC-VAR only).
#'
#' @param object A fitted model object.
#' @param ... Additional arguments (unused).
#'
#' @return A named list of data frames with columns `variable`, `mean`, `sd`,
#'   `q2.5`, `q97.5`.
#' @export
var_params <- function(object, ...) {
  UseMethod("var_params")
}

#' @rdname var_params
#' @export
var_params.default <- function(object, ...) {
  cli_abort("{.fun var_params} is not defined for objects of class {.cls {class(object)[[1]]}}.")
}

#' @rdname var_params
#' @export
var_params.dcvar_model_fit <- function(object, ...) {
  margins <- object$margins %||% "normal"
  copula <- object$copula %||% "gaussian"
  mixed <- .is_mixed_margins(margins)
  required_patterns <- c("^mu\\[", "^Phi\\[")
  if (mixed) {
    margin_groups <- names(.mixed_margin_report_vars(margins))
    required_patterns <- c(required_patterns, paste0("^", margin_groups, "\\["))
  } else {
    switch(margins[[1L]],
      normal = {
        required_patterns <- c(required_patterns, "^sigma_eps\\[")
      },
      exponential = {
        required_patterns <- c(required_patterns, "^sigma_exp\\[")
      },
      skew_normal = {
        required_patterns <- c(required_patterns, "^omega\\[", "^delta\\[")
      },
      gamma = {
        required_patterns <- c(required_patterns, "^sigma_gam\\[", "^shape_gam$")
      },
      {
        required_patterns <- c(required_patterns, "^sigma_eps\\[")
      }
    )
  }
  has_sigma_omega <- identical(object$model, "dcvar") ||
    identical(object$model, "dcvar_covariate")
  if (has_sigma_omega) {
    required_patterns <- c(required_patterns, "^sigma_omega$")
  }
  if (identical(copula, "clayton")) {
    required_patterns <- c(required_patterns, "^theta$")
  }

  summ <- .fit_summary(
    object$fit, variables = NULL, backend = object$backend,
    required = required_patterns,
    required_type = "pattern",
    context = "var_params.dcvar_model_fit()",
    output_type = "parameter group",
    mean, sd,
    ~posterior::quantile2(.x, probs = c(0.025, 0.975))
  )
  extract_param <- function(pattern) {
    rows <- grep(pattern, summ$variable)
    data.frame(
      variable = summ$variable[rows],
      mean = summ$mean[rows],
      sd = summ$sd[rows],
      q2.5 = summ$q2.5[rows],
      q97.5 = summ$q97.5[rows]
    )
  }

  result <- list(
    mu = extract_param("^mu\\["),
    Phi = extract_param("^Phi\\[")
  )

  # Margin-specific scale parameters
  if (mixed) {
    # Report each dimension under its own family, restricted to that family's
    # dimensions (e.g. sigma_eps[1] for a normal dim, sigma_exp[2] for an
    # exponential dim).
    extract_vars <- function(vars) {
      rows <- match(vars, summ$variable)
      rows <- rows[!is.na(rows)]
      data.frame(
        variable = summ$variable[rows],
        mean = summ$mean[rows],
        sd = summ$sd[rows],
        q2.5 = summ$q2.5[rows],
        q97.5 = summ$q97.5[rows]
      )
    }
    specs <- .mixed_margin_report_vars(margins)
    for (nm in names(specs)) {
      result[[nm]] <- extract_vars(specs[[nm]])
    }
  } else {
    switch(margins[[1L]],
      normal = {
        result$sigma_eps <- extract_param("^sigma_eps\\[")
      },
      exponential = {
        result$sigma_exp <- extract_param("^sigma_exp\\[")
      },
      skew_normal = {
        result$omega <- extract_param("^omega\\[")
        result$delta <- extract_param("^delta\\[")
      },
      gamma = {
        result$sigma_gam <- extract_param("^sigma_gam\\[")
        result$shape_gam <- extract_param("^shape_gam$")
      },
      {
        result$sigma_eps <- extract_param("^sigma_eps\\[")
      }
    )
  }

  # sigma_omega is present in random-walk DC-VAR variants only.
  so <- if (has_sigma_omega) extract_param("^sigma_omega$") else NULL
  if (!is.null(so)) result$sigma_omega <- so
  if (identical(copula, "clayton")) {
    result$theta <- extract_param("^theta$")
  }

  result
}


#' Extract covariate effect summaries
#'
#' Returns posterior summaries for the Fisher-z intercept and covariate
#' effects. The residual random-walk innovation scale `sigma_omega` is
#' reported separately by [var_params()] for `drift = TRUE` fits.
#'
#' @param object A `dcvar_covariate_fit` object.
#' @param probs Numeric vector of quantile probabilities (default:
#'   `c(0.025, 0.5, 0.975)`).
#' @param ... Additional arguments (unused).
#'
#' @return A data frame with one row per effect and columns `term`, `variable`,
#'   `mean`, `sd`, and one column per requested quantile.
#' @export
covariate_effects <- function(object, ...) {
  UseMethod("covariate_effects")
}

#' @rdname covariate_effects
#' @export
covariate_effects.default <- function(object, ...) {
  cli_abort("{.fun covariate_effects} is not defined for objects of class {.cls {class(object)[[1]]}}.")
}

#' @rdname covariate_effects
#' @export
covariate_effects.dcvar_covariate_fit <- function(object, probs = c(0.025, 0.5, 0.975), ...) {
  .validate_probs(probs)

  summ <- .fit_summary(
    object$fit,
    variables = NULL,
    backend = object$backend,
    required = c("^beta_0$", "^beta\\["),
    required_type = "pattern",
    context = "covariate_effects.dcvar_covariate_fit()",
    output_type = "parameter group",
    mean,
    sd,
    ~posterior::quantile2(.x, probs = probs)
  )

  rows <- c(
    grep("^beta_0$", summ$variable),
    grep("^beta\\[", summ$variable)
  )
  out <- data.frame(summ[rows, , drop = FALSE], row.names = NULL)
  out$term <- out$variable
  beta_rows <- grep("^beta\\[", out$variable)
  if (length(beta_rows) > 0L) {
    out$term[beta_rows] <- object$covariates
  }
  out$term[out$variable == "beta_0"] <- "(Intercept)"

  out[, c("term", setdiff(names(out), "term")), drop = FALSE]
}


#' Extract HMM state information
#'
#' Returns state posteriors, Viterbi path, state-specific rho values,
#' and the transition matrix from an HMM copula fit.
#'
#' @param object A `dcvar_hmm_fit` object.
#' @param ... Additional arguments (unused).
#'
#' @return A named list with:
#'   - `gamma`: T_eff x K matrix of posterior state probabilities
#'   - `viterbi`: integer vector, the Viterbi (MAP) state path decoded from
#'     the posterior-mean emission log-likelihoods, transition matrix, and
#'     initial state probabilities (a plug-in estimator, not a full posterior
#'     summary of the path)
#'   - `rho_state`: list with `mean`, `lower`, `upper` for each state
#'   - `A`: K x K posterior mean transition matrix
#'   - `rho_hmm`: posterior-averaged rho trajectory
#' @export
hmm_states <- function(object, ...) {
  UseMethod("hmm_states")
}

#' Internal: Viterbi decoding for fixed log-scale HMM quantities
#' @noRd
.viterbi_path <- function(obs_ll, log_A, log_pi0) {
  n_time <- nrow(obs_ll)
  K <- ncol(obs_ll)

  log_delta <- matrix(-Inf, n_time, K)
  back_ptr <- matrix(NA_integer_, n_time, K)
  log_delta[1, ] <- log_pi0 + obs_ll[1, ]
  if (n_time >= 2) {
    for (t in 2:n_time) {
      for (k in seq_len(K)) {
        cand <- log_delta[t - 1, ] + log_A[, k]
        j <- which.max(cand)
        back_ptr[t, k] <- j
        log_delta[t, k] <- cand[j] + obs_ll[t, k]
      }
    }
  }

  path <- integer(n_time)
  path[n_time] <- which.max(log_delta[n_time, ])
  if (n_time >= 2) {
    for (t in seq(n_time - 1L, 1L)) {
      path[t] <- back_ptr[t + 1L, path[t + 1L]]
    }
  }
  path
}

#' @rdname hmm_states
#' @export
hmm_states.default <- function(object, ...) {
  cli_abort("{.fun hmm_states} is not defined for objects of class {.cls {class(object)[[1]]}}.")
}

#' @rdname hmm_states
#' @export
hmm_states.dcvar_hmm_fit <- function(object, ...) {
  K <- object$K
  n_time_eff <- object$stan_data$n_time - 1L

  .safe_draws <- function(var_name) {
    tryCatch(
      posterior::as_draws_matrix(.fit_draws(
        object$fit, var_name, backend = object$backend,
        required = .stan_output_group_pattern(var_name),
        required_type = "pattern",
        context = "hmm_states.dcvar_hmm_fit()",
        output_type = "Stan output group"
      )),
      error = function(e) {
        cli_abort("Failed to extract {.val {var_name}} draws: {e$message}")
      }
    )
  }

  # State posteriors: gamma[t, k]
  gamma_draws <- .safe_draws("gamma")

  # Validate gamma columns exist upfront
  expected_gamma_cols <- paste0("gamma[", rep(seq_len(n_time_eff), K), ",", rep(seq_len(K), each = n_time_eff), "]")
  missing_gamma <- setdiff(expected_gamma_cols, colnames(gamma_draws))
  if (length(missing_gamma) > 0) {
    cli_abort("Expected gamma columns missing from draws: {.val {head(missing_gamma, 5)}}")
  }

  gamma_mean <- matrix(NA_real_, n_time_eff, K)
  for (k in 1:K) {
    cols <- paste0("gamma[", seq_len(n_time_eff), ",", k, "]")
    gamma_mean[, k] <- colMeans(gamma_draws[, cols, drop = FALSE])
  }

  # State-specific rho
  rho_state_draws <- .safe_draws("rho_state")
  rho_state_mean <- colMeans(rho_state_draws)
  rho_state_lower <- apply(rho_state_draws, 2, quantile, 0.025)
  rho_state_upper <- apply(rho_state_draws, 2, quantile, 0.975)

  # Transition matrix
  A_draws <- .safe_draws("A")

  # Validate A columns exist upfront
  expected_A_cols <- paste0("A[", rep(seq_len(K), K), ",", rep(seq_len(K), each = K), "]")
  missing_A <- setdiff(expected_A_cols, colnames(A_draws))
  if (length(missing_A) > 0) {
    cli_abort("Expected transition matrix columns missing from draws: {.val {head(missing_A, 5)}}")
  }

  A_mean <- matrix(NA_real_, K, K)
  for (i in 1:K) {
    for (j in 1:K) {
      col <- paste0("A[", i, ",", j, "]")
      A_mean[i, j] <- mean(A_draws[, col])
    }
  }

  # MAP state sequence: Viterbi decoding on posterior-mean log quantities.
  # (The previous most-frequent-sampled-path estimator degenerates to an
  # arbitrary single draw's path when parameter uncertainty makes nearly all
  # sampled paths unique.)
  obs_ll_draws <- .safe_draws("obs_ll")
  obs_ll_mean <- matrix(NA_real_, n_time_eff, K)
  for (k in 1:K) {
    cols <- paste0("obs_ll[", seq_len(n_time_eff), ",", k, "]")
    obs_ll_mean[, k] <- colMeans(obs_ll_draws[, cols, drop = FALSE])
  }
  pi0_draws <- .safe_draws("pi0")
  pi0_mean <- colMeans(pi0_draws[, paste0("pi0[", seq_len(K), "]"), drop = FALSE])
  viterbi_map <- .viterbi_path(obs_ll_mean, log(A_mean), log(pi0_mean))

  # Posterior-averaged rho
  rho_hmm_draws <- .safe_draws("rho_hmm")
  rho_hmm_mean <- colMeans(rho_hmm_draws)

  list(
    gamma = gamma_mean,
    viterbi = viterbi_map,
    rho_state = list(
      mean = rho_state_mean,
      lower = rho_state_lower,
      upper = rho_state_upper
    ),
    A = A_mean,
    rho_hmm = rho_hmm_mean
  )
}


#' @rdname var_params
#' @details
#' For a Markov-switching HMM fit (`dcvar_hmm()` with `switch` beyond `"rho"` or
#' per-state `margins`), the VAR parameters are state-indexed: `mu[k, d]`, the Phi
#' baseline `Phi_base[i, j]` plus per-state deviations `Phi_dev[k, i, j]` on the
#' switching coefficients, and per-state margin scales. A non-switching HMM fit
#' delegates to the shared method. See [hmm_state_params()] for a per-state
#' assembled view.
#' @export
var_params.dcvar_hmm_fit <- function(object, ...) {
  if (!isTRUE(object$switching)) {
    return(NextMethod())
  }

  sw <- object$switch
  K <- object$K
  Mrg_K <- if (isTRUE(sw$margins == 1L)) K else 1L
  report <- .hmm_switching_report_vars(object$margins_matrix, Mrg_K)

  required_patterns <- c("^mu\\[", "^Phi_base\\[", "^rho_state\\[",
                         paste0("^", names(report), "\\["))
  if (any(sw$phi_mask > 0L)) {
    required_patterns <- c(required_patterns, "^Phi_dev\\[")
  }

  summ <- .fit_summary(
    object$fit, variables = NULL, backend = object$backend,
    required = required_patterns,
    required_type = "pattern",
    context = "var_params.dcvar_hmm_fit()",
    output_type = "parameter group",
    mean, sd,
    ~posterior::quantile2(.x, probs = c(0.025, 0.975))
  )
  extract_param <- function(pattern) {
    rows <- grep(pattern, summ$variable)
    data.frame(
      variable = summ$variable[rows],
      mean = summ$mean[rows],
      sd = summ$sd[rows],
      q2.5 = summ$q2.5[rows],
      q97.5 = summ$q97.5[rows]
    )
  }
  extract_vars <- function(vars) {
    rows <- match(vars, summ$variable)
    rows <- rows[!is.na(rows)]
    data.frame(
      variable = summ$variable[rows],
      mean = summ$mean[rows],
      sd = summ$sd[rows],
      q2.5 = summ$q2.5[rows],
      q97.5 = summ$q97.5[rows]
    )
  }

  result <- list(
    mu = extract_param("^mu\\["),
    Phi = extract_param("^Phi_base\\[")
  )
  if (any(sw$phi_mask > 0L)) {
    result$Phi_dev <- extract_param("^Phi_dev\\[")
  }
  for (nm in names(report)) {
    result[[nm]] <- extract_vars(report[[nm]])
  }
  result$rho_state <- extract_param("^rho_state\\[")
  result
}


#' Extract per-state parameters from a Markov-switching HMM fit
#'
#' Assembles the state-specific VAR parameters of a switching `dcvar_hmm()` fit
#' into a per-state view: the intercepts, the effective VAR(1) coefficient matrix
#' (`Phi_base` plus the per-state deviations on the switching coefficients), the
#' margin scale/shape parameters, and the per-state marginal family labels. For
#' components that are NOT state-specific the shared estimate is tiled across all
#' states, so the return shape does not depend on which components switch.
#'
#' @param object A `dcvar_hmm_fit` object.
#' @param ... Additional arguments (unused).
#'
#' @return A list with `families` (a K x D character matrix), `rho_state`
#'   (length-K), `mu` (a K x D matrix of posterior means), `Phi` (a length-K list
#'   of D x D posterior-mean coefficient matrices), and `scales` (a data frame of
#'   per-state margin scale/shape posterior means; shared scales are tiled across
#'   states).
#' @seealso [hmm_states()], [var_params()]
#' @export
hmm_state_params <- function(object, ...) {
  UseMethod("hmm_state_params")
}

#' @rdname hmm_state_params
#' @export
hmm_state_params.default <- function(object, ...) {
  cli_abort("{.fun hmm_state_params} is not defined for objects of class {.cls {class(object)[[1]]}}.")
}

#' @rdname hmm_state_params
#' @export
hmm_state_params.dcvar_hmm_fit <- function(object, ...) {
  if (!isTRUE(object$switching)) {
    cli_abort(c(
      "{.fun hmm_state_params} is for state-specific HMM fits.",
      "i" = "This fit only switches the copula correlation; use {.fun hmm_states} and {.fun coef}."
    ))
  }
  K <- object$K
  D <- object$stan_data$D
  sw <- object$switch
  margins_char <- object$margins_matrix
  summ <- .fit_summary(object$fit, backend = object$backend)
  pm <- function(name) {
    row <- match(name, summ$variable)
    if (is.na(row)) NA_real_ else summ$mean[row]
  }

  # Per-state intercepts (tiled when mu is global).
  mu <- matrix(NA_real_, nrow = K, ncol = D)
  for (k in seq_len(K)) {
    mk <- if (isTRUE(sw$mu == 1L)) k else 1L
    for (d in seq_len(D)) mu[k, d] <- pm(sprintf("mu[%d,%d]", mk, d))
  }

  # Effective per-state Phi = baseline + masked per-state deviation.
  phi_base <- matrix(NA_real_, nrow = D, ncol = D)
  for (i in seq_len(D)) for (j in seq_len(D)) phi_base[i, j] <- pm(sprintf("Phi_base[%d,%d]", i, j))
  ij <- list(c(1L, 1L), c(1L, 2L), c(2L, 1L), c(2L, 2L))
  active <- which(sw$phi_mask > 0L)
  Phi <- lapply(seq_len(K), function(k) {
    P <- phi_base
    for (a in active) {
      i <- ij[[a]][1]
      j <- ij[[a]][2]
      P[i, j] <- P[i, j] + pm(sprintf("Phi_dev[%d,%d,%d]", k, i, j))
    }
    P
  })

  rho_state <- vapply(seq_len(K), function(k) pm(sprintf("rho_state[%d]", k)), numeric(1))

  # Per-state margin scale/shape posterior means. Shared margin configs are
  # tiled across states so the returned shape is flag-agnostic.
  Mrg_K <- if (isTRUE(sw$margins == 1L)) K else 1L
  scale_rows <- list()
  row_idx <- 1L
  for (k in seq_len(K)) {
    mk <- if (Mrg_K == K) k else 1L
    for (d in seq_len(D)) {
      fam <- margins_char[k, d]
      vars <- switch(fam,
        normal = c(sigma_eps = sprintf("sigma_eps[%d,%d]", mk, d)),
        exponential = c(sigma_exp = sprintf("sigma_exp[%d,%d]", mk, d)),
        skew_normal = c(
          omega = sprintf("omega[%d,%d]", mk, d),
          delta = sprintf("delta[%d,%d]", mk, d)
        ),
        gamma = c(
          sigma_gam = sprintf("sigma_gam[%d,%d]", mk, d),
          shape_gam = sprintf("shape_gam[%d,%d]", mk, d)
        )
      )
      for (param in names(vars)) {
        scale_rows[[row_idx]] <- data.frame(
          state = k,
          variable = object$vars[d],
          dimension = d,
          family = fam,
          parameter = param,
          stan_variable = unname(vars[[param]]),
          mean = pm(unname(vars[[param]])),
          stringsAsFactors = FALSE
        )
        row_idx <- row_idx + 1L
      }
    }
  }
  scales <- do.call(rbind, scale_rows)
  rownames(scales) <- NULL

  list(
    families = margins_char,
    rho_state = rho_state,
    mu = mu,
    Phi = Phi,
    scales = scales
  )
}


#' Extract random effects from a multilevel fit
#'
#' Returns posterior summaries for unit-specific VAR coefficients.
#'
#' @param object A `dcvar_multilevel_fit` object.
#' @param ... Additional arguments (unused).
#'
#' @return A data frame with columns `unit`, `parameter`, `mean`, `sd`,
#'   `q2.5`, `q97.5`.
#' @export
random_effects <- function(object, ...) {
  UseMethod("random_effects")
}

#' @rdname random_effects
#' @export
random_effects.default <- function(object, ...) {
  cli_abort("{.fun random_effects} is not defined for objects of class {.cls {class(object)[[1]]}}.")
}

#' @rdname random_effects
#' @export
random_effects.dcvar_multilevel_fit <- function(object, ...) {
  N <- object$N
  unit_ids <- attr(object$stan_data, "ids")
  if (is.null(unit_ids)) {
    unit_ids <- seq_len(N)
  }
  summ <- .fit_summary(
    object$fit, variables = NULL, backend = object$backend,
    required = paste0(
      "phi_unit[",
      rep(seq_len(N), each = 4),
      ",",
      rep(seq_len(4), times = N),
      "]"
    ),
    required_type = "exact",
    context = "random_effects.dcvar_multilevel_fit()",
    output_type = "parameter",
    mean, sd,
    ~posterior::quantile2(.x, probs = c(0.025, 0.975))
  )

  param_names <- c("phi11", "phi12", "phi21", "phi22")
  results <- vector("list", N * 4)
  idx <- 1

  for (i in seq_len(N)) {
    for (k in 1:4) {
      var_name <- paste0("phi_unit[", i, ",", k, "]")
      row <- which(summ$variable == var_name)
      if (length(row) != 1L) {
        cli_abort(c(
          "{.fun random_effects} requires unit-specific VAR coefficients that are not present in the fitted model.",
          "i" = "Missing parameter: {.val {var_name}}.",
          "i" = "Custom Stan files must preserve the expected parameter names."
        ))
      }
      results[[idx]] <- data.frame(
        unit = unit_ids[i],
        parameter = param_names[k],
        mean = summ$mean[row],
        sd = summ$sd[row],
        q2.5 = summ$q2.5[row],
        q97.5 = summ$q97.5[row]
      )
      idx <- idx + 1
    }
  }

  do.call(rbind, results)
}


#' @rdname var_params
#' @details
#' For multilevel models, returns population-level parameters `phi_bar`
#' (mean VAR coefficients), `tau_phi` (between-unit SDs), `sigma`
#' (innovation SDs), and `rho` (copula correlation). These correspond to
#' `Phi`, `sigma_eps`, and `rho` in single-level models.
#' @export
var_params.dcvar_multilevel_fit <- function(object, ...) {
  margins <- object$margins %||% "normal"
  mixed <- .uses_sem_multilevel_mixed_engine("multilevel", margins)
  engine_margins <- .sem_multilevel_engine_margins("multilevel", margins, 2L)
  scale_pattern <- if (mixed) {
    paste0("^", names(.mixed_margin_report_vars(engine_margins)), "\\[")
  } else if (identical(margins, "exponential")) {
    "^sigma_exp\\["
  } else {
    "^sigma\\["
  }
  summ <- .fit_summary(
    object$fit, variables = NULL, backend = object$backend,
    required = c("^phi_bar\\[", "^tau_phi\\[", scale_pattern, "^rho$"),
    required_type = "pattern",
    context = "var_params.dcvar_multilevel_fit()",
    output_type = "parameter group",
    mean, sd,
    ~posterior::quantile2(.x, probs = c(0.025, 0.975))
  )
  extract_param <- function(pattern) {
    rows <- grep(pattern, summ$variable)
    data.frame(
      variable = summ$variable[rows],
      mean = summ$mean[rows],
      sd = summ$sd[rows],
      q2.5 = summ$q2.5[rows],
      q97.5 = summ$q97.5[rows]
    )
  }

  scale_param <- if (mixed) {
    specs <- .mixed_margin_report_vars(engine_margins)
    lapply(specs, function(vars) {
      rows <- match(vars, summ$variable)
      rows <- rows[!is.na(rows)]
      data.frame(
        variable = summ$variable[rows],
        mean = summ$mean[rows],
        sd = summ$sd[rows],
        q2.5 = summ$q2.5[rows],
        q97.5 = summ$q97.5[rows]
      )
    })
  } else if (identical(margins, "exponential")) {
    list(sigma_exp = extract_param("^sigma_exp\\["))
  } else {
    list(sigma = extract_param("^sigma\\["))
  }

  c(list(
    phi_bar = extract_param("^phi_bar\\["),
    tau_phi = extract_param("^tau_phi\\[")
  ), scale_param, list(
    rho = extract_param("^rho$")
  ))
}


#' @rdname rho_trajectory
#' @export
rho_trajectory.dcvar_multilevel_fit <- function(object, probs = c(0.025, 0.1, 0.5, 0.9, 0.975), ...) {
  .rho_trajectory_constant_impl(object, probs)
}

#' @rdname dependence_summary
#' @export
dependence_summary.dcvar_multilevel_fit <- function(object, probs = c(0.025, 0.1, 0.5, 0.9, 0.975), ...) {
  .dependence_summary_constant_impl(object, probs)
}


#' Extract latent states from a SEM fit
#'
#' Returns posterior summaries for the estimated latent states at each
#' time point.
#'
#' @param object A `dcvar_sem_fit` object.
#' @param probs Numeric vector of quantile probabilities.
#' @param ... Additional arguments (unused).
#'
#' @return A data frame with columns `time`, `variable`, `mean`, `sd`,
#'   and quantile columns.
#' @export
latent_states <- function(object, ...) {
  UseMethod("latent_states")
}

#' @rdname latent_states
#' @export
latent_states.default <- function(object, ...) {
  cli_abort("{.fun latent_states} is not defined for objects of class {.cls {class(object)[[1]]}}.")
}

#' @rdname latent_states
#' @export
latent_states.dcvar_sem_fit <- function(object, probs = c(0.025, 0.5, 0.975), ...) {
  if (identical(object$method %||% "indicator", "naive")) {
    cli_abort("{.fun latent_states} is not defined for naive SEM fits because no latent measurement model is estimated.")
  }
  .validate_probs(probs)

  state_draws <- posterior::as_draws_matrix(.fit_draws(
    object$fit, "state", backend = object$backend,
    required = .stan_output_group_pattern("state"),
    required_type = "pattern",
    context = "latent_states.dcvar_sem_fit()",
    output_type = "transformed parameter group"
  ))
  n_time_obs <- object$stan_data$n_time
  vars <- object$vars
  time_values <- .observed_time_values(object$stan_data)

  results <- vector("list", 2)
  for (d in 1:2) {
    cols <- paste0("state[", seq_len(n_time_obs), ",", d, "]")
    draws_d <- state_draws[, cols, drop = FALSE]

    df <- data.frame(
      time = time_values,
      variable = vars[d],
      mean = colMeans(draws_d),
      sd = apply(draws_d, 2, sd)
    )

    quants <- apply(draws_d, 2, quantile, probs = probs)
    if (is.null(dim(quants))) {
      quants <- matrix(quants, nrow = 1)
    }
    for (i in seq_along(probs)) {
      df[[paste0("q", probs[i] * 100)]] <- quants[i, ]
    }

    results[[d]] <- df
  }

  do.call(rbind, results)
}


#' @rdname var_params
#' @export
var_params.dcvar_sem_fit <- function(object, ...) {
  margins <- object$margins %||% "normal"
  mixed <- .uses_sem_multilevel_mixed_engine("sem", margins)
  engine_margins <- .sem_multilevel_engine_margins("sem", margins, 2L)
  required_patterns <- c("^mu\\[", "^Phi\\[", "^rho$")
  if (mixed) {
    required_patterns <- c(required_patterns,
                           paste0("^", names(.mixed_margin_report_vars(engine_margins)), "\\["))
  } else if (identical(margins, "normal")) {
    required_patterns <- c(required_patterns, "^sigma\\[")
  } else if (identical(margins, "exponential")) {
    required_patterns <- c(required_patterns, "^sigma_exp\\[")
  } else {
    cli_abort("Unsupported SEM margin type in {.fun var_params}: {.val {margins}}.")
  }

  summ <- .fit_summary(
    object$fit, variables = NULL, backend = object$backend,
    required = required_patterns,
    required_type = "pattern",
    context = "var_params.dcvar_sem_fit()",
    output_type = "parameter group",
    mean, sd,
    ~posterior::quantile2(.x, probs = c(0.025, 0.975))
  )
  extract_param <- function(pattern) {
    rows <- grep(pattern, summ$variable)
    data.frame(
      variable = summ$variable[rows],
      mean = summ$mean[rows],
      sd = summ$sd[rows],
      q2.5 = summ$q2.5[rows],
      q97.5 = summ$q97.5[rows]
    )
  }

  result <- list(
    mu = extract_param("^mu\\["),
    Phi = extract_param("^Phi\\["),
    rho = extract_param("^rho$")
  )
  scale_param <- if (mixed) {
    specs <- .mixed_margin_report_vars(engine_margins)
    lapply(specs, function(vars) {
      rows <- match(vars, summ$variable)
      rows <- rows[!is.na(rows)]
      data.frame(
        variable = summ$variable[rows],
        mean = summ$mean[rows],
        sd = summ$sd[rows],
        q2.5 = summ$q2.5[rows],
        q97.5 = summ$q97.5[rows]
      )
    })
  } else if (identical(margins, "normal")) {
    list(sigma = extract_param("^sigma\\["))
  } else {
    list(sigma_exp = extract_param("^sigma_exp\\["))
  }

  c(result[1:2], scale_param, result[3])
}


#' @rdname rho_trajectory
#' @export
rho_trajectory.dcvar_sem_fit <- function(object, probs = c(0.025, 0.1, 0.5, 0.9, 0.975), ...) {
  # SEM has constant rho, expand to all n_time - 1 time points
  rho_draws <- posterior::as_draws_matrix(.fit_draws(
    object$fit, "rho", backend = object$backend,
    required = "rho",
    context = "rho_trajectory.dcvar_sem_fit()",
    output_type = "transformed parameter"
  ))
  n_time_obs <- object$stan_data$n_time
  n_time_eff <- n_time_obs - 1L

  rho_mean <- mean(rho_draws[, 1])
  rho_sd <- sd(rho_draws[, 1])
  quants <- quantile(rho_draws[, 1], probs = probs)
  time_values <- .observed_time_values(object$stan_data, drop_first = TRUE)

  rho_summary <- data.frame(
    time = time_values,
    mean = rep(rho_mean, n_time_eff),
    sd = rep(rho_sd, n_time_eff)
  )

  for (i in seq_along(probs)) {
    rho_summary[[paste0("q", probs[i] * 100)]] <- rep(quants[i], n_time_eff)
  }

  rho_summary
}

#' @rdname dependence_summary
#' @export
dependence_summary.dcvar_sem_fit <- function(object, probs = c(0.025, 0.1, 0.5, 0.9, 0.975), ...) {
  .dependence_summary_constant_impl(object, probs)
}


#' Extract posterior draws
#'
#' Extract posterior draws from a fitted model.
#'
#' @param object A fitted model object.
#' @param variable Character vector of parameter names. `NULL` returns all.
#' @param format Draw format: `"draws_array"`, `"draws_matrix"`, or
#'   `"draws_df"` (default: `"draws_array"`).
#' @param ... Additional arguments (unused).
#'
#' @return A posterior draws object.
#' @export
draws <- function(object, ...) {
  UseMethod("draws")
}

#' @rdname draws
#' @export
draws.default <- function(object, ...) {
  cli_abort("{.fun draws} is not defined for objects of class {.cls {class(object)[[1]]}}.")
}

#' @rdname draws
#' @export
draws.dcvar_model_fit <- function(object, variable = NULL, format = "draws_array", ...) {
  format <- match.arg(format, c("draws_array", "draws_matrix", "draws_df"))
  .fit_draws(object$fit, variables = variable, format = format, backend = object$backend)
}


# ============================================================================
# Time-varying trajectory extractors (dcvar_tv_fit)
# ============================================================================

#' Extract the time-varying VAR coefficient trajectories
#'
#' Returns posterior summaries of the four VAR(1) coefficient paths
#' phi11(t), phi12(t), phi21(t), phi22(t) from a time-varying DC-VAR fit.
#' For fits with `tv_phi = FALSE` the constant baseline coefficients are
#' tiled over time so the return shape does not depend on the flag.
#'
#' @param object A fitted model object.
#' @param probs Numeric vector of quantile probabilities
#'   (default: `c(0.025, 0.1, 0.5, 0.9, 0.975)`).
#' @param unit For `dcvar_multilevel_tv_fit` objects, optional unit selector.
#'   Use `NULL` (default) for the population-level Phi path, `"all"` for all
#'   unit-specific paths, or a vector of unit IDs for selected unit paths.
#' @param ... Additional arguments (unused).
#'
#' @return A data frame with columns `time`, `coefficient` (one of
#'   `"phi11"`, `"phi12"`, `"phi21"`, `"phi22"`), `mean`, `sd`, and one
#'   column per requested quantile.
#' @seealso [sigma_trajectory()], [rho_trajectory()], [plot_phi_trajectory()]
#' @export
phi_trajectory <- function(object, ...) {
  UseMethod("phi_trajectory")
}

#' @rdname phi_trajectory
#' @export
phi_trajectory.default <- function(object, ...) {
  cli_abort("{.fun phi_trajectory} is not defined for objects of class {.cls {class(object)[[1]]}}.")
}

#' Internal: coefficient labels in the row-major Stan column order
#' @noRd
.phi_tv_labels <- c("phi11", "phi12", "phi21", "phi22")

#' Internal: matching baseline Phi draw column for each phi_t column
#' @noRd
.phi_tv_baseline_cols <- c("Phi[1,1]", "Phi[1,2]", "Phi[2,1]", "Phi[2,2]")

#' @rdname phi_trajectory
#' @export
phi_trajectory.dcvar_tv_fit <- function(object, probs = c(0.025, 0.1, 0.5, 0.9, 0.975), ...) {
  n_time_eff <- object$stan_data$n_time - 1L
  time_values <- .observed_time_values(object$stan_data, drop_first = TRUE)

  if (isTRUE(object$tv_phi)) {
    phi_draws <- posterior::as_draws_matrix(.fit_draws(
      object$fit, "phi_t", backend = object$backend,
      required = .stan_output_group_pattern("phi_t"),
      required_type = "pattern",
      context = "phi_trajectory.dcvar_tv_fit()",
      output_type = "generated quantity"
    ))
    out <- lapply(1:4, function(k) {
      cols <- paste0("phi_t[", seq_len(n_time_eff), ",", k, "]")
      df <- .summarise_rho_draws(as.matrix(phi_draws[, cols, drop = FALSE]), probs, time_values)
      df$coefficient <- .phi_tv_labels[k]
      df
    })
  } else {
    # Constant Phi: tile the baseline so the contract is flag-agnostic.
    base_draws <- posterior::as_draws_matrix(.fit_draws(
      object$fit, "Phi", backend = object$backend,
      required = .stan_output_group_pattern("Phi"),
      required_type = "pattern",
      context = "phi_trajectory.dcvar_tv_fit()",
      output_type = "parameter group"
    ))
    out <- lapply(1:4, function(k) {
      one <- .summarise_rho_draws(
        as.matrix(base_draws[, .phi_tv_baseline_cols[k], drop = FALSE]),
        probs, time_values[1]
      )
      df <- one[rep(1L, n_time_eff), , drop = FALSE]
      df$time <- time_values
      rownames(df) <- NULL
      df$coefficient <- .phi_tv_labels[k]
      df
    })
  }

  out <- do.call(rbind, out)
  rownames(out) <- NULL
  out[, c("time", "coefficient", setdiff(names(out), c("time", "coefficient")))]
}


#' Extract the time-varying residual scale trajectories
#'
#' Returns posterior summaries of the per-variable residual scale paths from
#' a time-varying DC-VAR fit. The reported value is each margin family's
#' natural residual scale: the innovation SD for normal/skew-normal
#' dimensions, and the (time-constant) `sigma_exp` / `sigma_gam` for
#' exponential and gamma dimensions. For fits with `tv_sigma = FALSE` the
#' constant baselines are tiled over time so the return shape does not depend
#' on the flag.
#'
#' @param object A fitted model object.
#' @param probs Numeric vector of quantile probabilities
#'   (default: `c(0.025, 0.1, 0.5, 0.9, 0.975)`).
#' @param ... Additional arguments (unused).
#'
#' @return A data frame with columns `time`, `variable`, `mean`, `sd`, and
#'   one column per requested quantile.
#' @seealso [phi_trajectory()], [rho_trajectory()], [plot_sigma_trajectory()]
#' @export
sigma_trajectory <- function(object, ...) {
  UseMethod("sigma_trajectory")
}

#' @rdname sigma_trajectory
#' @export
sigma_trajectory.default <- function(object, ...) {
  cli_abort("{.fun sigma_trajectory} is not defined for objects of class {.cls {class(object)[[1]]}}.")
}

#' Internal: baseline scale draw column for one dimension of a TV fit
#' @noRd
.sigma_tv_baseline_col <- function(family, d) {
  switch(family,
    normal = paste0("sigma_eps[", d, "]"),
    exponential = paste0("sigma_exp[", d, "]"),
    skew_normal = paste0("omega[", d, "]"),
    gamma = paste0("sigma_gam[", d, "]")
  )
}

#' Internal: baseline scale draws on the sigma_trajectory() reporting scale
#' @noRd
.sigma_tv_baseline_draws <- function(object, family, d, context) {
  if (identical(family, "skew_normal")) {
    omega_col <- paste0("omega[", d, "]")
    delta_col <- paste0("delta[", d, "]")
    base_draws <- posterior::as_draws_matrix(.fit_draws(
      object$fit, c("omega", "delta"), backend = object$backend,
      required = c(paste0("^omega\\[", d, "\\]$"), paste0("^delta\\[", d, "\\]$")),
      required_type = "pattern",
      context = context,
      output_type = "Stan output group"
    ))
    out <- matrix(
      base_draws[, omega_col] *
        .skew_normal_residual_sd_factor(base_draws[, delta_col]),
      ncol = 1L
    )
    colnames(out) <- paste0("sigma_sd[", d, "]")
    return(out)
  }

  col <- .sigma_tv_baseline_col(family, d)
  base_draws <- posterior::as_draws_matrix(.fit_draws(
    object$fit, sub("\\[.*$", "", col), backend = object$backend,
    required = paste0("^", sub("\\[.*$", "\\\\[", col)),
    required_type = "pattern",
    context = context,
    output_type = "Stan output group"
  ))
  as.matrix(base_draws[, col, drop = FALSE])
}

#' @rdname sigma_trajectory
#' @export
sigma_trajectory.dcvar_tv_fit <- function(object, probs = c(0.025, 0.1, 0.5, 0.9, 0.975), ...) {
  D <- object$stan_data$D
  n_time_eff <- object$stan_data$n_time - 1L
  time_values <- .observed_time_values(object$stan_data, drop_first = TRUE)
  margins_vec <- rep(object$margins %||% "normal", length.out = D)

  if (isTRUE(object$tv_sigma)) {
    sigma_draws <- posterior::as_draws_matrix(.fit_draws(
      object$fit, "sigma_t", backend = object$backend,
      required = .stan_output_group_pattern("sigma_t"),
      required_type = "pattern",
      context = "sigma_trajectory.dcvar_tv_fit()",
      output_type = "generated quantity"
    ))
    out <- lapply(seq_len(D), function(d) {
      cols <- paste0("sigma_t[", seq_len(n_time_eff), ",", d, "]")
      df <- .summarise_rho_draws(as.matrix(sigma_draws[, cols, drop = FALSE]), probs, time_values)
      df$variable <- object$vars[d]
      df
    })
  } else {
    out <- lapply(seq_len(D), function(d) {
      one <- .summarise_rho_draws(
        .sigma_tv_baseline_draws(
          object, margins_vec[d], d, "sigma_trajectory.dcvar_tv_fit()"
        ),
        probs, time_values[1]
      )
      df <- one[rep(1L, n_time_eff), , drop = FALSE]
      df$time <- time_values
      rownames(df) <- NULL
      df$variable <- object$vars[d]
      df
    })
  }

  out <- do.call(rbind, out)
  rownames(out) <- NULL
  out[, c("time", "variable", setdiff(names(out), c("time", "variable")))]
}


#' @rdname var_params
#' @export
var_params.dcvar_tv_fit <- function(object, ...) {
  D <- object$stan_data$D
  margins_vec <- rep(object$margins %||% "normal", length.out = D)
  margin_groups <- names(.mixed_margin_report_vars(margins_vec))

  required_patterns <- c("^mu\\[", "^Phi\\[", "^sigma_omega$",
                         paste0("^", margin_groups, "\\["))
  if (isTRUE(object$tv_phi)) {
    required_patterns <- c(required_patterns, "^tau_phi\\[")
  }
  if (isTRUE(object$tv_sigma)) {
    required_patterns <- c(required_patterns, "^tau_sigma\\[")
  }

  summ <- .fit_summary(
    object$fit, variables = NULL, backend = object$backend,
    required = required_patterns,
    required_type = "pattern",
    context = "var_params.dcvar_tv_fit()",
    output_type = "parameter group",
    mean, sd,
    ~posterior::quantile2(.x, probs = c(0.025, 0.975))
  )
  extract_param <- function(pattern) {
    rows <- grep(pattern, summ$variable)
    data.frame(
      variable = summ$variable[rows],
      mean = summ$mean[rows],
      sd = summ$sd[rows],
      q2.5 = summ$q2.5[rows],
      q97.5 = summ$q97.5[rows]
    )
  }

  result <- list(
    mu = extract_param("^mu\\["),
    Phi = extract_param("^Phi\\[")
  )
  for (group in margin_groups) {
    result[[group]] <- extract_param(paste0("^", group, "\\["))
  }
  if (isTRUE(object$tv_phi)) {
    tau_phi <- extract_param("^tau_phi\\[")
    # Stan indexes tau_phi by active position; relabel by coefficient name.
    tau_phi$variable <- paste0("tau_phi[", .tv_active_phi_coefs(object), "]")
    result$tau_phi <- tau_phi
  }
  if (isTRUE(object$tv_sigma)) {
    result$tau_sigma <- extract_param("^tau_sigma\\[")
  }
  result$sigma_omega <- extract_param("^sigma_omega$")
  result
}


# ============================================================================
# Time-varying SEM and multilevel extractors
# ============================================================================

#' @rdname rho_trajectory
#' @export
rho_trajectory.dcvar_sem_tv_fit <- function(object, probs = c(0.025, 0.1, 0.5, 0.9, 0.975), ...) {
  rho_draws <- posterior::as_draws_matrix(.fit_draws(
    object$fit, "rho", backend = object$backend,
    required = .stan_output_group_pattern("rho"),
    required_type = "pattern",
    context = "rho_trajectory.dcvar_sem_tv_fit()",
    output_type = "transformed parameter group"
  ))
  .summarise_rho_draws(rho_draws, probs, .observed_time_values(object$stan_data, drop_first = TRUE))
}

#' @rdname dependence_summary
#' @export
dependence_summary.dcvar_sem_tv_fit <- function(object, probs = c(0.025, 0.1, 0.5, 0.9, 0.975), ...) {
  rho_draws <- posterior::as_draws_matrix(.fit_draws(
    object$fit, "rho", backend = object$backend,
    required = .stan_output_group_pattern("rho"),
    required_type = "pattern",
    context = "dependence_summary.dcvar_sem_tv_fit()",
    output_type = "transformed parameter group"
  ))
  .summarise_rho_draws(2 / pi * asin(rho_draws), probs,
                       .observed_time_values(object$stan_data, drop_first = TRUE))
}

#' @rdname rho_trajectory
#' @export
rho_trajectory.dcvar_multilevel_tv_fit <- function(object, probs = c(0.025, 0.1, 0.5, 0.9, 0.975), ...) {
  .rho_trajectory_constant_impl(object, probs)
}

#' @rdname dependence_summary
#' @export
dependence_summary.dcvar_multilevel_tv_fit <- function(object, probs = c(0.025, 0.1, 0.5, 0.9, 0.975), ...) {
  .dependence_summary_constant_impl(object, probs)
}

.phi_trajectory_tv_matrix <- function(object, probs, context, draw_name = "phi_t") {
  n_time_eff <- object$stan_data$n_time - 1L
  time_values <- .observed_time_values(object$stan_data, drop_first = TRUE)
  phi_draws <- posterior::as_draws_matrix(.fit_draws(
    object$fit, draw_name, backend = object$backend,
    required = .stan_output_group_pattern(draw_name),
    required_type = "pattern",
    context = context,
    output_type = "generated quantity"
  ))
  out <- lapply(1:4, function(k) {
    cols <- paste0(draw_name, "[", seq_len(n_time_eff), ",", k, "]")
    df <- .summarise_rho_draws(as.matrix(phi_draws[, cols, drop = FALSE]), probs, time_values)
    df$coefficient <- .phi_tv_labels[k]
    df
  })
  out <- do.call(rbind, out)
  rownames(out) <- NULL
  out[, c("time", "coefficient", setdiff(names(out), c("time", "coefficient")))]
}

.phi_trajectory_tile_baseline <- function(draws, cols, labels, probs, time_values) {
  n_time_eff <- length(time_values)
  out <- lapply(seq_along(cols), function(k) {
    one <- .summarise_rho_draws(
      as.matrix(draws[, cols[k], drop = FALSE]),
      probs,
      time_values[1]
    )
    df <- one[rep(1L, n_time_eff), , drop = FALSE]
    df$time <- time_values
    rownames(df) <- NULL
    df$coefficient <- labels[k]
    df
  })
  out <- do.call(rbind, out)
  rownames(out) <- NULL
  out[, c("time", "coefficient", setdiff(names(out), c("time", "coefficient")))]
}

#' @rdname phi_trajectory
#' @export
phi_trajectory.dcvar_sem_tv_fit <- function(object, probs = c(0.025, 0.1, 0.5, 0.9, 0.975), ...) {
  if (isTRUE(object$tv_phi)) {
    return(.phi_trajectory_tv_matrix(object, probs, "phi_trajectory.dcvar_sem_tv_fit()"))
  }
  base_draws <- posterior::as_draws_matrix(.fit_draws(
    object$fit, "Phi", backend = object$backend,
    required = .stan_output_group_pattern("Phi"),
    required_type = "pattern",
    context = "phi_trajectory.dcvar_sem_tv_fit()",
    output_type = "parameter group"
  ))
  .phi_trajectory_tile_baseline(
    base_draws, .phi_tv_baseline_cols, .phi_tv_labels, probs,
    .observed_time_values(object$stan_data, drop_first = TRUE)
  )
}

#' @rdname phi_trajectory
#' @export
phi_trajectory.dcvar_multilevel_tv_fit <- function(object, probs = c(0.025, 0.1, 0.5, 0.9, 0.975),
                                                   unit = NULL, ...) {
  n_time_eff <- object$stan_data$n_time - 1L
  time_values <- .observed_time_values(object$stan_data, drop_first = TRUE)
  unit_ids <- attr(object$stan_data, "ids") %||% seq_len(object$N)

  if (is.null(unit)) {
    if (isTRUE(object$tv_phi)) {
      return(.phi_trajectory_tv_matrix(object, probs, "phi_trajectory.dcvar_multilevel_tv_fit()"))
    }
    base_draws <- posterior::as_draws_matrix(.fit_draws(
      object$fit, "phi_bar", backend = object$backend,
      required = .stan_output_group_pattern("phi_bar"),
      required_type = "pattern",
      context = "phi_trajectory.dcvar_multilevel_tv_fit()",
      output_type = "parameter group"
    ))
    return(.phi_trajectory_tile_baseline(
      base_draws, paste0("phi_bar[", 1:4, "]"), .phi_tv_labels, probs, time_values
    ))
  }

  selected_units <- if (identical(unit, "all")) unit_ids else unit
  selected_idx <- match(as.character(selected_units), as.character(unit_ids))
  if (anyNA(selected_idx)) {
    cli_abort("Unknown {.arg unit} value{?s}: {.val {selected_units[is.na(selected_idx)]}}.")
  }

  if (isTRUE(object$tv_phi)) {
    unit_draws <- posterior::as_draws_matrix(.fit_draws(
      object$fit, "phi_unit_t", backend = object$backend,
      required = .stan_output_group_pattern("phi_unit_t"),
      required_type = "pattern",
      context = "phi_trajectory.dcvar_multilevel_tv_fit()",
      output_type = "generated quantity"
    ))
    out <- lapply(seq_along(selected_idx), function(ii) {
      i <- selected_idx[ii]
      rows <- lapply(1:4, function(k) {
        cols <- paste0("phi_unit_t[", i, ",", seq_len(n_time_eff), ",", k, "]")
        df <- .summarise_rho_draws(as.matrix(unit_draws[, cols, drop = FALSE]), probs, time_values)
        df$unit <- unit_ids[i]
        df$coefficient <- .phi_tv_labels[k]
        df
      })
      do.call(rbind, rows)
    })
  } else {
    base_draws <- posterior::as_draws_matrix(.fit_draws(
      object$fit, "phi_unit", backend = object$backend,
      required = .stan_output_group_pattern("phi_unit"),
      required_type = "pattern",
      context = "phi_trajectory.dcvar_multilevel_tv_fit()",
      output_type = "transformed parameter group"
    ))
    out <- lapply(seq_along(selected_idx), function(ii) {
      i <- selected_idx[ii]
      df <- .phi_trajectory_tile_baseline(
        base_draws, paste0("phi_unit[", i, ",", 1:4, "]"),
        .phi_tv_labels, probs, time_values
      )
      df$unit <- unit_ids[i]
      df
    })
  }
  out <- do.call(rbind, out)
  rownames(out) <- NULL
  out[, c("unit", "time", "coefficient", setdiff(names(out), c("unit", "time", "coefficient")))]
}

.sigma_trajectory_tv_generic <- function(object, probs, context) {
  D <- 2L
  n_time_eff <- object$stan_data$n_time - 1L
  time_values <- .observed_time_values(object$stan_data, drop_first = TRUE)
  margins_vec <- rep(object$margins %||% "normal", length.out = D)

  if (isTRUE(object$tv_sigma)) {
    sigma_draws <- posterior::as_draws_matrix(.fit_draws(
      object$fit, "sigma_t", backend = object$backend,
      required = .stan_output_group_pattern("sigma_t"),
      required_type = "pattern",
      context = context,
      output_type = "generated quantity"
    ))
    out <- lapply(seq_len(D), function(d) {
      cols <- paste0("sigma_t[", seq_len(n_time_eff), ",", d, "]")
      df <- .summarise_rho_draws(as.matrix(sigma_draws[, cols, drop = FALSE]), probs, time_values)
      df$variable <- object$vars[d]
      df
    })
  } else {
    out <- lapply(seq_len(D), function(d) {
      one <- .summarise_rho_draws(
        .sigma_tv_baseline_draws(object, margins_vec[d], d, context),
        probs,
        time_values[1]
      )
      df <- one[rep(1L, n_time_eff), , drop = FALSE]
      df$time <- time_values
      rownames(df) <- NULL
      df$variable <- object$vars[d]
      df
    })
  }

  out <- do.call(rbind, out)
  rownames(out) <- NULL
  out[, c("time", "variable", setdiff(names(out), c("time", "variable")))]
}

#' @rdname sigma_trajectory
#' @export
sigma_trajectory.dcvar_sem_tv_fit <- function(object, probs = c(0.025, 0.1, 0.5, 0.9, 0.975), ...) {
  .sigma_trajectory_tv_generic(object, probs, "sigma_trajectory.dcvar_sem_tv_fit()")
}

#' @rdname sigma_trajectory
#' @export
sigma_trajectory.dcvar_multilevel_tv_fit <- function(object, probs = c(0.025, 0.1, 0.5, 0.9, 0.975), ...) {
  .sigma_trajectory_tv_generic(object, probs, "sigma_trajectory.dcvar_multilevel_tv_fit()")
}

.tv_margin_groups <- function(object, D = 2L) {
  names(.mixed_margin_report_vars(rep(object$margins %||% "normal", length.out = D)))
}

#' @rdname var_params
#' @export
var_params.dcvar_sem_tv_fit <- function(object, ...) {
  margin_groups <- .tv_margin_groups(object, 2L)
  required_patterns <- c("^mu\\[", "^Phi\\[", "^rho\\[", "^sigma_omega$",
                         paste0("^", margin_groups, "\\["))
  if (isTRUE(object$tv_phi)) required_patterns <- c(required_patterns, "^tau_phi\\[")
  if (isTRUE(object$tv_sigma)) required_patterns <- c(required_patterns, "^tau_sigma\\[")

  summ <- .fit_summary(
    object$fit, variables = NULL, backend = object$backend,
    required = required_patterns,
    required_type = "pattern",
    context = "var_params.dcvar_sem_tv_fit()",
    output_type = "parameter group",
    mean, sd,
    ~posterior::quantile2(.x, probs = c(0.025, 0.975))
  )
  extract_param <- function(pattern) {
    rows <- grep(pattern, summ$variable)
    data.frame(
      variable = summ$variable[rows],
      mean = summ$mean[rows],
      sd = summ$sd[rows],
      q2.5 = summ$q2.5[rows],
      q97.5 = summ$q97.5[rows]
    )
  }

  result <- list(mu = extract_param("^mu\\["), Phi = extract_param("^Phi\\["))
  for (group in margin_groups) result[[group]] <- extract_param(paste0("^", group, "\\["))
  result$sigma_omega <- extract_param("^sigma_omega$")
  if (isTRUE(object$tv_phi)) {
    tau_phi <- extract_param("^tau_phi\\[")
    tau_phi$variable <- paste0("tau_phi[", .tv_active_phi_coefs(object), "]")
    result$tau_phi <- tau_phi
  }
  if (isTRUE(object$tv_sigma)) result$tau_sigma <- extract_param("^tau_sigma\\[")
  result$rho <- extract_param("^rho\\[")
  result
}

#' @rdname var_params
#' @export
var_params.dcvar_multilevel_tv_fit <- function(object, ...) {
  margin_groups <- .tv_margin_groups(object, 2L)
  required_patterns <- c("^phi_bar\\[", "^tau_phi\\[", "^rho$",
                         paste0("^", margin_groups, "\\["))
  if (isTRUE(object$tv_phi)) required_patterns <- c(required_patterns, "^tau_phi_tv\\[")
  if (isTRUE(object$tv_sigma)) required_patterns <- c(required_patterns, "^tau_sigma\\[")

  summ <- .fit_summary(
    object$fit, variables = NULL, backend = object$backend,
    required = required_patterns,
    required_type = "pattern",
    context = "var_params.dcvar_multilevel_tv_fit()",
    output_type = "parameter group",
    mean, sd,
    ~posterior::quantile2(.x, probs = c(0.025, 0.975))
  )
  extract_param <- function(pattern) {
    rows <- grep(pattern, summ$variable)
    data.frame(
      variable = summ$variable[rows],
      mean = summ$mean[rows],
      sd = summ$sd[rows],
      q2.5 = summ$q2.5[rows],
      q97.5 = summ$q97.5[rows]
    )
  }

  result <- list(
    phi_bar = extract_param("^phi_bar\\["),
    tau_phi = extract_param("^tau_phi\\[")
  )
  for (group in margin_groups) result[[group]] <- extract_param(paste0("^", group, "\\["))
  result$rho <- extract_param("^rho$")
  if (isTRUE(object$tv_phi)) {
    tau_phi_tv <- extract_param("^tau_phi_tv\\[")
    tau_phi_tv$variable <- paste0("tau_phi_tv[", .tv_active_phi_coefs(object), "]")
    result$tau_phi_tv <- tau_phi_tv
  }
  if (isTRUE(object$tv_sigma)) result$tau_sigma <- extract_param("^tau_sigma\\[")
  result
}

# ============================================================================
# Unified dynamic engine (dcvar_dynamic.stan): routing + density equivalence
#
# The DC-VAR time-varying path and the drift covariate model are both served by
# the single dcvar_dynamic.stan engine. These tests pin the routing and prove
# the engine reproduces the legacy densities: an independent R recomputation of
# the covariate log-likelihood (the z-score copula fast path), the beta_0 alias
# identity, and (gated, slow) posterior equivalence with the legacy covariate
# Stan file fitted side by side.
# ============================================================================

# --- routing (fast, no fit) -------------------------------------------------

test_that("the dynamic engine is reachable and public paths remain prepare-compatible", {
  dyn <- dcvar_stan_path("dcvar_dynamic")
  expect_true(file.exists(dyn))
  expect_match(dyn, "dcvar_dynamic\\.stan$")

  # Public model-specific paths match the exported prepare_*() data contracts;
  # wrappers route bundled TV / drift-covariate fits to dcvar_dynamic internally.
  expect_match(dcvar_stan_path("dcvar_tv"), "dcvar_tv_mixed\\.stan$")
  expect_match(dcvar_stan_path("dcvar_covariate"), "dcvar_covariate_ncp\\.stan$")
  expect_match(dcvar_stan_path("dcvar_covariate_nodrift"), "dcvar_covariate_nodrift\\.stan$")

  expect_true(.uses_dynamic_stan_file(NULL))
  expect_true(.uses_dynamic_stan_file(dyn))
  expect_false(.uses_dynamic_stan_file(dcvar_stan_path("dcvar_tv")))
  expect_false(.uses_dynamic_stan_file(dcvar_stan_path("dcvar_covariate")))
})

test_that(".as_dynamic_stan_data fills the engine block for both entry shapes", {
  df <- data.frame(time = 1:30, y1 = rnorm(30), y2 = rnorm(30))

  # TV entry: prepare_dcvar_data output (no covariate fields) -> P = 0 predictor.
  tv <- prepare_dcvar_data(df, c("y1", "y2"), tv_phi = "ar")
  tv_aug <- .as_dynamic_stan_data(tv)
  expect_identical(tv_aug$P, 0L)
  expect_identical(dim(tv_aug$X), c(30L, 0L))
  expect_identical(tv_aug$zero_init_eta, 0L)
  expect_identical(tv_aug$copula_normal_fastpath, 0L)
  # TV settings are preserved, not overwritten.
  expect_identical(tv_aug$tv_phi, 1L)
  expect_identical(unname(tv_aug$phi_tv_mask), c(1L, 0L, 0L, 1L))

  # Covariate entry: prepare_dcvar_covariate_data output -> margin/TV defaults.
  dfc <- df
  dfc$phase <- as.numeric(dfc$time > 15)
  cov <- prepare_dcvar_covariate_data(dfc, c("y1", "y2"), covariates = "phase")
  cov$copula_normal_fastpath <- 1L
  cov_aug <- .as_dynamic_stan_data(cov)
  expect_identical(cov_aug$family, c(1L, 1L))
  expect_identical(cov_aug$tv_phi, 0L)
  expect_identical(cov_aug$tv_sigma, 0L)
  expect_identical(cov_aug$P, 1L)
  expect_identical(cov_aug$copula_normal_fastpath, 1L)
  # The full superset block must be present (every engine-declared field).
  needed <- c("family", "skew_direction", "tv_phi", "phi_tv_mask", "tv_sigma",
              "tau_phi_prior", "tau_sigma_prior", "barrier_k", "sigma_eps_prior",
              "P", "X", "sigma_beta_prior", "zero_init_eta", "copula_normal_fastpath")
  expect_true(all(needed %in% names(cov_aug)))
})

test_that(".init_dcvar_dynamic_params omits inactive components and shapes length-1 vectors", {
  # P = 0, drift first state non-zero, no TV: beta and TV containers absent.
  i0 <- .init_dcvar_dynamic_params(2, 20, "normal", FALSE, FALSE, P = 0L, zero_init_eta = FALSE)
  expect_null(i0$beta)
  expect_null(i0$tau_phi)
  expect_null(i0$sigma_raw)
  expect_false("beta_0" %in% names(i0))      # the engine samples z_rho_init, not beta_0
  expect_true("z_rho_init" %in% names(i0))
  expect_length(i0$omega_raw, 19L)           # n_eff - 0

  # P = 1, zero_init_eta = TRUE: beta is length-1 array, omega_raw is n_eff - 1.
  i1 <- .init_dcvar_dynamic_params(2, 20, "normal", FALSE, FALSE, P = 1L, zero_init_eta = TRUE)
  expect_identical(dim(i1$beta), 1L)
  expect_length(i1$omega_raw, 18L)
})

# --- density equivalence (fast: one compiled fit, deterministic recomputation) --

test_that("covariate engine exposes beta_0 as a draw-for-draw alias of z_rho_init", {
  skip_if_no_rstan()

  fit <- get_covariate_fit(drift = TRUE)
  dm <- posterior::as_draws_matrix(draws(fit))
  expect_true(all(c("beta_0", "z_rho_init") %in% posterior::variables(dm)))
  expect_equal(as.numeric(dm[, "beta_0"]), as.numeric(dm[, "z_rho_init"]))
  expect_true(isTRUE(fit$dynamic_engine))
  monitored <- .diagnostic_parameter_variables(fit)
  expect_true("z_rho_init" %in% monitored)
  expect_false("beta_0" %in% monitored)
})

test_that("covariate engine log_lik matches an independent R recomputation", {
  skip_if_no_rstan()

  # Recompute log_lik[t] for one posterior draw using the documented covariate
  # density: independent normal margins + the z-score Gaussian copula on the
  # standardized residuals (the engine's fast path). A wrong fast path, scale, or
  # rho assembly would make these disagree.
  fit <- get_covariate_fit(drift = TRUE)
  dm <- posterior::as_draws_matrix(draws(fit))
  s <- 1L
  g <- function(nm) as.numeric(dm[s, nm])

  n_eff <- fit$stan_data$n_time - 1L
  sigma_eps <- c(g("sigma_eps[1]"), g("sigma_eps[2]"))

  eps <- matrix(NA_real_, n_eff, 2L)
  for (d in 1:2) {
    eps[, d] <- vapply(seq_len(n_eff), function(t) g(sprintf("eps[%d,%d]", t, d)), numeric(1))
  }
  rho <- vapply(seq_len(n_eff), function(t) g(sprintf("rho[%d]", t)), numeric(1))

  gauss_copula_z <- function(z1, z2, r) {
    rs <- r^2
    -0.5 * log1p(-rs) - (rs * (z1^2 + z2^2) - 2 * r * z1 * z2) / (2 * (1 - rs))
  }
  ll_R <- vapply(seq_len(n_eff), function(t) {
    z1 <- eps[t, 1] / sigma_eps[1]
    z2 <- eps[t, 2] / sigma_eps[2]
    stats::dnorm(eps[t, 1], 0, sigma_eps[1], log = TRUE) +
      stats::dnorm(eps[t, 2], 0, sigma_eps[2], log = TRUE) +
      gauss_copula_z(z1, z2, rho[t])
  }, numeric(1))

  ll_stan <- vapply(seq_len(n_eff), function(t) g(sprintf("log_lik[%d]", t)), numeric(1))
  expect_equal(ll_R, ll_stan, tolerance = 1e-8)
})

# --- engine coverage: TV path, multiple covariates, alt drift, methods ------

test_that("the TV path runs on the unified engine and emits its outputs (incl. beta_0 alias)", {
  skip_if_no_rstan()

  fit <- get_dcvar_tv_fit()
  expect_s3_class(fit, "dcvar_tv_fit")
  v <- posterior::variables(draws(fit))
  # The engine exposes beta_0 (= z_rho_init) even on the P = 0 TV path.
  expect_true(all(c("z_rho_init", "beta_0", "rho[1]", "log_lik[1]") %in% v))
  expect_identical(setdiff(.diagnostic_parameter_variables(fit), v), character(0))
})

test_that("explicit dcvar_dynamic stan_file uses the engine data block", {
  skip_if_no_rstan()

  dyn <- dcvar_stan_path("dcvar_dynamic")

  sim_tv <- simulate_dcvar(n_time = 24, rho_trajectory = rho_decreasing(24), seed = 21)
  fit_tv <- dcvar(
    sim_tv$Y_df, vars = c("y1", "y2"), tv_phi = "ar",
    stan_file = dyn,
    chains = 1, iter_warmup = 40, iter_sampling = 40, refresh = 0, seed = 22
  )
  expect_s3_class(fit_tv, "dcvar_tv_fit")
  expect_true("beta_0" %in% posterior::variables(draws(fit_tv)))

  sim_cov <- simulate_dcvar(n_time = 24, rho_trajectory = rho_step(24), seed = 23)
  df <- sim_cov$Y_df
  df$phase <- as.numeric(df$time > 12)
  fit_cov <- dcvar_covariate(
    df, vars = c("y1", "y2"), covariates = "phase",
    stan_file = dyn,
    chains = 1, iter_warmup = 40, iter_sampling = 40, refresh = 0, seed = 24
  )
  expect_s3_class(fit_cov, "dcvar_covariate_fit")
  expect_true(isTRUE(fit_cov$dynamic_engine))
  expect_true("z_rho_init" %in% .diagnostic_parameter_variables(fit_cov))
})

test_that("covariate engine fits with multiple covariates and zero_init_eta = FALSE", {
  skip_if_no_rstan()

  sim <- simulate_dcvar(
    n_time = 35,
    rho_trajectory = rho_step(35, rho_before = 0.6, rho_after = 0.2, breakpoint = 0.5),
    seed = 7
  )
  df <- sim$Y_df
  df$phase <- as.numeric(df$time > 17)
  df$trend <- as.numeric(scale(df$time))

  # P = 2 exercises the dot_product(X, beta) predictor; zero_init_eta = FALSE
  # exercises the compute_rw_ncp drift branch of the engine.
  fit <- dcvar_covariate(
    df, vars = c("y1", "y2"), covariates = c("phase", "trend"),
    drift = TRUE, zero_init_eta = FALSE,
    chains = 1, iter_warmup = 60, iter_sampling = 60, refresh = 0, seed = 11
  )
  expect_s3_class(fit, "dcvar_covariate_fit")
  expect_identical(fit$stan_data$P, 2L)
  expect_identical(fit$stan_data$zero_init_eta, 0L)

  ce <- covariate_effects(fit)
  expect_true(all(c("phase", "trend") %in% ce$term))
  expect_length(coef(fit)$beta, 2L)
  expect_equal(nrow(rho_trajectory(fit)), fit$stan_data$n_time - 1L)
  # Name-pin holds for P = 2, drift, zero_init_eta = FALSE (n_omega = n_eff).
  expect_identical(
    setdiff(.diagnostic_parameter_variables(fit), posterior::variables(draws(fit))),
    character(0)
  )
})

test_that("covariate engine fit supports predict / fitted / pit_values", {
  skip_if_no_rstan()

  fit <- get_covariate_fit(drift = TRUE)
  pr <- predict(fit)
  expect_true(all(pr$lower <= pr$upper))
  ft <- fitted(fit)
  expect_false(is.null(ft))
  pv <- pit_values(fit)
  expect_true(all(pv$pit >= 0 & pv$pit <= 1, na.rm = TRUE))
})

test_that("covariate engine fits under the cmdstanr backend", {
  skip_if_no_cmdstanr_backend()

  # Validates the engine's zero-extent init wrapping on the cmdstanr serializer
  # for the covariate (drift) path specifically.
  sim <- simulate_dcvar(
    n_time = 24,
    rho_trajectory = rho_step(24, rho_before = 0.6, rho_after = 0.2, breakpoint = 0.5),
    seed = 9
  )
  df <- sim$Y_df
  df$phase <- as.numeric(df$time > 12)
  fit <- dcvar_covariate(
    df, vars = c("y1", "y2"), covariates = "phase", drift = TRUE,
    chains = 1, iter_warmup = 50, iter_sampling = 50, refresh = 0,
    seed = 13, backend = "cmdstanr"
  )
  expect_s3_class(fit, "dcvar_covariate_fit")
  expect_identical(fit$backend, "cmdstanr")
  expect_true("beta_0" %in% posterior::variables(draws(fit)))
  expect_equal(nrow(rho_trajectory(fit)), fit$stan_data$n_time - 1L)
})

test_that("a custom stan_file bypasses the dynamic engine on the TV path", {
  skip_if_no_rstan()
  skip_if_not_slow()

  # With a user-supplied legacy TV Stan file, the data is NOT augmented (no
  # covariate predictor / drift fields) and the legacy parameter block is used.
  sim <- simulate_dcvar(
    n_time = 30, rho_trajectory = rho_decreasing(30, 0.6, 0.2), seed = 5
  )
  legacy_tv <- dcvar_stan_path("dcvar_tv")
  expect_true(nzchar(legacy_tv))
  fit <- dcvar(
    sim$Y_df, vars = c("y1", "y2"), tv_phi = "ar",
    stan_file = legacy_tv,
    chains = 1, iter_warmup = 60, iter_sampling = 60, refresh = 0, seed = 6
  )
  expect_s3_class(fit, "dcvar_tv_fit")
  v <- posterior::variables(draws(fit))
  # The legacy TV file has no covariate predictor, so beta_0 is absent.
  expect_false("beta_0" %in% v)
  expect_true(all(c("z_rho_init", "rho[1]") %in% v))
})

# --- posterior equivalence vs the legacy covariate Stan file (slow, gated) ---

test_that("covariate engine reproduces the legacy covariate posterior", {
  skip_if_no_rstan()
  skip_if_not_slow()

  sim <- simulate_dcvar(
    n_time = 50,
    rho_trajectory = rho_step(50, rho_before = 0.6, rho_after = 0.2, breakpoint = 0.5),
    seed = 42
  )
  df <- sim$Y_df
  df$phase <- as.numeric(df$time > 25)

  common <- list(
    data = df, vars = c("y1", "y2"), covariates = "phase",
    chains = 2, iter_warmup = 500, iter_sampling = 500,
    adapt_delta = 0.99, refresh = 0, seed = 999
  )

  fit_engine <- do.call(dcvar_covariate, common)
  legacy_path <- dcvar_stan_path("dcvar_covariate")
  expect_true(nzchar(legacy_path))
  fit_legacy <- do.call(dcvar_covariate, c(common, list(stan_file = legacy_path)))

  ce <- coef(fit_engine)
  cl <- coef(fit_legacy)
  expect_equal(unname(ce$mu), unname(cl$mu), tolerance = 0.05)
  expect_equal(unname(ce$Phi), unname(cl$Phi), tolerance = 0.06)
  expect_equal(unname(ce$sigma_eps), unname(cl$sigma_eps), tolerance = 0.05)
  expect_equal(unname(ce$beta_0), unname(cl$beta_0), tolerance = 0.08)
  expect_equal(unname(ce$beta), unname(cl$beta), tolerance = 0.08)
  expect_equal(unname(ce$sigma_omega), unname(cl$sigma_omega), tolerance = 0.06)

  # The dependence trajectories should agree across the whole series.
  re <- rho_trajectory(fit_engine)$mean
  rl <- rho_trajectory(fit_legacy)$mean
  expect_equal(re, rl, tolerance = 0.05)
})

# ============================================================================
# Routing, data preparation, and fit-object tests for the time-varying DC-VAR
# ============================================================================

test_that("dcvar_tv public paths stay compatible with prepared TV data", {
  # dcvar() routes bundled TV fits to dcvar_dynamic.stan internally, but the
  # public path helper returns the legacy-compatible TV file that matches
  # prepare_dcvar_data(tv_*).
  expect_identical(.margin_stan_file("dcvar_tv", "normal"), "dcvar_tv_mixed.stan")
  expect_identical(.margin_stan_file("dcvar_tv", c("normal", "exponential")), "dcvar_tv_mixed.stan")
  # Homogeneous non-normal also routes to the generic file (no specialised zoo)
  expect_identical(.margin_stan_file("dcvar_tv", c("gamma", "gamma")), "dcvar_tv_mixed.stan")
  expect_error(.margin_stan_file("dcvar_tv", "normal", copula = "clayton"), "Gaussian")

  path <- dcvar_stan_path("dcvar_tv")
  expect_true(file.exists(path))
  expect_match(path, "dcvar_tv_mixed\\.stan$")
})

test_that(".resolve_phi_tv_mask maps logicals and selectors to a row-major mask", {
  expect_identical(unname(.resolve_phi_tv_mask(TRUE)), c(1L, 1L, 1L, 1L))
  expect_identical(unname(.resolve_phi_tv_mask(FALSE)), c(0L, 0L, 0L, 0L))
  # AR = phi11, phi22 (positions 1, 4); cross = phi12, phi21 (positions 2, 3)
  expect_identical(unname(.resolve_phi_tv_mask("ar")), c(1L, 0L, 0L, 1L))
  expect_identical(unname(.resolve_phi_tv_mask("cross")), c(0L, 1L, 1L, 0L))
  expect_identical(unname(.resolve_phi_tv_mask("crosslag")), c(0L, 1L, 1L, 0L))
  expect_identical(unname(.resolve_phi_tv_mask(c("phi11", "phi21"))), c(1L, 0L, 1L, 0L))
  expect_identical(names(.resolve_phi_tv_mask(TRUE)),
                   c("phi11", "phi12", "phi21", "phi22"))

  expect_error(.resolve_phi_tv_mask("nonsense"), "Invalid")
  expect_error(.resolve_phi_tv_mask(NA), "logical")
  expect_error(.resolve_phi_tv_mask(c(TRUE, FALSE)), "logical")
  expect_error(.resolve_phi_tv_mask(character(0)), "empty")
})

test_that("prepare_dcvar_data resolves a tv_phi selector to a coefficient mask", {
  df <- data.frame(time = 1:30, y1 = rnorm(30), y2 = rnorm(30))

  ar <- prepare_dcvar_data(df, vars = c("y1", "y2"), tv_phi = "ar")
  expect_identical(ar$tv_phi, 1L)
  expect_identical(ar$phi_tv_mask, c(1L, 0L, 0L, 1L))
  expect_identical(ar$tv_sigma, 0L)
  expect_identical(attr(ar, "phi_tv_mask"), .resolve_phi_tv_mask("ar"))

  cross <- prepare_dcvar_data(df, vars = c("y1", "y2"), tv_phi = "cross")
  expect_identical(cross$phi_tv_mask, c(0L, 1L, 1L, 0L))

  full <- prepare_dcvar_data(df, vars = c("y1", "y2"), tv_phi = TRUE)
  expect_identical(full$phi_tv_mask, c(1L, 1L, 1L, 1L))
})

test_that("prepare_dcvar_data with flags off is byte-identical to before", {
  df <- data.frame(time = 1:30, y1 = rnorm(30), y2 = rnorm(30))
  plain <- prepare_dcvar_data(df, vars = c("y1", "y2"))
  flagged_off <- prepare_dcvar_data(df, vars = c("y1", "y2"),
                                    tv_phi = FALSE, tv_sigma = FALSE)
  expect_identical(plain, flagged_off)
  expect_null(plain$tv_phi)
  expect_null(plain$family)
})

test_that("prepare_dcvar_data adds the TV fields when a flag is set", {
  df <- data.frame(time = 1:30, y1 = rnorm(30), y2 = rnorm(30))
  out <- prepare_dcvar_data(df, vars = c("y1", "y2"), tv_phi = TRUE, tv_sigma = TRUE)

  expect_identical(out$tv_phi, 1L)
  expect_identical(out$tv_sigma, 1L)
  expect_identical(out$family, c(1L, 1L))
  expect_identical(out$skew_direction, c(1, 1))
  expect_identical(out$tau_phi_prior, 0.025)
  expect_identical(out$tau_sigma_prior, 0.05)
  expect_identical(out$barrier_k, 8)
  expect_true(attr(out, "tv_phi"))
  expect_true(attr(out, "tv_sigma"))

  mixed <- prepare_dcvar_data(df, vars = c("y1", "y2"),
                              margins = c("normal", "exponential"),
                              skew_direction = c(1, -1),
                              tv_phi = TRUE)
  expect_identical(mixed$family, c(1L, 2L))
  expect_identical(mixed$skew_direction, c(1, -1))
})

test_that("prepare_dcvar_data validates the TV flags", {
  df <- data.frame(time = 1:30, y1 = rnorm(30), y2 = rnorm(30))
  expect_error(prepare_dcvar_data(df, c("y1", "y2"), tv_phi = NA), "tv_phi")
  expect_error(prepare_dcvar_data(df, c("y1", "y2"), tv_phi = c(TRUE, TRUE)), "tv_phi")
  expect_error(prepare_dcvar_data(df, c("y1", "y2"), tv_phi = TRUE,
                                  prior_tau_phi_rate = -1), "prior_tau_phi_rate")
})

# --- Smoke fit: structure, extractors, diagnostics name pin -----------------

test_that("dcvar(tv_phi, tv_sigma) fits and returns the subclass", {
  skip_if_no_rstan()

  fit <- get_dcvar_tv_fit()
  expect_s3_class(fit, "dcvar_tv_fit")
  expect_identical(class(fit), c("dcvar_tv_fit", "dcvar_fit", "dcvar_model_fit"))
  expect_identical(fit$model, "dcvar_tv")
  expect_true(fit$tv_phi)
  expect_true(fit$tv_sigma)
  expect_named(fit$priors, c("mu_sd", "phi_sd", "sigma_eps_rate", "sigma_omega_rate",
                             "rho_init_sd", "tau_phi_rate", "tau_sigma_rate"))
})

test_that("TV trajectory extractors return the documented structure", {
  skip_if_no_rstan()

  fit <- get_dcvar_tv_fit()
  n_eff <- fit$stan_data$n_time - 1L

  phi_df <- phi_trajectory(fit)
  expect_s3_class(phi_df, "data.frame")
  expect_equal(nrow(phi_df), 4 * n_eff)
  expect_true(all(c("time", "coefficient", "mean", "sd", "q2.5", "q97.5") %in% names(phi_df)))
  expect_identical(sort(unique(phi_df$coefficient)), c("phi11", "phi12", "phi21", "phi22"))

  sigma_df <- sigma_trajectory(fit)
  expect_equal(nrow(sigma_df), 2 * n_eff)
  expect_identical(sort(unique(sigma_df$variable)), sort(fit$vars))
  expect_true(all(sigma_df$mean > 0))

  rho_df <- rho_trajectory(fit)
  expect_equal(nrow(rho_df), n_eff)
  expect_true(all(abs(rho_df$mean) <= 1))
})

test_that("sigma_trajectory reports skew-normal baselines as residual SD", {
  n_draw <- 20
  variables <- c("omega[1]", "delta[1]", "sigma_eps[2]")
  draws <- array(
    0,
    dim = c(n_draw, 1, length(variables)),
    dimnames = list(NULL, NULL, variables)
  )
  draws[, , "omega[1]"] <- 2
  draws[, , "delta[1]"] <- 0.6
  draws[, , "sigma_eps[2]"] <- 1.25
  stan_data <- structure(
    list(D = 2L, n_time = 4L),
    time_values = 11:14
  )
  fit <- new_dcvar_tv_fit(
    fit = posterior::as_draws_array(draws),
    stan_data = stan_data,
    vars = c("y1", "y2"),
    standardized = FALSE,
    margins = c("skew_normal", "normal"),
    tv_phi = FALSE,
    tv_sigma = FALSE,
    backend = "rstan",
    priors = list(),
    meta = list(chains = 1, iter_warmup = 0, iter_sampling = n_draw)
  )

  sigma_df <- sigma_trajectory(fit)
  y1 <- sigma_df[sigma_df$variable == "y1", ]
  y2 <- sigma_df[sigma_df$variable == "y2", ]
  expect_equal(y1$mean, rep(2 * sqrt(1 - 2 * 0.6^2 / pi), 3), tolerance = 1e-12)
  expect_equal(y2$mean, rep(1.25, 3), tolerance = 1e-12)
})

test_that("TV Stan generated quantities report skew-normal sigma_t as residual SD", {
  read_stan <- function(model) paste(readLines(dcvar_stan_path(model), warn = FALSE), collapse = " ")
  residual_sd_expr <- "sqrt\\(1 - 2 \\* square\\(delta\\[i\\]\\) / pi\\(\\)\\)"

  expect_match(read_stan("dcvar_dynamic"), residual_sd_expr)
  expect_match(read_stan("dcvar_tv"), residual_sd_expr)
  expect_match(
    paste(readLines(dcvar_stan_path("sem_tv"), warn = FALSE), collapse = " "),
    residual_sd_expr
  )
  expect_match(
    paste(readLines(dcvar_stan_path("multilevel_tv"), warn = FALSE), collapse = " "),
    "sqrt\\(1 - 2 \\* square\\(delta\\[j\\]\\) / pi\\(\\)\\)"
  )
})

test_that("TV fit methods run and report the time-varying components", {
  skip_if_no_rstan()

  fit <- get_dcvar_tv_fit()

  out <- capture.output(print(fit))
  expect_true(any(grepl("Phi\\(t\\)", out)))

  s <- summary(fit)
  expect_s3_class(s, "dcvar_tv_summary")
  expect_false(is.null(s$phi_summary))
  expect_false(is.null(s$sigma_summary))
  expect_silent(invisible(capture.output(print(s))))

  co <- coef(fit)
  expect_true(all(c("mu", "Phi", "tau_phi", "tau_sigma", "sigma_omega") %in% names(co)))
  expect_length(co$tau_phi, 4)

  vp <- var_params(fit)
  expect_true(all(c("mu", "Phi", "tau_phi", "tau_sigma", "sigma_omega") %in% names(vp)))

  expect_s3_class(plot(fit, type = "phi"), "ggplot")
  expect_s3_class(plot(fit, type = "sigma"), "ggplot")
  expect_s3_class(plot(fit, type = "rho"), "ggplot")

  pit_df <- pit_values(fit)
  expect_true(all(pit_df$pit >= 0 & pit_df$pit <= 1, na.rm = TRUE))

  pred <- predict(fit)
  expect_true(all(pred$lower <= pred$upper))

  ll <- loo(fit)
  expect_s3_class(ll, "loo")
})

test_that("diagnostics monitoring list matches the sampled Stan parameters", {
  skip_if_no_rstan()

  fit <- get_dcvar_tv_fit()
  monitored <- .diagnostic_parameter_variables(fit)
  available <- posterior::variables(draws(fit))
  missing <- setdiff(monitored, available)
  expect_identical(missing, character(0))

  diag <- dcvar_diagnostics(fit)
  expect_true(is.finite(diag$max_rhat))
})

test_that("phi/sigma trajectories tile constants when a flag is off", {
  skip_if_no_rstan()

  fit <- get_dcvar_tv_sigma_only_fit()
  expect_false(fit$tv_phi)
  n_eff <- fit$stan_data$n_time - 1L

  phi_df <- phi_trajectory(fit)
  expect_equal(nrow(phi_df), 4 * n_eff)
  # Constant baseline tiled: one unique mean per coefficient
  per_coef <- tapply(phi_df$mean, phi_df$coefficient, function(x) length(unique(x)))
  expect_true(all(per_coef == 1L))

  sigma_df <- sigma_trajectory(fit)
  expect_equal(nrow(sigma_df), 2 * n_eff)
})

test_that("AR-only tv_phi fits, sizes tau_phi compactly, and pins diagnostics", {
  skip_if_no_rstan()

  fit <- get_dcvar_tv_ar_fit()
  expect_s3_class(fit, "dcvar_tv_fit")
  expect_true(fit$tv_phi)
  expect_false(fit$tv_sigma)
  expect_identical(unname(fit$phi_tv_mask), c(1L, 0L, 0L, 1L))
  expect_identical(names(fit$phi_tv_mask), c("phi11", "phi12", "phi21", "phi22"))

  # tau_phi is sized to the 2 active coefficients and labelled by name
  co <- coef(fit)
  expect_length(co$tau_phi, 2)
  expect_identical(names(co$tau_phi), c("tau_phi[phi11]", "tau_phi[phi22]"))

  vp <- var_params(fit)
  expect_identical(vp$tau_phi$variable, c("tau_phi[phi11]", "tau_phi[phi22]"))

  # The monitored diagnostic parameters must exist in the draws AND the draws
  # must not contain a tau_phi index beyond the active count -- pins the masked
  # tau_phi sizing in both directions (over- and under-counting).
  monitored <- .diagnostic_parameter_variables(fit)
  available <- posterior::variables(draws(fit))
  expect_identical(setdiff(monitored, available), character(0))
  expect_true(all(c("tau_phi[1]", "tau_phi[2]") %in% available))
  expect_false("tau_phi[3]" %in% available)

  # phi_trajectory still returns all four coefficients; the cross-lagged ones
  # are constant (deviation forced to zero), while the active AR coefficients
  # genuinely vary -- the latter ties the masked-in coefficients to actual
  # time variation (catches a mask that only leaves phi12/phi21 off).
  phi_df <- phi_trajectory(fit)
  expect_identical(sort(unique(phi_df$coefficient)), c("phi11", "phi12", "phi21", "phi22"))
  uniq <- tapply(phi_df$mean, phi_df$coefficient, function(x) length(unique(x)))
  expect_equal(unname(uniq[["phi12"]]), 1L)
  expect_equal(unname(uniq[["phi21"]]), 1L)
  # phi11 is simulated as a decreasing path, so its posterior-mean trajectory
  # must not collapse to a constant.
  expect_gt(unname(uniq[["phi11"]]), 1L)

  out <- capture.output(print(fit))
  expect_true(any(grepl("phi11", out)))

  expect_s3_class(plot(fit, type = "diagnostics"), "ggplot")
})

test_that("dcvar() warns about ignored TV prior args and exp/gamma scales", {
  df <- data.frame(time = 1:25, y1 = rnorm(25), y2 = rnorm(25))
  expect_warning(
    tryCatch(
      dcvar(df, c("y1", "y2"), tv_phi = FALSE, prior_tau_phi_rate = 0.5,
            chains = 1, iter_warmup = 2, iter_sampling = 2, refresh = 0,
            stan_file = NA_character_),
      error = function(e) NULL
    ),
    "prior_tau_phi_rate"
  )
})

test_that("dcvar_compare warns when mixing TV and non-TV fits", {
  skip_if_no_rstan()

  # The cached fits use different data lengths, so loo_compare() itself errors
  # after the warning under test has fired; swallow the error.
  w <- testthat::capture_warnings(
    try(dcvar_compare(tv = get_dcvar_tv_fit(), constant = get_constant_fit()),
        silent = TRUE)
  )
  expect_true(any(grepl("latent paths", w)))
})

test_that("soft-barrier exp margin with tv_sigma fits and exposes a scale path", {
  skip_if_no_rstan()

  fit <- get_dcvar_tv_exp_fit()
  expect_s3_class(fit, "dcvar_tv_fit")
  expect_true(fit$tv_sigma)
  expect_identical(unname(rep(fit$margins, length.out = 2)), c("normal", "exponential"))

  # sigma_t exists for both dims and is positive; the exp dim genuinely varies
  sigma_df <- sigma_trajectory(fit)
  expect_identical(sort(unique(sigma_df$variable)), sort(fit$vars))
  expect_true(all(sigma_df$mean > 0))
  exp_dim <- sigma_df[sigma_df$variable == fit$vars[2], ]
  expect_gt(length(unique(exp_dim$mean)), 1L)

  # Monitored diagnostics exist (the soft-barrier path keeps the union names)
  monitored <- .diagnostic_parameter_variables(fit)
  expect_identical(setdiff(monitored, posterior::variables(draws(fit))), character(0))

  ll <- loo(fit)
  expect_s3_class(ll, "loo")
})

test_that("Stan log_lik matches an independent R recomputation (soft-barrier Jacobian is used)", {
  skip_if_no_rstan()

  # Recompute log_lik[t] in R for one posterior draw of the normal+exponential
  # soft-barrier fit (skew_direction = c(1, 1), barrier_k = 8) and compare to
  # the Stan-emitted log_lik. A dropped, doubled, or sign-flipped Jacobian in
  # the Stan model/GQ would make these disagree, which the orientation tests
  # cannot detect.
  fit <- get_dcvar_tv_exp_fit()
  k <- 8
  dm <- posterior::as_draws_matrix(draws(fit))
  s <- 1L  # a single draw suffices; the formula is deterministic given it
  g <- function(name) as.numeric(dm[s, name])

  n_eff <- fit$stan_data$n_time - 1L
  sigma_eps1 <- g("sigma_eps[1]")
  eta2 <- g("eta[2]")

  softplus <- function(a) log1p(exp(k * a)) / k
  log_jac <- function(a) stats::plogis(k * a, log.p = TRUE)
  cop_uv <- function(u1, u2, rho) {
    uu <- min(max(u1, 1e-9), 1 - 1e-9)
    vv <- min(max(u2, 1e-9), 1 - 1e-9)
    z1 <- stats::qnorm(uu); z2 <- stats::qnorm(vv)
    -0.5 * log1p(-rho^2) -
      0.5 / (1 - rho^2) * (z1^2 - 2 * rho * z1 * z2 + z2^2) +
      0.5 * (z1^2 + z2^2)
  }

  for (t in c(1L, 5L, 10L)) {
    e1 <- g(paste0("eps[", t, ",1]"))
    e2 <- g(paste0("eps[", t, ",2]"))
    sd1 <- g(paste0("sigma_dev[", t, ",1]"))
    sd2 <- g(paste0("sigma_dev[", t, ",2]"))
    rho_t <- g(paste0("rho[", t, "]"))

    # normal dim (multiplicative log-scale walk)
    s1 <- sigma_eps1 * exp(sd1)
    ll1 <- stats::dnorm(e1, 0, s1, log = TRUE)
    u1 <- stats::pnorm(e1 / s1)
    # exponential dim (soft-barrier, skew = +1): includes the eps->x Jacobian
    m_t <- exp(eta2 + sd2)
    arg <- m_t + e2
    x <- softplus(arg)
    ll2 <- stats::dexp(x, rate = 1 / m_t, log = TRUE) + log_jac(arg)
    u2 <- stats::pexp(x, rate = 1 / m_t)

    ll_r <- ll1 + ll2 + cop_uv(u1, u2, rho_t)
    expect_equal(g(paste0("log_lik[", t, "]")), ll_r, tolerance = 1e-5,
                 info = sprintf("t=%d", t))
  }
})

# --- Gated recovery tests (DCVAR_SLOW_TESTS) ---------------------------------

test_that("TV fit recovers a phi12 step and a sigma ramp directionally", {
  skip_if_no_rstan()
  skip_if_not_slow()

  n <- 200
  sim <- simulate_dcvar(
    n, rho_constant(n, 0.4),
    phi_trajectory = list(
      rho_constant(n, 0.3),
      rho_step(n, rho_before = 0.5, rho_after = 0.0, breakpoint = 0.5),
      rho_constant(n, 0.1),
      rho_constant(n, 0.3)
    ),
    sigma_trajectory = cbind(seq(0.7, 1.8, length.out = n - 1), rep(1, n - 1)),
    seed = 7
  )
  fit <- dcvar(sim$Y_df, vars = c("y1", "y2"),
               tv_phi = TRUE, tv_sigma = TRUE,
               standardize = FALSE,
               chains = 2,
               iter_warmup = margin_iter_warmup,
               iter_sampling = margin_iter_sampling,
               refresh = 0, seed = 11)

  phi_df <- phi_trajectory(fit)
  phi12 <- phi_df[phi_df$coefficient == "phi12", ]
  n_eff <- nrow(phi12)
  expect_gt(mean(phi12$mean[1:50]), mean(phi12$mean[(n_eff - 49):n_eff]))

  sigma_df <- sigma_trajectory(fit)
  s1 <- sigma_df[sigma_df$variable == "y1", ]
  expect_gt(mean(s1$mean[(n_eff - 49):n_eff]), mean(s1$mean[1:50]))
})

test_that("tv_phi = 'ar' ties the fitted variation to the correct coefficient", {
  skip_if_no_rstan()
  skip_if_not_slow()

  # Only phi11 genuinely varies (a clear ramp); phi22 is constant. Under
  # tv_phi = "ar" both phi11 and phi22 carry a walk, so this test confirms the
  # FITTED phi11(t) path moves substantially more than phi22(t) -- i.e. the
  # active tau_phi/phi_dev columns map to the coefficient they are labelled
  # with. A Stan-idx vs R-name transposition (phi11 <-> phi22) would fail here.
  n <- 200
  sim <- simulate_dcvar(
    n, rho_constant(n, 0.4),
    phi_trajectory = list(
      rho_decreasing(n, 0.5, 0.05),  # phi11: strong ramp
      rho_constant(n, 0.1),          # phi12: constant (fixed by the mask)
      rho_constant(n, 0.1),          # phi21: constant (fixed by the mask)
      rho_constant(n, 0.3)           # phi22: constant
    ),
    seed = 13
  )
  fit <- dcvar(sim$Y_df, vars = c("y1", "y2"),
               tv_phi = "ar",
               standardize = FALSE,
               chains = 2,
               iter_warmup = margin_iter_warmup,
               iter_sampling = margin_iter_sampling,
               refresh = 0, seed = 17)

  phi_df <- phi_trajectory(fit)
  rng <- tapply(phi_df$mean, phi_df$coefficient, function(x) diff(range(x)))
  # phi11 (the truly varying coefficient) must move far more than phi22.
  expect_gt(rng[["phi11"]], rng[["phi22"]])
  # And phi11's recovered path must decrease (it was simulated decreasing).
  phi11 <- phi_df[phi_df$coefficient == "phi11", ]
  n_eff <- nrow(phi11)
  expect_gt(mean(phi11$mean[1:50]), mean(phi11$mean[(n_eff - 49):n_eff]))
})

test_that("soft-barrier recovers a time-varying exponential scale ramp", {
  skip_if_no_rstan()
  skip_if_not_slow()

  n <- 200
  scale_path <- seq(0.6, 1.8, length.out = n - 1)  # rising exp scale
  sim <- simulate_dcvar(
    n, rho_constant(n, 0.4),
    margins = c("normal", "exponential"),
    skew_direction = c(1, 1),
    sigma_trajectory = cbind(rep(1, n - 1), scale_path),
    tv_sigma_k = 8,
    seed = 23
  )
  fit <- dcvar(sim$Y_df, vars = c("y1", "y2"),
               margins = c("normal", "exponential"),
               skew_direction = c(1, 1),
               tv_sigma = TRUE, tv_sigma_k = 8,
               standardize = FALSE,
               chains = 2,
               iter_warmup = margin_iter_warmup,
               iter_sampling = margin_iter_sampling,
               refresh = 0, seed = 29)

  sigma_df <- sigma_trajectory(fit)
  exp_dim <- sigma_df[sigma_df$variable == "y2", ]
  n_eff <- nrow(exp_dim)
  # The recovered scale path must rise (matching the simulated ramp).
  expect_gt(mean(exp_dim$mean[(n_eff - 49):n_eff]), mean(exp_dim$mean[1:50]))
})

# ============================================================================
# Real-MCMC recovery tests for the time-varying SEM and multilevel engines
# (v0.9.0). The other TV test files exercise routing / data-prep / extractor
# contracts on stub fits; these tests actually sample sem_tv_mixed.stan and
# multilevel_tv_mixed.stan and check that the time-varying trajectories are
# recovered. Gated as slow MCMC fits (set DCVAR_SLOW_TESTS=true to run).
# ============================================================================

# Correlation of a posterior-mean trajectory (ordered by time) with the truth.
.traj_corr <- function(df, key_col, key, truth) {
  sub <- df[df[[key_col]] == key, , drop = FALSE]
  sub <- sub[order(sub$time), , drop = FALSE]
  stats::cor(sub$mean, truth)
}

test_that("time-varying SEM recovers AR / scale / coupling trajectories (MCMC)", {
  skip_if_not_slow()
  skip_if_no_rstan()

  J <- 2L
  n_time <- 200L
  n_eff <- n_time - 1L
  # phi11 (AR) ramps 0.0 -> 0.5; phi22 held constant (nominally TV under "ar");
  # sigma_1 ramps up; the Gaussian-copula correlation declines.
  phi_traj <- list(
    seq(0.0, 0.5, length.out = n_eff), # phi11 (varying)
    rep(0.10, n_eff),                  # phi12
    rep(0.05, n_eff),                  # phi21
    rep(0.30, n_eff)                   # phi22 (constant)
  )
  sigma_traj <- cbind(seq(0.8, 1.4, length.out = n_eff), rep(1.0, n_eff))
  sim <- simulate_dcvar_sem(
    n_time = n_time, J = J, lambda = rep(sqrt(0.8), J), sigma_e = sqrt(0.2),
    rho_trajectory = rho_decreasing(n_time, 0.7, 0.1),
    phi_trajectory = phi_traj, sigma_trajectory = sigma_traj, seed = 42
  )
  indicators <- list(
    latent1 = paste0("y1_", seq_len(J)),
    latent2 = paste0("y2_", seq_len(J))
  )

  fit <- dcvar_sem(
    sim$data, indicators = indicators, J = J,
    lambda = rep(sqrt(0.8), J), sigma_e = sqrt(0.2),
    tv_phi = "ar", tv_sigma = TRUE,
    chains = 2, iter_warmup = 500, iter_sampling = 500,
    adapt_delta = 0.995, max_treedepth = 12, refresh = 0, seed = 123
  )

  expect_s3_class(fit, "dcvar_sem_tv_fit")

  tp <- sim$true_params
  phi_df <- phi_trajectory(fit)
  sigma_df <- sigma_trajectory(fit)
  rho_df <- rho_trajectory(fit)

  # Varying AR coefficient is recovered; the nominally-TV constant one stays flat.
  expect_gt(.traj_corr(phi_df, "coefficient", "phi11", tp$Phi[, 1]), 0.5)
  expect_lt(stats::sd(phi_df$mean[phi_df$coefficient == "phi22"]), 0.15)
  # Varying innovation scale and time-varying coupling are recovered.
  svar <- sort(unique(sigma_df$variable))
  expect_gt(.traj_corr(sigma_df, "variable", svar[1], tp$sigma[, 1]), 0.6)
  expect_gt(stats::cor(rho_df$mean[order(rho_df$time)], tp$rho), 0.7)

  # No catastrophic non-convergence.
  expect_lt(dcvar_diagnostics(fit)$max_rhat, 1.15)
})

test_that("time-varying multilevel recovers cross-lag / scale trajectories (MCMC)", {
  skip_if_not_slow()
  skip_if_no_rstan()

  N <- 8L
  n_time <- 80L
  n_eff <- n_time - 1L
  # phi12 (cross-lag) population drift ramps 0.0 -> 0.3; phi21 held at 0;
  # sigma_1 ramps up; rho is constant (multilevel copula correlation is global).
  phi_traj <- list(
    rep(0.30, n_eff),                  # phi11 (constant baseline)
    seq(0.0, 0.30, length.out = n_eff),# phi12 (varying, "cross")
    rep(0.0, n_eff),                   # phi21
    rep(0.30, n_eff)                   # phi22
  )
  sigma_traj <- cbind(seq(0.8, 1.3, length.out = n_eff), rep(1.0, n_eff))
  sim <- simulate_dcvar_multilevel(
    N = N, n_time = n_time, rho = 0.4,
    phi_trajectory = phi_traj, sigma_trajectory = sigma_traj, seed = 42
  )

  fit <- dcvar_multilevel(
    sim$data, vars = c("y1", "y2"), id_var = "id",
    tv_phi = "cross", tv_sigma = TRUE,
    chains = 2, iter_warmup = 500, iter_sampling = 500,
    adapt_delta = 0.95, max_treedepth = 13, refresh = 0, seed = 123
  )

  expect_s3_class(fit, "dcvar_multilevel_tv_fit")

  tp <- sim$true_params
  phi_pop <- phi_trajectory(fit) # population drift (unit = NULL)
  sigma_df <- sigma_trajectory(fit)

  # Varying cross-lag (population path) recovered; constant baseline stays flat.
  expect_gt(.traj_corr(phi_pop, "coefficient", "phi12", tp$Phi_population[, 2]), 0.5)
  expect_lt(stats::sd(phi_pop$mean[phi_pop$coefficient == "phi11"]), 0.1)
  # Varying innovation scale recovered.
  svar <- sort(unique(sigma_df$variable))
  expect_gt(.traj_corr(sigma_df, "variable", svar[1], tp$sigma[, 1]), 0.6)

  expect_lt(dcvar_diagnostics(fit)$max_rhat, 1.15)
})

# ============================================================================
# Full Markov-switching HMM (hmm_switching.stan): routing, resolvers, data prep,
# init shapes, positional compatibility, and (gated) MCMC behaviour.
# ============================================================================

# --- routing (fast, no fit) -------------------------------------------------

test_that("the switching engine is reachable and the public HMM path is unchanged", {
  sw_path <- dcvar_stan_path("hmm_switching")
  expect_true(file.exists(sw_path))
  expect_match(sw_path, "hmm_switching\\.stan$")

  # The public default HMM path is the legacy file; the engine is internal.
  expect_match(dcvar_stan_path("hmm"), "hmm_copula_model\\.stan$")
  expect_identical(.margin_stan_file("hmm_switching", "normal"), "hmm_switching.stan")

  # The switching predicate only recognises the bundled engine file (not NULL).
  expect_false(.uses_hmm_switching_stan_file(NULL))
  expect_true(.uses_hmm_switching_stan_file(sw_path))
  expect_false(.uses_hmm_switching_stan_file(dcvar_stan_path("hmm")))
})

test_that("dcvar_hmm keeps time_var positional and adds switch after skew_direction", {
  fmls <- names(formals(dcvar_hmm))
  # Positional contract: data, vars, K, time_var, standardize, margins, skew_direction
  expect_identical(fmls[1:7],
                   c("data", "vars", "K", "time_var", "standardize", "margins", "skew_direction"))
  expect_gt(which(fmls == "switch"), which(fmls == "skew_direction"))
})

# --- switch resolver --------------------------------------------------------

test_that(".resolve_switch_spec maps selectors and enforces the rho anchor", {
  rho_only <- .resolve_switch_spec("rho")
  expect_identical(rho_only$mu, 0L)
  expect_identical(unname(rho_only$phi_mask), c(0L, 0L, 0L, 0L))
  expect_identical(rho_only$rho, 1L)
  expect_identical(rho_only$margins, 0L)

  expect_identical(.resolve_switch_spec(c("rho", "mu"))$mu, 1L)
  expect_identical(unname(.resolve_switch_spec(c("rho", "ar"))$phi_mask), c(1L, 0L, 0L, 1L))
  expect_identical(unname(.resolve_switch_spec(c("rho", "cross"))$phi_mask), c(0L, 1L, 1L, 0L))
  expect_identical(unname(.resolve_switch_spec(c("rho", "phi"))$phi_mask), c(1L, 1L, 1L, 1L))
  expect_identical(.resolve_switch_spec(c("rho", "sigma"))$margins, 1L)
  expect_identical(unname(.resolve_switch_spec(c("rho", "mu", "phi", "sigma"))$phi_mask), c(1L, 1L, 1L, 1L))
  expect_identical(.resolve_switch_spec(c("rho", "mu", "phi", "sigma"))$margins, 1L)

  full <- .resolve_switch_spec(TRUE)
  expect_identical(full$mu, 1L)
  expect_identical(unname(full$phi_mask), c(1L, 1L, 1L, 1L))
  expect_identical(full$margins, 1L)

  # rho is mandatory whenever anything else switches
  expect_error(.resolve_switch_spec("mu"), "rho")
  expect_error(.resolve_switch_spec(c("mu", "phi")), "rho")
  # degenerate / invalid selectors
  expect_error(.resolve_switch_spec(FALSE), "at least one")
  expect_error(.resolve_switch_spec("none"), "at least one")
  expect_error(.resolve_switch_spec(character(0)), "at least one")
  expect_error(.resolve_switch_spec(c("rho", "nonsense")), "Invalid")
  expect_error(.resolve_switch_spec(c("rho", "mu", "Phi", "sigma")), "Invalid")
})

# --- per-state margin configuration -----------------------------------------

test_that(".hmm_margin_config handles global, per-state, and collapse", {
  # Global scalar
  g <- .hmm_margin_config("normal", NULL, K = 2)
  expect_false(g$per_state)
  expect_identical(g$family, matrix(1L, 2, 2))
  expect_identical(g$margins_global, "normal")

  # Per-state with differing families
  ps <- .hmm_margin_config(list(c("normal", "normal"), c("exponential", "gamma")),
                           skew_direction = c(1, 1), K = 2)
  expect_true(ps$per_state)
  expect_true(ps$per_state_differ)
  # state 1 = (normal, normal) -> (1, 1); state 2 = (exponential, gamma) -> (2, 4)
  expect_identical(ps$family, matrix(c(1L, 1L, 2L, 4L), nrow = 2, byrow = TRUE))

  # All-identical per-state list collapses to the global form (legacy path)
  col <- .hmm_margin_config(list("normal", "normal"), NULL, K = 2)
  expect_false(col$per_state)
  expect_false(col$per_state_differ)

  col_exp <- .hmm_margin_config(list("exponential", "exponential"),
                                skew_direction = list(c(1, -1), c(1, -1)),
                                K = 2)
  expect_false(col_exp$per_state)
  expect_identical(col_exp$skew_global, c(1L, -1L))

  # Differing consulted skew orientation is itself state-specific and needs the engine.
  skew_diff <- .hmm_margin_config(list("exponential", "exponential"),
                                  skew_direction = list(c(1, 1), c(-1, -1)),
                                  K = 2)
  expect_true(skew_diff$per_state)
  expect_true(skew_diff$per_state_differ)

  # Length mismatch and missing skew error (naming the state)
  expect_error(.hmm_margin_config(list("normal"), NULL, K = 2), "length-2 list")
  expect_error(
    .hmm_margin_config(list(c("normal", "normal"), c("exponential", "normal")), NULL, K = 2),
    "State 2"
  )
})

test_that(".as_hmm_switching_stan_data builds the engine block; init has the right shapes", {
  df <- data.frame(time = 1:30, y1 = rnorm(30), y2 = rnorm(30))
  base <- prepare_hmm_data(df, c("y1", "y2"), K = 2)
  sw <- .resolve_switch_spec(c("rho", "mu", "phi"))
  cfg <- .hmm_margin_config(list(c("normal", "normal"), c("exponential", "exponential")),
                            skew_direction = c(1, 1), K = 2)
  sw$margins <- 1L
  aug <- .as_hmm_switching_stan_data(base, sw, cfg$family, cfg$skew, prior_phi_dev_sd = 0.5)
  expect_identical(dim(aug$family), c(2L, 2L))
  expect_identical(dim(aug$skew_direction), c(2L, 2L))
  expect_identical(aug$switch_mu, 1L)
  expect_identical(aug$switch_phi, 1L)
  expect_identical(aug$phi_switch_mask, c(1L, 1L, 1L, 1L))
  expect_identical(aug$switch_margins, 1L)
  expect_true(is.numeric(aug$prior_phi_dev_sd))

  # init: state-indexed switching groups; Phi_dev present (phi switches)
  i1 <- .init_hmm_switching_params(2, 2, sw)
  expect_identical(dim(i1$mu), c(2L, 2L))
  expect_identical(dim(i1$sigma_eps), c(2L, 2L))
  expect_identical(dim(i1$Phi_dev), c(2L, 2L, 2L))
  expect_length(i1$z_rho, 2L)

  # rho-only: shared groups length 1, no Phi_dev
  i0 <- .init_hmm_switching_params(2, 2, .resolve_switch_spec("rho"))
  expect_identical(dim(i0$mu), c(1L, 2L))
  expect_identical(dim(i0$sigma_eps), c(1L, 2L))
  expect_null(i0$Phi_dev)
})

test_that("the engine config / init helpers handle K > 2 with per-state families", {
  cfg <- .hmm_margin_config(
    list(c("normal", "normal"), c("exponential", "exponential"), c("gamma", "gamma")),
    skew_direction = c(1, 1), K = 3
  )
  expect_true(cfg$per_state)
  expect_identical(dim(cfg$family), c(3L, 2L))
  expect_identical(cfg$family[3, ], c(4L, 4L))

  sw <- .resolve_switch_spec(c("rho", "mu", "phi"))
  sw$margins <- 1L # the wrapper forces this for differing per-state families
  init <- .init_hmm_switching_params(2, 3, sw)
  expect_identical(dim(init$mu), c(3L, 2L))
  expect_identical(dim(init$sigma_eps), c(3L, 2L))
  expect_identical(dim(init$Phi_dev), c(3L, 2L, 2L))
  expect_length(init$z_rho, 3L)

  df <- data.frame(time = 1:30, y1 = rnorm(30), y2 = rnorm(30))
  base <- prepare_hmm_data(df, c("y1", "y2"), K = 3)
  aug <- .as_hmm_switching_stan_data(base, sw, cfg$family, cfg$skew, prior_phi_dev_sd = 0.5)
  expect_identical(dim(aug$family), c(3L, 2L))
})

# --- MCMC behaviour (gated) -------------------------------------------------

test_that("a state-specific HMM fit exposes per-state parameters and pins diagnostics", {
  skip_if_no_rstan()

  fit <- get_hmm_switching_fit()
  expect_s3_class(fit, "dcvar_hmm_fit")
  expect_true(isTRUE(fit$switching))

  sp <- hmm_state_params(fit)
  expect_identical(dim(sp$mu), c(2L, 2L))
  expect_length(sp$Phi, 2L)
  expect_length(sp$rho_state, 2L)
  expect_identical(dim(sp$families), c(2L, 2L))
  expect_true(all(seq_len(2) %in% sp$scales$state))

  # Name-pin: every monitored name exists in the draws.
  monitored <- .diagnostic_parameter_variables(fit)
  available <- posterior::variables(draws(fit))
  expect_identical(setdiff(monitored, available), character(0))

  # Per-state coef + var_params resolve (state-indexed Phi baseline + deviations).
  co <- coef(fit)
  expect_false(is.null(co$mu))
  expect_false(is.null(co$Phi))
  vp <- var_params(fit)
  expect_false(is.null(vp$Phi))

  # fitted() returns the gamma-weighted one-step prediction.
  ft <- fitted(fit)
  expect_equal(nrow(ft), fit$stan_data$n_time - 1L)
  expect_true(all(is.finite(ft[[2]])))

  # Mixture predictive / PIT / PPC are guarded for switching fits.
  expect_error(predict(fit), "state-specific")
  expect_error(pit_values(fit), "state-specific")
  expect_error(plot_phi(fit), "state-specific")
  expect_error(plot_ppc(fit), "state-specific")
  expect_error(plot(fit, type = "ppc"), "state-specific")
  expect_s3_class(plot(fit, type = "diagnostics"), "ggplot")
})

test_that("default dcvar_hmm is unchanged (legacy path, not the engine)", {
  skip_if_no_rstan()

  fit <- get_hmm_fit()
  expect_false(isTRUE(fit$switching))
  co <- coef(fit)
  expect_false(is.null(co$sigma_eps))
  # Legacy downstream methods keep working.
  expect_s3_class(fitted(fit), "data.frame")
  expect_true(all(predict(fit)$lower <= predict(fit)$upper))
})

test_that("the switching engine fits under cmdstanr with per-state margins", {
  skip_if_no_cmdstanr_backend()
  skip_if_not_slow()

  sim <- simulate_dcvar(n_time = 45, rho_trajectory = rho_step(45, 0.8, 0.1), seed = 42)
  fit <- dcvar_hmm(
    sim$Y_df, vars = c("y1", "y2"), K = 2,
    switch = c("rho", "mu", "phi"),
    margins = list(c("normal", "normal"), c("exponential", "exponential")),
    skew_direction = c(1, 1),
    chains = 1, iter_warmup = 60, iter_sampling = 60, refresh = 0,
    seed = 13, backend = "cmdstanr"
  )
  expect_s3_class(fit, "dcvar_hmm_fit")
  expect_identical(fit$backend, "cmdstanr")
  expect_true(isTRUE(fit$switching))
  # Name-pin holds on the cmdstanr serialization too (state-indexed names).
  expect_identical(
    setdiff(.diagnostic_parameter_variables(fit), posterior::variables(draws(fit))),
    character(0)
  )
})

test_that("the switching engine reproduces the legacy HMM on a rho-only config", {
  skip_if_no_rstan()
  skip_if_not_slow()

  sim <- simulate_dcvar(
    n_time = 60,
    rho_trajectory = rho_step(60, rho_before = 0.8, rho_after = 0.1),
    seed = 42
  )
  common <- list(
    data = sim$Y_df, vars = c("y1", "y2"), K = 2,
    chains = 2, iter_warmup = 500, iter_sampling = 500,
    adapt_delta = 0.99, max_treedepth = 13, refresh = 0, seed = 321
  )

  fit_legacy <- do.call(dcvar_hmm, common)
  # Force the engine on the same rho-only config via the bundled switching file.
  fit_engine <- do.call(dcvar_hmm, c(common, list(stan_file = dcvar_stan_path("hmm_switching"))))
  expect_true(isTRUE(fit_engine$switching))

  cl <- coef(fit_legacy)
  ce <- coef(fit_engine)
  expect_equal(unname(cl$mu), unname(ce$mu), tolerance = 0.08)
  expect_equal(sort(unname(cl$rho_state)), sort(unname(ce$rho_state)), tolerance = 0.08)
  expect_equal(unname(cl$sigma_eps), unname(ce$sigma_eps), tolerance = 0.08)

  rl <- rho_trajectory(fit_legacy)$mean
  re <- rho_trajectory(fit_engine)$mean
  expect_equal(rl, re, tolerance = 0.08)
})

test_that("cmdstanr backend fits the core models", {
  skip_if_no_cmdstanr_backend()

  dc_fit <- get_dcvar_cmdstanr_fit()
  hmm_fit <- get_hmm_cmdstanr_fit()
  con_fit <- get_constant_cmdstanr_fit()

  expect_s3_class(dc_fit, "dcvar_fit")
  expect_s3_class(hmm_fit, "dcvar_hmm_fit")
  expect_s3_class(con_fit, "dcvar_constant_fit")

  expect_equal(dc_fit$backend, "cmdstanr")
  expect_equal(hmm_fit$backend, "cmdstanr")
  expect_equal(con_fit$backend, "cmdstanr")
})

test_that("cmdstanr fits support the standard extractors", {
  skip_if_no_cmdstanr_backend()

  dc_fit <- get_dcvar_cmdstanr_fit()
  hmm_fit <- get_hmm_cmdstanr_fit()
  con_fit <- get_constant_cmdstanr_fit()

  expect_s3_class(summary(dc_fit), "dcvar_summary")
  expect_named(coef(dc_fit), c("mu", "Phi", "sigma_eps", "sigma_omega"))
  expect_s3_class(rho_trajectory(dc_fit), "data.frame")
  expect_type(dcvar_diagnostics(dc_fit), "list")
  expect_s3_class(fitted(dc_fit), "data.frame")
  expect_s3_class(predict(dc_fit), "data.frame")
  expect_s3_class(as.data.frame(dc_fit), "data.frame")
  expect_s3_class(loo(dc_fit), "loo")

  expect_named(coef(hmm_fit), c("mu", "Phi", "sigma_eps", "z_rho", "rho_state"))
  expect_type(dcvar_diagnostics(hmm_fit), "list")
  expect_type(hmm_states(hmm_fit), "list")
  expect_s3_class(fitted(hmm_fit), "data.frame")
  expect_s3_class(predict(hmm_fit), "data.frame")
  expect_s3_class(loo(hmm_fit), "loo")

  expect_named(coef(con_fit), c("mu", "Phi", "sigma_eps", "rho"))
  expect_type(dcvar_diagnostics(con_fit), "list")
  expect_s3_class(fitted(con_fit), "data.frame")
  expect_s3_class(predict(con_fit), "data.frame")
  expect_s3_class(plot(con_fit, type = "rho"), "ggplot")
  expect_s3_class(loo(con_fit), "loo")
})

test_that("cmdstanr backend fits partial time-varying models (zero-sized init params omitted)", {
  skip_if_no_cmdstanr_backend()

  sim <- simulate_dcvar(
    n_time = 30,
    rho_trajectory = rho_constant(30, 0.4),
    phi_trajectory = list(
      rho_decreasing(30, 0.3, 0.05), rho_constant(30, 0.1),
      rho_constant(30, 0.1), rho_constant(30, 0.3)
    ),
    seed = 91
  )

  # tv_phi only: sigma_raw / tau_sigma are inactive (zero-sized) -> must be
  # omitted from the default init so cmdstanr does not abort on a shapeless
  # empty matrix.
  fit_phi <- dcvar(
    sim$Y_df, vars = c("y1", "y2"),
    tv_phi = "ar", tv_sigma = FALSE,
    backend = "cmdstanr",
    chains = 1, iter_warmup = 50, iter_sampling = 50, refresh = 0, seed = 7
  )
  expect_s3_class(fit_phi, "dcvar_tv_fit")
  expect_equal(fit_phi$backend, "cmdstanr")
  expect_s3_class(phi_trajectory(fit_phi), "data.frame")

  # tv_sigma only: phi_raw / tau_phi are inactive (zero-sized).
  sim2 <- simulate_dcvar(
    n_time = 30,
    rho_trajectory = rho_constant(30, 0.4),
    sigma_trajectory = cbind(seq(0.8, 1.4, length.out = 29), rep(1, 29)),
    seed = 92
  )
  fit_sig <- dcvar(
    sim2$Y_df, vars = c("y1", "y2"),
    tv_phi = FALSE, tv_sigma = TRUE,
    backend = "cmdstanr",
    chains = 1, iter_warmup = 50, iter_sampling = 50, refresh = 0, seed = 8
  )
  expect_s3_class(fit_sig, "dcvar_tv_fit")
  expect_equal(fit_sig$backend, "cmdstanr")
  expect_s3_class(sigma_trajectory(fit_sig), "data.frame")
})

# ============================================================================
# Convergence Diagnostics and Parameter Recovery Tests
# ============================================================================
# These tests validate that fitted models produce reasonable MCMC diagnostics
# and that parameter estimates are in the right ballpark. They use the cached
# minimal fits from helper-fits.R (small iter counts), so thresholds are
# deliberately lenient.

# --- Convergence diagnostics ------------------------------------------------

test_that("cached baseline fits emit only known diagnostic warnings", {
  skip_if_no_rstan()

  expect_known_fit_warnings(get_dcvar_fit_warnings(), "dcvar")
  expect_known_fit_warnings(get_hmm_fit_warnings(), "hmm")
  expect_known_fit_warnings(get_constant_fit_warnings(), "constant")
})

test_that("gamma fits emit only known diagnostic warnings", {
  skip_if_no_rstan()

  expect_known_fit_warnings(get_dcvar_gamma_fit_warnings(), "dcvar gamma")
  expect_known_fit_warnings(get_hmm_gamma_fit_warnings(), "hmm gamma")
  expect_known_fit_warnings(get_constant_gamma_fit_warnings(), "constant gamma")
})

.expect_gamma_support_consistent <- function(fit, co, fit_name) {
  required_mean <- pmax(.gamma_support_bound(fit), 0)
  implied_mean <- sqrt(unname(co$shape_gam[[1]])) * unname(co$sigma_gam)

  expect_true(
    all(implied_mean + 1e-8 >= required_mean),
    info = paste(
      fit_name,
      "gamma support violated:",
      paste(
        sprintf(
          "dim %d mean=%.4f lb=%.4f",
          seq_along(implied_mean),
          implied_mean,
          required_mean
        ),
        collapse = ", "
      )
    )
  )
}

test_that("dcvar fit has no divergences and finite Rhat", {
  skip_if_no_rstan()

  fit <- get_dcvar_fit()
  diag <- dcvar_diagnostics(fit)

  expect_equal(diag$n_divergent, 0)
  expect_equal(diag$n_max_treedepth, 0)
  expect_true(is.finite(diag$max_rhat))
  expect_true(diag$max_rhat < 1.25)
  expect_true(diag$min_ess_bulk > 10)
  expect_true(diag$min_ess_tail > 10)
  expect_true(diag$mean_accept_prob > 0 && diag$mean_accept_prob < 1)
})

test_that("hmm fit has finite Rhat", {
  skip_if_no_rstan()

  fit <- get_hmm_fit()
  diag <- dcvar_diagnostics(fit)

  expect_lte(diag$n_divergent, 2)
  expect_equal(diag$n_max_treedepth, 0)
  expect_true(is.finite(diag$max_rhat))
  expect_true(diag$max_rhat < 1.30)
  expect_true(diag$min_ess_bulk > 8)
  expect_true(diag$min_ess_tail > 8)
})

test_that("constant fit has finite Rhat", {
  skip_if_no_rstan()

  fit <- get_constant_fit()
  diag <- dcvar_diagnostics(fit)

  expect_equal(diag$n_divergent, 0)
  expect_equal(diag$n_max_treedepth, 0)
  expect_true(is.finite(diag$max_rhat))
  expect_true(diag$max_rhat < 1.25)
  expect_true(diag$min_ess_bulk > 10)
  expect_true(diag$min_ess_tail > 10)
})

test_that("dcvar exponential fit has usable diagnostics", {
  skip_if_no_rstan()

  fit <- get_dcvar_exponential_fit()
  diag <- dcvar_diagnostics(fit)
  co <- coef(fit)

  expect_equal(diag$n_divergent, 0)
  expect_equal(diag$n_max_treedepth, 0)
  expect_true(is.finite(diag$max_rhat))
  expect_true(diag$max_rhat < 1.30)
  expect_true(all(co$sigma_exp > 0))
})

test_that("dcvar gamma fit has usable diagnostics", {
  skip_if_no_rstan()

  fit <- get_dcvar_gamma_fit()
  diag <- dcvar_diagnostics(fit)
  co <- coef(fit)

  expect_equal(diag$n_divergent, 0)
  expect_equal(diag$n_max_treedepth, 0)
  expect_true(is.finite(diag$max_rhat))
  expect_true(diag$max_rhat < 1.35, info = paste("max_rhat =", signif(diag$max_rhat, 4)))
  expect_true(all(co$sigma_gam > 0))
  expect_true(co$shape_gam > 0)
  .expect_gamma_support_consistent(fit, co, "dcvar gamma")
})

test_that("hmm exponential fit has usable diagnostics", {
  skip_if_no_rstan()

  fit <- get_hmm_exponential_fit()
  diag <- dcvar_diagnostics(fit)
  co <- coef(fit)

  expect_equal(diag$n_divergent, 0)
  expect_equal(diag$n_max_treedepth, 0)
  expect_true(is.finite(diag$max_rhat))
  expect_true(diag$max_rhat < 1.30)
  expect_length(co$sigma_exp, 2)
  expect_true(all(co$sigma_exp > 0))
})

test_that("hmm gamma fit has usable diagnostics", {
  skip_if_no_rstan()

  fit <- get_hmm_gamma_fit()
  diag <- dcvar_diagnostics(fit)
  co <- coef(fit)

  expect_lte(diag$n_divergent, 1)
  expect_equal(diag$n_max_treedepth, 0)
  expect_true(is.finite(diag$max_rhat))
  expect_true(diag$max_rhat < 1.35, info = paste("max_rhat =", signif(diag$max_rhat, 4)))
  expect_true(all(co$sigma_gam > 0))
  expect_true(co$shape_gam > 0)
  .expect_gamma_support_consistent(fit, co, "hmm gamma")
})

test_that("constant gamma fit has usable diagnostics", {
  skip_if_no_rstan()

  fit <- get_constant_gamma_fit()
  diag <- dcvar_diagnostics(fit)
  co <- coef(fit)

  expect_equal(diag$n_divergent, 0)
  expect_equal(diag$n_max_treedepth, 0)
  expect_true(is.finite(diag$max_rhat))
  expect_true(diag$max_rhat < 1.35, info = paste("max_rhat =", signif(diag$max_rhat, 4)))
  expect_true(all(co$sigma_gam > 0))
  expect_true(co$shape_gam > 0)
  .expect_gamma_support_consistent(fit, co, "constant gamma")
})

# --- Parameter recovery: rho estimates are bounded ----------------------------

test_that("dcvar rho trajectory is within valid bounds", {
  skip_if_no_rstan()

  fit <- get_dcvar_fit()
  rho_df <- rho_trajectory(fit)

  # All posterior means must be in (-1, 1)
  expect_true(all(rho_df$mean > -1 & rho_df$mean < 1))
  # Credible intervals must not extend beyond bounds
  expect_true(all(rho_df$q2.5 >= -1))
  expect_true(all(rho_df$q97.5 <= 1))
  # SDs must be positive

  expect_true(all(rho_df$sd > 0))
})

test_that("hmm rho_state values are ordered and bounded", {
  skip_if_no_rstan()

  fit <- get_hmm_fit()
  states <- hmm_states(fit)

  rho_means <- states$rho_state$mean
  # Ordered constraint: state 1 rho < state 2 rho (for K=2)
  expect_true(all(diff(rho_means) >= 0))
  # Bounded in (-1, 1)
  expect_true(all(rho_means > -1 & rho_means < 1))
})

test_that("constant rho is bounded", {
  skip_if_no_rstan()

  fit <- get_constant_fit()
  rho_df <- rho_trajectory(fit)

  expect_true(all(rho_df$mean > -1 & rho_df$mean < 1))
  # All values should be identical (constant model)
  expect_equal(length(unique(rho_df$mean)), 1)
})

# --- Parameter recovery: VAR coefficients -----------------------------------

test_that("dcvar sigma_eps estimates are positive", {
  skip_if_no_rstan()

  fit <- get_dcvar_fit()
  co <- coef(fit)

  expect_true(all(co$sigma_eps > 0))
  expect_true(co$sigma_omega > 0)
})

test_that("hmm sigma_eps estimates are positive", {
  skip_if_no_rstan()

  fit <- get_hmm_fit()
  co <- coef(fit)

  expect_true(all(co$sigma_eps > 0))
})

test_that("constant sigma_eps estimates are positive", {
  skip_if_no_rstan()

  fit <- get_constant_fit()
  co <- coef(fit)

  expect_true(all(co$sigma_eps > 0))
})

# --- Parameter recovery: HMM transition matrix is valid ----------------------

test_that("hmm transition matrix rows sum to 1", {
  skip_if_no_rstan()

  fit <- get_hmm_fit()
  states <- hmm_states(fit)

  row_sums <- rowSums(states$A)
  expect_equal(row_sums, rep(1, fit$K), tolerance = 1e-6)
  # All entries non-negative
  expect_true(all(states$A >= 0))
})

test_that("hmm gamma posteriors sum to 1 at each time point", {
  skip_if_no_rstan()

  fit <- get_hmm_fit()
  states <- hmm_states(fit)

  row_sums <- rowSums(states$gamma)
  expect_equal(row_sums, rep(1, nrow(states$gamma)), tolerance = 1e-4)
})

test_that("hmm viterbi states are valid integers", {
  skip_if_no_rstan()

  fit <- get_hmm_fit()
  states <- hmm_states(fit)

  expect_true(all(states$viterbi %in% 1:fit$K))
  expect_equal(length(states$viterbi), fit$stan_data$n_time - 1)
})

# --- simulate_dcvar input validation ----------------------------------------

test_that("simulate_dcvar rejects D != 2", {
  expect_error(
    simulate_dcvar(n_time = 50, rho_trajectory = rho_constant(50, 0.5),
                   mu = c(0, 0, 0)),
    "bivariate"
  )
})

test_that("simulate_dcvar rejects mismatched Phi dimensions", {
  expect_error(
    simulate_dcvar(n_time = 50, rho_trajectory = rho_constant(50, 0.5),
                   Phi = matrix(0, 3, 3)),
    "matrix"
  )
})

test_that("simulate_dcvar rejects mismatched sigma_eps length", {
  expect_error(
    simulate_dcvar(n_time = 50, rho_trajectory = rho_constant(50, 0.5),
                   sigma_eps = c(1, 1, 1)),
    "length"
  )
})

test_that("prepare_dcvar_covariate_data aligns covariates after sorting and outcome filtering", {
  df <- data.frame(
    time = c(4, 1, 3, 2),
    y1 = c(40, 10, NA, 20),
    y2 = c(44, 11, 33, 22),
    phase = c(4, 1, 3, 2)
  )

  expect_warning(
    out <- prepare_dcvar_covariate_data(
      df,
      vars = c("y1", "y2"),
      covariates = "phase",
      standardize = FALSE,
      allow_gaps = TRUE
    ),
    "Removing 1 row"
  )

  expect_equal(out$n_time, 3)
  expect_equal(as.numeric(out$Y[, 1]), c(10, 20, 40))
  expect_equal(as.numeric(out$X[, 1]), c(1, 2, 4))
  expect_equal(attr(out, "time_values"), c(1, 2, 4))
  expect_equal(attr(out, "covariates"), "phase")
})

test_that("prepare_dcvar_covariate_data validates covariates", {
  df <- data.frame(
    time = 1:5,
    y1 = rnorm(5),
    y2 = rnorm(5),
    phase = c(0, 0, 1, 1, 1),
    label = letters[1:5],
    missing_x = c(0, 1, NA, 1, 0),
    constant_x = 1
  )

  expect_error(
    prepare_dcvar_covariate_data(df, vars = c("y1", "y2"), covariates = "missing_col"),
    "not found"
  )
  expect_error(
    prepare_dcvar_covariate_data(df, vars = c("y1", "y2"), covariates = "label"),
    "must be numeric"
  )
  expect_error(
    prepare_dcvar_covariate_data(df, vars = c("y1", "y2"), covariates = "missing_x"),
    "complete finite"
  )
  expect_error(
    prepare_dcvar_covariate_data(
      df,
      vars = c("y1", "y2"),
      covariates = "constant_x",
      standardize_covariates = TRUE
    ),
    "zero"
  )
})

test_that("prepare_dcvar_covariate_data allows binary phase indicators by default", {
  df <- data.frame(
    time = 1:6,
    y1 = rnorm(6),
    y2 = rnorm(6),
    phase = c(0, 0, 0, 1, 1, 1)
  )

  out <- prepare_dcvar_covariate_data(df, vars = c("y1", "y2"), covariates = "phase")

  expect_equal(as.numeric(out$X[, 1]), df$phase)
  expect_false(attr(out, "standardized_covariates"))
  expect_equal(out$P, 1)
})

test_that("covariate init preserves Stan vector dimensions when length is one", {
  init <- .init_dcvar_covariate_params(D = 2, T_obs = 3, P = 1, drift = TRUE)

  expect_equal(dim(init$beta), 1L)
  expect_equal(dim(init$omega_raw), 1L)
})

test_that("dcvar_stan_path exposes covariate Stan models", {
  drift_path <- dcvar_stan_path("dcvar_covariate")
  nodrift_path <- dcvar_stan_path("dcvar_covariate_nodrift")

  expect_true(file.exists(drift_path))
  expect_true(file.exists(nodrift_path))
  # The public path helper returns the legacy-compatible Stan files that match
  # prepare_dcvar_covariate_data(); dcvar_covariate() routes bundled drift fits
  # to dcvar_dynamic.stan internally.
  expect_match(basename(drift_path), "^dcvar_covariate_ncp\\.stan$")
  expect_match(basename(nodrift_path), "^dcvar_covariate_nodrift\\.stan$")
  expect_error(dcvar_stan_path("dcvar_covariate", margins = "gamma"), "normal")
})

make_covariate_stub_fit <- function(drift = TRUE) {
  variables <- c(
    "mu[1]", "mu[2]",
    "Phi[1,1]", "Phi[2,1]", "Phi[1,2]", "Phi[2,2]",
    "sigma_eps[1]", "sigma_eps[2]",
    "beta_0", "beta[1]", "beta[2]",
    paste0("rho[", 1:4, "]"),
    paste0("log_lik[", 1:4, "]")
  )
  if (drift) {
    variables <- c(variables, "sigma_omega")
  }

  draws <- array(
    rnorm(40 * length(variables), 0, 0.1),
    dim = c(40, 1, length(variables)),
    dimnames = list(NULL, NULL, variables)
  )
  rho_cols <- match(paste0("rho[", 1:4, "]"), variables)
  draws[, , rho_cols] <- tanh(draws[, , rho_cols])
  log_lik_cols <- match(paste0("log_lik[", 1:4, "]"), variables)
  draws[, , log_lik_cols] <- -abs(draws[, , log_lik_cols])

  stan_data <- structure(
    list(n_time = 5, D = 2, P = 2),
    time_values = 101:105
  )

  new_dcvar_covariate_fit(
    fit = posterior::as_draws_array(draws),
    stan_data = stan_data,
    vars = c("y1", "y2"),
    covariates = c("phase", "load"),
    standardized = TRUE,
    standardized_covariates = FALSE,
    drift = drift,
    zero_init_eta = TRUE,
    backend = "rstan",
    priors = list(),
    meta = list(chains = 1, iter_warmup = 10, iter_sampling = 40)
  )
}

test_that("covariate fit extractors work for drift and no-drift objects", {
  fit <- make_covariate_stub_fit(drift = TRUE)
  no_drift_fit <- make_covariate_stub_fit(drift = FALSE)

  expect_s3_class(fit, "dcvar_covariate_fit")
  expect_equal(fit$model, "dcvar_covariate")
  expect_equal(no_drift_fit$model, "dcvar_covariate_nodrift")

  co <- coef(fit)
  expect_named(co, c("mu", "Phi", "sigma_eps", "beta_0", "beta", "sigma_omega"))
  expect_equal(names(co$beta), c("phase", "load"))

  co_no_drift <- coef(no_drift_fit)
  expect_named(co_no_drift, c("mu", "Phi", "sigma_eps", "beta_0", "beta"))

  rho_df <- rho_trajectory(fit)
  expect_equal(rho_df$time, 102:105)
  expect_true(all(rho_df$mean >= -1 & rho_df$mean <= 1))

  effects <- covariate_effects(fit)
  expect_true(all(c("(Intercept)", "phase", "load") %in% effects$term))
  expect_false("sigma_omega" %in% effects$term)

  effects_no_drift <- covariate_effects(no_drift_fit)
  expect_false("sigma_omega" %in% effects_no_drift$term)
})

test_that("loo works for covariate fits with log_lik", {
  fit <- make_covariate_stub_fit(drift = TRUE)

  out <- suppressWarnings(loo::loo(fit))

  expect_s3_class(out, "loo")
})

test_that("dcvar_covariate fits end to end (drift and no-drift)", {
  skip_if_no_rstan()

  fit <- get_covariate_fit(drift = TRUE)
  expect_s3_class(fit, "dcvar_covariate_fit")
  eff <- covariate_effects(fit)
  expect_true(is.data.frame(eff))
  traj <- rho_trajectory(fit)
  expect_true(all(is.finite(traj$mean)))
  expect_true(all(traj$mean > -1 & traj$mean < 1))

  fit_nodrift <- get_covariate_fit(drift = FALSE)
  expect_s3_class(fit_nodrift, "dcvar_covariate_fit")
  traj_nodrift <- rho_trajectory(fit_nodrift)
  expect_true(all(is.finite(traj_nodrift$mean)))
})

test_that("rho_trajectory() returns correct structure for dcvar", {
  skip_if_no_rstan()

  fit <- get_dcvar_fit()
  rho_df <- rho_trajectory(fit)

  expect_s3_class(rho_df, "data.frame")
  expect_true(all(c("time", "mean", "sd") %in% names(rho_df)))
  expect_true("q2.5" %in% names(rho_df))
  expect_true("q97.5" %in% names(rho_df))
  expect_true(all(rho_df$mean >= -1 & rho_df$mean <= 1))
  expect_equal(rho_df$time, attr(fit$stan_data, "time_values")[-1])
})

test_that(".validate_probs rejects invalid probability vectors", {
  expect_invisible(.validate_probs(c(0, 0.5, 1)))

  expect_error(.validate_probs(numeric()), "finite numeric values")
  expect_error(.validate_probs(NA_real_), "finite numeric values")
  expect_error(.validate_probs(NaN), "finite numeric values")
  expect_error(.validate_probs(Inf), "finite numeric values")
  expect_error(.validate_probs(c(-0.1, 0.5)), "finite numeric values")
  expect_error(.validate_probs(c(0.5, 1.1)), "finite numeric values")
})

test_that("rho_trajectory() returns correct structure for hmm", {
  skip_if_no_rstan()

  fit <- get_hmm_fit()
  rho_df <- rho_trajectory(fit)

  expect_s3_class(rho_df, "data.frame")
  expect_true(all(c("time", "mean", "sd") %in% names(rho_df)))
  expect_equal(rho_df$time, attr(fit$stan_data, "time_values")[-1])
})

test_that("rho_trajectory() returns full trajectory for constant", {
  skip_if_no_rstan()

  fit <- get_constant_fit()
  rho_df <- rho_trajectory(fit)

  expect_s3_class(rho_df, "data.frame")
  # Constant model now expands rho to all T-1 time points
  expect_equal(nrow(rho_df), fit$stan_data$n_time - 1)
  expect_true(all(c("time", "mean", "sd") %in% names(rho_df)))
  # All means should be identical (constant)
  expect_equal(length(unique(rho_df$mean)), 1)
  expect_true(all(rho_df$mean >= -1 & rho_df$mean <= 1))
  expect_equal(rho_df$time, attr(fit$stan_data, "time_values")[-1])
})

test_that("var_params() returns correct structure", {
  skip_if_no_rstan()

  fit <- get_dcvar_fit()
  vp <- var_params(fit)

  expect_type(vp, "list")
  expect_true(all(c("mu", "Phi", "sigma_eps") %in% names(vp)))
  expect_true("sigma_omega" %in% names(vp))
  expect_true(all(c("variable", "mean", "sd", "q2.5", "q97.5") %in% names(vp$mu)))
})

test_that("extractors work for non-normal fitted objects", {
  skip_if_no_rstan()

  rho_df <- rho_trajectory(get_dcvar_exponential_fit())
  expect_s3_class(rho_df, "data.frame")
  expect_true(all(rho_df$mean >= -1 & rho_df$mean <= 1))

  gamma_params <- var_params(get_constant_gamma_fit())
  expect_true("sigma_gam" %in% names(gamma_params))
  expect_true("shape_gam" %in% names(gamma_params))

  skip_if_not_installed("sn")
  skew_params <- var_params(get_constant_skew_normal_fit())
  expect_true("omega" %in% names(skew_params))
  expect_true("delta" %in% names(skew_params))
})

test_that("hmm_states() returns correct structure", {
  skip_if_no_rstan()

  fit <- get_hmm_fit()
  states <- hmm_states(fit)

  expect_type(states, "list")
  expect_named(states, c("gamma", "viterbi", "rho_state", "A", "rho_hmm"))
  expect_equal(ncol(states$gamma), fit$K)
  expect_equal(dim(states$A), c(fit$K, fit$K))
})

test_that("hmm_states() returns a valid Viterbi path on posterior-mean quantities", {
  skip_if_no_rstan()

  fit <- get_hmm_fit()
  states <- hmm_states(fit)
  n_time_eff <- fit$stan_data$n_time - 1L

  expect_length(states$viterbi, n_time_eff)
  expect_true(all(states$viterbi %in% seq_len(fit$K)))
  # Decoding is deterministic given the fit
  expect_identical(states$viterbi, hmm_states(fit)$viterbi)
})

test_that(".viterbi_path decodes the MAP sequence for a known HMM", {
  # Two states; state 2 strongly favored by emissions except at t = 2, where
  # a sticky transition matrix keeps the path in state 2 anyway.
  obs_ll <- rbind(c(-5, -1), c(-1.5, -2), c(-5, -1))
  log_A <- log(matrix(c(0.9, 0.1, 0.1, 0.9), 2, 2, byrow = TRUE))
  log_pi0 <- log(c(0.5, 0.5))
  expect_identical(.viterbi_path(obs_ll, log_A, log_pi0), c(2L, 2L, 2L))

  # With a flat transition matrix, the emission at t = 2 wins instead.
  log_A_flat <- log(matrix(0.5, 2, 2))
  expect_identical(.viterbi_path(obs_ll, log_A_flat, log_pi0), c(2L, 1L, 2L))
})

test_that("draws() returns posterior draws", {
  skip_if_no_rstan()

  fit <- get_dcvar_fit()
  d <- draws(fit, variable = "mu")
  expect_true(inherits(d, "draws_array"))
})

test_that("dcvar_diagnostics() returns correct fields", {
  skip_if_no_rstan()

  fit <- get_dcvar_fit()
  diag <- dcvar_diagnostics(fit)

  expect_type(diag, "list")
  expect_named(diag, c("n_divergent", "n_max_treedepth", "max_rhat",
                        "min_ess_bulk", "min_ess_tail", "mean_accept_prob"))
  expect_true(diag$max_rhat >= 1)
})

test_that("rho_trajectory() supports a single quantile request", {
  skip_if_no_rstan()

  fit <- get_dcvar_fit()
  rho_df <- rho_trajectory(fit, probs = 0.5)

  expect_true("q50" %in% names(rho_df))
})

test_that("latent_states() supports a single quantile request", {
  skip_if_no_rstan()

  fit <- get_sem_fit()
  states <- latent_states(fit, probs = 0.5)

  expect_true("q50" %in% names(states))
})

test_that("rho_trajectory() and latent_states() honor preserved time values", {
  skip_if_no_rstan()

  fit <- get_dcvar_fit()
  attr(fit$stan_data, "time_values") <- 101:(100 + fit$stan_data$n_time)
  expect_equal(rho_trajectory(fit)$time, 102:(100 + fit$stan_data$n_time))

  sem_fit <- get_sem_fit()
  attr(sem_fit$stan_data, "time_values") <- seq.Date(
    as.Date("2020-01-01"),
    by = "day",
    length.out = sem_fit$stan_data$n_time
  )
  sem_rho <- rho_trajectory(sem_fit)
  sem_states <- latent_states(sem_fit)
  expect_equal(sem_rho$time, attr(sem_fit$stan_data, "time_values")[-1])
  expect_equal(
    unique(sem_states$time),
    attr(sem_fit$stan_data, "time_values")
  )
})

test_that("random_effects() preserves original unit ids", {
  skip_if_no_rstan()

  fit <- get_multilevel_fit()
  attr(fit$stan_data, "ids") <- c("A", "B", "C")
  re <- random_effects(fit)

  expect_equal(unique(re$unit), c("A", "B", "C"))
})

test_that("custom Stan output mismatches fail clearly in extractors", {
  make_stub_draws <- function(variables) {
    posterior::as_draws_array(
      array(
        seq_len(4L * length(variables)),
        dim = c(4L, 1L, length(variables)),
        dimnames = list(NULL, NULL, variables)
      )
    )
  }

  dcvar_fit <- structure(
    list(
      fit = make_stub_draws(c("Phi[1,1]", "Phi[1,2]", "Phi[2,1]", "Phi[2,2]", "sigma_eps[1]", "sigma_eps[2]", "sigma_omega")),
      stan_data = list(n_time = 5, D = 2),
      model = "dcvar",
      vars = c("y1", "y2"),
      standardized = TRUE,
      margins = "normal",
      backend = "rstan",
      priors = list(),
      meta = list()
    ),
    class = c("dcvar_fit", "dcvar_model_fit")
  )
  expect_error(coef(dcvar_fit), "Custom Stan files must preserve the expected parameter names")
  expect_error(var_params(dcvar_fit), "Custom Stan files must preserve")

  multilevel_fit <- structure(
    list(
      fit = make_stub_draws(c("phi_unit[1,2]", "phi_unit[1,3]", "phi_unit[1,4]")),
      stan_data = structure(list(n_time = 4), ids = c("A")),
      model = "multilevel",
      N = 1,
      vars = c("y1", "y2"),
      centered = TRUE,
      person_means = NULL,
      backend = "rstan",
      priors = list(),
      meta = list()
    ),
    class = c("dcvar_multilevel_fit", "dcvar_model_fit")
  )
  expect_error(random_effects(multilevel_fit), "Custom Stan files must preserve")

  sem_fit <- structure(
    list(
      fit = make_stub_draws("rho"),
      stan_data = list(n_time = 4),
      model = "sem",
      vars = c("latent1", "latent2"),
      backend = "rstan",
      priors = list(),
      meta = list()
    ),
    class = c("dcvar_sem_fit", "dcvar_model_fit")
  )
  expect_error(latent_states(sem_fit), "Custom Stan files must preserve the expected parameter and generated-quantity names")
})

# ============================================================================
# Tests for time-varying SEM support
# ============================================================================

make_sem_tv_stub_fit <- function(tv_phi = TRUE, tv_sigma = TRUE) {
  n_draw <- 30
  n_time <- 4L
  n_eff <- n_time - 1L
  phi_mask <- .resolve_phi_tv_mask(if (isTRUE(tv_phi)) "ar" else FALSE)
  n_phi_tv <- sum(phi_mask)
  indicators <- list(latent1 = c("y1_1", "y1_2"), latent2 = c("y2_1", "y2_2"))

  variables <- c(
    "mu[1]", "mu[2]",
    "Phi[1,1]", "Phi[1,2]", "Phi[2,1]", "Phi[2,2]",
    "sigma_eps[1]", "sigma_eps[2]",
    "z_rho_init", "sigma_omega",
    paste0("omega_raw[", seq_len(n_eff), "]"),
    paste0("rho[", seq_len(n_eff), "]"),
    paste0(
      "state[",
      rep(seq_len(n_time), each = 2),
      ",",
      rep(seq_len(2), times = n_time),
      "]"
    ),
    paste0(
      "zeta[",
      rep(seq_len(n_time), each = 2),
      ",",
      rep(seq_len(2), times = n_time),
      "]"
    )
  )
  if (isTRUE(tv_phi)) {
    variables <- c(
      variables,
      paste0("tau_phi[", seq_len(n_phi_tv), "]"),
      paste0(
        "phi_raw[",
        rep(seq_len(n_eff), each = n_phi_tv),
        ",",
        rep(seq_len(n_phi_tv), times = n_eff),
        "]"
      ),
      paste0(
        "phi_t[",
        rep(seq_len(n_eff), each = 4),
        ",",
        rep(seq_len(4), times = n_eff),
        "]"
      )
    )
  }
  if (isTRUE(tv_sigma)) {
    variables <- c(
      variables,
      "tau_sigma[1]", "tau_sigma[2]",
      paste0(
        "sigma_raw[",
        rep(seq_len(n_eff), each = 2),
        ",",
        rep(seq_len(2), times = n_eff),
        "]"
      ),
      paste0(
        "sigma_t[",
        rep(seq_len(n_eff), each = 2),
        ",",
        rep(seq_len(2), times = n_eff),
        "]"
      )
    )
  }

  draws <- array(
    0,
    dim = c(n_draw, 1, length(variables)),
    dimnames = list(NULL, NULL, variables)
  )
  fill <- function(name, value) {
    draws[, , name] <<- rep(value, length.out = n_draw)
  }

  fill("mu[1]", 0.1)
  fill("mu[2]", -0.2)
  fill("Phi[1,1]", 0.30)
  fill("Phi[1,2]", 0.10)
  fill("Phi[2,1]", -0.05)
  fill("Phi[2,2]", 0.40)
  fill("sigma_eps[1]", 0.8)
  fill("sigma_eps[2]", 1.2)
  fill("z_rho_init", 0.0)
  fill("sigma_omega", 0.08)
  for (t in seq_len(n_eff)) {
    fill(paste0("omega_raw[", t, "]"), 0)
    fill(paste0("rho[", t, "]"), -0.2 + 0.2 * t)
  }
  for (t in seq_len(n_time)) {
    fill(paste0("state[", t, ",1]"), 0.1 * t)
    fill(paste0("state[", t, ",2]"), -0.1 * t)
    fill(paste0("zeta[", t, ",1]"), 0.05 * t)
    fill(paste0("zeta[", t, ",2]"), -0.05 * t)
  }
  if (isTRUE(tv_phi)) {
    for (k in seq_len(n_phi_tv)) fill(paste0("tau_phi[", k, "]"), 0.03 + 0.01 * k)
    for (t in seq_len(n_eff)) {
      for (k in seq_len(n_phi_tv)) {
        fill(paste0("phi_raw[", t, ",", k, "]"), 0.01 * t)
      }
    }
    base_phi <- c(0.30, 0.10, -0.05, 0.40)
    for (t in seq_len(n_eff)) {
      for (k in seq_len(4)) {
        fill(paste0("phi_t[", t, ",", k, "]"), base_phi[k] + 0.02 * t)
      }
    }
  }
  if (isTRUE(tv_sigma)) {
    fill("tau_sigma[1]", 0.04)
    fill("tau_sigma[2]", 0.05)
    for (t in seq_len(n_eff)) {
      for (d in seq_len(2)) {
        fill(paste0("sigma_raw[", t, ",", d, "]"), 0.01 * d)
      }
    }
    for (t in seq_len(n_eff)) {
      fill(paste0("sigma_t[", t, ",1]"), 0.8 + 0.05 * t)
      fill(paste0("sigma_t[", t, ",2]"), 1.2 - 0.03 * t)
    }
  }

  stan_data <- structure(
    list(
      n_time = n_time,
      J = 2L,
      y = matrix(rnorm(n_time * 4), n_time, 4),
      lambda = c(0.8, 0.8),
      sigma_e = 0.5,
      family = c(1L, 1L),
      tv_phi = as.integer(isTRUE(tv_phi)),
      phi_tv_mask = unname(phi_mask),
      tv_sigma = as.integer(isTRUE(tv_sigma)),
      tau_phi_prior = 0.025,
      tau_sigma_prior = 0.05,
      sigma_omega_prior = 0.1,
      barrier_k = 8
    ),
    time_values = 11:14,
    indicators = indicators,
    vars = names(indicators),
    margins = "normal",
    method = "indicator",
    J = 2L,
    tv_phi = isTRUE(tv_phi),
    phi_tv_mask = phi_mask,
    tv_sigma = isTRUE(tv_sigma)
  )

  new_dcvar_sem_tv_fit(
    fit = posterior::as_draws_array(draws),
    stan_data = stan_data,
    vars = names(indicators),
    J = 2L,
    lambda = c(0.8, 0.8),
    sigma_e = 0.5,
    indicators = indicators,
    margins = "normal",
    method = "indicator",
    tv_phi = isTRUE(tv_phi),
    phi_tv_mask = phi_mask,
    tv_sigma = isTRUE(tv_sigma),
    backend = "rstan",
    priors = list(
      mu_sd = 0.25,
      phi_sd = 0.5,
      sigma_sd = 0.5,
      sigma_omega_rate = 0.1,
      rho_sd = 0.75,
      tau_phi_rate = 0.025,
      tau_sigma_rate = 0.05
    ),
    meta = list(chains = 1, iter_warmup = 10, iter_sampling = n_draw)
  )
}

test_that("SEM TV routing and prepared data expose the mixed engine contract", {
  expect_identical(.margin_stan_file("sem_tv", "normal"), "sem_tv_mixed.stan")
  expect_identical(.margin_stan_file("sem_tv", c("normal", "gamma")), "sem_tv_mixed.stan")
  expect_match(dcvar_stan_path("sem_tv"), "sem_tv_mixed\\.stan$")
  expect_true(file.exists(dcvar_stan_path("sem_tv")))

  sim <- simulate_dcvar_sem(
    n_time = 6,
    J = 2,
    lambda = rep(sqrt(0.8), 2),
    sigma_e = sqrt(0.2),
    rho_trajectory = rho_decreasing(6),
    phi_trajectory = list(rep(0.3, 5), rep(0.1, 5), rep(-0.05, 5), seq(0.2, 0.4, length.out = 5)),
    sigma_trajectory = cbind(seq(0.8, 1.1, length.out = 5), rep(1, 5)),
    seed = 101
  )
  expect_equal(dim(sim$true_params$Phi), c(5L, 4L))
  expect_equal(dim(sim$true_params$sigma), c(5L, 2L))
  expect_length(sim$true_params$rho, 5L)

  sd <- prepare_sem_data(
    sim$data,
    indicators = list(latent1 = c("y1_1", "y1_2"), latent2 = c("y2_1", "y2_2")),
    J = 2,
    lambda = rep(sqrt(0.8), 2),
    sigma_e = sqrt(0.2),
    tv_phi = "ar",
    tv_sigma = TRUE
  )
  expect_identical(sd$tv_phi, 1L)
  expect_identical(sd$phi_tv_mask, c(1L, 0L, 0L, 1L))
  expect_identical(sd$tv_sigma, 1L)
  expect_identical(sd$family, c(1L, 1L))
  expect_identical(sd$skew_direction, c(1, 1))
  expect_true(attr(sd, "tv_phi"))
  expect_true(attr(sd, "tv_sigma"))

  expect_error(
    prepare_sem_data(
      sim$data,
      indicators = list(latent1 = c("y1_1", "y1_2"), latent2 = c("y2_1", "y2_2")),
      J = 2,
      method = "naive",
      tv_phi = TRUE
    ),
    "not implemented"
  )
})

test_that("SEM TV initial values size active random-walk containers only", {
  init <- .init_sem_tv_params(6, "normal", tv_phi = "cross", tv_sigma = TRUE)
  expect_length(init$tau_phi, 2)
  expect_identical(dim(init$phi_raw), c(5L, 2L))
  expect_length(init$tau_sigma, 2)
  expect_identical(dim(init$sigma_raw), c(5L, 2L))
  expect_length(init$omega_raw, 5)

  constant <- .init_sem_tv_params(6, "normal", tv_phi = FALSE, tv_sigma = FALSE)
  expect_null(constant$tau_phi)
  expect_null(constant$phi_raw)
  expect_null(constant$tau_sigma)
  expect_null(constant$sigma_raw)
})

test_that("SEM TV fit object methods report trajectories and walk parameters", {
  fit <- make_sem_tv_stub_fit(tv_phi = TRUE, tv_sigma = TRUE)
  expect_s3_class(fit, "dcvar_sem_tv_fit")
  expect_identical(class(fit), c("dcvar_sem_tv_fit", "dcvar_sem_fit", "dcvar_model_fit"))
  expect_identical(fit$model, "sem_tv")
  expect_true(fit$tv_phi)
  expect_true(fit$tv_sigma)
  expect_identical(unname(fit$phi_tv_mask), c(1L, 0L, 0L, 1L))

  co <- coef(fit)
  expect_true(all(c("mu", "Phi", "sigma_eps", "sigma_omega", "rho", "tau_phi", "tau_sigma") %in% names(co)))
  expect_identical(names(co$tau_phi), c("tau_phi[phi11]", "tau_phi[phi22]"))
  expect_length(co$rho, fit$stan_data$n_time - 1L)

  vp <- var_params(fit)
  expect_true(all(c("mu", "Phi", "sigma_eps", "sigma_omega", "tau_phi", "tau_sigma", "rho") %in% names(vp)))
  expect_identical(vp$tau_phi$variable, c("tau_phi[phi11]", "tau_phi[phi22]"))

  rho_df <- rho_trajectory(fit)
  expect_equal(nrow(rho_df), 3L)
  expect_identical(rho_df$time, 12:14)

  phi_df <- phi_trajectory(fit)
  expect_equal(nrow(phi_df), 12L)
  expect_identical(sort(unique(phi_df$coefficient)), c("phi11", "phi12", "phi21", "phi22"))
  expect_gt(length(unique(phi_df$mean[phi_df$coefficient == "phi11"])), 1L)

  sigma_df <- sigma_trajectory(fit)
  expect_equal(nrow(sigma_df), 6L)
  expect_identical(sort(unique(sigma_df$variable)), sort(fit$vars))
  expect_true(all(sigma_df$mean > 0))

  dep_df <- dependence_summary(fit)
  expect_equal(nrow(dep_df), 3L)
  expect_true(all(abs(dep_df$mean) <= 1))

  expect_output(print(fit), "TV SEM")
  s <- summary(fit)
  expect_s3_class(s, "dcvar_sem_tv_summary")
  expect_false(is.null(s$phi_summary))
  expect_false(is.null(s$sigma_summary))
  expect_silent(invisible(capture.output(print(s))))
})

test_that("SEM TV phi_trajectory tiles the baseline Phi when tv_phi is off", {
  fit <- make_sem_tv_stub_fit(tv_phi = FALSE, tv_sigma = TRUE)
  expect_false(fit$tv_phi)

  phi_df <- phi_trajectory(fit)
  expect_equal(nrow(phi_df), 12L)
  per_coef <- tapply(phi_df$mean, phi_df$coefficient, function(x) length(unique(x)))
  expect_true(all(per_coef == 1L))

  sigma_df <- sigma_trajectory(fit)
  expect_equal(nrow(sigma_df), 6L)
  expect_true(all(sigma_df$mean > 0))
})

# ============================================================================
# Tests for dcvar_multilevel() fit and methods
# ============================================================================

# --- Fit object structure ----------------------------------------------------

make_multilevel_mixed_engine_stub_fit <- function(margins = c("skew_normal", "gamma")) {
  margins <- match.arg(margins)
  N <- 2
  n_time <- 4
  variables <- c(
    paste0("phi_bar[", 1:4, "]"),
    paste0("tau_phi[", 1:4, "]"),
    paste0(
      "z_phi[",
      rep(seq_len(N), each = 4),
      ",",
      rep(seq_len(4), times = N),
      "]"
    ),
    paste0(
      "phi_unit[",
      rep(seq_len(N), each = 4),
      ",",
      rep(seq_len(4), times = N),
      "]"
    ),
    "rho",
    paste0(
      "log_lik[",
      rep(seq_len(N), each = n_time - 1),
      ",",
      rep(seq_len(n_time - 1), times = N),
      "]"
    )
  )
  if (identical(margins, "skew_normal")) {
    variables <- c(variables, "omega[1]", "omega[2]", "delta[1]", "delta[2]")
  } else {
    variables <- c(
      variables,
      "eta[1]", "eta[2]",
      "sigma_gam[1]", "sigma_gam[2]",
      "shape_gam[1]", "shape_gam[2]"
    )
  }

  draws <- array(
    rnorm(40 * length(variables), 0, 0.1),
    dim = c(40, 1, length(variables)),
    dimnames = list(NULL, NULL, variables)
  )
  draws[, , paste0("tau_phi[", 1:4, "]")] <- runif(40 * 4, 0.05, 0.2)
  draws[, , "rho"] <- runif(40, -0.4, 0.4)
  draws[, , grep("^log_lik\\[", variables, value = TRUE)] <-
    -abs(draws[, , grep("^log_lik\\[", variables, value = TRUE)])
  if (identical(margins, "skew_normal")) {
    draws[, , c("omega[1]", "omega[2]")] <- runif(40 * 2, 0.5, 1.5)
    draws[, , c("delta[1]", "delta[2]")] <- runif(40 * 2, -0.5, 0.5)
  } else {
    draws[, , c("sigma_gam[1]", "sigma_gam[2]")] <- runif(40 * 2, 0.5, 1.5)
    draws[, , c("shape_gam[1]", "shape_gam[2]")] <- runif(40 * 2, 0.5, 2.0)
  }

  stan_data <- structure(
    list(
      N = N,
      n_time = n_time,
      y = list(matrix(rnorm(n_time * 2), n_time, 2), matrix(rnorm(n_time * 2), n_time, 2))
    ),
    ids = c("a", "b"),
    time_values = 11:14,
    margins = margins,
    skew_direction = if (identical(margins, "gamma")) c(1, -1) else NULL,
    person_means = matrix(0, N, 2)
  )

  new_dcvar_multilevel_fit(
    fit = posterior::as_draws_array(draws),
    stan_data = stan_data,
    N = N,
    vars = c("y1", "y2"),
    centered = TRUE,
    person_means = attr(stan_data, "person_means"),
    margins = margins,
    skew_direction = attr(stan_data, "skew_direction"),
    backend = "rstan",
    priors = list(),
    meta = list(chains = 1, iter_warmup = 10, iter_sampling = 40)
  )
}

test_that("dcvar_multilevel returns correct class", {
  skip_if_no_rstan()
  fit <- get_multilevel_fit()
  expect_s3_class(fit, "dcvar_multilevel_fit")
  expect_s3_class(fit, "dcvar_model_fit")
})

test_that("dcvar_multilevel rejects center = FALSE with bundled Stan model", {
  df <- data.frame(
    id = rep(1:2, each = 4),
    time = rep(1:4, times = 2),
    y1 = rnorm(8),
    y2 = rnorm(8)
  )

  expect_error(
    dcvar_multilevel(df, vars = c("y1", "y2"), center = FALSE),
    "not supported by the bundled multilevel model"
  )
})

test_that("dcvar_multilevel rejects explicit bundled stan_file with center = FALSE", {
  df <- data.frame(
    id = rep(1:2, each = 4),
    time = rep(1:4, times = 2),
    y1 = rnorm(8),
    y2 = rnorm(8)
  )

  expect_error(
    dcvar_multilevel(
      df,
      vars = c("y1", "y2"),
      center = FALSE,
      stan_file = dcvar_stan_path("multilevel")
    ),
    "not supported by the bundled multilevel model"
  )
})

test_that("multilevel coef() returns expected structure", {
  skip_if_no_rstan()
  fit <- get_multilevel_fit()
  co <- coef(fit)

  expect_type(co, "list")
  expect_named(co, c("phi_bar", "tau_phi", "sigma", "rho"))
  expect_true(length(co$phi_bar) == 4)
  expect_true(length(co$tau_phi) == 4)
  expect_true(length(co$sigma) == 2)
  expect_true(length(co$rho) == 1)
})

test_that("multilevel summary() returns correct class", {
  skip_if_no_rstan()
  fit <- get_multilevel_fit()
  s <- summary(fit)

  expect_s3_class(s, "dcvar_multilevel_summary")
  expect_true(all(c("var_params", "random_effects", "diagnostics") %in% names(s)))
})

test_that("multilevel print() works", {
  skip_if_no_rstan()
  fit <- get_multilevel_fit()
  expect_output(print(fit), "multilevel|Multilevel")
})

# --- Extraction functions ----------------------------------------------------

test_that("multilevel rho_trajectory() returns expected structure", {
  skip_if_no_rstan()
  fit <- get_multilevel_fit()
  rho_df <- rho_trajectory(fit)

  expect_s3_class(rho_df, "data.frame")
  expect_true(all(c("time", "mean", "sd") %in% names(rho_df)))
  # Constant rho: all means should be identical
  expect_equal(length(unique(rho_df$mean)), 1)
})

test_that("multilevel var_params() returns expected structure", {
  skip_if_no_rstan()
  fit <- get_multilevel_fit()
  vp <- var_params(fit)

  expect_type(vp, "list")
  expect_named(vp, c("phi_bar", "tau_phi", "sigma", "rho"))
  expect_s3_class(vp$phi_bar, "data.frame")
  expect_true(all(c("variable", "mean", "sd", "q2.5", "q97.5") %in% names(vp$phi_bar)))
})

test_that("multilevel homogeneous skew_normal and gamma stubs report mixed-engine params", {
  skew_fit <- make_multilevel_mixed_engine_stub_fit("skew_normal")
  skew_co <- coef(skew_fit)
  skew_vp <- var_params(skew_fit)
  expect_named(skew_co, c("phi_bar", "tau_phi", "omega", "delta", "rho"))
  expect_equal(names(skew_co$omega), c("omega[1]", "omega[2]"))
  expect_true(all(c("omega", "delta") %in% names(skew_vp)))

  gamma_fit <- make_multilevel_mixed_engine_stub_fit("gamma")
  gamma_co <- coef(gamma_fit)
  gamma_vp <- var_params(gamma_fit)
  expect_named(gamma_co, c("phi_bar", "tau_phi", "sigma_gam", "shape_gam", "rho"))
  expect_equal(names(gamma_co$shape_gam), c("shape_gam[1]", "shape_gam[2]"))
  expect_true(all(c("sigma_gam", "shape_gam") %in% names(gamma_vp)))
})

test_that("dcvar_multilevel fits homogeneous skew_normal and gamma margins", {
  skip_if_no_rstan()
  skip_if_not_slow()

  for (margin in c("skew_normal", "gamma")) {
    skew_direction <- if (identical(margin, "gamma")) c(1, 1) else NULL
    sim <- simulate_dcvar_multilevel(
      N = 6, n_time = 40, rho = 0.5,
      margins = margin,
      skew_direction = skew_direction,
      skew_params = if (identical(margin, "skew_normal")) list(alpha = c(2, 2)) else list(shape = 2),
      seed = 600 + match(margin, c("skew_normal", "gamma"))
    )
    fit <- dcvar_multilevel(
      sim$data, vars = c("y1", "y2"), id_var = "id",
      margins = margin,
      skew_direction = skew_direction,
      chains = 2, iter_warmup = 300, iter_sampling = 300,
      adapt_delta = 0.95, max_treedepth = 13, refresh = 0, seed = 123
    )

    expect_equal(fit$margins, margin)
    expect_equal(fit$stan_data$family, rep(unname(.family_codes[[margin]]), 2))
    expect_true(coef(fit)$rho > -0.2)
  }
})

test_that("multilevel random_effects() returns expected structure", {
  skip_if_no_rstan()
  fit <- get_multilevel_fit()
  re <- random_effects(fit)

  expect_s3_class(re, "data.frame")
  expect_true(all(c("unit", "parameter", "mean", "sd", "q2.5", "q97.5") %in% names(re)))
  expect_true(all(re$parameter %in% c("phi11", "phi12", "phi21", "phi22")))
  expect_equal(nrow(re), fit$N * 4)
})

# --- Plotting ----------------------------------------------------------------

test_that("multilevel plot random_effects returns ggplot", {
  skip_if_no_rstan()
  fit <- get_multilevel_fit()
  p <- plot(fit, type = "random_effects")
  expect_s3_class(p, "ggplot")
})

test_that("multilevel plot diagnostics returns ggplot", {
  skip_if_no_rstan()
  fit <- get_multilevel_fit()
  p <- plot(fit, type = "diagnostics")
  expect_s3_class(p, "ggplot")
})

# --- Fitted values and prediction --------------------------------------------

test_that("multilevel fitted() returns unit-specific trajectories", {
  skip_if_no_rstan()
  fit <- get_multilevel_fit()

  fit_link <- fitted(fit, type = "link")
  fit_resp <- fitted(fit, type = "response")

  expect_s3_class(fit_link, "data.frame")
  expect_named(fit_link, c("unit", "time", "y1", "y2"))
  expect_equal(nrow(fit_link), fit$N * (fit$stan_data$n_time - 1))
  expect_equal(length(unique(fit_link$unit)), fit$N)
  expect_false(identical(fit_link[, fit$vars], fit_resp[, fit$vars]))
})

test_that("multilevel predict() returns unit-specific intervals", {
  skip_if_no_rstan()
  fit <- get_multilevel_fit()

  pred <- predict(fit)

  expect_s3_class(pred, "data.frame")
  expect_named(pred, c("unit", "time", "variable", "mean", "lower", "upper"))
  expect_equal(nrow(pred), fit$N * (fit$stan_data$n_time - 1) * length(fit$vars))
  expect_equal(sort(unique(pred$variable)), sort(fit$vars))
  expect_true(all(pred$lower <= pred$mean))
  expect_true(all(pred$upper >= pred$mean))
})

test_that("multilevel pit_values() errors informatively", {
  skip_if_no_rstan()
  fit <- get_multilevel_fit()
  expect_error(pit_values(fit), "not.*supported")
})

# --- Diagnostics -------------------------------------------------------------

test_that("multilevel diagnostics are finite", {
  skip_if_no_rstan()
  fit <- get_multilevel_fit()
  diag <- dcvar_diagnostics(fit)

  expect_lte(diag$n_divergent, 1)
  expect_equal(diag$n_max_treedepth, 0)
  expect_true(is.finite(diag$max_rhat))
  expect_true(diag$max_rhat < 1.25)
  expect_true(diag$min_ess_bulk > 8)
  expect_true(diag$min_ess_tail > 8)
})

test_that("multilevel fit cache emits only known diagnostic warnings", {
  skip_if_no_rstan()

  expect_known_fit_warnings(get_multilevel_fit_warnings(), "multilevel")
})

# ============================================================================
# Tests for time-varying multilevel support
# ============================================================================

make_multilevel_tv_stub_fit <- function(tv_phi = TRUE, tv_sigma = TRUE) {
  n_draw <- 30
  N <- 2L
  n_time <- 4L
  n_eff <- n_time - 1L
  phi_mask <- .resolve_phi_tv_mask(if (isTRUE(tv_phi)) "cross" else FALSE)
  n_phi_tv <- sum(phi_mask)

  variables <- c(
    paste0("phi_bar[", seq_len(4), "]"),
    paste0("tau_phi[", seq_len(4), "]"),
    paste0(
      "z_phi[",
      rep(seq_len(N), each = 4),
      ",",
      rep(seq_len(4), times = N),
      "]"
    ),
    paste0(
      "phi_unit[",
      rep(seq_len(N), each = 4),
      ",",
      rep(seq_len(4), times = N),
      "]"
    ),
    "rho",
    "sigma_eps[1]", "sigma_eps[2]",
    paste0(
      "log_lik[",
      rep(seq_len(N), each = n_eff),
      ",",
      rep(seq_len(n_eff), times = N),
      "]"
    )
  )
  if (isTRUE(tv_phi)) {
    variables <- c(
      variables,
      paste0("tau_phi_tv[", seq_len(n_phi_tv), "]"),
      paste0(
        "phi_raw[",
        rep(seq_len(n_eff), each = n_phi_tv),
        ",",
        rep(seq_len(n_phi_tv), times = n_eff),
        "]"
      ),
      paste0(
        "phi_t[",
        rep(seq_len(n_eff), each = 4),
        ",",
        rep(seq_len(4), times = n_eff),
        "]"
      ),
      paste0(
        "phi_unit_t[",
        rep(seq_len(N), each = n_eff * 4),
        ",",
        rep(rep(seq_len(n_eff), each = 4), times = N),
        ",",
        rep(seq_len(4), times = N * n_eff),
        "]"
      )
    )
  }
  if (isTRUE(tv_sigma)) {
    variables <- c(
      variables,
      "tau_sigma[1]", "tau_sigma[2]",
      paste0(
        "sigma_raw[",
        rep(seq_len(n_eff), each = 2),
        ",",
        rep(seq_len(2), times = n_eff),
        "]"
      ),
      paste0(
        "sigma_t[",
        rep(seq_len(n_eff), each = 2),
        ",",
        rep(seq_len(2), times = n_eff),
        "]"
      )
    )
  }

  draws <- array(
    0,
    dim = c(n_draw, 1, length(variables)),
    dimnames = list(NULL, NULL, variables)
  )
  fill <- function(name, value) {
    draws[, , name] <<- rep(value, length.out = n_draw)
  }

  phi_bar <- c(0.15, 0.05, -0.02, 0.25)
  unit_phi <- rbind(
    c(0.20, 0.40, -0.10, 0.30),
    c(0.10, -0.20, 0.30, 0.40)
  )
  for (k in seq_len(4)) {
    fill(paste0("phi_bar[", k, "]"), phi_bar[k])
    fill(paste0("tau_phi[", k, "]"), 0.10 + 0.01 * k)
    for (i in seq_len(N)) {
      fill(paste0("z_phi[", i, ",", k, "]"), 0)
      fill(paste0("phi_unit[", i, ",", k, "]"), unit_phi[i, k])
    }
  }
  fill("rho", 0.35)
  fill("sigma_eps[1]", 0.9)
  fill("sigma_eps[2]", 1.1)
  for (i in seq_len(N)) {
    for (t in seq_len(n_eff)) {
      fill(paste0("log_lik[", i, ",", t, "]"), -1)
    }
  }

  if (isTRUE(tv_phi)) {
    for (k in seq_len(n_phi_tv)) fill(paste0("tau_phi_tv[", k, "]"), 0.03 + 0.01 * k)
    for (t in seq_len(n_eff)) {
      for (k in seq_len(n_phi_tv)) {
        fill(paste0("phi_raw[", t, ",", k, "]"), 0.01 * t)
      }
    }
    for (t in seq_len(n_eff)) {
      drift <- c(0.01, 0.02, -0.01, 0.015) * (t - 1L)
      for (k in seq_len(4)) {
        fill(paste0("phi_t[", t, ",", k, "]"), phi_bar[k] + drift[k])
        for (i in seq_len(N)) {
          fill(paste0("phi_unit_t[", i, ",", t, ",", k, "]"), unit_phi[i, k] + drift[k])
        }
      }
    }
  }

  if (isTRUE(tv_sigma)) {
    fill("tau_sigma[1]", 0.04)
    fill("tau_sigma[2]", 0.05)
    for (t in seq_len(n_eff)) {
      for (d in seq_len(2)) {
        fill(paste0("sigma_raw[", t, ",", d, "]"), 0.01 * d)
      }
    }
    for (t in seq_len(n_eff)) {
      fill(paste0("sigma_t[", t, ",1]"), 0.9 + 0.04 * t)
      fill(paste0("sigma_t[", t, ",2]"), 1.1 - 0.02 * t)
    }
  }

  y_list <- list(
    matrix(c(1, 2, 2, 1, 3, 0, 4, -1), ncol = 2, byrow = TRUE),
    matrix(c(-1, 1, 0, 2, 1, 3, 2, 4), ncol = 2, byrow = TRUE)
  )
  stan_data <- structure(
    list(
      N = N,
      n_time = n_time,
      y = y_list,
      family = c(1L, 1L),
      tv_phi = as.integer(isTRUE(tv_phi)),
      phi_tv_mask = unname(phi_mask),
      tv_sigma = as.integer(isTRUE(tv_sigma)),
      tau_phi_prior = 0.025,
      tau_sigma_prior = 0.05,
      barrier_k = 8
    ),
    ids = c("a", "b"),
    time_values = 11:14,
    margins = "normal",
    vars = c("y1", "y2"),
    person_means = matrix(c(10, 20, 30, 40), nrow = N, byrow = TRUE),
    tv_phi = isTRUE(tv_phi),
    phi_tv_mask = phi_mask,
    tv_sigma = isTRUE(tv_sigma)
  )

  new_dcvar_multilevel_tv_fit(
    fit = posterior::as_draws_array(draws),
    stan_data = stan_data,
    N = N,
    vars = c("y1", "y2"),
    centered = TRUE,
    person_means = attr(stan_data, "person_means"),
    margins = "normal",
    tv_phi = isTRUE(tv_phi),
    phi_tv_mask = phi_mask,
    tv_sigma = isTRUE(tv_sigma),
    backend = "rstan",
    priors = list(
      phi_bar_sd = 0.5,
      tau_phi_scale = 0.2,
      sigma_sd = 1,
      rho_sd = 0.5,
      tau_phi_rate = 0.025,
      tau_sigma_rate = 0.05
    ),
    meta = list(chains = 1, iter_warmup = 10, iter_sampling = n_draw)
  )
}

test_that("multilevel TV routing and prepared data expose the mixed engine contract", {
  expect_identical(.margin_stan_file("multilevel_tv", "normal"), "multilevel_tv_mixed.stan")
  expect_identical(.margin_stan_file("multilevel_tv", c("normal", "gamma")), "multilevel_tv_mixed.stan")
  expect_match(dcvar_stan_path("multilevel_tv"), "multilevel_tv_mixed\\.stan$")
  expect_true(file.exists(dcvar_stan_path("multilevel_tv")))

  sim <- simulate_dcvar_multilevel(
    N = 3,
    n_time = 6,
    burnin = 0,
    phi_trajectory = list(rep(0.3, 5), rep(0.1, 5), rep(-0.05, 5), seq(0.2, 0.4, length.out = 5)),
    sigma_trajectory = cbind(seq(0.8, 1.1, length.out = 5), rep(1, 5)),
    seed = 202
  )
  expect_equal(dim(sim$true_params$Phi_population), c(5L, 4L))
  expect_length(sim$true_params$Phi_unit_paths, 3L)
  expect_equal(dim(sim$true_params$sigma), c(5L, 2L))

  md <- prepare_multilevel_data(
    sim$data,
    vars = c("y1", "y2"),
    tv_phi = "cross",
    tv_sigma = TRUE
  )
  expect_identical(md$tv_phi, 1L)
  expect_identical(md$phi_tv_mask, c(0L, 1L, 1L, 0L))
  expect_identical(md$tv_sigma, 1L)
  expect_identical(md$family, c(1L, 1L))
  expect_identical(md$skew_direction, c(1, 1))
  expect_true(attr(md, "tv_phi"))
  expect_true(attr(md, "tv_sigma"))
})

test_that("multilevel TV initial values size active random-walk containers only", {
  init <- .init_multilevel_tv_params(2, 3, 6, "normal", tv_phi = "ar", tv_sigma = TRUE)
  expect_length(init$tau_phi_tv, 2)
  expect_identical(dim(init$phi_raw), c(5L, 2L))
  expect_length(init$tau_sigma, 2)
  expect_identical(dim(init$sigma_raw), c(5L, 2L))
  expect_identical(dim(init$z_phi), c(3L, 4L))

  constant <- .init_multilevel_tv_params(2, 3, 6, "normal", tv_phi = FALSE, tv_sigma = FALSE)
  expect_null(constant$tau_phi_tv)
  expect_null(constant$phi_raw)
  expect_null(constant$tau_sigma)
  expect_null(constant$sigma_raw)
})

test_that("multilevel TV fit object methods report population and unit trajectories", {
  fit <- make_multilevel_tv_stub_fit(tv_phi = TRUE, tv_sigma = TRUE)
  expect_s3_class(fit, "dcvar_multilevel_tv_fit")
  expect_identical(class(fit), c("dcvar_multilevel_tv_fit", "dcvar_multilevel_fit", "dcvar_model_fit"))
  expect_identical(fit$model, "multilevel_tv")
  expect_true(fit$tv_phi)
  expect_true(fit$tv_sigma)
  expect_identical(unname(fit$phi_tv_mask), c(0L, 1L, 1L, 0L))

  co <- coef(fit)
  expect_true(all(c("phi_bar", "tau_phi", "sigma_eps", "rho", "tau_phi_tv", "tau_sigma") %in% names(co)))
  expect_identical(names(co$tau_phi_tv), c("tau_phi_tv[phi12]", "tau_phi_tv[phi21]"))

  vp <- var_params(fit)
  expect_true(all(c("phi_bar", "tau_phi", "sigma_eps", "rho", "tau_phi_tv", "tau_sigma") %in% names(vp)))
  expect_identical(vp$tau_phi_tv$variable, c("tau_phi_tv[phi12]", "tau_phi_tv[phi21]"))

  rho_df <- rho_trajectory(fit)
  expect_equal(nrow(rho_df), 3L)
  expect_identical(rho_df$time, 12:14)
  expect_true(length(unique(rho_df$mean)) == 1L)

  phi_pop <- phi_trajectory(fit)
  expect_equal(nrow(phi_pop), 12L)
  expect_false("unit" %in% names(phi_pop))
  expect_identical(sort(unique(phi_pop$coefficient)), c("phi11", "phi12", "phi21", "phi22"))

  phi_units <- phi_trajectory(fit, unit = "all")
  expect_equal(nrow(phi_units), 24L)
  expect_identical(sort(unique(as.character(phi_units$unit))), c("a", "b"))

  sigma_df <- sigma_trajectory(fit)
  expect_equal(nrow(sigma_df), 6L)
  expect_true(all(sigma_df$mean > 0))

  dep_df <- dependence_summary(fit)
  expect_equal(nrow(dep_df), 3L)

  expect_output(print(fit), "TV Multilevel")
  s <- summary(fit)
  expect_s3_class(s, "dcvar_multilevel_tv_summary")
  expect_false(is.null(s$phi_summary))
  expect_false(is.null(s$sigma_summary))
  expect_silent(invisible(capture.output(print(s))))
})

test_that("multilevel TV fitted and predict use unit-time Phi and sigma paths", {
  fit <- make_multilevel_tv_stub_fit(tv_phi = TRUE, tv_sigma = TRUE)

  fv <- fitted(fit)
  expect_equal(nrow(fv), 6L)
  expect_identical(names(fv), c("unit", "time", "y1", "y2"))
  expect_equal(fv$y1[1], 1 * 0.20 + 2 * 0.40)
  expect_equal(fv$y2[1], 1 * -0.10 + 2 * 0.30)
  a_t13 <- fv[fv$unit == "a" & fv$time == 13, ]
  expect_equal(a_t13$y1, 0.84)
  expect_equal(a_t13$y2, 0.095)

  response <- fitted(fit, type = "response")
  expect_equal(response$y1[1], fv$y1[1] + 10)
  expect_equal(response$y2[1], fv$y2[1] + 20)

  pr <- predict(fit, ci_level = 0.8)
  expect_equal(nrow(pr), 12L)
  expect_true(all(c("unit", "time", "variable", "mean", "lower", "upper") %in% names(pr)))
  expect_true(all(pr$lower <= pr$mean))
  expect_true(all(pr$mean <= pr$upper))
  pr_a_t13_y1 <- pr[pr$unit == "a" & pr$time == 13 & pr$variable == "y1", ]
  expect_equal(pr_a_t13_y1$mean, 0.84)
  expect_equal(pr_a_t13_y1$upper - pr_a_t13_y1$mean, stats::qnorm(0.9) * 0.98)
  expect_equal(pr_a_t13_y1$mean - pr_a_t13_y1$lower, stats::qnorm(0.9) * 0.98)
})

test_that("multilevel TV phi_trajectory tiles baselines when tv_phi is off", {
  fit <- make_multilevel_tv_stub_fit(tv_phi = FALSE, tv_sigma = TRUE)
  expect_false(fit$tv_phi)

  phi_pop <- phi_trajectory(fit)
  expect_equal(nrow(phi_pop), 12L)
  per_coef <- tapply(phi_pop$mean, phi_pop$coefficient, function(x) length(unique(x)))
  expect_true(all(per_coef == 1L))

  phi_units <- phi_trajectory(fit, unit = c("a", "b"))
  expect_equal(nrow(phi_units), 24L)
  expect_identical(sort(unique(as.character(phi_units$unit))), c("a", "b"))
})

make_constant_clayton_stub_fit <- function() {
  variables <- c(
    "mu[1]", "mu[2]",
    "Phi[1,1]", "Phi[1,2]", "Phi[2,1]", "Phi[2,2]",
    "sigma_eps[1]", "sigma_eps[2]",
    "theta",
    paste0("log_lik[", 1:4, "]")
  )
  draws <- array(
    rnorm(40 * length(variables), 0, 0.1),
    dim = c(40, 1, length(variables)),
    dimnames = list(NULL, NULL, variables)
  )
  draws[, , "sigma_eps[1]"] <- runif(40, 0.8, 1.2)
  draws[, , "sigma_eps[2]"] <- runif(40, 0.8, 1.2)
  draws[, , "theta"] <- runif(40, 0.2, 2.5)
  draws[, , paste0("log_lik[", 1:4, "]")] <- -abs(draws[, , paste0("log_lik[", 1:4, "]")])

  stan_data <- structure(
    list(n_time = 5, D = 2),
    time_values = 101:105,
    copula = "clayton"
  )

  new_dcvar_constant_fit(
    fit = posterior::as_draws_array(draws),
    stan_data = stan_data,
    vars = c("y1", "y2"),
    standardized = TRUE,
    margins = "normal",
    copula = "clayton",
    backend = "rstan",
    priors = list(),
    meta = list(chains = 1, iter_warmup = 10, iter_sampling = 40)
  )
}

test_that("Clayton constant dispatch resolves the bundled Stan file", {
  expect_equal(.margin_stan_file("constant", "normal", copula = "clayton"),
               "constant_NCl.stan")
  expect_true(file.exists(dcvar_stan_path("constant", copula = "clayton")))
  expect_error(.margin_stan_file("constant", "exponential", copula = "clayton"),
               "normal margins")
})

test_that("Clayton constant stub fit extractors use theta and Kendall tau", {
  fit <- make_constant_clayton_stub_fit()

  co <- coef(fit)
  expect_named(co, c("mu", "Phi", "sigma_eps", "theta"))
  expect_false("rho" %in% names(co))

  vp <- var_params(fit)
  expect_true("theta" %in% names(vp))

  dep <- dependence_summary(fit, probs = c(0.025, 0.5, 0.975))
  expect_s3_class(dep, "data.frame")
  expect_equal(dep$time, 102:105)
  expect_true(all(dep$mean > 0 & dep$mean < 1))

  expect_error(rho_trajectory(fit), "dependence_summary")
})

test_that("Clayton constant summary, print, and loo work on stubs", {
  fit <- make_constant_clayton_stub_fit()

  expect_no_error(capture.output(print(fit)))
  s <- summary(fit)
  expect_s3_class(s, "dcvar_constant_summary")
  expect_equal(s$copula, "clayton")
  expect_s3_class(s$dependence, "data.frame")

  out <- suppressWarnings(loo::loo(fit))
  expect_s3_class(out, "loo")
})

test_that("dcvar_constant fits the Clayton-normal model end to end", {
  skip_if_no_rstan()

  fit <- get_constant_clayton_fit()
  expect_s3_class(fit, "dcvar_constant_fit")
  expect_identical(fit$copula, "clayton")

  dep <- dependence_summary(fit)
  expect_true(all(is.finite(dep$mean)))
  expect_true(all(dep$mean > -1 & dep$mean < 1))

  ll <- loo(fit)
  expect_s3_class(ll, "loo")

  # Regression: interpret_rho_trajectory() previously errored on Clayton fits
  out <- capture.output(msg <- interpret_rho_trajectory(fit))
  expect_match(msg, "Kendall")
})

# Per-variable (mixed) margins for the dynamic (DC-VAR) and HMM models (Phase 2).

# --- routing (fast, no fit) -------------------------------------------------

test_that("mixed margins route to the generic dynamic and HMM Stan models", {
  expect_equal(.margin_stan_file("dcvar", c("normal", "exponential")),
               "dcvar_mixed_ncp.stan")
  expect_equal(.margin_stan_file("hmm", c("normal", "gamma")),
               "hmm_mixed.stan")
  # Identical-family vectors still route to the specialised models.
  expect_equal(.margin_stan_file("dcvar", c("normal", "normal")),
               "dcvar_model_ncp.stan")
  expect_equal(.margin_stan_file("hmm", c("gamma", "gamma")),
               "hmm_GG.stan")
})

test_that("prepare_dcvar_data / prepare_hmm_data build the family array for mixed", {
  df <- data.frame(time = 1:40, y1 = rnorm(40), y2 = cumsum(rnorm(40)))
  sd_dc <- prepare_dcvar_data(df, vars = c("y1", "y2"),
                              margins = c("normal", "exponential"),
                              skew_direction = c(1, 1))
  expect_equal(sd_dc$family, c(1L, 2L))
  expect_false(is.null(sd_dc$sigma_eps_prior))
  expect_equal(sd_dc$skew_direction, c(1, 1))

  sd_hmm <- prepare_hmm_data(df, vars = c("y1", "y2"), K = 2,
                             margins = c("normal", "skew_normal"))
  expect_equal(sd_hmm$family, c(1L, 3L))
  # skew_direction defaults to +1 when no exp/gamma dimension is present.
  expect_equal(sd_hmm$skew_direction, c(1, 1))
})

# --- DC-VAR mixed fit -------------------------------------------------------

test_that("dcvar() fits mixed normal + exponential margins", {
  skip_if_no_rstan()

  fit <- get_dcvar_mixed_fit()
  expect_s3_class(fit, "dcvar_fit")
  expect_equal(fit$margins, c("normal", "exponential"))
  expect_equal(fit$stan_data$family, c(1L, 2L))
})

test_that("dcvar mixed fit emits only known diagnostic warnings", {
  skip_if_no_rstan()

  expect_known_fit_warnings(get_dcvar_mixed_fit_warnings(), "dcvar mixed")
})

test_that("dcvar mixed coef() and var_params() report per-dimension families", {
  skip_if_no_rstan()

  fit <- get_dcvar_mixed_fit()
  co <- coef(fit)
  expect_equal(names(co$sigma_eps), "sigma_eps[1]")
  expect_equal(names(co$sigma_exp), "sigma_exp[2]")

  vp <- var_params(fit)
  expect_true(all(c("sigma_eps", "sigma_exp", "sigma_omega") %in% names(vp)))
})

test_that("dcvar mixed recovers a declining rho trajectory", {
  skip_if_no_rstan()

  rt <- rho_trajectory(get_dcvar_mixed_fit())
  expect_equal(nrow(rt), 119)
  expect_true(all(rt$mean > -1 & rt$mean < 1))
  # True trajectory is decreasing: the early correlation should exceed the late.
  expect_true(mean(rt$mean[1:30]) > mean(rt$mean[90:119]))
})

test_that("dcvar mixed pit_values dispatches per dimension", {
  skip_if_no_rstan()

  pit_df <- pit_values(get_dcvar_mixed_fit())
  expect_setequal(unique(pit_df$variable), c("y1", "y2"))
  expect_true(all(pit_df$pit >= 0 & pit_df$pit <= 1))
})

# --- HMM mixed fit ----------------------------------------------------------

test_that("dcvar_hmm() fits mixed normal + exponential margins", {
  skip_if_no_rstan()

  fit <- get_hmm_mixed_fit()
  expect_s3_class(fit, "dcvar_hmm_fit")
  expect_equal(fit$margins, c("normal", "exponential"))
  expect_equal(fit$stan_data$family, c(1L, 2L))
})

test_that("hmm mixed fit emits only known diagnostic warnings", {
  skip_if_no_rstan()

  expect_known_fit_warnings(get_hmm_mixed_fit_warnings(), "hmm mixed")
})

test_that("hmm mixed coef() reports per-dimension families", {
  skip_if_no_rstan()

  co <- coef(get_hmm_mixed_fit())
  expect_equal(names(co$sigma_eps), "sigma_eps[1]")
  expect_equal(names(co$sigma_exp), "sigma_exp[2]")
})

test_that("hmm mixed recovers two separated correlation regimes", {
  skip_if_no_rstan()

  st <- hmm_states(get_hmm_mixed_fit())
  # Ordered states; the true regimes are rho ~ 0.1 and rho ~ 0.8.
  expect_true(st$rho_state$mean[1] < st$rho_state$mean[2])
  expect_true(st$rho_state$mean[1] < 0.5)
  expect_true(st$rho_state$mean[2] > 0.4)
  expect_equal(dim(st$gamma), c(119L, 2L))
})

# ============================================================================
# Tests for dcvar_sem() fit and methods
# ============================================================================

# --- Fit object structure ----------------------------------------------------

make_sem_mixed_engine_stub_fit <- function(margins = c("skew_normal", "gamma")) {
  margins <- match.arg(margins)
  n_time <- 4
  variables <- c(
    "mu[1]", "mu[2]",
    "phi11", "phi12", "phi21", "phi22",
    "Phi[1,1]", "Phi[1,2]", "Phi[2,1]", "Phi[2,2]",
    "rho_raw", "rho",
    paste0("state[", rep(seq_len(n_time), each = 2), ",", rep(1:2, times = n_time), "]"),
    paste0("log_lik[", seq_len(n_time), "]")
  )
  if (identical(margins, "skew_normal")) {
    variables <- c(variables, "omega[1]", "omega[2]", "delta[1]", "delta[2]")
  } else {
    variables <- c(
      variables,
      "eta[1]", "eta[2]",
      "sigma_gam[1]", "sigma_gam[2]",
      "shape_gam[1]", "shape_gam[2]"
    )
  }

  draws <- array(
    rnorm(40 * length(variables), 0, 0.1),
    dim = c(40, 1, length(variables)),
    dimnames = list(NULL, NULL, variables)
  )
  draws[, , "rho"] <- runif(40, -0.5, 0.5)
  draws[, , paste0("log_lik[", seq_len(n_time), "]")] <-
    -abs(draws[, , paste0("log_lik[", seq_len(n_time), "]")])
  if (identical(margins, "skew_normal")) {
    draws[, , c("omega[1]", "omega[2]")] <- runif(40 * 2, 0.5, 1.5)
    draws[, , c("delta[1]", "delta[2]")] <- runif(40 * 2, -0.5, 0.5)
  } else {
    draws[, , c("sigma_gam[1]", "sigma_gam[2]")] <- runif(40 * 2, 0.5, 1.5)
    draws[, , c("shape_gam[1]", "shape_gam[2]")] <- runif(40 * 2, 0.5, 2.0)
  }

  indicators <- list(
    latent1 = c("y1_1", "y1_2"),
    latent2 = c("y2_1", "y2_2")
  )
  stan_data <- structure(
    list(
      n_time = n_time,
      J = 2,
      y = matrix(rnorm(n_time * 4), n_time, 4),
      lambda = c(0.8, 0.8),
      sigma_e = 0.5
    ),
    time_values = 21:24,
    indicators = indicators,
    vars = names(indicators),
    margins = margins,
    method = "indicator",
    J = 2,
    skew_direction = if (identical(margins, "gamma")) c(1, -1) else NULL
  )

  new_dcvar_sem_fit(
    fit = posterior::as_draws_array(draws),
    stan_data = stan_data,
    vars = names(indicators),
    J = 2,
    lambda = c(0.8, 0.8),
    sigma_e = 0.5,
    indicators = indicators,
    margins = margins,
    method = "indicator",
    skew_direction = attr(stan_data, "skew_direction"),
    backend = "rstan",
    priors = list(),
    meta = list(chains = 1, iter_warmup = 10, iter_sampling = 40)
  )
}

test_that("dcvar_sem returns correct class", {
  skip_if_no_rstan()
  fit <- get_sem_fit()
  expect_s3_class(fit, "dcvar_sem_fit")
  expect_s3_class(fit, "dcvar_model_fit")
  expect_equal(fit$margins, "normal")
})

test_that("SEM coef() returns expected structure", {
  skip_if_no_rstan()
  fit <- get_sem_fit()
  co <- coef(fit)

  expect_type(co, "list")
  expect_named(co, c("mu", "Phi", "sigma", "rho"))
  expect_true(length(co$mu) == 2)
  expect_true(length(co$Phi) == 4)
  expect_true(length(co$sigma) == 2)
  expect_true(length(co$rho) == 1)
})

test_that("SEM exponential margins return expected structure", {
  skip_if_no_rstan()
  fit <- get_sem_exponential_fit()
  co <- coef(fit)
  vp <- var_params(fit)

  expect_equal(fit$margins, "exponential")
  expect_equal(fit$skew_direction, c(1, -1))
  expect_named(co, c("mu", "Phi", "sigma_exp", "rho"))
  expect_true(length(co$sigma_exp) == 2)
  expect_named(vp, c("mu", "Phi", "sigma_exp", "rho"))
  expect_s3_class(vp$sigma_exp, "data.frame")
})

test_that("SEM homogeneous skew_normal and gamma stubs report mixed-engine params", {
  skew_fit <- make_sem_mixed_engine_stub_fit("skew_normal")
  skew_co <- coef(skew_fit)
  skew_vp <- var_params(skew_fit)
  expect_named(skew_co, c("mu", "Phi", "omega", "delta", "rho"))
  expect_equal(names(skew_co$omega), c("omega[1]", "omega[2]"))
  expect_true(all(c("omega", "delta") %in% names(skew_vp)))

  gamma_fit <- make_sem_mixed_engine_stub_fit("gamma")
  gamma_co <- coef(gamma_fit)
  gamma_vp <- var_params(gamma_fit)
  expect_named(gamma_co, c("mu", "Phi", "sigma_gam", "shape_gam", "rho"))
  expect_equal(names(gamma_co$shape_gam), c("shape_gam[1]", "shape_gam[2]"))
  expect_true(all(c("sigma_gam", "shape_gam") %in% names(gamma_vp)))
})

test_that("SEM homogeneous skew_normal and gamma slow fits run", {
  skip_if_no_rstan()
  skip_if_not_slow()

  for (margin in c("skew_normal", "gamma")) {
    J <- 2
    skew_direction <- if (identical(margin, "gamma")) c(1, 1) else NULL
    sim <- simulate_dcvar_sem(
      n_time = 80, J = J, lambda = rep(0.8, J), sigma_e = sqrt(0.2),
      margins = margin,
      skew_direction = skew_direction,
      skew_params = if (identical(margin, "skew_normal")) list(alpha = c(2, 2)) else list(shape = 2),
      rho = 0.5,
      seed = 500 + match(margin, c("skew_normal", "gamma"))
    )
    fit <- dcvar_sem(
      sim$data,
      indicators = list(latent1 = paste0("y1_", seq_len(J)), latent2 = paste0("y2_", seq_len(J))),
      J = J,
      lambda = rep(0.8, J),
      sigma_e = sqrt(0.2),
      margins = margin,
      skew_direction = skew_direction,
      chains = 2, iter_warmup = 250, iter_sampling = 250,
      adapt_delta = 0.999, max_treedepth = 13, refresh = 0, seed = 123
    )

    expect_equal(fit$margins, margin)
    expect_equal(fit$stan_data$family, rep(unname(.family_codes[[margin]]), 2))
    expect_true(coef(fit)$rho > -0.2)
  }
})

test_that("SEM summary() returns correct class", {
  skip_if_no_rstan()
  fit <- get_sem_fit()
  s <- summary(fit)

  expect_s3_class(s, "dcvar_sem_summary")
  expect_true(all(c("var_params", "diagnostics") %in% names(s)))
})

test_that("SEM print() works", {
  skip_if_no_rstan()
  fit <- get_sem_fit()
  expect_output(print(fit), "SEM|sem")
})

# --- Extraction functions ----------------------------------------------------

test_that("SEM rho_trajectory() returns expected structure", {
  skip_if_no_rstan()
  fit <- get_sem_fit()
  rho_df <- rho_trajectory(fit)

  expect_s3_class(rho_df, "data.frame")
  expect_true(all(c("time", "mean", "sd") %in% names(rho_df)))
  # SEM has constant rho: all means should be identical
  expect_equal(length(unique(rho_df$mean)), 1)
  # rho should be bounded within (-0.97, 0.97)
  expect_true(all(rho_df$mean > -1 & rho_df$mean < 1))
})

test_that("SEM latent_states() returns expected structure", {
  skip_if_no_rstan()
  fit <- get_sem_fit()
  ls <- latent_states(fit)

  expect_s3_class(ls, "data.frame")
  expect_true(all(c("time", "variable", "mean", "sd") %in% names(ls)))
  expect_true(all(fit$vars %in% ls$variable))
  # Should have T rows per variable (2 variables)
  expect_equal(nrow(ls), fit$stan_data$n_time * 2)
})

test_that("SEM var_params() returns expected structure", {
  skip_if_no_rstan()
  fit <- get_sem_fit()
  vp <- var_params(fit)

  expect_type(vp, "list")
  expect_named(vp, c("mu", "Phi", "sigma", "rho"))
  expect_s3_class(vp$mu, "data.frame")
  expect_true(all(c("variable", "mean", "sd", "q2.5", "q97.5") %in% names(vp$mu)))
})

test_that("SEM exponential fitted() and predict() return expected structures", {
  skip_if_no_rstan()
  fit <- get_sem_exponential_fit()

  fit_link <- fitted(fit, type = "link")
  fit_resp <- fitted(fit, type = "response")
  pred_resp <- predict(fit, type = "response")
  indicator_names <- unlist(fit$indicators, use.names = FALSE)

  expect_s3_class(fit_link, "data.frame")
  expect_named(fit_link, c("time", fit$vars))
  expect_s3_class(fit_resp, "data.frame")
  expect_named(fit_resp, c("time", indicator_names))
  expect_s3_class(pred_resp, "data.frame")
  expect_named(pred_resp, c("time", "variable", "mean", "lower", "upper"))
})

# --- Plotting ----------------------------------------------------------------

test_that("SEM plot latent_states returns ggplot", {
  skip_if_no_rstan()
  fit <- get_sem_fit()
  p <- plot(fit, type = "latent_states")
  expect_s3_class(p, "ggplot")
})

test_that("SEM plot diagnostics returns ggplot", {
  skip_if_no_rstan()
  fit <- get_sem_fit()
  p <- plot(fit, type = "diagnostics")
  expect_s3_class(p, "ggplot")
})

# --- Fitted values and prediction --------------------------------------------

test_that("SEM fitted() returns latent and indicator trajectories", {
  skip_if_no_rstan()
  fit <- get_sem_fit()

  fit_link <- fitted(fit, type = "link")
  fit_resp <- fitted(fit, type = "response")
  indicator_names <- unlist(fit$indicators, use.names = FALSE)

  expect_s3_class(fit_link, "data.frame")
  expect_named(fit_link, c("time", fit$vars))
  expect_equal(nrow(fit_link), fit$stan_data$n_time)

  expect_s3_class(fit_resp, "data.frame")
  expect_named(fit_resp, c("time", indicator_names))
  expect_equal(nrow(fit_resp), fit$stan_data$n_time)
})

test_that("SEM predict() returns latent and indicator intervals", {
  skip_if_no_rstan()
  fit <- get_sem_fit()

  pred_link <- predict(fit, type = "link")
  pred_resp <- predict(fit, type = "response")
  indicator_names <- unlist(fit$indicators, use.names = FALSE)

  expect_s3_class(pred_link, "data.frame")
  expect_named(pred_link, c("time", "variable", "mean", "lower", "upper"))
  expect_equal(nrow(pred_link), fit$stan_data$n_time * length(fit$vars))
  expect_equal(sort(unique(pred_link$variable)), sort(fit$vars))

  expect_s3_class(pred_resp, "data.frame")
  expect_named(pred_resp, c("time", "variable", "mean", "lower", "upper"))
  expect_equal(nrow(pred_resp), fit$stan_data$n_time * length(indicator_names))
  expect_equal(sort(unique(pred_resp$variable)), sort(indicator_names))
  expect_true(all(pred_resp$lower <= pred_resp$mean))
  expect_true(all(pred_resp$upper >= pred_resp$mean))
})

# --- Diagnostics -------------------------------------------------------------

test_that("SEM diagnostics are finite", {
  skip_if_no_rstan()
  fit <- get_sem_fit()
  diag <- dcvar_diagnostics(fit)

  expect_lte(diag$n_divergent, 1)
  expect_equal(diag$n_max_treedepth, 0)
  expect_true(is.finite(diag$max_rhat))
  expect_true(diag$max_rhat < 1.30)
  expect_true(diag$min_ess_bulk > 8)
  expect_true(diag$min_ess_tail > 8)
})

test_that("SEM fit cache emits only known diagnostic warnings", {
  skip_if_no_rstan()

  expect_known_fit_warnings(get_sem_fit_warnings(), "SEM")
})

test_that("SEM exponential fit cache emits only known diagnostic warnings", {
  skip_if_no_rstan()

  expect_known_fit_warnings(get_sem_exponential_fit_warnings(), "SEM exponential")
})

# Per-variable (mixed) margins for the constant copula model (Phase 1).

test_that("dcvar_constant() fits mixed normal + exponential margins", {
  skip_if_no_rstan()

  fit <- get_constant_mixed_fit()
  expect_s3_class(fit, "dcvar_constant_fit")
  expect_equal(fit$margins, c("normal", "exponential"))
  # The mixed Stan model receives a per-dimension family array.
  expect_equal(fit$stan_data$family, c(1L, 2L))
})

test_that("mixed fit emits only known diagnostic warnings", {
  skip_if_no_rstan()

  expect_known_fit_warnings(get_constant_mixed_fit_warnings(), "constant mixed")
})

test_that("coef() reports each dimension under its own family", {
  skip_if_no_rstan()

  co <- coef(get_constant_mixed_fit())
  expect_named(co, c("mu", "Phi", "sigma_eps", "sigma_exp", "rho"))
  # Normal dim reports sigma_eps[1]; exponential dim reports sigma_exp[2].
  expect_equal(names(co$sigma_eps), "sigma_eps[1]")
  expect_equal(names(co$sigma_exp), "sigma_exp[2]")
})

test_that("var_params() returns per-dimension scale groups for mixed fits", {
  skip_if_no_rstan()

  vp <- var_params(get_constant_mixed_fit())
  expect_true(all(c("mu", "Phi", "sigma_eps", "sigma_exp") %in% names(vp)))
  expect_equal(vp$sigma_eps$variable, "sigma_eps[1]")
  expect_equal(vp$sigma_exp$variable, "sigma_exp[2]")
})

test_that("summary() and print() run for mixed fits", {
  skip_if_no_rstan()

  fit <- get_constant_mixed_fit()
  expect_no_error(capture.output(print(fit)))
  s <- summary(fit)
  out <- capture.output(print(s))
  expect_true(any(grepl("sigma_eps", out)))
  expect_true(any(grepl("sigma_exp", out)))
})

test_that("pit_values() dispatches per dimension for mixed fits", {
  skip_if_no_rstan()

  pit_df <- pit_values(get_constant_mixed_fit())
  expect_true(all(c("time", "variable", "pit") %in% names(pit_df)))
  expect_setequal(unique(pit_df$variable), c("y1", "y2"))
  expect_true(all(pit_df$pit >= 0 & pit_df$pit <= 1))
})

test_that("predict() rejects mixed margins (non-normal)", {
  skip_if_no_rstan()

  expect_error(predict(get_constant_mixed_fit()), "normal margins")
})

test_that("mixed normal + exponential recovers the constant rho", {
  skip_if_no_rstan()

  rho_df <- rho_trajectory(get_constant_mixed_fit())
  # Standardisation shrinks the recovered correlation slightly; allow a
  # generous band around the simulated rho = 0.6.
  expect_true(rho_df$mean[1] > 0.35 && rho_df$mean[1] < 0.8)
})

test_that("identical-family vectors reuse the specialised model and collapse margins", {
  skip_if_no_rstan()
  skip_if_not_slow()

  sim <- simulate_dcvar(
    n_time = 40, rho_trajectory = rho_constant(40, 0.5), seed = 42
  )
  fit <- dcvar_constant(
    sim$Y_df, vars = c("y1", "y2"),
    margins = c("normal", "normal"),
    chains = 1, iter_warmup = 75, iter_sampling = 75,
    refresh = 0, seed = 123
  )
  # Homogeneous vector is treated exactly like the scalar form.
  expect_identical(fit$margins, "normal")
  expect_null(fit$stan_data$family)
  expect_named(coef(fit), c("mu", "Phi", "sigma_eps", "rho"))
})


# ---------------------------------------------------------------------------
# Coverage across every ordered family pair
#
# Exercises each marginal branch of constant_mixed.stan (normal / exponential /
# skew_normal / gamma) in combination, and checks the R extraction picks up the
# right per-dimension scale/shape variables. These are short smoke fits: they
# only need the Stan variables to exist, not full convergence, so warnings are
# suppressed and recovery is asserted separately below.
# ---------------------------------------------------------------------------

.simulate_mixed_pair <- function(fams, n = 120, rho = 0.6, seed = 7) {
  args <- list(n_time = n, rho_trajectory = rho_constant(n, rho),
               margins = fams, seed = seed)
  if (any(fams %in% c("exponential", "gamma"))) args$skew_direction <- c(1, 1)
  sp <- list()
  if (any(fams == "gamma")) sp$shape <- 2
  if (any(fams == "skew_normal")) sp$alpha <- c(3, 3)
  if (length(sp) > 0L) args$skew_params <- sp
  do.call(simulate_dcvar, args)
}

.fit_mixed_pair <- function(fams, seed = 7,
                            iter_warmup = 150, iter_sampling = 150,
                            chains = 1, adapt_delta = 0.95,
                            max_treedepth = 11) {
  sim <- .simulate_mixed_pair(fams, seed = seed)
  args <- list(
    sim$Y_df, vars = c("y1", "y2"), margins = fams,
    chains = chains, iter_warmup = iter_warmup, iter_sampling = iter_sampling,
    adapt_delta = adapt_delta, max_treedepth = max_treedepth,
    refresh = 0, seed = 123
  )
  if (any(fams %in% c("exponential", "gamma"))) args$skew_direction <- c(1, 1)
  suppressWarnings(do.call(dcvar_constant, args))
}

mixed_family_pairs <- list(
  c("normal", "gamma"),
  c("normal", "skew_normal"),
  c("exponential", "gamma"),
  c("exponential", "skew_normal"),
  c("gamma", "skew_normal")
)

for (.fams in mixed_family_pairs) {
  local({
    fams <- .fams
    label <- paste(fams, collapse = " + ")
    test_that(sprintf("mixed %s fits and reports the right per-dim families", label), {
      skip_if_no_rstan()
      skip_if_not_slow()
      if (any(fams == "skew_normal")) skip_if_not_installed("sn")

      fit <- .fit_mixed_pair(fams)
      expect_s3_class(fit, "dcvar_constant_fit")
      expect_equal(fit$margins, fams)
      expect_equal(fit$stan_data$family, unname(.family_codes[fams]))

      # coef() reports each dimension under its own family's scale/shape group.
      co <- coef(fit)
      expect_true("rho" %in% names(co))
      expected_groups <- names(.mixed_margin_report_vars(fams))
      expect_true(all(expected_groups %in% names(co)))

      # var_params() exposes the same per-dimension groups.
      vp <- var_params(fit)
      expect_true(all(expected_groups %in% names(vp)))

      # PIT runs per dimension and stays in the unit interval.
      pit_df <- pit_values(fit)
      expect_true(all(pit_df$pit >= 0 & pit_df$pit <= 1))
    })
  })
}

test_that("mixed normal + gamma recovers rho and the VAR autoregression", {
  skip_if_no_rstan()
  skip_if_not_slow()

  fams <- c("normal", "gamma")
  sim <- .simulate_mixed_pair(fams, n = 150, rho = 0.6, seed = 13)
  fit <- suppressWarnings(dcvar_constant(
    sim$Y_df, vars = c("y1", "y2"), margins = fams,
    skew_direction = c(1, 1),
    chains = 2, iter_warmup = 500, iter_sampling = 500,
    adapt_delta = 0.999, max_treedepth = 14, refresh = 0, seed = 123
  ))

  rho_mean <- rho_trajectory(fit)$mean[1]
  expect_true(rho_mean > 0.3 && rho_mean < 0.8)

  # The VAR(1) diagonal (true 0.3 on each series) should be recovered loosely
  # even after standardisation.
  phi <- coef(fit)$Phi
  expect_true(phi[["Phi[1,1]"]] > 0 && phi[["Phi[1,1]"]] < 0.6)
  expect_true(phi[["Phi[2,2]"]] > 0 && phi[["Phi[2,2]"]] < 0.6)
})


# ---------------------------------------------------------------------------
# Mixed margins with the Clayton copula (Phase 3)
# ---------------------------------------------------------------------------

test_that("mixed margins route to the Clayton mixed model", {
  expect_equal(
    .margin_stan_file("constant", c("normal", "exponential"), copula = "clayton"),
    "constant_mixed_clayton.stan"
  )
  # Clayton mixed is constant-only.
  expect_error(
    .margin_stan_file("dcvar", c("normal", "exponential"), copula = "clayton"),
    "Clayton.*constant"
  )
})

test_that("dcvar_constant() fits mixed margins under the Clayton copula", {
  skip_if_no_rstan()

  fit <- get_constant_mixed_clayton_fit()
  expect_s3_class(fit, "dcvar_constant_fit")
  expect_equal(fit$margins, c("normal", "exponential"))
  expect_equal(fit$copula, "clayton")
  expect_equal(fit$stan_data$family, c(1L, 2L))

  co <- coef(fit)
  # Clayton fits report theta (not rho) plus per-dimension scale groups.
  expect_true(all(c("sigma_eps", "sigma_exp", "theta") %in% names(co)))
  expect_false("rho" %in% names(co))

  # Kendall's tau is available and positive for the simulated positive dependence.
  dep <- dependence_summary(fit)
  expect_true(dep$mean[1] > 0)
})

test_that("clayton mixed fit emits only known diagnostic warnings", {
  skip_if_no_rstan()

  expect_known_fit_warnings(get_constant_mixed_clayton_fit_warnings(), "constant mixed clayton")
})

make_sem_naive_stub_fit <- function(margins = c("normal", "exponential", "skew_normal", "gamma")) {
  margins <- match.arg(margins)
  n_time <- 4
  variables <- c(
    "mu[1]", "mu[2]",
    "phi11", "phi12", "phi21", "phi22",
    "Phi[1,1]", "Phi[1,2]", "Phi[2,1]", "Phi[2,2]",
    "rho_raw", "rho",
    paste0("log_lik[", seq_len(n_time), "]")
  )
  if (identical(margins, "exponential")) {
    variables <- c(variables, "eta[1]", "eta[2]", "sigma_exp[1]", "sigma_exp[2]")
  } else if (identical(margins, "skew_normal")) {
    variables <- c(variables, "omega[1]", "omega[2]", "delta[1]", "delta[2]")
  } else if (identical(margins, "gamma")) {
    variables <- c(
      variables,
      "eta[1]", "eta[2]",
      "sigma_gam[1]", "sigma_gam[2]",
      "shape_gam[1]", "shape_gam[2]"
    )
  } else {
    variables <- c(variables, "sigma[1]", "sigma[2]")
  }

  draws <- array(
    rnorm(40 * length(variables), 0, 0.1),
    dim = c(40, 1, length(variables)),
    dimnames = list(NULL, NULL, variables)
  )
  draws[, , "rho"] <- runif(40, -0.5, 0.5)
  draws[, , paste0("log_lik[", seq_len(n_time), "]")] <-
    -abs(draws[, , paste0("log_lik[", seq_len(n_time), "]")])
  if (identical(margins, "exponential")) {
    draws[, , c("sigma_exp[1]", "sigma_exp[2]")] <- runif(40 * 2, 0.5, 1.5)
  } else if (identical(margins, "skew_normal")) {
    draws[, , c("omega[1]", "omega[2]")] <- runif(40 * 2, 0.5, 1.5)
    draws[, , c("delta[1]", "delta[2]")] <- runif(40 * 2, -0.5, 0.5)
  } else if (identical(margins, "gamma")) {
    draws[, , c("sigma_gam[1]", "sigma_gam[2]")] <- runif(40 * 2, 0.5, 1.5)
    draws[, , c("shape_gam[1]", "shape_gam[2]")] <- runif(40 * 2, 0.5, 2.0)
  } else {
    draws[, , c("sigma[1]", "sigma[2]")] <- runif(40 * 2, 0.5, 1.5)
  }

  indicators <- list(
    latent1 = c("y1_1", "y1_2"),
    latent2 = c("y2_1", "y2_2")
  )
  stan_data <- structure(
    list(
      n_time = n_time,
      y = matrix(rnorm(n_time * 2), n_time, 2)
    ),
    time_values = 21:24,
    indicators = indicators,
    vars = names(indicators),
    margins = margins,
    method = "naive",
    J = 2
  )
  if (margins %in% c("exponential", "gamma")) {
    stan_data$skew_direction <- c(1, -1)
    attr(stan_data, "skew_direction") <- c(1, -1)
  }

  new_dcvar_sem_fit(
    fit = posterior::as_draws_array(draws),
    stan_data = stan_data,
    vars = names(indicators),
    J = 2,
    lambda = NULL,
    sigma_e = NULL,
    indicators = indicators,
    margins = margins,
    method = "naive",
    skew_direction = attr(stan_data, "skew_direction"),
    backend = "rstan",
    priors = list(),
    meta = list(chains = 1, iter_warmup = 10, iter_sampling = 40)
  )
}

test_that("prepare_sem_data builds row-mean scores for naive SEM", {
  df <- data.frame(
    time = 1:4,
    y1_1 = c(1, 2, 3, 4),
    y1_2 = c(3, 4, 5, 6),
    y2_1 = c(2, 4, 6, 8),
    y2_2 = c(4, 6, 8, 10)
  )
  indicators <- list(
    latent1 = c("y1_1", "y1_2"),
    latent2 = c("y2_1", "y2_2")
  )

  out <- prepare_sem_data(
    df,
    indicators = indicators,
    J = 2,
    lambda = NULL,
    sigma_e = NULL,
    method = "naive"
  )

  expect_equal(out$y[, 1], rowMeans(df[, indicators[[1]]]))
  expect_equal(out$y[, 2], rowMeans(df[, indicators[[2]]]))
  expect_equal(attr(out, "method"), "naive")
  expect_equal(.margin_stan_file("sem_naive", "normal"), "sem_naive_NG.stan")
  expect_equal(.margin_stan_file("sem_naive", "exponential"), "sem_naive_EG.stan")
})

test_that("SEM naive normal stub fit reports sigma and supports loo", {
  fit <- make_sem_naive_stub_fit("normal")

  co <- coef(fit)
  expect_named(co, c("mu", "Phi", "sigma", "rho"))

  vp <- var_params(fit)
  expect_named(vp, c("mu", "Phi", "sigma", "rho"))

  s <- summary(fit)
  expect_s3_class(s, "dcvar_sem_summary")
  expect_equal(s$method, "naive")

  out <- suppressWarnings(loo::loo(fit))
  expect_s3_class(out, "loo")
  expect_error(latent_states(fit), "naive SEM")
})

test_that("SEM naive exponential stub fit reports sigma_exp and supports loo", {
  fit <- make_sem_naive_stub_fit("exponential")

  co <- coef(fit)
  expect_named(co, c("mu", "Phi", "sigma_exp", "rho"))

  vp <- var_params(fit)
  expect_named(vp, c("mu", "Phi", "sigma_exp", "rho"))

  out <- suppressWarnings(loo::loo(fit))
  expect_s3_class(out, "loo")
})

test_that("SEM naive homogeneous skew_normal and gamma stubs report mixed-engine params", {
  skew_fit <- make_sem_naive_stub_fit("skew_normal")
  skew_co <- coef(skew_fit)
  skew_vp <- var_params(skew_fit)
  expect_named(skew_co, c("mu", "Phi", "omega", "delta", "rho"))
  expect_equal(names(skew_co$omega), c("omega[1]", "omega[2]"))
  expect_true(all(c("omega", "delta") %in% names(skew_vp)))

  gamma_fit <- make_sem_naive_stub_fit("gamma")
  gamma_co <- coef(gamma_fit)
  gamma_vp <- var_params(gamma_fit)
  expect_named(gamma_co, c("mu", "Phi", "sigma_gam", "shape_gam", "rho"))
  expect_equal(names(gamma_co$shape_gam), c("shape_gam[1]", "shape_gam[2]"))
  expect_true(all(c("sigma_gam", "shape_gam") %in% names(gamma_vp)))

  out <- suppressWarnings(loo::loo(gamma_fit))
  expect_s3_class(out, "loo")
})

test_that("SEM naive fits homogeneous skew_normal and gamma margins", {
  skip_if_no_rstan()
  skip_if_not_slow()

  for (margin in c("skew_normal", "gamma")) {
    J <- 2
    skew_direction <- if (identical(margin, "gamma")) c(1, 1) else NULL
    sim <- simulate_dcvar_sem(
      n_time = 80, J = J, lambda = rep(0.8, J), sigma_e = sqrt(0.2),
      margins = margin,
      skew_direction = skew_direction,
      skew_params = if (identical(margin, "skew_normal")) list(alpha = c(2, 2)) else list(shape = 2),
      rho = 0.5,
      seed = 700 + match(margin, c("skew_normal", "gamma"))
    )
    fit <- dcvar_sem(
      sim$data,
      indicators = list(latent1 = paste0("y1_", seq_len(J)), latent2 = paste0("y2_", seq_len(J))),
      J = J,
      method = "naive",
      margins = margin,
      skew_direction = skew_direction,
      chains = 2, iter_warmup = 250, iter_sampling = 250,
      adapt_delta = 0.999, max_treedepth = 13, refresh = 0, seed = 123
    )

    expect_equal(fit$method, "naive")
    expect_equal(fit$margins, margin)
    expect_equal(fit$stan_data$family, rep(unname(.family_codes[[margin]]), 2))
    expect_true(coef(fit)$rho > -0.2)
  }
})

# ============================================================================
# S3 Class: dcvar_fit
# ============================================================================

#' Internal: assemble the shared dcvar fit-object skeleton
#'
#' Every `dcvar_*_fit` constructor stores the same core slots (`fit`,
#' `stan_data`, `model`, `vars`, `backend`, `priors`, `meta`) plus a handful of
#' model-specific slots and a class vector. This helper centralises that
#' assembly so the constructors are thin shims; all slots are accessed by name,
#' so the model-specific extras (passed via `...`) may appear in any order.
#'
#' @param fit,stan_data,model,vars,backend,priors,meta Core fit slots.
#' @param class Character vector of S3 classes (most specific first).
#' @param ... Model-specific slots stored verbatim on the object.
#' @return An object with class `class`.
#' @noRd
.new_dcvar_model_fit <- function(fit, stan_data, model, vars, class,
                                 backend = "rstan", priors, meta, ...) {
  structure(
    c(
      list(fit = fit, stan_data = stan_data, model = model, vars = vars),
      list(...),
      list(backend = backend, priors = priors, meta = meta)
    ),
    class = class
  )
}

#' Construct a dcvar_fit object
#' @noRd
new_dcvar_fit <- function(fit, stan_data, vars, standardized,
                          margins = "normal", skew_direction = NULL,
                          backend = "rstan", priors, meta) {
  .new_dcvar_model_fit(
    fit = fit, stan_data = stan_data, model = "dcvar", vars = vars,
    class = c("dcvar_fit", "dcvar_model_fit"),
    backend = backend, priors = priors, meta = meta,
    standardized = standardized,
    margins = margins,
    skew_direction = skew_direction
  )
}


#' S3 methods for dcvar_fit objects
#'
#' @param x,object A `dcvar_fit` object.
#' @param ... Additional arguments (unused).
#'
#' @name dcvar_fit-methods
NULL

#' @describeIn dcvar_fit-methods Print a concise overview of the DC-VAR fit.
#' @return Invisibly returns `x`.
#' @export
print.dcvar_fit <- function(x, ...) {
  .print_fit_header(x, "DC-VAR Model Fit")
  cat(sprintf("n_time = %d, D = %d\n", x$stan_data$n_time, x$stan_data$D))
  .print_fit_footer(x)

  rho_df <- rho_trajectory(x)
  cat(sprintf("rho range: [%.3f, %.3f] (posterior mean)\n",
              min(rho_df$mean), max(rho_df$mean)))
  invisible(x)
}


#' @describeIn dcvar_fit-methods Produce a detailed summary including rho
#'   trajectory, VAR parameters, and diagnostics.
#' @param probs Numeric vector of quantile probabilities for the rho trajectory
#'   (default: `c(0.025, 0.5, 0.975)`).
#' @return A `dcvar_summary` object (a list).
#' @export
summary.dcvar_fit <- function(object, probs = c(0.025, 0.5, 0.975), ...) {
  rho_df <- rho_trajectory(object, probs = probs)
  vp <- var_params(object)
  diag <- dcvar_diagnostics(object)

  out <- list(
    model = "dcvar",
    n_time = object$stan_data$n_time,
    D = object$stan_data$D,
    rho_trajectory = rho_df,
    rho_summary = data.frame(
      statistic = c("start", "end", "delta", "min", "max", "mean"),
      value = c(
        rho_df$mean[1],
        rho_df$mean[nrow(rho_df)],
        rho_df$mean[nrow(rho_df)] - rho_df$mean[1],
        min(rho_df$mean),
        max(rho_df$mean),
        mean(rho_df$mean)
      )
    ),
    var_params = vp,
    diagnostics = diag
  )
  class(out) <- "dcvar_summary"
  out
}


#' Print a dcvar_summary object
#'
#' @param x A `dcvar_summary` object as returned by [summary.dcvar_fit()].
#' @param ... Additional arguments (unused).
#' @return Invisibly returns `x`.
#' @export
print.dcvar_summary <- function(x, ...) {
  cat("DC-VAR Model Summary\n")
  cat(strrep("=", 50), "\n")
  cat(sprintf("n_time = %d, D = %d\n\n", x$n_time, x$D))

  cat("Rho Trajectory:\n")
  print(x$rho_summary, row.names = FALSE)

  cat("\nVAR(1) Parameters:\n")
  cat("  mu:\n")
  print(x$var_params$mu[, c("variable", "mean", "q2.5", "q97.5")], row.names = FALSE)
  cat("\n  Phi:\n")
  print(x$var_params$Phi[, c("variable", "mean", "q2.5", "q97.5")], row.names = FALSE)
  .print_margin_params(x$var_params)

  if (!is.null(x$var_params$sigma_omega)) {
    cat("\n  sigma_omega:\n")
    print(x$var_params$sigma_omega[, c("variable", "mean", "q2.5", "q97.5")], row.names = FALSE)
  }

  cat("\nDiagnostics:\n")
  cat(sprintf("  Divergences: %d\n", x$diagnostics$n_divergent))
  cat(sprintf("  Max Rhat: %.3f\n", x$diagnostics$max_rhat))
  cat(sprintf("  Min ESS bulk: %.0f\n", x$diagnostics$min_ess_bulk))

  invisible(x)
}


#' @describeIn dcvar_fit-methods Extract posterior means of model coefficients.
#' @return A named list of posterior means: `mu`, `Phi`, margin-specific
#'   scale/shape parameters (e.g. `sigma_eps` for normal margins), and
#'   `sigma_omega`.
#' @export
coef.dcvar_fit <- function(object, ...) {
  summ <- .fit_summary(object$fit, backend = object$backend)
  result <- list(
    mu = .extract_required_coef(summ, "^mu\\[", "mu", "coef.dcvar_fit()"),
    Phi = .extract_required_coef(summ, "^Phi\\[", "Phi", "coef.dcvar_fit()")
  )
  # Margin-specific scale params before sigma_omega
  margins <- object$margins %||% "normal"
  result <- c(result, .extract_margin_coefs(summ, margins))
  result$sigma_omega <- .extract_required_coef(summ, "^sigma_omega$", "sigma_omega", "coef.dcvar_fit()")
  result
}


#' @describeIn dcvar_fit-methods Dispatch to a plot type: `"rho"`, `"phi"`,
#'   `"diagnostics"`, `"ppc"`, or `"pit"`.
#' @param type Character; one of `"rho"`, `"phi"`, `"diagnostics"`, `"ppc"`,
#'   or `"pit"`.
#' @return A ggplot object.
#' @export
plot.dcvar_fit <- function(x, type = c("rho", "phi", "diagnostics", "ppc", "pit"), ...) {
  type <- match.arg(type)
  switch(type,
    rho = plot_rho(x, ...),
    phi = plot_phi(x, ...),
    diagnostics = plot_diagnostics(x, ...),
    ppc = plot_ppc(x, ...),
    pit = plot_pit(x, ...)
  )
}

# ============================================================================
# LOO-CV and Model Comparison
# ============================================================================

#' Compute LOO-CV for a fitted model
#'
#' @param x A fitted model object.
#' @param ... Additional arguments passed to [loo::loo()].
#'
#' @return A `loo` object from the loo package.
#'
#' @details PSIS-LOO is available for Gaussian and Clayton single-level fits,
#'   covariate fits, multilevel fits, and naive SEM score fits. Indicator SEM
#'   fits are not supported because their stored `log_lik` conditions on
#'   latent innovations rather than being an observation-level predictive
#'   density. Multilevel `log_lik` values are conditional one-step-ahead
#'   densities given the unit-level random effects.
#'
#'   Note that the pointwise `log_lik` estimands differ across model families:
#'   HMM fits store the state-marginalized one-step-ahead predictive density,
#'   while dynamic (DC-VAR and covariate) fits condition on the smoothed
#'   latent `rho` trajectory, which is itself informed by the observation.
#'   Comparisons across these families can systematically favor the
#'   latent-conditioned models; see [dcvar_compare()].
#' @importFrom loo loo
#' @name loo.dcvar
NULL

.loo_dcvar <- function(x, ...) {
  log_lik <- .fit_draws(
    x$fit, "log_lik",
    format = "draws_array",
    backend = x$backend,
    required = .stan_output_group_pattern("log_lik"),
    required_type = "pattern",
    context = ".loo_dcvar()",
    output_type = "generated quantity"
  )
  r_eff <- loo::relative_eff(exp(log_lik))
  loo::loo(log_lik, r_eff = r_eff, ...)
}

.abort_unsupported_loo <- function(x, reason) {
  model_label <- switch(class(x)[[1]],
    dcvar_sem_fit = "SEM",
    x$model %||% class(x)[[1]]
  )

  cli_abort(c(
    "{.fun loo} is not supported for {.val {model_label}} fits.",
    "i" = reason
  ))
}

.is_supported_loo_fit <- function(x) {
  if (inherits(x, "dcvar_sem_fit")) {
    return(identical(x$method %||% "indicator", "naive"))
  }
  TRUE
}

#' @rdname loo.dcvar
#' @method loo dcvar_fit
#' @export
loo.dcvar_fit <- function(x, ...) .loo_dcvar(x, ...)

#' @rdname loo.dcvar
#' @method loo dcvar_covariate_fit
#' @export
loo.dcvar_covariate_fit <- function(x, ...) .loo_dcvar(x, ...)

#' @rdname loo.dcvar
#' @method loo dcvar_hmm_fit
#' @export
loo.dcvar_hmm_fit <- function(x, ...) .loo_dcvar(x, ...)

#' @rdname loo.dcvar
#' @method loo dcvar_constant_fit
#' @export
loo.dcvar_constant_fit <- function(x, ...) .loo_dcvar(x, ...)

#' @rdname loo.dcvar
#' @method loo dcvar_multilevel_fit
#' @export
loo.dcvar_multilevel_fit <- function(x, ...) .loo_dcvar(x, ...)

#' @rdname loo.dcvar
#' @method loo dcvar_sem_fit
#' @export
loo.dcvar_sem_fit <- function(x, ...) {
  if (.is_supported_loo_fit(x)) {
    return(.loo_dcvar(x, ...))
  }
  .abort_unsupported_loo(
    x,
    paste(
      "The stored {.code log_lik} conditions on latent innovations rather than using",
      "an observation-level predictive density integrated over the latent states."
    )
  )
}


#' Compare multiple fitted models using LOO-CV
#'
#' Convenience wrapper around [loo::loo_compare()] that accepts named
#' dcvar model fits.
#'
#' @param ... Named fitted model objects (e.g., `dcvar = fit1, hmm = fit2`).
#'
#' @return A `loo_compare` matrix.
#'
#' @details The pointwise `log_lik` estimands are not identical across model
#'   families: HMM fits marginalize the discrete states out of each one-step
#'   predictive density, while dynamic (DC-VAR and covariate) fits condition
#'   on the smoothed latent `rho` trajectory, which is informed by the
#'   observation itself. Comparing an HMM fit against a dynamic fit can
#'   therefore systematically favor the dynamic model; a warning is emitted
#'   for such comparisons and the resulting elpd differences should be
#'   interpreted with caution.
#'
#' @seealso [loo::loo_compare()] for details on the comparison method,
#'   [dcvar()], [dcvar_hmm()], [dcvar_constant()] for fitting models.
#' @export
#'
#' @examples
#' \donttest{
#' sim <- simulate_dcvar(
#'   n_time = 12,
#'   rho_trajectory = rho_decreasing(12),
#'   seed = 1
#' )
#' fit_dcvar <- dcvar(
#'   sim$Y_df,
#'   vars = c("y1", "y2"),
#'   chains = 1,
#'   iter_warmup = 10,
#'   iter_sampling = 10,
#'   refresh = 0,
#'   seed = 1
#' )
#' fit_constant <- dcvar_constant(
#'   sim$Y_df,
#'   vars = c("y1", "y2"),
#'   chains = 1,
#'   iter_warmup = 10,
#'   iter_sampling = 10,
#'   refresh = 0,
#'   seed = 1
#' )
#' dcvar_compare(dcvar = fit_dcvar, constant = fit_constant)
#' }
dcvar_compare <- function(...) {
  fits <- list(...)
  if (is.null(names(fits)) || any(names(fits) == "")) {
    unnamed <- which(names(fits) == "" | is.na(names(fits)))
    cli_abort(c(
      "All arguments must be named.",
      "i" = "Unnamed argument{?s} at position{?s}: {.val {unnamed}}.",
      "i" = "Example: {.code dcvar_compare(dcvar = fit1, hmm = fit2)}"
    ))
  }

  for (nm in names(fits)) {
    if (!inherits(fits[[nm]], "dcvar_model_fit")) {
      cli_abort("Argument {.val {nm}} is not a dcvar model fit object.")
    }
  }

  unsupported <- names(fits)[!vapply(fits, .is_supported_loo_fit, logical(1))]
  if (length(unsupported) > 0) {
    cli_abort(c(
      "{.fun dcvar_compare} does not support one or more supplied fits.",
      "i" = "Unsupported argument{?s}: {.val {unsupported}}.",
      "i" = paste(
        "Indicator SEM fits store {.code log_lik} conditional on latent innovations,",
        "which is not a valid pointwise predictive density for PSIS-LOO."
      )
    ))
  }

  has_hmm <- vapply(fits, inherits, logical(1), what = "dcvar_hmm_fit")
  has_latent_conditioned <- vapply(
    fits,
    function(f) inherits(f, "dcvar_fit") || inherits(f, "dcvar_covariate_fit"),
    logical(1)
  )
  if (any(has_hmm) && any(has_latent_conditioned)) {
    cli_warn(c(
      "Comparing HMM and dynamic (DC-VAR/covariate) fits mixes two {.code log_lik} estimands.",
      "i" = paste(
        "HMM fits store the state-marginalized one-step predictive density, while dynamic",
        "fits condition on the smoothed latent rho trajectory (informed by the observation itself)."
      ),
      "!" = "elpd differences can systematically favor the dynamic model; interpret with caution."
    ))
  }

  has_tv <- vapply(fits, inherits, logical(1), what = "dcvar_tv_fit")
  if (any(has_tv) && any(!has_tv)) {
    cli_warn(c(
      "Time-varying DC-VAR fits condition their {.code log_lik} on additional smoothed latent paths (Phi(t) and/or sigma(t)).",
      "!" = "elpd differences can systematically favor the more heavily latent-conditioned model; interpret comparisons across the model ladder with caution."
    ))
  }

  loo_list <- lapply(fits, loo)
  loo::loo_compare(loo_list)
}

test_that("loo() works for supported model types", {
  skip_if_no_rstan()

  loo_dc <- loo(get_dcvar_fit())
  loo_hmm <- loo(get_hmm_fit())
  loo_con <- loo(get_constant_fit())

  expect_s3_class(loo_dc, "loo")
  expect_s3_class(loo_hmm, "loo")
  expect_s3_class(loo_con, "loo")
})

test_that("loo() supports multilevel fits and rejects indicator SEM fits", {
  skip_if_no_rstan()

  # multilevel_copula_var.stan now stores per-observation log_lik like the EG
  # and mixed variants, so all multilevel fits are valid PSIS-LOO targets.
  loo_ml <- loo(get_multilevel_fit())
  expect_s3_class(loo_ml, "loo")

  expect_error(
    loo(get_sem_fit()),
    "not supported.*SEM|SEM.*not supported"
  )
})

test_that("dcvar_compare() works with named fits", {
  skip_if_no_rstan()

  result <- dcvar_compare(
    dcvar = get_dcvar_fit(),
    constant = get_constant_fit()
  )

  # loo_compare() returns a two-model comparison table. Whether that object
  # carries the base `matrix` class is loo-build-dependent (the dev build that
  # some CI runners resolve from r-universe does not always), so coerce before
  # asserting the structure rather than relying on is.matrix().
  comparison <- as.matrix(result)
  expect_equal(nrow(comparison), 2L)
  expect_true("elpd_diff" %in% colnames(comparison))
})

test_that("dcvar_compare() rejects unnamed arguments", {
  skip_if_no_rstan()

  expect_error(
    dcvar_compare(get_dcvar_fit(), get_constant_fit()),
    "must be named"
  )
})

test_that("dcvar_compare() rejects non-fit objects", {
  skip_if_no_rstan()

  expect_error(
    dcvar_compare(dcvar = get_dcvar_fit(), bad = "not a fit"),
    "not a dcvar model fit"
  )
})

test_that("dcvar_compare() rejects unsupported LOO targets", {
  skip_if_no_rstan()

  expect_error(
    dcvar_compare(dcvar = get_dcvar_fit(), sem = get_sem_fit()),
    "does not support one or more supplied fits"
  )
})

test_that("dcvar_compare() warns when mixing HMM and latent-conditioned fits", {
  skip_if_no_rstan()

  expect_warning(
    dcvar_compare(dcvar = get_dcvar_fit(), hmm = get_hmm_fit()),
    "mixes two"
  )
})

test_that("loo() fails clearly when custom Stan output omits log_lik", {
  fit <- structure(
    list(
      fit = posterior::as_draws_array(
        array(
          1:8,
          dim = c(4L, 1L, 2L),
          dimnames = list(NULL, NULL, c("mu[1]", "mu[2]"))
        )
      ),
      stan_data = list(n_time = 5, D = 2),
      model = "dcvar",
      vars = c("y1", "y2"),
      standardized = TRUE,
      margins = "normal",
      backend = "rstan",
      priors = list(),
      meta = list()
    ),
    class = c("dcvar_fit", "dcvar_model_fit")
  )

  expect_error(
    loo(fit),
    "Custom Stan files must preserve the expected parameter and generated-quantity names"
  )
})

#' Prepare data for the covariate DC-VAR model
#'
#' Transforms a data frame into a list suitable for the Gaussian covariate
#' DC-VAR Stan models. Outcome rows are sorted by `time_var`; rows with missing
#' outcomes are removed with the same adjacency rules used by [prepare_dcvar_data()],
#' and covariates are filtered in the same order so `X[t + 1, ]` aligns with the
#' outcome occasion of transition `Y[t, ] -> Y[t + 1, ]`.
#'
#' @inheritParams prepare_dcvar_data
#' @param covariates Character vector of covariate column names.
#' @param standardize_covariates Logical; whether to z-score covariates
#'   (default: `FALSE`). Binary phase indicators should usually be left on
#'   their original scale.
#' @param prior_beta_sd Prior SD for the covariate effects:
#'   `beta ~ normal(0, prior_beta_sd)`.
#' @param prior_rho_init_sd Prior SD for the dependence intercept `beta_0` on
#'   the Fisher-z scale (the covariate model has no separate initial-rho
#'   parameter; `beta_0` plays that role).
#' @param zero_init_eta Logical; if `TRUE` (default), fixes the first residual
#'   drift state at zero (`eta[1] = 0`). If `FALSE`, the first transition can
#'   receive an immediate residual random-walk shock.
#'
#' @return A named list suitable as Stan data input.
#' @export
prepare_dcvar_covariate_data <- function(data, vars, covariates,
                                         time_var = "time",
                                         standardize = TRUE,
                                         standardize_covariates = FALSE,
                                         allow_gaps = FALSE,
                                         prior_mu_sd = 2,
                                         prior_phi_sd = 0.5,
                                         prior_sigma_eps_rate = 1,
                                         prior_sigma_omega_rate = 0.1,
                                         prior_rho_init_sd = 1,
                                         prior_beta_sd = 1,
                                         zero_init_eta = TRUE) {
  if (!is.character(covariates) || length(covariates) == 0L || anyNA(covariates)) {
    cli_abort("{.arg covariates} must be a non-empty character vector.")
  }
  .prep_validate_unique_vars(covariates, "covariates")
  .prep_validate_scalar_logical(standardize_covariates, "standardize_covariates")
  .prep_validate_scalar_logical(zero_init_eta, "zero_init_eta")
  .prep_validate_positive_scalar(prior_mu_sd, "prior_mu_sd")
  .prep_validate_positive_scalar(prior_phi_sd, "prior_phi_sd")
  .prep_validate_positive_scalar(prior_sigma_eps_rate, "prior_sigma_eps_rate")
  .prep_validate_positive_scalar(prior_sigma_omega_rate, "prior_sigma_omega_rate")
  .prep_validate_positive_scalar(prior_rho_init_sd, "prior_rho_init_sd")
  .prep_validate_positive_scalar(prior_beta_sd, "prior_beta_sd")

  missing_covariates <- setdiff(covariates, names(data))
  if (length(missing_covariates) > 0L) {
    cli_abort("Covariate column{?s} not found in data: {.val {missing_covariates}}")
  }

  prep <- .prepare_var_data(data, vars, time_var, standardize, allow_gaps)
  aligned_data <- prep$data

  X_df <- aligned_data[, covariates, drop = FALSE]
  for (v in covariates) {
    if (!is.numeric(X_df[[v]])) {
      cli_abort("Covariate {.val {v}} must be numeric, got {.cls {class(X_df[[v]])[1]}}.")
    }
  }

  X <- as.matrix(X_df)
  if (any(is.na(X) | is.nan(X) | is.infinite(X))) {
    cli_abort("{.arg covariates} must be complete finite numeric values after outcome filtering.")
  }

  X_means <- NULL
  X_sds <- NULL
  if (standardize_covariates) {
    X_means <- colMeans(X)
    X_sds <- apply(X, 2, stats::sd)
    if (any(X_sds < 1e-10)) {
      cli_abort("Cannot standardize covariates: one or more covariates have zero (or near-zero) variance.")
    }
    X <- scale(X)
  }

  stan_data <- list(
    n_time = prep$T_obs,
    D = prep$D,
    Y = prep$Y,
    sigma_mu_prior = prior_mu_sd,
    sigma_phi_prior = prior_phi_sd,
    sigma_eps_prior = prior_sigma_eps_rate,
    sigma_omega_prior = prior_sigma_omega_rate,
    rho_init_prior_sd = prior_rho_init_sd,
    P = ncol(X),
    X = X,
    sigma_beta_prior = prior_beta_sd,
    zero_init_eta = as.integer(zero_init_eta)
  )

  attr(stan_data, "vars") <- prep$vars
  attr(stan_data, "covariates") <- covariates
  attr(stan_data, "standardized") <- prep$standardized
  attr(stan_data, "standardized_covariates") <- standardize_covariates
  attr(stan_data, "time_values") <- prep$time_values
  attr(stan_data, "zero_init_eta") <- zero_init_eta
  if (prep$standardized) {
    attr(stan_data, "Y_means") <- prep$Y_means
    attr(stan_data, "Y_sds") <- prep$Y_sds
  }
  if (standardize_covariates) {
    attr(stan_data, "X_means") <- X_means
    attr(stan_data, "X_sds") <- X_sds
  }

  stan_data
}


#' Fit the covariate DC-VAR model
#'
#' Fits a Gaussian Dynamic Copula VAR(1) model in which contemporaneous
#' innovation dependence varies on the Fisher-z scale as a function of observed
#' covariates. With `drift = TRUE`, the model adds residual random-walk drift:
#' `zeta_i = beta_0 + x_{i+1}' beta + eta_i`. With `drift = FALSE`, the
#' dependence trajectory is explained entirely by the covariates.
#'
#' @inheritParams dcvar
#' @param covariates Character vector of covariate column names.
#' @param standardize_covariates Logical; whether to z-score covariates
#'   (default: `FALSE`).
#' @param drift Logical; if `TRUE` (default), include residual random-walk drift
#'   after the covariate effect. If `FALSE`, fit the no-drift sensitivity model.
#' @param zero_init_eta Logical; if `TRUE` (default), fixes `eta[1] = 0` in the
#'   residual-drift model.
#' @param prior_beta_sd Prior SD for covariate effects.
#' @param prior_rho_init_sd Prior SD for the dependence intercept `beta_0` on
#'   the Fisher-z scale (the covariate model has no separate initial-rho
#'   parameter; `beta_0` plays that role).
#'
#' @return A `dcvar_covariate_fit` object.
#'
#' @seealso [prepare_dcvar_covariate_data()], [covariate_effects()],
#'   [rho_trajectory()], and [dcvar()] for the covariate-free random-walk model.
#' @export
#'
#' @examples
#' \donttest{
#' sim <- simulate_dcvar(
#'   n_time = 12,
#'   rho_trajectory = rho_step(12),
#'   seed = 1
#' )
#' sim$Y_df$phase <- as.numeric(sim$Y_df$time > 6)
#' fit <- dcvar_covariate(
#'   sim$Y_df,
#'   vars = c("y1", "y2"),
#'   covariates = "phase",
#'   chains = 1,
#'   iter_warmup = 10,
#'   iter_sampling = 10,
#'   refresh = 0,
#'   seed = 1
#' )
#' covariate_effects(fit)
#'
#' fit_nodrift <- dcvar_covariate(
#'   sim$Y_df,
#'   vars = c("y1", "y2"),
#'   covariates = "phase",
#'   drift = FALSE,
#'   chains = 1,
#'   iter_warmup = 10,
#'   iter_sampling = 10,
#'   refresh = 0,
#'   seed = 1
#' )
#' rho_trajectory(fit_nodrift)
#' }
dcvar_covariate <- function(data, vars, covariates, time_var = "time",
                            standardize = TRUE,
                            standardize_covariates = FALSE,
                            drift = TRUE,
                            zero_init_eta = TRUE,
                            allow_gaps = FALSE,
                            prior_mu_sd = 2,
                            prior_phi_sd = 0.5,
                            prior_sigma_eps_rate = 1,
                            prior_sigma_omega_rate = 0.1,
                            prior_rho_init_sd = 1,
                            prior_beta_sd = 1,
                            chains = 4,
                            iter_warmup = 2000,
                            iter_sampling = 4000,
                            adapt_delta = 0.99,
                            max_treedepth = 12,
                            seed = NULL,
                            cores = NULL,
                            refresh = 500,
                            init = NULL,
                            stan_file = NULL,
                            backend = getOption("dcvar.backend", "auto"),
                            ...) {
  .prep_validate_scalar_logical(drift, "drift")
  backend <- .resolve_backend(backend)
  .validate_sampling_args(chains, iter_warmup, iter_sampling,
                          adapt_delta, max_treedepth)

  stan_data <- prepare_dcvar_covariate_data(
    data = data,
    vars = vars,
    covariates = covariates,
    time_var = time_var,
    standardize = standardize,
    standardize_covariates = standardize_covariates,
    allow_gaps = allow_gaps,
    prior_mu_sd = prior_mu_sd,
    prior_phi_sd = prior_phi_sd,
    prior_sigma_eps_rate = prior_sigma_eps_rate,
    prior_sigma_omega_rate = prior_sigma_omega_rate,
    prior_rho_init_sd = prior_rho_init_sd,
    prior_beta_sd = prior_beta_sd,
    zero_init_eta = zero_init_eta
  )

  model_type <- if (drift) "dcvar_covariate" else "dcvar_covariate_nodrift"
  drift_label <- if (drift) "with residual drift" else "no residual drift"
  cli_inform("Fitting covariate DC-VAR model ({drift_label}; n_time = {stan_data$n_time}, D = {stan_data$D}, P = {stan_data$P})...")

  # The drift covariate model is served by the unified dynamic engine
  # (dcvar_dynamic.stan); the no-drift model keeps its specialised legacy file.
  # The bundled/default engine takes the augmented data block; a user-supplied
  # legacy covariate file keeps the legacy covariate block.
  uses_engine <- drift && .uses_dynamic_stan_file(stan_file, margins = "normal")
  compile_stan_file <- if (uses_engine) dcvar_stan_path("dcvar_dynamic") else stan_file

  model <- .compile_model(model_type, margins = "normal", stan_file = compile_stan_file,
                          backend = backend)

  if (!drift) {
    # The no-drift model has no residual random walk, so the drift-specific
    # arguments are inert; warn when the user supplied non-default values.
    if (!isTRUE(all.equal(prior_sigma_omega_rate, 0.1))) {
      cli_warn(
        "{.arg prior_sigma_omega_rate} is ignored when {.code drift = FALSE}: the no-drift model has no residual random walk."
      )
    }
    if (!isTRUE(zero_init_eta)) {
      cli_warn(
        "{.arg zero_init_eta} is ignored when {.code drift = FALSE}: the no-drift model has no residual drift states."
      )
    }
    stan_data$sigma_omega_prior <- NULL
    stan_data$zero_init_eta <- NULL
  }

  if (uses_engine) {
    # All-normal, constant-scale config: the z-score copula fast path reproduces
    # the legacy covariate density term-by-term. Augment to the engine block.
    stan_data$copula_normal_fastpath <- 1L
    stan_data <- .as_dynamic_stan_data(stan_data)
  }

  if (is.null(init)) {
    D <- stan_data$D
    n_time_obs <- stan_data$n_time
    P <- stan_data$P
    init <- if (uses_engine) {
      # The engine samples z_rho_init (beta_0 is a transformed-parameter alias)
      # and carries the full mixed-margin union, so it needs the dynamic init.
      function() .init_dcvar_dynamic_params(
        D = D,
        T_obs = n_time_obs,
        margins = "normal",
        tv_phi = FALSE,
        tv_sigma = FALSE,
        P = P,
        zero_init_eta = zero_init_eta
      )
    } else {
      function() .init_dcvar_covariate_params(
        D = D,
        T_obs = n_time_obs,
        P = P,
        drift = drift,
        zero_init_eta = zero_init_eta
      )
    }
  }

  cores <- .normalize_cores(cores, chains)

  fit <- .sample_model(
    compiled_model = model,
    stan_data = stan_data,
    backend = backend,
    chains = chains,
    iter_warmup = iter_warmup,
    iter_sampling = iter_sampling,
    adapt_delta = adapt_delta,
    max_treedepth = max_treedepth,
    seed = seed,
    cores = cores,
    init = init,
    refresh = refresh,
    ...
  )

  out <- new_dcvar_covariate_fit(
    fit = fit,
    stan_data = stan_data,
    vars = vars,
    covariates = covariates,
    standardized = standardize,
    standardized_covariates = standardize_covariates,
    drift = drift,
    zero_init_eta = zero_init_eta,
    backend = backend,
    priors = c(
      list(
        mu_sd = prior_mu_sd,
        phi_sd = prior_phi_sd,
        sigma_eps_rate = prior_sigma_eps_rate
      ),
      # sigma_omega only exists in the drift model; recording its prior for a
      # no-drift fit would suggest it was used.
      if (drift) list(sigma_omega_rate = prior_sigma_omega_rate),
      list(
        rho_init_sd = prior_rho_init_sd,
        beta_sd = prior_beta_sd
      )
    ),
    meta = list(
      chains = chains,
      iter_warmup = iter_warmup,
      iter_sampling = iter_sampling,
      adapt_delta = adapt_delta,
      max_treedepth = max_treedepth,
      seed = seed
    ),
    dynamic_engine = uses_engine
  )

  .report_sampling_outcome(out$fit, "Covariate DC-VAR",
                           chains = chains, backend = backend, object = out)
  out
}

# ============================================================================
# Clinical Interpretation Helper
# ============================================================================

# Default thresholds for classifying correlation strength (Cohen's conventions
# adapted for copula correlations). These are used by interpret_rho_trajectory()
# and can be overridden via the strength_breaks parameter.
.default_strength_breaks <- c(strong = 0.7, moderate = 0.4, weak = 0.2)

# Default thresholds for classifying the magnitude of change in a trajectory.
.default_magnitude_breaks <- c(large = 0.5, moderate = 0.3, small = 0.1)

#' Interpret a rho trajectory in clinical terms
#'
#' Generates a human-readable interpretation of the estimated rho trajectory,
#' describing the overall trend, magnitude of change, and key features.
#'
#' @param object A fitted model object (`dcvar_fit`, `dcvar_covariate_fit`,
#'   `dcvar_hmm_fit`, `dcvar_constant_fit`, `dcvar_multilevel_fit`, or
#'   `dcvar_sem_fit`).
#' @param threshold Minimum absolute change in posterior-mean rho to be
#'   considered "meaningful" (default: 0.1).
#' @param strength_breaks Named numeric vector of thresholds for classifying
#'   correlation strength (default: `c(strong = 0.7, moderate = 0.4,
#'   weak = 0.2)`). Values above the highest threshold are "strong", etc.;
#'   values at or below the lowest threshold are classified as "negligible".
#' @param magnitude_breaks Named numeric vector of thresholds for classifying
#'   the magnitude of trajectory range (default: `c(large = 0.5, moderate = 0.3,
#'   small = 0.1)`). Values at or below the lowest threshold are classified as
#'   "negligible".
#' @param fluctuation_threshold Proportion of sign changes in first differences
#'   to flag "substantial fluctuation" (default: 0.3).
#' @param ... Additional arguments (unused).
#'
#' @return A character string with the interpretation (invisibly). The
#'   interpretation is also printed to the console.
#' @export
#'
#' @examples
#' \donttest{
#' sim <- simulate_dcvar(
#'   n_time = 12,
#'   rho_trajectory = rho_decreasing(12),
#'   seed = 1
#' )
#' fit <- dcvar(
#'   sim$Y_df,
#'   vars = c("y1", "y2"),
#'   chains = 1,
#'   iter_warmup = 10,
#'   iter_sampling = 10,
#'   refresh = 0,
#'   seed = 1
#' )
#' interpret_rho_trajectory(fit)
#' }
interpret_rho_trajectory <- function(object, threshold = 0.1,
                                     strength_breaks = .default_strength_breaks,
                                     magnitude_breaks = .default_magnitude_breaks,
                                     fluctuation_threshold = 0.3,
                                     ...) {
  .classify <- function(value, breaks) {
    # breaks must be sorted descending (e.g., c(strong = 0.7, moderate = 0.4, weak = 0.2))
    for (i in seq_along(breaks)) {
      if (value > breaks[i]) return(names(breaks)[i])
    }
    "negligible"
  }

  # Clayton fits have no Gaussian-copula rho trajectory; interpret the
  # dependence via Kendall's tau instead of calling rho_trajectory().
  if (object$model == "constant" && identical(object$copula %||% "gaussian", "clayton")) {
    tau_df <- dependence_summary(object)
    tau_val <- tau_df$mean[1]
    # tau lives on a compressed scale relative to rho (rho = sin(pi * tau / 2)
    # for a Gaussian copula), so classify the rho-equivalent value to keep the
    # strength labels comparable with the Gaussian branch.
    rho_equiv <- sin(pi * tau_val / 2)
    strength <- .classify(abs(rho_equiv), strength_breaks)
    direction <- if (tau_val > 0) "positive" else "negative"

    msg <- sprintf(
      "Constant Clayton copula model: The estimated Kendall's tau is %.3f (%s %s coupling).\nThis suggests a stable, time-invariant dependence structure between the variables.",
      tau_val, strength, direction
    )
    cat(msg, "\n")
    return(invisible(msg))
  }

  rho_df <- rho_trajectory(object)

  if (object$model == "constant") {
    rho_val <- rho_df$mean[1]
    strength <- .classify(abs(rho_val), strength_breaks)
    direction <- if (rho_val > 0) "positive" else "negative"

    msg <- sprintf(
      "Constant copula model: The estimated correlation is %.3f (%s %s coupling).\nThis suggests a stable, time-invariant dependence structure between the variables.",
      rho_val, strength, direction
    )
    cat(msg, "\n")
    return(invisible(msg))
  }

  rho_mean <- rho_df$mean
  n <- length(rho_mean)
  rho_start <- rho_mean[1]
  rho_end <- rho_mean[n]
  delta <- rho_end - rho_start
  rho_min <- min(rho_mean)
  rho_max <- max(rho_mean)
  rho_range <- rho_max - rho_min

  # Classify overall trend
  if (abs(delta) < threshold) {
    trend <- "relatively stable"
  } else if (delta < -threshold) {
    trend <- "decreasing (decoupling)"
  } else {
    trend <- "increasing (strengthening coupling)"
  }

  # Classify magnitude
  magnitude <- .classify(rho_range, magnitude_breaks)

  # Detect non-monotonicity
  diffs <- diff(rho_mean)
  n_sign_changes <- sum(diff(sign(diffs)) != 0)
  nonmono <- if (n_sign_changes > (n - 2) * fluctuation_threshold) {
    " The trajectory shows substantial fluctuation, suggesting non-monotonic dynamics."
  } else {
    ""
  }

  model_label <- switch(object$model,
    dcvar = "DC-VAR",
    hmm = "HMM Copula",
    multilevel = "Multilevel Copula VAR",
    sem = "SEM Copula VAR",
    object$model
  )

  msg <- sprintf(
    paste0(
      "%s model: The dependence trajectory is %s (rho: %.3f -> %.3f, delta = %.3f).\n",
      "Range: [%.3f, %.3f] (%s variation).%s"
    ),
    model_label, trend, rho_start, rho_end, delta,
    rho_min, rho_max, magnitude, nonmono
  )

  if (object$model == "hmm") {
    states <- hmm_states(object)
    state_rhos <- states$rho_state$mean
    msg <- paste0(msg, sprintf(
      "\nHMM states: %s (rho = %s).",
      paste(paste0("State ", seq_along(state_rhos)), collapse = ", "),
      paste(sprintf("%.3f", state_rhos), collapse = ", ")
    ))
  }

  cat(msg, "\n")
  invisible(msg)
}

test_that("interpret_rho_trajectory() works for dcvar", {
  skip_if_no_rstan()

  fit <- get_dcvar_fit()
  msg <- expect_output(interpret_rho_trajectory(fit), "DC-VAR")
  expect_type(msg, "character")
  expect_true(nchar(msg) > 0)
})

test_that("interpret_rho_trajectory() works for hmm", {
  skip_if_no_rstan()

  fit <- get_hmm_fit()
  msg <- expect_output(interpret_rho_trajectory(fit), "HMM")
  expect_type(msg, "character")
  expect_true(grepl("HMM states", msg))
})

test_that("interpret_rho_trajectory() works for constant", {
  skip_if_no_rstan()

  fit <- get_constant_fit()
  msg <- expect_output(interpret_rho_trajectory(fit), "Constant")
  expect_type(msg, "character")
  expect_true(grepl("time-invariant", msg))
})

test_that("interpret_rho_trajectory() respects threshold", {
  skip_if_no_rstan()

  fit <- get_dcvar_fit()
  msg1 <- capture.output(interpret_rho_trajectory(fit, threshold = 0.01))
  msg2 <- capture.output(interpret_rho_trajectory(fit, threshold = 10))
  # With very high threshold, everything should be "stable"
  expect_true(any(grepl("stable", msg2)))
})

#' Fit the HMM copula model
#'
#' Fits a Hidden Markov Model copula VAR(1) with K discrete states and
#' state-specific correlations. Uses ordered z_rho constraint to prevent
#' label switching and a sticky Dirichlet prior to encourage state persistence.
#'
#' @section State-specific parameters and per-state margins:
#' By default only the copula correlation switches by state. The `switch`
#' argument turns a full Markov-switching VAR on: the intercepts, the VAR
#' coefficients, and the residual scales can each be made state-specific. The
#' marginal **family** can also switch by state by passing a length-`K`
#' `margins` list. Because the states are identified by an ordered correlation
#' (`z_rho`), `rho` is the mandatory anchor and a per-state `margins` list is
#' consumed in increasing-correlation order (`margins[[1]]` is the lowest-rho
#' state). Richly-parameterized configurations on a single short bivariate series
#' are weakly identified; fit a ladder (`switch = "rho"` ->
#' `switch = c("rho", "ar")` -> full) and widen `prior_phi_sd`. See
#' [hmm_state_params()] for the per-state parameter view.
#'
#' @inheritParams dcvar
#' @param margins Marginal distribution specification. Either a single string
#'   applied to both variables, a length-2 character vector giving a per-variable
#'   (mixed) margin, e.g. `c("normal", "exponential")`, or -- for a state-specific
#'   family -- a length-`K` list of such specifications, one per hidden state,
#'   e.g. `list(c("normal", "normal"), c("exponential", "gamma"))`. Each entry is
#'   one of `"normal"` (default), `"exponential"`, `"skew_normal"`, or `"gamma"`.
#'   A per-state list is consumed in increasing-`rho` order.
#' @param skew_direction Skew direction for asymmetric margins: each element `1`
#'   (right-skewed) or `-1` (left-skewed). A length-2 vector (recycled across
#'   states) or, for per-state margins, a length-`K` list of length-2 vectors.
#'   Required whenever any (state, dimension) uses an `"exponential"` or
#'   `"gamma"` margin; only those entries consult it.
#' @param switch Which parameters become state-specific. `"rho"` (default) keeps
#'   only the copula correlation switching (the classic HMM). A character vector
#'   drawn from `c("rho", "mu", "phi", "ar", "cross", "sigma")` (with `"ar"` /
#'   `"cross"` / coefficient names like `"phi11"` selecting a subset of the VAR
#'   coefficients), or `TRUE` for everything. `rho` must be included whenever any
#'   other component switches (it is the label-switching anchor).
#' @param K Number of hidden states (default: 2).
#' @param prior_kappa Sticky Dirichlet self-transition concentration (default: 10).
#' @param prior_alpha_off Sticky Dirichlet off-diagonal concentration (default: 1).
#' @param prior_z_rho_sd Prior SD for state-specific z_rho (default: 1.0).
#'
#' @param backend Character: `"auto"` (default, uses rstan), `"rstan"`, or
#'   `"cmdstanr"`. Can also be set globally via
#'   `options(dcvar.backend = "cmdstanr")`.
#' @param ... Additional backend-specific sampling arguments.
#'
#' @return A `dcvar_hmm_fit` object.
#'
#' @seealso [dcvar()] for the smooth time-varying model,
#'   [dcvar_constant()] for the time-invariant baseline,
#'   [hmm_states()] for state extraction, [plot_hmm_states()] for visualisation,
#'   [dcvar_compare()] for LOO-CV model comparison.
#' @export
#'
#' @examples
#' \donttest{
#' sim <- simulate_dcvar(
#'   n_time = 12,
#'   rho_trajectory = rho_step(12),
#'   seed = 1
#' )
#' fit <- dcvar_hmm(
#'   sim$Y_df,
#'   vars = c("y1", "y2"),
#'   K = 2,
#'   chains = 1,
#'   iter_warmup = 10,
#'   iter_sampling = 10,
#'   refresh = 0,
#'   seed = 1
#' )
#' print(fit)
#' hmm_states(fit)
#' }
dcvar_hmm <- function(data, vars, K = 2,
                      time_var = "time",
                      standardize = TRUE,
                      margins = "normal",
                      skew_direction = NULL,
                      switch = "rho",
                      allow_gaps = FALSE,
                      prior_mu_sd = 2,
                      prior_phi_sd = 0.5,
                      prior_sigma_eps_rate = 1,
                      prior_kappa = 10,
                      prior_alpha_off = 1,
                      prior_z_rho_sd = 1.0,
                      chains = 4,
                      iter_warmup = 2000,
                      iter_sampling = 4000,
                      adapt_delta = 0.99,
                      max_treedepth = 12,
                      seed = NULL,
                      cores = NULL,
                      refresh = 500,
                      init = NULL,
                      stan_file = NULL,
                      backend = getOption("dcvar.backend", "auto"),
                      ...) {
  backend <- .resolve_backend(backend)
  .validate_sampling_args(chains, iter_warmup, iter_sampling,
                          adapt_delta, max_treedepth)

  if (!is.numeric(K) || length(K) != 1 || K < 2 || K != as.integer(K)) {
    cli_abort("{.arg K} must be an integer >= 2, got {.val {K}}.")
  }
  K <- as.integer(K)

  # Resolve the switching spec and the (global or per-state) margin configuration.
  sw <- .resolve_switch_spec(switch)
  cfg <- .hmm_margin_config(margins, skew_direction, K)

  # Force state-specific margin parameters when residuals are state-specific and
  # any exp/gamma family is present (the feasibility-bounded scale needs one
  # bijection per state), when families differ per state, or on explicit request.
  any_exp_gamma <- any(cfg$family %in% c(2L, 4L))
  if (cfg$per_state_differ ||
      ((sw$mu == 1L || any(sw$phi_mask > 0L)) && any_exp_gamma)) {
    sw$margins <- 1L
  }

  needs_engine <- cfg$per_state || sw$mu == 1L || any(sw$phi_mask > 0L) || sw$margins == 1L
  uses_engine <- .uses_hmm_switching_stan_file(stan_file) ||
    (is.null(stan_file) && needs_engine)

  if (needs_engine && !is.null(stan_file) && !uses_engine) {
    cli_abort(c(
      "{.arg switch} / per-state {.arg margins} require the bundled switching engine.",
      "i" = "Omit {.arg stan_file}, or pass {.code stan_file = dcvar_stan_path(\"hmm_switching\")}."
    ))
  }

  # The engine overrides family/skew via .as_hmm_switching_stan_data(), so a
  # normal placeholder suffices for prepare_hmm_data(); the legacy path uses the
  # real global margins.
  base_margins <- if (uses_engine) "normal" else cfg$margins_global
  base_skew <- if (uses_engine) NULL else cfg$skew_global

  stan_data <- prepare_hmm_data(
    data, vars, K, time_var, standardize, base_margins, base_skew,
    prior_mu_sd, prior_phi_sd, prior_sigma_eps_rate,
    prior_kappa, prior_alpha_off, prior_z_rho_sd,
    allow_gaps = allow_gaps
  )

  if (uses_engine) {
    comps <- c(
      "rho",
      if (sw$mu == 1L) "mu",
      if (any(sw$phi_mask > 0L)) {
        if (all(sw$phi_mask == 1L)) "Phi" else paste0("Phi:", paste(names(sw$phi_mask)[sw$phi_mask == 1L], collapse = "/"))
      },
      if (sw$margins == 1L) "margins"
    )
    cli_inform("Fitting Markov-switching HMM [state-specific: {paste(comps, collapse = ', ')}] (n_time = {stan_data$n_time}, D = {stan_data$D}, K = {K})...")
  } else {
    margins_label <- if (all(cfg$margins_char == "normal")) {
      ""
    } else {
      paste0(" [", paste(cfg$margins_char[1, ], collapse = ", "), "]")
    }
    cli_inform("Fitting HMM copula model{margins_label} (n_time = {stan_data$n_time}, D = {stan_data$D}, K = {K})...")
  }

  # Compile model (the engine compiles the bundled hmm_switching.stan)
  model_type <- if (uses_engine) "hmm_switching" else "hmm"
  compile_stan_file <- if (uses_engine) dcvar_stan_path("hmm_switching") else stan_file
  model <- .compile_model(model_type, margins = base_margins,
                          stan_file = compile_stan_file, backend = backend)

  # Augment the prepared data into the engine's superset block.
  if (uses_engine) {
    stan_data <- .as_hmm_switching_stan_data(stan_data, sw, cfg$family, cfg$skew,
                                             prior_phi_dev_sd = prior_phi_sd)
  }

  # Weak-identification warnings for richly-parameterized switching configs.
  if (uses_engine) {
    if (sw$mu == 1L && all(sw$phi_mask == 1L) && stan_data$n_time < 150) {
      cli_warn(c(
        "All-switching (mu + Phi + rho) on a short series (n_time = {stan_data$n_time}) is weakly identified.",
        "i" = "Consider a switching ladder ({.code \"rho\"} -> {.code c(\"rho\", \"ar\")} -> full) and widen {.arg prior_phi_sd}."
      ))
    }
    if (K >= 4L) {
      cli_warn("Recovering {K} regimes from a single bivariate series is hard; prefer K = 2-3 unless the design is strong.")
    }
    if (cfg$per_state_differ) {
      cli_warn(c(
        "Per-state {.arg margins} or {.arg skew_direction} differ across states.",
        "i" = "States are ordered by increasing correlation, so the first state specification is the lowest-rho state."
      ))
    }
  }

  # Default init
  if (is.null(init)) {
    D <- stan_data$D
    init <- if (uses_engine) {
      function() .init_hmm_switching_params(D, K, sw)
    } else {
      function() .init_hmm_params(D, K, base_margins)
    }
  }

  cores <- .normalize_cores(cores, chains)

  # Fit
  fit <- .sample_model(
    compiled_model = model,
    stan_data = stan_data,
    backend = backend,
    chains = chains,
    iter_warmup = iter_warmup,
    iter_sampling = iter_sampling,
    adapt_delta = adapt_delta,
    max_treedepth = max_treedepth,
    seed = seed,
    cores = cores,
    init = init,
    refresh = refresh,
    ...
  )

  # Wrap in S3 class
  out <- new_dcvar_hmm_fit(
    fit = fit,
    stan_data = stan_data,
    K = K,
    vars = vars,
    standardized = standardize,
    margins = cfg$margins_global %||% "normal",
    skew_direction = skew_direction,
    switch = sw,
    switching = uses_engine,
    margins_matrix = if (uses_engine) cfg$margins_char else NULL,
    backend = backend,
    priors = list(
      mu_sd = prior_mu_sd,
      phi_sd = prior_phi_sd,
      sigma_eps_rate = prior_sigma_eps_rate,
      kappa = prior_kappa,
      alpha_off = prior_alpha_off,
      z_rho_sd = prior_z_rho_sd
    ),
    meta = list(
      chains = chains,
      iter_warmup = iter_warmup,
      iter_sampling = iter_sampling,
      adapt_delta = adapt_delta,
      max_treedepth = max_treedepth,
      seed = seed
    )
  )

  .report_sampling_outcome(out$fit, if (uses_engine) "Markov-switching HMM" else "HMM copula",
                           chains = chains, backend = backend, object = out)
  out
}

test_that("dcvar_hmm() returns a dcvar_hmm_fit object", {
  skip_if_no_rstan()

  fit <- get_hmm_fit()
  expect_s3_class(fit, "dcvar_hmm_fit")
  expect_s3_class(fit, "dcvar_model_fit")
  expect_equal(fit$model, "hmm")
  expect_equal(fit$K, 2)
})

test_that("print.dcvar_hmm_fit runs without error", {
  skip_if_no_rstan()

  fit <- get_hmm_fit()
  out <- capture.output(print(fit))
  expect_true(any(grepl("HMM", out)))
  expect_true(any(grepl("State", out)))
})

test_that("summary.dcvar_hmm_fit returns expected class", {
  skip_if_no_rstan()

  fit <- get_hmm_fit()
  s <- summary(fit)
  expect_s3_class(s, "dcvar_hmm_summary")
  out <- capture.output(print(s))
  expect_true(any(grepl("HMM", out)))
})

test_that("coef.dcvar_hmm_fit returns named list", {
  skip_if_no_rstan()

  fit <- get_hmm_fit()
  co <- coef(fit)
  expect_type(co, "list")
  expect_named(co, c("mu", "Phi", "sigma_eps", "z_rho", "rho_state"))
})

test_that("plot.dcvar_hmm_fit dispatches without error", {
  skip_if_no_rstan()

  fit <- get_hmm_fit()
  expect_s3_class(plot(fit, type = "rho"), "ggplot")
  expect_s3_class(plot(fit, type = "phi"), "ggplot")
})

test_that("dcvar_hmm() fits exponential margins", {
  skip_if_no_rstan()

  fit <- get_hmm_exponential_fit()
  expect_s3_class(fit, "dcvar_hmm_fit")
  expect_equal(fit$margins, "exponential")
  expect_equal(fit$K, 2)
})

test_that("dcvar_hmm() fits gamma margins", {
  skip_if_no_rstan()

  fit <- get_hmm_gamma_fit()
  expect_s3_class(fit, "dcvar_hmm_fit")
  expect_equal(fit$margins, "gamma")
  expect_equal(fit$K, 2)
})

test_that("dcvar_hmm() fits skew-normal margins", {
  skip_if_no_rstan()
  skip_if_not_installed("sn")

  fit <- get_hmm_skew_normal_fit()
  expect_s3_class(fit, "dcvar_hmm_fit")
  expect_equal(fit$margins, "skew_normal")
  expect_equal(fit$K, 2)
})

test_that("hmm fit cache emits only known diagnostic warnings", {
  skip_if_no_rstan()

  expect_known_fit_warnings(get_hmm_fit_warnings(), "hmm")
  expect_known_fit_warnings(get_hmm_exponential_fit_warnings(), "hmm exponential")
  expect_known_fit_warnings(get_hmm_gamma_fit_warnings(), "hmm gamma")
  expect_known_fit_warnings(get_hmm_skew_normal_fit_warnings(), "hmm skew-normal")
})

#' Fit the DC-VAR model
#'
#' Fits a Dynamic Copula VAR(1) model with time-varying correlation following
#' a random walk on the Fisher-z scale. Uses non-centered parameterisation
#' for efficient HMC sampling.
#'
#' Beyond the always time-varying correlation, the VAR coefficients and the
#' residual scales can optionally vary over time as well (`tv_phi`,
#' `tv_sigma`): each enabled component evolves as a non-centered random walk
#' around its constant baseline, with innovation-SD priors that shrink toward
#' the constant-parameter model. With both flags off (the default) the fit is
#' exactly the classic DC-VAR. With any flag on, the fit uses a generic
#' time-varying Stan model and returns a `dcvar_tv_fit` object (a subclass of
#' `dcvar_fit`), with trajectory extractors [phi_trajectory()] and
#' [sigma_trajectory()].
#'
#' @section Time-varying scales and shifted margins:
#' `tv_sigma = TRUE` gives normal and skew-normal dimensions a multiplicative
#' log-scale random walk. Exponential and gamma dimensions use a **soft-barrier**
#' construction: the shifted variate `x = scale + skew * eps` (which has a hard
#' support boundary at 0) is replaced by `x = softplus_k(m_t + skew * eps)`
#' with a time-varying scale `m_t`. This keeps `x` positive, matches the exact
#' shifted margin in the interior, and rounds the boundary smoothly so the
#' scale can vary freely. `tv_sigma_k` controls the sharpness; larger values
#' track the exact exponential/gamma more closely at the cost of a stiffer
#' posterior geometry, smaller values are numerically gentler but introduce a
#' small residual-mean bias in the lower tail.
#'
#' @section Recommended workflow:
#' For typical series lengths (T of 100--300), fit the model ladder
#' `dcvar()` (constant Phi/sigma) -> `dcvar(tv_phi = TRUE)` ->
#' `dcvar(tv_phi = TRUE, tv_sigma = TRUE)` and compare with
#' [dcvar_compare()]. Up to seven latent random walks are estimated from a
#' single bivariate series; the tight innovation-SD priors are what keeps
#' this identifiable, so widen them deliberately.
#'
#' @param data A data frame with time series observations.
#' @param vars Character vector of two variable names to model.
#' @param time_var Name of the time column (default: `"time"`).
#' @param standardize Logical; whether to z-score variables (default: `TRUE`).
#' @param margins Marginal distribution specification. Either a single string
#'   applied to both variables, or a length-2 character vector giving a
#'   per-variable (mixed) margin, e.g. `c("normal", "exponential")`. Each entry
#'   is one of `"normal"` (default), `"exponential"`, `"skew_normal"`, or
#'   `"gamma"`. When the two entries differ the fit uses a generic
#'   mixed-margins Stan model under the Gaussian copula; identical entries reuse
#'   the specialised single-family model.
#' @param skew_direction Integer vector of length 2 indicating skew direction
#'   for asymmetric margins. Each element must be `1` (right-skewed) or `-1`
#'   (left-skewed). Required whenever any dimension uses an `"exponential"` or
#'   `"gamma"` margin; only those dimensions consult it.
#' @param allow_gaps Logical; if `FALSE` (default), interior missing values
#'   cause an error because they break VAR(1) time series adjacency. Set to
#'   `TRUE` to allow fitting with a warning instead.
#' @param prior_mu_sd Prior SD for intercepts: `mu ~ normal(0, prior_mu_sd)`.
#' @param prior_phi_sd Prior SD for VAR coefficients: `Phi ~ normal(0, prior_phi_sd)`.
#' @param prior_sigma_eps_rate Prior mean for innovation SDs (see
#'   [prepare_dcvar_data()]).
#' @param prior_sigma_omega_rate Prior mean for rho process SD (see
#'   [prepare_dcvar_data()]).
#' @param prior_rho_init_sd Prior SD for initial rho on Fisher-z scale.
#' @param tv_phi Selects which VAR(1) coefficients evolve as independent
#'   random walks around the constant baseline `Phi`. Either a logical
#'   (`TRUE` = all four, `FALSE` = none, the default) or a character selector:
#'   `"ar"` (the autoregressive effects phi11, phi22 -- e.g. for modelling
#'   changing emotional inertia / critical slowing down), `"cross"` (the
#'   cross-lagged effects phi12, phi21), or specific names from
#'   `c("phi11", "phi12", "phi21", "phi22")`.
#' @param tv_sigma Logical; if `TRUE`, the residual scales evolve as log-scale
#'   random walks around their constant baselines (default: `FALSE`). Applies
#'   to all margin families; see the section on shifted margins for how
#'   exponential and gamma dimensions are handled.
#' @param prior_tau_phi_rate Prior mean for the Phi random-walk innovation
#'   SDs (`tau_phi ~ exponential(1/prior_tau_phi_rate)`; default `0.025`).
#'   Used only when `tv_phi = TRUE`.
#' @param prior_tau_sigma_rate Prior mean for the log-scale random-walk
#'   innovation SDs (`tau_sigma ~ exponential(1/prior_tau_sigma_rate)`;
#'   default `0.05`). Used only when `tv_sigma = TRUE`.
#' @param tv_sigma_k Soft-barrier sharpness for time-varying exponential/gamma
#'   scales (default `8`). Larger values approximate the exact shifted margin
#'   more closely but stiffen the geometry. Used only when `tv_sigma = TRUE`
#'   and an exponential or gamma margin is present.
#' @param chains Number of MCMC chains (default: 4).
#' @param iter_warmup Warmup iterations per chain (default: 2000).
#' @param iter_sampling Sampling iterations per chain (default: 4000).
#' @param adapt_delta Target acceptance rate (default: 0.99). The DC-VAR model
#'   uses a lower default than `dcvar_constant()` (0.999) because the
#'   non-centered parameterisation already handles posterior geometry well.
#' @param max_treedepth Maximum tree depth (default: 12).
#' @param seed Random seed. When supplied, it seeds both the Stan sampler and
#'   the default per-chain initial values, so repeated fits are reproducible.
#' @param cores Number of parallel chains. `NULL` uses all available cores.
#' @param refresh How often to print progress (default: 500). Set to 0 for
#'   silent operation.
#' @param init Custom init function or `NULL` for smart defaults.
#' @param stan_file Path to a custom Stan file, or `NULL` to use the bundled
#'   model.
#' @param backend Character: `"auto"` (default, uses rstan), `"rstan"`, or
#'   `"cmdstanr"`. Can also be set globally via
#'   `options(dcvar.backend = "cmdstanr")`.
#' @param ... Additional backend-specific sampling arguments.
#'
#' @return A `dcvar_fit` object.
#'
#' @seealso [dcvar_constant()] for the time-invariant baseline,
#'   [dcvar_hmm()] for the regime-switching model,
#'   [dcvar_compare()] for LOO-CV model comparison,
#'   [rho_trajectory()] and [plot_rho()] for inspecting results.
#' @export
#'
#' @examples
#' \donttest{
#' sim <- simulate_dcvar(
#'   n_time = 12,
#'   rho_trajectory = rho_decreasing(12),
#'   seed = 1
#' )
#' fit <- dcvar(
#'   sim$Y_df,
#'   vars = c("y1", "y2"),
#'   chains = 1,
#'   iter_warmup = 10,
#'   iter_sampling = 10,
#'   refresh = 0,
#'   seed = 1
#' )
#' print(fit)
#' summary(fit)
#' plot(fit)
#' }
dcvar <- function(data, vars, time_var = "time",
                  standardize = TRUE,
                  margins = "normal",
                  skew_direction = NULL,
                  allow_gaps = FALSE,
                  prior_mu_sd = 2,
                  prior_phi_sd = 0.5,
                  prior_sigma_eps_rate = 1,
                  prior_sigma_omega_rate = 0.1,
                  prior_rho_init_sd = 1,
                  tv_phi = FALSE,
                  tv_sigma = FALSE,
                  prior_tau_phi_rate = 0.025,
                  prior_tau_sigma_rate = 0.05,
                  tv_sigma_k = 8,
                  chains = 4,
                  iter_warmup = 2000,
                  iter_sampling = 4000,
                  adapt_delta = 0.99,
                  max_treedepth = 12,
                  seed = NULL,
                  cores = NULL,
                  refresh = 500,
                  init = NULL,
                  stan_file = NULL,
                  backend = getOption("dcvar.backend", "auto"),
                  ...) {
  backend <- .resolve_backend(backend)
  .validate_sampling_args(chains, iter_warmup, iter_sampling,
                          adapt_delta, max_treedepth)
  margins <- .normalize_margins_spec(margins)
  .validate_margins(margins, skew_direction)
  phi_mask <- .resolve_phi_tv_mask(tv_phi)
  any_phi <- sum(phi_mask) > 0L
  .prep_validate_scalar_logical(tv_sigma, "tv_sigma")

  is_tv <- any_phi || tv_sigma
  model_type <- if (is_tv) "dcvar_tv" else "dcvar"
  margins_vec <- rep(margins, length.out = 2L)

  if (!any_phi && !isTRUE(all.equal(prior_tau_phi_rate, 0.025))) {
    cli_warn("{.arg prior_tau_phi_rate} is ignored when no VAR coefficient is time-varying.")
  }
  if (!tv_sigma && !isTRUE(all.equal(prior_tau_sigma_rate, 0.05))) {
    cli_warn("{.arg prior_tau_sigma_rate} is ignored when {.code tv_sigma = FALSE}.")
  }
  if (tv_sigma) {
    .prep_validate_positive_scalar(tv_sigma_k, "tv_sigma_k")
  }

  # Prepare data
  stan_data <- prepare_dcvar_data(
    data, vars, time_var, standardize, margins, skew_direction,
    prior_mu_sd, prior_phi_sd, prior_sigma_eps_rate,
    prior_sigma_omega_rate, prior_rho_init_sd,
    tv_phi = tv_phi, tv_sigma = tv_sigma,
    prior_tau_phi_rate = prior_tau_phi_rate,
    prior_tau_sigma_rate = prior_tau_sigma_rate,
    tv_sigma_k = tv_sigma_k,
    allow_gaps = allow_gaps
  )

  margins_label <- if (all(margins == "normal")) {
    ""
  } else {
    paste0(" [", paste(margins, collapse = ", "), "]")
  }
  if (is_tv) {
    phi_label <- if (any_phi) {
      if (all(phi_mask == 1L)) "Phi(t)" else paste0("Phi(t):", paste(names(phi_mask)[phi_mask == 1L], collapse = "/"))
    }
    components <- c("rho(t)", phi_label, if (tv_sigma) "sigma(t)")
    cli_inform("Fitting TV DC-VAR model{margins_label} [{paste(components, collapse = ', ')}] (n_time = {stan_data$n_time}, D = {stan_data$D})...")
  } else {
    cli_inform("Fitting DC-VAR model{margins_label} (n_time = {stan_data$n_time}, D = {stan_data$D})...")
  }

  uses_engine <- is_tv && .uses_dynamic_stan_file(stan_file, margins = margins)
  compile_stan_file <- if (uses_engine) {
    dcvar_stan_path("dcvar_dynamic", margins = margins)
  } else {
    stan_file
  }

  # Compile model
  model <- .compile_model(model_type, margins = margins, stan_file = compile_stan_file,
                          backend = backend)

  # The time-varying path is served by the unified dynamic engine
  # (dcvar_dynamic.stan), whose data block is a superset of the TV output of
  # prepare_dcvar_data(). Augment the prepared data with the engine's covariate
  # predictor / drift / fast-path fields just before sampling when the bundled
  # engine is selected. A user-supplied legacy TV Stan file keeps the legacy
  # TV block.
  if (uses_engine) {
    stan_data <- .as_dynamic_stan_data(stan_data)
  }

  # Default init
  if (is.null(init)) {
    D <- stan_data$D
    n_time_obs <- stan_data$n_time
    init <- if (is_tv) {
      # The TV path runs on the unified dynamic engine, so use its init (P = 0
      # omits beta, zero_init_eta = FALSE matches the TV omega_raw length). This
      # is also valid for a custom legacy dcvar_tv_mixed.stan file, whose
      # parameter block is the same minus the (omitted) covariate effects.
      function() .init_dcvar_dynamic_params(D, n_time_obs, margins,
                                            tv_phi = phi_mask, tv_sigma = tv_sigma,
                                            P = 0L, zero_init_eta = FALSE)
    } else {
      function() .init_dcvar_params(D, n_time_obs, margins)
    }
  }

  cores <- .normalize_cores(cores, chains)

  # Fit
  fit <- .sample_model(
    compiled_model = model,
    stan_data = stan_data,
    backend = backend,
    chains = chains,
    iter_warmup = iter_warmup,
    iter_sampling = iter_sampling,
    adapt_delta = adapt_delta,
    max_treedepth = max_treedepth,
    seed = seed,
    cores = cores,
    init = init,
    refresh = refresh,
    ...
  )

  priors <- c(
    list(
      mu_sd = prior_mu_sd,
      phi_sd = prior_phi_sd,
      sigma_eps_rate = prior_sigma_eps_rate,
      sigma_omega_rate = prior_sigma_omega_rate,
      rho_init_sd = prior_rho_init_sd
    ),
    # tau priors only act on enabled components; recording them otherwise
    # would suggest they were used.
    if (any_phi) list(tau_phi_rate = prior_tau_phi_rate),
    if (tv_sigma) list(tau_sigma_rate = prior_tau_sigma_rate)
  )
  meta <- list(
    chains = chains,
    iter_warmup = iter_warmup,
    iter_sampling = iter_sampling,
    adapt_delta = adapt_delta,
    max_treedepth = max_treedepth,
    seed = seed
  )

  # Wrap in S3 class
  out <- if (is_tv) {
    new_dcvar_tv_fit(
      fit = fit,
      stan_data = stan_data,
      vars = vars,
      standardized = standardize,
      margins = margins,
      skew_direction = skew_direction,
      tv_phi = any_phi,
      phi_tv_mask = phi_mask,
      tv_sigma = tv_sigma,
      backend = backend,
      priors = priors,
      meta = meta
    )
  } else {
    new_dcvar_fit(
      fit = fit,
      stan_data = stan_data,
      vars = vars,
      standardized = standardize,
      margins = margins,
      skew_direction = skew_direction,
      backend = backend,
      priors = priors,
      meta = meta
    )
  }

  .report_sampling_outcome(out$fit, if (is_tv) "TV DC-VAR" else "DC-VAR",
                           chains = chains, backend = backend, object = out)
  out
}

test_that("dcvar() returns a dcvar_fit object", {
  skip_if_no_rstan()

  fit <- get_dcvar_fit()
  expect_s3_class(fit, "dcvar_fit")
  expect_s3_class(fit, "dcvar_model_fit")
  expect_equal(fit$model, "dcvar")
})

test_that("print.dcvar_fit runs without error", {
  skip_if_no_rstan()

  fit <- get_dcvar_fit()
  out <- capture.output(print(fit))
  expect_true(any(grepl("DC-VAR", out)))
  expect_true(any(grepl("rho range", out)))
})

test_that("summary.dcvar_fit returns expected class", {
  skip_if_no_rstan()

  fit <- get_dcvar_fit()
  s <- summary(fit)
  expect_s3_class(s, "dcvar_summary")
  out <- capture.output(print(s))
  expect_true(any(grepl("DC-VAR", out)))
})

test_that("coef.dcvar_fit returns named list", {
  skip_if_no_rstan()

  fit <- get_dcvar_fit()
  co <- coef(fit)
  expect_type(co, "list")
  expect_named(co, c("mu", "Phi", "sigma_eps", "sigma_omega"))
})

test_that("plot.dcvar_fit dispatches without error", {
  skip_if_no_rstan()

  fit <- get_dcvar_fit()
  expect_s3_class(plot(fit, type = "rho"), "ggplot")
  expect_s3_class(plot(fit, type = "phi"), "ggplot")
})

test_that("dcvar() fits exponential margins", {
  skip_if_no_rstan()

  fit <- get_dcvar_exponential_fit()
  expect_s3_class(fit, "dcvar_fit")
  expect_equal(fit$margins, "exponential")
})

test_that("dcvar() fits gamma margins", {
  skip_if_no_rstan()

  fit <- get_dcvar_gamma_fit()
  expect_s3_class(fit, "dcvar_fit")
  expect_equal(fit$margins, "gamma")
})

test_that("dcvar() fits skew-normal margins", {
  skip_if_no_rstan()
  skip_if_not_installed("sn")

  fit <- get_dcvar_skew_normal_fit()
  expect_s3_class(fit, "dcvar_fit")
  expect_equal(fit$margins, "skew_normal")
})

test_that("dcvar fit cache emits only known diagnostic warnings", {
  skip_if_no_rstan()

  expect_known_fit_warnings(get_dcvar_fit_warnings(), "dcvar")
  expect_known_fit_warnings(get_dcvar_exponential_fit_warnings(), "dcvar exponential")
  expect_known_fit_warnings(get_dcvar_gamma_fit_warnings(), "dcvar gamma")
  expect_known_fit_warnings(get_dcvar_skew_normal_fit_warnings(), "dcvar skew-normal")
})

# ============================================================================
# S3 Class: dcvar_covariate_fit
# ============================================================================

#' Construct a dcvar_covariate_fit object
#' @noRd
new_dcvar_covariate_fit <- function(fit, stan_data, vars, covariates,
                                    standardized, standardized_covariates,
                                    drift, zero_init_eta,
                                    backend = "rstan", priors, meta,
                                    dynamic_engine = FALSE) {
  .new_dcvar_model_fit(
    fit = fit, stan_data = stan_data,
    model = if (drift) "dcvar_covariate" else "dcvar_covariate_nodrift",
    vars = vars,
    class = c("dcvar_covariate_fit", "dcvar_model_fit"),
    backend = backend, priors = priors, meta = meta,
    covariates = covariates,
    standardized = standardized,
    standardized_covariates = standardized_covariates,
    margins = "normal",
    drift = drift,
    zero_init_eta = zero_init_eta,
    dynamic_engine = dynamic_engine
  )
}


#' S3 methods for covariate DC-VAR fits
#'
#' @param x,object A `dcvar_covariate_fit` object.
#' @param ... Additional arguments (unused).
#'
#' @name dcvar_covariate_fit-methods
NULL

#' @describeIn dcvar_covariate_fit-methods Print a concise overview of the
#'   covariate DC-VAR fit.
#' @return Invisibly returns `x`.
#' @export
print.dcvar_covariate_fit <- function(x, ...) {
  .print_fit_header(x, "Covariate DC-VAR Model Fit")
  cat(sprintf("Covariates: %s\n", paste(x$covariates, collapse = ", ")))
  cat(sprintf("n_time = %d, D = %d, P = %d\n",
              x$stan_data$n_time, x$stan_data$D, x$stan_data$P))
  cat(sprintf("Residual drift: %s\n", if (x$drift) "yes" else "no"))
  .print_fit_footer(x)

  rho_df <- rho_trajectory(x)
  cat(sprintf("rho range: [%.3f, %.3f] (posterior mean)\n",
              min(rho_df$mean), max(rho_df$mean)))
  invisible(x)
}


#' @describeIn dcvar_covariate_fit-methods Produce a detailed summary including
#'   rho trajectory, VAR parameters, covariate effects, and diagnostics.
#' @param probs Numeric vector of quantile probabilities (default:
#'   `c(0.025, 0.5, 0.975)`).
#' @return A `dcvar_covariate_summary` object (a list).
#' @export
summary.dcvar_covariate_fit <- function(object, probs = c(0.025, 0.5, 0.975), ...) {
  rho_df <- rho_trajectory(object, probs = probs)
  vp <- var_params(object)
  effects <- covariate_effects(object, probs = probs)
  diag <- dcvar_diagnostics(object)

  out <- list(
    model = object$model,
    n_time = object$stan_data$n_time,
    D = object$stan_data$D,
    P = object$stan_data$P,
    drift = object$drift,
    rho_trajectory = rho_df,
    rho_summary = data.frame(
      statistic = c("start", "end", "delta", "min", "max", "mean"),
      value = c(
        rho_df$mean[1],
        rho_df$mean[nrow(rho_df)],
        rho_df$mean[nrow(rho_df)] - rho_df$mean[1],
        min(rho_df$mean),
        max(rho_df$mean),
        mean(rho_df$mean)
      )
    ),
    var_params = vp,
    covariate_effects = effects,
    diagnostics = diag
  )
  class(out) <- "dcvar_covariate_summary"
  out
}


#' Print a dcvar_covariate_summary object
#'
#' @param x A `dcvar_covariate_summary` object as returned by `summary()`.
#' @param ... Additional arguments (unused).
#' @return Invisibly returns `x`.
#' @export
print.dcvar_covariate_summary <- function(x, ...) {
  cat("Covariate DC-VAR Model Summary\n")
  cat(strrep("=", 50), "\n")
  cat(sprintf("n_time = %d, D = %d, P = %d\n", x$n_time, x$D, x$P))
  cat(sprintf("Residual drift: %s\n\n", if (x$drift) "yes" else "no"))

  cat("Rho Trajectory:\n")
  print(x$rho_summary, row.names = FALSE)

  cat("\nCovariate Effects:\n")
  print(x$covariate_effects, row.names = FALSE)

  cat("\nVAR(1) Parameters:\n")
  cat("  mu:\n")
  print(x$var_params$mu[, c("variable", "mean", "q2.5", "q97.5")], row.names = FALSE)
  cat("\n  Phi:\n")
  print(x$var_params$Phi[, c("variable", "mean", "q2.5", "q97.5")], row.names = FALSE)
  .print_margin_params(x$var_params)

  if (!is.null(x$var_params$sigma_omega)) {
    cat("\n  sigma_omega:\n")
    print(x$var_params$sigma_omega[, c("variable", "mean", "q2.5", "q97.5")], row.names = FALSE)
  }

  cat("\nDiagnostics:\n")
  cat(sprintf("  Divergences: %d\n", x$diagnostics$n_divergent))
  cat(sprintf("  Max Rhat: %.3f\n", x$diagnostics$max_rhat))
  cat(sprintf("  Min ESS bulk: %.0f\n", x$diagnostics$min_ess_bulk))

  invisible(x)
}


#' @describeIn dcvar_covariate_fit-methods Extract posterior means of model
#'   coefficients.
#' @return A named list with elements `mu`, `Phi`, `sigma_eps`, `beta_0`,
#'   `beta`, and, when `drift = TRUE`, `sigma_omega`.
#' @export
coef.dcvar_covariate_fit <- function(object, ...) {
  summ <- .fit_summary(object$fit, backend = object$backend)
  beta <- .extract_required_coef(summ, "^beta\\[", "beta", "coef.dcvar_covariate_fit()")
  names(beta) <- object$covariates

  result <- list(
    mu = .extract_required_coef(summ, "^mu\\[", "mu", "coef.dcvar_covariate_fit()"),
    Phi = .extract_required_coef(summ, "^Phi\\[", "Phi", "coef.dcvar_covariate_fit()"),
    sigma_eps = .extract_required_coef(summ, "^sigma_eps\\[", "sigma_eps", "coef.dcvar_covariate_fit()"),
    beta_0 = .extract_required_coef(summ, "^beta_0$", "beta_0", "coef.dcvar_covariate_fit()"),
    beta = beta
  )

  if (isTRUE(object$drift)) {
    result$sigma_omega <- .extract_required_coef(
      summ, "^sigma_omega$", "sigma_omega", "coef.dcvar_covariate_fit()"
    )
  }

  result
}


#' @describeIn dcvar_covariate_fit-methods Dispatch to a plot type: `"rho"`,
#'   `"phi"`, `"diagnostics"`, `"ppc"`, or `"pit"`.
#' @param type Character; one of `"rho"`, `"phi"`, `"diagnostics"`, `"ppc"`,
#'   or `"pit"`.
#' @return A ggplot object.
#' @export
plot.dcvar_covariate_fit <- function(x, type = c("rho", "phi", "diagnostics", "ppc", "pit"), ...) {
  type <- match.arg(type)
  switch(type,
    rho = plot_rho(x, ...),
    phi = plot_phi(x, ...),
    diagnostics = plot_diagnostics(x, ...),
    ppc = plot_ppc(x, ...),
    pit = plot_pit(x, ...)
  )
}

#' Fit an experimental SEM copula VAR(1) model
#'
#' Fits a copula VAR(1) model with a fixed measurement model (factor
#' loadings and measurement error SD are not estimated). Latent innovations
#' are treated as parameters, making this model computationally intensive
#' for large T.
#'
#' @param data A data frame with time series of indicator variables.
#' @param indicators A list of two character vectors, each naming J indicator
#'   columns per latent variable. For example:
#'   `list(PA = c("y1_1", "y1_2", "y1_3"), NA_ = c("y2_1", "y2_2", "y2_3"))`.
#' @param J Number of indicators per latent variable. If `method = "naive"`
#'   and `J` is `NULL`, it is inferred from `indicators`.
#' @param lambda Numeric vector of length J with fixed factor loadings.
#'   Required when `method = "indicator"`.
#' @param sigma_e Fixed measurement error SD (scalar). Required when
#'   `method = "indicator"`.
#' @param method Character string: `"indicator"` (default) fits the fixed
#'   measurement model; `"naive"` averages indicators within each latent block
#'   and fits the observed score VAR.
#' @param margins Latent-innovation marginal specification. A single string
#'   applies the same family to both latent variables. A length-2 character
#'   vector gives a per-variable (mixed) margin (for example
#'   `c("normal", "gamma")`). Normal, exponential, skew-normal, and gamma
#'   margins are supported; homogeneous skew-normal/gamma and per-variable specs
#'   route through the generic mixed-margin Stan models under the Gaussian
#'   copula. Applies to both the indicator and naive methods.
#' @param skew_direction Integer vector of length 2 of `1` (right-skewed) or
#'   `-1` (left-skewed). Required whenever any dimension uses an
#'   `"exponential"` or `"gamma"` margin; only those dimensions consult it.
#' @param time_var Name of the time column (default: `"time"`).
#' @param prior_mu_sd Prior SD for intercepts: `mu ~ normal(0, prior_mu_sd)`.
#' @param prior_phi_sd Prior SD for VAR coefficients:
#'   `phi ~ normal(0, prior_phi_sd)`.
#' @param prior_sigma_sd Prior SD for the lognormal prior on the latent
#'   innovation scale parameter. For normal margins this is applied to
#'   `sigma`; for exponential margins it is applied to `sigma_exp`.
#' @param prior_rho_sd Prior SD for rho_raw:
#'   `rho_raw ~ normal(0, prior_rho_sd)`, with `rho = 0.97 * tanh(rho_raw)`.
#'   For TV SEM fits this is the prior SD for the initial Fisher-z rho.
#' @param tv_phi Selects which latent VAR(1) coefficients evolve as random walks
#'   around their baseline. Either a logical scalar or a character selector as in
#'   [dcvar()]. Time-varying SEM currently requires `method = "indicator"`.
#' @param tv_sigma Logical; if `TRUE`, latent innovation scales evolve as
#'   log-scale random walks. Time-varying SEM currently requires
#'   `method = "indicator"`.
#' @param prior_sigma_omega_rate Prior mean for the TV SEM Fisher-z rho
#'   random-walk SD.
#' @param prior_tau_phi_rate Prior mean for Phi random-walk innovation SDs.
#' @param prior_tau_sigma_rate Prior mean for log-scale random-walk innovation
#'   SDs.
#' @param tv_sigma_k Soft-barrier sharpness for time-varying exponential/gamma
#'   scales.
#' @param chains Number of MCMC chains.
#' @param iter_warmup Warmup iterations per chain.
#' @param iter_sampling Sampling iterations per chain.
#' @param adapt_delta Target acceptance rate.
#' @param max_treedepth Maximum tree depth.
#' @param seed Random seed. When supplied, it seeds both the Stan sampler and
#'   the default per-chain initial values, so repeated fits are reproducible.
#' @param cores Number of parallel chains.
#' @param refresh How often to print progress.
#' @param init Custom init function or `NULL`.
#' @param stan_file Custom Stan file path or `NULL`.
#' @param backend Character: `"auto"` (default, uses rstan), `"rstan"`, or
#'   `"cmdstanr"`. Can also be set globally via
#'   `options(dcvar.backend = "cmdstanr")`.
#' @param ... Additional backend-specific sampling arguments.
#'
#' @return A `dcvar_sem_fit` object.
#'
#' @details
#' **Experimental extension.** This SEM variant supports `fitted()` and
#' `predict()`. PSIS-LOO is available for naive score fits. PIT diagnostics are
#' not yet implemented.
#'
#' **Boundary constraints.** The SEM model constrains each VAR coefficient
#' (Phi) to the interval \eqn{[-0.99, 0.99]}, unlike other dcvar models
#' where Phi is unconstrained. Very strong autoregressive or cross-lag
#' dynamics near \eqn{\pm 1} cannot be captured by this variant.
#'
#' The copula correlation \eqn{\rho} is constrained to \eqn{(-0.97, 0.97)}
#' via `rho = 0.97 * tanh(rho_raw)` to avoid boundary singularity in the
#' Gaussian copula density. Extremely high correlations near \eqn{\pm 1}
#' are truncated.
#'
#' **Margins.** SEM fits support normal, exponential, skew-normal, and gamma
#' latent innovation margins. Exponential and gamma margins use the same shifted
#' positive-support parameterization as the single-level models and therefore
#' require `skew_direction`. Homogeneous skew-normal/gamma and per-variable
#' margin specs route to the generic mixed-margins Stan model.
#'
#' **Post-estimation.** `fitted()` and `predict()` are available for both the
#' latent-state scale (`type = "link"`) and the observed-indicator scale
#' (`type = "response"`). Use [latent_states()] when you specifically need the
#' full posterior summaries of the latent trajectories.
#'
#' @seealso [latent_states()] for extracting estimated latent states,
#'   [simulate_dcvar_sem()] for data generation.
#' @export
dcvar_sem <- function(data, indicators, J = NULL, lambda = NULL, sigma_e = NULL,
                      margins = "normal",
                      skew_direction = NULL,
                      time_var = "time",
                      method = c("indicator", "naive"),
                      prior_mu_sd = 0.25,
                      prior_phi_sd = 0.5,
                      prior_sigma_sd = 0.5,
                      prior_rho_sd = 0.75,
                      tv_phi = FALSE,
                      tv_sigma = FALSE,
                      prior_sigma_omega_rate = 0.1,
                      prior_tau_phi_rate = 0.025,
                      prior_tau_sigma_rate = 0.05,
                      tv_sigma_k = 8,
                      chains = 4,
                      iter_warmup = 2000,
                      iter_sampling = 4000,
                      adapt_delta = 0.95,
                      max_treedepth = 13,
                      seed = NULL,
                      cores = NULL,
                      refresh = 500,
                      init = NULL,
                      stan_file = NULL,
                      backend = getOption("dcvar.backend", "auto"),
                      ...) {
  backend <- .resolve_backend(backend)
  .validate_sampling_args(chains, iter_warmup, iter_sampling,
                          adapt_delta, max_treedepth)
  margins <- .normalize_margins_spec(margins)
  .validate_sem_margins(margins, skew_direction)
  method <- match.arg(method)
  phi_mask <- .resolve_phi_tv_mask(tv_phi)
  any_phi <- sum(phi_mask) > 0L
  .prep_validate_scalar_logical(tv_sigma, "tv_sigma")
  is_tv <- any_phi || tv_sigma

  if (is_tv && identical(method, "naive")) {
    cli_abort(c(
      "Time-varying SEM parameters are not implemented for {.code method = \"naive\"}.",
      "i" = "Use {.code method = \"indicator\"} with fixed measurement parameters, or fit {.fun dcvar} to score data."
    ))
  }
  if (!any_phi && !isTRUE(all.equal(prior_tau_phi_rate, 0.025))) {
    cli_warn("{.arg prior_tau_phi_rate} is ignored when no VAR coefficient is time-varying.")
  }
  if (!tv_sigma && !isTRUE(all.equal(prior_tau_sigma_rate, 0.05))) {
    cli_warn("{.arg prior_tau_sigma_rate} is ignored when {.code tv_sigma = FALSE}.")
  }
  if (!is_tv && !isTRUE(all.equal(prior_sigma_omega_rate, 0.1))) {
    cli_warn("{.arg prior_sigma_omega_rate} is ignored when SEM time-varying components are disabled.")
  }

  if (identical(method, "indicator") &&
      (!is.numeric(J) || length(J) != 1 || J < 1 || J != as.integer(J))) {
    cli_abort("{.arg J} must be a positive integer, got {.val {J}}.")
  }

  # Prepare SEM data
  stan_data <- prepare_sem_data(
    data, indicators, J, lambda, sigma_e,
    margins = margins,
    skew_direction = skew_direction,
    time_var = time_var,
    prior_mu_sd = prior_mu_sd,
    prior_phi_sd = prior_phi_sd,
    prior_sigma_sd = prior_sigma_sd,
    prior_rho_sd = prior_rho_sd,
    tv_phi = phi_mask,
    tv_sigma = tv_sigma,
    prior_sigma_omega_rate = prior_sigma_omega_rate,
    prior_tau_phi_rate = prior_tau_phi_rate,
    prior_tau_sigma_rate = prior_tau_sigma_rate,
    tv_sigma_k = tv_sigma_k,
    method = method
  )

  n_time_obs <- stan_data$n_time
  vars <- attr(stan_data, "vars")
  J_fit <- attr(stan_data, "J") %||% J

  method_label <- if (identical(method, "naive")) "naive SEM" else if (is_tv) "TV SEM" else "SEM"
  margins_text <- paste(margins, collapse = ", ")
  cli_inform("Fitting {method_label} copula VAR model [{margins_text}] (n_time = {n_time_obs}, J = {J_fit})...")

  # Compile model
  model_type <- if (is_tv) {
    "sem_tv"
  } else if (identical(method, "naive")) {
    "sem_naive"
  } else {
    "sem"
  }
  model <- .compile_model(model_type, margins = margins, stan_file = stan_file, backend = backend)

  # Default init
  if (is.null(init)) {
    if (is_tv) {
      init <- function() .init_sem_tv_params(n_time_obs, margins,
                                             tv_phi = phi_mask,
                                             tv_sigma = tv_sigma)
    } else if (identical(method, "naive")) {
      y_scores <- stan_data$y
      init <- function() .init_sem_naive_params(y_scores, margins)
    } else {
      init <- function() .init_sem_params(n_time_obs, margins)
    }
  }

  cores <- .normalize_cores(cores, chains)

  fit <- .sample_model(
    compiled_model = model,
    stan_data = stan_data,
    backend = backend,
    chains = chains,
    iter_warmup = iter_warmup,
    iter_sampling = iter_sampling,
    adapt_delta = adapt_delta,
    max_treedepth = max_treedepth,
    seed = seed,
    cores = cores,
    init = init,
    refresh = refresh,
    ...
  )

  priors <- c(
    list(
      mu_sd = prior_mu_sd,
      phi_sd = prior_phi_sd,
      sigma_sd = prior_sigma_sd,
      rho_sd = prior_rho_sd
    ),
    if (is_tv) list(sigma_omega_rate = prior_sigma_omega_rate),
    if (any_phi) list(tau_phi_rate = prior_tau_phi_rate),
    if (tv_sigma) list(tau_sigma_rate = prior_tau_sigma_rate)
  )
  meta <- list(
    chains = chains,
    iter_warmup = iter_warmup,
    iter_sampling = iter_sampling,
    adapt_delta = adapt_delta,
    max_treedepth = max_treedepth,
    seed = seed
  )

  common_args <- list(
    fit = fit,
    stan_data = stan_data,
    vars = vars,
    J = J_fit,
    lambda = lambda,
    sigma_e = sigma_e,
    indicators = attr(stan_data, "indicators"),
    margins = margins,
    method = method,
    skew_direction = attr(stan_data, "skew_direction"),
    backend = backend,
    priors = priors,
    meta = meta
  )
  out <- if (is_tv) {
    do.call(new_dcvar_sem_tv_fit, c(common_args, list(
      tv_phi = any_phi,
      phi_tv_mask = phi_mask,
      tv_sigma = tv_sigma
    )))
  } else {
    do.call(new_dcvar_sem_fit, common_args)
  }

  .report_sampling_outcome(out$fit, if (is_tv) "TV SEM copula VAR" else if (identical(method, "naive")) "Naive SEM copula VAR" else "SEM copula VAR",
                           chains = chains, backend = backend, object = out)
  out
}

## ----setup, include = FALSE---------------------------------------------------
has_stan <- requireNamespace("rstan", quietly = TRUE)
eval_fits <- identical(Sys.getenv("DCVAR_EVAL_VIGNETTES"), "true") && has_stan
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = eval_fits
)

## ----mixed-margins------------------------------------------------------------
# library(dcvar)
# 
# sim_mix <- simulate_dcvar(
#   n_time = 200,
#   rho_trajectory = rho_constant(200, rho = 0.6),
#   margins = c("normal", "exponential"),
#   skew_direction = c(1, 1),
#   seed = 11
# )
# 
# fit_mix <- dcvar_constant(
#   sim_mix$Y_df,
#   vars = c("y1", "y2"),
#   margins = c("normal", "exponential"),
#   skew_direction = c(1, 1),
#   seed = 11
# )
# 
# # Each dimension is reported under its own family: sigma_eps for the normal
# # variable, sigma_exp for the exponential variable.
# coef(fit_mix)[c("sigma_eps", "sigma_exp")]

## ----tv-fit-------------------------------------------------------------------
# library(dcvar)
# 
# # A series with declining coupling to illustrate the time-varying fits
# sim_tv <- simulate_dcvar(
#   n_time = 150,
#   rho_trajectory = rho_decreasing(150, rho_start = 0.7, rho_end = 0.3),
#   seed = 3
# )
# 
# # Only the autoregressive coefficients evolve over time
# fit_ar <- dcvar(
#   sim_tv$Y_df,
#   vars = c("y1", "y2"),
#   tv_phi = "ar",
#   seed = 4
# )
# 
# # Time-varying coefficients and time-varying scales together, with a
# # right-skewed second margin (the exponential dimension uses the soft barrier)
# fit_full <- dcvar(
#   sim_tv$Y_df,
#   vars = c("y1", "y2"),
#   margins = c("normal", "exponential"),
#   skew_direction = c(1, 1),
#   tv_phi = TRUE,
#   tv_sigma = TRUE,
#   seed = 5
# )

## ----tv-extract---------------------------------------------------------------
# # Posterior summaries of phi11(t), phi12(t), phi21(t), phi22(t)
# head(phi_trajectory(fit_ar))
# 
# # Per-variable residual-scale paths
# head(sigma_trajectory(fit_full))
# 
# # Facetted trajectory plots
# plot_phi_trajectory(fit_ar)
# plot_sigma_trajectory(fit_full)

## ----fit-all------------------------------------------------------------------
# library(dcvar)
# 
# # Simulate data with a step-function trajectory
# sim <- simulate_dcvar(
#   n_time = 200,
#   rho_trajectory = rho_step(200, rho_before = 0.7, rho_after = 0.3),
#   seed = 42
# )
# 
# # Fit all three models on the same data
# fit_dc  <- dcvar(sim$Y_df, vars = c("y1", "y2"), seed = 1)
# fit_hmm <- dcvar_hmm(sim$Y_df, vars = c("y1", "y2"), K = 2, seed = 2)
# fit_con <- dcvar_constant(sim$Y_df, vars = c("y1", "y2"), seed = 3)

## ----compare------------------------------------------------------------------
# dcvar_compare(dcvar = fit_dc, hmm = fit_hmm, constant = fit_con)

## ----tidy---------------------------------------------------------------------
# # Full parameter summary as a data frame
# param_df <- as.data.frame(fit_dc)
# head(param_df)
# 
# # Predictions with marginal intervals
# pred_df <- predict(fit_hmm)
# head(pred_df)

## ----hmm----------------------------------------------------------------------
# # State posteriors, Viterbi path, transition matrix
# states <- hmm_states(fit_hmm)
# 
# # State-specific rho values
# states$rho_state
# 
# # Transition matrix
# states$A
# 
# # Visualise state posteriors
# plot(fit_hmm, type = "states")
# 
# # Transition matrix heatmap
# plot(fit_hmm, type = "transition")

## ----interpret----------------------------------------------------------------
# interpret_rho_trajectory(fit_dc)
# interpret_rho_trajectory(fit_hmm)
# interpret_rho_trajectory(fit_con)

# ============================================================================
# Breakpoint Data Simulation Convenience
# ============================================================================

#' Simulate data with a breakpoint rho trajectory
#'
#' Convenience wrapper that combines [rho_step()] or [rho_double_step()] with
#' [simulate_dcvar()] for quick breakpoint simulation studies.
#'
#' @param n_time Number of time points.
#' @param type Character; one of `"single"` (single breakpoint) or `"double"`
#'   (double breakpoint / relapse pattern).
#' @param rho_before Rho before breakpoint (default: 0.7).
#' @param rho_after Rho after breakpoint (default: 0.3).
#' @param rho_levels Numeric vector of three rho levels for double breakpoint
#'   (default: `c(0.7, 0.3, 0.7)`). Only used when `type = "double"`.
#' @param breakpoint Breakpoint location as proportion of `n_time - 1` (default: 0.5).
#' @param breakpoints Numeric vector of two breakpoints for double type
#'   (default: `c(1/3, 2/3)`).
#' @param transition_width Number of time points for smooth transition
#'   (default: 0 = abrupt).
#' @param mu Intercept vector of length 2 (default: `c(0, 0)`).
#' @param Phi VAR(1) coefficient matrix, 2x2 (default:
#'   `matrix(c(0.3, 0.1, 0.1, 0.3), 2, 2)`).
#' @param sigma_eps Innovation SDs, length 2 (default: `c(1, 1)`).
#' @param seed Random seed.
#'
#' @return A named list as returned by [simulate_dcvar()]. Its `true_params`
#'   additionally records the breakpoint specification: `type`, plus
#'   `breakpoint` (single) or `breakpoints` (double), as proportions of
#'   `n_time - 1`.
#' @export
#'
#' @examples
#' sim <- simulate_breakpoint_data(n_time = 100, type = "single", seed = 42)
#' plot(sim$true_params$rho, type = "l")
simulate_breakpoint_data <- function(n_time,
                                     type = c("single", "double"),
                                     rho_before = 0.7,
                                     rho_after = 0.3,
                                     rho_levels = c(0.7, 0.3, 0.7),
                                     breakpoint = 0.5,
                                     breakpoints = c(1 / 3, 2 / 3),
                                     transition_width = 0,
                                     mu = c(0, 0),
                                     Phi = matrix(c(0.3, 0.1, 0.1, 0.3), 2, 2),
                                     sigma_eps = c(1, 1),
                                     seed = NULL) {
  type <- match.arg(type)

  rho_traj <- if (type == "single") {
    rho_step(n_time, rho_before = rho_before, rho_after = rho_after,
             breakpoint = breakpoint, transition_width = transition_width)
  } else {
    rho_double_step(n_time, rho_levels = rho_levels,
                    breakpoints = breakpoints,
                    transition_width = transition_width)
  }

  sim <- simulate_dcvar(n_time = n_time, rho_trajectory = rho_traj,
                        mu = mu, Phi = Phi, sigma_eps = sigma_eps, seed = seed)

  # Record the breakpoint specification so it is available alongside the other
  # true parameters (as proportions of n_time - 1, matching the arguments).
  sim$true_params$type <- type
  if (type == "single") {
    sim$true_params$breakpoint <- breakpoint
  } else {
    sim$true_params$breakpoints <- breakpoints
  }

  sim
}

test_that("simulate_breakpoint_data() works for single breakpoint", {
  sim <- simulate_breakpoint_data(n_time = 50, type = "single", seed = 42)

  expect_equal(nrow(sim$Y), 50)
  expect_equal(ncol(sim$Y), 2)
  expect_equal(length(sim$true_params$rho), 49)
  expect_true(all(c("time", "y1", "y2") %in% names(sim$Y_df)))
})

test_that("simulate_breakpoint_data() works for double breakpoint", {
  sim <- simulate_breakpoint_data(n_time = 50, type = "double", seed = 42)

  expect_equal(nrow(sim$Y), 50)
  expect_equal(length(sim$true_params$rho), 49)
})

test_that("simulate_breakpoint_data() is reproducible with seed", {
  s1 <- simulate_breakpoint_data(n_time = 50, type = "single", seed = 123)
  s2 <- simulate_breakpoint_data(n_time = 50, type = "single", seed = 123)
  expect_identical(s1$Y, s2$Y)
})

test_that("simulate_breakpoint_data() respects custom parameters", {
  sim <- simulate_breakpoint_data(
    n_time = 50, type = "single",
    rho_before = 0.9, rho_after = 0.1,
    mu = c(1, -1),
    sigma_eps = c(0.5, 0.5),
    seed = 42
  )

  expect_equal(sim$true_params$mu, c(1, -1))
  expect_equal(sim$true_params$sigma_eps, c(0.5, 0.5))
})

# ============================================================================
# S3 Class: dcvar_multilevel_fit
# ============================================================================

#' Construct a dcvar_multilevel_fit object
#' @noRd
new_dcvar_multilevel_fit <- function(fit, stan_data, N, vars, centered,
                                     person_means, priors, meta,
                                     standardized = FALSE,
                                     margins = "normal",
                                     skew_direction = NULL,
                                     backend = "rstan") {
  structure(
    list(
      fit = fit,
      stan_data = stan_data,
      model = "multilevel",
      N = N,
      vars = vars,
      standardized = standardized,
      margins = margins,
      skew_direction = skew_direction,
      centered = centered,
      person_means = person_means,
      backend = backend,
      priors = priors,
      meta = meta
    ),
    class = c("dcvar_multilevel_fit", "dcvar_model_fit")
  )
}


#' Construct a dcvar_multilevel_tv_fit object
#' @noRd
new_dcvar_multilevel_tv_fit <- function(fit, stan_data, N, vars, centered,
                                        person_means, priors, meta,
                                        standardized = FALSE,
                                        margins = "normal",
                                        skew_direction = NULL,
                                        tv_phi = FALSE, phi_tv_mask = NULL,
                                        tv_sigma = FALSE,
                                        backend = "rstan") {
  phi_tv_mask <- phi_tv_mask %||% .resolve_phi_tv_mask(tv_phi)
  out <- new_dcvar_multilevel_fit(
    fit = fit,
    stan_data = stan_data,
    N = N,
    vars = vars,
    centered = centered,
    person_means = person_means,
    priors = priors,
    meta = meta,
    standardized = standardized,
    margins = margins,
    skew_direction = skew_direction,
    backend = backend
  )
  out$model <- "multilevel_tv"
  out$tv_phi <- isTRUE(sum(phi_tv_mask) > 0L)
  out$phi_tv_mask <- phi_tv_mask
  out$tv_sigma <- isTRUE(tv_sigma)
  class(out) <- c("dcvar_multilevel_tv_fit", "dcvar_multilevel_fit", "dcvar_model_fit")
  out
}


#' S3 methods for dcvar_multilevel_fit objects
#'
#' @param x,object A `dcvar_multilevel_fit` object.
#' @param ... Additional arguments (unused).
#'
#' @name dcvar_multilevel_fit-methods
NULL

#' @describeIn dcvar_multilevel_fit-methods Print a concise overview.
#' @return Invisibly returns `x`.
#' @export
print.dcvar_multilevel_fit <- function(x, ...) {
  .print_fit_header(x, "Multilevel Copula VAR Model Fit")
  cat(sprintf("N = %d units, n_time = %d per unit\n", x$N, x$stan_data$n_time))
  .print_fit_footer(x)

  cat(sprintf("rho (global): %.3f\n", coef(x)$rho[[1]]))
  invisible(x)
}


#' @describeIn dcvar_multilevel_fit-methods Produce a detailed summary.
#' @return A `dcvar_multilevel_summary` object (a list).
#' @export
summary.dcvar_multilevel_fit <- function(object, ...) {
  vp <- var_params(object)
  diag <- dcvar_diagnostics(object)
  re <- random_effects(object)

  out <- list(
    model = "multilevel",
    N = object$N,
    n_time = object$stan_data$n_time,
    var_params = vp,
    random_effects = re,
    diagnostics = diag
  )
  class(out) <- "dcvar_multilevel_summary"
  out
}


#' Internal: human-readable list of time-varying multilevel components
#' @noRd
.multilevel_tv_components_label <- function(x) {
  phi_label <- if (isTRUE(x$tv_phi)) {
    active <- .tv_active_phi_coefs(x)
    if (length(active) == 4L) "shared Phi(t) drift" else paste0("shared Phi(t) drift: ", paste(active, collapse = ", "))
  }
  paste(
    c(phi_label, if (isTRUE(x$tv_sigma)) "sigma(t)"),
    collapse = ", "
  )
}


#' @describeIn dcvar_multilevel_fit-methods Print a concise overview of a TV
#'   multilevel fit.
#' @return Invisibly returns `x`.
#' @export
print.dcvar_multilevel_tv_fit <- function(x, ...) {
  .print_fit_header(x, "TV Multilevel Copula VAR Model Fit")
  cat(sprintf("N = %d units, n_time = %d per unit\n", x$N, x$stan_data$n_time))
  cat(sprintf("time-varying components: %s\n", .multilevel_tv_components_label(x)))
  .print_fit_footer(x)

  cat(sprintf("rho (global): %.3f\n", coef(x)$rho[[1]]))
  invisible(x)
}


#' @describeIn dcvar_multilevel_fit-methods Produce a detailed TV multilevel
#'   summary.
#' @param probs Numeric vector of quantile probabilities for trajectories.
#' @return A `dcvar_multilevel_tv_summary` object (a list).
#' @export
summary.dcvar_multilevel_tv_fit <- function(object, probs = c(0.025, 0.5, 0.975), ...) {
  vp <- var_params(object)
  diag <- dcvar_diagnostics(object)
  re <- random_effects(object)
  rho_df <- rho_trajectory(object, probs = probs)
  rho_quantiles <- grep("^q", names(rho_df), value = TRUE)

  out <- list(
    model = "multilevel_tv",
    N = object$N,
    n_time = object$stan_data$n_time,
    components = .multilevel_tv_components_label(object),
    rho_trajectory = rho_df,
    rho_summary = data.frame(
      statistic = c("mean", "sd", rho_quantiles),
      value = c(rho_df$mean[1], rho_df$sd[1], unname(as.numeric(rho_df[1, rho_quantiles, drop = TRUE])))
    ),
    phi_summary = if (isTRUE(object$tv_phi)) {
      .tv_trajectory_summary_block(phi_trajectory(object, probs = probs), "coefficient")
    } else {
      NULL
    },
    sigma_summary = if (isTRUE(object$tv_sigma)) {
      .tv_trajectory_summary_block(sigma_trajectory(object, probs = probs), "variable")
    } else {
      NULL
    },
    var_params = vp,
    random_effects = re,
    diagnostics = diag
  )
  class(out) <- c("dcvar_multilevel_tv_summary", "dcvar_multilevel_summary")
  out
}


#' Print a dcvar_multilevel_summary object
#' @param x A `dcvar_multilevel_summary` object.
#' @param ... Additional arguments (unused).
#' @return Invisibly returns `x`.
#' @export
print.dcvar_multilevel_summary <- function(x, ...) {
  cat("Multilevel Copula VAR Model Summary\n")
  cat(strrep("=", 50), "\n")
  cat(sprintf("N = %d units, n_time = %d per unit\n\n", x$N, x$n_time))

  cat("Population-Level Parameters:\n")
  if (!is.null(x$var_params$phi_bar)) {
    cat("  phi_bar:\n")
    print(x$var_params$phi_bar[, c("variable", "mean", "q2.5", "q97.5")], row.names = FALSE)
  }
  if (!is.null(x$var_params$tau_phi)) {
    cat("\n  tau_phi:\n")
    print(x$var_params$tau_phi[, c("variable", "mean", "q2.5", "q97.5")], row.names = FALSE)
  }
  if (!is.null(x$var_params$sigma)) {
    cat("\n  sigma:\n")
    print(x$var_params$sigma[, c("variable", "mean", "q2.5", "q97.5")], row.names = FALSE)
  } else {
    # Exponential and per-variable (mixed) margins report scale/shape under
    # family-specific names (sigma_exp / omega / delta / sigma_gam / shape_gam).
    .print_margin_params(x$var_params)
  }
  if (!is.null(x$var_params$rho)) {
    cat("\n  rho:\n")
    print(x$var_params$rho[, c("variable", "mean", "q2.5", "q97.5")], row.names = FALSE)
  }

  cat("\nDiagnostics:\n")
  cat(sprintf("  Divergences: %d\n", x$diagnostics$n_divergent))
  cat(sprintf("  Max Rhat: %.3f\n", x$diagnostics$max_rhat))
  cat(sprintf("  Min ESS bulk: %.0f\n", x$diagnostics$min_ess_bulk))

  invisible(x)
}


#' Print a dcvar_multilevel_tv_summary object
#' @param x A `dcvar_multilevel_tv_summary` object.
#' @param ... Additional arguments (unused).
#' @return Invisibly returns `x`.
#' @export
print.dcvar_multilevel_tv_summary <- function(x, ...) {
  cat("TV Multilevel Copula VAR Model Summary\n")
  cat(strrep("=", 50), "\n")
  cat(sprintf("N = %d units, n_time = %d per unit\n", x$N, x$n_time))
  cat(sprintf("time-varying components: %s\n\n", x$components))

  cat("Rho (global):\n")
  print(x$rho_summary, row.names = FALSE)
  if (!is.null(x$phi_summary)) {
    cat("\nPopulation Phi(t) Drift (posterior mean):\n")
    print(x$phi_summary, row.names = FALSE)
  }
  if (!is.null(x$sigma_summary)) {
    cat("\nsigma(t) Trajectories (posterior mean):\n")
    print(x$sigma_summary, row.names = FALSE)
  }

  cat("\nPopulation-Level Parameters:\n")
  cat("  phi_bar:\n")
  print(x$var_params$phi_bar[, c("variable", "mean", "q2.5", "q97.5")], row.names = FALSE)
  cat("\n  tau_phi:\n")
  print(x$var_params$tau_phi[, c("variable", "mean", "q2.5", "q97.5")], row.names = FALSE)
  if (!is.null(x$var_params$tau_phi_tv)) {
    cat("\n  tau_phi_tv (shared drift walk SDs):\n")
    print(x$var_params$tau_phi_tv[, c("variable", "mean", "q2.5", "q97.5")], row.names = FALSE)
  }
  if (!is.null(x$var_params$tau_sigma)) {
    cat("\n  tau_sigma (log-scale walk SDs):\n")
    print(x$var_params$tau_sigma[, c("variable", "mean", "q2.5", "q97.5")], row.names = FALSE)
  }
  .print_margin_params(x$var_params)

  cat("\nDiagnostics:\n")
  cat(sprintf("  Divergences: %d\n", x$diagnostics$n_divergent))
  cat(sprintf("  Max Rhat: %.3f\n", x$diagnostics$max_rhat))
  cat(sprintf("  Min ESS bulk: %.0f\n", x$diagnostics$min_ess_bulk))

  invisible(x)
}


#' @describeIn dcvar_multilevel_fit-methods Extract posterior means of
#'   population-level coefficients.
#' @details
#' Unlike single-level models (where `coef()` returns `$Phi`), the multilevel
#' model returns hierarchical parameters:
#' \describe{
#'   \item{`phi_bar`}{Population-mean VAR coefficients (analogous to `Phi`
#'     in single-level models, vectorised as phi11, phi12, phi21, phi22).}
#'   \item{`tau_phi`}{Between-unit SD of VAR coefficients.}
#'   \item{scale parameters}{`sigma` (innovation SDs) for normal margins,
#'     `sigma_exp` for exponential margins, or per-family scale/shape parameters
#'     (e.g. `sigma_eps`, `sigma_gam`, `shape_gam`) for per-variable,
#'     skew-normal, and gamma mixed-engine margins.}
#'   \item{`rho`}{Copula correlation (constant across units).}
#' }
#' Use [random_effects()] to obtain unit-specific VAR coefficients.
#' @return A named list of posterior means.
#' @export
coef.dcvar_multilevel_fit <- function(object, ...) {
  summ <- .fit_summary(object$fit, backend = object$backend)
  margins <- object$margins %||% "normal"
  scale_coef <- if (.uses_sem_multilevel_mixed_engine("multilevel", margins)) {
    .extract_margin_coefs(
      summ,
      .sem_multilevel_engine_margins("multilevel", margins, 2L)
    )
  } else if (identical(margins, "exponential")) {
    list(sigma_exp = .extract_required_coef(summ, "^sigma_exp\\[", "sigma_exp", "coef.dcvar_multilevel_fit()"))
  } else {
    list(sigma = .extract_required_coef(summ, "^sigma\\[", "sigma", "coef.dcvar_multilevel_fit()"))
  }

  c(list(
    phi_bar = .extract_required_coef(summ, "^phi_bar\\[", "phi_bar", "coef.dcvar_multilevel_fit()"),
    tau_phi = .extract_required_coef(summ, "^tau_phi\\[", "tau_phi", "coef.dcvar_multilevel_fit()")
  ), scale_coef, list(
    rho = .extract_required_coef(summ, "^rho$", "rho", "coef.dcvar_multilevel_fit()")
  ))
}


#' @export
coef.dcvar_multilevel_tv_fit <- function(object, ...) {
  summ <- .fit_summary(object$fit, backend = object$backend)
  margins_vec <- rep(object$margins %||% "normal", length.out = 2L)
  margin_coefs <- .extract_margin_coefs(summ, margins_vec)
  result <- list(
    phi_bar = .extract_required_coef(summ, "^phi_bar\\[", "phi_bar", "coef.dcvar_multilevel_tv_fit()"),
    tau_phi = .extract_required_coef(summ, "^tau_phi\\[", "tau_phi", "coef.dcvar_multilevel_tv_fit()")
  )
  extras <- list(
    rho = .extract_required_coef(summ, "^rho$", "rho", "coef.dcvar_multilevel_tv_fit()")
  )
  if (isTRUE(object$tv_phi)) {
    tau_phi_tv <- .extract_required_coef(summ, "^tau_phi_tv\\[", "tau_phi_tv", "coef.dcvar_multilevel_tv_fit()")
    names(tau_phi_tv) <- paste0("tau_phi_tv[", .tv_active_phi_coefs(object), "]")
    extras$tau_phi_tv <- tau_phi_tv
  }
  if (isTRUE(object$tv_sigma)) {
    extras$tau_sigma <- .extract_required_coef(summ, "^tau_sigma\\[", "tau_sigma", "coef.dcvar_multilevel_tv_fit()")
  }
  c(result, margin_coefs, extras)
}


#' @describeIn dcvar_multilevel_fit-methods Dispatch to a plot type.
#' @param type Character; one of `"random_effects"`, `"diagnostics"`.
#' @return A ggplot object.
#' @export
plot.dcvar_multilevel_fit <- function(x,
                                      type = c("random_effects", "diagnostics"),
                                      ...) {
  type <- match.arg(type)
  switch(type,
    random_effects = plot_random_effects(x, ...),
    diagnostics = plot_diagnostics(x, ...)
  )
}

# ============================================================================
# S3 Class: dcvar_tv_fit (time-varying DC-VAR; subclass of dcvar_fit)
# ============================================================================

#' Construct a dcvar_tv_fit object
#'
#' Subclass of `dcvar_fit`: rho-based methods (`rho_trajectory()`,
#' `plot_rho()`, `loo()`, `dependence_summary()`) inherit unchanged, while
#' Phi/sigma-shaped methods dispatch to the trajectory-aware overrides.
#' @noRd
new_dcvar_tv_fit <- function(fit, stan_data, vars, standardized,
                             margins = "normal", skew_direction = NULL,
                             tv_phi = FALSE, phi_tv_mask = NULL, tv_sigma = FALSE,
                             backend = "rstan", priors, meta) {
  if (is.null(phi_tv_mask)) {
    phi_tv_mask <- .resolve_phi_tv_mask(tv_phi)
  }
  .new_dcvar_model_fit(
    fit = fit, stan_data = stan_data, model = "dcvar_tv", vars = vars,
    class = c("dcvar_tv_fit", "dcvar_fit", "dcvar_model_fit"),
    backend = backend, priors = priors, meta = meta,
    standardized = standardized,
    margins = margins,
    skew_direction = skew_direction,
    tv_phi = tv_phi,
    phi_tv_mask = phi_tv_mask,
    tv_sigma = tv_sigma
  )
}

#' Internal: active (time-varying) coefficient names of a TV fit
#' @noRd
.tv_active_phi_coefs <- function(object) {
  mask <- object$phi_tv_mask %||% .resolve_phi_tv_mask(isTRUE(object$tv_phi))
  names(mask)[mask == 1L]
}


#' S3 methods for dcvar_tv_fit objects
#'
#' Methods for fits from [dcvar()] with `tv_phi = TRUE` and/or
#' `tv_sigma = TRUE` (time-varying VAR coefficients and residual scales).
#'
#' @param x,object A `dcvar_tv_fit` object.
#' @param ... Additional arguments (unused).
#'
#' @name dcvar_tv_fit-methods
NULL

#' Internal: human-readable list of time-varying components
#' @noRd
.tv_components_label <- function(x) {
  phi_label <- if (isTRUE(x$tv_phi)) {
    active <- .tv_active_phi_coefs(x)
    if (length(active) == 4L) "Phi(t)" else paste0("Phi(t): ", paste(active, collapse = ", "))
  }
  paste(
    c("rho(t)", phi_label, if (isTRUE(x$tv_sigma)) "sigma(t)"),
    collapse = ", "
  )
}

#' Internal: compact start/end/range summaries for trajectory data frames
#' @noRd
.tv_trajectory_summary_block <- function(df, group_col) {
  do.call(rbind, lapply(split(df, df[[group_col]]), function(g) {
    data.frame(
      group = g[[group_col]][1],
      start = g$mean[1],
      end = g$mean[nrow(g)],
      delta = g$mean[nrow(g)] - g$mean[1],
      min = min(g$mean),
      max = max(g$mean)
    )
  }))
}

#' @describeIn dcvar_tv_fit-methods Print a concise overview of the TV DC-VAR
#'   fit.
#' @return Invisibly returns `x`.
#' @export
print.dcvar_tv_fit <- function(x, ...) {
  .print_fit_header(x, "TV DC-VAR Model Fit")
  cat(sprintf("n_time = %d, D = %d\n", x$stan_data$n_time, x$stan_data$D))
  cat(sprintf("time-varying components: %s\n", .tv_components_label(x)))
  .print_fit_footer(x)

  rho_df <- rho_trajectory(x)
  cat(sprintf("rho range: [%.3f, %.3f] (posterior mean)\n",
              min(rho_df$mean), max(rho_df$mean)))
  if (isTRUE(x$tv_phi)) {
    phi_df <- phi_trajectory(x, probs = 0.5)
    rng <- range(phi_df$mean)
    cat(sprintf("Phi(t) coefficient range: [%.3f, %.3f] (posterior mean)\n",
                rng[1], rng[2]))
  }
  invisible(x)
}


#' @describeIn dcvar_tv_fit-methods Produce a detailed summary including the
#'   rho, Phi, and sigma trajectories, constant parameters, and diagnostics.
#' @param probs Numeric vector of quantile probabilities for the trajectories
#'   (default: `c(0.025, 0.5, 0.975)`).
#' @return A `dcvar_tv_summary` object (a list).
#' @export
summary.dcvar_tv_fit <- function(object, probs = c(0.025, 0.5, 0.975), ...) {
  rho_df <- rho_trajectory(object, probs = probs)
  vp <- var_params(object)
  diag <- dcvar_diagnostics(object)

  phi_summary <- if (isTRUE(object$tv_phi)) {
    .tv_trajectory_summary_block(phi_trajectory(object, probs = probs), "coefficient")
  } else {
    NULL
  }
  sigma_summary <- if (isTRUE(object$tv_sigma)) {
    .tv_trajectory_summary_block(sigma_trajectory(object, probs = probs), "variable")
  } else {
    NULL
  }

  out <- list(
    model = "dcvar_tv",
    n_time = object$stan_data$n_time,
    D = object$stan_data$D,
    components = .tv_components_label(object),
    rho_trajectory = rho_df,
    rho_summary = data.frame(
      statistic = c("start", "end", "delta", "min", "max", "mean"),
      value = c(
        rho_df$mean[1],
        rho_df$mean[nrow(rho_df)],
        rho_df$mean[nrow(rho_df)] - rho_df$mean[1],
        min(rho_df$mean),
        max(rho_df$mean),
        mean(rho_df$mean)
      )
    ),
    phi_summary = phi_summary,
    sigma_summary = sigma_summary,
    var_params = vp,
    diagnostics = diag
  )
  class(out) <- "dcvar_tv_summary"
  out
}


#' Print a dcvar_tv_summary object
#'
#' @param x A `dcvar_tv_summary` object as returned by
#'   [summary.dcvar_tv_fit()].
#' @param ... Additional arguments (unused).
#' @return Invisibly returns `x`.
#' @export
print.dcvar_tv_summary <- function(x, ...) {
  cat("TV DC-VAR Model Summary\n")
  cat(strrep("=", 50), "\n")
  cat(sprintf("n_time = %d, D = %d\n", x$n_time, x$D))
  cat(sprintf("time-varying components: %s\n\n", x$components))

  cat("Rho Trajectory:\n")
  print(x$rho_summary, row.names = FALSE)

  if (!is.null(x$phi_summary)) {
    cat("\nPhi(t) Trajectories (posterior mean):\n")
    print(x$phi_summary, row.names = FALSE)
  }
  if (!is.null(x$sigma_summary)) {
    cat("\nsigma(t) Trajectories (posterior mean):\n")
    print(x$sigma_summary, row.names = FALSE)
  }

  cat("\nConstant Parameters:\n")
  cat("  mu:\n")
  print(x$var_params$mu[, c("variable", "mean", "q2.5", "q97.5")], row.names = FALSE)
  cat("\n  Phi (baseline):\n")
  print(x$var_params$Phi[, c("variable", "mean", "q2.5", "q97.5")], row.names = FALSE)
  if (!is.null(x$var_params$tau_phi)) {
    cat("\n  tau_phi (walk SDs):\n")
    print(x$var_params$tau_phi[, c("variable", "mean", "q2.5", "q97.5")], row.names = FALSE)
  }
  if (!is.null(x$var_params$tau_sigma)) {
    cat("\n  tau_sigma (log-scale walk SDs):\n")
    print(x$var_params$tau_sigma[, c("variable", "mean", "q2.5", "q97.5")], row.names = FALSE)
  }
  .print_margin_params(x$var_params)
  if (!is.null(x$var_params$sigma_omega)) {
    cat("\n  sigma_omega:\n")
    print(x$var_params$sigma_omega[, c("variable", "mean", "q2.5", "q97.5")], row.names = FALSE)
  }

  cat("\nDiagnostics:\n")
  cat(sprintf("  Divergences: %d\n", x$diagnostics$n_divergent))
  cat(sprintf("  Max Rhat: %.3f\n", x$diagnostics$max_rhat))
  cat(sprintf("  Min ESS bulk: %.0f\n", x$diagnostics$min_ess_bulk))

  invisible(x)
}


#' @describeIn dcvar_tv_fit-methods Extract posterior means of the constant
#'   model coefficients (baselines, walk innovation SDs, margin shape
#'   parameters). Use [phi_trajectory()] and [sigma_trajectory()] for the
#'   time-varying paths.
#' @return A named list of posterior means.
#' @export
coef.dcvar_tv_fit <- function(object, ...) {
  summ <- .fit_summary(object$fit, backend = object$backend)
  result <- list(
    mu = .extract_required_coef(summ, "^mu\\[", "mu", "coef.dcvar_tv_fit()"),
    Phi = .extract_required_coef(summ, "^Phi\\[", "Phi", "coef.dcvar_tv_fit()")
  )

  # The generic TV model reports per-dimension margin parameters (union
  # block), so use the per-dimension report regardless of homogeneity.
  margins_vec <- rep(object$margins %||% "normal", length.out = object$stan_data$D)
  specs <- .mixed_margin_report_vars(margins_vec)
  margin_coefs <- lapply(specs, function(vars) {
    rows <- match(vars, summ$variable)
    rows <- rows[!is.na(rows)]
    if (length(rows) == 0L) return(NULL)
    setNames(summ$mean[rows], summ$variable[rows])
  })
  result <- c(result, Filter(Negate(is.null), margin_coefs))

  if (isTRUE(object$tv_phi)) {
    tau_phi <- .extract_required_coef(summ, "^tau_phi\\[", "tau_phi", "coef.dcvar_tv_fit()")
    # Stan indexes tau_phi by active position; relabel by coefficient name.
    names(tau_phi) <- paste0("tau_phi[", .tv_active_phi_coefs(object), "]")
    result$tau_phi <- tau_phi
  }
  if (isTRUE(object$tv_sigma)) {
    result$tau_sigma <- .extract_required_coef(summ, "^tau_sigma\\[", "tau_sigma", "coef.dcvar_tv_fit()")
  }
  result$sigma_omega <- .extract_required_coef(summ, "^sigma_omega$", "sigma_omega", "coef.dcvar_tv_fit()")
  result
}


#' @describeIn dcvar_tv_fit-methods Dispatch to a plot type: `"rho"`,
#'   `"phi"` (coefficient trajectories), `"sigma"` (scale trajectories),
#'   `"diagnostics"`, or `"pit"`.
#' @param type Character; one of `"rho"`, `"phi"`, `"sigma"`,
#'   `"diagnostics"`, or `"pit"`.
#' @return A ggplot object.
#' @export
plot.dcvar_tv_fit <- function(x, type = c("rho", "phi", "sigma", "diagnostics", "pit"), ...) {
  type <- match.arg(type)
  switch(type,
    rho = plot_rho(x, ...),
    phi = plot_phi_trajectory(x, ...),
    sigma = plot_sigma_trajectory(x, ...),
    diagnostics = plot_diagnostics(x, ...),
    pit = plot_pit(x, ...)
  )
}

# ============================================================================
# as.data.frame methods for fit objects
# ============================================================================

#' Convert a dcvar fit summary to a data frame
#'
#' Returns the full parameter summary as a tidy data frame with
#' correct 2.5%/97.5% quantiles.
#'
#' @param x A fitted dcvar model object (any subclass of `dcvar_model_fit`:
#'   `dcvar_fit`, `dcvar_constant_fit`, `dcvar_hmm_fit`, `dcvar_covariate_fit`,
#'   `dcvar_sem_fit`, or `dcvar_multilevel_fit`).
#' @param row.names Ignored.
#' @param optional Ignored.
#' @param ... Additional arguments (unused).
#'
#' @return A data frame with columns `variable`, `mean`, `sd`, `q2.5`,
#'   `q97.5`, `rhat`, `ess_bulk`, `ess_tail`.
#' @export
as.data.frame.dcvar_model_fit <- function(x, row.names = NULL, optional = FALSE, ...) {
  summ <- .fit_summary(
    x$fit, variables = NULL, backend = x$backend,
    mean = mean,
    sd = sd,
    ~posterior::quantile2(.x, probs = c(0.025, 0.975)),
    rhat = posterior::rhat,
    ess_bulk = posterior::ess_bulk,
    ess_tail = posterior::ess_tail
  )
  as.data.frame(summ)
}

# ============================================================================
# Plotting Functions
# ============================================================================

#' Plot the rho trajectory with credible intervals
#'
#' @param object A fitted model object with a rho trajectory (`dcvar_fit`,
#'   `dcvar_covariate_fit`, `dcvar_hmm_fit`, or `dcvar_constant_fit`).
#' @param show_ci Logical; show credible interval ribbons (default: `TRUE`).
#' @param ci_level Credible interval level for the outer ribbon (default: 0.95).
#' @param inner_level Credible interval level for the inner ribbon (default:
#'   0.80). Set to `NULL` to disable the inner ribbon.
#' @param true_rho Optional numeric vector of true rho values for overlay
#'   (useful for simulation studies).
#' @param title Plot title.
#' @param ... Additional arguments (unused).
#'
#' @return A ggplot object.
#' @export
plot_rho <- function(object, show_ci = TRUE, ci_level = 0.95, inner_level = 0.80,
                     true_rho = NULL, title = NULL, ...) {
  .validate_interval_level(ci_level, arg_name = "ci_level")
  if (!is.null(inner_level)) {
    .validate_interval_level(inner_level, arg_name = "inner_level")
    if (inner_level >= ci_level) {
      cli_abort(c(
        "{.arg inner_level} must be smaller than {.arg ci_level}.",
        "i" = "The inner ribbon must be narrower than the outer ribbon."
      ))
    }
  }

  # Compute the quantile probs needed for the requested CI levels
  alpha_outer <- (1 - ci_level) / 2
  probs <- c(alpha_outer, 1 - alpha_outer, 0.5)
  if (!is.null(inner_level)) {
    alpha_inner <- (1 - inner_level) / 2
    probs <- c(alpha_outer, alpha_inner, 0.5, 1 - alpha_inner, 1 - alpha_outer)
  }
  probs <- sort(unique(probs))

  rho_df <- rho_trajectory(object, probs = probs)

  if (is.null(title)) {
    model_name <- switch(object$model,
      dcvar = "DC-VAR",
      hmm = "HMM Copula",
      constant = "Constant Copula",
      object$model
    )
    title <- paste0("Time-Varying Copula Dependence (", model_name, ")")
  }

  q_outer_low <- paste0("q", alpha_outer * 100)
  q_outer_high <- paste0("q", (1 - alpha_outer) * 100)

  p <- ggplot2::ggplot(rho_df, ggplot2::aes(x = .data$time)) +
    ggplot2::geom_hline(yintercept = 0, linetype = "dashed", color = "gray50")

  if (show_ci && q_outer_low %in% names(rho_df)) {
    # Outer CI
    p <- p + ggplot2::geom_ribbon(
      ggplot2::aes(ymin = .data[[q_outer_low]], ymax = .data[[q_outer_high]]),
      fill = "steelblue", alpha = 0.2
    )
    # Inner CI
    if (!is.null(inner_level)) {
      q_inner_low <- paste0("q", alpha_inner * 100)
      q_inner_high <- paste0("q", (1 - alpha_inner) * 100)
      if (q_inner_low %in% names(rho_df) && q_inner_high %in% names(rho_df)) {
        p <- p + ggplot2::geom_ribbon(
          ggplot2::aes(ymin = .data[[q_inner_low]], ymax = .data[[q_inner_high]]),
          fill = "steelblue", alpha = 0.3
        )
      }
    }
  }

  p <- p +
    ggplot2::geom_line(ggplot2::aes(y = .data$mean), color = "steelblue", linewidth = 1)

  if ("q50" %in% names(rho_df)) {
    p <- p +
      ggplot2::geom_line(ggplot2::aes(y = .data$q50), color = "darkblue", linetype = "dashed")
  }

  if (!is.null(true_rho)) {
    time_values <- .observed_time_values(object$stan_data, drop_first = TRUE)
    if (!is.numeric(true_rho)) {
      cli_abort("{.arg true_rho} must be a numeric vector.")
    }
    if (length(true_rho) != length(time_values)) {
      cli_abort(c(
        "{.arg true_rho} must have the same length as the observed rho time axis.",
        "i" = "Use the fit's stored time values for simulation overlays."
      ))
    }
    true_df <- data.frame(time = time_values, rho = true_rho)
    p <- p + ggplot2::geom_line(
      data = true_df,
      ggplot2::aes(x = .data$time, y = .data$rho),
      color = "red", linewidth = 1, linetype = "solid"
    )
  }

  # Build subtitle dynamically from the actual CI levels
  subtitle <- if (show_ci) {
    if (!is.null(inner_level)) {
      sprintf("%d%% and %d%% credible intervals",
              round(ci_level * 100), round(inner_level * 100))
    } else {
      sprintf("%d%% credible interval", round(ci_level * 100))
    }
  } else {
    NULL
  }

  p <- p +
    ggplot2::scale_y_continuous(limits = c(-1, 1)) +
    ggplot2::labs(
      x = "Time",
      y = expression(rho[t]),
      title = title,
      subtitle = subtitle
    ) +
    ggplot2::theme_minimal() +
    ggplot2::theme(
      plot.title = ggplot2::element_text(face = "bold"),
      axis.title = ggplot2::element_text(face = "bold")
    )

  p
}


#' Plot VAR(1) coefficient matrix as a heatmap
#'
#' @param object A fitted model object.
#' @param var_names Character vector of variable names for axis labels.
#' @param ... Additional arguments (unused).
#'
#' @return A ggplot object.
#' @export
plot_phi <- function(object, var_names = NULL, ...) {
  if (inherits(object, "dcvar_hmm_fit") && isTRUE(object$switching)) {
    cli_abort(c(
      "{.fun plot_phi} is not available for state-specific HMM fits.",
      "i" = "Use {.fun hmm_state_params} to inspect the effective per-state Phi matrices."
    ))
  }

  summ <- .fit_summary(
    object$fit, variables = NULL, backend = object$backend,
    required = .stan_output_group_pattern("Phi"),
    required_type = "pattern",
    context = "plot_phi()",
    output_type = "parameter group",
    mean,
    ~posterior::quantile2(.x, probs = c(0.025, 0.975))
  )

  phi_rows <- grep("^Phi\\[", summ$variable)
  phi_df <- data.frame(
    variable = summ$variable[phi_rows],
    mean = summ$mean[phi_rows],
    q2.5 = summ$q2.5[phi_rows],
    q97.5 = summ$q97.5[phi_rows]
  )

  idx <- regmatches(phi_df$variable, regexec("Phi\\[(\\d+),(\\d+)\\]", phi_df$variable))
  if (any(lengths(idx) != 3)) {
    cli_abort("Unexpected Phi variable name format in Stan output.")
  }
  phi_df$row <- as.integer(vapply(idx, `[`, character(1), 2))
  phi_df$col <- as.integer(vapply(idx, `[`, character(1), 3))

  if (is.null(var_names)) {
    var_names <- if (!is.null(object$vars)) object$vars else paste0("Y", 1:max(phi_df$row))
  }

  phi_df$from <- var_names[phi_df$col]
  phi_df$to <- var_names[phi_df$row]
  phi_df$label <- sprintf("%.2f\n[%.2f, %.2f]", phi_df$mean, phi_df$q2.5, phi_df$q97.5)

  # Symmetric limits around zero based on actual coefficient range
  max_abs <- max(abs(phi_df$mean), na.rm = TRUE)
  fill_limit <- max(max_abs * 1.1, 0.1)  # at least 0.1 to avoid degenerate scale

  ggplot2::ggplot(phi_df, ggplot2::aes(x = .data$from, y = .data$to)) +
    ggplot2::geom_tile(ggplot2::aes(fill = .data$mean), color = "white") +
    ggplot2::geom_text(ggplot2::aes(label = .data$label), size = 3) +
    ggplot2::scale_fill_gradient2(
      low = "blue", mid = "white", high = "red",
      midpoint = 0, limits = c(-fill_limit, fill_limit)
    ) +
    ggplot2::labs(
      x = "From (t-1)", y = "To (t)",
      fill = expression(Phi),
      title = "VAR(1) Coefficient Matrix"
    ) +
    ggplot2::theme_minimal() +
    ggplot2::theme(
      axis.text.x = ggplot2::element_text(angle = 45, hjust = 1),
      plot.title = ggplot2::element_text(face = "bold")
    ) +
    ggplot2::coord_fixed()
}


#' Plot MCMC diagnostics
#'
#' Creates a combined panel with trace plots, Rhat, and ESS diagnostics.
#'
#' @param object A fitted model object.
#' @param ... Additional arguments (unused).
#'
#' @return A combined ggplot object (via patchwork).
#' @export
plot_diagnostics <- function(object, ...) {
  margins <- object$margins %||% "normal"

  trace_pars <- if (object$model == "hmm" && isTRUE(object$switching)) {
    K <- object$K
    D <- object$stan_data$D
    sw <- object$switch
    Mu_K <- if (isTRUE(sw$mu == 1L)) K else 1L
    Mrg_K <- if (isTRUE(sw$margins == 1L)) K else 1L
    mu_pars <- paste0(
      "mu[",
      rep(seq_len(Mu_K), each = D), ",", rep(seq_len(D), times = Mu_K),
      "]"
    )
    phi_pars <- paste0(
      "Phi_base[",
      rep(seq_len(D), each = D), ",", rep(seq_len(D), times = D),
      "]"
    )
    if (any(sw$phi_mask > 0L)) {
      ij <- list(c(1L, 1L), c(1L, 2L), c(2L, 1L), c(2L, 2L))
      active <- which(sw$phi_mask > 0L)
      phi_dev <- unlist(lapply(seq_len(K), function(k) {
        vapply(active, function(a) {
          sprintf("Phi_dev[%d,%d,%d]", k, ij[[a]][1], ij[[a]][2])
        }, character(1))
      }), use.names = FALSE)
      phi_pars <- c(phi_pars, phi_dev)
    }
    margin_pars <- unlist(
      .hmm_switching_report_vars(object$margins_matrix, Mrg_K),
      use.names = FALSE
    )
    c(mu_pars, phi_pars, margin_pars, paste0("rho_state[", seq_len(K), "]"))
  } else if (object$model == "multilevel") {
    phi_bars <- paste0("phi_bar[", 1:4, "]")
    if (.uses_sem_multilevel_mixed_engine("multilevel", margins)) {
      c(
        phi_bars,
        .mixed_plot_margin_vars(.sem_multilevel_engine_margins("multilevel", margins, 2L)),
        "rho"
      )
    } else if (identical(margins, "exponential")) {
      c(phi_bars, "sigma_exp[1]", "sigma_exp[2]", "rho")
    } else {
      c(phi_bars, "rho")
    }
  } else if (object$model == "sem") {
    if (.uses_sem_multilevel_mixed_engine("sem", margins)) {
      c(
        "mu[1]", "mu[2]", "phi11", "phi22",
        .mixed_plot_margin_vars(.sem_multilevel_engine_margins("sem", margins, 2L)),
        "rho"
      )
    } else if (identical(margins, "exponential")) {
      c("mu[1]", "mu[2]", "phi11", "phi22", "sigma_exp[1]", "sigma_exp[2]", "rho")
    } else {
      c("mu[1]", "mu[2]", "phi11", "phi22", "sigma[1]", "sigma[2]", "rho")
    }
  } else if (.is_mixed_margins(margins)) {
    c("mu[1]", "mu[2]", .mixed_plot_margin_vars(margins))
  } else if (margins[[1L]] == "normal") {
    c("mu[1]", "mu[2]", "sigma_eps[1]", "sigma_eps[2]")
  } else if (margins[[1L]] == "exponential") {
    c("mu[1]", "mu[2]", "sigma_exp[1]", "sigma_exp[2]")
  } else if (margins[[1L]] == "skew_normal") {
    c("mu[1]", "mu[2]", "omega[1]", "omega[2]", "delta[1]", "delta[2]")
  } else if (margins[[1L]] == "gamma") {
    c("mu[1]", "mu[2]", "sigma_gam[1]", "sigma_gam[2]", "shape_gam")
  } else {
    c("mu[1]", "mu[2]")
  }
  if (object$model == "dcvar") trace_pars <- c(trace_pars, "sigma_omega")
  if (object$model == "dcvar_tv") {
    # The generic TV model uses the mixed-union parameter block (per-dimension
    # margin parameters) even for homogeneous margins; add the walk SDs.
    n_phi_tv <- sum(object$phi_tv_mask %||% .resolve_phi_tv_mask(isTRUE(object$tv_phi)))
    trace_pars <- c(
      "mu[1]", "mu[2]",
      .mixed_plot_margin_vars(rep(margins, length.out = 2L)),
      "sigma_omega",
      if (isTRUE(object$tv_phi)) paste0("tau_phi[", seq_len(n_phi_tv), "]"),
      if (isTRUE(object$tv_sigma)) paste0("tau_sigma[", 1:2, "]")
    )
  }
  if (object$model %in% c("dcvar_covariate", "dcvar_covariate_nodrift")) {
    trace_pars <- c(trace_pars, "beta_0", paste0("beta[", seq_len(object$stan_data$P), "]"))
    if (object$model == "dcvar_covariate") trace_pars <- c(trace_pars, "sigma_omega")
  }
  if (object$model == "hmm" && !isTRUE(object$switching)) {
    K <- object$K
    trace_pars <- c(trace_pars, paste0("rho_state[", 1:K, "]"))
  }
  if (object$model == "constant") {
    trace_pars <- c(
      trace_pars,
      if (identical(object$copula %||% "gaussian", "clayton")) "theta" else "rho"
    )
  }

  required_patterns <- unique(vapply(
    trace_pars,
    .stan_output_group_pattern,
    character(1)
  ))

  draws_array <- .fit_draws(
    object$fit,
    format = "draws_array",
    backend = object$backend,
    required = required_patterns,
    required_type = "pattern",
    context = "plot_diagnostics()",
    output_type = "parameter group"
  )

  p1 <- bayesplot::mcmc_trace(draws_array, pars = trace_pars) +
    ggplot2::ggtitle("Trace Plots")

  summ <- .fit_summary(
    object$fit,
    backend = object$backend,
    required = required_patterns,
    required_type = "pattern",
    context = "plot_diagnostics()",
    output_type = "parameter group"
  )
  rhats <- summ$rhat[is.finite(summ$rhat)]
  p2 <- bayesplot::mcmc_rhat(rhats) +
    ggplot2::ggtitle("R-hat Diagnostics")

  n_eff_ratio <- summ$ess_bulk[is.finite(summ$ess_bulk)]
  total_draws <- object$meta$iter_sampling * object$meta$chains
  p3 <- bayesplot::mcmc_neff(n_eff_ratio / total_draws) +
    ggplot2::ggtitle("Effective Sample Size Ratio")

  bottom_row <- patchwork::wrap_plots(p2, p3, ncol = 2)
  (p1 / bottom_row) +
    patchwork::plot_annotation(
      title = "MCMC Diagnostics",
      theme = ggplot2::theme(
        plot.title = ggplot2::element_text(face = "bold", size = 14)
      )
    )
}


#' Posterior predictive check for residual correlations
#'
#' @param object A fitted model object.
#' @param n_sample Number of posterior draws to use (default: 100).
#' @param ... Additional arguments (unused).
#'
#' @return A ggplot object.
#'
#' @details Posterior predictive checks are currently available for normal and
#'   exponential margins. Gamma and skew-normal fits store copula-level
#'   replicated z-scores in `eps_rep`, so their replicated draws are not on the
#'   same residual scale as `eps`.
#' @export
plot_ppc <- function(object, n_sample = 100, ...) {
  if (inherits(object, "dcvar_multilevel_fit") || inherits(object, "dcvar_sem_fit")) {
    cli_abort("Posterior predictive checks are not yet supported for {.cls {class(object)[1]}} models.")
  }
  if (inherits(object, "dcvar_hmm_fit") && isTRUE(object$switching)) {
    cli_abort(c(
      "{.fun plot_ppc} is not yet available for state-specific HMM fits.",
      "i" = "The observed residuals are state-specific; use {.fun hmm_states} and {.fun hmm_state_params} for regime diagnostics."
    ))
  }

  margins <- object$margins %||% "normal"
  copula_z_fams <- intersect(margins, c("gamma", "skew_normal"))
  if (length(copula_z_fams) > 0L) {
    cli_abort(
      c(
        "Posterior predictive checks are not supported for {.val {copula_z_fams}} margins.",
        "i" = "The stored {.field eps_rep} draws are copula-level z-scores rather than replicated residuals on the observed margin scale."
      )
    )
  }

  if (!all(margins %in% c("normal", "exponential"))) {
    cli_abort("Posterior predictive checks are not implemented for {.val {margins}} margins.")
  }

  eps_rep_draws <- posterior::as_draws_matrix(.fit_draws(
    object$fit, "eps_rep", backend = object$backend,
    required = .stan_output_group_pattern("eps_rep"),
    required_type = "pattern",
    context = "plot_ppc()",
    output_type = "generated quantity"
  ))
  eps_obs_draws <- posterior::as_draws_matrix(.fit_draws(
    object$fit, "eps", backend = object$backend,
    required = .stan_output_group_pattern("eps"),
    required_type = "pattern",
    context = "plot_ppc()",
    output_type = "transformed parameter"
  ))
  eps_rep_cols_1 <- grep("eps_rep\\[.*,1\\]", colnames(eps_rep_draws))
  eps_rep_cols_2 <- grep("eps_rep\\[.*,2\\]", colnames(eps_rep_draws))
  eps_obs_cols_1 <- grep("eps\\[.*,1\\]", colnames(eps_obs_draws))
  eps_obs_cols_2 <- grep("eps\\[.*,2\\]", colnames(eps_obs_draws))

  # Observed residuals (posterior mean)
  eps_obs_1 <- colMeans(eps_obs_draws[, eps_obs_cols_1, drop = FALSE])
  eps_obs_2 <- colMeans(eps_obs_draws[, eps_obs_cols_2, drop = FALSE])
  cor_obs <- cor(eps_obs_1, eps_obs_2, use = "complete.obs")

  # Replicated correlations
  n_draws <- nrow(eps_rep_draws)
  n_sample <- min(n_sample, n_draws)
  sample_idx <- sample(seq_len(n_draws), n_sample)

  cor_rep <- numeric(n_sample)
  for (i in seq_along(sample_idx)) {
    idx <- sample_idx[i]
    rep_1 <- as.numeric(eps_rep_draws[idx, eps_rep_cols_1, drop = TRUE])
    rep_2 <- as.numeric(eps_rep_draws[idx, eps_rep_cols_2, drop = TRUE])
    cor_rep[i] <- cor(rep_1, rep_2, use = "complete.obs")
  }

  ggplot2::ggplot(data.frame(cor = cor_rep), ggplot2::aes(x = .data$cor)) +
    ggplot2::geom_histogram(bins = 30, fill = "steelblue", alpha = 0.7) +
    ggplot2::geom_vline(xintercept = cor_obs, color = "red", linewidth = 1.5) +
    ggplot2::labs(
      x = "Residual Correlation",
      y = "Count",
      title = "Posterior Predictive Check",
      subtitle = "Red line: observed; Histogram: replicated data"
    ) +
    ggplot2::theme_minimal()
}


#' Plot HMM state posteriors
#'
#' @param object A `dcvar_hmm_fit` object.
#' @param show_viterbi Logical; overlay the Viterbi (MAP) state sequence
#'   (default: `TRUE`).
#' @param ... Additional arguments (unused).
#'
#' @return A ggplot object.
#' @export
plot_hmm_states <- function(object, show_viterbi = TRUE, ...) {
  states <- hmm_states(object)
  K <- object$K
  T_eff <- nrow(states$gamma)
  time_values <- .observed_time_values(object$stan_data, drop_first = TRUE)

  # Build long-format data for gamma. Pin the state levels so the fill and
  # Viterbi color scales stay aligned even when the MAP path skips a state.
  state_levels <- paste0("State ", 1:K)
  gamma_df <- data.frame(
    time = rep(time_values, K),
    state = factor(rep(state_levels, each = T_eff), levels = state_levels),
    probability = as.vector(states$gamma)
  )

  p <- ggplot2::ggplot(gamma_df, ggplot2::aes(x = .data$time, y = .data$probability,
                                                fill = .data$state)) +
    ggplot2::geom_area(alpha = 0.7, position = "stack") +
    ggplot2::scale_fill_brewer(palette = "Set2", limits = state_levels) +
    ggplot2::labs(
      x = "Time", y = "State Probability",
      title = "HMM State Posteriors",
      fill = "State"
    ) +
    ggplot2::theme_minimal() +
    ggplot2::theme(
      plot.title = ggplot2::element_text(face = "bold"),
      legend.position = "bottom"
    )

  if (show_viterbi) {
    viterbi_df <- data.frame(
      time = time_values,
      state = factor(paste0("State ", states$viterbi), levels = state_levels),
      y = 0.5
    )
    p <- p + ggplot2::geom_point(
      data = viterbi_df,
      ggplot2::aes(x = .data$time, y = .data$y, color = .data$state),
      size = 0.5, alpha = 0.5, inherit.aes = FALSE
    ) +
      ggplot2::scale_color_brewer(palette = "Set2", limits = state_levels, guide = "none")
  }

  p
}


#' Plot random effects (caterpillar plot)
#'
#' Displays unit-specific VAR coefficients with credible intervals.
#'
#' @param object A `dcvar_multilevel_fit` object.
#' @param ... Additional arguments (unused).
#'
#' @return A ggplot object.
#' @export
plot_random_effects <- function(object, ...) {
  re <- random_effects(object)

  re$unit <- factor(re$unit)

  ggplot2::ggplot(re, ggplot2::aes(
    x = .data$unit, y = .data$mean,
    ymin = .data$q2.5, ymax = .data$q97.5
  )) +
    ggplot2::geom_pointrange(size = 0.3, color = "steelblue") +
    ggplot2::geom_hline(yintercept = 0, linetype = "dashed", color = "gray50") +
    ggplot2::facet_wrap(~ parameter, scales = "free_y") +
    ggplot2::labs(
      x = "Unit",
      y = "Posterior mean [95% CrI]",
      title = "Unit-Specific VAR Coefficients"
    ) +
    ggplot2::theme_minimal() +
    ggplot2::theme(
      plot.title = ggplot2::element_text(face = "bold"),
      axis.text.x = ggplot2::element_text(size = 6)
    )
}


#' Plot estimated latent states with credible intervals
#'
#' @param object A `dcvar_sem_fit` object.
#' @param true_states Optional T x 2 matrix of true latent states for overlay.
#' @param ... Additional arguments (unused).
#'
#' @return A ggplot object.
#' @export
plot_latent_states <- function(object, true_states = NULL, ...) {
  states_df <- latent_states(object)

  p <- ggplot2::ggplot(states_df, ggplot2::aes(x = .data$time)) +
    ggplot2::geom_ribbon(
      ggplot2::aes(ymin = .data$q2.5, ymax = .data$q97.5),
      fill = "steelblue", alpha = 0.2
    ) +
    ggplot2::geom_line(
      ggplot2::aes(y = .data$mean),
      color = "steelblue", linewidth = 0.8
    ) +
    ggplot2::facet_wrap(~ variable, scales = "free_y") +
    ggplot2::labs(
      x = "Time", y = "Latent State",
      title = "Estimated Latent States",
      subtitle = "95% credible intervals"
    ) +
    ggplot2::theme_minimal() +
    ggplot2::theme(
      plot.title = ggplot2::element_text(face = "bold")
  )

  if (!is.null(true_states)) {
    time_values <- .observed_time_values(object$stan_data)
    if (!is.matrix(true_states) || ncol(true_states) != 2) {
      cli_abort("{.arg true_states} must be a matrix with 2 columns.")
    }
    if (nrow(true_states) != length(time_values)) {
      cli_abort(c(
        "{.arg true_states} must have one row per observed time point.",
        "i" = "Use the fit's stored time values for simulation overlays."
      ))
    }
    true_df <- data.frame(
      time = rep(time_values, 2),
      variable = rep(object$vars, each = nrow(true_states)),
      value = c(true_states[, 1], true_states[, 2])
    )
    p <- p + ggplot2::geom_line(
      data = true_df,
      ggplot2::aes(x = .data$time, y = .data$value),
      color = "red", linewidth = 0.5, linetype = "dashed"
    )
  }

  p
}


#' Internal: plot transition matrix as a heatmap
#' @noRd
.plot_transition_matrix <- function(A, K) {
  A_df <- expand.grid(from = 1:K, to = 1:K)
  A_df$prob <- as.vector(A)
  A_df$label <- sprintf("%.3f", A_df$prob)
  A_df$from <- paste0("State ", A_df$from)
  A_df$to <- paste0("State ", A_df$to)

  ggplot2::ggplot(A_df, ggplot2::aes(x = .data$from, y = .data$to)) +
    ggplot2::geom_tile(ggplot2::aes(fill = .data$prob), color = "white") +
    ggplot2::geom_text(ggplot2::aes(label = .data$label), size = 4) +
    ggplot2::scale_fill_gradient(low = "white", high = "steelblue", limits = c(0, 1)) +
    ggplot2::labs(
      x = "From State", y = "To State",
      fill = "P(transition)",
      title = "Transition Matrix"
    ) +
    ggplot2::theme_minimal() +
    ggplot2::coord_fixed()
}


# ============================================================================
# Time-varying trajectory plots (dcvar_tv_fit)
# ============================================================================

#' Internal: faceted ribbon plot shared by the Phi(t)/sigma(t) trajectories
#' @noRd
.plot_tv_trajectory <- function(object, trajectory_fun, group_col,
                                show_ci, ci_level, inner_level,
                                true_values, true_arg_name,
                                title, ylab, zero_line) {
  .validate_interval_level(ci_level, arg_name = "ci_level")
  if (!is.null(inner_level)) {
    .validate_interval_level(inner_level, arg_name = "inner_level")
    if (inner_level >= ci_level) {
      cli_abort(c(
        "{.arg inner_level} must be smaller than {.arg ci_level}.",
        "i" = "The inner ribbon must be narrower than the outer ribbon."
      ))
    }
  }

  alpha_outer <- (1 - ci_level) / 2
  probs <- c(alpha_outer, 1 - alpha_outer, 0.5)
  if (!is.null(inner_level)) {
    alpha_inner <- (1 - inner_level) / 2
    probs <- c(alpha_outer, alpha_inner, 0.5, 1 - alpha_inner, 1 - alpha_outer)
  }
  probs <- sort(unique(probs))

  df <- trajectory_fun(object, probs = probs)
  groups <- unique(df[[group_col]])

  if (!is.null(true_values)) {
    time_values <- .observed_time_values(object$stan_data, drop_first = TRUE)
    true_values <- as.matrix(true_values)
    if (nrow(true_values) != length(time_values) || ncol(true_values) != length(groups)) {
      cli_abort(c(
        "{.arg {true_arg_name}} must be a {length(time_values)} x {length(groups)} matrix.",
        "i" = "Rows follow the observed time axis (transitions 2..n_time); columns follow {.val {groups}}."
      ))
    }
    true_df <- data.frame(
      time = rep(time_values, length(groups)),
      value = as.vector(true_values),
      group = rep(groups, each = length(time_values))
    )
    names(true_df)[names(true_df) == "group"] <- group_col
  }

  q_outer_low <- paste0("q", alpha_outer * 100)
  q_outer_high <- paste0("q", (1 - alpha_outer) * 100)

  p <- ggplot2::ggplot(df, ggplot2::aes(x = .data$time))
  if (zero_line) {
    p <- p + ggplot2::geom_hline(yintercept = 0, linetype = "dashed", color = "gray50")
  }

  if (show_ci && q_outer_low %in% names(df)) {
    p <- p + ggplot2::geom_ribbon(
      ggplot2::aes(ymin = .data[[q_outer_low]], ymax = .data[[q_outer_high]]),
      fill = "steelblue", alpha = 0.2
    )
    if (!is.null(inner_level)) {
      q_inner_low <- paste0("q", alpha_inner * 100)
      q_inner_high <- paste0("q", (1 - alpha_inner) * 100)
      if (q_inner_low %in% names(df) && q_inner_high %in% names(df)) {
        p <- p + ggplot2::geom_ribbon(
          ggplot2::aes(ymin = .data[[q_inner_low]], ymax = .data[[q_inner_high]]),
          fill = "steelblue", alpha = 0.3
        )
      }
    }
  }

  p <- p +
    ggplot2::geom_line(ggplot2::aes(y = .data$mean), color = "steelblue", linewidth = 1)

  if ("q50" %in% names(df)) {
    p <- p +
      ggplot2::geom_line(ggplot2::aes(y = .data$q50), color = "darkblue", linetype = "dashed")
  }

  if (!is.null(true_values)) {
    p <- p + ggplot2::geom_line(
      data = true_df,
      ggplot2::aes(x = .data$time, y = .data$value),
      color = "red", linewidth = 0.8
    )
  }

  p +
    ggplot2::facet_wrap(stats::as.formula(paste("~", group_col))) +
    ggplot2::labs(x = "Time", y = ylab, title = title) +
    ggplot2::theme_minimal() +
    ggplot2::theme(
      plot.title = ggplot2::element_text(face = "bold"),
      strip.text = ggplot2::element_text(face = "bold")
    )
}


#' Plot the time-varying VAR coefficient trajectories
#'
#' Faceted ribbon plot of the posterior phi11(t), phi12(t), phi21(t),
#' phi22(t) trajectories from a time-varying DC-VAR fit (see
#' [phi_trajectory()]).
#'
#' @param object A `dcvar_tv_fit` object.
#' @param show_ci Logical; draw credible ribbons (default: `TRUE`).
#' @param ci_level Outer credible level (default: 0.95).
#' @param inner_level Inner credible level, or `NULL` to skip the inner
#'   ribbon (default: 0.80).
#' @param true_phi Optional `(n_time - 1) x 4` matrix of true coefficient
#'   paths (columns in phi11/phi12/phi21/phi22 order) overlaid in red, e.g.
#'   from `simulate_dcvar()`'s `true_params`.
#' @param title Plot title, or `NULL` for the default.
#' @param ... Additional arguments (unused).
#'
#' @return A ggplot object.
#' @seealso [phi_trajectory()], [plot_rho()], [plot_sigma_trajectory()]
#' @export
plot_phi_trajectory <- function(object, show_ci = TRUE, ci_level = 0.95,
                                inner_level = 0.80, true_phi = NULL,
                                title = NULL, ...) {
  if (is.null(title)) {
    title <- "Time-Varying VAR(1) Coefficients"
  }
  .plot_tv_trajectory(
    object, phi_trajectory, "coefficient",
    show_ci = show_ci, ci_level = ci_level, inner_level = inner_level,
    true_values = true_phi, true_arg_name = "true_phi",
    title = title, ylab = "Coefficient", zero_line = TRUE
  )
}


#' Plot the time-varying residual scale trajectories
#'
#' Faceted ribbon plot of the posterior per-variable residual scale paths
#' from a time-varying DC-VAR fit (see [sigma_trajectory()] for the
#' per-family scale semantics).
#'
#' @param object A `dcvar_tv_fit` object.
#' @param show_ci Logical; draw credible ribbons (default: `TRUE`).
#' @param ci_level Outer credible level (default: 0.95).
#' @param inner_level Inner credible level, or `NULL` to skip the inner
#'   ribbon (default: 0.80).
#' @param true_sigma Optional `(n_time - 1) x 2` matrix of true scale paths
#'   (columns in variable order) overlaid in red.
#' @param title Plot title, or `NULL` for the default.
#' @param ... Additional arguments (unused).
#'
#' @return A ggplot object.
#' @seealso [sigma_trajectory()], [plot_phi_trajectory()]
#' @export
plot_sigma_trajectory <- function(object, show_ci = TRUE, ci_level = 0.95,
                                  inner_level = 0.80, true_sigma = NULL,
                                  title = NULL, ...) {
  if (is.null(title)) {
    title <- "Time-Varying Residual Scales"
  }
  .plot_tv_trajectory(
    object, sigma_trajectory, "variable",
    show_ci = show_ci, ci_level = ci_level, inner_level = inner_level,
    true_values = true_sigma, true_arg_name = "true_sigma",
    title = title, ylab = "Scale", zero_line = FALSE
  )
}

test_that("plot_rho() returns ggplot for dcvar", {
  skip_if_no_rstan()

  fit <- get_dcvar_fit()
  p <- plot_rho(fit)
  expect_s3_class(p, "ggplot")
})

test_that("plot_rho() returns ggplot for hmm", {
  skip_if_no_rstan()

  fit <- get_hmm_fit()
  p <- plot_rho(fit)
  expect_s3_class(p, "ggplot")
})

test_that("plot_rho() returns ggplot for constant", {
  skip_if_no_rstan()

  fit <- get_constant_fit()
  p <- plot_rho(fit)
  expect_s3_class(p, "ggplot")
})

test_that("plot_rho() rejects invalid interval levels", {
  fit <- structure(
    list(model = "dcvar", stan_data = list(n_time = 5)),
    class = "dcvar_fit"
  )
  expect_error(plot_rho(fit, ci_level = 0), "ci_level")
  expect_error(plot_rho(fit, ci_level = 0.9, inner_level = 0.9), "inner_level")
  expect_error(plot_rho(fit, ci_level = 0.9, inner_level = 1), "inner_level")
})

test_that("plot_phi() returns ggplot", {
  skip_if_no_rstan()

  fit <- get_dcvar_fit()
  p <- plot_phi(fit)
  expect_s3_class(p, "ggplot")
})

test_that("plot_diagnostics() works for dcvar", {
  skip_if_no_rstan()

  fit <- get_dcvar_fit()
  p <- plot_diagnostics(fit)
  expect_true(inherits(p, "patchwork") || inherits(p, "ggplot"))
})

test_that("plot_diagnostics() works for hmm", {
  skip_if_no_rstan()

  fit <- get_hmm_fit()
  p <- plot_diagnostics(fit)
  expect_true(inherits(p, "patchwork") || inherits(p, "ggplot"))
})

test_that("plot_diagnostics() works for constant", {
  skip_if_no_rstan()

  fit <- get_constant_fit()
  p <- plot_diagnostics(fit)
  expect_true(inherits(p, "patchwork") || inherits(p, "ggplot"))
})

test_that("plot_hmm_states() returns ggplot with expected structure", {
  skip_if_no_rstan()

  fit <- get_hmm_fit()
  p <- plot_hmm_states(fit)
  expect_s3_class(p, "ggplot")
  # Verify the plot has data
  expect_true(nrow(p$data) > 0)
})

test_that("plot_rho() uses preserved time values for true_rho overlays", {
  fit <- structure(
    list(model = "dcvar", stan_data = list(n_time = 5)),
    class = "dcvar_fit"
  )
  time_values <- seq.Date(
    as.Date("2021-01-01"),
    by = "day",
    length.out = fit$stan_data$n_time
  )
  fit$stan_data <- structure(fit$stan_data, time_values = time_values)

  testthat::local_mocked_bindings(
    rho_trajectory = function(object, probs = c(0.025, 0.1, 0.5, 0.9, 0.975), ...) {
      time_axis <- .observed_time_values(object$stan_data, drop_first = TRUE)
      data.frame(
        time = time_axis,
        mean = rep(0.2, length(time_axis)),
        sd = rep(0.05, length(time_axis)),
        q2.5 = rep(0.1, length(time_axis)),
        q10 = rep(0.15, length(time_axis)),
        q50 = rep(0.2, length(time_axis)),
        q90 = rep(0.25, length(time_axis)),
        q97.5 = rep(0.3, length(time_axis))
      )
    },
    .package = "dcvar"
  )

  true_rho <- rep(0.2, fit$stan_data$n_time - 1)
  p <- plot_rho(fit, true_rho = true_rho)
  overlay_data <- p$layers[[length(p$layers)]]$data

  expect_equal(overlay_data$time, time_values[-1])
  expect_equal(overlay_data$rho, true_rho)
})

test_that("plot_latent_states() uses preserved time values for true_states overlays", {
  fit <- structure(
    list(
      vars = c("latent1", "latent2"),
      stan_data = list(n_time = 5)
    ),
    class = "dcvar_sem_fit"
  )
  time_values <- seq.Date(
    as.Date("2021-01-01"),
    by = "day",
    length.out = fit$stan_data$n_time
  )
  fit$stan_data <- structure(fit$stan_data, time_values = time_values)

  testthat::local_mocked_bindings(
    latent_states = function(object, probs = c(0.025, 0.5, 0.975), ...) {
      data.frame(
        time = rep(time_values, 2),
        variable = rep(object$vars, each = length(time_values)),
        mean = rep(0, 2 * length(time_values)),
        sd = rep(1, 2 * length(time_values)),
        q2.5 = rep(-1, 2 * length(time_values)),
        q50 = rep(0, 2 * length(time_values)),
        q97.5 = rep(1, 2 * length(time_values))
      )
    },
    .package = "dcvar"
  )

  true_states <- cbind(
    latent1 = seq_len(fit$stan_data$n_time),
    latent2 = seq_len(fit$stan_data$n_time) * 2
  )
  p <- plot_latent_states(fit, true_states = true_states)
  overlay_data <- p$layers[[length(p$layers)]]$data

  expect_equal(overlay_data$time, rep(time_values, 2))
  expect_equal(overlay_data$value, c(true_states[, 1], true_states[, 2]))
})

test_that("plot_ppc() returns ggplot for dcvar", {
  skip_if_no_rstan()

  fit <- get_dcvar_fit()
  expect_no_warning(p <- plot_ppc(fit))
  expect_s3_class(p, "ggplot")
  expect_false(all(is.na(p$data$cor)))
})

test_that("plot_ppc() returns ggplot for hmm", {
  skip_if_no_rstan()

  fit <- get_hmm_fit()
  expect_no_warning(p <- plot_ppc(fit))
  expect_s3_class(p, "ggplot")
  expect_false(all(is.na(p$data$cor)))
})

test_that("plot_ppc() returns ggplot for constant", {
  skip_if_no_rstan()

  fit <- get_constant_fit()
  expect_no_warning(p <- plot_ppc(fit))
  expect_s3_class(p, "ggplot")
  expect_false(all(is.na(p$data$cor)))
})

test_that("plot_ppc() works for exponential fits", {
  skip_if_no_rstan()

  fit <- get_dcvar_exponential_fit()
  expect_no_warning(p <- plot_ppc(fit))
  expect_s3_class(p, "ggplot")
  expect_false(all(is.na(p$data$cor)))
})

test_that("plot_ppc() rejects unsupported non-normal margins", {
  gamma_fit <- structure(list(margins = "gamma"), class = "dcvar_fit")
  skew_fit <- structure(list(margins = "skew_normal"), class = "dcvar_fit")

  expect_error(plot_ppc(gamma_fit), "not supported")
  expect_error(plot_ppc(skew_fit), "not supported")
})

test_that("plot_ppc() rejects unsupported fitted gamma and skew-normal objects", {
  skip_if_no_rstan()

  expect_error(plot_ppc(get_constant_gamma_fit()), "not supported")

  skip_if_not_installed("sn")
  expect_error(plot_ppc(get_constant_skew_normal_fit()), "not supported")
})

test_that("plot methods fail clearly when required Stan outputs are missing", {
  make_stub_draws <- function(variables) {
    posterior::as_draws_array(
      array(
        seq_len(4L * length(variables)),
        dim = c(4L, 1L, length(variables)),
        dimnames = list(NULL, NULL, variables)
      )
    )
  }

  phi_fit <- structure(
    list(
      fit = make_stub_draws(c("mu[1]", "mu[2]")),
      model = "dcvar",
      vars = c("y1", "y2"),
      margins = "normal",
      stan_data = list(n_time = 5, D = 2),
      backend = "rstan",
      meta = list(iter_sampling = 10, chains = 1)
    ),
    class = c("dcvar_fit", "dcvar_model_fit")
  )
  expect_error(plot_phi(phi_fit), "Custom Stan files must preserve")

  ppc_fit <- structure(
    list(
      fit = make_stub_draws(c("eps[1,1]", "eps[1,2]", "eps[2,1]", "eps[2,2]")),
      model = "dcvar",
      vars = c("y1", "y2"),
      margins = "normal",
      stan_data = list(n_time = 3, D = 2),
      backend = "rstan",
      meta = list(iter_sampling = 10, chains = 1)
    ),
    class = c("dcvar_fit", "dcvar_model_fit")
  )
  expect_error(plot_ppc(ppc_fit), "Custom Stan files must preserve")
})

test_that("plot_trajectories() returns ggplot", {
  p <- plot_trajectories(50)
  expect_s3_class(p, "ggplot")
  expect_true(nrow(p$data) > 0)
})

test_that("plot_trajectories() works with subset of scenarios", {
  p <- plot_trajectories(50, scenarios = c("constant", "decreasing"))
  expect_s3_class(p, "ggplot")
})

test_that("plot(constant_fit, type = 'rho') works", {
  skip_if_no_rstan()
  skip_if_not_installed("ggplot2")
  fit <- get_constant_fit()
  p <- plot(fit, type = "rho")
  expect_s3_class(p, "gg")
})

# ============================================================================
# Trajectory Generation Functions
# ============================================================================

#' Internal: validate a rho scalar in [-1, 1]
#' @noRd
.trajectory_validate_rho_scalar <- function(x, arg_name) {
  if (!is.numeric(x) || length(x) != 1L || !is.finite(x) || x < -1 || x > 1) {
    cli_abort("{.arg {arg_name}} must be a single finite numeric value in [-1, 1].")
  }
}

#' Internal: validate a rho level vector in [-1, 1]
#' @noRd
.trajectory_validate_rho_levels <- function(x, arg_name) {
  if (!is.numeric(x) || length(x) != 3L || any(!is.finite(x)) || any(x < -1 | x > 1)) {
    cli_abort("{.arg {arg_name}} must be a length-3 finite numeric vector in [-1, 1].")
  }
}

#' Internal: validate and resolve a breakpoint specification
#' @noRd
.trajectory_resolve_breakpoint <- function(breakpoint, T_eff, arg_name = "breakpoint") {
  if (!is.numeric(breakpoint) || length(breakpoint) != 1L || !is.finite(breakpoint)) {
    cli_abort("{.arg {arg_name}} must be a single finite numeric value.")
  }

  if (breakpoint >= 0 && breakpoint <= 1) {
    bp_time <- round(breakpoint * T_eff)
  } else {
    if (breakpoint != as.integer(breakpoint) || breakpoint < 1) {
      cli_abort(c(
        "{.arg {arg_name}} must be either a proportion in [0, 1] or a positive integer breakpoint.",
        "i" = "Use a whole number for an absolute breakpoint index."
      ))
    }
    bp_time <- as.integer(breakpoint)
  }

  max(1L, min(T_eff - 1L, as.integer(bp_time)))
}

#' Internal: validate a non-negative transition width
#' @noRd
.trajectory_validate_transition_width <- function(x, arg_name = "transition_width") {
  if (!is.numeric(x) || length(x) != 1L || !is.finite(x) || x < 0) {
    cli_abort("{.arg {arg_name}} must be a single non-negative finite numeric value.")
  }
}

#' Generate a constant rho trajectory
#'
#' @param n_time Number of time points.
#' @param rho Constant correlation value (default: 0.5). Must be in \[-1, 1\].
#'
#' @return Numeric vector of length `n_time - 1`.
#' @export
#'
#' @examples
#' rho_constant(100, rho = 0.5)
rho_constant <- function(n_time, rho = 0.5) {
  if (n_time < 2) cli_abort("{.arg n_time} must be >= 2, got {.val {n_time}}.")
  .trajectory_validate_rho_scalar(rho, "rho")
  rep(rho, n_time - 1)
}


#' Generate a logistically decreasing rho trajectory
#'
#' Mimics a therapy effect where coupling decreases from high to low.
#'
#' @param n_time Number of time points.
#' @param rho_start Starting rho value (default: 0.7). Must be in \[-1, 1\].
#' @param rho_end Ending rho value (default: 0.3). Must be in \[-1, 1\].
#' @param midpoint Time point of inflection (default: `n_time/2`).
#' @param steepness Controls transition sharpness (default: 0.05).
#'
#' @return Numeric vector of length `n_time - 1`.
#' @export
#'
#' @examples
#' rho_decreasing(100)
rho_decreasing <- function(n_time, rho_start = 0.7, rho_end = 0.3,
                            midpoint = NULL, steepness = 0.05) {
  if (n_time < 2) cli_abort("{.arg n_time} must be >= 2, got {.val {n_time}}.")
  .trajectory_validate_rho_scalar(rho_start, "rho_start")
  .trajectory_validate_rho_scalar(rho_end, "rho_end")
  if (is.null(midpoint)) midpoint <- n_time / 2
  t_vec <- seq_len(n_time - 1L)
  rho_end + (rho_start - rho_end) / (1 + exp(steepness * (t_vec - midpoint)))
}


#' Generate a logistically increasing rho trajectory
#'
#' Mimics deterioration where coupling increases from low to high.
#'
#' @inheritParams rho_decreasing
#' @param rho_start Starting rho value (default: 0.3). Must be in \[-1, 1\].
#' @param rho_end Ending rho value (default: 0.7). Must be in \[-1, 1\].
#'
#' @return Numeric vector of length `n_time - 1`.
#' @export
#'
#' @examples
#' rho_increasing(100)
rho_increasing <- function(n_time, rho_start = 0.3, rho_end = 0.7,
                            midpoint = NULL, steepness = 0.05) {
  if (n_time < 2) cli_abort("{.arg n_time} must be >= 2, got {.val {n_time}}.")
  .trajectory_validate_rho_scalar(rho_start, "rho_start")
  .trajectory_validate_rho_scalar(rho_end, "rho_end")
  if (is.null(midpoint)) midpoint <- n_time / 2
  t_vec <- seq_len(n_time - 1L)
  rho_start + (rho_end - rho_start) / (1 + exp(-steepness * (t_vec - midpoint)))
}


#' Generate a random walk rho trajectory on the Fisher-z scale
#'
#' Stochastic trajectory matching the DC-VAR data-generating process.
#'
#' @param n_time Number of time points.
#' @param z_init Initial value on Fisher-z scale (default: 0.5, corresponding
#'   to rho = 0.46).
#' @param sigma_omega Innovation SD for the random walk (default: 0.05).
#' @param seed Random seed for reproducibility.
#'
#' @return Numeric vector of length `n_time - 1`.
#' @export
#'
#' @examples
#' rho_random_walk(100, seed = 42)
rho_random_walk <- function(n_time, z_init = 0.5, sigma_omega = 0.05, seed = NULL) {
  if (n_time < 2) cli_abort("{.arg n_time} must be >= 2, got {.val {n_time}}.")
  if (!is.null(seed)) set.seed(seed)
  if (sigma_omega < 0) {
    cli_abort("{.arg sigma_omega} must be non-negative, got {.val {sigma_omega}}.")
  }

  z <- numeric(n_time - 1L)
  z[1] <- z_init + rnorm(1, 0, sigma_omega)
  if (n_time > 2) {
    for (time_index in 2:(n_time - 1L)) {
      z[time_index] <- z[time_index - 1L] + rnorm(1, 0, sigma_omega)
    }
  }

  # Clamp extreme z values to avoid rho numerically equal to +/- 1
  z <- pmax(pmin(z, 5), -5)

  tanh(z)
}


#' Generate a single-breakpoint (step function) rho trajectory
#'
#' Abrupt change from one rho level to another at a specified time.
#'
#' @param n_time Number of time points.
#' @param rho_before Rho before breakpoint (default: 0.7).
#' @param rho_after Rho after breakpoint (default: 0.3).
#' @param breakpoint Breakpoint location as a proportion of `n_time - 1` (if <= 1)
#'   or an absolute time index (default: 0.5).
#' @param transition_width Number of time points for smooth transition.
#'   0 = abrupt (default: 0).
#'
#' @return Numeric vector of length `n_time - 1`.
#' @export
#'
#' @examples
#' rho_step(100, rho_before = 0.7, rho_after = 0.3)
rho_step <- function(n_time, rho_before = 0.7, rho_after = 0.3,
                      breakpoint = 0.5, transition_width = 0) {
  if (n_time < 2) cli_abort("{.arg n_time} must be >= 2, got {.val {n_time}}.")
  .trajectory_validate_rho_scalar(rho_before, "rho_before")
  .trajectory_validate_rho_scalar(rho_after, "rho_after")
  n_time_eff <- n_time - 1L
  .trajectory_validate_transition_width(transition_width)
  bp_time <- .trajectory_resolve_breakpoint(breakpoint, n_time_eff)

  t_vec <- seq_len(n_time_eff)

  if (transition_width == 0) {
    ifelse(t_vec <= bp_time, rho_before, rho_after)
  } else {
    steep <- 10 / transition_width
    rho_after + (rho_before - rho_after) / (1 + exp(steep * (t_vec - bp_time)))
  }
}


#' Generate a double-breakpoint (relapse pattern) rho trajectory
#'
#' Three-phase trajectory: level A -> level B -> level C.
#'
#' @param n_time Number of time points.
#' @param rho_levels Numeric vector of three rho levels (default:
#'   `c(0.7, 0.3, 0.7)`).
#' @param breakpoints Numeric vector of two breakpoint positions (default:
#'   `c(1/3, 2/3)`). Interpreted as proportions of `n_time - 1` if <= 1.
#' @param transition_width Number of time points for smooth transitions
#'   (default: 0).
#'
#' @return Numeric vector of length `n_time - 1`.
#' @export
#'
#' @examples
#' rho_double_step(100, rho_levels = c(0.7, 0.3, 0.7))
rho_double_step <- function(n_time, rho_levels = c(0.7, 0.3, 0.7),
                             breakpoints = c(1 / 3, 2 / 3),
                             transition_width = 0) {
  if (n_time < 2) cli_abort("{.arg n_time} must be >= 2, got {.val {n_time}}.")
  .trajectory_validate_rho_levels(rho_levels, "rho_levels")
  if (!is.numeric(breakpoints) || length(breakpoints) != 2L || any(!is.finite(breakpoints))) {
    cli_abort("{.arg breakpoints} must be a length-2 finite numeric vector.")
  }
  .trajectory_validate_transition_width(transition_width)
  n_time_eff <- n_time - 1L

  bp1 <- .trajectory_resolve_breakpoint(breakpoints[1], n_time_eff, "breakpoints[1]")
  bp2 <- .trajectory_resolve_breakpoint(breakpoints[2], n_time_eff, "breakpoints[2]")
  if (bp2 <= bp1) {
    cli_abort(c(
      "{.arg breakpoints} must resolve to two strictly increasing breakpoint positions.",
      "i" = "Choose distinct breakpoint values so the middle phase is non-empty."
    ))
  }

  t_vec <- seq_len(n_time_eff)

  if (transition_width == 0) {
    ifelse(t_vec <= bp1, rho_levels[1],
           ifelse(t_vec <= bp2, rho_levels[2], rho_levels[3]))
  } else {
    steep <- 10 / transition_width
    trans1 <- rho_levels[2] + (rho_levels[1] - rho_levels[2]) / (1 + exp(steep * (t_vec - bp1)))
    trans2 <- rho_levels[3] + (rho_levels[2] - rho_levels[3]) / (1 + exp(steep * (t_vec - bp2)))
    ifelse(t_vec <= (bp1 + bp2) / 2, trans1, trans2)
  }
}


#' Get a named trajectory scenario
#'
#' Convenience function to retrieve a standard scenario by name.
#'
#' @param scenario Character string. One of:
#'   - Smooth: `"constant"`, `"decreasing"`, `"increasing"`, `"random_walk"`
#'   - Step: `"single_middle"`, `"large_change"`, `"small_change"`,
#'     `"increase"`, `"double_relapse"`
#' @param n_time Number of time points.
#' @param ... Additional arguments passed to the generator.
#'
#' @return Numeric vector of length `n_time - 1`.
#' @export
#'
#' @examples
#' rho_scenario("decreasing", n_time = 100)
#' rho_scenario("double_relapse", n_time = 150)
rho_scenario <- function(scenario, n_time, ...) {
  if (!scenario %in% names(.rho_scenarios)) {
    cli_abort("Unknown scenario: {.val {scenario}}. Available: {.val {names(.rho_scenarios)}}")
  }

  scenario_spec <- .rho_scenarios[[scenario]]
  args <- utils::modifyList(scenario_spec$defaults, list(...))
  do.call(scenario_spec$fn, c(list(n_time = n_time), args))
}

#' Registry of named trajectory scenarios (shared with plot_trajectories)
#' @noRd
.rho_scenarios <- list(
  constant = list(fn = rho_constant, defaults = list(rho = 0.5)),
  decreasing = list(fn = rho_decreasing, defaults = list(rho_start = 0.7, rho_end = 0.3)),
  increasing = list(fn = rho_increasing, defaults = list(rho_start = 0.3, rho_end = 0.7)),
  random_walk = list(fn = rho_random_walk, defaults = list(z_init = 0.5, sigma_omega = 0.05)),
  single_middle = list(fn = rho_step, defaults = list(rho_before = 0.7, rho_after = 0.3, breakpoint = 0.5)),
  large_change = list(fn = rho_step, defaults = list(rho_before = 0.8, rho_after = 0.2, breakpoint = 0.5)),
  small_change = list(fn = rho_step, defaults = list(rho_before = 0.6, rho_after = 0.4, breakpoint = 0.5)),
  increase = list(fn = rho_step, defaults = list(rho_before = 0.3, rho_after = 0.7, breakpoint = 0.5)),
  double_relapse = list(fn = rho_double_step, defaults = list(rho_levels = c(0.7, 0.3, 0.7)))
)

test_that("simulate_dcvar returns correct dimensions", {
  sim <- simulate_dcvar(n_time = 50, rho_trajectory = rho_constant(50, 0.5))

  expect_equal(nrow(sim$Y), 50)
  expect_equal(ncol(sim$Y), 2)
  expect_equal(nrow(sim$Y_df), 50)
  expect_true(all(c("time", "y1", "y2") %in% names(sim$Y_df)))
})

test_that("simulate_dcvar rejects wrong rho_trajectory length", {
  expect_error(
    simulate_dcvar(n_time = 50, rho_trajectory = rep(0.5, 10)),
    "length n_time - 1"
  )
})

test_that("simulate_dcvar validates T >= 2", {
  expect_error(
    simulate_dcvar(n_time = 1, rho_trajectory = numeric(0)),
    "integer >= 2"
  )
})

test_that("simulate_dcvar validates sigma_eps length", {
  expect_error(
    simulate_dcvar(n_time = 50, rho_trajectory = rho_constant(50, 0.5),
                   sigma_eps = c(1, 1, 1)),
    "sigma_eps"
  )
})

test_that("simulate_dcvar validates gamma shape", {
  expect_error(
    simulate_dcvar(
      n_time = 50,
      rho_trajectory = rho_constant(50, 0.5),
      margins = "gamma",
      skew_direction = c(1, 1),
      skew_params = list(shape = 0)
    ),
    "shape"
  )
})

test_that("simulate_dcvar is reproducible with seed", {
  traj <- rho_constant(50, 0.5)
  s1 <- simulate_dcvar(n_time = 50, rho_trajectory = traj, seed = 42)
  s2 <- simulate_dcvar(n_time = 50, rho_trajectory = traj, seed = 42)
  expect_identical(s1$Y, s2$Y)
})

test_that("simulate_dcvar stores true parameters", {
  traj <- rho_decreasing(50)
  sim <- simulate_dcvar(n_time = 50, rho_trajectory = traj)

  expect_identical(sim$true_params$rho, traj)
  expect_equal(sim$true_params$mu, c(0, 0))
  expect_equal(dim(sim$true_params$Phi), c(2, 2))
  expect_equal(sim$true_params$sigma_eps, c(1, 1))
})

test_that("simulate_dcvar stores default non-normal margin parameters in true_params", {
  traj <- rho_constant(30, 0.2)

  gamma_sim <- simulate_dcvar(
    n_time = 30,
    rho_trajectory = traj,
    margins = "gamma",
    skew_direction = c(1, 1)
  )
  expect_equal(gamma_sim$true_params$skew_params$shape, 1)

  skew_sim <- simulate_dcvar(
    n_time = 30,
    rho_trajectory = traj,
    margins = "skew_normal"
  )
  expect_equal(skew_sim$true_params$skew_params$alpha, c(0, 0))
})

test_that("simulate_dcvar Y_df time column is 1:T", {
  sim <- simulate_dcvar(n_time = 30, rho_trajectory = rho_constant(30, 0.3))
  expect_equal(sim$Y_df$time, 1:30)
})

test_that("rho_constant returns correct length and value", {
  result <- rho_constant(100, rho = 0.5)
  expect_equal(length(result), 99)
  expect_true(all(result == 0.5))
})

test_that("rho_constant rejects out-of-range rho", {
  expect_error(rho_constant(100, rho = 1.5))
  expect_error(rho_constant(100, rho = -1.5))
})

test_that("rho_decreasing returns correct length and trend", {
  result <- rho_decreasing(100)
  expect_equal(length(result), 99)
  expect_true(result[1] > result[99])
})

test_that("rho_increasing returns correct length and trend", {
  result <- rho_increasing(100)
  expect_equal(length(result), 99)
  expect_true(result[1] < result[99])
})

test_that("rho_random_walk returns correct length", {
  result <- rho_random_walk(100, seed = 42)
  expect_equal(length(result), 99)
  expect_true(all(result > -1 & result < 1))
})

test_that("rho_random_walk is reproducible with seed", {
  r1 <- rho_random_walk(50, seed = 123)
  r2 <- rho_random_walk(50, seed = 123)
  expect_identical(r1, r2)
})

test_that("rho_step returns correct length", {
  result <- rho_step(100)
  expect_equal(length(result), 99)
})

test_that("rho_step has correct levels before and after breakpoint", {
  result <- rho_step(101, rho_before = 0.8, rho_after = 0.2, breakpoint = 0.5)
  expect_equal(result[1], 0.8)
  expect_equal(result[99], 0.2)
})

test_that("rho_step validates rho bounds and breakpoint arguments", {
  expect_error(rho_step(100, rho_before = 1.2), "\\[-1, 1\\]")
  expect_error(rho_step(100, rho_after = -1.2), "\\[-1, 1\\]")
  expect_error(rho_step(100, breakpoint = -0.1), "breakpoint")
  expect_error(rho_step(100, breakpoint = 1.4), "breakpoint")
  expect_error(rho_step(100, transition_width = -1), "transition_width")
})

test_that("rho_double_step returns correct length", {
  result <- rho_double_step(100)
  expect_equal(length(result), 99)
})

test_that("rho_double_step has three phases", {
  result <- rho_double_step(100, rho_levels = c(0.8, 0.2, 0.8))
  expect_equal(result[1], 0.8)
  expect_equal(result[50], 0.2)
  expect_equal(result[99], 0.8)
})

test_that("rho_double_step validates levels and breakpoints", {
  expect_error(rho_double_step(100, rho_levels = c(0.8, 1.2, 0.7)), "\\[-1, 1\\]")
  expect_error(rho_double_step(100, breakpoints = c(0.7, 0.2)), "strictly increasing")
  expect_error(rho_double_step(100, breakpoints = c(-0.1, 0.2)), "breakpoints")
  expect_error(rho_double_step(100, transition_width = -0.5), "transition_width")
})

test_that("rho_scenario works for all named scenarios", {
  scenarios <- c("constant", "decreasing", "increasing", "random_walk",
                 "single_middle", "large_change", "small_change",
                 "increase", "double_relapse")

  for (s in scenarios) {
    result <- rho_scenario(s, n_time = 50)
    expect_equal(length(result), 49, info = paste("scenario:", s))
    expect_true(all(result >= -1 & result <= 1), info = paste("scenario:", s))
  }
})

test_that("rho_scenario forwards overrides to scenario generators", {
  early <- rho_scenario("single_middle", n_time = 10, breakpoint = 0.1)
  late <- rho_scenario("single_middle", n_time = 10, breakpoint = 0.9)

  expect_false(identical(early, late))
})

test_that("rho_scenario errors for unknown scenario", {
  expect_error(rho_scenario("nonexistent", n_time = 50), "Unknown scenario")
})

# ============================================================================
# Shared test helper: fit minimal models once and cache for reuse
# ============================================================================

# Skip helpers
.skip_backend_sampling_tests <- function() {
  identical(Sys.getenv("DCVAR_SKIP_STAN_TESTS"), "true")
}

skip_if_no_rstan <- function() {
  skip_if_no_backend("rstan")
}

skip_if_no_cmdstanr_toolchain <- function() {
  skip_on_cran()
  skip_if(
    .skip_backend_sampling_tests(),
    "Backend/toolchain tests disabled by DCVAR_SKIP_STAN_TESTS."
  )
  skip_if_not_installed("cmdstanr")
  tryCatch(
    cmdstanr::cmdstan_path(),
    error = function(e) skip("CmdStan not found")
  )
}

skip_if_no_backend <- function(backend = "rstan") {
  skip_on_cran()
  skip_if(
    .skip_backend_sampling_tests(),
    "Backend sampling tests disabled by DCVAR_SKIP_STAN_TESTS."
  )
  if (backend == "cmdstanr") {
    skip_if_not_installed("cmdstanr")
    skip_if(
      is.null(tryCatch(cmdstanr::cmdstan_path(), error = function(e) NULL)),
      "CmdStan not installed"
    )
  }
  # rstan is always available (in Imports), no skip needed
}

skip_if_no_cmdstanr_backend <- function() {
  skip_if_no_backend("cmdstanr")
}

# Opt-in gate for the expensive per-variable (mixed) margin fits. `rcmdcheck`
# (and therefore the GitHub R-CMD-check matrix) sets NOT_CRAN=true, so the full
# MCMC suite runs during CI; the mixed-margin recovery/coverage fits add enough
# new model compilations and sampling to exceed the runner time budget. These
# fits are exercised locally (and in any lane that sets DCVAR_SLOW_TESTS=true);
# the routing, data-preparation, validation, and extraction tests for the
# feature still run everywhere.
skip_if_not_slow <- function() {
  if (!identical(Sys.getenv("DCVAR_SLOW_TESTS"), "true")) {
    skip("Slow mixed-margin MCMC fit; set DCVAR_SLOW_TESTS=true to run.")
  }
}

# Cache fitted objects in the test environment
cache_env <- new.env(parent = emptyenv())

.capture_fit_warnings <- function(expr) {
  warnings <- character()
  fit <- withCallingHandlers(
    expr,
    warning = function(w) {
      warnings <<- unique(c(warnings, conditionMessage(w)))
      invokeRestart("muffleWarning")
    }
  )
  list(fit = fit, warnings = warnings)
}

.cache_fit_result <- function(key, builder) {
  if (is.null(cache_env[[key]])) {
    cache_env[[key]] <- .capture_fit_warnings(builder())
  }
  cache_env[[key]]
}

expect_known_fit_warnings <- function(warnings, fit_name) {
  known_warning_patterns <- c(
    "divergent transitions after warmup",
    "Examine the pairs\\(\\) plot",
    "Bulk Effective Samples Size \\(ESS\\) is too low",
    "Tail Effective Samples Size \\(ESS\\) is too low",
    "largest R-hat is",
    "R-hat statistic is larger than",
    "sampling finished with diagnostic issues",
    "Inspect `dcvar_diagnostics\\(\\)`",
    "maximum treedepth",
    "There were [0-9]+ divergent transitions after warmup",
    "Bayesian Fraction of Missing Information",
    "E-BFMI",
    "BFMI"
  )

  unexpected_warnings <- warnings[!vapply(
    warnings,
    function(warning_text) any(vapply(
      known_warning_patterns,
      function(pattern) grepl(pattern, warning_text, perl = TRUE),
      logical(1)
    )),
    logical(1)
  )]

  expect_true(
    length(unexpected_warnings) == 0,
    info = paste0(
      fit_name,
      " fit emitted unexpected warnings: ",
      paste(unexpected_warnings, collapse = " | ")
    )
  )
}

core_iter_warmup <- 250
core_iter_sampling <- 250
hier_iter_warmup <- 300
hier_iter_sampling <- 300
smoke_iter_warmup <- 75
smoke_iter_sampling <- 75
margin_iter_warmup <- 300
margin_iter_sampling <- 300

.gamma_support_bound <- function(fit) {
  eps_draws <- posterior::as_draws_matrix(.fit_draws(
    fit$fit, "eps", backend = fit$backend,
    required = .stan_output_group_pattern("eps"),
    required_type = "pattern",
    context = ".gamma_support_bound()",
    output_type = "transformed parameter group"
  ))

  T_eff <- fit$stan_data$n_time - 1
  D <- fit$stan_data$D
  eps_mean <- matrix(NA_real_, nrow = T_eff, ncol = D)

  for (d in seq_len(D)) {
    cols <- paste0("eps[", seq_len(T_eff), ",", d, "]")
    eps_mean[, d] <- colMeans(eps_draws[, cols, drop = FALSE])
  }

  vapply(seq_len(D), function(d) {
    max(-fit$skew_direction[d] * eps_mean[, d])
  }, numeric(1))
}

get_dcvar_fit <- function() {
  .cache_fit_result("dcvar_fit", function() {
    sim <- simulate_dcvar(n_time = 50, rho_trajectory = rho_decreasing(50), seed = 42)
    dcvar(
      sim$Y_df, vars = c("y1", "y2"),
      chains = 2,
      iter_warmup = core_iter_warmup,
      iter_sampling = core_iter_sampling,
      adapt_delta = 0.995,
      refresh = 0, seed = 123
    )
  })$fit
}

get_dcvar_fit_warnings <- function() {
  .cache_fit_result("dcvar_fit", function() {
    sim <- simulate_dcvar(n_time = 50, rho_trajectory = rho_decreasing(50), seed = 42)
    dcvar(
      sim$Y_df, vars = c("y1", "y2"),
      chains = 2,
      iter_warmup = core_iter_warmup,
      iter_sampling = core_iter_sampling,
      adapt_delta = 0.995,
      refresh = 0, seed = 123
    )
  })$warnings
}

get_hmm_fit <- function() {
  .cache_fit_result("hmm_fit", function() {
    sim <- simulate_dcvar(n_time = 50, rho_trajectory = rho_step(50), seed = 42)
    dcvar_hmm(
      sim$Y_df, vars = c("y1", "y2"), K = 2,
      chains = 2,
      iter_warmup = max(core_iter_warmup, 250),
      iter_sampling = max(core_iter_sampling, 250),
      adapt_delta = 0.995,
      max_treedepth = 14,
      refresh = 0, seed = 123
    )
  })$fit
}

get_hmm_switching_fit <- function() {
  .cache_fit_result("hmm_switching_fit", function() {
    # Full Markov-switching config: rho + mu + Phi switch, with per-state
    # families (state 1 normal, state 2 exponential), exercising the engine.
    sim <- simulate_dcvar(
      n_time = 60,
      rho_trajectory = rho_step(60, rho_before = 0.8, rho_after = 0.1),
      seed = 42
    )
    dcvar_hmm(
      sim$Y_df, vars = c("y1", "y2"), K = 2,
      switch = c("rho", "mu", "phi"),
      margins = list(c("normal", "normal"), c("exponential", "exponential")),
      skew_direction = c(1, 1),
      chains = 1,
      iter_warmup = smoke_iter_warmup,
      iter_sampling = smoke_iter_sampling,
      adapt_delta = 0.95,
      max_treedepth = 12,
      refresh = 0, seed = 123
    )
  })$fit
}

get_hmm_fit_warnings <- function() {
  .cache_fit_result("hmm_fit", function() {
    sim <- simulate_dcvar(n_time = 50, rho_trajectory = rho_step(50), seed = 42)
    dcvar_hmm(
      sim$Y_df, vars = c("y1", "y2"), K = 2,
      chains = 2,
      iter_warmup = max(core_iter_warmup, 250),
      iter_sampling = max(core_iter_sampling, 250),
      adapt_delta = 0.995,
      max_treedepth = 14,
      refresh = 0, seed = 123
    )
  })$warnings
}

get_constant_fit <- function() {
  .cache_fit_result("constant_fit", function() {
    sim <- simulate_dcvar(n_time = 50, rho_trajectory = rho_constant(50, 0.5), seed = 42)
    dcvar_constant(
      sim$Y_df, vars = c("y1", "y2"),
      chains = 2,
      iter_warmup = core_iter_warmup,
      iter_sampling = core_iter_sampling,
      adapt_delta = 0.995,
      refresh = 0, seed = 123
    )
  })$fit
}

get_constant_fit_warnings <- function() {
  .cache_fit_result("constant_fit", function() {
    sim <- simulate_dcvar(n_time = 50, rho_trajectory = rho_constant(50, 0.5), seed = 42)
    dcvar_constant(
      sim$Y_df, vars = c("y1", "y2"),
      chains = 2,
      iter_warmup = core_iter_warmup,
      iter_sampling = core_iter_sampling,
      adapt_delta = 0.995,
      refresh = 0, seed = 123
    )
  })$warnings
}

get_multilevel_fit <- function() {
  .cache_fit_result("multilevel_fit", function() {
    sim <- simulate_dcvar_multilevel(N = 3, n_time = 20, rho = 0.5, seed = 42)
    dcvar_multilevel(
      sim$data, vars = c("y1", "y2"), id_var = "id",
      chains = 2,
      iter_warmup = hier_iter_warmup,
      iter_sampling = hier_iter_sampling,
      adapt_delta = 0.995,
      max_treedepth = 14,
      refresh = 0, seed = 123
    )
  })$fit
}

get_multilevel_fit_warnings <- function() {
  .cache_fit_result("multilevel_fit", function() {
    sim <- simulate_dcvar_multilevel(N = 3, n_time = 20, rho = 0.5, seed = 42)
    dcvar_multilevel(
      sim$data, vars = c("y1", "y2"), id_var = "id",
      chains = 2,
      iter_warmup = hier_iter_warmup,
      iter_sampling = hier_iter_sampling,
      adapt_delta = 0.995,
      max_treedepth = 14,
      refresh = 0, seed = 123
    )
  })$warnings
}

get_sem_fit <- function() {
  .cache_fit_result("sem_fit", function() {
    J <- 2
    sim <- simulate_dcvar_sem(n_time = 50, J = J, lambda = rep(0.8, J),
                              rho = 0.5, seed = 42)
    indicators <- list(
      latent1 = paste0("y1_", seq_len(J)),
      latent2 = paste0("y2_", seq_len(J))
    )
    dcvar_sem(
      sim$data, indicators = indicators, J = J,
      lambda = rep(0.8, J), sigma_e = sqrt(0.2),
      chains = 2,
      iter_warmup = max(hier_iter_warmup, 350),
      iter_sampling = max(hier_iter_sampling, 350),
      adapt_delta = 0.999,
      max_treedepth = 14,
      refresh = 0, seed = 123
    )
  })$fit
}

get_sem_fit_warnings <- function() {
  .cache_fit_result("sem_fit", function() {
    J <- 2
    sim <- simulate_dcvar_sem(n_time = 50, J = J, lambda = rep(0.8, J),
                              rho = 0.5, seed = 42)
    indicators <- list(
      latent1 = paste0("y1_", seq_len(J)),
      latent2 = paste0("y2_", seq_len(J))
    )
    dcvar_sem(
      sim$data, indicators = indicators, J = J,
      lambda = rep(0.8, J), sigma_e = sqrt(0.2),
      chains = 2,
      iter_warmup = max(hier_iter_warmup, 350),
      iter_sampling = max(hier_iter_sampling, 350),
      adapt_delta = 0.999,
      max_treedepth = 14,
      refresh = 0, seed = 123
    )
  })$warnings
}

get_sem_exponential_fit <- function() {
  .cache_fit_result("sem_fit_exponential", function() {
    J <- 2
    sim <- simulate_dcvar_sem(
      n_time = 50,
      J = J,
      lambda = rep(0.8, J),
      margins = "exponential",
      sigma_exp = c(0.8, 1.1),
      skew_direction = c(1, -1),
      rho = 0.5,
      seed = 42
    )
    indicators <- list(
      latent1 = paste0("y1_", seq_len(J)),
      latent2 = paste0("y2_", seq_len(J))
    )
    dcvar_sem(
      sim$data,
      indicators = indicators,
      J = J,
      lambda = rep(0.8, J),
      sigma_e = sqrt(0.2),
      margins = "exponential",
      skew_direction = c(1, -1),
      chains = 2,
      iter_warmup = max(hier_iter_warmup, 350),
      iter_sampling = max(hier_iter_sampling, 350),
      adapt_delta = 0.999,
      max_treedepth = 14,
      refresh = 0,
      seed = 123
    )
  })$fit
}

get_sem_exponential_fit_warnings <- function() {
  .cache_fit_result("sem_fit_exponential", function() {
    J <- 2
    sim <- simulate_dcvar_sem(
      n_time = 50,
      J = J,
      lambda = rep(0.8, J),
      margins = "exponential",
      sigma_exp = c(0.8, 1.1),
      skew_direction = c(1, -1),
      rho = 0.5,
      seed = 42
    )
    indicators <- list(
      latent1 = paste0("y1_", seq_len(J)),
      latent2 = paste0("y2_", seq_len(J))
    )
    dcvar_sem(
      sim$data,
      indicators = indicators,
      J = J,
      lambda = rep(0.8, J),
      sigma_e = sqrt(0.2),
      margins = "exponential",
      skew_direction = c(1, -1),
      chains = 2,
      iter_warmup = max(hier_iter_warmup, 350),
      iter_sampling = max(hier_iter_sampling, 350),
      adapt_delta = 0.999,
      max_treedepth = 14,
      refresh = 0,
      seed = 123
    )
  })$warnings
}

get_dcvar_exponential_fit <- function() {
  .cache_fit_result("dcvar_fit_exponential", function() {
    sim <- simulate_dcvar(
      n_time = 30,
      rho_trajectory = rho_decreasing(30),
      margins = "exponential",
      skew_direction = c(1, -1),
      seed = 42
    )
    dcvar(
      sim$Y_df,
      vars = c("y1", "y2"),
      margins = "exponential",
      skew_direction = c(1, -1),
      chains = 2,
      iter_warmup = margin_iter_warmup,
      iter_sampling = margin_iter_sampling,
      adapt_delta = 0.999,
      refresh = 0,
      seed = 123
    )
  })$fit
}

get_dcvar_exponential_fit_warnings <- function() {
  .cache_fit_result("dcvar_fit_exponential", function() {
    sim <- simulate_dcvar(
      n_time = 30,
      rho_trajectory = rho_decreasing(30),
      margins = "exponential",
      skew_direction = c(1, -1),
      seed = 42
    )
    dcvar(
      sim$Y_df,
      vars = c("y1", "y2"),
      margins = "exponential",
      skew_direction = c(1, -1),
      chains = 2,
      iter_warmup = margin_iter_warmup,
      iter_sampling = margin_iter_sampling,
      adapt_delta = 0.999,
      refresh = 0,
      seed = 123
    )
  })$warnings
}

get_dcvar_gamma_fit <- function() {
  .cache_fit_result("dcvar_fit_gamma", function() {
    sim <- simulate_dcvar(
      n_time = 100,
      rho_trajectory = rho_decreasing(100),
      margins = "gamma",
      skew_direction = c(1, 1),
      skew_params = list(shape = 2),
      seed = 42
    )
    dcvar(
      sim$Y_df,
      vars = c("y1", "y2"),
      margins = "gamma",
      skew_direction = c(1, 1),
      chains = 2,
      iter_warmup = 500,
      iter_sampling = 500,
      adapt_delta = 0.999,
      max_treedepth = 15,
      refresh = 0,
      seed = 123
    )
  })$fit
}

get_dcvar_gamma_fit_warnings <- function() {
  .cache_fit_result("dcvar_fit_gamma", function() {
    sim <- simulate_dcvar(
      n_time = 100,
      rho_trajectory = rho_decreasing(100),
      margins = "gamma",
      skew_direction = c(1, 1),
      skew_params = list(shape = 2),
      seed = 42
    )
    dcvar(
      sim$Y_df,
      vars = c("y1", "y2"),
      margins = "gamma",
      skew_direction = c(1, 1),
      chains = 2,
      iter_warmup = 500,
      iter_sampling = 500,
      adapt_delta = 0.999,
      max_treedepth = 15,
      refresh = 0,
      seed = 123
    )
  })$warnings
}

get_dcvar_skew_normal_fit <- function() {
  skip_if_not_installed("sn")

  .cache_fit_result("dcvar_fit_skew_normal", function() {
    sim <- simulate_dcvar(
      n_time = 30,
      rho_trajectory = rho_decreasing(30),
      margins = "skew_normal",
      skew_params = list(alpha = c(3, -3)),
      seed = 42
    )
    dcvar(
      sim$Y_df,
      vars = c("y1", "y2"),
      margins = "skew_normal",
      chains = 1,
      iter_warmup = smoke_iter_warmup,
      iter_sampling = smoke_iter_sampling,
      refresh = 0,
      seed = 123
    )
  })$fit
}

get_dcvar_skew_normal_fit_warnings <- function() {
  skip_if_not_installed("sn")

  .cache_fit_result("dcvar_fit_skew_normal", function() {
    sim <- simulate_dcvar(
      n_time = 30,
      rho_trajectory = rho_decreasing(30),
      margins = "skew_normal",
      skew_params = list(alpha = c(3, -3)),
      seed = 42
    )
    dcvar(
      sim$Y_df,
      vars = c("y1", "y2"),
      margins = "skew_normal",
      chains = 1,
      iter_warmup = smoke_iter_warmup,
      iter_sampling = smoke_iter_sampling,
      refresh = 0,
      seed = 123
    )
  })$warnings
}

get_hmm_exponential_fit <- function() {
  .cache_fit_result("hmm_fit_exponential", function() {
    # Use a strongly separated step trajectory so the tiny test fit remains
    # identifiable across platforms and avoids pathological treedepth behavior.
    sim <- simulate_dcvar(
      n_time = 30,
      rho_trajectory = rho_step(30, rho_before = 0.9, rho_after = 0.1),
      margins = "exponential",
      skew_direction = c(1, -1),
      seed = 42
    )
    dcvar_hmm(
      sim$Y_df,
      vars = c("y1", "y2"),
      K = 2,
      margins = "exponential",
      skew_direction = c(1, -1),
      chains = 2,
      iter_warmup = max(margin_iter_warmup, 350),
      iter_sampling = max(margin_iter_sampling, 350),
      adapt_delta = 0.999,
      max_treedepth = 15,
      refresh = 0,
      seed = 123
    )
  })$fit
}

get_hmm_exponential_fit_warnings <- function() {
  .cache_fit_result("hmm_fit_exponential", function() {
    sim <- simulate_dcvar(
      n_time = 30,
      rho_trajectory = rho_step(30, rho_before = 0.9, rho_after = 0.1),
      margins = "exponential",
      skew_direction = c(1, -1),
      seed = 42
    )
    dcvar_hmm(
      sim$Y_df,
      vars = c("y1", "y2"),
      K = 2,
      margins = "exponential",
      skew_direction = c(1, -1),
      chains = 2,
      iter_warmup = max(margin_iter_warmup, 350),
      iter_sampling = max(margin_iter_sampling, 350),
      adapt_delta = 0.999,
      max_treedepth = 15,
      refresh = 0,
      seed = 123
    )
  })$warnings
}

get_hmm_gamma_fit <- function() {
  .cache_fit_result("hmm_fit_gamma", function() {
    sim <- simulate_dcvar(
      n_time = 100,
      rho_trajectory = rho_step(100, rho_before = 0.9, rho_after = 0.1),
      margins = "gamma",
      skew_direction = c(1, 1),
      skew_params = list(shape = 2),
      seed = 42
    )
    dcvar_hmm(
      sim$Y_df,
      vars = c("y1", "y2"),
      K = 2,
      margins = "gamma",
      skew_direction = c(1, 1),
      chains = 2,
      iter_warmup = 500,
      iter_sampling = 500,
      adapt_delta = 0.999,
      max_treedepth = 15,
      refresh = 0,
      seed = 123
    )
  })$fit
}

get_hmm_gamma_fit_warnings <- function() {
  .cache_fit_result("hmm_fit_gamma", function() {
    sim <- simulate_dcvar(
      n_time = 100,
      rho_trajectory = rho_step(100, rho_before = 0.9, rho_after = 0.1),
      margins = "gamma",
      skew_direction = c(1, 1),
      skew_params = list(shape = 2),
      seed = 42
    )
    dcvar_hmm(
      sim$Y_df,
      vars = c("y1", "y2"),
      K = 2,
      margins = "gamma",
      skew_direction = c(1, 1),
      chains = 2,
      iter_warmup = 500,
      iter_sampling = 500,
      adapt_delta = 0.999,
      max_treedepth = 15,
      refresh = 0,
      seed = 123
    )
  })$warnings
}

get_hmm_skew_normal_fit <- function() {
  skip_if_not_installed("sn")

  .cache_fit_result("hmm_fit_skew_normal", function() {
    sim <- simulate_dcvar(
      n_time = 30,
      rho_trajectory = rho_step(30, rho_before = 0.8, rho_after = 0.2),
      margins = "skew_normal",
      skew_params = list(alpha = c(3, -3)),
      seed = 42
    )
    dcvar_hmm(
      sim$Y_df,
      vars = c("y1", "y2"),
      K = 2,
      margins = "skew_normal",
      chains = 1,
      iter_warmup = smoke_iter_warmup,
      iter_sampling = smoke_iter_sampling,
      adapt_delta = 0.99,
      max_treedepth = 14,
      refresh = 0,
      seed = 123
    )
  })$fit
}

get_hmm_skew_normal_fit_warnings <- function() {
  skip_if_not_installed("sn")

  .cache_fit_result("hmm_fit_skew_normal", function() {
    sim <- simulate_dcvar(
      n_time = 30,
      rho_trajectory = rho_step(30, rho_before = 0.8, rho_after = 0.2),
      margins = "skew_normal",
      skew_params = list(alpha = c(3, -3)),
      seed = 42
    )
    dcvar_hmm(
      sim$Y_df,
      vars = c("y1", "y2"),
      K = 2,
      margins = "skew_normal",
      chains = 1,
      iter_warmup = smoke_iter_warmup,
      iter_sampling = smoke_iter_sampling,
      adapt_delta = 0.99,
      max_treedepth = 14,
      refresh = 0,
      seed = 123
    )
  })$warnings
}

get_constant_gamma_fit <- function() {
  .cache_fit_result("constant_fit_gamma", function() {
    # Gamma-margin smoke fits need a bit more adaptation headroom than the
    # normal/exponential cases to stay stable across older R/rstan toolchains.
    sim <- simulate_dcvar(
      n_time = 100,
      rho_trajectory = rho_constant(100, 0.4),
      margins = "gamma",
      skew_direction = c(1, 1),
      skew_params = list(shape = 2),
      seed = 42
    )
    dcvar_constant(
      sim$Y_df,
      vars = c("y1", "y2"),
      margins = "gamma",
      skew_direction = c(1, 1),
      chains = 2,
      iter_warmup = 500,
      iter_sampling = 500,
      adapt_delta = 0.999,
      max_treedepth = 15,
      refresh = 0,
      seed = 123
    )
  })$fit
}

get_constant_gamma_fit_warnings <- function() {
  .cache_fit_result("constant_fit_gamma", function() {
    sim <- simulate_dcvar(
      n_time = 100,
      rho_trajectory = rho_constant(100, 0.4),
      margins = "gamma",
      skew_direction = c(1, 1),
      skew_params = list(shape = 2),
      seed = 42
    )
    dcvar_constant(
      sim$Y_df,
      vars = c("y1", "y2"),
      margins = "gamma",
      skew_direction = c(1, 1),
      chains = 2,
      iter_warmup = 500,
      iter_sampling = 500,
      adapt_delta = 0.999,
      max_treedepth = 15,
      refresh = 0,
      seed = 123
    )
  })$warnings
}

get_constant_skew_normal_fit <- function() {
  skip_if_not_installed("sn")

  .cache_fit_result("constant_fit_skew_normal", function() {
    sim <- simulate_dcvar(
      n_time = 30,
      rho_trajectory = rho_constant(30, 0.4),
      margins = "skew_normal",
      skew_params = list(alpha = c(3, -3)),
      seed = 42
    )
    dcvar_constant(
      sim$Y_df,
      vars = c("y1", "y2"),
      margins = "skew_normal",
      chains = 1,
      iter_warmup = smoke_iter_warmup,
      iter_sampling = smoke_iter_sampling,
      refresh = 0,
      seed = 123
    )
  })$fit
}

get_constant_skew_normal_fit_warnings <- function() {
  skip_if_not_installed("sn")

  .cache_fit_result("constant_fit_skew_normal", function() {
    sim <- simulate_dcvar(
      n_time = 30,
      rho_trajectory = rho_constant(30, 0.4),
      margins = "skew_normal",
      skew_params = list(alpha = c(3, -3)),
      seed = 42
    )
    dcvar_constant(
      sim$Y_df,
      vars = c("y1", "y2"),
      margins = "skew_normal",
      chains = 1,
      iter_warmup = smoke_iter_warmup,
      iter_sampling = smoke_iter_sampling,
      refresh = 0,
      seed = 123
    )
  })$warnings
}

get_constant_mixed_fit <- function() {
  skip_if_not_slow()
  .cache_fit_result("constant_fit_mixed", function() {
    # Mixed normal + exponential margins under a constant Gaussian copula.
    sim <- simulate_dcvar(
      n_time = 150,
      rho_trajectory = rho_constant(150, 0.6),
      margins = c("normal", "exponential"),
      skew_direction = c(1, 1),
      seed = 42
    )
    dcvar_constant(
      sim$Y_df,
      vars = c("y1", "y2"),
      margins = c("normal", "exponential"),
      skew_direction = c(1, 1),
      chains = 2,
      iter_warmup = 500,
      iter_sampling = 500,
      adapt_delta = 0.999,
      max_treedepth = 14,
      refresh = 0,
      seed = 123
    )
  })$fit
}

get_constant_mixed_fit_warnings <- function() {
  skip_if_not_slow()
  .cache_fit_result("constant_fit_mixed", function() {
    sim <- simulate_dcvar(
      n_time = 150,
      rho_trajectory = rho_constant(150, 0.6),
      margins = c("normal", "exponential"),
      skew_direction = c(1, 1),
      seed = 42
    )
    dcvar_constant(
      sim$Y_df,
      vars = c("y1", "y2"),
      margins = c("normal", "exponential"),
      skew_direction = c(1, 1),
      chains = 2,
      iter_warmup = 500,
      iter_sampling = 500,
      adapt_delta = 0.999,
      max_treedepth = 14,
      refresh = 0,
      seed = 123
    )
  })$warnings
}

.simulate_clayton_mixed <- function(n = 120, theta = 2, seed = 7) {
  # Clayton-correlated normal + shifted-exponential innovations under a VAR(1).
  set.seed(seed)
  u1 <- stats::runif(n)
  q <- stats::runif(n)
  u2 <- (1 + u1^(-theta) * (q^(-theta / (1 + theta)) - 1))^(-1 / theta)
  Y <- matrix(0, n, 2)
  mu <- c(0, 0)
  Phi <- diag(c(0.3, 0.3))
  for (t in 2:n) {
    e1 <- stats::qnorm(u1[t])
    e2 <- stats::qexp(u2[t], rate = 1) - 1
    Y[t, ] <- mu + Phi %*% (Y[t - 1, ] - mu) + c(e1, e2)
  }
  data.frame(time = seq_len(n), y1 = Y[, 1], y2 = Y[, 2])
}

get_constant_mixed_clayton_fit <- function() {
  skip_if_not_slow()
  .cache_fit_result("constant_fit_mixed_clayton", function() {
    df <- .simulate_clayton_mixed(n = 120, theta = 2, seed = 7)
    dcvar_constant(
      df, vars = c("y1", "y2"),
      margins = c("normal", "exponential"), skew_direction = c(1, 1),
      copula = "clayton",
      chains = 2, iter_warmup = 400, iter_sampling = 400,
      adapt_delta = 0.95, refresh = 0, seed = 123
    )
  })$fit
}

get_constant_mixed_clayton_fit_warnings <- function() {
  skip_if_not_slow()
  .cache_fit_result("constant_fit_mixed_clayton", function() {
    df <- .simulate_clayton_mixed(n = 120, theta = 2, seed = 7)
    dcvar_constant(
      df, vars = c("y1", "y2"),
      margins = c("normal", "exponential"), skew_direction = c(1, 1),
      copula = "clayton",
      chains = 2, iter_warmup = 400, iter_sampling = 400,
      adapt_delta = 0.95, refresh = 0, seed = 123
    )
  })$warnings
}

get_dcvar_mixed_fit <- function() {
  skip_if_not_slow()
  .cache_fit_result("dcvar_fit_mixed", function() {
    # Mixed normal + exponential margins under a time-varying Gaussian copula.
    sim <- simulate_dcvar(
      n_time = 120,
      rho_trajectory = rho_decreasing(120),
      margins = c("normal", "exponential"),
      skew_direction = c(1, 1),
      seed = 42
    )
    dcvar(
      sim$Y_df, vars = c("y1", "y2"),
      margins = c("normal", "exponential"), skew_direction = c(1, 1),
      chains = 2, iter_warmup = 400, iter_sampling = 400,
      adapt_delta = 0.99, max_treedepth = 12, refresh = 0, seed = 123
    )
  })$fit
}

get_dcvar_mixed_fit_warnings <- function() {
  skip_if_not_slow()
  .cache_fit_result("dcvar_fit_mixed", function() {
    sim <- simulate_dcvar(
      n_time = 120,
      rho_trajectory = rho_decreasing(120),
      margins = c("normal", "exponential"),
      skew_direction = c(1, 1),
      seed = 42
    )
    dcvar(
      sim$Y_df, vars = c("y1", "y2"),
      margins = c("normal", "exponential"), skew_direction = c(1, 1),
      chains = 2, iter_warmup = 400, iter_sampling = 400,
      adapt_delta = 0.99, max_treedepth = 12, refresh = 0, seed = 123
    )
  })$warnings
}

get_hmm_mixed_fit <- function() {
  skip_if_not_slow()
  .cache_fit_result("hmm_fit_mixed", function() {
    # Mixed normal + exponential margins with two correlation regimes.
    sim <- simulate_dcvar(
      n_time = 120,
      rho_trajectory = rho_step(120, rho_before = 0.8, rho_after = 0.1),
      margins = c("normal", "exponential"),
      skew_direction = c(1, 1),
      seed = 42
    )
    dcvar_hmm(
      sim$Y_df, vars = c("y1", "y2"), K = 2,
      margins = c("normal", "exponential"), skew_direction = c(1, 1),
      chains = 2, iter_warmup = 400, iter_sampling = 400,
      adapt_delta = 0.99, max_treedepth = 13, refresh = 0, seed = 123
    )
  })$fit
}

get_hmm_mixed_fit_warnings <- function() {
  skip_if_not_slow()
  .cache_fit_result("hmm_fit_mixed", function() {
    sim <- simulate_dcvar(
      n_time = 120,
      rho_trajectory = rho_step(120, rho_before = 0.8, rho_after = 0.1),
      margins = c("normal", "exponential"),
      skew_direction = c(1, 1),
      seed = 42
    )
    dcvar_hmm(
      sim$Y_df, vars = c("y1", "y2"), K = 2,
      margins = c("normal", "exponential"), skew_direction = c(1, 1),
      chains = 2, iter_warmup = 400, iter_sampling = 400,
      adapt_delta = 0.99, max_treedepth = 13, refresh = 0, seed = 123
    )
  })$warnings
}

get_multilevel_mixed_fit <- function() {
  skip_if_not_slow()
  .cache_fit_result("multilevel_fit_mixed", function() {
    sim <- simulate_dcvar_multilevel(
      N = 6, n_time = 40, rho = 0.5,
      margins = c("normal", "exponential"), skew_direction = c(1, 1), seed = 42
    )
    dcvar_multilevel(
      sim$data, vars = c("y1", "y2"), id_var = "id",
      margins = c("normal", "exponential"), skew_direction = c(1, 1),
      chains = 2, iter_warmup = 400, iter_sampling = 400,
      adapt_delta = 0.95, max_treedepth = 13, refresh = 0, seed = 123
    )
  })$fit
}

get_multilevel_mixed_fit_warnings <- function() {
  skip_if_not_slow()
  .cache_fit_result("multilevel_fit_mixed", function() {
    sim <- simulate_dcvar_multilevel(
      N = 6, n_time = 40, rho = 0.5,
      margins = c("normal", "exponential"), skew_direction = c(1, 1), seed = 42
    )
    dcvar_multilevel(
      sim$data, vars = c("y1", "y2"), id_var = "id",
      margins = c("normal", "exponential"), skew_direction = c(1, 1),
      chains = 2, iter_warmup = 400, iter_sampling = 400,
      adapt_delta = 0.95, max_treedepth = 13, refresh = 0, seed = 123
    )
  })$warnings
}

.simulate_sem_mixed <- function(n_time = 120, J = 2, seed = 42) {
  simulate_dcvar_sem(
    n_time = n_time, J = J, lambda = rep(0.8, J), sigma_e = sqrt(0.2),
    margins = c("normal", "exponential"), skew_direction = c(1, 1),
    rho = 0.5, seed = seed
  )
}

.sem_mixed_indicators <- function(J = 2) {
  list(latent1 = paste0("y1_", seq_len(J)), latent2 = paste0("y2_", seq_len(J)))
}

get_sem_mixed_fit <- function() {
  skip_if_not_slow()
  .cache_fit_result("sem_fit_mixed", function() {
    J <- 2
    sim <- .simulate_sem_mixed(J = J)
    dcvar_sem(
      sim$data, indicators = .sem_mixed_indicators(J), J = J,
      lambda = rep(0.8, J), sigma_e = sqrt(0.2),
      margins = c("normal", "exponential"), skew_direction = c(1, 1),
      chains = 2, iter_warmup = 400, iter_sampling = 400,
      adapt_delta = 0.999, max_treedepth = 13, refresh = 0, seed = 123
    )
  })$fit
}

get_sem_mixed_fit_warnings <- function() {
  skip_if_not_slow()
  .cache_fit_result("sem_fit_mixed", function() {
    J <- 2
    sim <- .simulate_sem_mixed(J = J)
    dcvar_sem(
      sim$data, indicators = .sem_mixed_indicators(J), J = J,
      lambda = rep(0.8, J), sigma_e = sqrt(0.2),
      margins = c("normal", "exponential"), skew_direction = c(1, 1),
      chains = 2, iter_warmup = 400, iter_sampling = 400,
      adapt_delta = 0.999, max_treedepth = 13, refresh = 0, seed = 123
    )
  })$warnings
}

get_sem_naive_mixed_fit <- function() {
  skip_if_not_slow()
  .cache_fit_result("sem_naive_fit_mixed", function() {
    J <- 2
    sim <- .simulate_sem_mixed(J = J)
    dcvar_sem(
      sim$data, indicators = .sem_mixed_indicators(J), J = J, method = "naive",
      margins = c("normal", "exponential"), skew_direction = c(1, 1),
      chains = 2, iter_warmup = 400, iter_sampling = 400,
      adapt_delta = 0.999, max_treedepth = 13, refresh = 0, seed = 123
    )
  })$fit
}

get_dcvar_cmdstanr_fit <- function() {
  .cache_fit_result("dcvar_cmdstanr_fit", function() {
    skip_if_no_cmdstanr_backend()

    sim <- simulate_dcvar(n_time = 24, rho_trajectory = rho_decreasing(24), seed = 42)
    dcvar(
      sim$Y_df, vars = c("y1", "y2"),
      chains = 1,
      iter_warmup = 50,
      iter_sampling = 50,
      adapt_delta = 0.95,
      max_treedepth = 10,
      refresh = 0, seed = 123,
      backend = "cmdstanr"
    )
  })$fit
}

get_hmm_cmdstanr_fit <- function() {
  .cache_fit_result("hmm_cmdstanr_fit", function() {
    skip_if_no_cmdstanr_backend()

    sim <- simulate_dcvar(n_time = 24, rho_trajectory = rho_step(24), seed = 42)
    dcvar_hmm(
      sim$Y_df, vars = c("y1", "y2"), K = 2,
      chains = 1,
      iter_warmup = 50,
      iter_sampling = 50,
      adapt_delta = 0.95,
      max_treedepth = 10,
      refresh = 0, seed = 123,
      backend = "cmdstanr"
    )
  })$fit
}

get_constant_cmdstanr_fit <- function() {
  .cache_fit_result("constant_cmdstanr_fit", function() {
    skip_if_no_cmdstanr_backend()

    sim <- simulate_dcvar(n_time = 24, rho_trajectory = rho_constant(24, 0.5), seed = 42)
    dcvar_constant(
      sim$Y_df, vars = c("y1", "y2"),
      chains = 1,
      iter_warmup = 50,
      iter_sampling = 50,
      adapt_delta = 0.95,
      max_treedepth = 10,
      refresh = 0, seed = 123,
      backend = "cmdstanr"
    )
  })$fit
}

get_constant_clayton_fit <- function() {
  .cache_fit_result("constant_fit_clayton", function() {
    sim <- simulate_dcvar(
      n_time = 30,
      rho_trajectory = rho_constant(30, 0.5),
      seed = 42
    )
    dcvar_constant(
      sim$Y_df,
      vars = c("y1", "y2"),
      copula = "clayton",
      chains = 1,
      iter_warmup = smoke_iter_warmup,
      iter_sampling = smoke_iter_sampling,
      refresh = 0,
      seed = 123
    )
  })$fit
}

get_covariate_fit <- function(drift = TRUE) {
  key <- paste0("covariate_fit_", if (drift) "drift" else "nodrift")
  .cache_fit_result(key, function() {
    sim <- simulate_dcvar(
      n_time = 30,
      rho_trajectory = rho_step(30, rho_before = 0.6, rho_after = 0.2, breakpoint = 0.5),
      seed = 42
    )
    df <- sim$Y_df
    df$phase <- as.numeric(df$time > 15)
    dcvar_covariate(
      df,
      vars = c("y1", "y2"),
      covariates = "phase",
      drift = drift,
      chains = 1,
      iter_warmup = smoke_iter_warmup,
      iter_sampling = smoke_iter_sampling,
      refresh = 0,
      seed = 123
    )
  })$fit
}

get_dcvar_tv_fit <- function() {
  .cache_fit_result("dcvar_tv_fit_both", function() {
    sim <- simulate_dcvar(
      n_time = 40,
      rho_trajectory = rho_decreasing(40, 0.6, 0.2),
      phi_trajectory = list(
        rho_constant(40, 0.3), rho_decreasing(40, 0.3, 0.0),
        rho_constant(40, 0.1), rho_constant(40, 0.3)
      ),
      sigma_trajectory = cbind(seq(0.8, 1.4, length.out = 39), rep(1, 39)),
      seed = 42
    )
    dcvar(
      sim$Y_df,
      vars = c("y1", "y2"),
      tv_phi = TRUE,
      tv_sigma = TRUE,
      chains = 1,
      iter_warmup = smoke_iter_warmup,
      iter_sampling = smoke_iter_sampling,
      refresh = 0,
      seed = 123
    )
  })$fit
}

get_dcvar_tv_sigma_only_fit <- function() {
  .cache_fit_result("dcvar_tv_fit_sigma_only", function() {
    sim <- simulate_dcvar(
      n_time = 40,
      rho_trajectory = rho_constant(40, 0.4),
      sigma_trajectory = cbind(seq(0.8, 1.4, length.out = 39), rep(1, 39)),
      seed = 43
    )
    dcvar(
      sim$Y_df,
      vars = c("y1", "y2"),
      tv_phi = FALSE,
      tv_sigma = TRUE,
      chains = 1,
      iter_warmup = smoke_iter_warmup,
      iter_sampling = smoke_iter_sampling,
      refresh = 0,
      seed = 124
    )
  })$fit
}

get_dcvar_tv_ar_fit <- function() {
  .cache_fit_result("dcvar_tv_fit_ar", function() {
    sim <- simulate_dcvar(
      n_time = 40,
      rho_trajectory = rho_constant(40, 0.4),
      phi_trajectory = list(
        rho_decreasing(40, 0.4, 0.1), rho_constant(40, 0.1),
        rho_constant(40, 0.1), rho_constant(40, 0.3)
      ),
      seed = 44
    )
    dcvar(
      sim$Y_df,
      vars = c("y1", "y2"),
      tv_phi = "ar",
      tv_sigma = FALSE,
      chains = 1,
      iter_warmup = smoke_iter_warmup,
      iter_sampling = smoke_iter_sampling,
      refresh = 0,
      seed = 125
    )
  })$fit
}

get_dcvar_tv_exp_fit <- function() {
  .cache_fit_result("dcvar_tv_fit_exp", function() {
    sim <- simulate_dcvar(
      n_time = 40,
      rho_trajectory = rho_constant(40, 0.4),
      margins = c("normal", "exponential"),
      skew_direction = c(1, 1),
      sigma_trajectory = cbind(rep(1, 39), seq(0.7, 1.6, length.out = 39)),
      tv_sigma_k = 8,
      seed = 46
    )
    dcvar(
      sim$Y_df,
      vars = c("y1", "y2"),
      margins = c("normal", "exponential"),
      skew_direction = c(1, 1),
      tv_sigma = TRUE,
      tv_sigma_k = 8,
      chains = 1,
      iter_warmup = smoke_iter_warmup,
      iter_sampling = smoke_iter_sampling,
      refresh = 0,
      seed = 126
    )
  })$fit
}

# ============================================================================
# S3 Class: dcvar_sem_fit
# ============================================================================

#' Construct a dcvar_sem_fit object
#' @noRd
new_dcvar_sem_fit <- function(fit, stan_data, vars, J, lambda, sigma_e,
                               indicators, margins = "normal",
                               method = "indicator",
                               skew_direction = NULL, backend = "rstan",
                               priors, meta) {
  structure(
    list(
      fit = fit,
      stan_data = stan_data,
      model = "sem",
      vars = vars,
      standardized = FALSE,
      margins = margins,
      method = method,
      J = J,
      lambda = lambda,
      sigma_e = sigma_e,
      indicators = indicators,
      skew_direction = skew_direction,
      backend = backend,
      priors = priors,
      meta = meta
    ),
    class = c("dcvar_sem_fit", "dcvar_model_fit")
  )
}


#' Construct a dcvar_sem_tv_fit object
#' @noRd
new_dcvar_sem_tv_fit <- function(fit, stan_data, vars, J, lambda, sigma_e,
                                 indicators, margins = "normal",
                                 method = "indicator",
                                 skew_direction = NULL,
                                 tv_phi = FALSE, phi_tv_mask = NULL,
                                 tv_sigma = FALSE,
                                 backend = "rstan", priors, meta) {
  phi_tv_mask <- phi_tv_mask %||% .resolve_phi_tv_mask(tv_phi)
  out <- new_dcvar_sem_fit(
    fit = fit,
    stan_data = stan_data,
    vars = vars,
    J = J,
    lambda = lambda,
    sigma_e = sigma_e,
    indicators = indicators,
    margins = margins,
    method = method,
    skew_direction = skew_direction,
    backend = backend,
    priors = priors,
    meta = meta
  )
  out$model <- "sem_tv"
  out$tv_phi <- isTRUE(sum(phi_tv_mask) > 0L)
  out$phi_tv_mask <- phi_tv_mask
  out$tv_sigma <- isTRUE(tv_sigma)
  class(out) <- c("dcvar_sem_tv_fit", "dcvar_sem_fit", "dcvar_model_fit")
  out
}


#' S3 methods for dcvar_sem_fit objects
#'
#' @param x,object A `dcvar_sem_fit` object.
#' @param ... Additional arguments (unused).
#'
#' @name dcvar_sem_fit-methods
NULL

#' @describeIn dcvar_sem_fit-methods Print a concise overview.
#' @return Invisibly returns `x`.
#' @export
print.dcvar_sem_fit <- function(x, ...) {
  title <- if (identical(x$method %||% "indicator", "naive")) {
    "Naive SEM Copula VAR Model Fit"
  } else {
    "SEM Copula VAR Model Fit"
  }
  .print_fit_header(x, title)
  cat(sprintf("n_time = %d, J = %d indicators per latent\n", x$stan_data$n_time, x$J))
  .print_fit_footer(x)

  cat(sprintf("rho: %.3f\n", coef(x)$rho[[1]]))
  invisible(x)
}


#' @describeIn dcvar_sem_fit-methods Produce a detailed summary.
#' @return A `dcvar_sem_summary` object (a list).
#' @export
summary.dcvar_sem_fit <- function(object, ...) {
  vp <- var_params(object)
  diag <- dcvar_diagnostics(object)

  out <- list(
    model = "sem",
    method = object$method %||% "indicator",
    n_time = object$stan_data$n_time,
    J = object$J,
    lambda = object$lambda,
    sigma_e = object$sigma_e,
    var_params = vp,
    diagnostics = diag
  )
  class(out) <- "dcvar_sem_summary"
  out
}


#' @describeIn dcvar_sem_fit-methods Print a concise overview of a TV SEM fit.
#' @return Invisibly returns `x`.
#' @export
print.dcvar_sem_tv_fit <- function(x, ...) {
  .print_fit_header(x, "TV SEM Copula VAR Model Fit")
  cat(sprintf("n_time = %d, J = %d indicators per latent\n", x$stan_data$n_time, x$J))
  cat(sprintf("time-varying components: %s\n", .tv_components_label(x)))
  .print_fit_footer(x)

  rho_df <- rho_trajectory(x)
  cat(sprintf("rho range: [%.3f, %.3f] (posterior mean)\n",
              min(rho_df$mean), max(rho_df$mean)))
  invisible(x)
}


#' @describeIn dcvar_sem_fit-methods Produce a detailed TV SEM summary.
#' @param probs Numeric vector of quantile probabilities for trajectories.
#' @return A `dcvar_sem_tv_summary` object (a list).
#' @export
summary.dcvar_sem_tv_fit <- function(object, probs = c(0.025, 0.5, 0.975), ...) {
  vp <- var_params(object)
  diag <- dcvar_diagnostics(object)
  rho_df <- rho_trajectory(object, probs = probs)

  out <- list(
    model = "sem_tv",
    method = object$method %||% "indicator",
    n_time = object$stan_data$n_time,
    J = object$J,
    lambda = object$lambda,
    sigma_e = object$sigma_e,
    components = .tv_components_label(object),
    rho_trajectory = rho_df,
    rho_summary = data.frame(
      statistic = c("start", "end", "delta", "min", "max", "mean"),
      value = c(
        rho_df$mean[1],
        rho_df$mean[nrow(rho_df)],
        rho_df$mean[nrow(rho_df)] - rho_df$mean[1],
        min(rho_df$mean),
        max(rho_df$mean),
        mean(rho_df$mean)
      )
    ),
    phi_summary = if (isTRUE(object$tv_phi)) {
      .tv_trajectory_summary_block(phi_trajectory(object, probs = probs), "coefficient")
    } else {
      NULL
    },
    sigma_summary = if (isTRUE(object$tv_sigma)) {
      .tv_trajectory_summary_block(sigma_trajectory(object, probs = probs), "variable")
    } else {
      NULL
    },
    var_params = vp,
    diagnostics = diag
  )
  class(out) <- c("dcvar_sem_tv_summary", "dcvar_sem_summary")
  out
}


#' Print a dcvar_sem_summary object
#' @param x A `dcvar_sem_summary` object.
#' @param ... Additional arguments (unused).
#' @return Invisibly returns `x`.
#' @export
print.dcvar_sem_summary <- function(x, ...) {
  if (identical(x$method %||% "indicator", "naive")) {
    cat("Naive SEM Copula VAR Model Summary\n")
  } else {
    cat("SEM Copula VAR Model Summary\n")
  }
  cat(strrep("=", 50), "\n")
  cat(sprintf("n_time = %d, J = %d indicators per latent\n", x$n_time, x$J))
  if (!identical(x$method %||% "indicator", "naive")) {
    cat(sprintf("Fixed lambda: %s\n", paste(round(x$lambda, 3), collapse = ", ")))
    cat(sprintf("Fixed sigma_e: %.3f\n", x$sigma_e))
  }
  cat("\n")

  cat("Latent VAR Parameters:\n")
  if (!is.null(x$var_params$mu)) {
    cat("  mu:\n")
    print(x$var_params$mu[, c("variable", "mean", "q2.5", "q97.5")], row.names = FALSE)
  }
  if (!is.null(x$var_params$Phi)) {
    cat("\n  Phi:\n")
    print(x$var_params$Phi[, c("variable", "mean", "q2.5", "q97.5")], row.names = FALSE)
  }
  if (!is.null(x$var_params$sigma)) {
    cat("\n  sigma:\n")
    print(x$var_params$sigma[, c("variable", "mean", "q2.5", "q97.5")], row.names = FALSE)
  } else {
    .print_margin_params(x$var_params)
  }
  if (!is.null(x$var_params$rho)) {
    cat("\n  rho:\n")
    print(x$var_params$rho[, c("variable", "mean", "q2.5", "q97.5")], row.names = FALSE)
  }

  cat("\nDiagnostics:\n")
  cat(sprintf("  Divergences: %d\n", x$diagnostics$n_divergent))
  cat(sprintf("  Max Rhat: %.3f\n", x$diagnostics$max_rhat))
  cat(sprintf("  Min ESS bulk: %.0f\n", x$diagnostics$min_ess_bulk))

  invisible(x)
}


#' Print a dcvar_sem_tv_summary object
#' @param x A `dcvar_sem_tv_summary` object.
#' @param ... Additional arguments (unused).
#' @return Invisibly returns `x`.
#' @export
print.dcvar_sem_tv_summary <- function(x, ...) {
  cat("TV SEM Copula VAR Model Summary\n")
  cat(strrep("=", 50), "\n")
  cat(sprintf("n_time = %d, J = %d indicators per latent\n", x$n_time, x$J))
  cat(sprintf("time-varying components: %s\n\n", x$components))

  cat("Rho Trajectory:\n")
  print(x$rho_summary, row.names = FALSE)
  if (!is.null(x$phi_summary)) {
    cat("\nPhi(t) Trajectories (posterior mean):\n")
    print(x$phi_summary, row.names = FALSE)
  }
  if (!is.null(x$sigma_summary)) {
    cat("\nsigma(t) Trajectories (posterior mean):\n")
    print(x$sigma_summary, row.names = FALSE)
  }

  cat("\nLatent VAR Parameters:\n")
  cat("  mu:\n")
  print(x$var_params$mu[, c("variable", "mean", "q2.5", "q97.5")], row.names = FALSE)
  cat("\n  Phi (baseline):\n")
  print(x$var_params$Phi[, c("variable", "mean", "q2.5", "q97.5")], row.names = FALSE)
  if (!is.null(x$var_params$tau_phi)) {
    cat("\n  tau_phi (walk SDs):\n")
    print(x$var_params$tau_phi[, c("variable", "mean", "q2.5", "q97.5")], row.names = FALSE)
  }
  if (!is.null(x$var_params$tau_sigma)) {
    cat("\n  tau_sigma (log-scale walk SDs):\n")
    print(x$var_params$tau_sigma[, c("variable", "mean", "q2.5", "q97.5")], row.names = FALSE)
  }
  .print_margin_params(x$var_params)
  if (!is.null(x$var_params$sigma_omega)) {
    cat("\n  sigma_omega (rho random-walk SD):\n")
    print(x$var_params$sigma_omega[, c("variable", "mean", "q2.5", "q97.5")], row.names = FALSE)
  }

  cat("\nDiagnostics:\n")
  cat(sprintf("  Divergences: %d\n", x$diagnostics$n_divergent))
  cat(sprintf("  Max Rhat: %.3f\n", x$diagnostics$max_rhat))
  cat(sprintf("  Min ESS bulk: %.0f\n", x$diagnostics$min_ess_bulk))

  invisible(x)
}


#' @describeIn dcvar_sem_fit-methods Extract posterior means of
#'   latent VAR coefficients.
#' @return A named list of posterior means.
#' @export
coef.dcvar_sem_fit <- function(object, ...) {
  summ <- .fit_summary(object$fit, backend = object$backend)
  result <- list(
    mu = .extract_required_coef(summ, "^mu\\[", "mu", "coef.dcvar_sem_fit()"),
    Phi = .extract_required_coef(summ, "^Phi\\[", "Phi", "coef.dcvar_sem_fit()"),
    rho = .extract_required_coef(summ, "^rho$", "rho", "coef.dcvar_sem_fit()")
  )
  margins <- object$margins %||% "normal"
  margin_coefs <- if (identical(margins, "normal")) {
    list(sigma = .extract_required_coef(summ, "^sigma\\[", "sigma", "coef.dcvar_sem_fit()"))
  } else if (.uses_sem_multilevel_mixed_engine("sem", margins)) {
    .extract_margin_coefs(
      summ,
      .sem_multilevel_engine_margins("sem", margins, 2L)
    )
  } else {
    .extract_margin_coefs(summ, margins)
  }
  c(result[1:2], margin_coefs, result[3])
}


#' @export
coef.dcvar_sem_tv_fit <- function(object, ...) {
  summ <- .fit_summary(object$fit, backend = object$backend)
  margins_vec <- rep(object$margins %||% "normal", length.out = 2L)
  result <- list(
    mu = .extract_required_coef(summ, "^mu\\[", "mu", "coef.dcvar_sem_tv_fit()"),
    Phi = .extract_required_coef(summ, "^Phi\\[", "Phi", "coef.dcvar_sem_tv_fit()")
  )
  margin_coefs <- .extract_margin_coefs(summ, margins_vec)
  extras <- list(
    sigma_omega = .extract_required_coef(summ, "^sigma_omega$", "sigma_omega", "coef.dcvar_sem_tv_fit()"),
    rho = .extract_required_coef(summ, "^rho\\[", "rho", "coef.dcvar_sem_tv_fit()")
  )
  if (isTRUE(object$tv_phi)) {
    tau_phi <- .extract_required_coef(summ, "^tau_phi\\[", "tau_phi", "coef.dcvar_sem_tv_fit()")
    names(tau_phi) <- paste0("tau_phi[", .tv_active_phi_coefs(object), "]")
    extras$tau_phi <- tau_phi
  }
  if (isTRUE(object$tv_sigma)) {
    extras$tau_sigma <- .extract_required_coef(summ, "^tau_sigma\\[", "tau_sigma", "coef.dcvar_sem_tv_fit()")
  }
  c(result, margin_coefs, extras)
}


#' @describeIn dcvar_sem_fit-methods Dispatch to a plot type.
#' @param type Character; one of `"latent_states"`, `"rho"`, `"diagnostics"`.
#' @return A ggplot object.
#' @export
plot.dcvar_sem_fit <- function(x,
                                type = c("latent_states", "rho", "diagnostics"),
                                ...) {
  type <- match.arg(type)
  if (identical(type, "latent_states") && identical(x$method %||% "indicator", "naive")) {
    cli_abort("Latent-state plots are not available for naive SEM fits because no latent measurement model is estimated.")
  }
  switch(type,
    latent_states = plot_latent_states(x, ...),
    rho = plot_rho(x, ...),
    diagnostics = plot_diagnostics(x, ...)
  )
}

# ============================================================================
# Applicability check for the flexible-margin copula-VAR
# ============================================================================

#' Check whether the flexible-margin copula-VAR is appropriate for the data
#'
#' Screens a fitted constant copula-VAR for the floor/ceiling pile-up pathology.
#' When a non-trivial point mass of observations sits exactly at a scale bound
#' (a floor or ceiling effect, common for raw single-item slider or Likert
#' responses), a flexible skewed margin (skew-normal, gamma, exponential) can
#' reproduce that pile-up only by concentrating the innovations at the bound,
#' which is achieved when the autoregressive prediction is flat. The margin then
#' absorbs the marginal shape and the VAR coefficients collapse toward zero, even
#' when the series is clearly autocorrelated. The decision whether the approach
#' suits a dataset cannot be made from the response format (a visual analog scale
#' looks continuous yet can have a heavy floor pile-up); it must be made from the
#' realized data and the fit.
#'
#' Three signals are combined:
#' \enumerate{
#'   \item \strong{Boundary atom}: the fraction of observations exactly at each
#'     series minimum and maximum, counted only when at least two observations
#'     coincide at the bound (a lone extreme value is not a point mass). A
#'     continuous margin cannot reproduce a point mass, so a non-trivial atom is
#'     a warning sign.
#'   \item \strong{Dynamics collapse}: the fitted self-lags against an OLS VAR(1)
#'     anchor (and, when supplied, a normal-margin reference fit). A
#'     flexible-margin fit whose self-lags collapse toward zero while an anchor
#'     shows clear autoregression (in either direction) is the signature of the
#'     pathology.
#'   \item \strong{Boundary and convergence flags}: a skew-normal slant estimated
#'     at its boundary, divergent transitions, or elevated split-Rhat.
#' }
#'
#' @param fit A `dcvar_constant_fit`.
#' @param reference Optional normal-margin `dcvar_constant_fit` on the same
#'   variables and number of observations. When supplied, the dynamics-collapse
#'   signal compares the fitted self-lags against this reference fit's self-lags
#'   in addition to the OLS VAR(1) anchor (which is always used). If `NULL`
#'   (default), only the OLS VAR(1) anchor is used.
#' @param atom_tol Minimum fraction at a bound to flag a point mass (default `0.05`).
#' @param delta_tol Magnitude of the skew-normal slant treated as a boundary value
#'   (default `0.98`).
#' @param phi_collapse_tol Self-lag magnitude treated as collapsed (default `0.10`).
#' @param ols_clear_tol Anchor self-lag magnitude treated as clear
#'   autoregression when detecting a dynamics collapse (default `0.20`).
#' @param rhat_tol Split-Rhat threshold (default `1.01`).
#'
#' @return An object of class `dcvar_applicability`: a list with the `verdict`
#'   (`"suitable"`, `"caution"`, or `"unsuitable"`), a `recommendation`, the
#'   triggered `reasons`, the variable names (`vars`) and `margins`, the
#'   boundary-atom fractions (`atom`, `atom_min`, `atom_max`), the fitted and
#'   OLS self-lags (`fit_self_lag`, `ols_self_lag`, plus `reference_self_lag`
#'   when a reference is supplied), the skew-normal slant means (`delta`, or
#'   `NULL`), the convergence `diagnostics`, and the `thresholds` used. Has a
#'   `print` method.
#'
#' @seealso [dcvar_constant()], [dcvar_diagnostics()], [var_params()].
#'
#' @examples
#' \donttest{
#' sim <- simulate_dcvar(
#'   n_time = 60,
#'   rho_trajectory = rho_constant(60, rho = 0.4),
#'   margins = "skew_normal",
#'   skew_params = list(alpha = c(4, 4)),
#'   seed = 1
#' )
#' fit <- dcvar_constant(
#'   sim$Y_df, vars = c("y1", "y2"), margins = "skew_normal",
#'   chains = 2, iter_warmup = 500, iter_sampling = 500, refresh = 0, seed = 1
#' )
#' applicability_check(fit)
#' }
#' @export
applicability_check <- function(fit, reference = NULL, atom_tol = 0.05,
                                delta_tol = 0.98, phi_collapse_tol = 0.10,
                                ols_clear_tol = 0.20, rhat_tol = 1.01) {
  if (!inherits(fit, "dcvar_constant_fit")) {
    cli_abort("{.fun applicability_check} expects a {.cls dcvar_constant_fit}, got {.cls {class(fit)[[1]]}}.")
  }
  Y <- fit$stan_data$Y
  if (is.null(Y)) cli_abort("Fitted object does not carry the data matrix {.field stan_data$Y}.")
  Y <- as.matrix(Y)
  if (anyNA(Y)) {
    cli_abort("{.field stan_data$Y} contains missing values; cannot run {.fun applicability_check}.")
  }
  n <- nrow(Y); D <- ncol(Y)
  vars <- fit$vars %||% paste0("y", seq_len(D))
  margins <- fit$margins %||% "normal"
  # margins is stored as a single string when homogeneous; expand to length D so
  # per-dimension indexing (e.g. which dims are skew-normal) is well defined.
  margins_full <- if (length(margins) == 1L) rep(margins, D) else margins
  is_flexible <- any(margins %in% c("skew_normal", "gamma", "exponential"))

  if (!is.null(reference)) {
    if (!inherits(reference, "dcvar_constant_fit")) {
      cli_abort("{.arg reference} must be a {.cls dcvar_constant_fit}, got {.cls {class(reference)[[1]]}}.")
    }
    Yref <- reference$stan_data$Y
    if (is.null(Yref)) {
      cli_abort("{.arg reference} does not carry the data matrix {.field stan_data$Y}.")
    }
    Yref <- as.matrix(Yref)
    if (ncol(Yref) != D) {
      cli_abort("{.arg reference} must be fitted on the same number of variables as {.arg fit} ({D}).")
    }
    if (nrow(Yref) != n) {
      cli_abort("{.arg reference} must be fitted on the same number of observations as {.arg fit} ({n}).")
    }
    ref_vars <- reference$vars %||% vars
    if (length(ref_vars) != D || !identical(as.character(ref_vars), as.character(vars))) {
      cli_abort("{.arg reference} must be fitted on the same variables as {.arg fit}, in the same order.")
    }
    ref_margins <- reference$margins %||% "normal"
    if (!all(ref_margins == "normal")) {
      cli_abort("{.arg reference} must be a normal-margin fit ({.arg margins = 'normal'}).")
    }
  }

  ## var_params() runs a full posterior re-summary, so compute it once and reuse.
  vp <- var_params(fit)

  ## (1) boundary atom: fraction exactly at each series min / max, but only when
  ## at least two observations tie at the bound (a single extreme is not a mass).
  atom_frac <- function(col, extreme) {
    k <- sum(col == extreme)
    if (k >= 2L) k / n else 0
  }
  atom_min <- vapply(seq_len(D), function(d) atom_frac(Y[, d], min(Y[, d])), numeric(1))
  atom_max <- vapply(seq_len(D), function(d) atom_frac(Y[, d], max(Y[, d])), numeric(1))
  atom <- pmax(atom_min, atom_max)
  atom_present <- any(atom >= atom_tol)

  ## (2) dynamics: OLS VAR(1) anchor (and optional reference) vs fitted self-lags
  zv <- vapply(seq_len(D), function(d) isTRUE(stats::sd(Y[, d]) == 0), logical(1))
  Ylag <- Y[-n, , drop = FALSE]; Ycur <- Y[-1, , drop = FALSE]
  ols_self <- vapply(seq_len(D), function(d) {
    unname(stats::coef(stats::lm(Ycur[, d] ~ Ylag))[1L + d])
  }, numeric(1))
  self_from <- function(phi_df) vapply(seq_len(D), function(d) {
    row <- phi_df[phi_df$variable == sprintf("Phi[%d,%d]", d, d), , drop = FALSE]
    if (nrow(row) == 0) NA_real_ else row$mean[[1]]
  }, numeric(1))
  fit_self <- self_from(vp$Phi)
  ref_self <- if (is.null(reference)) NULL else self_from(var_params(reference)$Phi)
  fit_self_bad <- !is.finite(fit_self)
  ols_self_bad <- !is.finite(ols_self)
  ref_self_bad <- if (is.null(ref_self)) rep(FALSE, D) else !is.finite(ref_self)
  collapsed_vs <- function(anchor) {
    any(is.finite(fit_self) & is.finite(anchor) &
          abs(fit_self) < phi_collapse_tol &
          abs(anchor) > ols_clear_tol, na.rm = TRUE)
  }
  collapse <- collapsed_vs(ols_self) || (!is.null(ref_self) && collapsed_vs(ref_self))

  ## (3) slant at boundary (any skew-normal dimension) and convergence
  delta_vals <- NULL; delta_rail <- FALSE; delta_bad <- FALSE; delta_names <- NULL
  if (any(margins == "skew_normal")) {
    vpd <- vp$delta
    if (!is.null(vpd) && length(vpd$mean) > 0L) {
      delta_vals <- vpd$mean
      delta_bad <- any(!is.finite(delta_vals))
      delta_rail <- any(abs(delta_vals) >= delta_tol, na.rm = TRUE)
      ## $delta carries one row per skew-normal dimension (all D in the
      ## homogeneous case, the skew-normal subset in the mixed case).
      skew_idx <- which(margins_full == "skew_normal")
      delta_names <- if (length(skew_idx) == length(delta_vals)) {
        vars[skew_idx]
      } else {
        vars[seq_along(delta_vals)]
      }
    }
  }
  diag <- dcvar_diagnostics(fit)
  conv_bad <- diag$n_divergent > 0 ||
    (is.finite(diag$max_rhat) && diag$max_rhat > rhat_tol)

  ## reasons (every triggered signal, regardless of the headline verdict)
  reasons <- character(0)
  if (atom_present)
    reasons <- c(reasons, sprintf("boundary atom present (up to %.0f%% of observations at a scale bound)", 100 * max(atom)))
  if (collapse)
    reasons <- c(reasons, "fitted self-lags collapse toward zero while the VAR(1) anchor shows clear autoregression")
  if (any(fit_self_bad))
    reasons <- c(reasons, sprintf("fitted self-lag summary undefined for series (%s)", paste(vars[fit_self_bad], collapse = ", ")))
  if (any(ols_self_bad))
    reasons <- c(reasons, sprintf("OLS VAR(1) anchor self-lag undefined for series (%s)", paste(vars[ols_self_bad], collapse = ", ")))
  if (any(ref_self_bad))
    reasons <- c(reasons, sprintf("reference self-lag undefined for series (%s)", paste(vars[ref_self_bad], collapse = ", ")))
  if (delta_rail)
    reasons <- c(reasons, sprintf("skew-normal slant at its boundary (delta = %s)", paste(sprintf("%.2f", delta_vals), collapse = ", ")))
  if (delta_bad)
    reasons <- c(reasons, sprintf("skew-normal slant summary is non-finite (delta = %s)", paste(sprintf("%.2f", delta_vals), collapse = ", ")))
  if (diag$n_divergent > 0)
    reasons <- c(reasons, sprintf("%d divergent transition%s", diag$n_divergent, if (diag$n_divergent == 1) "" else "s"))
  if (is.finite(diag$max_rhat) && diag$max_rhat > rhat_tol)
    reasons <- c(reasons, sprintf("max split-Rhat %.3f", diag$max_rhat))
  if (any(zv))
    reasons <- c(reasons, sprintf("zero-variance series (%s): self-lag undefined", paste(vars[zv], collapse = ", ")))

  ## verdict: convergence problems are a first-class signal and block "suitable"
  undefined_anchor <- any(fit_self_bad) || any(ols_self_bad) || any(ref_self_bad) || delta_bad
  verdict <- if (!atom_present && !collapse && !delta_rail && !conv_bad && !undefined_anchor) {
    "suitable"
  } else if (is_flexible && atom_present && (collapse || delta_rail || diag$n_divergent > 0)) {
    "unsuitable"
  } else {
    "caution"
  }
  recommendation <- switch(
    verdict,
    suitable = "No boundary atom detected and the sampler converged. The flexible-margin copula-VAR is appropriate; the asymmetry is a continuous feature of the innovations, separable from the dynamics.",
    unsuitable = paste0(
      "The flexible margin absorbs a boundary pile-up and collapses the dynamics. ",
      "Read the autoregressive effects from a normal margin (Gaussian quasi-maximum-likelihood recovers the conditional mean, and hence Phi, when the conditional mean is linear and only the innovation shape is distorted). ",
      "Note, however, that a genuine boundary atom usually reflects censoring, under which the conditional mean is nonlinear and the normal-margin Phi can itself be attenuated toward zero (the amount depends on the censoring severity); a censored latent VAR is then the principled remedy."
    ),
    caution = if (!is_flexible && atom_present && !collapse) {
      "A boundary atom is present. This normal-margin fit recovers the dynamics, but the margin does not model the atom, and a flexible skewed margin would risk collapsing the dynamics on these data. Prefer the normal margin for the dynamics, or a censored latent VAR if the boundedness itself is of interest (with a genuine atom the normal-margin Phi can itself be attenuated toward zero, depending on the censoring severity)."
    } else {
      sig <- c(
        if (atom_present) "a boundary atom is present",
        if (collapse) "the fitted self-lags collapse relative to the VAR(1) anchor",
        if (delta_rail) "the skew-normal slant sits at its boundary",
        if (undefined_anchor) "one or more summary anchors are undefined",
        if (conv_bad) "the sampler shows convergence problems (divergences or elevated Rhat)"
      )
      sprintf(
        "Mixed signals (%s). Inspect the fit before trusting the flexible-margin dynamics: check the slant, divergences, and the posterior-predictive fraction at the bound, and consider reading the dynamics from a normal margin or modelling the boundedness with a censored latent VAR.",
        paste(sig, collapse = "; ")
      )
    }
  )

  structure(
    list(
      verdict = verdict,
      recommendation = recommendation,
      reasons = reasons,
      vars = vars,
      margins = margins,
      atom = stats::setNames(round(atom, 3), vars),
      atom_min = stats::setNames(round(atom_min, 3), vars),
      atom_max = stats::setNames(round(atom_max, 3), vars),
      fit_self_lag = stats::setNames(round(fit_self, 3), vars),
      ols_self_lag = stats::setNames(round(ols_self, 3), vars),
      reference_self_lag = if (is.null(ref_self)) NULL else stats::setNames(round(ref_self, 3), vars),
      delta = if (is.null(delta_vals)) NULL else stats::setNames(round(delta_vals, 3), delta_names),
      diagnostics = diag,
      thresholds = list(atom_tol = atom_tol, delta_tol = delta_tol,
                        phi_collapse_tol = phi_collapse_tol,
                        ols_clear_tol = ols_clear_tol, rhat_tol = rhat_tol)
    ),
    class = "dcvar_applicability"
  )
}

#' Print a flexible-margin applicability check
#'
#' @param x A `dcvar_applicability` object, as returned by
#'   [applicability_check()].
#' @param ... Additional arguments (unused).
#' @return Invisibly returns `x`.
#' @export
print.dcvar_applicability <- function(x, ...) {
  status <- switch(x$verdict,
    suitable   = "SUITABLE",
    caution    = "CAUTION",
    unsuitable = "UNSUITABLE")
  cat(sprintf("Applicability check for the flexible-margin copula-VAR: %s\n", status))
  cat(sprintf("  margins: %s\n", paste(x$margins, collapse = ", ")))
  cat(sprintf("  boundary atom (max fraction at a bound): %s\n",
              paste(sprintf("%s %.0f%%", names(x$atom), 100 * x$atom), collapse = " | ")))
  cat(sprintf("  self-lags fitted vs OLS: %s\n",
              paste(sprintf("%s %.2f/%.2f", names(x$fit_self_lag), x$fit_self_lag, x$ols_self_lag), collapse = " | ")))
  if (!is.null(x$reference_self_lag))
    cat(sprintf("  self-lags fitted vs reference: %s\n",
                paste(sprintf("%s %.2f/%.2f", names(x$fit_self_lag), x$fit_self_lag, x$reference_self_lag), collapse = " | ")))
  if (!is.null(x$delta))
    cat(sprintf("  skew-normal slant delta: %s\n",
                paste(sprintf("%s %.2f", names(x$delta), x$delta), collapse = " | ")))
  cat(sprintf("  divergences: %d | max Rhat: %.3f\n", x$diagnostics$n_divergent, x$diagnostics$max_rhat))
  if (length(x$reasons))
    for (r in x$reasons) cat(sprintf("  - %s\n", r))
  cat(sprintf("  recommendation: %s\n", x$recommendation))
  invisible(x)
}

# Logic tests for applicability_check(). The two posterior accessors
# (var_params, dcvar_diagnostics) are mocked so the verdict logic can be driven
# deterministically without a Stan fit; rstan-gated smoke tests at the bottom
# exercise the real accessors end to end.

.mock_var_params <- function(object, ...) {
  D <- ncol(object$stan_data$Y)
  out <- list(
    Phi = data.frame(
      variable = sprintf("Phi[%d,%d]", seq_len(D), seq_len(D)),
      mean = object$.mock_phi,
      stringsAsFactors = FALSE
    )
  )
  if (!is.null(object$.mock_delta)) {
    k <- length(object$.mock_delta)
    out$delta <- data.frame(
      variable = sprintf("delta[%d]", seq_len(k)),
      mean = object$.mock_delta,
      stringsAsFactors = FALSE
    )
  }
  out
}

.mock_diagnostics <- function(object, ...) object$.mock_diag

mock_constant_fit <- function(Y, margins = "normal", phi_self,
                              delta = NULL, n_divergent = 0L, max_rhat = 1.0,
                              vars = NULL) {
  D <- ncol(Y)
  structure(
    list(
      stan_data = list(Y = Y),
      vars = if (is.null(vars)) paste0("v", seq_len(D)) else vars,
      margins = margins,
      backend = "rstan",
      .mock_phi = phi_self,
      .mock_delta = delta,
      .mock_diag = list(n_divergent = n_divergent, max_rhat = max_rhat)
    ),
    class = c("dcvar_constant_fit", "dcvar_model_fit")
  )
}

# A clean AR(1) series with no ties at the bounds.
ar1_series <- function(n, phi, sd = 0.3, seed = 1) {
  set.seed(seed)
  y <- numeric(n)
  for (t in 2:n) y[t] <- phi * y[t - 1] + stats::rnorm(1, sd = sd)
  y
}

# An AR(1) series with a floor pile-up (a genuine boundary atom).
ar1_with_floor <- function(n, phi, frac = 0.15, seed = 2) {
  y <- ar1_series(n, phi, seed = seed)
  thr <- stats::quantile(y, frac)
  y[y < thr] <- min(y)
  y
}

test_that("clean continuous series with converged sampler is suitable", {
  Y <- cbind(ar1_series(200, 0.6, seed = 1), ar1_series(200, 0.6, seed = 2))
  local_mocked_bindings(
    var_params = .mock_var_params, dcvar_diagnostics = .mock_diagnostics,
    .package = "dcvar"
  )
  res <- applicability_check(mock_constant_fit(Y, "normal", phi_self = c(0.58, 0.6)))
  expect_s3_class(res, "dcvar_applicability")
  expect_identical(res$verdict, "suitable")
  expect_length(res$reasons, 0L)
})

test_that("flexible margin with atom and collapse is unsuitable", {
  Y <- cbind(ar1_with_floor(200, 0.6), ar1_series(200, 0.6, seed = 3))
  local_mocked_bindings(
    var_params = .mock_var_params, dcvar_diagnostics = .mock_diagnostics,
    .package = "dcvar"
  )
  res <- applicability_check(
    mock_constant_fit(Y, "skew_normal", phi_self = c(0.02, 0.03), delta = c(0.99, 0.1))
  )
  expect_identical(res$verdict, "unsuitable")
  expect_true(any(grepl("boundary atom", res$reasons)))
  expect_true(any(grepl("collapse", res$reasons)))
  expect_true(any(grepl("slant at its boundary", res$reasons)))
})

test_that("convergence problems block a 'suitable' verdict (no self-contradiction)", {
  Y <- cbind(ar1_series(200, 0.6, seed = 1), ar1_series(200, 0.6, seed = 2))
  local_mocked_bindings(
    var_params = .mock_var_params, dcvar_diagnostics = .mock_diagnostics,
    .package = "dcvar"
  )
  res <- applicability_check(
    mock_constant_fit(Y, "normal", phi_self = c(0.58, 0.6),
                      n_divergent = 12L, max_rhat = 1.2)
  )
  expect_false(res$verdict == "suitable")
  expect_true(any(grepl("divergent", res$reasons)))
  expect_true(any(grepl("Rhat", res$reasons)))
})

test_that("collapse is detected for negative autoregression (two-sided anchor)", {
  Y <- cbind(ar1_series(200, -0.55, seed = 4), ar1_series(200, -0.55, seed = 5))
  local_mocked_bindings(
    var_params = .mock_var_params, dcvar_diagnostics = .mock_diagnostics,
    .package = "dcvar"
  )
  res <- applicability_check(
    mock_constant_fit(Y, "exponential", phi_self = c(0.01, 0.02))
  )
  expect_true(all(res$ols_self_lag < -0.2))
  expect_true(any(grepl("collapse", res$reasons)))
})

test_that("a lone extreme value on a short series is not counted as an atom", {
  set.seed(99)
  Y <- cbind(stats::rnorm(15), stats::rnorm(15))
  local_mocked_bindings(
    var_params = .mock_var_params, dcvar_diagnostics = .mock_diagnostics,
    .package = "dcvar"
  )
  res <- applicability_check(mock_constant_fit(Y, "normal", phi_self = c(0.1, 0.1)))
  expect_equal(unname(res$atom), c(0, 0))
  expect_identical(res$verdict, "suitable")
})

test_that("a genuine tie at the bound is counted as an atom", {
  set.seed(7)
  col <- stats::rnorm(40)
  col <- col - min(col) + 1     # shift so the series has a unique minimum at 1
  col[c(1, 2, 3)] <- 0          # introduce a clean three-way tie at a new floor
  Y <- cbind(col, stats::rnorm(40))
  local_mocked_bindings(
    var_params = .mock_var_params, dcvar_diagnostics = .mock_diagnostics,
    .package = "dcvar"
  )
  res <- applicability_check(mock_constant_fit(Y, "normal", phi_self = c(0.1, 0.1)))
  expect_equal(unname(res$atom[1]), 3 / 40)
})

test_that("undefined OLS anchor blocks a suitable verdict", {
  Y <- cbind(c(0.1, 0.3, -0.2), c(0.4, -0.1, 0.2))
  local_mocked_bindings(
    var_params = .mock_var_params, dcvar_diagnostics = .mock_diagnostics,
    .package = "dcvar"
  )
  res <- applicability_check(mock_constant_fit(Y, "normal", phi_self = c(0.5, 0.5)))
  expect_identical(res$verdict, "caution")
  expect_true(any(is.na(res$ols_self_lag)))
  expect_true(any(grepl("OLS VAR\\(1\\) anchor self-lag undefined", res$reasons)))
})

test_that("delta rail is checked for mixed margins and named by the skew dimension", {
  Y <- cbind(ar1_with_floor(200, 0.6), ar1_series(200, 0.6, seed = 8))
  local_mocked_bindings(
    var_params = .mock_var_params, dcvar_diagnostics = .mock_diagnostics,
    .package = "dcvar"
  )
  res <- applicability_check(
    mock_constant_fit(Y, c("skew_normal", "gamma"),
                      phi_self = c(0.02, 0.55), delta = c(0.99))
  )
  expect_named(res$delta, "v1")
  expect_true(any(grepl("slant at its boundary", res$reasons)))
})

test_that("non-finite delta summary produces caution instead of an error", {
  Y <- cbind(ar1_series(200, 0.6, seed = 1), ar1_series(200, 0.6, seed = 2))
  local_mocked_bindings(
    var_params = .mock_var_params, dcvar_diagnostics = .mock_diagnostics,
    .package = "dcvar"
  )
  res <- applicability_check(
    mock_constant_fit(Y, "skew_normal", phi_self = c(0.58, 0.6),
                      delta = c(NA_real_, 0.1))
  )
  expect_identical(res$verdict, "caution")
  expect_true(any(grepl("slant summary is non-finite", res$reasons)))
})

test_that("reference fit participates in collapse detection and is reported", {
  Y <- cbind(ar1_series(200, 0.6, seed = 1), ar1_series(200, 0.6, seed = 2))
  local_mocked_bindings(
    var_params = .mock_var_params, dcvar_diagnostics = .mock_diagnostics,
    .package = "dcvar"
  )
  reference <- mock_constant_fit(Y, "normal", phi_self = c(0.6, 0.6))
  fit <- mock_constant_fit(Y, "skew_normal", phi_self = c(0.01, 0.02), delta = c(0.2, 0.2))
  res <- applicability_check(fit, reference = reference)
  expect_false(is.null(res$reference_self_lag))
  expect_equal(unname(res$reference_self_lag), c(0.6, 0.6))
  expect_true(any(grepl("collapse", res$reasons)))
})

test_that("reference is validated for class and dimensionality", {
  Y <- cbind(ar1_series(60, 0.5, seed = 1), ar1_series(60, 0.5, seed = 2))
  local_mocked_bindings(
    var_params = .mock_var_params, dcvar_diagnostics = .mock_diagnostics,
    .package = "dcvar"
  )
  fit <- mock_constant_fit(Y, "normal", phi_self = c(0.5, 0.5))
  expect_error(applicability_check(fit, reference = list(1)), "reference")
  bad_dim <- mock_constant_fit(cbind(ar1_series(60, 0.5, seed = 3)), "normal", phi_self = 0.5)
  expect_error(applicability_check(fit, reference = bad_dim), "same number of variables")
  bad_n <- mock_constant_fit(
    cbind(ar1_series(50, 0.5, seed = 3), ar1_series(50, 0.5, seed = 4)),
    "normal",
    phi_self = c(0.5, 0.5)
  )
  expect_error(applicability_check(fit, reference = bad_n), "same number of observations")
  bad_vars <- mock_constant_fit(Y, "normal", phi_self = c(0.5, 0.5), vars = c("x", "y"))
  expect_error(applicability_check(fit, reference = bad_vars), "same variables")
  bad_margin <- mock_constant_fit(Y, "gamma", phi_self = c(0.5, 0.5))
  expect_error(applicability_check(fit, reference = bad_margin), "normal-margin")
})

test_that("ols_clear_tol is a tunable threshold and is reported", {
  Y <- cbind(ar1_series(200, 0.6, seed = 1), ar1_series(200, 0.6, seed = 2))
  local_mocked_bindings(
    var_params = .mock_var_params, dcvar_diagnostics = .mock_diagnostics,
    .package = "dcvar"
  )
  res <- applicability_check(mock_constant_fit(Y, "normal", phi_self = c(0.58, 0.6)))
  expect_true("ols_clear_tol" %in% names(res$thresholds))
  expect_equal(res$thresholds$ols_clear_tol, 0.20)
})

test_that("missing values in the data matrix abort with an informative error", {
  Y <- cbind(ar1_series(50, 0.5, seed = 1), ar1_series(50, 0.5, seed = 2))
  Y[5, 1] <- NA
  expect_error(
    applicability_check(mock_constant_fit(Y, "normal", phi_self = c(0.5, 0.5))),
    "missing values"
  )
})

test_that("non-constant-fit input is rejected", {
  expect_error(
    applicability_check(structure(list(), class = "dcvar_hmm_fit")),
    "dcvar_constant_fit"
  )
})

test_that("print method returns its input invisibly", {
  Y <- cbind(ar1_series(120, 0.6, seed = 1), ar1_series(120, 0.6, seed = 2))
  local_mocked_bindings(
    var_params = .mock_var_params, dcvar_diagnostics = .mock_diagnostics,
    .package = "dcvar"
  )
  res <- applicability_check(mock_constant_fit(Y, "normal", phi_self = c(0.58, 0.6)))
  expect_output(out <- withVisible(print(res)), "SUITABLE")
  expect_false(out$visible)
  expect_identical(out$value, res)
})

test_that("applicability_check runs on a real constant skew-normal fit", {
  skip_if_no_rstan()

  fit <- get_constant_skew_normal_fit()
  res <- applicability_check(fit)
  expect_s3_class(res, "dcvar_applicability")
  expect_true(res$verdict %in% c("suitable", "caution", "unsuitable"))
  expect_length(res$fit_self_lag, ncol(as.matrix(fit$stan_data$Y)))
  expect_output(print(res), "Applicability check")
})

# ============================================================================
# Trajectory Comparison Plot
# ============================================================================

#' Plot and compare multiple rho trajectory shapes
#'
#' Visualises several named trajectory scenarios side by side for comparison.
#'
#' @param n_time Number of time points.
#' @param scenarios Character vector of scenario names (see [rho_scenario()]).
#'   Default: all built-in scenarios.
#' @param ... Additional arguments passed to [rho_scenario()]. Each argument is
#'   forwarded only to the scenarios whose generator accepts it; arguments not
#'   used by any requested scenario produce a warning.
#'
#' @return A ggplot object.
#' @export
#'
#' @examples
#' plot_trajectories(100)
#' plot_trajectories(100, scenarios = c("decreasing", "single_middle"))
plot_trajectories <- function(n_time,
                              scenarios = c("constant", "decreasing", "increasing",
                                            "random_walk", "single_middle",
                                            "large_change", "double_relapse"),
                              ...) {
  unknown <- setdiff(scenarios, names(.rho_scenarios))
  if (length(unknown) > 0) {
    cli_abort("Unknown scenario{?s}: {.val {unknown}}. Available: {.val {names(.rho_scenarios)}}")
  }

  extra <- list(...)
  accepted <- lapply(scenarios, function(s) names(formals(.rho_scenarios[[s]]$fn)))
  unused <- setdiff(names(extra), unlist(accepted))
  if (length(unused) > 0) {
    cli_warn("Ignoring argument{?s} {.arg {unused}}: not used by any requested scenario.")
  }

  rows <- lapply(seq_along(scenarios), function(i) {
    s <- scenarios[[i]]
    # Forward only the arguments this scenario's generator accepts; passing
    # the shared dots verbatim would error on any scenario-specific argument.
    args <- extra[intersect(names(extra), accepted[[i]])]
    rho <- do.call(rho_scenario, c(list(scenario = s, n_time = n_time), args))
    data.frame(
      time = 2:(length(rho) + 1),
      rho = rho,
      scenario = s,
      stringsAsFactors = FALSE
    )
  })
  df <- do.call(rbind, rows)
  df$scenario <- factor(df$scenario, levels = scenarios)

  ggplot2::ggplot(df, ggplot2::aes(
    x = .data$time, y = .data$rho, color = .data$scenario
  )) +
    ggplot2::geom_line(linewidth = 0.8) +
    ggplot2::geom_hline(yintercept = 0, linetype = "dashed", color = "gray50") +
    ggplot2::scale_y_continuous(limits = c(-1, 1)) +
    ggplot2::labs(
      x = "Time",
      y = expression(rho[t]),
      color = "Scenario",
      title = "Rho Trajectory Comparison"
    ) +
    ggplot2::theme_minimal() +
    ggplot2::theme(
      plot.title = ggplot2::element_text(face = "bold"),
      legend.position = "bottom"
    )
}

#' dcvar: Copula VAR models for bivariate time series
#'
#' `dcvar` focuses on copula VAR(1) workflows for bivariate time series. Most
#' bundled models use Gaussian copulas. The core single-level Gaussian-copula
#' models [dcvar()], [dcvar_hmm()], and [dcvar_constant()] support normal,
#' exponential, skew-normal, and gamma margins. [dcvar_constant()] additionally
#' supports a Clayton copula baseline with normal margins or a per-variable
#' (mixed) margin vector.
#'
#' Per-variable (mixed) margins let each variable take its own marginal family.
#' Across all model families a length-2 `margins` vector may combine any of
#' normal, exponential, skew-normal, and gamma per dimension (routing to a
#' generic mixed-margins Stan model). Homogeneous skew-normal and gamma
#' multilevel/SEM fits also route through the mixed engines internally.
#'
#' Experimental extensions via [dcvar_multilevel()] and [dcvar_sem()] provide a
#' narrower diagnostic interface than the core models. Both support normal,
#' exponential, skew-normal, and gamma margins for single-family and
#' per-variable specifications; the SEM model also provides a naive row-mean
#' score method.
#'
#' Estimation uses Stan through `rstan` by default, with optional `cmdstanr`
#' backend support. PIT diagnostics are approximate posterior-mean plug-in
#' diagnostics. PSIS-LOO model comparison is available for the supported
#' single-level models.
#'
#' @keywords internal
"_PACKAGE"

## usethis namespace: start
#' @importFrom stats coef complete.cases cor quantile qnorm rnorm runif sd setNames
#' @importFrom utils head
#' @importFrom rlang .data %||%
#' @importFrom cli cli_abort cli_warn cli_inform cli_alert_info cli_alert_success
## usethis namespace: end
NULL

# ============================================================================
# Multilevel Simulation
# ============================================================================

#' Simulate data from a multilevel copula VAR(1) model
#'
#' Generates panel data with unit-specific VAR coefficients drawn from
#' a population distribution and a global copula correlation. The simulator
#' matches the fitted multilevel model support by leaving unit-level VAR
#' matrices unconstrained; nonstationary draws are possible.
#'
#' @param N Number of units.
#' @param n_time Number of time points per unit.
#' @param phi_bar Population mean for VAR coefficients (length-4 vector:
#'   phi11, phi12, phi21, phi22).
#' @param tau_phi Population SD for each VAR coefficient (length-4 vector).
#' @param sigma Innovation scale vector (length 2). Values are on each
#'   family's natural residual scale: innovation SD for normal/skew-normal,
#'   `sigma_exp` for exponential, and `sigma_gam` for gamma.
#' @param rho Global copula correlation.
#' @param margins Marginal family. Either a single string applied to both
#'   variables, or a length-2 character vector for per-variable (mixed) margins,
#'   e.g. `c("normal", "exponential")`. Each entry is one of `"normal"`
#'   (default), `"exponential"`, `"skew_normal"`, or `"gamma"`.
#' @param skew_direction Length-2 `1`/`-1` vector. Required whenever any
#'   dimension uses an `"exponential"` or `"gamma"` margin.
#' @param skew_params Named list of margin-specific parameters: `alpha`
#'   (length-2 skew-normal shape) and/or `shape` (scalar gamma shape).
#' @param phi_trajectory Optional population-level time-varying VAR coefficient
#'   path. Unit-specific coefficients follow this shared drift around their
#'   sampled baselines.
#' @param sigma_trajectory Optional shared time-varying innovation scale path.
#'   The supplied value is each family's natural scale (innovation SD for
#'   normal, residual SD for skew-normal, `sigma_exp` / `sigma_gam` for
#'   exponential / gamma).
#' @param tv_sigma_k Soft-barrier sharpness for time-varying exponential/gamma
#'   scales.
#' @param burnin Number of burn-in observations to discard (default: 30).
#' @param center Logical; person-mean center the data (default: `TRUE`).
#' @param seed Random seed for reproducibility.
#'
#' @return A named list with:
#'   - `data`: panel data frame with columns `id`, `time`, `y1`, `y2`
#'   - `true_params`: list of true parameter values, including `phi_bar`,
#'     `tau_phi`, `sigma`, `rho`, `margins`, `skew_direction`, `skew_params`,
#'     the per-unit VAR coefficients `Phi_mat` (an `N x 4` matrix) and
#'     `Phi_list` (a length-`N` list of `2 x 2` matrices), `Phi_population`
#'     (the shared population VAR path: `phi_bar` when constant, or an
#'     `(n_time - 1) x 4` matrix when `phi_trajectory` is supplied), and
#'     `Phi_unit_paths` (a length-`N` list of per-unit effective coefficient
#'     paths, each `(n_time - 1) x 4`)
#'   - `person_means`: N x 2 matrix of person means (before centering)
#' @export
simulate_dcvar_multilevel <- function(N = 40, n_time = 100,
                                      phi_bar = c(0.3, 0.1, 0.1, 0.3),
                                      tau_phi = c(0.1, 0.05, 0.05, 0.1),
                                      sigma = c(1, 1),
                                      rho = 0.3,
                                      margins = "normal",
                                      skew_direction = NULL,
                                      skew_params = NULL,
                                      phi_trajectory = NULL,
                                      sigma_trajectory = NULL,
                                      tv_sigma_k = NULL,
                                      burnin = 30,
                                      center = TRUE,
                                      seed = NULL) {
  if (!is.null(seed)) set.seed(seed)
  margins <- .normalize_margins_spec(margins)
  .validate_margins(margins, skew_direction)
  margins_vec <- if (length(margins) == 1L) rep(margins, 2L) else margins
  skew_params <- if (is.list(skew_params)) skew_params else list()
  if (any(margins_vec == "skew_normal")) {
    skew_params$alpha <- skew_params$alpha %||% rep(0, 2L)
  }
  if (any(margins_vec == "gamma")) {
    skew_params$shape <- skew_params$shape %||% 1
  }
  if (!is.numeric(N) || length(N) != 1L || N != as.integer(N) || N < 1) {
    cli_abort("{.arg N} must be an integer >= 1, got {.val {N}}.")
  }
  if (!is.numeric(n_time) || length(n_time) != 1L || n_time != as.integer(n_time) || n_time < 2) {
    cli_abort("{.arg n_time} must be an integer >= 2, got {.val {n_time}}.")
  }
  .simulate_validate_numeric_vector(phi_bar, "phi_bar")
  if (length(phi_bar) != 4L) {
    cli_abort("{.arg phi_bar} must have length 4, got {.val {length(phi_bar)}}.")
  }
  .simulate_validate_numeric_vector(tau_phi, "tau_phi")
  if (length(tau_phi) != 4L) {
    cli_abort("{.arg tau_phi} must have length 4, got {.val {length(tau_phi)}}.")
  }
  if (any(tau_phi < 0)) {
    cli_abort("{.arg tau_phi} values must be non-negative.")
  }
  .simulate_validate_numeric_vector(sigma, "sigma")
  if (length(sigma) != 2L) {
    cli_abort("{.arg sigma} must have length 2, got {.val {length(sigma)}}.")
  }
  if (any(sigma <= 0)) {
    cli_abort("{.arg sigma} values must be positive.")
  }
  if (!is.logical(center) || length(center) != 1L || is.na(center)) {
    cli_abort("{.arg center} must be TRUE or FALSE.")
  }
  if (!is.numeric(burnin) || length(burnin) != 1L ||
      burnin != as.integer(burnin) || burnin < 0) {
    cli_abort("{.arg burnin} must be a non-negative integer, got {.val {burnin}}.")
  }
  if (!is.numeric(rho) || length(rho) != 1 || !is.finite(rho) || rho < -1 || rho > 1) {
    cli_abort("{.arg rho} must be a single finite numeric value in [-1, 1], got {.val {rho}}.")
  }
  if (!is.null(sigma_trajectory) && !missing(sigma)) {
    cli_abort("Supply either {.arg sigma} (constant) or {.arg sigma_trajectory} (time-varying), not both.")
  }
  if (!is.null(tv_sigma_k)) {
    .simulate_validate_positive_scalar(tv_sigma_k, "tv_sigma_k")
  }

  n_time_sim <- n_time + burnin
  phi_pop <- if (is.null(phi_trajectory)) {
    matrix(rep(phi_bar, each = n_time - 1L), n_time - 1L, 4L)
  } else {
    .tv_resolve_trajectory(phi_trajectory, n_time, 4L, "phi_trajectory")
  }
  phi_dev_ret <- sweep(phi_pop, 2, phi_bar, "-")
  phi_dev_sim <- if (burnin > 0L) {
    rbind(matrix(rep(phi_dev_ret[1, ], each = burnin), burnin, 4L), phi_dev_ret)
  } else {
    phi_dev_ret
  }

  scale_ret <- if (is.null(sigma_trajectory)) {
    NULL
  } else {
    m <- .tv_resolve_trajectory(sigma_trajectory, n_time, 2L, "sigma_trajectory")
    if (any(m <= 0)) {
      cli_abort("{.arg sigma_trajectory} values must be positive.")
    }
    if (is.null(tv_sigma_k)) {
      for (d in seq_len(2L)) {
        if (margins_vec[d] %in% c("exponential", "gamma") && length(unique(m[, d])) > 1L) {
          cli_abort(c(
            "{.arg sigma_trajectory} is not constant on dimension {d} ({.val {margins_vec[d]}} margin).",
            "i" = "Set {.arg tv_sigma_k} to simulate from the soft-barrier model, or keep the scale constant."
          ))
        }
      }
    }
    m
  }
  scale_sim <- if (is.null(scale_ret)) {
    NULL
  } else if (burnin > 0L) {
    rbind(matrix(rep(scale_ret[1, ], each = burnin), burnin, 2L), scale_ret)
  } else {
    scale_ret
  }
  base_scales <- sigma

  Phi_mat <- matrix(NA_real_, N, 4)
  Phi_list <- vector("list", N)
  Phi_unit_paths <- vector("list", N)

  # Generate unit-specific VAR matrices
  for (i in seq_len(N)) {
    Phi_mat[i, ] <- rnorm(4, phi_bar, tau_phi)
    Phi_i <- matrix(Phi_mat[i, ], 2, 2, byrow = TRUE)
    Phi_list[[i]] <- Phi_i
    Phi_unit_paths[[i]] <- sweep(phi_pop, 2, phi_bar - Phi_mat[i, ], "-")
  }

  # Simulate data
  y_raw_list <- vector("list", N)
  for (i in seq_len(N)) {
    Y <- matrix(0, n_time_sim, 2)
    for (time_index in 2:n_time_sim) {
      # Correlated standard normals via Cholesky, then per-dimension margins.
      L <- matrix(c(1, rho, 0, sqrt(1 - rho^2)), 2, 2)
      z <- rnorm(2)
      w <- as.numeric(L %*% z)
      scales_t <- if (is.null(scale_sim)) base_scales else scale_sim[time_index - 1L, ]
      eps <- .sim_marginal_quantile_scaled(
        w, margins_vec, scales_t, skew_direction, skew_params,
        barrier_k = tv_sigma_k
      )

      Phi_t <- matrix(Phi_mat[i, ] + phi_dev_sim[time_index - 1L, ], 2, 2, byrow = TRUE)
      Y[time_index, ] <- Phi_t %*% Y[time_index - 1L, ] + eps
    }
    y_raw_list[[i]] <- Y[(burnin + 1L):n_time_sim, , drop = FALSE]
  }

  # Person-mean center
  person_means <- matrix(NA_real_, N, 2)
  y_list <- vector("list", N)
  for (i in seq_len(N)) {
    person_means[i, ] <- colMeans(y_raw_list[[i]])
    y_list[[i]] <- if (center) {
      sweep(y_raw_list[[i]], 2, person_means[i, ])
    } else {
      y_raw_list[[i]]
    }
  }

  # Build panel data frame
  all_rows <- vector("list", N)
  for (i in seq_len(N)) {
    all_rows[[i]] <- data.frame(
      id = i,
      time = seq_len(n_time),
      y1 = y_list[[i]][, 1],
      y2 = y_list[[i]][, 2]
    )
  }
  panel <- do.call(rbind, all_rows)

  list(
    data = panel,
    true_params = list(
      phi_bar = phi_bar,
      tau_phi = tau_phi,
      sigma = if (is.null(scale_ret)) sigma else scale_ret,
      rho = rho,
      margins = margins,
      skew_direction = skew_direction,
      skew_params = skew_params,
      Phi_mat = Phi_mat,
      Phi_list = Phi_list,
      Phi_population = if (is.null(phi_trajectory)) phi_bar else phi_pop,
      Phi_unit_paths = Phi_unit_paths
    ),
    person_means = person_means
  )
}

test_that("simulate_dcvar_multilevel returns correct structure", {
  set.seed(42)
  sim <- simulate_dcvar_multilevel(N = 5, n_time = 20, rho = 0.5, seed = 1)

  expect_type(sim, "list")
  expect_named(sim, c("data", "true_params", "person_means"))
  expect_s3_class(sim$data, "data.frame")
  expect_true(all(c("id", "time", "y1", "y2") %in% names(sim$data)))
})

test_that("simulate_dcvar_multilevel data has correct dimensions", {
  N <- 5
  n_time_val <- 20
  sim <- simulate_dcvar_multilevel(N = N, n_time = n_time_val, rho = 0.3, seed = 2)

  expect_equal(nrow(sim$data), N * n_time_val)
  expect_equal(ncol(sim$data), 4)
  expect_equal(length(unique(sim$data$id)), N)
})

test_that("simulate_dcvar_multilevel true_params contains expected elements", {
  sim <- simulate_dcvar_multilevel(N = 3, n_time = 15, rho = 0.4, seed = 3)

  expect_named(sim$true_params,
               c("phi_bar", "tau_phi", "sigma", "rho", "margins",
                 "skew_direction", "skew_params", "Phi_mat", "Phi_list",
                 "Phi_population", "Phi_unit_paths"))
  expect_equal(sim$true_params$margins, "normal")
  expect_equal(sim$true_params$rho, 0.4)
  expect_equal(length(sim$true_params$phi_bar), 4)
  expect_equal(length(sim$true_params$tau_phi), 4)
  expect_equal(length(sim$true_params$sigma), 2)
  expect_equal(nrow(sim$true_params$Phi_mat), 3)
  expect_equal(ncol(sim$true_params$Phi_mat), 4)
  expect_equal(length(sim$true_params$Phi_list), 3)
})

test_that("simulate_dcvar_multilevel person_means has correct shape", {
  N <- 8
  sim <- simulate_dcvar_multilevel(N = N, n_time = 25, rho = 0.5, seed = 4)

  expect_equal(nrow(sim$person_means), N)
  expect_equal(ncol(sim$person_means), 2)
})

test_that("simulate_dcvar_multilevel works with different N and T values", {
  sim_small <- simulate_dcvar_multilevel(N = 2, n_time = 10, rho = 0.5, seed = 5)
  expect_equal(nrow(sim_small$data), 2 * 10)

  sim_large <- simulate_dcvar_multilevel(N = 10, n_time = 50, rho = 0.5, seed = 6)
  expect_equal(nrow(sim_large$data), 10 * 50)
})

test_that("simulate_dcvar_multilevel validates rho bounds", {
  expect_error(
    simulate_dcvar_multilevel(N = 2, n_time = 10, rho = 1.2, seed = 7),
    "must be a single finite numeric value in \\[-1, 1\\]"
  )
})

test_that("simulate_dcvar_multilevel validates vector lengths and center flag", {
  expect_error(
    simulate_dcvar_multilevel(tau_phi = c(0.1, 0.2, 0.3), seed = 1),
    "tau_phi"
  )
  expect_error(
    simulate_dcvar_multilevel(sigma = c(1, 1, 1), seed = 1),
    "sigma"
  )
  expect_error(
    simulate_dcvar_multilevel(center = NA, seed = 1),
    "center"
  )
})

test_that("simulate_dcvar_multilevel uses recorded sigma for skew-normal and gamma innovations", {
  skip_if_not_installed("sn")

  sim_sn <- simulate_dcvar_multilevel(
    N = 1,
    n_time = 8000,
    burnin = 0,
    phi_bar = c(0, 0, 0, 0),
    tau_phi = c(0, 0, 0, 0),
    margins = "skew_normal",
    sigma = c(2, 2.5),
    skew_params = list(alpha = c(0, 0)),
    rho = 0,
    center = FALSE,
    seed = 31
  )
  y_sn <- as.matrix(sim_sn$data[, c("y1", "y2")])
  expect_equal(sim_sn$true_params$sigma, c(2, 2.5))
  expect_equal(unname(apply(y_sn[-1, , drop = FALSE], 2, stats::sd)), sim_sn$true_params$sigma, tolerance = 0.06)

  sim_gam <- simulate_dcvar_multilevel(
    N = 1,
    n_time = 8000,
    burnin = 0,
    phi_bar = c(0, 0, 0, 0),
    tau_phi = c(0, 0, 0, 0),
    margins = "gamma",
    sigma = c(2, 2.5),
    skew_direction = c(1, 1),
    skew_params = list(shape = 2),
    rho = 0,
    center = FALSE,
    seed = 32
  )
  y_gam <- as.matrix(sim_gam$data[, c("y1", "y2")])
  expect_equal(sim_gam$true_params$sigma, c(2, 2.5))
  expect_equal(unname(apply(y_gam[-1, , drop = FALSE], 2, stats::sd)), sim_gam$true_params$sigma, tolerance = 0.08)
})

test_that("simulate_dcvar_multilevel preserves sampled nonstationary Phi matrices", {
  sim <- simulate_dcvar_multilevel(
    N = 1,
    n_time = 20,
    phi_bar = c(1.2, 0, 0, 1.1),
    tau_phi = c(0, 0, 0, 0),
    rho = 0.5,
    seed = 8
  )

  ev <- eigen(sim$true_params$Phi_list[[1]], only.values = TRUE)$values
  expect_true(any(Mod(ev) >= 1))
  expect_equal(
    sim$true_params$Phi_list[[1]],
    matrix(c(1.2, 0, 0, 1.1), 2, 2, byrow = TRUE)
  )
})

test_that("simulate_dcvar_multilevel is reproducible with seed", {
  s1 <- simulate_dcvar_multilevel(N = 3, n_time = 15, rho = 0.5, seed = 99)
  s2 <- simulate_dcvar_multilevel(N = 3, n_time = 15, rho = 0.5, seed = 99)
  expect_identical(s1$data, s2$data)
  expect_identical(s1$true_params$Phi_mat, s2$true_params$Phi_mat)
})

test_that("simulate_dcvar_multilevel centering works", {
  sim_centered <- simulate_dcvar_multilevel(N = 3, n_time = 30, rho = 0.5,
                                            center = TRUE, seed = 10)
  sim_uncentered <- simulate_dcvar_multilevel(N = 3, n_time = 30, rho = 0.5,
                                              center = FALSE, seed = 10)

  # Centered data should have near-zero person means
  for (i in unique(sim_centered$data$id)) {
    sub <- sim_centered$data[sim_centered$data$id == i, ]
    expect_true(abs(mean(sub$y1)) < 0.5)
    expect_true(abs(mean(sub$y2)) < 0.5)
  }

  # Uncentered data differs from centered data
  expect_false(identical(sim_centered$data$y1, sim_uncentered$data$y1))
})

test_that("fitted() returns correct structure for dcvar", {
  skip_if_no_rstan()

  fit <- get_dcvar_fit()
  fit_df <- fitted(fit)

  expect_s3_class(fit_df, "data.frame")
  expect_true(all(c("time", "y1", "y2") %in% names(fit_df)))
  expect_equal(nrow(fit_df), fit$stan_data$n_time - 1)
  expect_equal(fit_df$time, attr(fit$stan_data, "time_values")[-1])
})

test_that("fitted() returns correct structure for hmm", {
  skip_if_no_rstan()

  fit <- get_hmm_fit()
  fit_df <- fitted(fit)

  expect_s3_class(fit_df, "data.frame")
  expect_equal(nrow(fit_df), fit$stan_data$n_time - 1)
})

test_that("fitted() returns correct structure for constant", {
  skip_if_no_rstan()

  fit <- get_constant_fit()
  fit_df <- fitted(fit)

  expect_s3_class(fit_df, "data.frame")
  expect_equal(nrow(fit_df), fit$stan_data$n_time - 1)
})

test_that("predict() returns correct structure", {
  skip_if_no_rstan()

  fit <- get_dcvar_fit()
  pred_df <- predict(fit)

  expect_s3_class(pred_df, "data.frame")
  expect_true(all(c("time", "variable", "mean", "lower", "upper") %in% names(pred_df)))
  # Two variables, each with T-1 rows
  expect_equal(nrow(pred_df), 2 * (fit$stan_data$n_time - 1))
  expect_true(all(pred_df$lower <= pred_df$mean))
  expect_true(all(pred_df$upper >= pred_df$mean))
})

test_that("predict() respects ci_level parameter", {
  skip_if_no_rstan()

  fit <- get_dcvar_fit()
  pred_95 <- predict(fit, ci_level = 0.95)
  pred_80 <- predict(fit, ci_level = 0.80)

  # 80% intervals should be narrower than 95%
  width_95 <- pred_95$upper - pred_95$lower
  width_80 <- pred_80$upper - pred_80$lower
  expect_true(all(width_80 < width_95))
})

test_that("predict() rejects invalid ci_level values", {
  fit <- structure(
    list(margins = "normal"),
    class = c("dcvar_fit", "dcvar_model_fit")
  )

  expect_error(predict(fit, ci_level = 0), "ci_level")
  expect_error(predict(fit, ci_level = 1), "ci_level")
  expect_error(predict(fit, ci_level = 2), "ci_level")
})

test_that("predict() works for hmm and constant", {
  skip_if_no_rstan()

  pred_hmm <- predict(get_hmm_fit())
  pred_con <- predict(get_constant_fit())

  expect_s3_class(pred_hmm, "data.frame")
  expect_s3_class(pred_con, "data.frame")
  expect_true(all(c("time", "variable", "mean", "lower", "upper") %in% names(pred_hmm)))
  expect_true(all(c("time", "variable", "mean", "lower", "upper") %in% names(pred_con)))
})

test_that("fitted() works and predict() errors for non-normal fits", {
  skip_if_no_rstan()

  fit <- get_dcvar_exponential_fit()
  fit_df <- fitted(fit)

  expect_s3_class(fit_df, "data.frame")
  expect_error(
    predict(fit),
    "only supported for normal margins"
  )
})

test_that("fitted() type='response' unstandardizes correctly", {
  skip_if_no_rstan()
  fit <- get_dcvar_fit()
  if (!isTRUE(fit$standardized)) skip("fit not standardized")

  fit_link <- fitted(fit, type = "link")
  fit_resp <- fitted(fit, type = "response")

  # Response should differ from link when standardized
  expect_false(identical(fit_link, fit_resp))

  # Response values should be on original data scale (larger magnitude)
  expect_true(any(abs(fit_resp[[fit$vars[1]]]) != abs(fit_link[[fit$vars[1]]])))
})

test_that("fitted() default type='link' is backward compatible", {
  skip_if_no_rstan()
  fit <- get_dcvar_fit()

  # Default should be same as explicit "link"
  fit_default <- fitted(fit)
  fit_link <- fitted(fit, type = "link")
  expect_equal(fit_default, fit_link)
})

test_that("predict() type='response' unstandardizes correctly", {
  skip_if_no_rstan()
  fit <- get_dcvar_fit()
  if (!isTRUE(fit$standardized)) skip("fit not standardized")

  pred_link <- predict(fit, type = "link")
  pred_resp <- predict(fit, type = "response")

  expect_false(identical(pred_link$mean, pred_resp$mean))
})

test_that("fitted() and predict() honor preserved time values", {
  skip_if_no_rstan()

  fit <- get_dcvar_fit()
  attr(fit$stan_data, "time_values") <- seq.Date(
    as.Date("2021-01-01"),
    by = "day",
    length.out = fit$stan_data$n_time
  )

  fit_df <- fitted(fit)
  pred_df <- predict(fit)

  expect_equal(fit_df$time, attr(fit$stan_data, "time_values")[-1])
  expect_equal(unique(pred_df$time), attr(fit$stan_data, "time_values")[-1])
})

test_that("multilevel and SEM fitted()/predict() honor preserved time values", {
  skip_if_no_rstan()

  ml_fit <- get_multilevel_fit()
  attr(ml_fit$stan_data, "time_values") <- seq.Date(
    as.Date("2022-01-01"),
    by = "day",
    length.out = ml_fit$stan_data$n_time
  )
  ml_fitted <- fitted(ml_fit)
  ml_pred <- predict(ml_fit)
  expect_equal(ml_fitted$time, rep(attr(ml_fit$stan_data, "time_values")[-1], ml_fit$N))
  expect_equal(unique(ml_pred$time), attr(ml_fit$stan_data, "time_values")[-1])

  sem_fit <- get_sem_fit()
  attr(sem_fit$stan_data, "time_values") <- seq.Date(
    as.Date("2022-03-01"),
    by = "day",
    length.out = sem_fit$stan_data$n_time
  )
  sem_fitted <- fitted(sem_fit, type = "link")
  sem_pred <- predict(sem_fit, type = "response")
  expect_equal(sem_fitted$time, attr(sem_fit$stan_data, "time_values"))
  expect_equal(unique(sem_pred$time), attr(sem_fit$stan_data, "time_values"))
})

test_that("predictive methods fail clearly when required Stan outputs are missing", {
  make_stub_draws <- function(variables) {
    posterior::as_draws_array(
      array(
        seq_len(4L * length(variables)),
        dim = c(4L, 1L, length(variables)),
        dimnames = list(NULL, NULL, variables)
      )
    )
  }

  single_fit <- structure(
    list(
      fit = make_stub_draws(c("Phi[1,1]", "Phi[1,2]", "Phi[2,1]", "Phi[2,2]", "sigma_eps[1]", "sigma_eps[2]")),
      stan_data = list(n_time = 5, D = 2, Y = matrix(0, nrow = 5, ncol = 2)),
      model = "dcvar",
      vars = c("y1", "y2"),
      standardized = FALSE,
      margins = "normal",
      backend = "rstan",
      priors = list(),
      meta = list()
    ),
    class = c("dcvar_fit", "dcvar_model_fit")
  )
  expect_error(fitted(single_fit), "Custom Stan files must preserve")
  expect_error(predict(single_fit), "Custom Stan files must preserve")

  sem_fit <- structure(
    list(
      fit = make_stub_draws(c("rho", "Phi[1,1]", "Phi[1,2]", "Phi[2,1]", "Phi[2,2]")),
      stan_data = list(n_time = 4),
      model = "sem",
      vars = c("latent1", "latent2"),
      J = 2,
      lambda = c(0.8, 0.8),
      sigma_e = 0.3,
      indicators = list(latent1 = c("y1_1", "y1_2"), latent2 = c("y2_1", "y2_2")),
      backend = "rstan",
      priors = list(),
      meta = list()
    ),
    class = c("dcvar_sem_fit", "dcvar_model_fit")
  )
  expect_error(fitted(sem_fit, type = "link"), "Custom Stan files must preserve")
  expect_error(predict(sem_fit, type = "response"), "Custom Stan files must preserve")
})

test_that("as.data.frame() works for dcvar_fit", {
  skip_if_no_rstan()

  fit <- get_dcvar_fit()
  df <- as.data.frame(fit)

  expect_s3_class(df, "data.frame")
  expect_true(all(c("variable", "mean", "sd", "q2.5", "q97.5",
                     "rhat", "ess_bulk", "ess_tail") %in% names(df)))
  expect_true(nrow(df) > 0)
})

test_that("as.data.frame() works for dcvar_hmm_fit", {
  skip_if_no_rstan()

  fit <- get_hmm_fit()
  df <- as.data.frame(fit)

  expect_s3_class(df, "data.frame")
  expect_true("variable" %in% names(df))
  expect_true(nrow(df) > 0)
})

test_that("as.data.frame() works for dcvar_constant_fit", {
  skip_if_no_rstan()

  fit <- get_constant_fit()
  df <- as.data.frame(fit)

  expect_s3_class(df, "data.frame")
  expect_true("variable" %in% names(df))
  expect_true(nrow(df) > 0)
})

---
title: "Comparing Copula VAR Models"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Comparing Copula VAR Models}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
has_stan <- requireNamespace("rstan", quietly = TRUE)
eval_fits <- identical(Sys.getenv("DCVAR_EVAL_VIGNETTES"), "true") && has_stan
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = eval_fits
)
```

## Overview

The dcvar package provides three copula VAR models:

1. **DC-VAR** (`dcvar()`): Time-varying correlation via a random walk on the Fisher-z scale. Best for smooth, gradual changes.

2. **HMM Copula** (`dcvar_hmm()`): Discrete regime-switching with K hidden states. Best for abrupt changes between distinct coupling regimes.

3. **Constant Copula** (`dcvar_constant()`): Single time-invariant correlation. Serves as a null/baseline model.

## Mathematical Background

For normal-margin fits, all three models share a common decomposition of the
bivariate likelihood into marginal and copula components. The package also
supports exponential, gamma, and skew-normal margins for these three models,
but the copula structure below is the common part.

### VAR(1) Marginals

Each variable follows a VAR(1) process with innovations:

$$y_t = \mu + \Phi \, (y_{t-1} - \mu) + \varepsilon_t, \qquad \varepsilon_t \sim N(0, \text{diag}(\sigma_{\varepsilon}^2))$$

The normal marginal densities provide the per-variable likelihoods in this
special case. For non-normal margins, dcvar uses the same VAR recursion and
Gaussian copula but swaps in the corresponding marginal likelihood and
transform.

### Gaussian Copula

The dependence between variables is captured by a Gaussian copula with
time-varying correlation $\rho_t$. The log-likelihood contribution of the
copula at time $t$ is:

$$\log c(u_{1t}, u_{2t}; \rho_t) = -\frac{1}{2}\log(1 - \rho_t^2) + \frac{\rho_t^2 \, (z_{1t}^2 + z_{2t}^2) - 2\rho_t \, z_{1t} z_{2t}}{2(1 - \rho_t^2)}$$

where $z_{it} = \Phi^{-1}(u_{it})$ and $u_{it} = \Phi(\varepsilon_{it} / \sigma_{\varepsilon,i})$.

### How the Models Differ

- **DC-VAR**: The correlation evolves as a random walk on the Fisher-z scale:
  $z_t = z_{t-1} + \omega_t$ with $\omega_t \sim N(0, \sigma_\omega^2)$, and
  $\rho_t = \tanh(z_t)$.

- **HMM Copula**: The correlation is state-dependent: $\rho_t = \rho_{s_t}$,
  where $s_t \in \{1, \ldots, K\}$ follows a Markov chain with transition
  matrix $A$. State-specific correlations use an ordered constraint
  ($\rho_1 < \rho_2 < \ldots$) to prevent label switching.

- **Constant Copula**: $\rho_t = \rho$ for all $t$. A special case of both
  the DC-VAR ($\sigma_\omega = 0$) and HMM ($K = 1$).

## Fitting All Three Models

```{r fit-all}
library(dcvar)

# Simulate data with a step-function trajectory
sim <- simulate_dcvar(
  n_time = 200,
  rho_trajectory = rho_step(200, rho_before = 0.7, rho_after = 0.3),
  seed = 42
)

# Fit all three models on the same data
fit_dc  <- dcvar(sim$Y_df, vars = c("y1", "y2"), seed = 1)
fit_hmm <- dcvar_hmm(sim$Y_df, vars = c("y1", "y2"), K = 2, seed = 2)
fit_con <- dcvar_constant(sim$Y_df, vars = c("y1", "y2"), seed = 3)
```

## LOO-CV Comparison

```{r compare}
dcvar_compare(dcvar = fit_dc, hmm = fit_hmm, constant = fit_con)
```

## Extracting Tidy Summaries

All fit objects support `as.data.frame()` for exporting tidy parameter
summaries. The three core time-series fits shown here also support
`fitted()`/`predict()` for one-step-ahead values; SEM and multilevel fits do
not currently implement those methods:

```{r tidy}
# Full parameter summary as a data frame
param_df <- as.data.frame(fit_dc)
head(param_df)

# Predictions with marginal intervals
pred_df <- predict(fit_hmm)
head(pred_df)
```

## HMM-Specific Outputs

The HMM model provides additional outputs related to state inference:

```{r hmm}
# State posteriors, Viterbi path, transition matrix
states <- hmm_states(fit_hmm)

# State-specific rho values
states$rho_state

# Transition matrix
states$A

# Visualise state posteriors
plot(fit_hmm, type = "states")

# Transition matrix heatmap
plot(fit_hmm, type = "transition")
```

## Clinical Interpretation

`interpret_rho_trajectory()` provides a model-aware narrative:

```{r interpret}
interpret_rho_trajectory(fit_dc)
interpret_rho_trajectory(fit_hmm)
interpret_rho_trajectory(fit_con)
```

## When to Use Which Model

- **Smooth dynamics** (therapy effects, developmental trends): Use DC-VAR
- **Abrupt regime switches** (treatment onset, relapse): Use HMM Copula
- **No time variation expected**: Use Constant Copula as baseline
- **Uncertain**: Fit all three and compare with LOO-CV

---
title: "Simulation Study Tools"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Simulation Study Tools}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
has_stan <- requireNamespace("rstan", quietly = TRUE)
eval_fits <- identical(Sys.getenv("DCVAR_EVAL_VIGNETTES"), "true") && has_stan
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = eval_fits
)
```

## Trajectory Generators

dcvar provides several trajectory generators for simulation studies. All return
a numeric vector of length T-1 (rho values for time steps 2 through n_time):

```{r trajectories, eval = TRUE}
library(dcvar)

# Smooth trajectories
rho_constant(100, rho = 0.5)
rho_decreasing(100, rho_start = 0.7, rho_end = 0.3)
rho_increasing(100, rho_start = 0.3, rho_end = 0.7)
rho_random_walk(100, sigma_omega = 0.05, seed = 42)

# Step-function trajectories
rho_step(100, rho_before = 0.7, rho_after = 0.3, breakpoint = 0.5)
rho_double_step(100, rho_levels = c(0.7, 0.3, 0.7))

# Named scenarios
rho_scenario("double_relapse", n_time = 150)
```

### Visualising Trajectories

`plot_trajectories()` shows all built-in scenarios side by side:

```{r plot-trajectories, eval = TRUE}
# Compare all built-in scenarios
plot_trajectories(100)

# Subset of scenarios
plot_trajectories(100, scenarios = c("decreasing", "single_middle", "double_relapse"))
```

## Simulating Data

`simulate_dcvar()` generates bivariate VAR(1) data with a given rho trajectory
and known ground-truth parameters:

```{r simulate, eval = TRUE}
sim <- simulate_dcvar(
  n_time = 150,
  rho_trajectory = rho_decreasing(150),
  mu = c(0, 0),
  Phi = matrix(c(0.3, 0.1, 0.1, 0.3), 2, 2),
  sigma_eps = c(1, 1),
  seed = 42
)
```

### Breakpoint Data Simulator

`simulate_breakpoint_data()` provides a simplified interface for generating
data with single or double breakpoints:

```{r breakpoint, eval = TRUE}
# Single breakpoint
sim_bp <- simulate_breakpoint_data(n_time = 100, type = "single", seed = 42)
head(sim_bp$Y_df)
sim_bp$true_params$breakpoint

# Double breakpoint
sim_bp2 <- simulate_breakpoint_data(n_time = 100, type = "double", seed = 42)
```

## Data Preparation

Before fitting, data must be prepared with `prepare_dcvar_data()`,
`prepare_hmm_data()`, or `prepare_constant_data()`. These functions handle
standardisation, prior hyperparameters, and validation:

```{r prepare, eval = TRUE}
df <- data.frame(time = 1:100, y1 = rnorm(100), y2 = rnorm(100))

# DC-VAR data (includes sigma_omega prior)
stan_data <- prepare_dcvar_data(df, vars = c("y1", "y2"))
str(stan_data[c("n_time", "D", "sigma_mu_prior", "sigma_omega_prior")])

# HMM data (includes K, kappa, alpha_off)
hmm_data <- prepare_hmm_data(df, vars = c("y1", "y2"), K = 3)
str(hmm_data[c("K", "kappa", "alpha_off")])

# Constant data (simplest)
const_data <- prepare_constant_data(df, vars = c("y1", "y2"))
```

## Parameter Recovery Metrics

After fitting, evaluate recovery with:

```{r metrics-fit, include = FALSE}
fit <- dcvar(
  sim$Y_df,
  vars = c("y1", "y2"),
  chains = 2,
  iter_warmup = 250,
  iter_sampling = 250,
  seed = 99,
  refresh = 0
)
```

```{r metrics}
# Extract estimated rho
rho_df <- rho_trajectory(fit)

# Compute recovery metrics
metrics <- compute_rho_metrics(
  rho_true = sim$true_params$rho,
  rho_est = rho_df$mean,
  rho_lower = rho_df$q2.5,
  rho_upper = rho_df$q97.5
)

metrics$bias
metrics$coverage
metrics$correlation
```

## Running a Mini Simulation Study

```{r mini-study}
n_reps <- 5
results <- lapply(1:n_reps, function(i) {
  sim <- simulate_dcvar(
    n_time = 100,
    rho_trajectory = rho_decreasing(100),
    seed = i * 1000
  )

  fit <- dcvar(sim$Y_df, vars = c("y1", "y2"),
               chains = 2, iter_warmup = 500, iter_sampling = 1000,
               seed = i, refresh = 0)

  rho_df <- rho_trajectory(fit)

  list(rho = compute_rho_metrics(
    rho_true = sim$true_params$rho,
    rho_est = rho_df$mean,
    rho_lower = rho_df$q2.5,
    rho_upper = rho_df$q97.5
  ))
})

# Aggregate across replications
agg <- aggregate_metrics(results)
print(agg$rho)
```

---
title: "Getting Started with dcvar"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Getting Started with dcvar}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
has_stan <- requireNamespace("rstan", quietly = TRUE)
eval_fits <- identical(Sys.getenv("DCVAR_EVAL_VIGNETTES"), "true") && has_stan
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = eval_fits
)
```

## Installation

The dcvar package uses [rstan](https://mc-stan.org/rstan/) as its default
backend for Bayesian model fitting.

```{r install, eval = FALSE}
# Install dcvar from CRAN
install.packages("dcvar")
```

For the development version:

```{r install-dev, eval = FALSE}
install.packages("remotes")
remotes::install_github("benlug/dcvar")
```

Optionally, you can use [cmdstanr](https://mc-stan.org/cmdstanr/) as an
alternative backend by installing it and CmdStan:

```{r install-cmdstanr, eval = FALSE}
install.packages("cmdstanr",
  repos = c("https://stan-dev.r-universe.dev", getOption("repos"))
)
cmdstanr::install_cmdstan()
```

## Simulating Data

dcvar includes trajectory generators and a data simulator for testing:

```{r simulate, eval = TRUE}
library(dcvar)

# Generate a decreasing rho trajectory (therapy effect)
rho_true <- rho_decreasing(n_time = 150, rho_start = 0.7, rho_end = 0.3)

# Simulate bivariate VAR(1) data with this trajectory
sim <- simulate_dcvar(
  n_time = 150,
  rho_trajectory = rho_true,
  seed = 42
)

head(sim$Y_df)
```

## Fitting the DC-VAR Model

The main fitting function is `dcvar()`. It accepts a data frame and the names
of two variables to model:

```{r fit}
fit <- dcvar(
  data = sim$Y_df,
  vars = c("y1", "y2"),
  chains = 4,
  iter_warmup = 1000,
  iter_sampling = 2000,
  seed = 123
)
```

## Inspecting Results

```{r inspect}
# Quick overview
print(fit)

# Detailed summary
summary(fit)

# Posterior means for VAR parameters
coef(fit)

# Time-varying rho as a data frame
rho_df <- rho_trajectory(fit)
head(rho_df)

# MCMC diagnostics
dcvar_diagnostics(fit)
```

### Fitted Values and Predictions

The `fitted()` method returns one-step-ahead fitted values from the VAR(1)
component. The `predict()` method adds marginal prediction intervals for normal
margin fits:

```{r fitted-predict}
# VAR(1) fitted values: y_hat[t] = mu + Phi * (y[t-1] - mu)
fit_df <- fitted(fit)
head(fit_df)

# Predictions with 95% marginal intervals
pred_df <- predict(fit)
head(pred_df)

# Custom interval level
pred_90 <- predict(fit, ci_level = 0.90)
```

### Tidy Parameter Summary

Use `as.data.frame()` to export the full parameter summary as a tidy data frame:

```{r as-data-frame}
param_df <- as.data.frame(fit)
head(param_df)
```

### Clinical Interpretation

`interpret_rho_trajectory()` provides a human-readable summary of the
estimated dependence trajectory:

```{r interpret}
interpret_rho_trajectory(fit)
```

## Visualisation

```{r plot}
# Rho trajectory with credible intervals
# Red line = true trajectory (simulation only)
plot_rho(fit, true_rho = sim$true_params$rho)

# VAR coefficient heatmap
plot_phi(fit)

# MCMC diagnostics (trace, Rhat, ESS)
plot(fit, type = "diagnostics")

# Posterior predictive check
# Currently available for normal and exponential margins
plot_ppc(fit)
```

## LOO-CV Model Comparison

Compare the DC-VAR against a constant-correlation baseline:

```{r compare}
fit_constant <- dcvar_constant(sim$Y_df, vars = c("y1", "y2"))

dcvar_compare(dcvar = fit, constant = fit_constant)
```

---
title: "Comparing Copula VAR Models"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Comparing Copula VAR Models}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
has_stan <- requireNamespace("rstan", quietly = TRUE)
eval_fits <- identical(Sys.getenv("DCVAR_EVAL_VIGNETTES"), "true") && has_stan
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = eval_fits
)
```

## Overview

The dcvar package provides three copula VAR models:

1. **DC-VAR** (`dcvar()`): Time-varying correlation via a random walk on the Fisher-z scale. Best for smooth, gradual changes.

2. **HMM Copula** (`dcvar_hmm()`): Discrete regime-switching with K hidden states. Best for abrupt changes between distinct coupling regimes.

3. **Constant Copula** (`dcvar_constant()`): Single time-invariant correlation. Serves as a null/baseline model.

## Mathematical Background

For normal-margin fits, all three models share a common decomposition of the
bivariate likelihood into marginal and copula components. The package also
supports exponential, gamma, and skew-normal margins for these three models,
but the copula structure below is the common part. All three single-level
models additionally support *per-variable* margins, where each variable can use
a different marginal family (see the mixed-margins section below).

### VAR(1) Marginals

Each variable follows a VAR(1) process with innovations:

$$y_t = \mu + \Phi \, (y_{t-1} - \mu) + \varepsilon_t, \qquad \varepsilon_t \sim N(0, \text{diag}(\sigma_{\varepsilon}^2))$$

The normal marginal densities provide the per-variable likelihoods in this
special case. For non-normal margins, dcvar uses the same VAR recursion and
Gaussian copula but swaps in the corresponding marginal likelihood and
transform.

### Gaussian Copula

The dependence between variables is captured by a Gaussian copula with
time-varying correlation $\rho_t$. The log-likelihood contribution of the
copula at time $t$ is:

$$\log c(u_{1t}, u_{2t}; \rho_t) = -\frac{1}{2}\log(1 - \rho_t^2) - \frac{\rho_t^2 \, (z_{1t}^2 + z_{2t}^2) - 2\rho_t \, z_{1t} z_{2t}}{2(1 - \rho_t^2)}$$

where $z_{it} = \Phi^{-1}(u_{it})$ and $u_{it} = \Phi(\varepsilon_{it} / \sigma_{\varepsilon,i})$.

### How the Models Differ

- **DC-VAR**: The correlation evolves as a random walk on the Fisher-z scale:
  $z_t = z_{t-1} + \omega_t$ with $\omega_t \sim N(0, \sigma_\omega^2)$, and
  $\rho_t = \tanh(z_t)$.

- **HMM Copula**: The correlation is state-dependent: $\rho_t = \rho_{s_t}$,
  where $s_t \in \{1, \ldots, K\}$ follows a Markov chain with transition
  matrix $A$. State-specific correlations use an ordered constraint
  ($\rho_1 < \rho_2 < \ldots$) to prevent label switching.

- **Constant Copula**: $\rho_t = \rho$ for all $t$. A special case of both
  the DC-VAR ($\sigma_\omega = 0$) and HMM ($K = 1$).

## Per-variable (mixed) margins

A copula separates the marginal distributions from the dependence structure, so
each variable is free to follow its own marginal family. `dcvar_constant()`,
`dcvar()`, and `dcvar_hmm()` expose this by accepting a length-2 `margins`
vector. For example, a roughly-symmetric first series paired with a
positively-skewed second series can be modelled as `normal` and `exponential`
margins under a single Gaussian copula:

```{r mixed-margins}
library(dcvar)

sim_mix <- simulate_dcvar(
  n_time = 200,
  rho_trajectory = rho_constant(200, rho = 0.6),
  margins = c("normal", "exponential"),
  skew_direction = c(1, 1),
  seed = 11
)

fit_mix <- dcvar_constant(
  sim_mix$Y_df,
  vars = c("y1", "y2"),
  margins = c("normal", "exponential"),
  skew_direction = c(1, 1),
  seed = 11
)

# Each dimension is reported under its own family: sigma_eps for the normal
# variable, sigma_exp for the exponential variable.
coef(fit_mix)[c("sigma_eps", "sigma_exp")]
```

`skew_direction` is required whenever any dimension uses an exponential or gamma
margin, and only those dimensions consult it. A single `margins` string still
applies the same family to both variables, and an all-identical vector such as
`c("normal", "normal")` is treated exactly like the scalar form (it reuses the
specialised single-family model), so existing code is unaffected. Mixed margins
currently require the Gaussian copula and are available for the three
single-level models (`dcvar_constant()`, `dcvar()`, `dcvar_hmm()`).

## Time-varying coefficients and scales

In `dcvar()` the copula correlation always evolves over time. The
autoregressive dynamics and the residual scales can be allowed to evolve as
well, each as a random walk around a constant baseline. Two flags turn these on:

- `tv_phi` lets the VAR(1) coefficients drift. `tv_phi = TRUE` varies all four
  (the autoregressive coefficients phi11, phi22 and the cross-lagged
  coefficients phi12, phi21); a selector varies only a subset. Use `"ar"` to
  let only the autoregressive coefficients drift (for example to capture
  changing inertia), `"cross"` for only the cross-lagged coefficients, or pass
  specific names such as `c("phi11", "phi22")`.
- `tv_sigma` lets the residual scales drift. Normal and skew-normal dimensions
  scale multiplicatively on the log scale. Exponential and gamma dimensions use
  a soft-barrier transform so their scale can vary even though the shifted
  margin has a hard support boundary; `tv_sigma_k` controls how closely it
  approximates the exact margin.

With both flags off, `dcvar()` is the usual dynamic copula model with constant
coefficients and scales. Smaller models are nested in larger ones, which
suggests fitting a short ladder and comparing the fits.

```{r tv-fit}
library(dcvar)

# A series with declining coupling to illustrate the time-varying fits
sim_tv <- simulate_dcvar(
  n_time = 150,
  rho_trajectory = rho_decreasing(150, rho_start = 0.7, rho_end = 0.3),
  seed = 3
)

# Only the autoregressive coefficients evolve over time
fit_ar <- dcvar(
  sim_tv$Y_df,
  vars = c("y1", "y2"),
  tv_phi = "ar",
  seed = 4
)

# Time-varying coefficients and time-varying scales together, with a
# right-skewed second margin (the exponential dimension uses the soft barrier)
fit_full <- dcvar(
  sim_tv$Y_df,
  vars = c("y1", "y2"),
  margins = c("normal", "exponential"),
  skew_direction = c(1, 1),
  tv_phi = TRUE,
  tv_sigma = TRUE,
  seed = 5
)
```

The coefficient and scale paths come back as posterior summaries, in the same
shape as the correlation trajectory, and have their own plotting helpers.

```{r tv-extract}
# Posterior summaries of phi11(t), phi12(t), phi21(t), phi22(t)
head(phi_trajectory(fit_ar))

# Per-variable residual-scale paths
head(sigma_trajectory(fit_full))

# Facetted trajectory plots
plot_phi_trajectory(fit_ar)
plot_sigma_trajectory(fit_full)
```

## Fitting All Three Models

```{r fit-all}
library(dcvar)

# Simulate data with a step-function trajectory
sim <- simulate_dcvar(
  n_time = 200,
  rho_trajectory = rho_step(200, rho_before = 0.7, rho_after = 0.3),
  seed = 42
)

# Fit all three models on the same data
fit_dc  <- dcvar(sim$Y_df, vars = c("y1", "y2"), seed = 1)
fit_hmm <- dcvar_hmm(sim$Y_df, vars = c("y1", "y2"), K = 2, seed = 2)
fit_con <- dcvar_constant(sim$Y_df, vars = c("y1", "y2"), seed = 3)
```

## LOO-CV Comparison

```{r compare}
dcvar_compare(dcvar = fit_dc, hmm = fit_hmm, constant = fit_con)
```

## Extracting Tidy Summaries

All fit objects support `as.data.frame()` for exporting tidy parameter
summaries. All five fit families also support `fitted()`/`predict()` for
one-step-ahead values (for SEM and multilevel fits as well). Prediction
intervals are restricted to normal margins for single-level, multilevel, and
naive-SEM fits; indicator-SEM predictions are available for all supported SEM
margins:

```{r tidy}
# Full parameter summary as a data frame
param_df <- as.data.frame(fit_dc)
head(param_df)

# Predictions with marginal intervals
pred_df <- predict(fit_hmm)
head(pred_df)
```

## HMM-Specific Outputs

The HMM model provides additional outputs related to state inference:

```{r hmm}
# State posteriors, Viterbi path, transition matrix
states <- hmm_states(fit_hmm)

# State-specific rho values
states$rho_state

# Transition matrix
states$A

# Visualise state posteriors
plot(fit_hmm, type = "states")

# Transition matrix heatmap
plot(fit_hmm, type = "transition")
```

## Clinical Interpretation

`interpret_rho_trajectory()` provides a model-aware narrative:

```{r interpret}
interpret_rho_trajectory(fit_dc)
interpret_rho_trajectory(fit_hmm)
interpret_rho_trajectory(fit_con)
```

## When to Use Which Model

- **Smooth dynamics** (therapy effects, developmental trends): Use DC-VAR
- **Abrupt regime switches** (treatment onset, relapse): Use HMM Copula
- **No time variation expected**: Use Constant Copula as baseline
- **Uncertain**: Fit all three and compare with LOO-CV

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data-preparation.R
\name{prepare_constant_data}
\alias{prepare_constant_data}
\title{Prepare data for the constant copula model}
\usage{
prepare_constant_data(
  data,
  vars,
  time_var = "time",
  standardize = TRUE,
  margins = "normal",
  skew_direction = NULL,
  prior_mu_sd = 2,
  prior_phi_sd = 0.5,
  prior_sigma_eps_rate = 1,
  prior_z_rho_sd = 1,
  allow_gaps = FALSE
)
}
\arguments{
\item{data}{A data frame with time series observations.}

\item{vars}{Character vector of two variable names to model.}

\item{time_var}{Name of the time column (default: \code{"time"}).}

\item{standardize}{Logical; whether to z-score variables (default: \code{TRUE}).}

\item{margins}{Marginal distribution specification. Either a single string
applied to both variables, or a length-2 character vector giving a
per-variable (mixed) margin (for example \code{c("normal", "exponential")}).
Each entry is one of \code{"normal"} (default), \code{"exponential"},
\code{"skew_normal"}, or \code{"gamma"}. When the two entries differ the returned
data carries an integer \code{family} array for the generic mixed-margins Stan
model; identical entries are equivalent to the scalar form and reuse the
specialised single-family model.}

\item{skew_direction}{Integer vector of length 2 of \code{1} (right-skewed) or
\code{-1} (left-skewed). Required whenever any dimension uses an
\code{"exponential"} or \code{"gamma"} margin; only those dimensions consult it.}

\item{prior_mu_sd}{Prior SD for intercepts: \code{mu ~ normal(0, prior_mu_sd)}.}

\item{prior_phi_sd}{Prior SD for VAR coefficients: \code{Phi ~ normal(0, prior_phi_sd)}.}

\item{prior_sigma_eps_rate}{Prior mean for innovation SDs:
\code{sigma_eps ~ exponential(1/prior_sigma_eps_rate)}. Default \code{1} gives
\code{exponential(1)} with prior mean 1.}

\item{prior_z_rho_sd}{Prior SD for rho on Fisher-z scale (default: 1.0).}

\item{allow_gaps}{Logical; if \code{FALSE} (default), interior missing values
cause an error. If \code{TRUE}, they produce a warning and are removed.}
}
\value{
A named list suitable as Stan data input.
}
\description{
Transforms a data frame into a list suitable for the constant copula Stan model.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extract.R
\name{hmm_state_params}
\alias{hmm_state_params}
\alias{hmm_state_params.default}
\alias{hmm_state_params.dcvar_hmm_fit}
\title{Extract per-state parameters from a Markov-switching HMM fit}
\usage{
hmm_state_params(object, ...)

\method{hmm_state_params}{default}(object, ...)

\method{hmm_state_params}{dcvar_hmm_fit}(object, ...)
}
\arguments{
\item{object}{A \code{dcvar_hmm_fit} object.}

\item{...}{Additional arguments (unused).}
}
\value{
A list with \code{families} (a K x D character matrix), \code{rho_state}
(length-K), \code{mu} (a K x D matrix of posterior means), \code{Phi} (a length-K list
of D x D posterior-mean coefficient matrices), and \code{scales} (a data frame of
per-state margin scale/shape posterior means; shared scales are tiled across
states).
}
\description{
Assembles the state-specific VAR parameters of a switching \code{dcvar_hmm()} fit
into a per-state view: the intercepts, the effective VAR(1) coefficient matrix
(\code{Phi_base} plus the per-state deviations on the switching coefficients), the
margin scale/shape parameters, and the per-state marginal family labels. For
components that are NOT state-specific the shared estimate is tiled across all
states, so the return shape does not depend on which components switch.
}
\seealso{
\code{\link[=hmm_states]{hmm_states()}}, \code{\link[=var_params]{var_params()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/metrics.R
\name{aggregate_metrics}
\alias{aggregate_metrics}
\title{Aggregate metrics across simulation replications}
\usage{
aggregate_metrics(metrics_list)
}
\arguments{
\item{metrics_list}{A list of metric lists (one per replication), each
containing a \verb{$rho} element as returned by \code{\link[=compute_rho_metrics]{compute_rho_metrics()}}.}
}
\value{
A named list with:
\itemize{
\item \code{rho}: data frame of aggregated rho metrics (mean, SD, quantiles) for
every field in the per-replication \code{rho} metric list
\item \code{n_reps}: number of replications
}
}
\description{
Aggregate metrics across simulation replications
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit-hmm.R
\name{dcvar_hmm}
\alias{dcvar_hmm}
\title{Fit the HMM copula model}
\usage{
dcvar_hmm(
  data,
  vars,
  K = 2,
  time_var = "time",
  standardize = TRUE,
  margins = "normal",
  skew_direction = NULL,
  switch = "rho",
  allow_gaps = FALSE,
  prior_mu_sd = 2,
  prior_phi_sd = 0.5,
  prior_sigma_eps_rate = 1,
  prior_kappa = 10,
  prior_alpha_off = 1,
  prior_z_rho_sd = 1,
  chains = 4,
  iter_warmup = 2000,
  iter_sampling = 4000,
  adapt_delta = 0.99,
  max_treedepth = 12,
  seed = NULL,
  cores = NULL,
  refresh = 500,
  init = NULL,
  stan_file = NULL,
  backend = getOption("dcvar.backend", "auto"),
  ...
)
}
\arguments{
\item{data}{A data frame with time series observations.}

\item{vars}{Character vector of two variable names to model.}

\item{K}{Number of hidden states (default: 2).}

\item{time_var}{Name of the time column (default: \code{"time"}).}

\item{standardize}{Logical; whether to z-score variables (default: \code{TRUE}).}

\item{margins}{Marginal distribution specification. Either a single string
applied to both variables, a length-2 character vector giving a per-variable
(mixed) margin, e.g. \code{c("normal", "exponential")}, or -- for a state-specific
family -- a length-\code{K} list of such specifications, one per hidden state,
e.g. \code{list(c("normal", "normal"), c("exponential", "gamma"))}. Each entry is
one of \code{"normal"} (default), \code{"exponential"}, \code{"skew_normal"}, or \code{"gamma"}.
A per-state list is consumed in increasing-\code{rho} order.}

\item{skew_direction}{Skew direction for asymmetric margins: each element \code{1}
(right-skewed) or \code{-1} (left-skewed). A length-2 vector (recycled across
states) or, for per-state margins, a length-\code{K} list of length-2 vectors.
Required whenever any (state, dimension) uses an \code{"exponential"} or
\code{"gamma"} margin; only those entries consult it.}

\item{switch}{Which parameters become state-specific. \code{"rho"} (default) keeps
only the copula correlation switching (the classic HMM). A character vector
drawn from \code{c("rho", "mu", "phi", "ar", "cross", "sigma")} (with \code{"ar"} /
\code{"cross"} / coefficient names like \code{"phi11"} selecting a subset of the VAR
coefficients), or \code{TRUE} for everything. \code{rho} must be included whenever any
other component switches (it is the label-switching anchor).}

\item{allow_gaps}{Logical; if \code{FALSE} (default), interior missing values
cause an error because they break VAR(1) time series adjacency. Set to
\code{TRUE} to allow fitting with a warning instead.}

\item{prior_mu_sd}{Prior SD for intercepts: \code{mu ~ normal(0, prior_mu_sd)}.}

\item{prior_phi_sd}{Prior SD for VAR coefficients: \code{Phi ~ normal(0, prior_phi_sd)}.}

\item{prior_sigma_eps_rate}{Prior mean for innovation SDs (see
\code{\link[=prepare_dcvar_data]{prepare_dcvar_data()}}).}

\item{prior_kappa}{Sticky Dirichlet self-transition concentration (default: 10).}

\item{prior_alpha_off}{Sticky Dirichlet off-diagonal concentration (default: 1).}

\item{prior_z_rho_sd}{Prior SD for state-specific z_rho (default: 1.0).}

\item{chains}{Number of MCMC chains (default: 4).}

\item{iter_warmup}{Warmup iterations per chain (default: 2000).}

\item{iter_sampling}{Sampling iterations per chain (default: 4000).}

\item{adapt_delta}{Target acceptance rate (default: 0.99). The DC-VAR model
uses a lower default than \code{dcvar_constant()} (0.999) because the
non-centered parameterisation already handles posterior geometry well.}

\item{max_treedepth}{Maximum tree depth (default: 12).}

\item{seed}{Random seed. When supplied, it seeds both the Stan sampler and
the default per-chain initial values, so repeated fits are reproducible.}

\item{cores}{Number of parallel chains. \code{NULL} uses all available cores.}

\item{refresh}{How often to print progress (default: 500). Set to 0 for
silent operation.}

\item{init}{Custom init function or \code{NULL} for smart defaults.}

\item{stan_file}{Path to a custom Stan file, or \code{NULL} to use the bundled
model.}

\item{backend}{Character: \code{"auto"} (default, uses rstan), \code{"rstan"}, or
\code{"cmdstanr"}. Can also be set globally via
\code{options(dcvar.backend = "cmdstanr")}.}

\item{...}{Additional backend-specific sampling arguments.}
}
\value{
A \code{dcvar_hmm_fit} object.
}
\description{
Fits a Hidden Markov Model copula VAR(1) with K discrete states and
state-specific correlations. Uses ordered z_rho constraint to prevent
label switching and a sticky Dirichlet prior to encourage state persistence.
}
\section{State-specific parameters and per-state margins}{

By default only the copula correlation switches by state. The \code{switch}
argument turns a full Markov-switching VAR on: the intercepts, the VAR
coefficients, and the residual scales can each be made state-specific. The
marginal \strong{family} can also switch by state by passing a length-\code{K}
\code{margins} list. Because the states are identified by an ordered correlation
(\code{z_rho}), \code{rho} is the mandatory anchor and a per-state \code{margins} list is
consumed in increasing-correlation order (\code{margins[[1]]} is the lowest-rho
state). Richly-parameterized configurations on a single short bivariate series
are weakly identified; fit a ladder (\code{switch = "rho"} ->
\code{switch = c("rho", "ar")} -> full) and widen \code{prior_phi_sd}. See
\code{\link[=hmm_state_params]{hmm_state_params()}} for the per-state parameter view.
}

\examples{
\donttest{
sim <- simulate_dcvar(
  n_time = 12,
  rho_trajectory = rho_step(12),
  seed = 1
)
fit <- dcvar_hmm(
  sim$Y_df,
  vars = c("y1", "y2"),
  K = 2,
  chains = 1,
  iter_warmup = 10,
  iter_sampling = 10,
  refresh = 0,
  seed = 1
)
print(fit)
hmm_states(fit)
}
}
\seealso{
\code{\link[=dcvar]{dcvar()}} for the smooth time-varying model,
\code{\link[=dcvar_constant]{dcvar_constant()}} for the time-invariant baseline,
\code{\link[=hmm_states]{hmm_states()}} for state extraction, \code{\link[=plot_hmm_states]{plot_hmm_states()}} for visualisation,
\code{\link[=dcvar_compare]{dcvar_compare()}} for LOO-CV model comparison.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pit.R
\name{pit_values}
\alias{pit_values}
\alias{pit_values.default}
\alias{pit_values.dcvar_hmm_fit}
\alias{pit_values.dcvar_model_fit}
\alias{pit_values.dcvar_tv_fit}
\title{Extract PIT values from a fitted model}
\usage{
pit_values(object, ...)

\method{pit_values}{default}(object, ...)

\method{pit_values}{dcvar_hmm_fit}(object, ...)

\method{pit_values}{dcvar_model_fit}(object, ...)

\method{pit_values}{dcvar_tv_fit}(object, ...)
}
\arguments{
\item{object}{A fitted model object.}

\item{...}{Additional arguments (unused).}
}
\value{
A data frame with columns \code{time}, \code{variable}, \code{pit}.
}
\description{
Computes approximate Probability Integral Transform values using posterior
mean residuals and posterior mean margin parameters. Large departures from
uniformity can indicate model misfit, but these are not exact posterior
predictive PIT values.
}
\details{
PIT values are computed from posterior mean residuals and posterior mean
margin parameters. Treat them as a fast plug-in diagnostic rather than an
exact posterior predictive transform that integrates over full posterior
uncertainty. PIT diagnostics are currently implemented for the three core
single-level fit classes only.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/loo.R
\name{loo.dcvar}
\alias{loo.dcvar}
\alias{loo.dcvar_fit}
\alias{loo.dcvar_covariate_fit}
\alias{loo.dcvar_hmm_fit}
\alias{loo.dcvar_constant_fit}
\alias{loo.dcvar_multilevel_fit}
\alias{loo.dcvar_sem_fit}
\title{Compute LOO-CV for a fitted model}
\usage{
\method{loo}{dcvar_fit}(x, ...)

\method{loo}{dcvar_covariate_fit}(x, ...)

\method{loo}{dcvar_hmm_fit}(x, ...)

\method{loo}{dcvar_constant_fit}(x, ...)

\method{loo}{dcvar_multilevel_fit}(x, ...)

\method{loo}{dcvar_sem_fit}(x, ...)
}
\arguments{
\item{x}{A fitted model object.}

\item{...}{Additional arguments passed to \code{\link[loo:loo]{loo::loo()}}.}
}
\value{
A \code{loo} object from the loo package.
}
\description{
Compute LOO-CV for a fitted model
}
\details{
PSIS-LOO is available for Gaussian and Clayton single-level fits,
covariate fits, multilevel fits, and naive SEM score fits. Indicator SEM
fits are not supported because their stored \code{log_lik} conditions on
latent innovations rather than being an observation-level predictive
density. Multilevel \code{log_lik} values are conditional one-step-ahead
densities given the unit-level random effects.

Note that the pointwise \code{log_lik} estimands differ across model families:
HMM fits store the state-marginalized one-step-ahead predictive density,
while dynamic (DC-VAR and covariate) fits condition on the smoothed
latent \code{rho} trajectory, which is itself informed by the observation.
Comparisons across these families can systematically favor the
latent-conditioned models; see \code{\link[=dcvar_compare]{dcvar_compare()}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/covariate_fit-class.R
\name{print.dcvar_covariate_summary}
\alias{print.dcvar_covariate_summary}
\title{Print a dcvar_covariate_summary object}
\usage{
\method{print}{dcvar_covariate_summary}(x, ...)
}
\arguments{
\item{x}{A \code{dcvar_covariate_summary} object as returned by \code{summary()}.}

\item{...}{Additional arguments (unused).}
}
\value{
Invisibly returns \code{x}.
}
\description{
Print a dcvar_covariate_summary object
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/covariate_fit-class.R
\name{dcvar_covariate_fit-methods}
\alias{dcvar_covariate_fit-methods}
\alias{print.dcvar_covariate_fit}
\alias{summary.dcvar_covariate_fit}
\alias{coef.dcvar_covariate_fit}
\alias{plot.dcvar_covariate_fit}
\title{S3 methods for covariate DC-VAR fits}
\usage{
\method{print}{dcvar_covariate_fit}(x, ...)

\method{summary}{dcvar_covariate_fit}(object, probs = c(0.025, 0.5, 0.975), ...)

\method{coef}{dcvar_covariate_fit}(object, ...)

\method{plot}{dcvar_covariate_fit}(x, type = c("rho", "phi", "diagnostics", "ppc", "pit"), ...)
}
\arguments{
\item{x, object}{A \code{dcvar_covariate_fit} object.}

\item{...}{Additional arguments (unused).}

\item{probs}{Numeric vector of quantile probabilities (default:
\code{c(0.025, 0.5, 0.975)}).}

\item{type}{Character; one of \code{"rho"}, \code{"phi"}, \code{"diagnostics"}, \code{"ppc"},
or \code{"pit"}.}
}
\value{
Invisibly returns \code{x}.

A \code{dcvar_covariate_summary} object (a list).

A named list with elements \code{mu}, \code{Phi}, \code{sigma_eps}, \code{beta_0},
\code{beta}, and, when \code{drift = TRUE}, \code{sigma_omega}.

A ggplot object.
}
\description{
S3 methods for covariate DC-VAR fits
}
\section{Functions}{
\itemize{
\item \code{print(dcvar_covariate_fit)}: Print a concise overview of the
covariate DC-VAR fit.

\item \code{summary(dcvar_covariate_fit)}: Produce a detailed summary including
rho trajectory, VAR parameters, covariate effects, and diagnostics.

\item \code{coef(dcvar_covariate_fit)}: Extract posterior means of model
coefficients.

\item \code{plot(dcvar_covariate_fit)}: Dispatch to a plot type: \code{"rho"},
\code{"phi"}, \code{"diagnostics"}, \code{"ppc"}, or \code{"pit"}.

}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit-covariate.R
\name{dcvar_covariate}
\alias{dcvar_covariate}
\title{Fit the covariate DC-VAR model}
\usage{
dcvar_covariate(
  data,
  vars,
  covariates,
  time_var = "time",
  standardize = TRUE,
  standardize_covariates = FALSE,
  drift = TRUE,
  zero_init_eta = TRUE,
  allow_gaps = FALSE,
  prior_mu_sd = 2,
  prior_phi_sd = 0.5,
  prior_sigma_eps_rate = 1,
  prior_sigma_omega_rate = 0.1,
  prior_rho_init_sd = 1,
  prior_beta_sd = 1,
  chains = 4,
  iter_warmup = 2000,
  iter_sampling = 4000,
  adapt_delta = 0.99,
  max_treedepth = 12,
  seed = NULL,
  cores = NULL,
  refresh = 500,
  init = NULL,
  stan_file = NULL,
  backend = getOption("dcvar.backend", "auto"),
  ...
)
}
\arguments{
\item{data}{A data frame with time series observations.}

\item{vars}{Character vector of two variable names to model.}

\item{covariates}{Character vector of covariate column names.}

\item{time_var}{Name of the time column (default: \code{"time"}).}

\item{standardize}{Logical; whether to z-score variables (default: \code{TRUE}).}

\item{standardize_covariates}{Logical; whether to z-score covariates
(default: \code{FALSE}).}

\item{drift}{Logical; if \code{TRUE} (default), include residual random-walk drift
after the covariate effect. If \code{FALSE}, fit the no-drift sensitivity model.}

\item{zero_init_eta}{Logical; if \code{TRUE} (default), fixes \code{eta[1] = 0} in the
residual-drift model.}

\item{allow_gaps}{Logical; if \code{FALSE} (default), interior missing values
cause an error because they break VAR(1) time series adjacency. Set to
\code{TRUE} to allow fitting with a warning instead.}

\item{prior_mu_sd}{Prior SD for intercepts: \code{mu ~ normal(0, prior_mu_sd)}.}

\item{prior_phi_sd}{Prior SD for VAR coefficients: \code{Phi ~ normal(0, prior_phi_sd)}.}

\item{prior_sigma_eps_rate}{Prior mean for innovation SDs (see
\code{\link[=prepare_dcvar_data]{prepare_dcvar_data()}}).}

\item{prior_sigma_omega_rate}{Prior mean for rho process SD (see
\code{\link[=prepare_dcvar_data]{prepare_dcvar_data()}}).}

\item{prior_rho_init_sd}{Prior SD for the dependence intercept \code{beta_0} on
the Fisher-z scale (the covariate model has no separate initial-rho
parameter; \code{beta_0} plays that role).}

\item{prior_beta_sd}{Prior SD for covariate effects.}

\item{chains}{Number of MCMC chains (default: 4).}

\item{iter_warmup}{Warmup iterations per chain (default: 2000).}

\item{iter_sampling}{Sampling iterations per chain (default: 4000).}

\item{adapt_delta}{Target acceptance rate (default: 0.99). The DC-VAR model
uses a lower default than \code{dcvar_constant()} (0.999) because the
non-centered parameterisation already handles posterior geometry well.}

\item{max_treedepth}{Maximum tree depth (default: 12).}

\item{seed}{Random seed. When supplied, it seeds both the Stan sampler and
the default per-chain initial values, so repeated fits are reproducible.}

\item{cores}{Number of parallel chains. \code{NULL} uses all available cores.}

\item{refresh}{How often to print progress (default: 500). Set to 0 for
silent operation.}

\item{init}{Custom init function or \code{NULL} for smart defaults.}

\item{stan_file}{Path to a custom Stan file, or \code{NULL} to use the bundled
model.}

\item{backend}{Character: \code{"auto"} (default, uses rstan), \code{"rstan"}, or
\code{"cmdstanr"}. Can also be set globally via
\code{options(dcvar.backend = "cmdstanr")}.}

\item{...}{Additional backend-specific sampling arguments.}
}
\value{
A \code{dcvar_covariate_fit} object.
}
\description{
Fits a Gaussian Dynamic Copula VAR(1) model in which contemporaneous
innovation dependence varies on the Fisher-z scale as a function of observed
covariates. With \code{drift = TRUE}, the model adds residual random-walk drift:
\verb{zeta_i = beta_0 + x_\{i+1\}' beta + eta_i}. With \code{drift = FALSE}, the
dependence trajectory is explained entirely by the covariates.
}
\examples{
\donttest{
sim <- simulate_dcvar(
  n_time = 12,
  rho_trajectory = rho_step(12),
  seed = 1
)
sim$Y_df$phase <- as.numeric(sim$Y_df$time > 6)
fit <- dcvar_covariate(
  sim$Y_df,
  vars = c("y1", "y2"),
  covariates = "phase",
  chains = 1,
  iter_warmup = 10,
  iter_sampling = 10,
  refresh = 0,
  seed = 1
)
covariate_effects(fit)

fit_nodrift <- dcvar_covariate(
  sim$Y_df,
  vars = c("y1", "y2"),
  covariates = "phase",
  drift = FALSE,
  chains = 1,
  iter_warmup = 10,
  iter_sampling = 10,
  refresh = 0,
  seed = 1
)
rho_trajectory(fit_nodrift)
}
}
\seealso{
\code{\link[=prepare_dcvar_covariate_data]{prepare_dcvar_covariate_data()}}, \code{\link[=covariate_effects]{covariate_effects()}},
\code{\link[=rho_trajectory]{rho_trajectory()}}, and \code{\link[=dcvar]{dcvar()}} for the covariate-free random-walk model.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{plot_diagnostics}
\alias{plot_diagnostics}
\title{Plot MCMC diagnostics}
\usage{
plot_diagnostics(object, ...)
}
\arguments{
\item{object}{A fitted model object.}

\item{...}{Additional arguments (unused).}
}
\value{
A combined ggplot object (via patchwork).
}
\description{
Creates a combined panel with trace plots, Rhat, and ESS diagnostics.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simulate-multilevel.R
\name{simulate_dcvar_multilevel}
\alias{simulate_dcvar_multilevel}
\title{Simulate data from a multilevel copula VAR(1) model}
\usage{
simulate_dcvar_multilevel(
  N = 40,
  n_time = 100,
  phi_bar = c(0.3, 0.1, 0.1, 0.3),
  tau_phi = c(0.1, 0.05, 0.05, 0.1),
  sigma = c(1, 1),
  rho = 0.3,
  margins = "normal",
  skew_direction = NULL,
  skew_params = NULL,
  phi_trajectory = NULL,
  sigma_trajectory = NULL,
  tv_sigma_k = NULL,
  burnin = 30,
  center = TRUE,
  seed = NULL
)
}
\arguments{
\item{N}{Number of units.}

\item{n_time}{Number of time points per unit.}

\item{phi_bar}{Population mean for VAR coefficients (length-4 vector:
phi11, phi12, phi21, phi22).}

\item{tau_phi}{Population SD for each VAR coefficient (length-4 vector).}

\item{sigma}{Innovation scale vector (length 2). Values are on each
family's natural residual scale: innovation SD for normal/skew-normal,
\code{sigma_exp} for exponential, and \code{sigma_gam} for gamma.}

\item{rho}{Global copula correlation.}

\item{margins}{Marginal family. Either a single string applied to both
variables, or a length-2 character vector for per-variable (mixed) margins,
e.g. \code{c("normal", "exponential")}. Each entry is one of \code{"normal"}
(default), \code{"exponential"}, \code{"skew_normal"}, or \code{"gamma"}.}

\item{skew_direction}{Length-2 \code{1}/\code{-1} vector. Required whenever any
dimension uses an \code{"exponential"} or \code{"gamma"} margin.}

\item{skew_params}{Named list of margin-specific parameters: \code{alpha}
(length-2 skew-normal shape) and/or \code{shape} (scalar gamma shape).}

\item{phi_trajectory}{Optional population-level time-varying VAR coefficient
path. Unit-specific coefficients follow this shared drift around their
sampled baselines.}

\item{sigma_trajectory}{Optional shared time-varying innovation scale path.
The supplied value is each family's natural scale (innovation SD for
normal, residual SD for skew-normal, \code{sigma_exp} / \code{sigma_gam} for
exponential / gamma).}

\item{tv_sigma_k}{Soft-barrier sharpness for time-varying exponential/gamma
scales.}

\item{burnin}{Number of burn-in observations to discard (default: 30).}

\item{center}{Logical; person-mean center the data (default: \code{TRUE}).}

\item{seed}{Random seed for reproducibility.}
}
\value{
A named list with:
\itemize{
\item \code{data}: panel data frame with columns \code{id}, \code{time}, \code{y1}, \code{y2}
\item \code{true_params}: list of true parameter values, including \code{phi_bar},
\code{tau_phi}, \code{sigma}, \code{rho}, \code{margins}, \code{skew_direction}, \code{skew_params},
the per-unit VAR coefficients \code{Phi_mat} (an \verb{N x 4} matrix) and
\code{Phi_list} (a length-\code{N} list of \verb{2 x 2} matrices), \code{Phi_population}
(the shared population VAR path: \code{phi_bar} when constant, or an
\verb{(n_time - 1) x 4} matrix when \code{phi_trajectory} is supplied), and
\code{Phi_unit_paths} (a length-\code{N} list of per-unit effective coefficient
paths, each \verb{(n_time - 1) x 4})
\item \code{person_means}: N x 2 matrix of person means (before centering)
}
}
\description{
Generates panel data with unit-specific VAR coefficients drawn from
a population distribution and a global copula correlation. The simulator
matches the fitted multilevel model support by leaving unit-level VAR
matrices unconstrained; nonstationary draws are possible.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extract.R
\name{random_effects}
\alias{random_effects}
\alias{random_effects.default}
\alias{random_effects.dcvar_multilevel_fit}
\title{Extract random effects from a multilevel fit}
\usage{
random_effects(object, ...)

\method{random_effects}{default}(object, ...)

\method{random_effects}{dcvar_multilevel_fit}(object, ...)
}
\arguments{
\item{object}{A \code{dcvar_multilevel_fit} object.}

\item{...}{Additional arguments (unused).}
}
\value{
A data frame with columns \code{unit}, \code{parameter}, \code{mean}, \code{sd},
\code{q2.5}, \code{q97.5}.
}
\description{
Returns posterior summaries for unit-specific VAR coefficients.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pit.R
\name{pit_test}
\alias{pit_test}
\alias{pit_test.default}
\alias{pit_test.dcvar_model_fit}
\title{KS test for PIT uniformity}
\usage{
pit_test(object, ...)

\method{pit_test}{default}(object, ...)

\method{pit_test}{dcvar_model_fit}(object, ...)
}
\arguments{
\item{object}{A fitted model object.}

\item{...}{Additional arguments (unused).}
}
\value{
A data frame with columns \code{variable}, \code{ks_statistic}, \code{p_value}, \code{n}.
}
\description{
Runs a Kolmogorov-Smirnov test per variable to assess whether PIT values
are approximately uniform. This is a heuristic check on the plug-in PIT
values returned by \code{\link[=pit_values]{pit_values()}}, not an exact posterior predictive test.
}
\details{
This applies a Kolmogorov-Smirnov test to the approximate PIT values
returned by \code{\link[=pit_values]{pit_values()}}. The result is a heuristic check and does not
account for serial dependence or full posterior uncertainty. PIT tests are
currently implemented for single-level fits only.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extract.R
\name{draws}
\alias{draws}
\alias{draws.default}
\alias{draws.dcvar_model_fit}
\title{Extract posterior draws}
\usage{
draws(object, ...)

\method{draws}{default}(object, ...)

\method{draws}{dcvar_model_fit}(object, variable = NULL, format = "draws_array", ...)
}
\arguments{
\item{object}{A fitted model object.}

\item{...}{Additional arguments (unused).}

\item{variable}{Character vector of parameter names. \code{NULL} returns all.}

\item{format}{Draw format: \code{"draws_array"}, \code{"draws_matrix"}, or
\code{"draws_df"} (default: \code{"draws_array"}).}
}
\value{
A posterior draws object.
}
\description{
Extract posterior draws from a fitted model.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{plot_ppc}
\alias{plot_ppc}
\title{Posterior predictive check for residual correlations}
\usage{
plot_ppc(object, n_sample = 100, ...)
}
\arguments{
\item{object}{A fitted model object.}

\item{n_sample}{Number of posterior draws to use (default: 100).}

\item{...}{Additional arguments (unused).}
}
\value{
A ggplot object.
}
\description{
Posterior predictive check for residual correlations
}
\details{
Posterior predictive checks are currently available for normal and
exponential margins. Gamma and skew-normal fits store copula-level
replicated z-scores in \code{eps_rep}, so their replicated draws are not on the
same residual scale as \code{eps}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extract.R
\name{phi_trajectory}
\alias{phi_trajectory}
\alias{phi_trajectory.default}
\alias{phi_trajectory.dcvar_tv_fit}
\alias{phi_trajectory.dcvar_sem_tv_fit}
\alias{phi_trajectory.dcvar_multilevel_tv_fit}
\title{Extract the time-varying VAR coefficient trajectories}
\usage{
phi_trajectory(object, ...)

\method{phi_trajectory}{default}(object, ...)

\method{phi_trajectory}{dcvar_tv_fit}(object, probs = c(0.025, 0.1, 0.5, 0.9, 0.975), ...)

\method{phi_trajectory}{dcvar_sem_tv_fit}(object, probs = c(0.025, 0.1, 0.5, 0.9, 0.975), ...)

\method{phi_trajectory}{dcvar_multilevel_tv_fit}(
  object,
  probs = c(0.025, 0.1, 0.5, 0.9, 0.975),
  unit = NULL,
  ...
)
}
\arguments{
\item{object}{A fitted model object.}

\item{...}{Additional arguments (unused).}

\item{probs}{Numeric vector of quantile probabilities
(default: \code{c(0.025, 0.1, 0.5, 0.9, 0.975)}).}

\item{unit}{For \code{dcvar_multilevel_tv_fit} objects, optional unit selector.
Use \code{NULL} (default) for the population-level Phi path, \code{"all"} for all
unit-specific paths, or a vector of unit IDs for selected unit paths.}
}
\value{
A data frame with columns \code{time}, \code{coefficient} (one of
\code{"phi11"}, \code{"phi12"}, \code{"phi21"}, \code{"phi22"}), \code{mean}, \code{sd}, and one
column per requested quantile.
}
\description{
Returns posterior summaries of the four VAR(1) coefficient paths
phi11(t), phi12(t), phi21(t), phi22(t) from a time-varying DC-VAR fit.
For fits with \code{tv_phi = FALSE} the constant baseline coefficients are
tiled over time so the return shape does not depend on the flag.
}
\seealso{
\code{\link[=sigma_trajectory]{sigma_trajectory()}}, \code{\link[=rho_trajectory]{rho_trajectory()}}, \code{\link[=plot_phi_trajectory]{plot_phi_trajectory()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sem_fit-class.R
\name{print.dcvar_sem_tv_summary}
\alias{print.dcvar_sem_tv_summary}
\title{Print a dcvar_sem_tv_summary object}
\usage{
\method{print}{dcvar_sem_tv_summary}(x, ...)
}
\arguments{
\item{x}{A \code{dcvar_sem_tv_summary} object.}

\item{...}{Additional arguments (unused).}
}
\value{
Invisibly returns \code{x}.
}
\description{
Print a dcvar_sem_tv_summary object
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit-multilevel.R
\name{dcvar_multilevel}
\alias{dcvar_multilevel}
\title{Fit an experimental multilevel copula VAR(1) model}
\usage{
dcvar_multilevel(
  data,
  vars,
  id_var = "id",
  time_var = "time",
  center = TRUE,
  margins = "normal",
  skew_direction = NULL,
  prior_phi_bar_sd = 0.5,
  prior_tau_phi_scale = 0.2,
  prior_sigma_sd = 1,
  prior_rho_sd = 0.5,
  tv_phi = FALSE,
  tv_sigma = FALSE,
  prior_tau_phi_rate = 0.025,
  prior_tau_sigma_rate = 0.05,
  tv_sigma_k = 8,
  chains = 4,
  iter_warmup = 2000,
  iter_sampling = 4000,
  adapt_delta = 0.9,
  max_treedepth = 14,
  seed = NULL,
  cores = NULL,
  refresh = 500,
  init = NULL,
  stan_file = NULL,
  backend = getOption("dcvar.backend", "auto"),
  ...
)
}
\arguments{
\item{data}{A data frame in long (panel) format with columns for unit ID,
time, and two outcome variables.}

\item{vars}{Character vector of two variable names to model.}

\item{id_var}{Name of the unit/person ID column (default: \code{"id"}).}

\item{time_var}{Name of the time column (default: \code{"time"}).}

\item{center}{Logical; whether to person-mean center the data
(default: \code{TRUE}). The bundled multilevel Stan model requires
\code{center = TRUE}; set \code{center = FALSE} only with a custom
\code{stan_file} that includes intercept terms.}

\item{margins}{Marginal distribution specification. A single string applies
the same family to both variables. A length-2 character vector gives a
per-variable (mixed) margin (for example \code{c("normal", "gamma")}). Normal,
exponential, skew-normal, and gamma margins are supported; homogeneous
skew-normal/gamma and per-variable specs route through the generic
mixed-margin Stan model under the Gaussian copula.}

\item{skew_direction}{Integer vector of length 2 of \code{1}/\code{-1}. Required
whenever any dimension uses an \code{"exponential"} or \code{"gamma"} margin; only
those dimensions consult it.}

\item{prior_phi_bar_sd}{Prior SD for population-mean VAR coefficients.}

\item{prior_tau_phi_scale}{Prior scale for half-t(3) on tau_phi.}

\item{prior_sigma_sd}{Prior SD for the innovation scales. For normal
margins this is the SD of a half-normal prior on \code{sigma}; for
exponential/gamma margins it is the SD of the lognormal prior on the
marginal scale.}

\item{prior_rho_sd}{Prior SD for normal on rho.}

\item{tv_phi}{Selects which population VAR(1) coefficients carry a shared
time-varying drift around the unit-specific baselines. Either a logical
scalar or a character selector as in \code{\link[=dcvar]{dcvar()}}.}

\item{tv_sigma}{Logical; if \code{TRUE}, residual scales evolve as shared
log-scale random walks across time.}

\item{prior_tau_phi_rate}{Prior mean for the time-drift Phi random-walk
innovation SDs.}

\item{prior_tau_sigma_rate}{Prior mean for log-scale random-walk innovation
SDs.}

\item{tv_sigma_k}{Soft-barrier sharpness for time-varying exponential/gamma
scales.}

\item{chains}{Number of MCMC chains.}

\item{iter_warmup}{Warmup iterations per chain.}

\item{iter_sampling}{Sampling iterations per chain.}

\item{adapt_delta}{Target acceptance rate.}

\item{max_treedepth}{Maximum tree depth.}

\item{seed}{Random seed. When supplied, it seeds both the Stan sampler and
the default per-chain initial values, so repeated fits are reproducible.}

\item{cores}{Number of parallel chains.}

\item{refresh}{How often to print progress.}

\item{init}{Custom init function or \code{NULL}.}

\item{stan_file}{Custom Stan file path or \code{NULL}.}

\item{backend}{Character: \code{"auto"} (default, uses rstan), \code{"rstan"}, or
\code{"cmdstanr"}. Can also be set globally via
\code{options(dcvar.backend = "cmdstanr")}.}

\item{...}{Additional backend-specific sampling arguments.}
}
\value{
A \code{dcvar_multilevel_fit} object.
}
\description{
Fits a hierarchical copula VAR(1) model with unit-specific VAR coefficients
(random effects) and a global copula correlation. Uses non-centered
parameterization for the random Phi coefficients.
}
\details{
\strong{Experimental extension.} This multilevel variant supports
\code{fitted()} and \code{predict()}. PSIS-LOO is available for all multilevel
fits; the stored \code{log_lik} values are conditional one-step-ahead
densities given the unit-level random effects. PIT diagnostics are not yet implemented.

\code{adapt_delta} defaults to 0.90 and \code{max_treedepth} to 14 because the
hierarchical structure with random effects benefits from deeper trees but
does not require aggressive step-size adaptation.
}
\note{
The bundled multilevel Stan program is defined for person-mean
centered data and omits intercept terms. With the bundled model,
\code{center = FALSE} is therefore not supported.
}
\seealso{
\code{\link[=random_effects]{random_effects()}} for extracting unit-specific coefficients,
\code{\link[=simulate_dcvar_multilevel]{simulate_dcvar_multilevel()}} for data generation.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/loo.R
\name{dcvar_compare}
\alias{dcvar_compare}
\title{Compare multiple fitted models using LOO-CV}
\usage{
dcvar_compare(...)
}
\arguments{
\item{...}{Named fitted model objects (e.g., \verb{dcvar = fit1, hmm = fit2}).}
}
\value{
A \code{loo_compare} matrix.
}
\description{
Convenience wrapper around \code{\link[loo:loo_compare]{loo::loo_compare()}} that accepts named
dcvar model fits.
}
\details{
The pointwise \code{log_lik} estimands are not identical across model
families: HMM fits marginalize the discrete states out of each one-step
predictive density, while dynamic (DC-VAR and covariate) fits condition
on the smoothed latent \code{rho} trajectory, which is informed by the
observation itself. Comparing an HMM fit against a dynamic fit can
therefore systematically favor the dynamic model; a warning is emitted
for such comparisons and the resulting elpd differences should be
interpreted with caution.
}
\examples{
\donttest{
sim <- simulate_dcvar(
  n_time = 12,
  rho_trajectory = rho_decreasing(12),
  seed = 1
)
fit_dcvar <- dcvar(
  sim$Y_df,
  vars = c("y1", "y2"),
  chains = 1,
  iter_warmup = 10,
  iter_sampling = 10,
  refresh = 0,
  seed = 1
)
fit_constant <- dcvar_constant(
  sim$Y_df,
  vars = c("y1", "y2"),
  chains = 1,
  iter_warmup = 10,
  iter_sampling = 10,
  refresh = 0,
  seed = 1
)
dcvar_compare(dcvar = fit_dcvar, constant = fit_constant)
}
}
\seealso{
\code{\link[loo:loo_compare]{loo::loo_compare()}} for details on the comparison method,
\code{\link[=dcvar]{dcvar()}}, \code{\link[=dcvar_hmm]{dcvar_hmm()}}, \code{\link[=dcvar_constant]{dcvar_constant()}} for fitting models.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hmm_fit-class.R
\name{dcvar_hmm_fit-methods}
\alias{dcvar_hmm_fit-methods}
\alias{print.dcvar_hmm_fit}
\alias{summary.dcvar_hmm_fit}
\alias{coef.dcvar_hmm_fit}
\alias{plot.dcvar_hmm_fit}
\title{S3 methods for dcvar_hmm_fit objects}
\usage{
\method{print}{dcvar_hmm_fit}(x, ...)

\method{summary}{dcvar_hmm_fit}(object, probs = c(0.025, 0.5, 0.975), ...)

\method{coef}{dcvar_hmm_fit}(object, ...)

\method{plot}{dcvar_hmm_fit}(
  x,
  type = c("rho", "states", "transition", "phi", "diagnostics", "ppc", "pit"),
  ...
)
}
\arguments{
\item{x, object}{A \code{dcvar_hmm_fit} object.}

\item{...}{Additional arguments (unused).}

\item{probs}{Numeric vector of quantile probabilities for the rho trajectory
(default: \code{c(0.025, 0.5, 0.975)}).}

\item{type}{Character; one of \code{"rho"}, \code{"states"}, \code{"transition"}, \code{"phi"},
\code{"diagnostics"}, \code{"ppc"}, or \code{"pit"}.}
}
\value{
Invisibly returns \code{x}.

A \code{dcvar_hmm_summary} object (a list).

A named list of posterior means: \code{mu}, \code{Phi}, margin-specific
scale/shape parameters (e.g. \code{sigma_eps} for normal margins), \code{z_rho}, and
\code{rho_state}.

A ggplot object.
}
\description{
S3 methods for dcvar_hmm_fit objects
}
\section{Functions}{
\itemize{
\item \code{print(dcvar_hmm_fit)}: Print a concise overview of the HMM fit.

\item \code{summary(dcvar_hmm_fit)}: Produce a detailed summary including
state information, VAR parameters, and diagnostics.

\item \code{coef(dcvar_hmm_fit)}: Extract posterior means of model
coefficients including state-specific rho values.

\item \code{plot(dcvar_hmm_fit)}: Dispatch to a plot type: \code{"rho"},
\code{"states"}, \code{"transition"}, \code{"phi"}, \code{"diagnostics"}, \code{"ppc"}, or \code{"pit"}.

}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/trajectories.R
\name{rho_step}
\alias{rho_step}
\title{Generate a single-breakpoint (step function) rho trajectory}
\usage{
rho_step(
  n_time,
  rho_before = 0.7,
  rho_after = 0.3,
  breakpoint = 0.5,
  transition_width = 0
)
}
\arguments{
\item{n_time}{Number of time points.}

\item{rho_before}{Rho before breakpoint (default: 0.7).}

\item{rho_after}{Rho after breakpoint (default: 0.3).}

\item{breakpoint}{Breakpoint location as a proportion of \code{n_time - 1} (if <= 1)
or an absolute time index (default: 0.5).}

\item{transition_width}{Number of time points for smooth transition.
0 = abrupt (default: 0).}
}
\value{
Numeric vector of length \code{n_time - 1}.
}
\description{
Abrupt change from one rho level to another at a specified time.
}
\examples{
rho_step(100, rho_before = 0.7, rho_after = 0.3)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sem_fit-class.R
\name{dcvar_sem_fit-methods}
\alias{dcvar_sem_fit-methods}
\alias{print.dcvar_sem_fit}
\alias{summary.dcvar_sem_fit}
\alias{print.dcvar_sem_tv_fit}
\alias{summary.dcvar_sem_tv_fit}
\alias{coef.dcvar_sem_fit}
\alias{plot.dcvar_sem_fit}
\title{S3 methods for dcvar_sem_fit objects}
\usage{
\method{print}{dcvar_sem_fit}(x, ...)

\method{summary}{dcvar_sem_fit}(object, ...)

\method{print}{dcvar_sem_tv_fit}(x, ...)

\method{summary}{dcvar_sem_tv_fit}(object, probs = c(0.025, 0.5, 0.975), ...)

\method{coef}{dcvar_sem_fit}(object, ...)

\method{plot}{dcvar_sem_fit}(x, type = c("latent_states", "rho", "diagnostics"), ...)
}
\arguments{
\item{x, object}{A \code{dcvar_sem_fit} object.}

\item{...}{Additional arguments (unused).}

\item{probs}{Numeric vector of quantile probabilities for trajectories.}

\item{type}{Character; one of \code{"latent_states"}, \code{"rho"}, \code{"diagnostics"}.}
}
\value{
Invisibly returns \code{x}.

A \code{dcvar_sem_summary} object (a list).

Invisibly returns \code{x}.

A \code{dcvar_sem_tv_summary} object (a list).

A named list of posterior means.

A ggplot object.
}
\description{
S3 methods for dcvar_sem_fit objects
}
\section{Functions}{
\itemize{
\item \code{print(dcvar_sem_fit)}: Print a concise overview.

\item \code{summary(dcvar_sem_fit)}: Produce a detailed summary.

\item \code{print(dcvar_sem_tv_fit)}: Print a concise overview of a TV SEM fit.

\item \code{summary(dcvar_sem_tv_fit)}: Produce a detailed TV SEM summary.

\item \code{coef(dcvar_sem_fit)}: Extract posterior means of
latent VAR coefficients.

\item \code{plot(dcvar_sem_fit)}: Dispatch to a plot type.

}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/multilevel_fit-class.R
\name{print.dcvar_multilevel_summary}
\alias{print.dcvar_multilevel_summary}
\title{Print a dcvar_multilevel_summary object}
\usage{
\method{print}{dcvar_multilevel_summary}(x, ...)
}
\arguments{
\item{x}{A \code{dcvar_multilevel_summary} object.}

\item{...}{Additional arguments (unused).}
}
\value{
Invisibly returns \code{x}.
}
\description{
Print a dcvar_multilevel_summary object
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit-dcvar.R
\name{dcvar}
\alias{dcvar}
\title{Fit the DC-VAR model}
\usage{
dcvar(
  data,
  vars,
  time_var = "time",
  standardize = TRUE,
  margins = "normal",
  skew_direction = NULL,
  allow_gaps = FALSE,
  prior_mu_sd = 2,
  prior_phi_sd = 0.5,
  prior_sigma_eps_rate = 1,
  prior_sigma_omega_rate = 0.1,
  prior_rho_init_sd = 1,
  tv_phi = FALSE,
  tv_sigma = FALSE,
  prior_tau_phi_rate = 0.025,
  prior_tau_sigma_rate = 0.05,
  tv_sigma_k = 8,
  chains = 4,
  iter_warmup = 2000,
  iter_sampling = 4000,
  adapt_delta = 0.99,
  max_treedepth = 12,
  seed = NULL,
  cores = NULL,
  refresh = 500,
  init = NULL,
  stan_file = NULL,
  backend = getOption("dcvar.backend", "auto"),
  ...
)
}
\arguments{
\item{data}{A data frame with time series observations.}

\item{vars}{Character vector of two variable names to model.}

\item{time_var}{Name of the time column (default: \code{"time"}).}

\item{standardize}{Logical; whether to z-score variables (default: \code{TRUE}).}

\item{margins}{Marginal distribution specification. Either a single string
applied to both variables, or a length-2 character vector giving a
per-variable (mixed) margin, e.g. \code{c("normal", "exponential")}. Each entry
is one of \code{"normal"} (default), \code{"exponential"}, \code{"skew_normal"}, or
\code{"gamma"}. When the two entries differ the fit uses a generic
mixed-margins Stan model under the Gaussian copula; identical entries reuse
the specialised single-family model.}

\item{skew_direction}{Integer vector of length 2 indicating skew direction
for asymmetric margins. Each element must be \code{1} (right-skewed) or \code{-1}
(left-skewed). Required whenever any dimension uses an \code{"exponential"} or
\code{"gamma"} margin; only those dimensions consult it.}

\item{allow_gaps}{Logical; if \code{FALSE} (default), interior missing values
cause an error because they break VAR(1) time series adjacency. Set to
\code{TRUE} to allow fitting with a warning instead.}

\item{prior_mu_sd}{Prior SD for intercepts: \code{mu ~ normal(0, prior_mu_sd)}.}

\item{prior_phi_sd}{Prior SD for VAR coefficients: \code{Phi ~ normal(0, prior_phi_sd)}.}

\item{prior_sigma_eps_rate}{Prior mean for innovation SDs (see
\code{\link[=prepare_dcvar_data]{prepare_dcvar_data()}}).}

\item{prior_sigma_omega_rate}{Prior mean for rho process SD (see
\code{\link[=prepare_dcvar_data]{prepare_dcvar_data()}}).}

\item{prior_rho_init_sd}{Prior SD for initial rho on Fisher-z scale.}

\item{tv_phi}{Selects which VAR(1) coefficients evolve as independent
random walks around the constant baseline \code{Phi}. Either a logical
(\code{TRUE} = all four, \code{FALSE} = none, the default) or a character selector:
\code{"ar"} (the autoregressive effects phi11, phi22 -- e.g. for modelling
changing emotional inertia / critical slowing down), \code{"cross"} (the
cross-lagged effects phi12, phi21), or specific names from
\code{c("phi11", "phi12", "phi21", "phi22")}.}

\item{tv_sigma}{Logical; if \code{TRUE}, the residual scales evolve as log-scale
random walks around their constant baselines (default: \code{FALSE}). Applies
to all margin families; see the section on shifted margins for how
exponential and gamma dimensions are handled.}

\item{prior_tau_phi_rate}{Prior mean for the Phi random-walk innovation
SDs (\code{tau_phi ~ exponential(1/prior_tau_phi_rate)}; default \code{0.025}).
Used only when \code{tv_phi = TRUE}.}

\item{prior_tau_sigma_rate}{Prior mean for the log-scale random-walk
innovation SDs (\code{tau_sigma ~ exponential(1/prior_tau_sigma_rate)};
default \code{0.05}). Used only when \code{tv_sigma = TRUE}.}

\item{tv_sigma_k}{Soft-barrier sharpness for time-varying exponential/gamma
scales (default \code{8}). Larger values approximate the exact shifted margin
more closely but stiffen the geometry. Used only when \code{tv_sigma = TRUE}
and an exponential or gamma margin is present.}

\item{chains}{Number of MCMC chains (default: 4).}

\item{iter_warmup}{Warmup iterations per chain (default: 2000).}

\item{iter_sampling}{Sampling iterations per chain (default: 4000).}

\item{adapt_delta}{Target acceptance rate (default: 0.99). The DC-VAR model
uses a lower default than \code{dcvar_constant()} (0.999) because the
non-centered parameterisation already handles posterior geometry well.}

\item{max_treedepth}{Maximum tree depth (default: 12).}

\item{seed}{Random seed. When supplied, it seeds both the Stan sampler and
the default per-chain initial values, so repeated fits are reproducible.}

\item{cores}{Number of parallel chains. \code{NULL} uses all available cores.}

\item{refresh}{How often to print progress (default: 500). Set to 0 for
silent operation.}

\item{init}{Custom init function or \code{NULL} for smart defaults.}

\item{stan_file}{Path to a custom Stan file, or \code{NULL} to use the bundled
model.}

\item{backend}{Character: \code{"auto"} (default, uses rstan), \code{"rstan"}, or
\code{"cmdstanr"}. Can also be set globally via
\code{options(dcvar.backend = "cmdstanr")}.}

\item{...}{Additional backend-specific sampling arguments.}
}
\value{
A \code{dcvar_fit} object.
}
\description{
Fits a Dynamic Copula VAR(1) model with time-varying correlation following
a random walk on the Fisher-z scale. Uses non-centered parameterisation
for efficient HMC sampling.
}
\details{
Beyond the always time-varying correlation, the VAR coefficients and the
residual scales can optionally vary over time as well (\code{tv_phi},
\code{tv_sigma}): each enabled component evolves as a non-centered random walk
around its constant baseline, with innovation-SD priors that shrink toward
the constant-parameter model. With both flags off (the default) the fit is
exactly the classic DC-VAR. With any flag on, the fit uses a generic
time-varying Stan model and returns a \code{dcvar_tv_fit} object (a subclass of
\code{dcvar_fit}), with trajectory extractors \code{\link[=phi_trajectory]{phi_trajectory()}} and
\code{\link[=sigma_trajectory]{sigma_trajectory()}}.
}
\section{Time-varying scales and shifted margins}{

\code{tv_sigma = TRUE} gives normal and skew-normal dimensions a multiplicative
log-scale random walk. Exponential and gamma dimensions use a \strong{soft-barrier}
construction: the shifted variate \code{x = scale + skew * eps} (which has a hard
support boundary at 0) is replaced by \code{x = softplus_k(m_t + skew * eps)}
with a time-varying scale \code{m_t}. This keeps \code{x} positive, matches the exact
shifted margin in the interior, and rounds the boundary smoothly so the
scale can vary freely. \code{tv_sigma_k} controls the sharpness; larger values
track the exact exponential/gamma more closely at the cost of a stiffer
posterior geometry, smaller values are numerically gentler but introduce a
small residual-mean bias in the lower tail.
}

\section{Recommended workflow}{

For typical series lengths (T of 100--300), fit the model ladder
\code{dcvar()} (constant Phi/sigma) -> \code{dcvar(tv_phi = TRUE)} ->
\code{dcvar(tv_phi = TRUE, tv_sigma = TRUE)} and compare with
\code{\link[=dcvar_compare]{dcvar_compare()}}. Up to seven latent random walks are estimated from a
single bivariate series; the tight innovation-SD priors are what keeps
this identifiable, so widen them deliberately.
}

\examples{
\donttest{
sim <- simulate_dcvar(
  n_time = 12,
  rho_trajectory = rho_decreasing(12),
  seed = 1
)
fit <- dcvar(
  sim$Y_df,
  vars = c("y1", "y2"),
  chains = 1,
  iter_warmup = 10,
  iter_sampling = 10,
  refresh = 0,
  seed = 1
)
print(fit)
summary(fit)
plot(fit)
}
}
\seealso{
\code{\link[=dcvar_constant]{dcvar_constant()}} for the time-invariant baseline,
\code{\link[=dcvar_hmm]{dcvar_hmm()}} for the regime-switching model,
\code{\link[=dcvar_compare]{dcvar_compare()}} for LOO-CV model comparison,
\code{\link[=rho_trajectory]{rho_trajectory()}} and \code{\link[=plot_rho]{plot_rho()}} for inspecting results.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tv_fit-class.R
\name{dcvar_tv_fit-methods}
\alias{dcvar_tv_fit-methods}
\alias{print.dcvar_tv_fit}
\alias{summary.dcvar_tv_fit}
\alias{coef.dcvar_tv_fit}
\alias{plot.dcvar_tv_fit}
\title{S3 methods for dcvar_tv_fit objects}
\usage{
\method{print}{dcvar_tv_fit}(x, ...)

\method{summary}{dcvar_tv_fit}(object, probs = c(0.025, 0.5, 0.975), ...)

\method{coef}{dcvar_tv_fit}(object, ...)

\method{plot}{dcvar_tv_fit}(x, type = c("rho", "phi", "sigma", "diagnostics", "pit"), ...)
}
\arguments{
\item{x, object}{A \code{dcvar_tv_fit} object.}

\item{...}{Additional arguments (unused).}

\item{probs}{Numeric vector of quantile probabilities for the trajectories
(default: \code{c(0.025, 0.5, 0.975)}).}

\item{type}{Character; one of \code{"rho"}, \code{"phi"}, \code{"sigma"},
\code{"diagnostics"}, or \code{"pit"}.}
}
\value{
Invisibly returns \code{x}.

A \code{dcvar_tv_summary} object (a list).

A named list of posterior means.

A ggplot object.
}
\description{
Methods for fits from \code{\link[=dcvar]{dcvar()}} with \code{tv_phi = TRUE} and/or
\code{tv_sigma = TRUE} (time-varying VAR coefficients and residual scales).
}
\section{Functions}{
\itemize{
\item \code{print(dcvar_tv_fit)}: Print a concise overview of the TV DC-VAR
fit.

\item \code{summary(dcvar_tv_fit)}: Produce a detailed summary including the
rho, Phi, and sigma trajectories, constant parameters, and diagnostics.

\item \code{coef(dcvar_tv_fit)}: Extract posterior means of the constant
model coefficients (baselines, walk innovation SDs, margin shape
parameters). Use \code{\link[=phi_trajectory]{phi_trajectory()}} and \code{\link[=sigma_trajectory]{sigma_trajectory()}} for the
time-varying paths.

\item \code{plot(dcvar_tv_fit)}: Dispatch to a plot type: \code{"rho"},
\code{"phi"} (coefficient trajectories), \code{"sigma"} (scale trajectories),
\code{"diagnostics"}, or \code{"pit"}.

}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extract.R
\name{rho_trajectory}
\alias{rho_trajectory}
\alias{rho_trajectory.default}
\alias{rho_trajectory.dcvar_fit}
\alias{rho_trajectory.dcvar_covariate_fit}
\alias{rho_trajectory.dcvar_hmm_fit}
\alias{rho_trajectory.dcvar_constant_fit}
\alias{rho_trajectory.dcvar_multilevel_fit}
\alias{rho_trajectory.dcvar_sem_fit}
\alias{rho_trajectory.dcvar_sem_tv_fit}
\alias{rho_trajectory.dcvar_multilevel_tv_fit}
\title{Extract the rho trajectory with credible intervals}
\usage{
rho_trajectory(object, ...)

\method{rho_trajectory}{default}(object, ...)

\method{rho_trajectory}{dcvar_fit}(object, probs = c(0.025, 0.1, 0.5, 0.9, 0.975), ...)

\method{rho_trajectory}{dcvar_covariate_fit}(object, probs = c(0.025, 0.1, 0.5, 0.9, 0.975), ...)

\method{rho_trajectory}{dcvar_hmm_fit}(object, probs = c(0.025, 0.1, 0.5, 0.9, 0.975), ...)

\method{rho_trajectory}{dcvar_constant_fit}(object, probs = c(0.025, 0.1, 0.5, 0.9, 0.975), ...)

\method{rho_trajectory}{dcvar_multilevel_fit}(object, probs = c(0.025, 0.1, 0.5, 0.9, 0.975), ...)

\method{rho_trajectory}{dcvar_sem_fit}(object, probs = c(0.025, 0.1, 0.5, 0.9, 0.975), ...)

\method{rho_trajectory}{dcvar_sem_tv_fit}(object, probs = c(0.025, 0.1, 0.5, 0.9, 0.975), ...)

\method{rho_trajectory}{dcvar_multilevel_tv_fit}(object, probs = c(0.025, 0.1, 0.5, 0.9, 0.975), ...)
}
\arguments{
\item{object}{A fitted model object (\code{dcvar_fit}, \code{dcvar_covariate_fit},
\code{dcvar_hmm_fit}, \code{dcvar_constant_fit}, \code{dcvar_multilevel_fit}, or
\code{dcvar_sem_fit}).}

\item{...}{Additional arguments (unused).}

\item{probs}{Numeric vector of quantile probabilities (default: \code{c(0.025, 0.1, 0.5, 0.9, 0.975)}).}
}
\value{
A data frame with columns \code{time}, \code{mean}, \code{sd}, and one column per
quantile (e.g., \code{q2.5}, \code{q10}, \code{q50}, \code{q90}, \code{q97.5}). For
\code{dcvar_constant_fit} objects, the constant rho is expanded to all \code{n_time - 1}
time points for consistency with the time-varying models.
}
\description{
Returns a data frame with the posterior mean, SD, and quantiles of the
time-varying correlation at each time point.
}
\seealso{
\code{\link[=plot_rho]{plot_rho()}} to visualise the trajectory,
\code{\link[=interpret_rho_trajectory]{interpret_rho_trajectory()}} for a text-based summary,
\code{\link[=var_params]{var_params()}} for VAR parameter extraction.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pit.R
\name{plot_pit}
\alias{plot_pit}
\title{Plot PIT histograms}
\usage{
plot_pit(object, bins = 20, ...)
}
\arguments{
\item{object}{A fitted model object.}

\item{bins}{Number of histogram bins (default: 20).}

\item{...}{Additional arguments (unused).}
}
\value{
A ggplot object.
}
\description{
Creates faceted histograms of the approximate PIT values returned by
\code{\link[=pit_values]{pit_values()}}. Under good model fit, these histograms should be roughly
uniform, but they remain plug-in diagnostics rather than exact posterior
predictive checks.
}
\details{
PIT histograms visualize the approximate plug-in PIT values
returned by \code{\link[=pit_values]{pit_values()}}. They are currently implemented for the three
core single-level fit classes only.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{plot_rho}
\alias{plot_rho}
\title{Plot the rho trajectory with credible intervals}
\usage{
plot_rho(
  object,
  show_ci = TRUE,
  ci_level = 0.95,
  inner_level = 0.8,
  true_rho = NULL,
  title = NULL,
  ...
)
}
\arguments{
\item{object}{A fitted model object with a rho trajectory (\code{dcvar_fit},
\code{dcvar_covariate_fit}, \code{dcvar_hmm_fit}, or \code{dcvar_constant_fit}).}

\item{show_ci}{Logical; show credible interval ribbons (default: \code{TRUE}).}

\item{ci_level}{Credible interval level for the outer ribbon (default: 0.95).}

\item{inner_level}{Credible interval level for the inner ribbon (default:
0.80). Set to \code{NULL} to disable the inner ribbon.}

\item{true_rho}{Optional numeric vector of true rho values for overlay
(useful for simulation studies).}

\item{title}{Plot title.}

\item{...}{Additional arguments (unused).}
}
\value{
A ggplot object.
}
\description{
Plot the rho trajectory with credible intervals
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/multilevel_fit-class.R
\name{dcvar_multilevel_fit-methods}
\alias{dcvar_multilevel_fit-methods}
\alias{print.dcvar_multilevel_fit}
\alias{summary.dcvar_multilevel_fit}
\alias{print.dcvar_multilevel_tv_fit}
\alias{summary.dcvar_multilevel_tv_fit}
\alias{coef.dcvar_multilevel_fit}
\alias{plot.dcvar_multilevel_fit}
\title{S3 methods for dcvar_multilevel_fit objects}
\usage{
\method{print}{dcvar_multilevel_fit}(x, ...)

\method{summary}{dcvar_multilevel_fit}(object, ...)

\method{print}{dcvar_multilevel_tv_fit}(x, ...)

\method{summary}{dcvar_multilevel_tv_fit}(object, probs = c(0.025, 0.5, 0.975), ...)

\method{coef}{dcvar_multilevel_fit}(object, ...)

\method{plot}{dcvar_multilevel_fit}(x, type = c("random_effects", "diagnostics"), ...)
}
\arguments{
\item{x, object}{A \code{dcvar_multilevel_fit} object.}

\item{...}{Additional arguments (unused).}

\item{probs}{Numeric vector of quantile probabilities for trajectories.}

\item{type}{Character; one of \code{"random_effects"}, \code{"diagnostics"}.}
}
\value{
Invisibly returns \code{x}.

A \code{dcvar_multilevel_summary} object (a list).

Invisibly returns \code{x}.

A \code{dcvar_multilevel_tv_summary} object (a list).

A named list of posterior means.

A ggplot object.
}
\description{
S3 methods for dcvar_multilevel_fit objects
}
\details{
Unlike single-level models (where \code{coef()} returns \verb{$Phi}), the multilevel
model returns hierarchical parameters:
\describe{
\item{\code{phi_bar}}{Population-mean VAR coefficients (analogous to \code{Phi}
in single-level models, vectorised as phi11, phi12, phi21, phi22).}
\item{\code{tau_phi}}{Between-unit SD of VAR coefficients.}
\item{scale parameters}{\code{sigma} (innovation SDs) for normal margins,
\code{sigma_exp} for exponential margins, or per-family scale/shape parameters
(e.g. \code{sigma_eps}, \code{sigma_gam}, \code{shape_gam}) for per-variable,
skew-normal, and gamma mixed-engine margins.}
\item{\code{rho}}{Copula correlation (constant across units).}
}
Use \code{\link[=random_effects]{random_effects()}} to obtain unit-specific VAR coefficients.
}
\section{Functions}{
\itemize{
\item \code{print(dcvar_multilevel_fit)}: Print a concise overview.

\item \code{summary(dcvar_multilevel_fit)}: Produce a detailed summary.

\item \code{print(dcvar_multilevel_tv_fit)}: Print a concise overview of a TV
multilevel fit.

\item \code{summary(dcvar_multilevel_tv_fit)}: Produce a detailed TV multilevel
summary.

\item \code{coef(dcvar_multilevel_fit)}: Extract posterior means of
population-level coefficients.

\item \code{plot(dcvar_multilevel_fit)}: Dispatch to a plot type.

}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit-sem.R
\name{dcvar_sem}
\alias{dcvar_sem}
\title{Fit an experimental SEM copula VAR(1) model}
\usage{
dcvar_sem(
  data,
  indicators,
  J = NULL,
  lambda = NULL,
  sigma_e = NULL,
  margins = "normal",
  skew_direction = NULL,
  time_var = "time",
  method = c("indicator", "naive"),
  prior_mu_sd = 0.25,
  prior_phi_sd = 0.5,
  prior_sigma_sd = 0.5,
  prior_rho_sd = 0.75,
  tv_phi = FALSE,
  tv_sigma = FALSE,
  prior_sigma_omega_rate = 0.1,
  prior_tau_phi_rate = 0.025,
  prior_tau_sigma_rate = 0.05,
  tv_sigma_k = 8,
  chains = 4,
  iter_warmup = 2000,
  iter_sampling = 4000,
  adapt_delta = 0.95,
  max_treedepth = 13,
  seed = NULL,
  cores = NULL,
  refresh = 500,
  init = NULL,
  stan_file = NULL,
  backend = getOption("dcvar.backend", "auto"),
  ...
)
}
\arguments{
\item{data}{A data frame with time series of indicator variables.}

\item{indicators}{A list of two character vectors, each naming J indicator
columns per latent variable. For example:
\code{list(PA = c("y1_1", "y1_2", "y1_3"), NA_ = c("y2_1", "y2_2", "y2_3"))}.}

\item{J}{Number of indicators per latent variable. If \code{method = "naive"}
and \code{J} is \code{NULL}, it is inferred from \code{indicators}.}

\item{lambda}{Numeric vector of length J with fixed factor loadings.
Required when \code{method = "indicator"}.}

\item{sigma_e}{Fixed measurement error SD (scalar). Required when
\code{method = "indicator"}.}

\item{margins}{Latent-innovation marginal specification. A single string
applies the same family to both latent variables. A length-2 character
vector gives a per-variable (mixed) margin (for example
\code{c("normal", "gamma")}). Normal, exponential, skew-normal, and gamma
margins are supported; homogeneous skew-normal/gamma and per-variable specs
route through the generic mixed-margin Stan models under the Gaussian
copula. Applies to both the indicator and naive methods.}

\item{skew_direction}{Integer vector of length 2 of \code{1} (right-skewed) or
\code{-1} (left-skewed). Required whenever any dimension uses an
\code{"exponential"} or \code{"gamma"} margin; only those dimensions consult it.}

\item{time_var}{Name of the time column (default: \code{"time"}).}

\item{method}{Character string: \code{"indicator"} (default) fits the fixed
measurement model; \code{"naive"} averages indicators within each latent block
and fits the observed score VAR.}

\item{prior_mu_sd}{Prior SD for intercepts: \code{mu ~ normal(0, prior_mu_sd)}.}

\item{prior_phi_sd}{Prior SD for VAR coefficients:
\code{phi ~ normal(0, prior_phi_sd)}.}

\item{prior_sigma_sd}{Prior SD for the lognormal prior on the latent
innovation scale parameter. For normal margins this is applied to
\code{sigma}; for exponential margins it is applied to \code{sigma_exp}.}

\item{prior_rho_sd}{Prior SD for rho_raw:
\code{rho_raw ~ normal(0, prior_rho_sd)}, with \code{rho = 0.97 * tanh(rho_raw)}.
For TV SEM fits this is the prior SD for the initial Fisher-z rho.}

\item{tv_phi}{Selects which latent VAR(1) coefficients evolve as random walks
around their baseline. Either a logical scalar or a character selector as in
\code{\link[=dcvar]{dcvar()}}. Time-varying SEM currently requires \code{method = "indicator"}.}

\item{tv_sigma}{Logical; if \code{TRUE}, latent innovation scales evolve as
log-scale random walks. Time-varying SEM currently requires
\code{method = "indicator"}.}

\item{prior_sigma_omega_rate}{Prior mean for the TV SEM Fisher-z rho
random-walk SD.}

\item{prior_tau_phi_rate}{Prior mean for Phi random-walk innovation SDs.}

\item{prior_tau_sigma_rate}{Prior mean for log-scale random-walk innovation
SDs.}

\item{tv_sigma_k}{Soft-barrier sharpness for time-varying exponential/gamma
scales.}

\item{chains}{Number of MCMC chains.}

\item{iter_warmup}{Warmup iterations per chain.}

\item{iter_sampling}{Sampling iterations per chain.}

\item{adapt_delta}{Target acceptance rate.}

\item{max_treedepth}{Maximum tree depth.}

\item{seed}{Random seed. When supplied, it seeds both the Stan sampler and
the default per-chain initial values, so repeated fits are reproducible.}

\item{cores}{Number of parallel chains.}

\item{refresh}{How often to print progress.}

\item{init}{Custom init function or \code{NULL}.}

\item{stan_file}{Custom Stan file path or \code{NULL}.}

\item{backend}{Character: \code{"auto"} (default, uses rstan), \code{"rstan"}, or
\code{"cmdstanr"}. Can also be set globally via
\code{options(dcvar.backend = "cmdstanr")}.}

\item{...}{Additional backend-specific sampling arguments.}
}
\value{
A \code{dcvar_sem_fit} object.
}
\description{
Fits a copula VAR(1) model with a fixed measurement model (factor
loadings and measurement error SD are not estimated). Latent innovations
are treated as parameters, making this model computationally intensive
for large T.
}
\details{
\strong{Experimental extension.} This SEM variant supports \code{fitted()} and
\code{predict()}. PSIS-LOO is available for naive score fits. PIT diagnostics are
not yet implemented.

\strong{Boundary constraints.} The SEM model constrains each VAR coefficient
(Phi) to the interval \eqn{[-0.99, 0.99]}, unlike other dcvar models
where Phi is unconstrained. Very strong autoregressive or cross-lag
dynamics near \eqn{\pm 1} cannot be captured by this variant.

The copula correlation \eqn{\rho} is constrained to \eqn{(-0.97, 0.97)}
via \code{rho = 0.97 * tanh(rho_raw)} to avoid boundary singularity in the
Gaussian copula density. Extremely high correlations near \eqn{\pm 1}
are truncated.

\strong{Margins.} SEM fits support normal, exponential, skew-normal, and gamma
latent innovation margins. Exponential and gamma margins use the same shifted
positive-support parameterization as the single-level models and therefore
require \code{skew_direction}. Homogeneous skew-normal/gamma and per-variable
margin specs route to the generic mixed-margins Stan model.

\strong{Post-estimation.} \code{fitted()} and \code{predict()} are available for both the
latent-state scale (\code{type = "link"}) and the observed-indicator scale
(\code{type = "response"}). Use \code{\link[=latent_states]{latent_states()}} when you specifically need the
full posterior summaries of the latent trajectories.
}
\seealso{
\code{\link[=latent_states]{latent_states()}} for extracting estimated latent states,
\code{\link[=simulate_dcvar_sem]{simulate_dcvar_sem()}} for data generation.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/applicability.R
\name{applicability_check}
\alias{applicability_check}
\title{Check whether the flexible-margin copula-VAR is appropriate for the data}
\usage{
applicability_check(
  fit,
  reference = NULL,
  atom_tol = 0.05,
  delta_tol = 0.98,
  phi_collapse_tol = 0.1,
  ols_clear_tol = 0.2,
  rhat_tol = 1.01
)
}
\arguments{
\item{fit}{A \code{dcvar_constant_fit}.}

\item{reference}{Optional normal-margin \code{dcvar_constant_fit} on the same
variables and number of observations. When supplied, the dynamics-collapse
signal compares the fitted self-lags against this reference fit's self-lags
in addition to the OLS VAR(1) anchor (which is always used). If \code{NULL}
(default), only the OLS VAR(1) anchor is used.}

\item{atom_tol}{Minimum fraction at a bound to flag a point mass (default \code{0.05}).}

\item{delta_tol}{Magnitude of the skew-normal slant treated as a boundary value
(default \code{0.98}).}

\item{phi_collapse_tol}{Self-lag magnitude treated as collapsed (default \code{0.10}).}

\item{ols_clear_tol}{Anchor self-lag magnitude treated as clear
autoregression when detecting a dynamics collapse (default \code{0.20}).}

\item{rhat_tol}{Split-Rhat threshold (default \code{1.01}).}
}
\value{
An object of class \code{dcvar_applicability}: a list with the \code{verdict}
(\code{"suitable"}, \code{"caution"}, or \code{"unsuitable"}), a \code{recommendation}, the
triggered \code{reasons}, the variable names (\code{vars}) and \code{margins}, the
boundary-atom fractions (\code{atom}, \code{atom_min}, \code{atom_max}), the fitted and
OLS self-lags (\code{fit_self_lag}, \code{ols_self_lag}, plus \code{reference_self_lag}
when a reference is supplied), the skew-normal slant means (\code{delta}, or
\code{NULL}), the convergence \code{diagnostics}, and the \code{thresholds} used. Has a
\code{print} method.
}
\description{
Screens a fitted constant copula-VAR for the floor/ceiling pile-up pathology.
When a non-trivial point mass of observations sits exactly at a scale bound
(a floor or ceiling effect, common for raw single-item slider or Likert
responses), a flexible skewed margin (skew-normal, gamma, exponential) can
reproduce that pile-up only by concentrating the innovations at the bound,
which is achieved when the autoregressive prediction is flat. The margin then
absorbs the marginal shape and the VAR coefficients collapse toward zero, even
when the series is clearly autocorrelated. The decision whether the approach
suits a dataset cannot be made from the response format (a visual analog scale
looks continuous yet can have a heavy floor pile-up); it must be made from the
realized data and the fit.
}
\details{
Three signals are combined:
\enumerate{
\item \strong{Boundary atom}: the fraction of observations exactly at each
series minimum and maximum, counted only when at least two observations
coincide at the bound (a lone extreme value is not a point mass). A
continuous margin cannot reproduce a point mass, so a non-trivial atom is
a warning sign.
\item \strong{Dynamics collapse}: the fitted self-lags against an OLS VAR(1)
anchor (and, when supplied, a normal-margin reference fit). A
flexible-margin fit whose self-lags collapse toward zero while an anchor
shows clear autoregression (in either direction) is the signature of the
pathology.
\item \strong{Boundary and convergence flags}: a skew-normal slant estimated
at its boundary, divergent transitions, or elevated split-Rhat.
}
}
\examples{
\donttest{
sim <- simulate_dcvar(
  n_time = 60,
  rho_trajectory = rho_constant(60, rho = 0.4),
  margins = "skew_normal",
  skew_params = list(alpha = c(4, 4)),
  seed = 1
)
fit <- dcvar_constant(
  sim$Y_df, vars = c("y1", "y2"), margins = "skew_normal",
  chains = 2, iter_warmup = 500, iter_sampling = 500, refresh = 0, seed = 1
)
applicability_check(fit)
}
}
\seealso{
\code{\link[=dcvar_constant]{dcvar_constant()}}, \code{\link[=dcvar_diagnostics]{dcvar_diagnostics()}}, \code{\link[=var_params]{var_params()}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data-preparation.R
\name{prepare_sem_data}
\alias{prepare_sem_data}
\title{Prepare data for the SEM copula VAR model}
\usage{
prepare_sem_data(
  data,
  indicators,
  J = NULL,
  lambda = NULL,
  sigma_e = NULL,
  margins = "normal",
  skew_direction = NULL,
  time_var = "time",
  prior_mu_sd = 0.25,
  prior_phi_sd = 0.5,
  prior_sigma_sd = 0.5,
  prior_rho_sd = 0.75,
  tv_phi = FALSE,
  tv_sigma = FALSE,
  prior_sigma_omega_rate = 0.1,
  prior_tau_phi_rate = 0.025,
  prior_tau_sigma_rate = 0.05,
  tv_sigma_k = 8,
  method = c("indicator", "naive")
)
}
\arguments{
\item{data}{A data frame with time series of indicator variables.}

\item{indicators}{A list of two character vectors, each naming J indicator
columns per latent variable.}

\item{J}{Number of indicators per latent variable.}

\item{lambda}{Numeric vector of length J with fixed factor loadings.}

\item{sigma_e}{Fixed measurement error SD (scalar).}

\item{margins}{Latent innovation margin specification. A single string
applies the same family to both latent variables. Normal and exponential
use specialised Stan data contracts; skew-normal, gamma, and length-2
per-variable specs use the generic \code{sem_mixed} / \code{sem_naive_mixed} Stan
model with per-dimension family codes.}

\item{skew_direction}{Integer vector of length 2 of +1/-1. Required whenever
any dimension uses an \code{"exponential"} or \code{"gamma"} margin.}

\item{time_var}{Name of the time column (default: \code{"time"}).}

\item{prior_mu_sd}{Prior SD for intercepts.}

\item{prior_phi_sd}{Prior SD for VAR coefficients.}

\item{prior_sigma_sd}{Prior SD for the lognormal prior on the latent
innovation scale parameter.}

\item{prior_rho_sd}{Prior SD for rho_raw.}

\item{tv_phi}{Selects which latent VAR(1) coefficients vary over time.
See \code{\link[=dcvar]{dcvar()}} for accepted selectors.}

\item{tv_sigma}{Logical; if \code{TRUE}, latent innovation scales evolve over time.}

\item{prior_sigma_omega_rate}{Prior mean for the Fisher-z rho random-walk SD
used by TV SEM fits.}

\item{prior_tau_phi_rate}{Prior mean for Phi random-walk innovation SDs.}

\item{prior_tau_sigma_rate}{Prior mean for log-scale random-walk SDs.}

\item{tv_sigma_k}{Soft-barrier sharpness for time-varying exponential/gamma
scales.}

\item{method}{Character string: \code{"indicator"} for the fixed measurement
model or \code{"naive"} for row-mean factor scores.}
}
\value{
A named list suitable as Stan data input.
}
\description{
Transforms a data frame of indicator variables into a list suitable for
the SEM copula Stan model. The measurement model parameters (lambda,
sigma_e) are fixed and passed through to Stan.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simulate-sem.R
\name{simulate_dcvar_sem}
\alias{simulate_dcvar_sem}
\title{Simulate data from a SEM copula VAR(1) model}
\usage{
simulate_dcvar_sem(
  n_time = 200,
  J = 3,
  lambda = rep(sqrt(0.8), 3),
  sigma_e = sqrt(0.2),
  Phi = matrix(c(0.5, 0.15, 0.15, 0.3), 2, 2),
  mu = c(0, 0),
  margins = "normal",
  sigma = c(1, 1),
  sigma_exp = c(1, 1),
  skew_direction = NULL,
  skew_params = NULL,
  rho = 0.3,
  rho_trajectory = NULL,
  phi_trajectory = NULL,
  sigma_trajectory = NULL,
  tv_sigma_k = NULL,
  burnin = 0,
  seed = NULL
)
}
\arguments{
\item{n_time}{Number of time points.}

\item{J}{Number of indicators per latent variable.}

\item{lambda}{Numeric vector of length J with factor loadings.}

\item{sigma_e}{Measurement error SD (scalar).}

\item{Phi}{2x2 VAR coefficient matrix.}

\item{mu}{Length-2 intercept vector.}

\item{margins}{Latent-innovation marginal family. Either a single string
applied to both latent variables, or a length-2 character vector for
per-variable (mixed) margins, e.g. \code{c("normal", "gamma")}, where each entry
is one of \code{"normal"}, \code{"exponential"}, \code{"skew_normal"}, or \code{"gamma"}.}

\item{sigma}{Length-2 latent innovation scale vector used by normal,
skew-normal, gamma, and mixed-margin specifications. Values are on each
family's natural residual scale: innovation SD for normal/skew-normal,
and \code{sigma_gam} for gamma. Homogeneous exponential margins use
\code{sigma_exp} instead.}

\item{sigma_exp}{Length-2 shifted-exponential scale vector for the
single-family exponential path.}

\item{skew_direction}{Integer vector of length 2 of \code{1}/\code{-1}. Required
whenever any dimension uses an \code{"exponential"} or \code{"gamma"} margin.}

\item{skew_params}{Named list of margin-specific parameters for mixed
margins: \code{alpha} (length-2 skew-normal shape) and/or \code{shape} (scalar gamma
shape).}

\item{rho}{Copula correlation.}

\item{rho_trajectory}{Optional numeric vector of length \code{n_time - 1} for a
time-varying copula correlation. Mutually exclusive with \code{rho}.}

\item{phi_trajectory}{Optional time-varying latent VAR coefficient paths,
accepted in the same forms as \code{\link[=simulate_dcvar]{simulate_dcvar()}}.}

\item{sigma_trajectory}{Optional time-varying latent innovation scale paths,
accepted in the same forms as \code{\link[=simulate_dcvar]{simulate_dcvar()}}. The supplied value is
each family's natural scale: innovation SD (normal), residual SD
(skew-normal), \code{sigma_exp} (exponential), \code{sigma_gam} (gamma).}

\item{tv_sigma_k}{Soft-barrier sharpness for time-varying exponential/gamma
scales.}

\item{burnin}{Retained for backward compatibility but ignored. Default \code{0}
keeps the default simulation path aligned with the fitted SEM model,
which conditions on \code{x_0 = 0} and treats the first returned state as
observed rather than drawn after a burn-in period.}

\item{seed}{Random seed for reproducibility.}
}
\value{
A named list with:
\itemize{
\item \code{data}: data frame with columns \code{time}, \code{y1_1}, ..., \code{y1_J},
\code{y2_1}, ..., \code{y2_J}
\item \code{true_params}: list of true parameter values
\item \code{latent_states}: \verb{n_time x 2} matrix of true latent states
\item \code{innovations}: \verb{n_time x 2} matrix of true innovations
}
}
\description{
Generates indicator-level time series data from a latent VAR(1) process
with Gaussian copula dependence and a fixed measurement model.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as-data-frame.R
\name{as.data.frame.dcvar_model_fit}
\alias{as.data.frame.dcvar_model_fit}
\title{Convert a dcvar fit summary to a data frame}
\usage{
\method{as.data.frame}{dcvar_model_fit}(x, row.names = NULL, optional = FALSE, ...)
}
\arguments{
\item{x}{A fitted dcvar model object (any subclass of \code{dcvar_model_fit}:
\code{dcvar_fit}, \code{dcvar_constant_fit}, \code{dcvar_hmm_fit}, \code{dcvar_covariate_fit},
\code{dcvar_sem_fit}, or \code{dcvar_multilevel_fit}).}

\item{row.names}{Ignored.}

\item{optional}{Ignored.}

\item{...}{Additional arguments (unused).}
}
\value{
A data frame with columns \code{variable}, \code{mean}, \code{sd}, \code{q2.5},
\code{q97.5}, \code{rhat}, \code{ess_bulk}, \code{ess_tail}.
}
\description{
Returns the full parameter summary as a tidy data frame with
correct 2.5\%/97.5\% quantiles.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/applicability.R
\name{print.dcvar_applicability}
\alias{print.dcvar_applicability}
\title{Print a flexible-margin applicability check}
\usage{
\method{print}{dcvar_applicability}(x, ...)
}
\arguments{
\item{x}{A \code{dcvar_applicability} object, as returned by
\code{\link[=applicability_check]{applicability_check()}}.}

\item{...}{Additional arguments (unused).}
}
\value{
Invisibly returns \code{x}.
}
\description{
Print a flexible-margin applicability check
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/constant_fit-class.R
\name{dcvar_constant_fit-methods}
\alias{dcvar_constant_fit-methods}
\alias{print.dcvar_constant_fit}
\alias{summary.dcvar_constant_fit}
\alias{coef.dcvar_constant_fit}
\alias{plot.dcvar_constant_fit}
\title{S3 methods for dcvar_constant_fit objects}
\usage{
\method{print}{dcvar_constant_fit}(x, ...)

\method{summary}{dcvar_constant_fit}(object, probs = c(0.025, 0.5, 0.975), ...)

\method{coef}{dcvar_constant_fit}(object, ...)

\method{plot}{dcvar_constant_fit}(x, type = c("rho", "phi", "diagnostics", "ppc", "pit"), ...)
}
\arguments{
\item{x, object}{A \code{dcvar_constant_fit} object.}

\item{...}{Additional arguments (unused).}

\item{probs}{Numeric vector of quantile probabilities for the rho estimate
(default: \code{c(0.025, 0.5, 0.975)}).}

\item{type}{Character; one of \code{"rho"}, \code{"phi"}, \code{"diagnostics"}, \code{"ppc"},
\code{"pit"}.}
}
\value{
Invisibly returns \code{x}.

A \code{dcvar_constant_summary} object (a list).

A named list of posterior means: \code{mu}, \code{Phi}, margin-specific
scale/shape parameters (e.g. \code{sigma_eps} for normal margins), and either
\code{rho} (Gaussian copula) or \code{theta} (Clayton copula).

A ggplot object.
}
\description{
S3 methods for dcvar_constant_fit objects
}
\section{Functions}{
\itemize{
\item \code{print(dcvar_constant_fit)}: Print a concise overview of the
constant copula fit.

\item \code{summary(dcvar_constant_fit)}: Produce a detailed summary including
constant rho, VAR parameters, and diagnostics.

\item \code{coef(dcvar_constant_fit)}: Extract posterior means of model
coefficients.

\item \code{plot(dcvar_constant_fit)}: Dispatch to a plot type: \code{"rho"},
\code{"phi"}, \code{"diagnostics"}, \code{"ppc"}, or \code{"pit"}.

}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{plot_sigma_trajectory}
\alias{plot_sigma_trajectory}
\title{Plot the time-varying residual scale trajectories}
\usage{
plot_sigma_trajectory(
  object,
  show_ci = TRUE,
  ci_level = 0.95,
  inner_level = 0.8,
  true_sigma = NULL,
  title = NULL,
  ...
)
}
\arguments{
\item{object}{A \code{dcvar_tv_fit} object.}

\item{show_ci}{Logical; draw credible ribbons (default: \code{TRUE}).}

\item{ci_level}{Outer credible level (default: 0.95).}

\item{inner_level}{Inner credible level, or \code{NULL} to skip the inner
ribbon (default: 0.80).}

\item{true_sigma}{Optional \verb{(n_time - 1) x 2} matrix of true scale paths
(columns in variable order) overlaid in red.}

\item{title}{Plot title, or \code{NULL} for the default.}

\item{...}{Additional arguments (unused).}
}
\value{
A ggplot object.
}
\description{
Faceted ribbon plot of the posterior per-variable residual scale paths
from a time-varying DC-VAR fit (see \code{\link[=sigma_trajectory]{sigma_trajectory()}} for the
per-family scale semantics).
}
\seealso{
\code{\link[=sigma_trajectory]{sigma_trajectory()}}, \code{\link[=plot_phi_trajectory]{plot_phi_trajectory()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diagnostics.R
\name{dcvar_diagnostics}
\alias{dcvar_diagnostics}
\alias{dcvar_diagnostics.default}
\alias{dcvar_diagnostics.dcvar_model_fit}
\title{Extract MCMC diagnostics}
\usage{
dcvar_diagnostics(object, ...)

\method{dcvar_diagnostics}{default}(object, ...)

\method{dcvar_diagnostics}{dcvar_model_fit}(object, ...)
}
\arguments{
\item{object}{A fitted model object.}

\item{...}{Additional arguments (unused).}
}
\value{
A named list with:
\itemize{
\item \code{n_divergent}: total number of divergent transitions
\item \code{n_max_treedepth}: transitions hitting max tree depth
\item \code{max_rhat}: worst (highest) Rhat across sampled parameters
\item \code{min_ess_bulk}: smallest bulk ESS among sampled parameters
\item \code{min_ess_tail}: smallest tail ESS among sampled parameters
\item \code{mean_accept_prob}: mean acceptance probability
}
}
\description{
Returns a summary of sampling diagnostics including divergences,
tree depth warnings, Rhat, and effective sample size.
The convergence headline is computed from sampled parameters only and
excludes generated quantities and deterministic transformed outputs.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit-constant.R
\name{dcvar_constant}
\alias{dcvar_constant}
\title{Fit the constant copula model}
\usage{
dcvar_constant(
  data,
  vars,
  time_var = "time",
  standardize = TRUE,
  margins = "normal",
  copula = "gaussian",
  skew_direction = NULL,
  allow_gaps = FALSE,
  prior_mu_sd = 2,
  prior_phi_sd = 0.5,
  prior_sigma_eps_rate = 1,
  prior_z_rho_sd = 1,
  chains = 4,
  iter_warmup = 2000,
  iter_sampling = 4000,
  adapt_delta = 0.999,
  max_treedepth = 12,
  seed = NULL,
  cores = NULL,
  refresh = 500,
  init = NULL,
  stan_file = NULL,
  backend = getOption("dcvar.backend", "auto"),
  ...
)
}
\arguments{
\item{data}{A data frame with time series observations.}

\item{vars}{Character vector of two variable names to model.}

\item{time_var}{Name of the time column (default: \code{"time"}).}

\item{standardize}{Logical; whether to z-score variables (default: \code{TRUE}).}

\item{margins}{Marginal distribution specification. Either a single string
applied to both variables, or a length-2 character vector giving a
per-variable (mixed) margin (for example
\code{c("normal", "exponential")}). Each entry is one of \code{"normal"} (default),
\code{"exponential"}, \code{"skew_normal"}, or \code{"gamma"}. When the two entries
differ the fit uses a generic mixed-margins Stan model; identical entries
are equivalent to the scalar form and reuse the specialised single-family
model. Mixed margins are supported with the Gaussian copula, and (for
\code{dcvar_constant}) additionally with the Clayton copula.}

\item{copula}{Character string specifying the copula family. One of
\code{"gaussian"} (default) or \code{"clayton"}. Clayton is currently available with
normal margins or a per-variable (mixed) margin vector; a single
non-normal family with Clayton is not yet supported.}

\item{skew_direction}{Integer vector of length 2 indicating skew direction
for asymmetric margins. Each element must be \code{1} (right-skewed) or \code{-1}
(left-skewed). Required whenever any dimension uses an \code{"exponential"} or
\code{"gamma"} margin; only those dimensions consult it.}

\item{allow_gaps}{Logical; if \code{FALSE} (default), interior missing values
cause an error because they break VAR(1) time series adjacency. Set to
\code{TRUE} to allow fitting with a warning instead.}

\item{prior_mu_sd}{Prior SD for intercepts: \code{mu ~ normal(0, prior_mu_sd)}.}

\item{prior_phi_sd}{Prior SD for VAR coefficients: \code{Phi ~ normal(0, prior_phi_sd)}.}

\item{prior_sigma_eps_rate}{Prior mean for innovation SDs (see
\code{\link[=prepare_dcvar_data]{prepare_dcvar_data()}}).}

\item{prior_z_rho_sd}{Prior SD for rho on Fisher-z scale (default: 1.0).}

\item{chains}{Number of MCMC chains (default: 4).}

\item{iter_warmup}{Warmup iterations per chain (default: 2000).}

\item{iter_sampling}{Sampling iterations per chain (default: 4000).}

\item{adapt_delta}{Target acceptance rate (default: 0.999). The constant
model uses a higher default than DC-VAR (0.99) because its simpler
posterior geometry benefits from tighter step-size adaptation without
significant cost, reducing occasional divergences near the rho boundary.}

\item{max_treedepth}{Maximum tree depth (default: 12).}

\item{seed}{Random seed. When supplied, it seeds both the Stan sampler and
the default per-chain initial values, so repeated fits are reproducible.}

\item{cores}{Number of parallel chains. \code{NULL} uses all available cores.}

\item{refresh}{How often to print progress (default: 500). Set to 0 for
silent operation.}

\item{init}{Custom init function or \code{NULL} for smart defaults.}

\item{stan_file}{Path to a custom Stan file, or \code{NULL} to use the bundled
model.}

\item{backend}{Character: \code{"auto"} (default, uses rstan), \code{"rstan"}, or
\code{"cmdstanr"}. Can also be set globally via
\code{options(dcvar.backend = "cmdstanr")}.}

\item{...}{Additional backend-specific sampling arguments.}
}
\value{
A \code{dcvar_constant_fit} object.
}
\description{
Fits a copula VAR(1) model with a single time-invariant correlation
parameter. This serves as a baseline for comparison with the DC-VAR
and HMM copula models.
}
\details{
\code{adapt_delta} defaults to 0.999 because the constant-rho model has
a simpler correlation structure that benefits from tighter step-size
adaptation without significant computational cost, reducing occasional
divergences near the rho boundary.
}
\examples{
\donttest{
sim <- simulate_dcvar(
  n_time = 12,
  rho_trajectory = rho_constant(12, rho = 0.5),
  seed = 1
)
fit <- dcvar_constant(
  sim$Y_df,
  vars = c("y1", "y2"),
  chains = 1,
  iter_warmup = 10,
  iter_sampling = 10,
  refresh = 0,
  seed = 1
)
print(fit)
}
}
\seealso{
\code{\link[=dcvar]{dcvar()}} for the time-varying model, \code{\link[=dcvar_hmm]{dcvar_hmm()}} for the
regime-switching model, \code{\link[=dcvar_compare]{dcvar_compare()}} for LOO-CV model comparison.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/metrics.R
\name{compute_param_metrics}
\alias{compute_param_metrics}
\title{Compute scalar parameter recovery metrics}
\usage{
compute_param_metrics(true_value, est_mean, est_lower, est_upper)
}
\arguments{
\item{true_value}{True parameter value.}

\item{est_mean}{Estimated posterior mean.}

\item{est_lower}{Lower interval bound (2.5\% quantile).}

\item{est_upper}{Upper interval bound (97.5\% quantile).}
}
\value{
A named list with \code{bias}, \code{relative_bias}, \code{covered},
\code{interval_width}.
}
\description{
Compute scalar parameter recovery metrics
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{plot_hmm_states}
\alias{plot_hmm_states}
\title{Plot HMM state posteriors}
\usage{
plot_hmm_states(object, show_viterbi = TRUE, ...)
}
\arguments{
\item{object}{A \code{dcvar_hmm_fit} object.}

\item{show_viterbi}{Logical; overlay the Viterbi (MAP) state sequence
(default: \code{TRUE}).}

\item{...}{Additional arguments (unused).}
}
\value{
A ggplot object.
}
\description{
Plot HMM state posteriors
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extract.R
\name{hmm_states}
\alias{hmm_states}
\alias{hmm_states.default}
\alias{hmm_states.dcvar_hmm_fit}
\title{Extract HMM state information}
\usage{
hmm_states(object, ...)

\method{hmm_states}{default}(object, ...)

\method{hmm_states}{dcvar_hmm_fit}(object, ...)
}
\arguments{
\item{object}{A \code{dcvar_hmm_fit} object.}

\item{...}{Additional arguments (unused).}
}
\value{
A named list with:
\itemize{
\item \code{gamma}: T_eff x K matrix of posterior state probabilities
\item \code{viterbi}: integer vector, the Viterbi (MAP) state path decoded from
the posterior-mean emission log-likelihoods, transition matrix, and
initial state probabilities (a plug-in estimator, not a full posterior
summary of the path)
\item \code{rho_state}: list with \code{mean}, \code{lower}, \code{upper} for each state
\item \code{A}: K x K posterior mean transition matrix
\item \code{rho_hmm}: posterior-averaged rho trajectory
}
}
\description{
Returns state posteriors, Viterbi path, state-specific rho values,
and the transition matrix from an HMM copula fit.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/trajectories.R
\name{rho_increasing}
\alias{rho_increasing}
\title{Generate a logistically increasing rho trajectory}
\usage{
rho_increasing(
  n_time,
  rho_start = 0.3,
  rho_end = 0.7,
  midpoint = NULL,
  steepness = 0.05
)
}
\arguments{
\item{n_time}{Number of time points.}

\item{rho_start}{Starting rho value (default: 0.3).}

\item{rho_end}{Ending rho value (default: 0.7).}

\item{midpoint}{Time point of inflection (default: \code{n_time/2}).}

\item{steepness}{Controls transition sharpness (default: 0.05).}
}
\value{
Numeric vector of length \code{n_time - 1}.
}
\description{
Mimics deterioration where coupling increases from low to high.
}
\examples{
rho_increasing(100)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stan-utils.R
\name{dcvar_stan_path}
\alias{dcvar_stan_path}
\title{Get path to bundled Stan model file}
\usage{
dcvar_stan_path(
  model = c("dcvar", "dcvar_tv", "dcvar_dynamic", "dcvar_covariate",
    "dcvar_covariate_nodrift", "hmm", "hmm_switching", "constant", "multilevel",
    "multilevel_tv", "sem", "sem_tv", "sem_naive"),
  margins = "normal",
  copula = "gaussian"
)
}
\arguments{
\item{model}{Character string: \code{"dcvar"}, \code{"dcvar_tv"}, \code{"dcvar_dynamic"}
(the unified time-varying / drift covariate engine), \code{"dcvar_covariate"},
\code{"dcvar_covariate_nodrift"}, \code{"hmm"}, \code{"constant"}, \code{"multilevel"},
\code{"multilevel_tv"}, \code{"sem"}, \code{"sem_tv"}, or \code{"sem_naive"}.}

\item{margins}{Character string: margin type (\code{"normal"}, \code{"exponential"},
\code{"skew_normal"}, \code{"gamma"}). Default: \code{"normal"}.}

\item{copula}{Character string: copula family (\code{"gaussian"} or
\code{"clayton"}). Default: \code{"gaussian"}.}
}
\value{
File path to the Stan model file.
}
\description{
Returns the file path to a Stan model file included with the package.
}
\examples{
dcvar_stan_path("dcvar")
dcvar_stan_path("constant", margins = "exponential")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data-preparation.R
\name{prepare_dcvar_data}
\alias{prepare_dcvar_data}
\title{Prepare data for the DC-VAR model}
\usage{
prepare_dcvar_data(
  data,
  vars,
  time_var = "time",
  standardize = TRUE,
  margins = "normal",
  skew_direction = NULL,
  prior_mu_sd = 2,
  prior_phi_sd = 0.5,
  prior_sigma_eps_rate = 1,
  prior_sigma_omega_rate = 0.1,
  prior_rho_init_sd = 1,
  tv_phi = FALSE,
  tv_sigma = FALSE,
  prior_tau_phi_rate = 0.025,
  prior_tau_sigma_rate = 0.05,
  tv_sigma_k = 8,
  allow_gaps = FALSE
)
}
\arguments{
\item{data}{A data frame with time series observations.}

\item{vars}{Character vector of two variable names to model.}

\item{time_var}{Name of the time column (default: \code{"time"}).}

\item{standardize}{Logical; whether to z-score variables (default: \code{TRUE}).}

\item{margins}{Marginal distribution specification. Either a single string
applied to both variables, or a length-2 character vector giving a
per-variable (mixed) margin (for example \code{c("normal", "exponential")}).
Each entry is one of \code{"normal"} (default), \code{"exponential"},
\code{"skew_normal"}, or \code{"gamma"}. When the two entries differ the returned
data carries an integer \code{family} array for the generic mixed-margins Stan
model; identical entries are equivalent to the scalar form and reuse the
specialised single-family model.}

\item{skew_direction}{Integer vector of length D indicating skew direction
for asymmetric margins. Each element must be \code{1} (right-skewed) or \code{-1}
(left-skewed). Required whenever any dimension uses an \code{"exponential"} or
\code{"gamma"} margin.}

\item{prior_mu_sd}{Prior SD for intercepts: \code{mu ~ normal(0, prior_mu_sd)}.}

\item{prior_phi_sd}{Prior SD for VAR coefficients: \code{Phi ~ normal(0, prior_phi_sd)}.}

\item{prior_sigma_eps_rate}{Prior mean for innovation SDs:
\code{sigma_eps ~ exponential(1/prior_sigma_eps_rate)}. Default \code{1} gives
\code{exponential(1)} with prior mean 1.}

\item{prior_sigma_omega_rate}{Prior mean for rho process SD:
\code{sigma_omega ~ exponential(1/prior_sigma_omega_rate)}. Default \code{0.1} gives
\code{exponential(10)} with prior mean 0.1.}

\item{prior_rho_init_sd}{Prior SD for initial rho on Fisher-z scale.}

\item{tv_phi}{Selects which VAR(1) coefficients evolve as random walks
around the constant baseline \code{Phi}. Either a logical (\code{TRUE} = all four,
\code{FALSE} = none, the default) or a character selector: \code{"ar"} (the
autoregressive effects phi11, phi22), \code{"cross"} (the cross-lagged effects
phi12, phi21), or specific names from
\code{c("phi11", "phi12", "phi21", "phi22")}.}

\item{tv_sigma}{Logical; if \code{TRUE}, the residual scales evolve as log-scale
random walks around their constant baselines (default: \code{FALSE}). Applies
to all margin families; exponential and gamma dimensions use a
soft-barrier construction (see \code{\link[=dcvar]{dcvar()}}).}

\item{prior_tau_phi_rate}{Prior mean for the four Phi random-walk
innovation SDs: \code{tau_phi ~ exponential(1/prior_tau_phi_rate)}. Only used
when \code{tv_phi = TRUE}. The default \code{0.025} allows a total drift SD of
about 0.3 over 150 time points while shrinking toward constant
coefficients.}

\item{prior_tau_sigma_rate}{Prior mean for the per-dimension log-scale
random-walk innovation SDs:
\code{tau_sigma ~ exponential(1/prior_tau_sigma_rate)}. Only used when
\code{tv_sigma = TRUE}. The default \code{0.05} allows scale factors of roughly
0.5--2 over 150 time points.}

\item{tv_sigma_k}{Soft-barrier sharpness for time-varying exponential/gamma
scales (default \code{8}); see \code{\link[=dcvar]{dcvar()}}.}

\item{allow_gaps}{Logical; if \code{FALSE} (default), interior missing values
cause an error. If \code{TRUE}, they produce a warning and are removed.}
}
\value{
A named list suitable as Stan data input.
}
\description{
Transforms a data frame into a list suitable for the DC-VAR Stan model.
Handles sorting, missing values, and optional standardization.
}
\section{Prior naming conventions}{

Parameters ending in \verb{_sd} specify normal prior standard deviations
(location parameters). Parameters ending in \verb{_rate} specify exponential
prior means (scale parameters), where the exponential rate is
\verb{1/prior_*_rate}. The constant and HMM models use \code{prior_z_rho_sd}
(normal prior on the Fisher-z scale), while the DC-VAR model uses
\code{prior_sigma_omega_rate} (exponential prior on the random-walk SD)
because the two quantities have fundamentally different roles.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit-covariate.R
\name{prepare_dcvar_covariate_data}
\alias{prepare_dcvar_covariate_data}
\title{Prepare data for the covariate DC-VAR model}
\usage{
prepare_dcvar_covariate_data(
  data,
  vars,
  covariates,
  time_var = "time",
  standardize = TRUE,
  standardize_covariates = FALSE,
  allow_gaps = FALSE,
  prior_mu_sd = 2,
  prior_phi_sd = 0.5,
  prior_sigma_eps_rate = 1,
  prior_sigma_omega_rate = 0.1,
  prior_rho_init_sd = 1,
  prior_beta_sd = 1,
  zero_init_eta = TRUE
)
}
\arguments{
\item{data}{A data frame with time series observations.}

\item{vars}{Character vector of two variable names to model.}

\item{covariates}{Character vector of covariate column names.}

\item{time_var}{Name of the time column (default: \code{"time"}).}

\item{standardize}{Logical; whether to z-score variables (default: \code{TRUE}).}

\item{standardize_covariates}{Logical; whether to z-score covariates
(default: \code{FALSE}). Binary phase indicators should usually be left on
their original scale.}

\item{allow_gaps}{Logical; if \code{FALSE} (default), interior missing values
cause an error. If \code{TRUE}, they produce a warning and are removed.}

\item{prior_mu_sd}{Prior SD for intercepts: \code{mu ~ normal(0, prior_mu_sd)}.}

\item{prior_phi_sd}{Prior SD for VAR coefficients: \code{Phi ~ normal(0, prior_phi_sd)}.}

\item{prior_sigma_eps_rate}{Prior mean for innovation SDs:
\code{sigma_eps ~ exponential(1/prior_sigma_eps_rate)}. Default \code{1} gives
\code{exponential(1)} with prior mean 1.}

\item{prior_sigma_omega_rate}{Prior mean for rho process SD:
\code{sigma_omega ~ exponential(1/prior_sigma_omega_rate)}. Default \code{0.1} gives
\code{exponential(10)} with prior mean 0.1.}

\item{prior_rho_init_sd}{Prior SD for the dependence intercept \code{beta_0} on
the Fisher-z scale (the covariate model has no separate initial-rho
parameter; \code{beta_0} plays that role).}

\item{prior_beta_sd}{Prior SD for the covariate effects:
\code{beta ~ normal(0, prior_beta_sd)}.}

\item{zero_init_eta}{Logical; if \code{TRUE} (default), fixes the first residual
drift state at zero (\code{eta[1] = 0}). If \code{FALSE}, the first transition can
receive an immediate residual random-walk shock.}
}
\value{
A named list suitable as Stan data input.
}
\description{
Transforms a data frame into a list suitable for the Gaussian covariate
DC-VAR Stan models. Outcome rows are sorted by \code{time_var}; rows with missing
outcomes are removed with the same adjacency rules used by \code{\link[=prepare_dcvar_data]{prepare_dcvar_data()}},
and covariates are filtered in the same order so \code{X[t + 1, ]} aligns with the
outcome occasion of transition \code{Y[t, ] -> Y[t + 1, ]}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extract.R
\name{covariate_effects}
\alias{covariate_effects}
\alias{covariate_effects.default}
\alias{covariate_effects.dcvar_covariate_fit}
\title{Extract covariate effect summaries}
\usage{
covariate_effects(object, ...)

\method{covariate_effects}{default}(object, ...)

\method{covariate_effects}{dcvar_covariate_fit}(object, probs = c(0.025, 0.5, 0.975), ...)
}
\arguments{
\item{object}{A \code{dcvar_covariate_fit} object.}

\item{...}{Additional arguments (unused).}

\item{probs}{Numeric vector of quantile probabilities (default:
\code{c(0.025, 0.5, 0.975)}).}
}
\value{
A data frame with one row per effect and columns \code{term}, \code{variable},
\code{mean}, \code{sd}, and one column per requested quantile.
}
\description{
Returns posterior summaries for the Fisher-z intercept and covariate
effects. The residual random-walk innovation scale \code{sigma_omega} is
reported separately by \code{\link[=var_params]{var_params()}} for \code{drift = TRUE} fits.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extract.R
\name{sigma_trajectory}
\alias{sigma_trajectory}
\alias{sigma_trajectory.default}
\alias{sigma_trajectory.dcvar_tv_fit}
\alias{sigma_trajectory.dcvar_sem_tv_fit}
\alias{sigma_trajectory.dcvar_multilevel_tv_fit}
\title{Extract the time-varying residual scale trajectories}
\usage{
sigma_trajectory(object, ...)

\method{sigma_trajectory}{default}(object, ...)

\method{sigma_trajectory}{dcvar_tv_fit}(object, probs = c(0.025, 0.1, 0.5, 0.9, 0.975), ...)

\method{sigma_trajectory}{dcvar_sem_tv_fit}(object, probs = c(0.025, 0.1, 0.5, 0.9, 0.975), ...)

\method{sigma_trajectory}{dcvar_multilevel_tv_fit}(object, probs = c(0.025, 0.1, 0.5, 0.9, 0.975), ...)
}
\arguments{
\item{object}{A fitted model object.}

\item{...}{Additional arguments (unused).}

\item{probs}{Numeric vector of quantile probabilities
(default: \code{c(0.025, 0.1, 0.5, 0.9, 0.975)}).}
}
\value{
A data frame with columns \code{time}, \code{variable}, \code{mean}, \code{sd}, and
one column per requested quantile.
}
\description{
Returns posterior summaries of the per-variable residual scale paths from
a time-varying DC-VAR fit. The reported value is each margin family's
natural residual scale: the innovation SD for normal/skew-normal
dimensions, and the (time-constant) \code{sigma_exp} / \code{sigma_gam} for
exponential and gamma dimensions. For fits with \code{tv_sigma = FALSE} the
constant baselines are tiled over time so the return shape does not depend
on the flag.
}
\seealso{
\code{\link[=phi_trajectory]{phi_trajectory()}}, \code{\link[=rho_trajectory]{rho_trajectory()}}, \code{\link[=plot_sigma_trajectory]{plot_sigma_trajectory()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predict.R
\name{fitted.dcvar_hmm_fit}
\alias{fitted.dcvar_hmm_fit}
\title{One-step-ahead fitted values for a Markov-switching HMM fit}
\usage{
\method{fitted}{dcvar_hmm_fit}(object, type = c("link", "response"), ...)
}
\arguments{
\item{object}{A \code{dcvar_hmm_fit} object.}

\item{type}{\code{"link"} (default) or \code{"response"} (back-transform to the data scale).}

\item{...}{Additional arguments (unused).}
}
\value{
A data frame with a \code{time} column and one column per variable.
}
\description{
For a state-specific \code{dcvar_hmm()} fit the one-step prediction is the
smoothed-state mixture \verb{sum_k gamma[t, k] * (mu_k + Phi_k (Y[t] - mu_k))},
evaluated at posterior means. A non-switching HMM fit delegates to the shared
method.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/interpret.R
\name{interpret_rho_trajectory}
\alias{interpret_rho_trajectory}
\title{Interpret a rho trajectory in clinical terms}
\usage{
interpret_rho_trajectory(
  object,
  threshold = 0.1,
  strength_breaks = .default_strength_breaks,
  magnitude_breaks = .default_magnitude_breaks,
  fluctuation_threshold = 0.3,
  ...
)
}
\arguments{
\item{object}{A fitted model object (\code{dcvar_fit}, \code{dcvar_covariate_fit},
\code{dcvar_hmm_fit}, \code{dcvar_constant_fit}, \code{dcvar_multilevel_fit}, or
\code{dcvar_sem_fit}).}

\item{threshold}{Minimum absolute change in posterior-mean rho to be
considered "meaningful" (default: 0.1).}

\item{strength_breaks}{Named numeric vector of thresholds for classifying
correlation strength (default: \code{c(strong = 0.7, moderate = 0.4, weak = 0.2)}). Values above the highest threshold are "strong", etc.;
values at or below the lowest threshold are classified as "negligible".}

\item{magnitude_breaks}{Named numeric vector of thresholds for classifying
the magnitude of trajectory range (default: \code{c(large = 0.5, moderate = 0.3, small = 0.1)}). Values at or below the lowest threshold are classified as
"negligible".}

\item{fluctuation_threshold}{Proportion of sign changes in first differences
to flag "substantial fluctuation" (default: 0.3).}

\item{...}{Additional arguments (unused).}
}
\value{
A character string with the interpretation (invisibly). The
interpretation is also printed to the console.
}
\description{
Generates a human-readable interpretation of the estimated rho trajectory,
describing the overall trend, magnitude of change, and key features.
}
\examples{
\donttest{
sim <- simulate_dcvar(
  n_time = 12,
  rho_trajectory = rho_decreasing(12),
  seed = 1
)
fit <- dcvar(
  sim$Y_df,
  vars = c("y1", "y2"),
  chains = 1,
  iter_warmup = 10,
  iter_sampling = 10,
  refresh = 0,
  seed = 1
)
interpret_rho_trajectory(fit)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/trajectories.R
\name{rho_scenario}
\alias{rho_scenario}
\title{Get a named trajectory scenario}
\usage{
rho_scenario(scenario, n_time, ...)
}
\arguments{
\item{scenario}{Character string. One of:
\itemize{
\item Smooth: \code{"constant"}, \code{"decreasing"}, \code{"increasing"}, \code{"random_walk"}
\item Step: \code{"single_middle"}, \code{"large_change"}, \code{"small_change"},
\code{"increase"}, \code{"double_relapse"}
}}

\item{n_time}{Number of time points.}

\item{...}{Additional arguments passed to the generator.}
}
\value{
Numeric vector of length \code{n_time - 1}.
}
\description{
Convenience function to retrieve a standard scenario by name.
}
\examples{
rho_scenario("decreasing", n_time = 100)
rho_scenario("double_relapse", n_time = 150)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predict.R
\name{predict.dcvar_model_fit}
\alias{predict.dcvar_model_fit}
\alias{predict.dcvar_multilevel_fit}
\alias{predict.dcvar_multilevel_tv_fit}
\alias{predict.dcvar_sem_fit}
\alias{predict.dcvar_tv_fit}
\title{One-step-ahead predictions from a copula VAR model}
\usage{
\method{predict}{dcvar_model_fit}(object, type = c("link", "response"), ci_level = 0.95, ...)

\method{predict}{dcvar_multilevel_fit}(object, type = c("link", "response"), ci_level = 0.95, ...)

\method{predict}{dcvar_multilevel_tv_fit}(object, type = c("link", "response"), ci_level = 0.95, ...)

\method{predict}{dcvar_sem_fit}(object, type = c("link", "response"), ci_level = 0.95, ...)

\method{predict}{dcvar_tv_fit}(object, type = c("link", "response"), ci_level = 0.95, ...)
}
\arguments{
\item{object}{A fitted model object.}

\item{type}{Character; \code{"link"} (default) returns values on the model's
internal scale (standardized if applicable), \code{"response"} back-transforms
to the original data scale.}

\item{ci_level}{Prediction interval level (default: 0.95).}

\item{...}{Additional arguments (unused).}
}
\value{
A data frame of marginal prediction intervals at the specified
level. Single-level and SEM fits return columns \code{time}, \code{variable},
\code{mean}, \code{lower}, \code{upper}. Multilevel fits additionally include \code{unit}.
}
\description{
Returns point predictions and \strong{marginal} prediction intervals by combining
the VAR(1) fitted values with the estimated innovation SDs. Intervals are
computed per-variable using a normal approximation and do not account for
the copula dependence structure between variables. They are plug-in
intervals built from posterior-mean parameters: posterior parameter
uncertainty is not propagated, so coverage is understated on short series
where that uncertainty is non-negligible.
}
\details{
\code{predict()} is implemented for the public fit classes. For multilevel fits,
the methods return unit-specific trajectories. For SEM fits, \code{type = "link"}
returns latent states and \code{type = "response"} returns observed indicator
predictions.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{plot_random_effects}
\alias{plot_random_effects}
\title{Plot random effects (caterpillar plot)}
\usage{
plot_random_effects(object, ...)
}
\arguments{
\item{object}{A \code{dcvar_multilevel_fit} object.}

\item{...}{Additional arguments (unused).}
}
\value{
A ggplot object.
}
\description{
Displays unit-specific VAR coefficients with credible intervals.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predict.R
\name{predict.dcvar_hmm_fit}
\alias{predict.dcvar_hmm_fit}
\title{Prediction intervals for a Markov-switching HMM fit}
\usage{
\method{predict}{dcvar_hmm_fit}(object, ...)
}
\arguments{
\item{object}{A \code{dcvar_hmm_fit} object.}

\item{...}{Passed to the shared method for non-switching fits.}
}
\value{
A data frame of prediction intervals (non-switching fits only).
}
\description{
State-specific HMM fits do not yet support marginal prediction intervals (the
predictive is a regime mixture). Non-switching fits delegate to the shared
method.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{plot_latent_states}
\alias{plot_latent_states}
\title{Plot estimated latent states with credible intervals}
\usage{
plot_latent_states(object, true_states = NULL, ...)
}
\arguments{
\item{object}{A \code{dcvar_sem_fit} object.}

\item{true_states}{Optional T x 2 matrix of true latent states for overlay.}

\item{...}{Additional arguments (unused).}
}
\value{
A ggplot object.
}
\description{
Plot estimated latent states with credible intervals
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/trajectories.R
\name{rho_decreasing}
\alias{rho_decreasing}
\title{Generate a logistically decreasing rho trajectory}
\usage{
rho_decreasing(
  n_time,
  rho_start = 0.7,
  rho_end = 0.3,
  midpoint = NULL,
  steepness = 0.05
)
}
\arguments{
\item{n_time}{Number of time points.}

\item{rho_start}{Starting rho value (default: 0.7). Must be in [-1, 1].}

\item{rho_end}{Ending rho value (default: 0.3). Must be in [-1, 1].}

\item{midpoint}{Time point of inflection (default: \code{n_time/2}).}

\item{steepness}{Controls transition sharpness (default: 0.05).}
}
\value{
Numeric vector of length \code{n_time - 1}.
}
\description{
Mimics a therapy effect where coupling decreases from high to low.
}
\examples{
rho_decreasing(100)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{plot_phi}
\alias{plot_phi}
\title{Plot VAR(1) coefficient matrix as a heatmap}
\usage{
plot_phi(object, var_names = NULL, ...)
}
\arguments{
\item{object}{A fitted model object.}

\item{var_names}{Character vector of variable names for axis labels.}

\item{...}{Additional arguments (unused).}
}
\value{
A ggplot object.
}
\description{
Plot VAR(1) coefficient matrix as a heatmap
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/trajectories.R
\name{rho_constant}
\alias{rho_constant}
\title{Generate a constant rho trajectory}
\usage{
rho_constant(n_time, rho = 0.5)
}
\arguments{
\item{n_time}{Number of time points.}

\item{rho}{Constant correlation value (default: 0.5). Must be in [-1, 1].}
}
\value{
Numeric vector of length \code{n_time - 1}.
}
\description{
Generate a constant rho trajectory
}
\examples{
rho_constant(100, rho = 0.5)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dcvar-package.R
\docType{package}
\name{dcvar-package}
\alias{dcvar-package}
\title{dcvar: Copula VAR models for bivariate time series}
\description{
\code{dcvar} focuses on copula VAR(1) workflows for bivariate time series. Most
bundled models use Gaussian copulas. The core single-level Gaussian-copula
models \code{\link[=dcvar]{dcvar()}}, \code{\link[=dcvar_hmm]{dcvar_hmm()}}, and \code{\link[=dcvar_constant]{dcvar_constant()}} support normal,
exponential, skew-normal, and gamma margins. \code{\link[=dcvar_constant]{dcvar_constant()}} additionally
supports a Clayton copula baseline with normal margins or a per-variable
(mixed) margin vector.
}
\details{
Per-variable (mixed) margins let each variable take its own marginal family.
Across all model families a length-2 \code{margins} vector may combine any of
normal, exponential, skew-normal, and gamma per dimension (routing to a
generic mixed-margins Stan model). Homogeneous skew-normal and gamma
multilevel/SEM fits also route through the mixed engines internally.

Experimental extensions via \code{\link[=dcvar_multilevel]{dcvar_multilevel()}} and \code{\link[=dcvar_sem]{dcvar_sem()}} provide a
narrower diagnostic interface than the core models. Both support normal,
exponential, skew-normal, and gamma margins for single-family and
per-variable specifications; the SEM model also provides a naive row-mean
score method.

Estimation uses Stan through \code{rstan} by default, with optional \code{cmdstanr}
backend support. PIT diagnostics are approximate posterior-mean plug-in
diagnostics. PSIS-LOO model comparison is available for the supported
single-level models.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/benlug/dcvar}
  \item Report bugs at \url{https://github.com/benlug/dcvar/issues}
}

}
\author{
\strong{Maintainer}: Benedikt Lugauer \email{benedikt.lugauer@uni-leipzig.de}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/trajectories.R
\name{rho_random_walk}
\alias{rho_random_walk}
\title{Generate a random walk rho trajectory on the Fisher-z scale}
\usage{
rho_random_walk(n_time, z_init = 0.5, sigma_omega = 0.05, seed = NULL)
}
\arguments{
\item{n_time}{Number of time points.}

\item{z_init}{Initial value on Fisher-z scale (default: 0.5, corresponding
to rho = 0.46).}

\item{sigma_omega}{Innovation SD for the random walk (default: 0.05).}

\item{seed}{Random seed for reproducibility.}
}
\value{
Numeric vector of length \code{n_time - 1}.
}
\description{
Stochastic trajectory matching the DC-VAR data-generating process.
}
\examples{
rho_random_walk(100, seed = 42)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extract.R
\name{dependence_summary}
\alias{dependence_summary}
\alias{dependence_summary.default}
\alias{dependence_summary.dcvar_fit}
\alias{dependence_summary.dcvar_covariate_fit}
\alias{dependence_summary.dcvar_hmm_fit}
\alias{dependence_summary.dcvar_constant_fit}
\alias{dependence_summary.dcvar_multilevel_fit}
\alias{dependence_summary.dcvar_sem_fit}
\alias{dependence_summary.dcvar_sem_tv_fit}
\alias{dependence_summary.dcvar_multilevel_tv_fit}
\title{Extract a unified dependence summary}
\usage{
dependence_summary(object, ...)

\method{dependence_summary}{default}(object, ...)

\method{dependence_summary}{dcvar_fit}(object, probs = c(0.025, 0.1, 0.5, 0.9, 0.975), ...)

\method{dependence_summary}{dcvar_covariate_fit}(object, probs = c(0.025, 0.1, 0.5, 0.9, 0.975), ...)

\method{dependence_summary}{dcvar_hmm_fit}(object, probs = c(0.025, 0.1, 0.5, 0.9, 0.975), ...)

\method{dependence_summary}{dcvar_constant_fit}(object, probs = c(0.025, 0.1, 0.5, 0.9, 0.975), ...)

\method{dependence_summary}{dcvar_multilevel_fit}(object, probs = c(0.025, 0.1, 0.5, 0.9, 0.975), ...)

\method{dependence_summary}{dcvar_sem_fit}(object, probs = c(0.025, 0.1, 0.5, 0.9, 0.975), ...)

\method{dependence_summary}{dcvar_sem_tv_fit}(object, probs = c(0.025, 0.1, 0.5, 0.9, 0.975), ...)

\method{dependence_summary}{dcvar_multilevel_tv_fit}(object, probs = c(0.025, 0.1, 0.5, 0.9, 0.975), ...)
}
\arguments{
\item{object}{A fitted model object.}

\item{...}{Additional arguments (unused).}

\item{probs}{Numeric vector of quantile probabilities.}
}
\value{
A data frame with columns \code{time}, \code{mean}, \code{sd}, and one column per
requested quantile.
}
\description{
Returns posterior summaries for Kendall's tau, using the fitted copula
family to transform the model-specific dependence parameter. For Gaussian
copulas, \code{tau = 2 / pi * asin(rho)}. For Clayton copulas,
\code{tau = theta / (theta + 2)}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data-preparation.R
\name{prepare_multilevel_data}
\alias{prepare_multilevel_data}
\title{Prepare data for the multilevel copula VAR model}
\usage{
prepare_multilevel_data(
  data,
  vars,
  id_var = "id",
  time_var = "time",
  center = TRUE,
  prior_phi_bar_sd = 0.5,
  prior_tau_phi_scale = 0.2,
  prior_sigma_sd = 1,
  prior_rho_sd = 0.5,
  tv_phi = FALSE,
  tv_sigma = FALSE,
  prior_tau_phi_rate = 0.025,
  prior_tau_sigma_rate = 0.05,
  tv_sigma_k = 8,
  margins = "normal",
  skew_direction = NULL
)
}
\arguments{
\item{data}{A data frame in long (panel) format.}

\item{vars}{Character vector of two variable names.}

\item{id_var}{Name of the unit/person ID column.}

\item{time_var}{Name of the time column.}

\item{center}{Logical scalar; person-mean center (default: \code{TRUE}).}

\item{prior_phi_bar_sd}{Prior SD for phi_bar.}

\item{prior_tau_phi_scale}{Prior scale for tau_phi.}

\item{prior_sigma_sd}{Prior SD for sigma.}

\item{prior_rho_sd}{Prior SD for rho.}

\item{tv_phi}{Selects which population VAR(1) coefficients carry a shared
time-varying drift. See \code{\link[=dcvar]{dcvar()}} for accepted selectors.}

\item{tv_sigma}{Logical; if \code{TRUE}, residual scales evolve over time.}

\item{prior_tau_phi_rate}{Prior mean for time-drift Phi random-walk SDs.}

\item{prior_tau_sigma_rate}{Prior mean for log-scale random-walk SDs.}

\item{tv_sigma_k}{Soft-barrier sharpness for time-varying exponential/gamma
scales.}

\item{margins}{Marginal distribution specification. A single string applies
the same family to both variables. Normal and exponential use specialised
Stan data contracts; skew-normal, gamma, and length-2 per-variable specs use
the generic \code{multilevel_mixed} Stan model with per-dimension family codes.}

\item{skew_direction}{Length-2 integer vector of +1/-1. Required whenever any
dimension uses an \code{"exponential"} or \code{"gamma"} margin.}
}
\value{
A named list suitable as Stan data input.
}
\description{
Prepare data for the multilevel copula VAR model
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simulate.R
\name{simulate_dcvar}
\alias{simulate_dcvar}
\title{Simulate data from a copula VAR(1) model}
\usage{
simulate_dcvar(
  n_time,
  rho_trajectory,
  mu = c(0, 0),
  Phi = matrix(c(0.3, 0.1, 0.1, 0.3), 2, 2),
  sigma_eps = c(1, 1),
  margins = "normal",
  skew_direction = NULL,
  skew_params = NULL,
  phi_trajectory = NULL,
  sigma_trajectory = NULL,
  tv_sigma_k = NULL,
  seed = NULL
)
}
\arguments{
\item{n_time}{Number of time points.}

\item{rho_trajectory}{Numeric vector of length \code{n_time - 1} specifying the
correlation at each time step. Use \code{\link[=rho_constant]{rho_constant()}}, \code{\link[=rho_decreasing]{rho_decreasing()}},
etc.}

\item{mu}{Intercept vector of length 2 (default: \code{c(0, 0)}).}

\item{Phi}{VAR(1) coefficient matrix, 2x2 (default:
\code{matrix(c(0.3, 0.1, 0.1, 0.3), 2, 2)}).}

\item{sigma_eps}{Innovation standard deviations, length 2 (default:
\code{c(1, 1)}). Used for normal margins.}

\item{margins}{Marginal family. Either a single string applied to both
variables, or a length-2 character vector for per-variable (mixed) margins,
e.g. \code{c("normal", "exponential")}. Each entry is one of \code{"normal"}
(default), \code{"exponential"}, \code{"skew_normal"}, or \code{"gamma"}.}

\item{skew_direction}{Length-2 integer vector of +1/-1. Required whenever any
dimension uses an \code{"exponential"} or \code{"gamma"} margin; only those
dimensions consult it.}

\item{skew_params}{Named list of margin-specific parameters. \code{alpha}
(length-2 vector of skew-normal shape params) is used by skew-normal
dimensions; \code{shape} (scalar gamma shape parameter) is used by gamma
dimensions. Both may be supplied together for mixed margins.}

\item{phi_trajectory}{Optional time-varying VAR coefficient paths for data
matching \code{dcvar(tv_phi = TRUE)}: an \verb{(n_time - 1) x 4} matrix with
columns in row-major order (phi11, phi12, phi21, phi22), a list of 4
length-\code{(n_time - 1)} vectors (the \verb{rho_*} trajectory generators can be
reused per coefficient), or a constant \verb{2 x 2} matrix. Mutually exclusive
with \code{Phi}. \code{NULL} (default) keeps the constant \code{Phi}.}

\item{sigma_trajectory}{Optional time-varying residual scale paths for data
matching \code{dcvar(tv_sigma = TRUE)}: an \verb{(n_time - 1) x 2} positive matrix,
a list of 2 length-\code{(n_time - 1)} vectors, or a constant length-2 vector.
The scale is each family's natural scale (innovation SD for normal,
residual SD for skew-normal, \code{sigma_exp} / \code{sigma_gam} for
exponential / gamma). A non-constant path on an exponential or gamma
dimension requires \code{tv_sigma_k} (soft-barrier generative model). Mutually
exclusive with \code{sigma_eps}. \code{NULL} (default) keeps constant scales.}

\item{tv_sigma_k}{Soft-barrier sharpness for time-varying exponential/gamma
scales. Supply the same value as the intended \code{dcvar(tv_sigma = TRUE, tv_sigma_k = ...)} fit so the simulated data matches that likelihood.
\code{NULL} (default) uses the exact affine shifted margin and requires
constant exp/gamma scales.}

\item{seed}{Random seed for reproducibility.}
}
\value{
A named list with:
\itemize{
\item \code{Y}: \verb{n_time x 2} observation matrix
\item \code{Y_df}: data frame with columns \code{time}, \code{y1}, \code{y2} (ready for
\code{\link[=dcvar]{dcvar()}})
\item \code{true_params}: list of true parameter values. These are on the raw
data scale; the fitting functions standardize by default, so round-trip
comparisons of \code{mu}, \code{Phi}, and \code{sigma_eps} require fitting with
\code{standardize = FALSE} (the \code{rho} trajectory is scale-invariant).
Without \code{sigma_trajectory}, exponential and gamma dimensions are
simulated with unit-SD standardized innovations, so their implied true
scale (\code{sigma_exp}/\code{sigma_gam}) is 1. With trajectories supplied,
\code{Phi} is the \verb{(n_time - 1) x 4} coefficient path and \code{sigma} the
\verb{(n_time - 1) x 2} scale path.
}
}
\description{
Generates bivariate time series data with correlated innovations
driven by a specified rho trajectory.
}
\examples{
sim <- simulate_dcvar(n_time = 100, rho_trajectory = rho_decreasing(100))
head(sim$Y_df)
plot(sim$true_params$rho, type = "l")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predict.R
\name{fitted.dcvar_model_fit}
\alias{fitted.dcvar_model_fit}
\alias{fitted.dcvar_multilevel_fit}
\alias{fitted.dcvar_multilevel_tv_fit}
\alias{fitted.dcvar_sem_fit}
\alias{fitted.dcvar_tv_fit}
\title{Fitted values from a copula VAR model}
\usage{
\method{fitted}{dcvar_model_fit}(object, type = c("link", "response"), ...)

\method{fitted}{dcvar_multilevel_fit}(object, type = c("link", "response"), ...)

\method{fitted}{dcvar_multilevel_tv_fit}(object, type = c("link", "response"), ...)

\method{fitted}{dcvar_sem_fit}(object, type = c("link", "response"), ...)

\method{fitted}{dcvar_tv_fit}(object, type = c("link", "response"), ...)
}
\arguments{
\item{object}{A fitted model object.}

\item{type}{Character; \code{"link"} (default) returns values on the model's
internal scale (standardized if applicable), \code{"response"} back-transforms
to the original data scale.}

\item{...}{Additional arguments (unused).}
}
\value{
A data frame of posterior-mean fitted values. Single-level fits
return columns \code{time} plus one column per modeled variable. Multilevel
fits additionally include \code{unit}. SEM fits return either latent-state
columns (\code{type = "link"}) or observed-indicator columns
(\code{type = "response"}).
}
\description{
Returns the one-step-ahead fitted values (posterior mean of y_hat) from the
VAR(1) component. For single-level fits (constant, dynamic, HMM, covariate)
this is the mean-centered form \code{y_hat[t] = mu + Phi * (y[t-1] - mu)}.
}
\details{
If the model was fit with \code{standardize = TRUE} (the default), fitted values
are on the standardized (z-scored) scale by default. Use \code{type = "response"}
to back-transform to the original data scale.

\code{fitted()} and \code{predict()} are implemented for the public fit classes. For
multilevel fits, the methods return unit-specific trajectories; because the
data are already person-mean-centered, the link-scale fitted values apply the
unit's VAR matrix to the previous observation with no \code{mu} term
(\code{y_hat[t] = Phi_unit \%*\% y[t-1]}), and the person mean is re-added only for
\code{type = "response"}. For SEM fits, \code{type = "link"}
returns latent-state summaries and \code{type = "response"} returns observed
indicator-scale summaries.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot-trajectories.R
\name{plot_trajectories}
\alias{plot_trajectories}
\title{Plot and compare multiple rho trajectory shapes}
\usage{
plot_trajectories(
  n_time,
  scenarios = c("constant", "decreasing", "increasing", "random_walk", "single_middle",
    "large_change", "double_relapse"),
  ...
)
}
\arguments{
\item{n_time}{Number of time points.}

\item{scenarios}{Character vector of scenario names (see \code{\link[=rho_scenario]{rho_scenario()}}).
Default: all built-in scenarios.}

\item{...}{Additional arguments passed to \code{\link[=rho_scenario]{rho_scenario()}}. Each argument is
forwarded only to the scenarios whose generator accepts it; arguments not
used by any requested scenario produce a warning.}
}
\value{
A ggplot object.
}
\description{
Visualises several named trajectory scenarios side by side for comparison.
}
\examples{
plot_trajectories(100)
plot_trajectories(100, scenarios = c("decreasing", "single_middle"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extract.R
\name{latent_states}
\alias{latent_states}
\alias{latent_states.default}
\alias{latent_states.dcvar_sem_fit}
\title{Extract latent states from a SEM fit}
\usage{
latent_states(object, ...)

\method{latent_states}{default}(object, ...)

\method{latent_states}{dcvar_sem_fit}(object, probs = c(0.025, 0.5, 0.975), ...)
}
\arguments{
\item{object}{A \code{dcvar_sem_fit} object.}

\item{...}{Additional arguments (unused).}

\item{probs}{Numeric vector of quantile probabilities.}
}
\value{
A data frame with columns \code{time}, \code{variable}, \code{mean}, \code{sd},
and quantile columns.
}
\description{
Returns posterior summaries for the estimated latent states at each
time point.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data-preparation.R
\name{prepare_hmm_data}
\alias{prepare_hmm_data}
\title{Prepare data for the HMM copula model}
\usage{
prepare_hmm_data(
  data,
  vars,
  K = 2,
  time_var = "time",
  standardize = TRUE,
  margins = "normal",
  skew_direction = NULL,
  prior_mu_sd = 2,
  prior_phi_sd = 0.5,
  prior_sigma_eps_rate = 1,
  prior_kappa = 10,
  prior_alpha_off = 1,
  prior_z_rho_sd = 1,
  allow_gaps = FALSE
)
}
\arguments{
\item{data}{A data frame with time series observations.}

\item{vars}{Character vector of two variable names to model.}

\item{K}{Number of hidden states (default: 2).}

\item{time_var}{Name of the time column (default: \code{"time"}).}

\item{standardize}{Logical; whether to z-score variables (default: \code{TRUE}).}

\item{margins}{Character string specifying the marginal distribution.
One of \code{"normal"} (default), \code{"exponential"}, \code{"skew_normal"}, or \code{"gamma"}.}

\item{skew_direction}{Integer vector of length D indicating skew direction
for asymmetric margins. Each element must be \code{1} (right-skewed) or \code{-1}
(left-skewed). Required for \code{"exponential"} and \code{"gamma"} margins.}

\item{prior_mu_sd}{Prior SD for intercepts: \code{mu ~ normal(0, prior_mu_sd)}.}

\item{prior_phi_sd}{Prior SD for VAR coefficients: \code{Phi ~ normal(0, prior_phi_sd)}.}

\item{prior_sigma_eps_rate}{Prior mean for innovation SDs:
\code{sigma_eps ~ exponential(1/prior_sigma_eps_rate)}. Default \code{1} gives
\code{exponential(1)} with prior mean 1.}

\item{prior_kappa}{Sticky Dirichlet self-transition concentration (default: 10).}

\item{prior_alpha_off}{Sticky Dirichlet off-diagonal concentration (default: 1).}

\item{prior_z_rho_sd}{Prior SD for state-specific z_rho values (default: 1.0).}

\item{allow_gaps}{Logical; if \code{FALSE} (default), interior missing values
cause an error. If \code{TRUE}, they produce a warning and are removed.}
}
\value{
A named list suitable as Stan data input.
}
\description{
Transforms a data frame into a list suitable for the HMM copula Stan model.
Includes HMM-specific prior hyperparameters.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{plot_phi_trajectory}
\alias{plot_phi_trajectory}
\title{Plot the time-varying VAR coefficient trajectories}
\usage{
plot_phi_trajectory(
  object,
  show_ci = TRUE,
  ci_level = 0.95,
  inner_level = 0.8,
  true_phi = NULL,
  title = NULL,
  ...
)
}
\arguments{
\item{object}{A \code{dcvar_tv_fit} object.}

\item{show_ci}{Logical; draw credible ribbons (default: \code{TRUE}).}

\item{ci_level}{Outer credible level (default: 0.95).}

\item{inner_level}{Inner credible level, or \code{NULL} to skip the inner
ribbon (default: 0.80).}

\item{true_phi}{Optional \verb{(n_time - 1) x 4} matrix of true coefficient
paths (columns in phi11/phi12/phi21/phi22 order) overlaid in red, e.g.
from \code{simulate_dcvar()}'s \code{true_params}.}

\item{title}{Plot title, or \code{NULL} for the default.}

\item{...}{Additional arguments (unused).}
}
\value{
A ggplot object.
}
\description{
Faceted ribbon plot of the posterior phi11(t), phi12(t), phi21(t),
phi22(t) trajectories from a time-varying DC-VAR fit (see
\code{\link[=phi_trajectory]{phi_trajectory()}}).
}
\seealso{
\code{\link[=phi_trajectory]{phi_trajectory()}}, \code{\link[=plot_rho]{plot_rho()}}, \code{\link[=plot_sigma_trajectory]{plot_sigma_trajectory()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/trajectories.R
\name{rho_double_step}
\alias{rho_double_step}
\title{Generate a double-breakpoint (relapse pattern) rho trajectory}
\usage{
rho_double_step(
  n_time,
  rho_levels = c(0.7, 0.3, 0.7),
  breakpoints = c(1/3, 2/3),
  transition_width = 0
)
}
\arguments{
\item{n_time}{Number of time points.}

\item{rho_levels}{Numeric vector of three rho levels (default:
\code{c(0.7, 0.3, 0.7)}).}

\item{breakpoints}{Numeric vector of two breakpoint positions (default:
\code{c(1/3, 2/3)}). Interpreted as proportions of \code{n_time - 1} if <= 1.}

\item{transition_width}{Number of time points for smooth transitions
(default: 0).}
}
\value{
Numeric vector of length \code{n_time - 1}.
}
\description{
Three-phase trajectory: level A -> level B -> level C.
}
\examples{
rho_double_step(100, rho_levels = c(0.7, 0.3, 0.7))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/metrics.R
\name{compute_rho_metrics}
\alias{compute_rho_metrics}
\title{Compute rho trajectory recovery metrics}
\usage{
compute_rho_metrics(rho_true, rho_est, rho_lower, rho_upper)
}
\arguments{
\item{rho_true}{Numeric vector of true rho values (length T-1).}

\item{rho_est}{Numeric vector of estimated rho (posterior mean).}

\item{rho_lower}{Numeric vector of lower interval bounds (2.5\% quantiles).}

\item{rho_upper}{Numeric vector of upper interval bounds (97.5\% quantiles).}
}
\value{
A named list with:
\itemize{
\item \code{bias}: mean bias
\item \code{relative_bias}: relative bias (\%), the mean error normalized by the
mean true value (by the mean absolute true value for mixed-sign
trajectories; \code{NA} with a warning when all true values are near zero)
\item \code{coverage}: proportion of intervals containing true value
\item \code{interval_width}: mean interval width
\item \code{correlation}: Pearson correlation between true and estimated
\item \code{bias_start}, \code{bias_end}: bias at first/last time point
\item \code{coverage_start}, \code{coverage_end}: coverage at endpoints
}
}
\description{
Evaluates how well the estimated rho trajectory recovers the true values.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dcvar_fit-class.R
\name{print.dcvar_summary}
\alias{print.dcvar_summary}
\title{Print a dcvar_summary object}
\usage{
\method{print}{dcvar_summary}(x, ...)
}
\arguments{
\item{x}{A \code{dcvar_summary} object as returned by \code{\link[=summary.dcvar_fit]{summary.dcvar_fit()}}.}

\item{...}{Additional arguments (unused).}
}
\value{
Invisibly returns \code{x}.
}
\description{
Print a dcvar_summary object
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simulate-breakpoint.R
\name{simulate_breakpoint_data}
\alias{simulate_breakpoint_data}
\title{Simulate data with a breakpoint rho trajectory}
\usage{
simulate_breakpoint_data(
  n_time,
  type = c("single", "double"),
  rho_before = 0.7,
  rho_after = 0.3,
  rho_levels = c(0.7, 0.3, 0.7),
  breakpoint = 0.5,
  breakpoints = c(1/3, 2/3),
  transition_width = 0,
  mu = c(0, 0),
  Phi = matrix(c(0.3, 0.1, 0.1, 0.3), 2, 2),
  sigma_eps = c(1, 1),
  seed = NULL
)
}
\arguments{
\item{n_time}{Number of time points.}

\item{type}{Character; one of \code{"single"} (single breakpoint) or \code{"double"}
(double breakpoint / relapse pattern).}

\item{rho_before}{Rho before breakpoint (default: 0.7).}

\item{rho_after}{Rho after breakpoint (default: 0.3).}

\item{rho_levels}{Numeric vector of three rho levels for double breakpoint
(default: \code{c(0.7, 0.3, 0.7)}). Only used when \code{type = "double"}.}

\item{breakpoint}{Breakpoint location as proportion of \code{n_time - 1} (default: 0.5).}

\item{breakpoints}{Numeric vector of two breakpoints for double type
(default: \code{c(1/3, 2/3)}).}

\item{transition_width}{Number of time points for smooth transition
(default: 0 = abrupt).}

\item{mu}{Intercept vector of length 2 (default: \code{c(0, 0)}).}

\item{Phi}{VAR(1) coefficient matrix, 2x2 (default:
\code{matrix(c(0.3, 0.1, 0.1, 0.3), 2, 2)}).}

\item{sigma_eps}{Innovation SDs, length 2 (default: \code{c(1, 1)}).}

\item{seed}{Random seed.}
}
\value{
A named list as returned by \code{\link[=simulate_dcvar]{simulate_dcvar()}}. Its \code{true_params}
additionally records the breakpoint specification: \code{type}, plus
\code{breakpoint} (single) or \code{breakpoints} (double), as proportions of
\code{n_time - 1}.
}
\description{
Convenience wrapper that combines \code{\link[=rho_step]{rho_step()}} or \code{\link[=rho_double_step]{rho_double_step()}} with
\code{\link[=simulate_dcvar]{simulate_dcvar()}} for quick breakpoint simulation studies.
}
\examples{
sim <- simulate_breakpoint_data(n_time = 100, type = "single", seed = 42)
plot(sim$true_params$rho, type = "l")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extract.R
\name{var_params}
\alias{var_params}
\alias{var_params.default}
\alias{var_params.dcvar_model_fit}
\alias{var_params.dcvar_hmm_fit}
\alias{var_params.dcvar_multilevel_fit}
\alias{var_params.dcvar_sem_fit}
\alias{var_params.dcvar_tv_fit}
\alias{var_params.dcvar_sem_tv_fit}
\alias{var_params.dcvar_multilevel_tv_fit}
\title{Extract VAR(1) parameter summaries}
\usage{
var_params(object, ...)

\method{var_params}{default}(object, ...)

\method{var_params}{dcvar_model_fit}(object, ...)

\method{var_params}{dcvar_hmm_fit}(object, ...)

\method{var_params}{dcvar_multilevel_fit}(object, ...)

\method{var_params}{dcvar_sem_fit}(object, ...)

\method{var_params}{dcvar_tv_fit}(object, ...)

\method{var_params}{dcvar_sem_tv_fit}(object, ...)

\method{var_params}{dcvar_multilevel_tv_fit}(object, ...)
}
\arguments{
\item{object}{A fitted model object.}

\item{...}{Additional arguments (unused).}
}
\value{
A named list of data frames with columns \code{variable}, \code{mean}, \code{sd},
\code{q2.5}, \code{q97.5}.
}
\description{
Returns posterior summaries for the VAR parameters: intercepts (mu),
coefficients (Phi), innovation SDs (sigma_eps), and sigma_omega (DC-VAR only).
}
\details{
For a Markov-switching HMM fit (\code{dcvar_hmm()} with \code{switch} beyond \code{"rho"} or
per-state \code{margins}), the VAR parameters are state-indexed: \code{mu[k, d]}, the Phi
baseline \code{Phi_base[i, j]} plus per-state deviations \code{Phi_dev[k, i, j]} on the
switching coefficients, and per-state margin scales. A non-switching HMM fit
delegates to the shared method. See \code{\link[=hmm_state_params]{hmm_state_params()}} for a per-state
assembled view.

For multilevel models, returns population-level parameters \code{phi_bar}
(mean VAR coefficients), \code{tau_phi} (between-unit SDs), \code{sigma}
(innovation SDs), and \code{rho} (copula correlation). These correspond to
\code{Phi}, \code{sigma_eps}, and \code{rho} in single-level models.
}
